/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007, 2012. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("jazz.ui.ListBox");

/*
 * Imports 
 */
dojo.require("dojo.i18n"); 
dojo.require("dijit._Widget");
dojo.require("dijit._Templated");

dojo.requireLocalization("jazz.ui", "ListBoxMessages");

(function() {
	
var messages;

/*
 * Aliases 
 */

/*
 * TODOs 
 */


/*
 * Constants 
 */
var SORT_ORDER_ASCENDING = 0;
var SORT_ORDER_DESCENDING = 1;
var SORT_ORDER_NONE = 2;

/*
 * Class 
 *
 * PROVISIONAL: This API is considered provisional and may be changed or removed in the
 * future with minimal or no warning.
 * 
 * @since 0.7
 */
dojo.declare("jazz.ui.ListBox",
	[dijit._Widget, dijit._Templated], {

	/*
	 * Properties
	 */	
	width: "",
	height: "250px",
	margin: "5px",
	
	/*
	 * Specify the entry field to use when sorting list box entries
	 * Options: "label", "value", "id", "type"
	 */		
	sortField: "label",	
	
	/*
	 * Specify the sort order of the list box entries
	 * Options: 
	 * 0:  Ascending 
	 * 1:  Descending
	 * 2:  None
	 */		
	sortOrder: 0,	
	
	/* 
 	 * Specify the callback function to use when an entry is double-clicked
 	 * 
 	 * Function (entry)
 	 * 
 	 * @param Object entry
 	 *				. String label
 	 * 				. Object value
 	 * 				. String id
 	 * 				. Number type	 
 	 */
	ondblClick: null,

	/* 
 	 * DEPRECATED
 	 * 
 	 * Use 'onChange' instead.
 	 * 
 	 * Specify the callback function to use when an entry is clicked
 	 * 
 	 * Function (entry)
 	 * 
 	 * @param Object entry
 	 *				. String label
 	 * 				. Object value
 	 * 				. String id
 	 * 				. Number type	 
 	 */
	onClick: null,

	/* 
 	 * Attach point for notification about when the user selects one or more entries in the control 
 	 * using the keyboard and / or mouse.  
 	 * 
 	 * void onChange(event) {...}
 	 * 
 	 * @param SelectionEvent event
 	 *				. ListBox widget
 	 */
	onChange: function(event) {},
	
	/*
	 * Specify whether to allow multiple selections using the CTRL and SHIFT
	 * keys
	 */	
	allowMultipleSelections: true,

	/*
 	 * Specify icons for list box entries by providing a map of entry types 
 	 * associated with icon pathes.
 	 * 
 	 * Array iconMap : [Number entryType] = String iconPath
 	 */	
	iconMap: [],

	/*
	 * Templates
	 */	
	templatePath: dojo.moduleUrl("jazz.ui", "templates/ListBox.html"),
	
	
	widgetsInTemplate: true,	

	/*
 	 * @param Object args
 	 * 
 	 * @return ListBox
 	 */
	constructor: function(args) {
		
		// Messages //------------------------------------------------------------->
	
		if (!messages) messages = dojo.i18n.getLocalization("jazz.ui", "ListBoxMessages");
		this.messages = messages;
		this.msgLoading = this.messages.msgLoading;
		
	},
	
	/*
 	 * @param void
 	 * 
 	 * @return void
 	 * @overrides super
 	 */	
	postCreate: function() {

		this.inherited(arguments);
		
		// Instance Variables //----------------------------------------------->
		
		this._domMap = {};
		this._dataMap = {};
		this._dataArray = [];
		this._selected = {};

		this._entryCount = 0;

		this._loadingFlag = false;
		
		this._lastSelectedEntryID = null;
		this._focusNode = null;

		// Initialization //--------------------------------------------------->

		if (this.height) {
			this.domNode.style.height = this.height;
		}
		if (this.width) {
			this.domNode.style.width = this.width;
		}
		if (this.margin) {
			this.domNode.style.margin = this.margin;
		}
		
		// Add double-click listener
		if (this.ondblClick) {
			this.connect(this._contentArea, "ondblclick", this._handleEventDoubleClickEntry);
		}
		
		if (this.onClick) {
			dojo.deprecated(this.id, "Use 'onChange' instead of 'onClick'.");
		}
		
		// Aria
		if (this.allowMultipleSelections) {
			dijit.setWaiState(this.domNode, "multiselectable", "true");
		}
	},
	
	/*
 	 * @param Element node
 	 * @param Boolean doIt
 	 * 
 	 * @return void
 	 */	
	_setFocus: function(node, doIt) {
		if (!this._nodeExists(node)) {
			return;
		}
		// Place focus on the specified node
		if (doIt) {
			dijit.focus(node);
		}
		if (this._isFocused(node)) {
			// Done. New and old focus nodes are the same
			return;
		}		
		// Remove old focus node from tab order
		if (this._nodeExists(this._focusNode)) {
			dojo.attr(this._focusNode, "tabindex", -1);
		}
		// Add new focus node to tab order
		dojo.removeAttr(node, "tabindex");
		this._focusNode = node;
	},	
	
	/*
 	 * @param Element node
 	 * 
 	 * @return Boolean
 	 */		
	_nodeExists: function(node) {
		if (!node) {
			return false;
		}
		var nodeID = dojo.attr(node, "entryid");
		if (this._domMap.hasOwnProperty(nodeID)) {
			return true;
		}
		return false;
	},	
	
	/*
 	 * @param Element node
 	 * 
 	 * @return Boolean
 	 */		
	_isFocused: function(node) {
		if (this._focusNode === node) {
			return true;
		}			
		return false;
	},		
	
	/*
 	 * @param void
 	 * 
 	 * @return Element
 	 */		
	_getFirstEntryNode: function() {
		if (this._entryCount === 0) {
			return null;
		}
		var entry = this._dataArray[0];
		return this._getEntryNode(entry);
	},		
	
	/*
 	 * @param Object entry
 	 * 
 	 * @return Element
 	 */		
	_getEntryNode: function(entry) {
		if (!entry) {
			// Unexpected error
			return null;
		}		
		var id = entry.id;
		if (!this._domMap.hasOwnProperty(id)) {
			return null;
		}
		return this._domMap[id];
	},		
	
	/*
 	 * @param Object entry
 	 * 				. String label
 	 * 				. Object value
 	 * 				. String id
 	 * 				. Number type
 	 * 
 	 * @return void
 	 */	
	addEntry: function(entry) {
		// Validate list		
		if (!this.domNode) {
			return;
		}

		var id = entry.id;
		// Check if the ID is already in use
		if (this._isObjectDefined(this._dataMap[id])) {
			// Ignore
			return;
		}

		// Create DOM representation
		// Using an anchor instead of a div to allow :hover without IE performance degradation
		var entryNode = document.createElement("a");
		entryNode.href = "#";
		dojo.style(entryNode, 'display', 'block');
		entryNode.setAttribute("entryid", id);
		dojo.attr(entryNode, "tabindex", -1);
		dojo.addClass(entryNode, "entry");
		dijit.setWaiRole(entryNode, "option");
		dijit.setWaiState(entryNode, "label", entry.label);
		dijit.setWaiState(entryNode, "selected", "false");

		this._addIcon(entryNode, this.iconMap[entry.type]);
		this._addLabel(entryNode, entry.label);

		// Add to data map
		this._dataMap[id] = entry;
		// Add to DOM map		
		this._domMap[id] = entryNode;
		// Add to DOM itself
		this._addNode(this._contentArea, entryNode, entry);
		// Add to selection map
		this._selected[id] = false;
		// Update entry count
		this._entryCount++;
		
		// Setup focus on the first entry in the container if the user hasn't explicitly selected an entry yet.
		if ((this._lastSelectedEntryID === null) && 
				(id === this._dataArray[0].id)) {
			this._setFocus(entryNode, false);
		}
		
	},
	
	/*
     * Sets the default sort order defined by sortOrder to "None" (i.e. 2) and sorts the entries 
     * using the specified custom sort function, comparator, and the standard JavaScript array sort.
     * See: https://developer.mozilla.org/en/JavaScript/Reference/Global_Objects/Array/sort
     * 
     * @param Function comparator(Object a, Object b)
     * 				Object a
     *       			. String label
     *       			. Object value
     *       			. String id
     *       			. Number type 
     *   			Object b
     *       			. String label
     *       			. Object value
     *       			. String id
     *       			. Number type 
     * 
     * @return Number Returns -1, if a is less than b. 1 if a is greater than b. 0 if a is equal to b. 
     */	
	sort: function(comparator) {
		// TODO: Should be integrated into sortOrder = CUSTOM
		// disable automatic sorting
		this.sortOrder = SORT_ORDER_NONE;
		// sort using the provided comparator
		this._dataArray.sort(comparator);
		// reorder the children off-dom
		this.domNode.removeChild(this._contentArea);
		for (var i = 0; i < this._dataArray.length; i++) {
			this._contentArea.appendChild(this._domMap[this._dataArray[i].id]);
		}
		this.domNode.appendChild(this._contentArea);
	},
	
	/*
 	 * @param Element parent
 	 * @param String iconPath
 	 * 
 	 * @return void
 	 */	
	_addIcon: function(parent, iconPath) {
		
		if (!iconPath) {
			return;	
		}
		
		var icon = document.createElement("span");
		dojo.addClass(icon, "entry-icon");
		var image = document.createElement("img");
		dojo.addClass(image, "entry-image");
		image.src = iconPath;
		image.alt = this.messages.msgIcon;
		
		icon.appendChild(image);	
		parent.appendChild(icon);	
	},	
	
	/*
 	 * Private Method
 	 * 
 	 * @param Element parent
 	 * @param String labelText
 	 * @return void
 	 */	
	_addLabel: function(parent, labelText) {
		var label = document.createElement("span");
		dojo.addClass(label, "entry-label");
		// TODO: Sanitize
		label.innerHTML = labelText;
		
		parent.appendChild(label);
	},		
	
	/*
 	 * Private Method
 	 * 
 	 * @param Element parent
 	 * @param Element targetChild
 	 * @param Object targetEntry
 	 * 				. String label
 	 * 				. Object value
 	 * 				. String id
 	 * 				. Number type
 	 * 
 	 * @return void
 	 */	
	_addNode: function(parent, targetChild, targetEntry) {

		if (parent.childNodes.length === 0) {
			// Only one entry
			parent.appendChild(targetChild);
			this._dataArray.push(targetEntry);
		} else if (this.sortOrder === SORT_ORDER_ASCENDING) {
			// Ascending
			this._addNodeInOrder(parent, targetChild, targetEntry, this._ascendingCompareFunction);
		} else if (this.sortOrder === SORT_ORDER_DESCENDING) {
			// Descending
			this._addNodeInOrder(parent, targetChild, targetEntry, this._descendingCompareFunction);
		} else {
			// None
			parent.appendChild(targetChild);
			this._dataArray.push(targetEntry);
		}		
		
	},		
	
	/*
 	 * Private Method
 	 * 
 	 * @param Element parent
 	 * @param Element targetChild
 	 * @param Object targetEntry
 	 * 				. String label
 	 * 				. Object value
 	 * 				. String id
 	 * 				. Number type
 	 * @param Function compareFunction(
 	 * 						Object currentEntry,
 	 *								. String label
 	 * 								. Object value
 	 * 								. String id
 	 * 								. Number type
 	 * 						Object targetEntry
 	 * 								. String label
 	 * 								. Object value
 	 * 								. String id
 	 * 								. Number type
 	 * 					)
 	 * 
 	 * @return void
 	 */	
	_addNodeInOrder: function(parent, targetChild, targetEntry, compareFunction) {

		var children = parent.childNodes;

		// Binary insert
		var low = 0;
		var high = children.length;
		while (low < high) {
			var mid = Math.floor((low + high) / 2);
			
			var currentChild = children[mid];
			var currentEntry = this._dataMap[dojo.attr(currentChild, "entryid")];
			
			if (compareFunction(this, currentEntry, targetEntry)) {
				low = mid + 1; 
			} else {
				high = mid; 
			}
		}
		
		if (this._isObjectDefined(children[low])) {
			parent.insertBefore(targetChild, children[low]);
			this._dataArray.splice(low,0,targetEntry);
		} else {
			parent.appendChild(targetChild);
			this._dataArray.push(targetEntry);
		}

	},		
	
	/*
 	 * Private Method
 	 * 
 	 * @param Object context
 	 * @param Object currentEntry
 	 * 				. String label
 	 * 				. Object value
 	 * 				. String id
 	 * 				. Number type
 	 * @param Object targetEntry
 	 * 				. String label
 	 * 				. Object value
 	 * 				. String id
 	 * 				. Number type
 	 * 
 	 * @return void
 	 */		
	_ascendingCompareFunction: function(context, currentEntry, targetEntry) {
		if (currentEntry[context.sortField] < targetEntry[context.sortField]) {
			return true;
		}
		return false;
	},	
	
	/*
 	 * Private Method
 	 * 
 	 * @param Object context
 	 * @param Object currentEntry
 	 * 				. String label
 	 * 				. Object value
 	 * 				. String id
 	 * 				. Number type
 	 * @param Object targetEntry
 	 * 				. String label
 	 * 				. Object value
 	 * 				. String id
 	 * 				. Number type
 	 * 
 	 * @return void
 	 */		
	_descendingCompareFunction: function(context, currentEntry, targetEntry) {
		if (currentEntry[context.sortField] > targetEntry[context.sortField]) {
			return true;
		}
		return false;
	},		

	/*
 	 * @param Object entry
 	 * 				. String label
 	 * 				. Object value
 	 * 				. String id
 	 * 				. Number type
 	 * @param Boolean ctrlKey
 	 * @param Boolean shiftKey
 	 * 
 	 * @return void
 	 */	
	_selectEntry: function(entry, ctrlKey, shiftKey) {
		
		// Determine selection
		var doSelect = true;
		if (this._isSelected(entry)) {
			doSelect = false;
		}
		
		// Clear previous selection unless CTRL is being pressed
		if (!this.allowMultipleSelections || 
			!ctrlKey) {
			this._removeAllSelections();
		}
		
		// Do selection
		this._setSelected(entry, doSelect);

		if (this.allowMultipleSelections && 
			shiftKey) {
			// Perform multi-selection
			this._doMultiSelectShiftKey(entry.id);
		} else {
			// Remember this entry as the last selection
			this._lastSelectedEntryID = entry.id;
		}

		// Fire event
		this._fireSelectionEvent();	
		
	},		
	
	/*
 	 * Private Method
 	 * 
 	 * @param String targetSelectedEntryID
 	 * 
 	 * @return void
 	 */	
	_doMultiSelectShiftKey: function(targetSelectedEntryID) {
		// Get current entry DOM nodes
		var children = this._contentArea.childNodes;

		if (children.length <= 1) {
			// Not enough entries to do multi-select
			return;
		}
		// Entries >= 2
		
		if (this.getEntry(this._lastSelectedEntryID) === null) {
			// No last selected entry
			// Use the first entry as the last selected entry
			this._lastSelectedEntryID = children[0].entryid;
		}

		if (this._lastSelectedEntryID === targetSelectedEntryID) {
			// No range to select, the start and end entries are the same
			// Select the single entry
			this._setSelected(this.getEntry(targetSelectedEntryID), true);
			return;
		}

		var foundLastSelectedEntry = false;
		var foundTargetSelectedEntry = false;
		var doSelect = false;
		
		// Find the start and end entries and select the intermediate range
		for (var index = 0; index < children.length; index++) {
			// Get the ID of the current entry
			var currentEntryID = dojo.attr(children[index], 'entryid');
			
			if (currentEntryID === this._lastSelectedEntryID) {
				// Found the start or end entry of the selection range
				foundLastSelectedEntry = true;
				// Select it
				this._setSelected(this.getEntry(currentEntryID), true);
				// Set flag to select all intermediate entries encountered
				// after this
				doSelect = true;
				// Exit if this is the end entry of the selection range
				if (foundTargetSelectedEntry) {
					break;
				}
			} else if (currentEntryID === targetSelectedEntryID) {
				// Found the start or end entry of the selection range
				foundTargetSelectedEntry = true;
				// Select it
				this._setSelected(this.getEntry(currentEntryID), true);
				// Set flag to select all intermediate entries encountered
				// after this
				doSelect = true;
				// Exit if this is the end entry of the selection range
				if (foundLastSelectedEntry) {
					break;
				}
			} else if (doSelect) {
				// Start entry of the selection range was found 
				// Select intermediate entry
				this._setSelected(this.getEntry(currentEntryID), true);
			}
		}
	},		

	/*
 	 * @param Event event
 	 * 
 	 * @return boolean
 	 */	
	_handleEventDoubleClickEntry: function(event) {

		if (!event) {
			return false;
		}
		dojo.stopEvent(event);
		
		var node = this._getOriginalTargetEntryNode(event);
		if (!node) {
			// Unexpected error
			return false;
		}		
		var entry = this._getEntry(node);
		if (!entry) {
			// Unexpected error
			return false;
		}		
		
		// Execute callback
		this.ondblClick(entry);

		return true;
	},		

	/*
 	 * @param Event event
 	 * 
 	 * @return boolean
 	 */	
	_handleEventOnMouseDown: function(event) {
		
		if (!event) {
			return false;
		}
		dojo.stopEvent(event);
		
		var node = this._getOriginalTargetEntryNode(event);
		if (!node) {
			// Unexpected error
			return false;
		}		
		var entry = this._getEntry(node);
		if (!entry) {
			// Unexpected error
			return false;
		}

		this._selectEntry(entry, event.ctrlKey, event.shiftKey);
		this._setFocus(node, true);
		
		// Execute callback
		// TODO: Deprecated. Remove post 4.0
		if (this.onClick) {
			this.onClick(entry);
		}

		return true;
	},		
	
	/*
 	 * @param Event event
 	 * 
 	 * @return boolean
 	 */	
	_handleEventOnKeyDown: function(event) {
		
		if (!event) {
			// Unexpected error
			return false;
		}
		
		var keyCode = event.keyCode;
		var keys = dojo.keys;
		var entry;
		var node = this._getOriginalTargetEntryNode(event);
		if (!node) {
			// Unexpected error
			return false;
		}		
		
		if (keyCode === keys.UP_ARROW) {
			// UP

			dojo.stopEvent(event);
			
			node = node.previousSibling;
			if (!node) {
				// No previous sibling
				return true;
			}					
			entry = this._getEntry(node);
			if (!entry) {
				// Unexpected error
				return false;
			}			
			// NOTE: If the CTRL key is being pressed, preserve the existing selection 
			// and do not select the previous entry (just move entry focus)
			if (!event.ctrlKey) {
				this._selectEntry(entry, event.ctrlKey, event.shiftKey);
			}	
			this._setFocus(node, true);
			
		} else if (keyCode === keys.DOWN_ARROW) {
			// DOWN

			dojo.stopEvent(event);
			
			node = node.nextSibling;
			if (!node) {
				// No next sibling
				return true;
			}					
			entry = this._getEntry(node);
			if (!entry) {
				// Unexpected error
				return false;
			}			
			// NOTE: If the CTRL key is being pressed, preserve the existing selection 
			// and do not select the next entry (just move entry focus)
			if (!event.ctrlKey) {
				this._selectEntry(entry, event.ctrlKey, event.shiftKey);
			}	
			this._setFocus(node, true);
			
		} else if (keyCode === keys.SPACE) {
			// SPACE
			
			dojo.stopEvent(event);
			
			entry = this._getEntry(node);
			if (!entry) {
				// Unexpected error
				return false;
			}
			this._selectEntry(entry, event.ctrlKey, event.shiftKey);
		}			
		
		return true;
	},	
	
	/*
 	 * @param Event event
 	 * 
 	 * @return Element
 	 */	
	_getOriginalTargetEntryNode: function(event) {
		// Get target
		var target = event.target;
		
		while (target) {
			// TEST
			//console.debug(target);
			
			if (target === this._contentArea) {
				return null;
			}
			
			if (this._isEntryNode(target)) {
				return target;
			}
			// Traverse up DOM hierarchy
			target = target.parentNode;
			
		}
		
		return null;
	},		
	
	/*
 	 * @param Element node
 	 * 
 	 * @return Boolean
 	 */	
	_isEntryNode: function(node) {
		return dojo.hasClass(node, "entry");
	},		
	
	/*
 	 * @param Event event
 	 * 
 	 * @return Element
 	 */	
	_getCurrentTargetEntryNode: function(event) {
		// Get target
		var target = event.currentTarget;
		if (!target) {
			return null;
		}
		return target;
	},		
	
	/*
 	 * @param EventTarget target
 	 * 
 	 * @return Object
 	 */	
	_getEntry: function(target) {
		// Get ID
		var id = dojo.attr(target, "entryid");
		// Get entry
		var entry = this._dataMap[id];
		return entry;
	},	
	
	/*
 	 * @param Event event
 	 * 
 	 * @return boolean
 	 */	
	_cancelEvent: function(event) {
		if (event) {
			dojo.stopEvent(event);
		}
		// Prevent text selection on IE during multi-selection:
		// onselectstart
		// ondragstart
		
		// Prevent accidental link navigation during entry multi-select:
		// onclick
		
		return false;
	},
	
	/*
 	 * Public Method
 	 * 
 	 * @param Array entries : Object
 	 * 					. String label
 	 * 					. Object value
 	 * 					. String id
 	 * 					. Number type
 	 * @return void
 	 */	
	addEntries: function(entries) {
		for (var index = 0; index < entries.length; index++) {
			this.addEntry(entries[index]);
		}
	},	

	/*
 	 * Public Method
 	 * 
 	 * @param Object entry
 	 * 				. String label
 	 * 				. Object value
 	 * 				. String id
 	 * 				. Number type
 	 * @return void
 	 */	
	removeEntry: function(entry) {
		// Validate list		
		if (!this.domNode) {
			return;
		}
		
		var id = entry.id;
		// Check if this operation is applicable
		if (this._isObjectDefined(this._dataMap[id]) === false) {
			// NO-OP
			return;
		}
		// Remove from the ordered representation
		for (var i = 0; i < this._dataArray.length; i++) {
			if (this._dataArray[i].id === id) {
				this._dataArray.splice(i,1);
				break;
			}
		}
		// Remove from list
		var node = this._domMap[id];
		this._contentArea.removeChild(node);
		// Remove from data map
		this._dataMap[id] = undefined;
		delete this._dataMap[id];
		// Remove from DOM map		
		this._domMap[id] = undefined;
		delete this._domMap[id];
		// Remove from selection map
		this._selected[id] = undefined;
		delete this._selected[id];
		// Update entry count
		this._entryCount--;

		// Update selection
		if (this._lastSelectedEntryID === id) {
			if (this.allowMultipleSelections) {
				this._lastSelectedEntryID = this._getNextSelectedEntryID();
			} else {
				this._lastSelectedEntryID = null;
			}
		}
		// Update focus
		if (this._isFocused(node)) {
			// Setup focus on the first entry in the container
			var newFocusNode = this._getFirstEntryNode();
			this._setFocus(newFocusNode, false);
		}			
		
	},

	/*
 	 * Public Method
 	 * 
 	 * @param Array entries : Object
 	 * 					. String label
 	 * 					. Object value
 	 * 					. String id
 	 * 					. Number type
 	 * @return void
 	 */	
	removeEntries: function(entries) {
		for (var index = 0; index < entries.length; index++) {
			this.removeEntry(entries[index]);
		}
	},

	/*
 	 * Public Method
 	 * 
 	 * @param void
 	 * @return void
 	 */	
	removeAllEntries: function() {
		for (var id in this._dataMap) {
			this.removeEntry(this._dataMap[id]);
		}
	},

	/*
 	 * Public Method
 	 * 
 	 * @param Object entry
 	 * 				. String label
 	 * 				. Object value
 	 * 				. String id
 	 * 				. Number type
 	 * @param Boolean show
 	 * 
 	 * @return void
 	 */	
	showEntry: function(entry, show) {
		// Validate list		
		if (this._isObjectDefined(this.domNode) === false) {
			return;
		}
		
		var id = entry.id;
		// Check if this operation is applicable
		if (this._isObjectDefined(this._dataMap[id]) === false) {
			// NO-OP
			return;
		}

		var node = this._domMap[id];
		this._updateVisibility(node, show);	
		
	},

	/*
 	 * @param void
 	 * 
 	 * @return Number
 	 */	
	getEntryCount: function() {
		return this._entryCount;
	},
	
	/*
 	 * Get an entry from the list box.
 	 * 
 	 * @param String id
 	 * 
 	 * @return Object
 	 * 				. String label
 	 * 				. Object value
 	 * 				. String id
 	 * 				. Number type
 	 */	
	getEntry: function(id) {
		if (this._dataMap.hasOwnProperty(id) &&
				this._dataMap[id]) {
			return this._dataMap[id];
		}
		return null;
	},	

	/*
	 * Returns an ordered array of entries in the control that correspond to the specified array of zero-relative indices.  
	 * The indices are based on the sort order of the entries in the control and DOM. 
	 * An empty array indicates that there are no entries corresponding to the indices.
	 * 
 	 * @param Number[] indices
 	 * 
 	 * @return Object[]
 	 * 					. String label
 	 * 					. Object value
 	 * 					. String id
 	 * 					. Number type
 	 */	
	getEntriesByIndices: function(indices) {
		var entryList = [];
		var lastIndex = this._dataArray.length - 1;
		for (var i = 0; i < indices.length; i++) {
			var index = indices[i];
			if (isNaN(index) ||
					(index < 0) ||
					(index > lastIndex)) {
				console.error("[ERROR]: " + this.id + "#getEntriesByIndices: " + "Index out of bounds: " + index);
				return [];
			}
			var entry = this._dataArray[index];
			entryList.push(entry);
		}
		return entryList;
	},	
	
	/*
 	 * Get all entries from the list box.
 	 * 
 	 * @param void
 	 * 
 	 * @return Object[]
 	 * 					. String label
 	 * 					. Object value
 	 * 					. String id
 	 * 					. Number type
 	 */	
	getAllEntries: function() {
		return this._dataArray.slice(0);
	},	
	
	/*
	 * Returns an array of entries that are currently selected in the control.  
	 * An empty array indicates that there are no entries selected.
	 * 
 	 * @param void
 	 * 
 	 * @return Object[]
 	 * 					. String label
 	 * 					. Object value
 	 * 					. String id
 	 * 					. Number type
 	 */	
	getSelection: function() {
		var entryList = [];
		for (var index = 0; index < this._dataArray.length; index++) {
			var entry = this._dataArray[index];
			if (entry && 
					this._selected.hasOwnProperty(entry.id) &&
					this._selected[entry.id]) {
				entryList.push(entry);
			}
		}
		return entryList;
	},		
	
	/*
	 * Returns an array of zero-relative indices of entries that are currently selected in the control.  
	 * The indices are based on the sort order of the entries in the control and DOM. 
	 * An empty array indicates that there are no entries selected.
	 * 
 	 * @param void
 	 * 
 	 * @return Number[]
 	 */	
	getSelectionIndices: function() {
		var entryIndices = [];
		for (var index = 0; index < this._dataArray.length; index++) {
			var entry = this._dataArray[index];
			if (entry && 
					this._selected.hasOwnProperty(entry.id) &&
					this._selected[entry.id]) {
				entryIndices.push(index);
			}
		}
		return entryIndices;
	},		
	
	/*
 	 * Selects the specified entries in the control.
 	 * The current selection is cleared before the new entries are selected.
 	 * If the control is single-select and multiple entries are specified,
 	 * then all entries are ignored.
 	 * Note: No selection event ('onChange') is fired.
 	 * 
 	 * @param entries[] : Object
 	 * 						. String label
 	 * 						. Object value
 	 * 						. String id
 	 * 						. Number type
 	 * @param Boolean select
 	 * 
 	 * @return void
 	 */	
	setSelection: function(entries, select) {
		
		if (!this.allowMultipleSelections &&
				(entries.length > 1)) {
			// Invalid input
			return;
		}		

		// Clear previous selection
		this._removeAllSelections();

		// Do selection
		for (var index = 0; index < entries.length; index++) {
			this._setSelected(entries[index], select);
		}

		this._lastSelectedEntryID = this._getNextSelectedEntryID();

	},		

	/*
 	 * @param void
 	 * 
 	 * @return void
 	 */	
	_fireSelectionEvent: function() {
		var event = new jazz.ui.ListBox.SelectionEvent({
			widget: this
		});
		this.onChange(event);
	},		
	
	/*
 	 * @param Object entry
 	 * 					. String label
 	 * 					. Object value
 	 * 					. String id
 	 * 					. Number type
 	 * @param boolean select
 	 * @return void
 	 */	
	_setSelected: function(entry, select) {
		var id = entry.id;
		// Check if this operation is applicable
		if (this._isObjectDefined(this._dataMap[id]) === false) {
			// NO-OP
			return;
		}
		// Check if a toggle is necessary
		if (this._selected[id] == select) {
			// NO-OP
			return;
		}
		
		// Store selection		
		this._selected[id] = select;
		// Update UI
		var node = this._domMap[id];
		if (select) {
			dojo.addClass(node, "selected");
		} else {
			dojo.removeClass(node, "selected");
		}
		dijit.setWaiState(node, "selected", String(select));
		
	},	

	/*
 	 * @param void
 	 * 
 	 * @return String
 	 */	
	_getNextSelectedEntryID: function() {
		for (var id in this._selected) {
			if (!this._selected.hasOwnProperty(id)) {
				continue;
			}
			if (this._selected[id]) {
				return id;
			}
		}
		return null;
	},	
	
	/*
 	 * @param Object entry
 	 * 					. String label
 	 * 					. Object value
 	 * 					. String id
 	 * 					. Number type
 	 * @return boolean
 	 */	
	_isSelected: function(entry) {
		if (this._selected[entry.id]) {
			return true;
		}
		return false;
	},	

	/*
	 * Clears the current selection in the control.
 	 * Note: No selection event ('onChange') is fired.
 	 *  
 	 * @param void
 	 * 
 	 * @return void
 	 */	
	removeAllSelections: function() {
		this._removeAllSelections();
		this._lastSelectedEntryID = null;
	},	

	/*
 	 * @param void
 	 * 
 	 * @return void
 	 */	
	_removeAllSelections: function() {
		for (var id in this._selected) {
			if (!this._selected.hasOwnProperty(id)) {
				continue;
			}
			if (this._selected[id]) {
				// Get entry
				var entry = this._dataMap[id];
				// Unselect it
				this._setSelected(entry, false);
			}
		}		
	},		
	
	/*
 	 * Public Method
 	 * 
 	 * @param boolean show
 	 * @return void
 	 */	
	showLoading: function(show) {
		if (this._isObjectDefined(this.domNode) === false) {
			// Not initialized
			return;
		} else if (this._loadingFlag === show) {
			// Action already done
			return;
		} else if (show) {
			// Show message area
			this._showMessageArea();
		} else {
			// Show content area
			this._showContentArea();
		}
		
		// Update flag
		this._loadingFlag = show;
	},	

	/*
 	 * Private Method
 	 * 
 	 * @param void
 	 * @return void
 	 */	
	_showMessageArea: function() {
		// Hide the content area
		this._updateVisibility(this._contentArea, false);
		// Show the message area		
		this._updateVisibility(this._messageArea, true);		
	},	

	/*
 	 * Private Method
 	 * 
 	 * @param void
 	 * @return void
 	 */	
	_showContentArea: function() {
		// Hide the message area
		this._updateVisibility(this._messageArea, false);	
		// Show the content area
		this._updateVisibility(this._contentArea, true);
	},	

	/*
 	 * Private Method
 	 * 
 	 * @param Element node
 	 * @param boolean show
 	 * @return void
 	 */	
	_updateVisibility: function(node, show) {
		if (show) {
			// Show
			if (node.style.display != "block") {
				node.style.display = "block";
			}
		} else {
			// Hide
			if (node.style.display != "none") {
				node.style.display = "none";
			}
		}
	},
		
	/*
 	 * Private Method
 	 * 
 	 * @param Object variable
 	 * @return boolean
 	 */	
	_isObjectDefined: function(variable) {
	    if (typeof(variable) != 'object') {
			return false;
		} else if (variable === null) {
			return false;
		}
		return true;
	},		
	
	/*
 	 * @param void
 	 * 
 	 * @return void
 	 * @overrides super
 	 */		
	destroy: function() {

		this.inherited(arguments);
	}
	
});

/*
 * Instances of this class are sent as a result of entries being selected in the 
 * List Box widget. 
 *
 * PROVISIONAL: This API is considered provisional and may be changed or removed in the
 * future with minimal or no warning.
 * 
 * @since 4.0
 */
dojo.declare("jazz.ui.ListBox.SelectionEvent", null, {
	
	/*
	 * The ListBox widget in which the selection occurred.
	 */		
	widget: null,
	
	/*
 	 * @param Object args
 	 * 
 	 * @return SelectionEvent
 	 */
	constructor: function(args) {
		this.widget = args.widget;
	}	
	
});	

})();
