/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("jazz.ui.ResourcePicker");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");

dojo.require("jazz.ui.Dialog");
dojo.require("jazz.ui.MessageBox");
dojo.require("jazz.ui.Select");
dojo.require("jazz.ui.menu.MenuItem");
dojo.require("jazz.ui.RemoteChooserContainer");

dojo.require("jazz.core.loader");

dojo.require("dojo.string");
dojo.require("dojo.i18n");
dojo.requireLocalization("jazz.ui", "ResourcePickerMessages");

(function() {

var messages;

dojo.declare("jazz.ui.ResourcePicker",[dijit._Widget, dijit._Templated], {
	
	title: "",
	secondaryTitle: "",
	artifactName: "",
	linkTypes: [],
	onResult: function(typeId, result) {},
	getContainers: function(linkTypeId) {},
	getDialogs: function(containerId) {},
	
	templatePath: dojo.moduleUrl("jazz.ui", "templates/ResourcePicker.html"),
	widgetsInTemplate: true,
	
	constructor: function() {
		if (!messages) messages = dojo.i18n.getLocalization("jazz.ui", "ResourcePickerMessages");
		this._loadingMessage = messages.loading;
		this.dialog = new jazz.ui.Dialog({
			width: "",
			destroyContents: true
		});
	},
	
	postMixInProperties: function() {
		 if (!this.title)
		 	this.title = messages.title;
	},
	
	buildRendering: function() {
		this.inherited(arguments);
		dojo.place(this.domNode, this.dialog.contentNode, "replace");
		this.dialog.updatePosition();
	},
	
	_setTitleAttr: function(title) {
		this.title = title;
		this.dialog.setTitle(title);
	},
	
	_setSecondaryTitleAttr: function(secondaryTitle) {
		this.secondaryTitle = secondaryTitle;
		this.dialog.setSecondaryTitle(secondaryTitle);
	},
	
	_setArtifactNameAttr: function(artifactName) {
		this.artifactName = artifactName;
		this.set('secondaryTitle',dojo.string.substitute(messages.secondaryTitle, [artifactName]));
	},
	
	_setLinkTypesAttr: function(linkTypes) {
		this.linkTypes = linkTypes;
		var select = this._linkTypeSelect;
		select.innerHTML = "";
		for (var i = 0; i < linkTypes.length; i++) {
			var t = linkTypes[i];
			var args = {
				innerHTML: t.name,
				value: t.id
			};
			if (t["default"]) args.selected = "selected";
			dojo.create("option", args, select);
		}
		this._onLinkTypeChange();
	},
	
	_onLinkTypeChange: function(e) {
		var value = this._linkTypeSelect.value;
		if (!value) return;
		var select = this._projectSelect;
		select.set({
			'defaultText': messages.loading,
			'enabled': false,
			'selected': null
		});
		select.destroyDescendants();
		dojo.when(this.getContainers(value), function(containers) {
			for (var i = 0; i < containers.length; i++) {
				var c = containers[i];
				var args = {
					label: c.name,
					pid: c.id
				};
				if (c.icon) args.icon = c.icon;
				if (c.iconClass) args.iconClass = c.iconClass;
				select.addChild(new jazz.ui.menu.MenuItem(args));
			}
			select.set({
				'defaultText': messages.selectContainer,
				'enabled': true
			});
		});
	},
	
	_onProjectChange: function() {
		var value = this._projectSelect.selected;
		this._messageBox.hideMessage(true);
		delete this.dialogs;
		dojo.addClass(this._usageRow, "hidden");
		if (!value) return;
		dojo.when(this.getDialogs(value.pid), dojo.hitch(this, function(dialogs) {
			this.dialogs = dialogs;
			var hasSelect = dialogs.select && dialogs.select.length ? true : false,
				hasCreate = dialogs.create && dialogs.create.length ? true : false,
				selectOption = this._inputSelect,
				createOption = this._inputCreate;
			if (!hasSelect) {
				if (selectOption.checked) {
					this._showWarning(messages.selectNotAvailable);
				}
				selectOption.checked = false;
				selectOption.disabled = true;
			} else {
				selectOption.disabled = false;
			}
			if (!hasCreate) {
				if (createOption.checked) {
					this._showWarning(messages.createNotAvailable);
				}
				createOption.checked = false;
				createOption.disabled = true;
			} else {
				createOption.disabled = false;
			}
			if (hasSelect && !hasCreate) selectOption.checked = "checked";
			if (!hasSelect && hasCreate) createOption.checked = "checked";
			this._populateUsage();
		}));
	},
	
	_populateUsage: function() {
		var dialogs = this._getAvailableDialogs();
		if (!dialogs) return;
		var select = this._usageSelect;
		select.innerHTML = "";
		for (var i = 0; i < dialogs.length; i++) {
			var d = dialogs[i];
			dojo.create("option", {
				innerHTML: d.name,
				value: i
			}, select);
		}
		dojo.toggleClass(this._usageRow, "hidden", dialogs.length === 1);
		this._onUsageChange();
	},
	
	_onUsageChange: function() {
		var value = this._usageSelect.value;
		if (!value) return;
		if (this.currentContent) {
			this.currentContent.destroy();
			delete this.currentContent;
		}
		var dialog = this._getAvailableDialogs()[parseInt(value)];
		this.currentDialog = dialog;
		dojo.style(this._contentArea,{
			width: dialog.initialWidth || "",
			height: dialog.initialHeight || ""
		});
		if (dialog.widgetId) {
			this._showLoading();
			jazz.core.loader.load_async(dialog.widgetId, dojo.hitch(this,function() {
				var Widget = eval(dialog.widgetId);
				this.currentContent = new Widget().placeAt(this._contentArea, "only");
				this.connect(this.currentContent,"onResult",this._handleResult);
				this.connect(this.currentContent,"onResize",this._onResize);
			}));
		} else if (dialog.url) {
			this.currentContent = new jazz.ui.RemoteChooserContainer({
				remoteLocation: dialog.url,
				mode: (dialog.resultNamespace === "oslc") ? "OSLC2" : "OSLC1",
				onData: dojo.hitch(this,this._onData),
				onResize: dojo.hitch(this,this._onResize)
			}).placeAt(this._contentArea, "only");
		}
		this.dialog.updatePosition();
	},
	
	_onResize: function(width, height) {
		if (!this.currentContent)
			return;
		dojo.style(this._contentArea,{
			width: width || "",
			height: height || ""
		});
		this.dialog.updatePosition();
	},
	
	_onData: function(data) {
		var result= [];
		if (data.length !== 0) {
			var response= dojo.fromJson(data);
			var namespace = this.currentDialog.resultNamespace;
			var resourceKey= "rdf:resource";
			var labelKey= namespace + ":label";
			var newItemsElem= response[namespace + ":results"];
			if (!dojo.isArray(newItemsElem) && namespace === "oslc_rm") {
				newItemsElem= response["http://open-services.net/xmlns/rm/1.0/web/results"];
				resourceKey= "http://www.w3.org/1999/02/22-rdf-syntax-ns#resource";
				labelKey= "http://www.w3.org/2000/01/rdf-schema#label";
			}
			if (dojo.isArray(newItemsElem)) {
				var newItems= newItemsElem;
				if (newItems.length > 0) {
					for (var i= 0; i < newItems.length; i++){
						var item= newItems[i];
						var elem= {};
						elem.url= item[resourceKey];
						elem.label= item[labelKey];
						result.push(elem);
					}
				}
			}
		}
		this._handleResult(result);
	},
	
	_handleResult: function(result) {
		this.onResult(this._linkTypeSelect.value, result);
		this.dialog.destroy();
	},
	
	_getAvailableDialogs: function() {
		if (!this.dialogs) return;
		return this._inputSelect.checked ? this.dialogs.select : this.dialogs.create;
	},
	
	_showLoading: function() {
		this._contentArea.innerHTML = "<div class='loading'>" + messages.loading + "</div>";
	},
	
	_showWarning: function(message) {
		this._messageBox.showMessage({
			type: jazz.ui.MessageBox.STATUS_TYPES.WARNING,
			summary: message,
			closable: true
		});
	}
	
});

})();
