/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008, 2011. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("jazz.ui.ScrollableMenu");

dojo.require("jazz.ui.Menu");
dojo.require("dijit._Templated");
dojo.require("dijit._Widget");

(function () {
	
var buttonHeight = 12;

/**
 * Provides a scrollable menu.  API is the same as a jazz menu
 * however optional "maxHeight" parameter can be passed into constructor (default is 500px) 
 * which when exceeded causes scroll buttons to appear allowing the user to access the menu
 * items that would normally be outside the viewport.   
 * 
 * e.g. new jazz.ui.ScrollableMenu({maxHeight: 400});
 * 
 * @name jazz.ui.ScrollableMenu
 * @class <p>A scrollable menu widget for web UIs.</p>
 * 	<p>PROVISIONAL: This API is considered provisional and may be changed or removed in the
 * 	future with minimal or no warning.</p>
 * @constructor
 * @extends jazz.ui.Menu
 * @since 3.0
 */ 
dojo.declare("jazz.ui.ScrollableMenu", [jazz.ui.Menu], {
	
	/**
	 * Specify the maxHeight of the menu in pixels.
	 * @type Number
	 * @default 500
	 */
	maxHeight: 500,

	/**
	 * Specify the minHeight of the menu in pixels.
	 * @type Number
	 * @default 500
	 */
	minHeight: 120,
	
	// internals - do not call
	
	templateString: "<div class=\"jazz-ui-ScrollableMenu jazz-ui-Menu scrolledToTop\" dojoattachevent=\"onclick: _stopEvent\">"+
	    "  <div class=\"jazz-ui-Menu-border\"><div dojoAttachPoint=\"scrollUpButton\" dojoAttachEvent=\"onselectstart:_stopEvent,onmouseover:_scrollUp,onclick:_scrollUp,onmouseout:_clearTimeout\" class=\"scrollButton up\" tabIndex=\"0\"><span class=\"arrow\">&nbsp;</span></div>"+
		"  <div dojoAttachPoint=\"menuWrapper\" class=\"menuWrapper wrapper\">"+
		"     <table dojoattachpoint=\"wrappedMenu\" class=\"dijit dijitMenu dijitMenuPassive dijitReset dijitMenuTable\" waiRole=\"menu\" tabIndex=\"${tabIndex}\" dojoAttachEvent=\"onkeypress:_onKeyPress\">\n\t<tbody class=\"dijitReset\" dojoAttachPoint=\"containerNode\"></tbody>\n</table>"+
		"  </div>"+
		"  <div dojoAttachPoint=\"scrollDownButton\" dojoAttachEvent=\"onselectstart:_stopEvent,onmouseover:_scrollDown,onclick:_scrollDown,onmouseout:_clearTimeout\" class=\"scrollButton down\" tabIndex=\"0\"><span class=\"arrow\">&nbsp;</span></div></div>"+
		"</div>",
	
	
	postCreate: function() {
		this.inherited(arguments);
		//create scrolling pane
		this.connect(dijit.popup, "open", function (args) {
			if (args.popup === this) {
				if (args.around) {
					if (!args.orient) {
						args.orient = (args.parent ? args.parent.isLeftToRight() : dojo._isBodyLtr()) ?
							{'BL':'TL', 'BR':'TR', 'TL':'BL', 'TR':'BR'} :
							{'BR':'TR', 'BL':'TL', 'TR':'BR', 'TL':'BL'}
					}
					this._popupArgs = args;
					dijit.placeOnScreenAroundElement(args.popup.domNode.parentNode, args.around, args.orient, dojo.hitch(this,this.orient));					
				} else {
					var maxHeight = Math.min(this.maxHeight, dijit.getViewport().h),
						wrapperHeight = this.wrappedMenu.clientHeight;
					this._setHeight(maxHeight > wrapperHeight ? null : Math.max(maxHeight, this.minHeight) - 2 * buttonHeight);
				}
			}
		});

		// handle scroll wheel
		this.connect(this.domNode, (!dojo.isMozilla ? "onmousewheel" : "DOMMouseScroll"), this._onMouseWheel);
		this._setHeight();
	},
	
	_marginTop: 0,

	_count: 0,
	
	// called by dijit.popup when the menu is placed, and by jazz.ui.MenuPopup onresize and onscroll.
	orient: function (node, aroundNodeCorner, nodeCorner, size, aroundNodeCoords){
		if (!this._popupArgs) return; //wait until the menu is placed, then orient.
		var viewport = dijit.getViewport(),
			bodyHeight = viewport.h,
			aroundPos = dojo.position(dojo.byId(node.getAttribute("dijitpopupparent"))),
			wrapperHeight = this.wrappedMenu.clientHeight,
			minHeight = this._height || this._maxScroll == buttonHeight ? this.minHeight : this.domNode.clientHeight;
			placeAbove = (bodyHeight - aroundPos.y + aroundPos.h + viewport.t < minHeight + 2 * buttonHeight + 11) || (!!this._height && (aroundPos.y > bodyHeight - aroundPos.y + aroundPos.h)),
			maxHeight = Math.min(placeAbove ? aroundPos.y : bodyHeight - (aroundPos.y + aroundPos.h), this.maxHeight, bodyHeight) - buttonHeight;
		this._setHeight(maxHeight > wrapperHeight ? null : Math.max(maxHeight, this.minHeight - buttonHeight));
		if (placeAbove && nodeCorner == "TL") {
			return size.h;
		} else {
			return 0;
		}
	},

	_setHeight: function(height) {
		if (height && height == this._height) return;
		this._height = height;
		var menuWrapper = this.menuWrapper, 
			marginTop = -this._marginTop;
		if (!this._height) {
			dojo.style(menuWrapper, {"height": "", "marginTop": ""});
			this._marginTop = 0;
			dojo.addClass(this.domNode, "scrolledToTop");
		} else {
			dojo.style(menuWrapper, "height", (marginTop + this._height) + "px");
		}
		this._maxScroll = this.wrappedMenu.offsetHeight - menuWrapper.offsetHeight + buttonHeight + marginTop;
		dojo.toggleClass(this.domNode, "scrolledToBottom", !height || marginTop >= this._maxScroll);
	},

	_stopEvent: function(e) {
		if (e.target == e.currentTarget || e.target == this.menuWrapper) {
			dojo.stopEvent(e);
			return false;
		}
	},
	
	_onMouseWheel: function(e) {
		if (e[(dojo.isMozilla ? "detail" : "wheelDelta")] * (dojo.isMozilla ? -1 : 1) < 0) {
	   		this._scrollDown(e);
		} else {
			this._scrollUp(e);
		}
	},
	
	_scrollDown: function(event) {
		return this._scrollEvent(event, "down");
	},
	
	_scrollUp: function(event) {
		return this._scrollEvent(event, "up");
	},
	
	_scrollEvent: function(event, dir) {
		if (event.type == "mouseover") {
			//delay on first hover before scrolling
			if (!this._timer) this._timer = setTimeout(dojo.hitch(this, this._scroll, dir, true), 300);
			return;
		} else {
			this._clearTimeout();
		}
		if (!this._scroll(dir, false)) {
			dojo.stopEvent(event);
		}
	},
	
	_canScroll: function(dir) {
		return this._height && ((dir == "down" && -this._marginTop < this._maxScroll) || (dir == "up" && this._marginTop < buttonHeight));
	},
	
	_scroll: function(dir, automate) {
		this._clearTimeout();
		if (!this._canScroll(dir)) {
			return !this._height; //return true if the menu isn't scrollable to prevent scrollwheel events from being stopped.
		}
		//calculate how much to scroll down based on height of top visible item
		var topItem = this.getChildren()[(dir == "down") ? this._count : this._count + 1];
		if (topItem) {
			var nextDelta = topItem.domNode.offsetHeight;
			if (dir == "down" && -this._marginTop + nextDelta >= this._maxScroll - buttonHeight) {
				nextDelta = this._marginTop + this._maxScroll;
			}
		} else {
			nextDelta = buttonHeight;
		}	
		if ((dir == "down") ? -this._marginTop + nextDelta <= this._maxScroll : this._marginTop < buttonHeight) {
			var newMargin = this._marginTop + (dir == "down" ? -nextDelta : nextDelta);
			if (newMargin >= 0) newMargin = buttonHeight;
			dojo.animateProperty({node: this.menuWrapper, onEnd: dojo.hitch(this, this._onEnd), duration: 100,
			    properties: {
					marginTop: {start: this._marginTop, end: newMargin, unit: "px"},
					height: {start: -this._marginTop + this._height, end: -newMargin + this._height, unit: "px"}
			    }
			}).play();
			this._marginTop = newMargin;
			if (dir == "down") {
				this._count++;
			} else {
				this._count--;
			}
		}
		if (automate && this._canScroll(dir)) {
			this._timer = setTimeout(dojo.hitch(this, this._scroll, dir, automate),160);
		}
		return false;
	},
	
	_onEnd: function() {
		dojo.toggleClass(this.domNode, "scrolledToTop", this._marginTop >= buttonHeight);
		dojo.toggleClass(this.domNode, "scrolledToBottom", -this._marginTop >= this._maxScroll);
	},
	
	_clearTimeout: function(event) {
		if (this._timer) {
			clearTimeout(this._timer);
			delete this._timer;
		}
	}
});

})();