/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2012, 2013. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("jazz.app._JspApplication"); //$NON-NLS-1$

dojo.require("jazz.ui.MenuPopup"); //$NON-NLS-1$
dojo.require("jazz.ui.PageTemplate"); //$NON-NLS-1$
dojo.require("jazz.app.internal.HomeMenu"); //$NON-NLS-1$
dojo.require("jazz.app.internal.NavigationMenuBar"); //$NON-NLS-1$
dojo.require("jazz.app.internal.Footer"); //$NON-NLS-1$
dojo.require("jazz.app.internal.BannerButtons"); //$NON-NLS-1$
dojo.require("jazz.app.internal.LogoutArea"); //$NON-NLS-1$
dojo.require("jazz.app.ThemableNode"); //$NON-NLS-1$
dojo.require("jazz.app.internal.BannerSearch"); //$NON-NLS-1$
dojo.require("jazz.core.loader"); //$NON-NLS-1$
dojo.require("net.jazz.ajax.ui._Application"); //$NON-NLS-1$
dojo.require("net.jazz.ajax.ui.PlatformUI"); //$NON-NLS-1$
dojo.require("net.jazz.ajax.ui.internal.PageRegistry"); //$NON-NLS-1$
dojo.require("net.jazz.ajax.ui.PageContainer"); //$NON-NLS-1$
dojo.require("net.jazz.ajax.core.internal.util"); //$NON-NLS-1$
dojo.require("jazz.app.internal.applicationAbout"); //$NON-NLS-1$
dojo.require("jazz.util.browserSupport"); //$NON-NLS-1$
dojo.require("jazz.util.theming"); //$NON-NLS-1$
dojo.require("dojo.i18n"); //$NON-NLS-1$
dojo.requireLocalization("jazz.app.internal", "_AjaxApplicationMessages"); //$NON-NLS-1$ //$NON-NLS-2$

var messages = dojo.i18n.getLocalization("jazz.app.internal", "_AjaxApplicationMessages"); //$NON-NLS-1$ //$NON-NLS-2$
var PlatformUI = net.jazz.ajax.ui.PlatformUI;
var browserSupport = jazz.util.browserSupport;
var applicationAbout = jazz.app.internal.applicationAbout;

dojo.declare("jazz.app._JspApplication", net.jazz.ajax.ui._Application, { //$NON-NLS-1$

	name: "",
	iconSmall: "",
	iconLarge: "",
	applicationId: "",

	config: {	// manipulate the config object before calling start().  
		
		// Additional parameters:
		//
		//   login, logout 			Replace the behaviour of the login/logout links.
		// 							this.config.login = { href: "/someUrl", callback: this.someFunction }
		//
		//
		//   user					Puts the username in the header and customizes menu url defaults. 
		//							{
		//								name: "John Doe", 
		//								userId: "jdoe", 
		//								admin: true, 
		//								menuItemsProvider: function,
		//								userSmallPreviewUrl: string
		//								userProfileUrl: string
		//								allProfilesUrl: string
		//								personalDashboardUrl: string
		//							}
		//
		//	 footer					args:  {
		//								leftLogo: { src: path to image, href (optional) },
		//								rightLogo: { same as above }
		//							}  // both optional
		//
	},

	ui: null,
	
	createHomeMenu: function() {
		if (!this._homeMenu)
			this._homeMenu = new jazz.app.internal.HomeMenu();
		return this._homeMenu;
	},
	
	createBannerSearch: function() {
		return new jazz.app.internal.BannerSearch();
	},
	
	start: function() {
		if (this._deferStart) {
			return;
		} else {
			if (this.started) {
				return;
			}
		}	
		this.started = true;
		var config = this.config;

		jazz.util.theming.initialize();
		this._setupThemingHooks();
		
//		this._footer.init(this.config.footer);
		
		this.inherited(arguments);
	}, 

	postCreate: function() {
		if (this.renderedPageId)
			net.jazz.ajax.ui.PlatformUI.getWorkbench()._alreadyViewingPage(this.renderedPageId);
		this.ui.set("homeMenuProvider", dojo.hitch(this, this.createHomeMenu));
		dojo.subscribe(net.jazz.ajax.internal.events.showMessage, this.ui, "showStatusMessage");
		dojo.subscribe(net.jazz.ajax.internal.events.hideMessageArea, this.ui, "hideStatusMessage");
	},
	
	constructor: function(args) {
		dojo.setObject("jazz.app.currentApplication", this);
		if (args.serverTemplated) {
			this.workbench = net.jazz.ajax.ui.PlatformUI.runWorkbench(args.applicationId);
		} else {
			var support = browserSupport.getSupportLevel();
			if (support != browserSupport.SUPPORTED) {
				if (support == browserSupport.NOT_SUPPORTED) {
					this._deferStart = true;
					jazz.core.loader.load_async("jazz.ui.UnsupportedBrowserWarning", dojo.hitch(this, function() {
						var warn = new jazz.ui.UnsupportedBrowserWarning();
						var contConnect = dojo.connect(warn, "onContinue", this, function (){
							dojo.disconnect(contConnect);
							warn.destroy();
							this._deferStart = false;
							if (!this._started) this.start();
						});
						document.body.appendChild(warn.domNode);
					}));
				}
				this._footer.showBrowserWarning();
			}
		}
	},

	_setupThemingHooks: function() {
		var node = dojo.create("div");
		this.ui.set("aboveBanner", node);
		var themeNode = new jazz.app.ThemableNode({
			htmlPath: "html/aboveBanner.html",
			targetNode: node,
			widgetName : "jazz.theme.AboveBannerWidget",
			shouldLookupHTML : jazz.app.theming.currentTheme().aboveBanner
		});
	}
});
})();
