/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2009, 2013. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function() {
	dojo.provide("com.ibm.team.process.authoring.web.common.common.internal.NavigationData"); //$NON-NLS-1$
	dojo.require("com.ibm.team.process.authoring.web.common.common.internal.UIHelper"); //$NON-NLS-1$
	dojo.require("com.ibm.team.process.authoring.web.common.common.internal.ProcessAuthoringConstants"); //$NON-NLS-1$
	dojo.require("dojox.uuid.Uuid"); //$NON-NLS-1$
	dojo.require("dojox.xml.parser"); //$NON-NLS-1$
	var UIHelper = com.ibm.team.process.authoring.web.common.common.internal.UIHelper;
	var ProcessAuthoringConstants = com.ibm.team.process.authoring.web.common.common.internal.ProcessAuthoringConstants;
	var ProcessAuthoringClient = com.ibm.team.process.authoring.web.common.client.internal.ProcessAuthoringClient;
	dojo.declare("com.ibm.team.process.authoring.web.common.common.internal.NavigationData", null, { //$NON-NLS-1$
		jsonData: null,
		eTag: null,
		isOutlineData: false,
		urlToXmlMap: null,
		currentLanguage: null,
		
		constructor: function(args) {
			this.eTag = args.eTag;
			if (args.urlToXmlMap) {
				this.urlToXmlMap = args.urlToXmlMap;
			}
			this.isOutlineData = args.isOutlineData;
			this.init(args.xmlData);
		},
		
		init: function(xmlData) {
			var resourceUrl = UIHelper.getResourceUrl(xmlData.documentElement);
			var languages = UIHelper.getLanguages(xmlData.documentElement);
			var languagesObject = {};
			if (languages) {
				languages = languages.split(','); //$NON-NLS-1$
				dojo.forEach(languages, function(language) {
					if (language) {
						languagesObject[language] = {};
						languagesObject[language].persisted = true;
					}
				});
			}
			
			var ETagGroup = UIHelper.extractETag(this.eTag);
			this.currentLanguage = UIHelper.getLanguage(xmlData.documentElement);
			var id = dojox.uuid.generateTimeBasedUuid();
			languagesObject[this.currentLanguage].ETag = ETagGroup.textContentETag;
			languagesObject[this.currentLanguage].basicInformation = {};
			languagesObject[this.currentLanguage].basicInformation.label = UIHelper.getTitle(xmlData);
			languagesObject[this.currentLanguage].basicInformation.description = UIHelper.getDescription(xmlData);
			languagesObject[this.currentLanguage].textContent = {};
			var model = {};
			model.ETag = ETagGroup.modelETag;
			
			this.jsonData = {
				isOutlineElement: new Boolean(this.isOutlineData),
				identifier: 'id', //$NON-NLS-1$
				label: 'label', //$NON-NLS-1$
				resource: resourceUrl,
				id: id,
				title: languagesObject[this.currentLanguage].basicInformation.label,
				model: model,
				languages: languagesObject,
				items: []
			};
			var presentation = UIHelper.getPresentation(xmlData);
			if (presentation) {
				dojo.forEach(presentation.childNodes, function(node) {
					this.addToNavigation(this.jsonData.items, node);
				}, this);
			}
			
		},
		
		getPracticeElement: function(practiceElement) {
			var resourceUrl = UIHelper.getNodeResource(practiceElement);
			var jsonElement = {
				type: ProcessAuthoringConstants.NAVIGATORTYPE_ELEMENT,
				isOutlineElement: new Boolean(this.isOutlineData),
				id: dojox.uuid.generateTimeBasedUuid(),
				label: UIHelper.getNodeLabel(practiceElement),
				icon: UIHelper.getNodeIcon(practiceElement),
				resource: resourceUrl,
				isArchived: UIHelper.isArchivedXmlAttribute(practiceElement)
			};
			return jsonElement;
		},
		
		getCategoryNode: function(arr, categoryNode) {
			var jsonNode = {
				type: ProcessAuthoringConstants.NAVIGATORTYPE_CATEGORY,
				isOutlineElement: new Boolean(this.isOutlineData),
				id: dojox.uuid.generateTimeBasedUuid(),
				label: UIHelper.getNodeLabel(categoryNode),
				isArchived: UIHelper.isArchivedXmlAttribute(categoryNode),
				children: []
			};
			arr.push(jsonNode);
			dojo.forEach(categoryNode.childNodes, function(node) {
				this.addToNavigation(jsonNode.children, node);
			}, this);
		},
		
		addSubNavigation: function(arr, node) {
			var resourceUrl = UIHelper.getNodeResource(node);
			var nodeName = node.localName ? node.localName : node.baseName;
			if (!nodeName) {
				nodeName = ProcessAuthoringConstants.NAVIGATORTYPE_ELEMENT;
			}
			var jsonElement = {
				type: nodeName,
				isOutlineElement: new Boolean(this.isOutlineData),
				id: dojox.uuid.generateTimeBasedUuid(),
				label: UIHelper.getNodeLabel(node),
				icon: UIHelper.getNodeIcon(node),
				resource: resourceUrl,
				isArchived: UIHelper.isArchivedXmlAttribute(node),
				children: []
			};
			//This is for displaying shared practice in outline pane
			if (nodeName === ProcessAuthoringConstants.NAVIGATORTYPE_PRACTICE) {
				jsonElement.shared = UIHelper.getNodeShareAttr(node);
			}
			arr.push(jsonElement);
			var xml = this.urlToXmlMap[resourceUrl];
			if (xml) {
				var navData = new com.ibm.team.process.authoring.web.common.common.internal.NavigationData({
					eTag: this.eTag,
					xmlData: xml
				});
				
				dojo.forEach(navData.getJsonData().items, function(item) {
					jsonElement.children.push(item);
				}, this);
			}
		},
		
		addToNavigation: function(arr, node) {
			if (UIHelper.isPracticeElement(node)) {
				arr.push(this.getPracticeElement(node));
			} else if (UIHelper.isCategoryNode(node)) {
				this.getCategoryNode(arr, node);
			} else if (UIHelper.isPractice(node)) {
				this.addSubNavigation(arr, node);
			}
		},
		
		getJsonData: function() {
			return this.jsonData;
		}
		
	});
})();
