/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010, 2013. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function() {
	dojo.provide("com.ibm.team.process.authoring.web.common.common.internal.PracticeElement"); //$NON-NLS-1$
	dojo.require("com.ibm.team.process.authoring.web.common.common.internal.UIHelper"); //$NON-NLS-1$
	dojo.require("com.ibm.team.process.authoring.web.common.common.internal.ProcessAuthoringConstants"); //$NON-NLS-1$
	var UIHelper = com.ibm.team.process.authoring.web.common.common.internal.UIHelper;
	var ProcessAuthoringConstants = com.ibm.team.process.authoring.web.common.common.internal.ProcessAuthoringConstants;
	dojo.declare("com.ibm.team.process.authoring.web.common.common.internal.PracticeElement", null, { //$NON-NLS-1$
		url: null,
		language: "", //$NON-NLS-1$
		title: "", //$NON-NLS-1$
		description: "", //$NON-NLS-1$
		languages: "",  //$NON-NLS-1$ // comma-separate list of language codes
		presentation: null, // XML element <presentation>
		otherProperties: null, // map of name to XML element
		_sections: null,
		docElement: null,
		id: null,
				
		constructor: function(args) {
			if (!args.docElement) {
				// pass the whole xml document
				this.docElement = args.xml.documentElement;
			} else {
				// pass part of the xml document
				this.docElement = args.docElement;
			}
			this.url = UIHelper.getAttributeNode(this.docElement, ProcessAuthoringConstants.XMLATTRIBUTE_ABOUT, ProcessAuthoringConstants.NAMESPACE_RDF).value;
			this.language = UIHelper.getLanguage(this.docElement);
			this.id = this.url.substring(this.url.lastIndexOf("/") + 1); //$NON-NLS-1$
			this.archived = UIHelper.isArchivedXmlElement(this.docElement);
			this.otherProperties = {};
			dojo.some(this.docElement.childNodes, function(node) {
				return this.parseNode(node);
			}, this);
		},
		
		parseNode: function(node) {
			if (!this.title && UIHelper.isSpecifiedNode(node, ProcessAuthoringConstants.XMLELEMENT_TITLE, ProcessAuthoringConstants.NAMESPACE_DC)) {
				this.title = UIHelper.unescapeXML(node.firstChild.data);
			} else if (!this.description && UIHelper.isSpecifiedNode(node, ProcessAuthoringConstants.XMLELEMENT_DESCRIPTION, ProcessAuthoringConstants.NAMESPACE_DC)) {
				this.description = node.firstChild && node.firstChild.data ? UIHelper.unescapeXML(node.firstChild.data) : ""; //$NON-NLS-1$
			} else if (!this.languages && UIHelper.isSpecifiedNode(node, ProcessAuthoringConstants.XMLELEMENT_LANGUAGES, ProcessAuthoringConstants.NAMESPACE_JTP)) {
				this.languages = node.firstChild.data;
			} else if (node.nodeType == ProcessAuthoringConstants.NodeType.ELEMENT){
				if (UIHelper.isSpecifiedNode(node, ProcessAuthoringConstants.XMLELEMENT_PRESENTATION, ProcessAuthoringConstants.NAMESPACE_JTP)) {
					this.presentation = node;
				} else {
					this.otherProperties[node.tagName] = node;
				}
			}
			return this.doneInitialization();
		},
		
		doneInitialization: function() {
//			return this.title && this.description;
			return false;
		},
		
		isSharedByElement: function() {
			var sharedNode = this.otherProperties[ProcessAuthoringConstants.XMLELEMENT_SHARED];
			if (sharedNode && sharedNode.firstChild) {
				var value = sharedNode.firstChild.data;
				if (value == "true") { //$NON-NLS-1$
					return true;
				}
			}
			return false;
		},

		isSharedByOwner: function() {
        	var ownerURI = this.getOwnerURL();
            
            if (ownerURI) {
            	return ownerURI === ProcessAuthoringConstants.getJtsPracticeLibraryUrl();
            } else {
            	return false;
            }
		},
		
		getOwnerURL: function() {
			var xmlOwnerNode = this.otherProperties[ProcessAuthoringConstants.XMLELEMENT_OWNER];
			if (xmlOwnerNode) {
				var ownerAttribute = UIHelper.getAttributeNode(xmlOwnerNode, ProcessAuthoringConstants.XMLATTRIBUTE_RESOURCE, ProcessAuthoringConstants.NAMESPACE_RDF);
				var ownerURI = ownerAttribute.value;
			}
			return ownerURI;
		},
		
		getOwnerName: function() {
			var xmlOwnerNode = this.otherProperties[ProcessAuthoringConstants.XMLELEMENT_OWNER];
			var ownerName = null;
			if (xmlOwnerNode) {
				var attribute = UIHelper.getAttributeNode(xmlOwnerNode, ProcessAuthoringConstants.XMLATTRIBUTE_LABEL, ""); //$NON-NLS-1$
				if (attribute) {
					ownerName = UIHelper.unescapeXML(attribute.value);
				}
			}
			return ownerName;
		},
		
		getSections: function() {
			if(!this._sections && this.presentation) {
				this._sections = UIHelper.getNodes(this.presentation.childNodes, ProcessAuthoringConstants.XMLELEMENT_SECTION, ProcessAuthoringConstants.NAMESPACE_JTP);
			}
			return this._sections;
		},
		
		getElementID: function() {
			return this.id;
		},
		
        getPromotedElementXML: function() {
        	//remove owner node
        	var xmlOwnerNode = this.otherProperties[ProcessAuthoringConstants.XMLELEMENT_OWNER];
        	if (xmlOwnerNode) {
        		var ownerAttribute = UIHelper.getAttributeNode(xmlOwnerNode, ProcessAuthoringConstants.XMLATTRIBUTE_RESOURCE, ProcessAuthoringConstants.NAMESPACE_RDF);
        		var oldOwner = ownerAttribute.value;
				//this could change after 110045
        		ownerAttribute.value = ProcessAuthoringConstants.getJtsPracticeLibraryUrl();
        	}
        	
        	//change shared attribute 
        	var sharedNode = this.otherProperties[ProcessAuthoringConstants.XMLELEMENT_SHARED];
        	if (!sharedNode) {
        		sharedNode = UIHelper.createElementNS(ProcessAuthoringConstants.NAMESPACE_JTP, ProcessAuthoringConstants.XMLELEMENT_SHARED);
                
                if (dojo.isIE) {
                	sharedNode.text = ProcessAuthoringConstants.VALUE_TRUE;
                } else {
                	sharedNode.textContent = ProcessAuthoringConstants.VALUE_TRUE;
                }
        		this.docElement.appendChild(sharedNode);
        	} else {
        		UIHelper.setNodeText(sharedNode, ProcessAuthoringConstants.VALUE_TRUE);
        	}
        	
        	return dojox.xml.parser.innerXML(this.docElement);
        }
	});
})();
