/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008-2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("jazz.util.i18n");

(function() {

jazz.util.i18n = {
	
	STRONG: "b",
	SPAN: "s",
	LRM: "\u200e",
	RLM: "\u200f",
	LRE: "\u202a",
	RLE: "\u202b",
	PDF: "\u202c",
	
	/*
	 * Takes a string of the form: "Start<0>link</0><b>bold</b>Middle<s2>span</s2><b1>bold</b1>End"
	 * and returns an array of the form:
	 * 		[{text: "Start"},
	 * 		{text: "link", id: 0, type: "a"},
	 * 		{text: "bold", type: "strong"},
	 * 		{text: "Middle"},
	 * 		{text: "span", id: 2, type: "span"},
	 * 		{text: "bold", id: 1, type: "strong"},
	 * 		{text: "End"}]
	 */
	split: function(message) {
		try {
			var result = [];
			if (typeof(message) == "string") {
				var frag = message;
				var index = frag.indexOf("<");
				while (index != -1) {
					if (index != 0) {
						result.push({
							text: frag.substring(0, index)
						});
					}
					frag = frag.substring(index+1);
					if (frag.charAt(0) == "/")
						throw "End tag with no start tag.";
					index = frag.indexOf(">");
					var id = frag.substring(0,index);
					frag = frag.substring(index + 1);
					if (id.charAt(id.length - 1) == "/") {
						id = id.substring(0,id.length-1);
						var text = "";
					}
					else {
						var endTag = "</" + id + ">";
						index = frag.indexOf(endTag);
						if (index == -1) 
							throw "Start tag with no end tag for id " + id + ".";
						var text = frag.substring(0, index);
						frag = frag.substring(index + endTag.length);
					}
					var r = this._parseId(id);
					r.text = text;
					result.push(r);
					index = frag.indexOf("<");
				}
				result.push({
					text: frag
				});
			}
			return result;
		}
		catch (e) {
			console.error("I18nUtil error in message \"" + message + "\":" + e);
		}
	},
	
	/*
	 * Takes a string of the format specified in the split function and builds the represented
	 * dom structure by appending to the specified node. Returns an array of references to nodes
	 * using their ids as the indecies. Nodes with no id are not returned.
	 */
	addMessage: function(node, message) {
		var r = [];
		var s = this.split(message);
		for (var i = 0; i < s.length; i++) {
			if (typeof s[i].type == "undefined") {
				node.appendChild(document.createTextNode(s[i].text));
			}
			else {
				var c = document.createElement(s[i].type);
				if (s[i].type == "a")
					c.href = "#";
				c.innerHTML = s[i].text;
				node.appendChild(c);
				if (s[i].id == 0 || s[i].id)
					r[s[i].id] = c;
			}
		}
		return r;
	},
	
	_parseId: function(id) {
		var c = id.charAt(0).toLowerCase();
		var result = {};
		if (c >= "a" && c <= "z") {
			switch(c) {
				case this.SPAN:
					result.type = "span";
					break;
				case this.STRONG:
					result.type = "strong";
					break;
			}
			id = id.substring(1);
			if (id.length > 0)
				result.id = id;
		} else {
			result.type = "a";
			result.id = id;
		}
		return result;
	},
	
	getStringBaseTextDir: function(/* string */ text, /* string */ defDir){
		if (!defDir) defDir = "ltr";
		if (!text){
			return defDir;
		}
   		var len = text.length;
   		for(var i = 0;i < len;i++)  {
  			var symbol = text.charCodeAt(i);
  			if(this._isBidiChar(symbol)){
     			return "rtl";
     		}
  			else if(this._isLatinChar(symbol)){
     			return "ltr";
     		}
   		}
   		return defDir;		
	},

	/* string */ getElementBaseTextDir: function(/* object */ element, /* string */ dir){
		var defDir = !!dir? defDir : "ltr";
		if (!element || !element.innerHTML){
			return defDir;
		}
		var text = element.innerHTML.replace(/<[^<]*>/g,"");
		if (!!text && text.length > 0){
			return i18n.getStringBaseTextDir(text);
		}
   		return defDir;		
	},
	
	/* string */ getDefaultAlignment: function(/* string */ dir){
		if (dir == "rtl"){
			return {current: "right", opposite: "left"};
		}
		else{
			return {current: "left", opposite: "right"};
		}
	},
	
	/* string */ getSegmentSeparator: function(/* string */ dir){
		if (dir == "rtl"){
			return i18n.RLM;
		}
		else{
			return i18n.LRM;
		}
	},
	
	/* string */ removeAllMarks: function(/* string */ text){
		return text.replace(/[\u200e\u200f\u202a\u202b\u202c]/g,"");
	},
	
	/* string */ setElementBaseTextDir: function(element, textDir, newClass, segmentStart, segmentEnd){
		var dir = textDir;
		var defDir = dojo.style(dojo.body(),"direction");
		if (!!newClass){
			dojo.addClass(element, newClass);
		}
		var text = element.innerHTML.replace(/<[^<]*>/g,"");
		if (!text){
       		return defDir;
		}
		var tNode = element.firstChild; 
		while(tNode){
			if (tNode.nodeType == 3) break;
			if (tNode.firstChild){
				tNode = tNode.firstChild;
				continue;
			}
			if (tNode.nextSibling){
				tNode = tNode.nextSibling;
				continue;
			}
			if (tNode.parentNode !== element && tNode.parentNode.nextSibling){
				tNode = tNode.parentNode.nextSibling;
				continue;
			}
			break;
		}
		var posStart = -1, posEnd = -1;			
		if (!!segmentStart && segmentStart.length > 0){				 
			posStart = text.replace(/[\u202a\u202b\u202c]/g,"").indexOf(segmentStart);
			if (posStart >=0){
				posStart += segmentStart.length;
			}
		}
		if (!!segmentEnd && segmentEnd.length > 0){
			posEnd = text.replace(/[\u202a\u202b\u202c]/g,"").indexOf(segmentEnd);
		}
		if (posStart >=0 || (posEnd >=0 && posEnd >= posStart)){
			if (posStart == posEnd) posEnd = -1;
			text = text.replace(/[\u202a\u202b\u202c]/g,"");
			if (dir != "none"){					
				var start = posStart > 0? posStart : 0;
				var end = posEnd > 0? posEnd : text.length;
				var text1 = text.substring(start,end);
				if (dir == "auto"){
					dir = this.getStringBaseTextDir(text1, defDir);
				}
				var mark = dir == "rtl"? this.RLE : this.LRE;
				text = text.substring(0,start) + mark + text1 + this.PDF + (posEnd > 0? text.substring(end) : "");
			}
			if (tNode && tNode.nodeValue){
				tNode.nodeValue = text;
			}
			dir = defDir;
		}			
		else if (dir == "auto"){
			dir = this.getStringBaseTextDir(text, defDir);
		}
		if (dir != "none"){							
			dojo.style(element,"direction",dir);
			dojo.style(element,"unicodeBidi","embed");
		}			
		return dir;
	},
	
	/* string */ getDirectionAndHighlightSegment: function(node, textDir){
		if (!node || !node.nodeValue) return textDir;
		var defDir = dojo.style(dojo.body(),"direction");
		var nodeDir = textDir;
		var text = node.nodeValue;
		var sep = i18n.getSegmentSeparator(defDir);
		if (text.indexOf(sep) >= 0){
			nodeDir = defDir;
			text = text.replace(/[\u202a\u202b\u202c]/g,"");
			if (textDir != "none"){
				var segDir = textDir;
				if (segDir == "auto"){
					segDir = i18n.getStringBaseTextDir(text);
				}
				var txt1 = text.substring(0,text.indexOf(sep) + 1);
				var txt2 = text.substring(text.indexOf(sep) + 1);
				var txt3 = "";
				var mark = segDir == "rtl"? i18n.RLE : i18n.LRE;
				if (txt2.indexOf(sep) >=0){
					txt3 = txt2.substring(txt2.indexOf(sep)+1);
					txt2 = txt2.substring(0,txt2.indexOf(sep));
				}	
				node.nodeValue = txt1 + mark + txt2 + i18n.PDF + sep + txt3;
			}
		}
		return nodeDir;
	}
};

function _isBidiChar(/* char */ c){
	if (c >= 0x05d0 && c <= 0x05ff ||
		c >= 0x0600 && c <= 0x065f ||
		c >= 0x06fa && c <= 0x07ff ||
		c >= 0xfb50 && c <= 0xfdff ||
		c >= 0xfe70 && c <= 0xfefc)
		return true;
	else
		return false;
};

function _isLatinChar(/* char */ c){
	if((c > 64 && c < 91)||(c > 96 && c < 123))
		return true;
	else
		return false;
};

var i18n = jazz.util.i18n; 

})();
