/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2012. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("jazz.app.internal.UserMenu");

dojo.require("net.jazz.ajax.contextRoot");
dojo.require("jazz.ui.Menu");
dojo.require("dijit.MenuSeparator");
dojo.require("jazz.ui.menu.MenuItem");
dojo.require("jazz.ui.menu.NodeMenuItem");
dojo.require("dojo.i18n");
dojo.require("jazz.app.auth");
dojo.require("jazz.core.loader");
dojo.require("net.jazz.ajax.isDelegatingAuthentication");
dojo.requireLocalization("jazz.app.internal", "LoginLogoutMessages");

var messages = dojo.i18n.getLocalization("jazz.app.internal", "LoginLogoutMessages");

dojo.declare("jazz.app.internal.UserMenu", jazz.ui.Menu, {
	"class": "jazz-app-UserMenu",
	userSmallPreviewUrl: "",
	config: null,
	items: null,
	_setItemsAttr: function(items) {
		this.items = items;
		if (this._menuItems) {
			while(this._menuItems.length) {
				var item = this._menuItems.pop();
				this.removeChild(item);
				item.destroy();
			}
		} else {
			this._menuItems = [];	
		}
		for (var i = 0; i < items.length; i++) {
			if (items[i]) {
				item = new jazz.ui.menu.MenuItem(items[i]);
				item.iconNode && dojo.addClass(item.iconNode, "no-icon");
			} else {
				item = new dijit.MenuSeparator();
			}
			this.addChild(item);
			this._menuItems.push(item);
		}
		
		var logoutObj = this.config && this.config.logout;
		var args = {
			label: messages.logout,
			onClick: dojo.hitch(this, this._logout)
		};
		var logoutItem = new jazz.ui.menu.MenuItem(args);
		logoutItem.iconNode && dojo.addClass(logoutItem.iconNode, "no-icon");
		this.addChild(logoutItem);
		this._menuItems.push(logoutItem);
		this.onReflow();			
	},
	_setUserSmallPreviewUrlAttr: function(url) {
		this.userSmallPreviewUrl = url;
		if (this._userItem) {
			this.removeChild(this._userItem);
			this._userItem.destroy();
			this._userItem = null;
		}
		if (this._userSeparator) {
			this.removeChild(this._userSeparator);
			this._userSeparator.destroy();
			this._userSeparator = null;
		}
		this._loading = this.addLoading(0);
		dojo.addClass(this.domNode, "menu-loading");
		var deferred = jazz.client.xhrGet({
			url: url,
			failOk: true,
			skipInterceptors: ["jazz.oauth"]
		});
		deferred.addCallbacks(dojo.hitch(this,this._onUserPreviewSuccess),dojo.hitch(this,this._onUserPreviewFail));
	},
	_onUserPreviewSuccess: function(data){
		if (!this.domNode) return;
		var userString = dojo.cache("jazz/bogus", {value: data, sanitize: true});
		if (userString.indexOf("com-ibm-team-jfs-user-preview") >= 0) {
			var item = new jazz.ui.menu.NodeMenuItem({label: userString, className: "menu-text j-hint"});	
			this.addChild(item, 0);
			this._userItem = item;
			if (!this._userSeparator) {
				this._userSeparator = new dijit.MenuSeparator();
				this.addChild(this._userSeparator, 1);
			}
			dojo.addClass(this._userItem.domNode, "preview-node");
		}
		if (this._loading) {
			this.removeChild(this._loading);
			this._loading.destroy();
			delete this._loading;
		}
		
		dojo.removeClass(this.domNode, "menu-loading");
		this.onReflow();
		return data;
	},
	_onUserPreviewFail: function(error) {
		if (!this.domNode) return;
		if (error && error.status == 401) {
			this._login = this._getServerLogin();
			this.addChild(this._login, 0);
			if (!this._userSeparator) {
				this._userSeparator = new dijit.MenuSeparator();
				this.addChild(this._userSeparator, 1);
			}
		} else if (error && error.dojoType && error.dojoType == "popupBlocked") {
			var messageItem = new jazz.ui.menu.NodeMenuItem({
				"class": "message-item",
				focusable: false
			});	
			dojo.place(document.createTextNode(error.message), messageItem.containerNode, "only");
			this._popupMessage = this.addChild(messageItem,0);
			this._login = this.addChild(this._getServerLogin(),1);
		} else if (this._userSeparator) {
			this.removeChild(this._userSeparator);
			this._userSeparator.destroy();
			delete this._userSeparator;
		}
		if (this._loading) {
			this.removeChild(this._loading);
			this._loading.destroy();
			delete this._loading;	
		}
		dojo.removeClass(this.domNode, "menu-loading");
		this.onReflow();
	},
	_getServerLogin: function(group, service) {
		var item = new jazz.ui.menu.NodeMenuItem({className: "menu-text j-hint"});
		var node = dojo.create("div", {"class": "login-item"});
		var loginLink = jazz.util.internal.login.insertLoginLink(node, false);
		this.connect(loginLink, "onclick", function(e){
			dojo.stopEvent(e);
			var deferred = jazz.client.xhrGet({
				url: this.userSmallPreviewUrl
			});
			if (this._login) {
				this.removeChild(this._login);
				this._login.destroy();
				delete this._login;	
			}
			if (this._popupMessage) {
				this.removeChild(this._popupMessage);
				this._popupMessage.destroy();
				delete this._popupMessage;	
			}
			this._loading = this.addLoading(0);
			deferred.addCallbacks(dojo.hitch(this,this._onUserPreviewSuccess), dojo.hitch(this,this._onUserPreviewFail));
		});
		dojo.place(node, item.containerNode, "only");
		return item;
	},
	_logout: function(e) {
		var logoutObj = this.config && this.config.logout ? this.config.logout : {};
		if ((logoutObj.callback ? logoutObj.callback(e) : true) === false) { // if there's a callback defined and it returns boolean false, cancel the click
			dojo.stopEvent(e);
		} else {
			dojo.stopEvent(e);
			var PlatformUI = dojo.getObject("net.jazz.ajax.ui.PlatformUI");
			if (PlatformUI && PlatformUI.getWorkbench) {
				var workbench = PlatformUI.getWorkbench();
			}
			if (!workbench || !workbench.shouldVetoNavigation()){
				/*
				 * Note that this has only been tested for DM.  See comments in
				 * the IsDelegatingAuthentication class nested within DynamicResources.java.
				 */
				if (net.jazz.ajax.isDelegatingAuthentication()) {
					var daLogoutUrl = net.jazz.ajax.contextRoot() + "/service/com.ibm.team.repository.service.internal.ILogoutRestService";
					if (logoutObj.href) {
						daLogoutUrl = daLogoutUrl + "?" + "redirectURL=" + logoutObj.href;
					}
					else {
						daLogoutUrl = daLogoutUrl + "?" + "redirectURL=" + document.location;						
					}
					document.location = daLogoutUrl;
				}
				else {
					var logoutDef = jazz.app.auth.logout(); 
					logoutObj.href && logoutDef.addCallback(function(){
						document.location = logoutObj.href;
					});
					logoutDef.addErrback(function(err) {
						if (logoutObj.href) {
							document.location = logoutObj.href;
						} else {
							var error = messages.logoutError;
							if (err && err.status == 404) {
								error += "\n\n" + messages.basicAuthWarn;
							}
							alert(error);						
						}
					});
				}
			}
		}
	},
	addLoading: function(insertIndex) {
		var item = new jazz.ui.menu.NodeMenuItem({label: messages.loading, className: "menu-text j-hint"});
		dojo.addClass(item.domNode, "loading");	
		this.addChild(item, insertIndex);
		return item;
	},
	//stub for MenuPopup connect
	onReflow: function() {}
});

})();