/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2011, 2012. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("jazz.ensemble.internal.catalog.WidgetConfigurationPane");

/*
 * Imports 
 */
dojo.require("dojo.i18n"); 
dojo.require("dojox.xml.parser");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dijit.form.TextBox");

dojo.require("jazz.util.binding");
dojo.require("jazz.ensemble.internal.util.bindings");

dojo.require("jazz.ensemble.internal.catalog.WidgetCodeSnippetDialog");
dojo.require("jazz.ensemble.internal.util.WidgetCodeSnippetGenerator");
dojo.require("jazz.ensemble.internal.WidgetContainer");

dojo.requireLocalization("jazz.ensemble.internal", "WidgetCatalogMessages");

/*
 * 
 */

(function() {

/*
 * Aliases 
 */
var WidgetCodeSnippetDialog = jazz.ensemble.internal.catalog.WidgetCodeSnippetDialog;
var WidgetContainer = jazz.ensemble.internal.WidgetContainer;
var WidgetCodeSnippetGenerator = jazz.ensemble.internal.util.WidgetCodeSnippetGenerator;
var binding = jazz.util.binding;
var bindings = jazz.ensemble.internal.util.bindings;

/*
 * Constants
 */
var DEFAULT_WIDTH = 400;


/*
 * 
 */
dojo.declare("jazz.ensemble.internal.catalog.WidgetConfigurationPane",
	[dijit._Widget, dijit._Templated], {

	// Properties //----------------------------------------------------------->

	catalog: null,
	data: null,
	
	// ------------------------------------------------------------------------>
	
	/*
	 * Templates
	 */	
	templatePath: dojo.moduleUrl("jazz.ensemble", "internal/catalog/templates/WidgetConfigurationPane.html"),
	widgetsInTemplate: true,	
	
	/*
 	 * @param Object args
 	 * 
 	 * @return WidgetConfigurationPane
 	 */
	constructor: function(args) {
		
		// Messages //--------------------------------------------------------->

		this.msg = dojo.i18n.getLocalization("jazz.ensemble.internal", "WidgetCatalogMessages");

		// Images //----------------------------------------------------------->

	},
	
	/*
 	 * @param void
 	 * 
 	 * @return void
 	 * @overrides super
 	 */	
	postCreate: function() {

		this.inherited(arguments);
		
		// Instance Variables //----------------------------------------------->

		this._widgetState = null;
		this._dialog = null;
		
		// Initialization //--------------------------------------------------->
		
		this._initHandlers();
	},

	/*
 	 * @param void
 	 * 
 	 * @return void
 	 * @overrides super
 	 */	
	startup: function() {
		this.inherited(arguments);
		
		this._initFields();
	},	
	
	/*
 	 * @param void
 	 * 
 	 * @return void
 	 */		
	_initFields: function() {
		if (this.data) {
			this._initWidgetContainer();
		}
		// TODO: Error no data
	},
	
	/*
 	 * @param void
 	 * 
 	 * @return void
 	 */		
	_initHandlers: function() {
		this.connect(this._textBoxWidth.textbox, "onchange", this._handleEventOnChangeWidth);
		this.connect(this._textBoxHeight.textbox, "onchange", this._handleEventOnChangeHeight);
	},	
	
	/*
 	 * @param void
 	 * 
 	 * @return void
 	 */		
	_resetFields: function() {
		// TODO: Reset width / height values
	},		
	
	/*
 	 * @param void
 	 * 
 	 * @return Object
 	 */		
	_getWidgetXMLElement: function() {
		var dc = "dc";
		var jw = "jw";
		var rdf = "rdf";
		var xml = '<?xml version="1.0" encoding="UTF-8"?>' +
				  '<' + jw + ':Widget ' + 
				  'xmlns:' + rdf + '="' + bindings.NAMESPACES.rdf + '" ' +
				  'xmlns:' + dc +  '="' + bindings.NAMESPACES.dc  + '" ' +
				  'xmlns:' + jw +  '="' + bindings.NAMESPACES.jw  + '">' + 
				  '</' + jw + ':Widget>';
		var xmlDocument = dojox.xml.parser.parse(xml);
		
		return xmlDocument.documentElement;
	},	
	
	/*
 	 * @param void
 	 * 
 	 * @return void
 	 */		
	_initWidgetContainer: function() {

		var widgetElem = this._getWidgetXMLElement();
		// TODO: Why does this have to be binded twice? Can't we just use _widgetState from the beginning?
		var widget = binding.bind(widgetElem, bindings.Widget);

		widget.setType(this.data.type);
		widget.setDefinition(this.data.widgetUri);
		widget.setTitle(this.data.title);
		if (this.data.icon) {
			widget.setIcon(this.data.icon);
		}
		
		var preferences = this.data.preferences;
		if ((preferences instanceof Array) && 
				preferences.length > 0) {
			var prefs = [];
			widget.setPreferencesParseType("Collection");
			
			for (var index = 0; index < preferences.length; index++) {
				var pref = preferences[index];
				prefs.push({ 
					name:  pref.name, 
					value: pref.value
				});
			}
			widget.setPreferences(prefs);
		}		
		
		// Create widget container from state
		this._widgetState = binding.bind(widgetElem, bindings.Widget); 
		this._widgetState.setId(WidgetCodeSnippetGenerator.getUniqueID());
		this._widgetContainer = this._createWidgetContainer(this._widgetState);
		dojo.place(this._widgetContainer.domNode, this._widgetWrapper);
		this._widgetContainer.startup();
		this._addWidget();
		
	},	
	
	/*
 	 * @param Object state
 	 * 
 	 * @return WidgetContainer
 	 */	
	_createWidgetContainer: function(state) {
		// Creates a widget container initialized from the given state.
		var widgetContainer = new WidgetContainer({
			isCollapsible: false,
			readOnly: false,
			// shared: false, // TODO: Verify if matters - disable mementos
			state: state,
			onStateChange: dojo.hitch(this, this._stateChanged),
			// properties: this.catalog.widgetProperties // TODO: Provide empty properties, everything should be null guarded, only used for defaults
			properties: {}
		});
		widgetContainer.setClosable(false);
		return widgetContainer;
	},	

	/*
 	 * @param void
 	 * 
 	 * @return void
 	 */		
	_stateChanged: function() {
		// NO-OP?
	},	

	/*
 	 * @param void
 	 * 
 	 * @return void
 	 */		
	_addWidget: function() {
		// Add a widget to the panel, given an initial widget state.
		// Make the widget short and hidden for fade effect
		this._widgetContainer.domNode.style.height = "1px";
		dojo.style(this._widgetContainer.domNode,{
			opacity: 0
		});
		
		// Setup fade
		var args = {
			node: this._widgetContainer.domNode, 
			duration: 200
		};
		var animation = dojo.fx.chain([dojo.fx.wipeIn(args), dojo.fadeIn(args)]);
		var fadeEvent = dojo.connect(animation, "onEnd", this._widgetContainer, function() {
			dojo.disconnect(fadeEvent);
			var style = this.domNode.style;
			style.overflow = "";
			style.opacity = "";
			if (style.removeAttribute) {
				// Work-around: IE cleartype bug.
				style.removeAttribute("filter");
			}
		});
		
		animation.play();
	},	

	/*
 	 * @param Event event
 	 * 
 	 * @return Boolean
 	 */		
	_handleEventOnClickBack: function(event) {		
		if (event) {
			dojo.stopEvent(event);
		}
		this.catalog._showWidgetDetailsPane(this.data);
			
		return true;
	},

	/*
 	 * @param Event event
 	 * 
 	 * @return Boolean
 	 */		
	_handleEventOnClickGetCode: function(event) {		
		if (event) {
			dojo.stopEvent(event);
		}
		
		var state = this._widgetState;
		var width = this._getWidth();
		var height = this._getHeight();
		var codeSnippet = WidgetCodeSnippetGenerator.getCode(state, width, height);
		
		this._dialog = new WidgetCodeSnippetDialog({
			codeSnippet: codeSnippet
		});
		
		this._dialog.show();

		// TEST
		// TODO: Enable on debug=true
		//console.log(WidgetCodeSnippetGenerator.debugState(this._widgetState));
		
		return true;
	},	
	
	/*
 	 * @param Event event
 	 * 
 	 * @return Boolean
 	 */		
	_handleEventOnChangeWidth: function(event) {		
		if (event) {
			dojo.stopEvent(event);
		}
		
		var width = this._getWidth();
		
		if (width) {
			dojo.style(this._widgetWrapper, "width", width + "px");
		} else {
			// TODO: Error
			// Invalid number
			this._textBoxWidth.setValue(DEFAULT_WIDTH);
			dojo.style(this._widgetWrapper, "width", "");		
		}
		
		return true;
	},		
	
	/*
 	 * @param void
 	 * 
 	 * @return Number
 	 */		
	_getWidth: function() {		
	
		var width = parseInt(this._textBoxWidth.get('value'), 10);
		
		if (isNaN(width) ||
				(width < 100) ||
				(width > 9999)) {
			return 0;
		} 
		return width;
		
	},		
	
	/*
 	 * @param Event event
 	 * 
 	 * @return Boolean
 	 */		
	_handleEventOnChangeHeight: function(event) {		
		if (event) {
			dojo.stopEvent(event);
		}
		
		var height = this._getHeight();
		var node;
		
		if (height) {
			node = this._getViewletDOMNode();
			if (node) {
				dojo.style(node, "height", height + "px");
			}			
		} else {
			// TODO: Error
			// Invalid number
			this._textBoxHeight.setValue(this.msg.auto);
			node = this._getViewletDOMNode();
			if (node) {
				dojo.style(node, "height", "");
			}			
		}
		
		return true;
	},			
	
	/*
 	 * @param void
 	 * 
 	 * @return Number
 	 */		
	_getHeight: function() {		

		var height = parseInt(this._textBoxHeight.get('value'), 10);
		
		if (isNaN(height) ||
				(height < 50) ||
				(height > 9999)) {
			return 0;
		}
		return height;
		
	},		
	
	/*
 	 * @param void
 	 * 
 	 * @return Element
 	 */		
	_getViewletDOMNode: function() {
		// TEST
		//console.debug(this._widgetContainer.widget.viewlet.domNode);
		
		var widget = this._widgetContainer.widget;
		if (widget && 
				widget.viewlet &&
				widget.viewlet.domNode) {
			return widget.viewlet.domNode;
		}
		return null;
	},	
	
	/*
 	 * @param Object data
 	 * 
 	 * @return void
 	 */		
	_updateFields: function(data) {
		this.data = data;
		this._shutdownFields();
		this._initFields();
	},

	/*
 	 * @param void
 	 * 
 	 * @return void
 	 */		
	_shutdownFields: function() {
		this._shutdownWidgetContainer();
		this._resetFields();
	},		
	
	/*
 	 * @param void
 	 * 
 	 * @return void
 	 */		
	_shutdownWidgetContainer: function() {
		if (this._widgetContainer) {
			this._widgetContainer.destroy();
			this._widgetContainer = null;
		}		
	},	
	
	/*
 	 * @param void
 	 * 
 	 * @return void
 	 * @overrides super
 	 */		
	destroy: function() {

		this._shutdownWidgetContainer();
		
		// Dialog
		if (this._dialog) {
			this._dialog.destroy();
			delete this._dialog;
		}
		
		this.inherited(arguments);
	}

});

})();
