/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2012. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("jazz.ensemble.internal.util.WidgetCodeSnippetGenerator");

/*
 * Imports 
 */
dojo.require("dojox.xml.parser");

dojo.require("jazz.ensemble.internal.catalog.ErrorUtility");
dojo.require("jazz.app.internal.applicationAbout");
dojo.require("net.jazz.ajax.BootstrapProperties");

(function() {

/*
 * Aliases 
 */
var BootstrapProperties = net.jazz.ajax.BootstrapProperties;
var applicationAbout = jazz.app.internal.applicationAbout;
var ErrorUtility = jazz.ensemble.internal.catalog.ErrorUtility;

/*
 * Constants
 */
// Note: Increment version when HTML snippet format changes (for N-1 compatibility)
var CURRENT_SNIPPET_VERSION = "1.0";	

/*
 * Class 
 */
jazz.ensemble.internal.util.WidgetCodeSnippetGenerator = {

	/*
	 * @param void
	 * 
	 * @return String
	 */		
	getUniqueID: function() {
		return 'jazz-viewlet_' + this._getCurrentTimeMS();
	},		
		
	/*
 	 * @param Object state
 	 * @param Number width
 	 * @param Number height
 	 * 
 	 * @return String
	 */		
	getCode: function(state, width, height) {
		
		var id = this.getUniqueID();
		var viewletDefinition = state.getDefinition();
		var frontSideURL = this._getFrontSideURL(viewletDefinition);
		var currentApplication = applicationAbout.getCurrent();
		var serverVersion;
		
		// Determine the version of the application generating this HTML snippet
		if (currentApplication &&
				currentApplication.version) {
			serverVersion = currentApplication.version;
		} else {
			// Unexpected error
			ErrorUtility.logError(new Error("Unable to determine the verson of the current application"), this.id);
			serverVersion = "?";
		}
		
		// Update widget id (no longer in the widget panel context)
		state.setId(id);
		
		// e.g. Script url
		// https://localhost:9443/jazz/web/jazz.ensemble/internal/IFrameWidgetLoader.js
		// Note: Use the loader script from the same sever (origin) as the viewlet definition.   
		var scriptSRC = frontSideURL + '/web/jazz.ensemble/internal/IFrameWidgetLoader.js';
		
		var stateID = id + '_widget-state';
		var urlID = id + '_frontside-url';
		var heightID = id + '_height';
		var widthID = id + '_width';

		var serializedState = this._serializeWidgetState(state);
		
		var snippet = 	'<div id="' + id + 
							'" data-version="' + CURRENT_SNIPPET_VERSION +  
							'" data-server-version="' + serverVersion + '">' + "\n" + 
						'   ' + this._getHTMLParameter(widthID, width) + "\n" +
						'   ' + this._getHTMLParameter(heightID, height) + "\n" +
						'   ' + this._getHTMLParameter(urlID, frontSideURL) + "\n" +
						'   ' + this._getHTMLParameter(stateID, serializedState) + "\n" +
						'</div>' + "\n" +
						'<script src="' + scriptSRC + '"></script>' + "\n" +
						this._generateIFrameLoaderHelperScript(id) + "\n";
			
		// TODO: Does the frontside url need to be encoded?
			
		// TEST
		//console.debug(serializedState);
		//console.debug(dojox.xml.parser.innerXML(state.elem));
			
		return snippet;
			
	},		

	/*
 	 * Returns the Web UI root given a viewlet definition URL.
 	 * 
	 * e.g. 
	 * Definition URL: "https://localhost:9443/jazz/viewlet/com.ibm.team.dashboard.viewlets.web.feed"
	 * Web UI Root: "https://localhost:9443/jazz"
	 * 
	 * e.g. 
	 * Definition URL: "https://localhost:9443/jazz/service/com.ibm.team.dashboard.common.internal.service.IDashboardRestService/viewletDefinitions?definitionIds=com.ibm.team.dashboard.viewlets.web.feed&amp;webUIRoot=https%3A%2F%2Flocalhost%3A9443%2Fjazz"
	 * Web UI Root: "https://localhost:9443/jazz"
 	 * 
 	 * @param String url
 	 * 
 	 * @return String
 	 */		
	_getWebUIRoot: function(url) {
		// Detect 4.x or higher viewlet definitions
		var index = url.indexOf("/viewlet/");
		if (index > 0) {
			return url.substring(0, index);
		}		
		// Detect legacy (pre-4.x) viewlet definitions
		var legacyURL = new dojo._Url(url);
		if (!legacyURL.query) {
			// Invalid URL
			return "";
		}
		var result = legacyURL.query.match(/webUIRoot=([^&]*)/);
		if ((result instanceof Array) &&
				(result.length === 2)) {
			return decodeURIComponent(result[1]);
		}
		// Invalid URL
		return "";
	},	
	
	/*
 	 * @param String url
 	 * 
 	 * @return String
 	 */		
	_getFrontSideURL: function(url) {
		// Use the same public web UI root as the viewlet definition
		// See: 202998: [Embedabble Widgets]: HTML snippets generated from remote viewlets are rendered as an iframe within an iframe when embedded
		var frontSideURL = this._getWebUIRoot(url);
		if (!frontSideURL) {
			ErrorUtility.logError(new Error("Encountered an invalid viewlet definition URL: " + url), this.id);
			frontSideURL = BootstrapProperties.FRONTSIDE_URL; 
		}
		return frontSideURL;
	},		
	
	/*
 	 * @param Object state
 	 * 
 	 * @return String
 	 */		
	debugState: function(state) {
		var result = this._widgetPreferencesToString(state) +
					this._mementoToString(state);
		return result;
	},		
	
	/*
	 * Returns the milliseconds elapsed since 1 January 1970 00:00:00 UTC up until now as a Number.
	 * 
	 * @param void
	 * 
	 * @return Number
	 */		
	_getCurrentTimeMS: function() {
		// Date.now() doesn't work on IE8
		return +(new Date());
	},		
	
	/*
 	 * @param Object state
 	 * 
 	 * @return String
 	 */		
	_serializeWidgetState: function(state) {
		// Convert XML to String
		var serializedState = dojox.xml.parser.innerXML(state.elem);
		// Encode XML String to safely embed in HTML
		// <, >, &, ~
		// Note: Tilde (~) is used as a delimeter.
		var result = serializedState.replace(/&/gm, "&amp;").replace(/</gm, "&lt;").replace(/>/gm, "&gt;").replace(/~/gm, "&tilde;");
		return result;
	},		

	/*
 	 * @param String id
 	 * @param String value
 	 * 
 	 * @return String
 	 */		
	_getHTMLParameter: function(id, value) {
		var result = '<textarea id="' + id + '" style="display:none;">' + value + '</textarea>';
		return result;
	},		
	
	/*
 	 * @param String id
 	 * 
 	 * @return String
 	 */		
	_generateIFrameLoaderHelperScript: function(id) {
		var result = '<script> new IFrameWidgetLoader("' + id + '").instance.run(); </script>';		
		return result;
	},
	
	/*
 	 * @param @param Object state
 	 * 
 	 * @return String
 	 */		
	_widgetPreferencesToString: function(state) {
		
		var result = "--------------------------------\n" +
					 "PREFERENCES:\n" +
					 "--------------------------------\n";
		
		var preferences = state.getPreferences();
		// Not a real Array...
		if (preferences && 
				preferences.length) {
			for (var index = 0; index < preferences.length; index++) {
				var preference = preferences[index];
				if (!preference) {
					// TODO: Error
					continue;
				}
				result+= preference.getName() + " = " + preference.getValue() + "\n";
			}
		}
		
		return result + "\n";
		
	},		
	
	/*
 	 * @param @param Object state
 	 * 
 	 * @return String
 	 */		
	_mementoToString: function(state) {
		
		var result = "--------------------------------\n" +
					 "MEMENTO:\n" +
					 "--------------------------------\n";
		var memento = state.getMemento();
		if (!memento) {
			memento = "";
		}
		
		return 	result +
				memento +
				"\n";
		
	}	

};


})();

