/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010, 2013. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.dashboard.web.ui.DashboardMini");

dojo.require("com.ibm.team.dashboard.web.ui.DashboardConstants");
dojo.require("com.ibm.team.dashboard.web.ui.internal.DashboardInitializationData");
dojo.require("com.ibm.team.dashboard.web.ui.internal.catalog.WidgetCatalogDialog");
dojo.require("com.ibm.team.dashboard.web.ui.internal.util.ErrorReport");

dojo.require("dojo.i18n");
dojo.require("dojo.string");
dojo.require("dojox.xml.parser");

dojo.require("net.jazz.ajax.contextRoot");
dojo.require("jazz.app.ua.internal.HelpHoverPopup");
dojo.require("jazz.client.xhr");
dojo.require("jazz.core.loader");
dojo.require("jazz.ensemble.internal.client.HttpConstants");
dojo.require("jazz.ensemble.internal.util.bindings");
dojo.require("jazz.ui.MessageBox");
dojo.require("jazz.ui.internal.Drawer");
dojo.require("jazz.util.binding");
dojo.require("jazz.util.i18n");

dojo.requireLocalization("com.ibm.team.dashboard.web.ui.internal", "DashboardMiniMessages");

(function(){

var binding = jazz.util.binding;
var bindings = jazz.ensemble.internal.util.bindings;
var DashboardConstants = com.ibm.team.dashboard.web.ui.DashboardConstants;
var DashboardInitializationData = com.ibm.team.dashboard.web.ui.internal.DashboardInitializationData;
var ErrorReport = com.ibm.team.dashboard.web.ui.internal.util.ErrorReport;
var HelpHoverPopup = jazz.app.ua.internal.HelpHoverPopup;
var HttpConstants = jazz.ensemble.internal.client.HttpConstants;
var MessageBox = jazz.ui.MessageBox;

var TRIM_BY_INDEX = [null, "GRAY", "GOLD", "GREEN", "TEAL", "PURPLE", "ORANGE", "TRIMLESS"];
var TYPE_MINI = "com.ibm.team.dashboard.mini";

var _messages = dojo.i18n.getLocalization("com.ibm.team.dashboard.web.ui.internal", "DashboardMiniMessages");

dojo.declare("com.ibm.team.dashboard.web.ui.DashboardMini", [jazz.ui.internal.Drawer], {
	
	tooltip			: _messages.dashboardMini,
	headerTitle		: _messages.dashboardMini,
	_errorReport	: null,
	_catalog		: null,
	_dialogClosedFn : null, // cached the hitch
	_widgetEnsemble	: null,
	
	constructor: function() {
		this.menuItems = [{
			label: _messages.paste,
			onClick: dojo.hitch(this, this._paste),
			iconClass: "com-ibm-team-dashboard-web-ui-DashboardMini-paste j-mirror"
		},{
			label: _messages.refresh,
			onClick: dojo.hitch(this, function() {
				if (this._widgetEnsemble) {
					this._widgetEnsemble.refresh();
				}
			}),
			iconClass: "com-ibm-team-dashboard-web-ui-DashboardMini-refresh j-mirror"
		}]
	},

	postCreate: function() {
		this.inherited(arguments);
		dojo.addClass(this.containerNode, "com-ibm-team-dashboard-web-ui-DashboardMini");
		this._catalogBtn = new jazz.ui.toolbar.TwoStateButton({
			imgClass: 	"com-ibm-team-dashboard-web-ui-DashboardMini-addWidget j-mirror", 
			title: 		_messages.addWidget,
			onClick:	dojo.hitch(this, this._addWidget)
		});
		this.toolbar.addChild(this._catalogBtn, 1);
		
		this._dialogClosedFn = dojo.hitch(this, "_dialogClosed");

		this.hoverPopup = new jazz.app.ua.internal.HelpHoverPopup({
			uaKey: 'com.ibm.team.dashboard.web.dash_mini',
			around: this._headerText
		});
	},
	
	destroy: function() {
		this.inherited(arguments);
		if (this._messageBox) {
			this._messageBox.destroyRecursive();
		}
		if (this._errorReport) {
			this._errorReport.destroyRecursive();
		}
		if (this._widgetEnsemble) {
			this._widgetEnsemble.destroyRecursive();
		}
		if (this._helpHover) {
			this._helpHover.destroy();
		}
	},
		
	/*
 	 * @param Object data
 	 * 
 	 * @return dojo.Deferred
 	 */
	onAddWidget: function(data) {
		var widgetElem = this._widgetEnsemble.createWidget();
		var widget = binding.bind(widgetElem, bindings.Widget);
		widget.setType(data.type);
		widget.setDefinition(data.widgetUri);
		widget.setTitle(data.title);
		
		var icon = data.icon;
		if (icon) widget.setIcon(icon);

		if (data.preferences instanceof Array && data.preferences.length > 0) {
			var prefs = [];
			widget.setPreferencesParseType("Collection");
			var len = data.preferences.length;
			for (var i=0;i<len;++i) {
				var pref = data.preferences[i];
				prefs.push({name: pref.name, value: pref.value});
			}
			widget.setPreferences(prefs);
		}
		
		this._widgetEnsemble.addWidget(widgetElem);
		
		var deferred = new dojo.Deferred();
		deferred.callback({});
		this._scrollableView._containerWrapper.scrollTop = "0";
		return deferred;				
		
	},
	
	onClose: function() {
		if (this._widgetEnsemble) {
			this._widgetEnsemble.onHide();
		}
	},
	
	onOpen: function() {
		if (this._widgetEnsemble) {
			this._widgetEnsemble.onShow();
		} else {
			// lazy load; on first open
			this._load();
		}
	},
	
	refresh: function() {
		this._widgetEnsemble.refresh();
	},
	
	/*
	 * Loads the mini dashboard.
	 * Step 1: Fetch dashboard initialization data.
	 */
	_load: function() {
		var lazyLoad = new dojo.Deferred();
		//load WidgetEnsemble and ViewletAdapterFactory together to save a roundtrip
		jazz.core.loader.batch_load_async(["jazz.ensemble.internal.WidgetEnsemble","com.ibm.team.dashboard.web.ui.internal.ViewletAdapterFactory"], dojo.hitch(this,function() {
			lazyLoad.callback();
		}));
		
		var dfd = DashboardInitializationData.load();
		dfd.addErrback(dojo.hitch(this, this._showError));
		
		var lazyLoadList = new dojo.DeferredList([dfd, lazyLoad]);
		lazyLoadList.addCallback(dojo.hitch(this, this._load2));
	},
	
	/*
	 * Loads the mini dashboard.
	 * Step 2: Query for the current user's mini dashboard URL.
	 */
	_load2: function() {
		var _this = this;
		var errback = dojo.hitch(this, this._showError);
		var userUrl = DashboardInitializationData.getAuthenticatedContributorUri();
		var dashboardServiceUrl = DashboardInitializationData.getDashboardServiceUri();
		var dashboardQueryServiceUrl = DashboardInitializationData.getDashboardQueryServiceUri();
		
		// lack of these means there was no JTS configured
		if (!dashboardServiceUrl || !dashboardQueryServiceUrl) {
			this._showError(_messages.errorNoJTS);
			return;
		}
		
		// the "?properties=dc:created" part is a workaround for:
		// 160573: QueryHelper generates incorrect SPARQL when /sort by otherwise unspecified property
		var queryUrl = dashboardQueryServiceUrl + "?q=" + encodeURIComponent(dojo.string.substitute("dc:type=\"${0}\" and dc:relation=<${1}> /sort=dc:created /limit=1", [TYPE_MINI, userUrl])) + "&properties=" + encodeURIComponent("dc:created");
		var headers = {};
		headers[HttpConstants.ACCEPT] = HttpConstants.CT_APPLICATION_JSON;
		jazz.client.xhrGet({
			url: queryUrl,
			handleAs: "json",
			headers: headers,
			load: function(/* object */ response) {
				var bindings = response.results.bindings;
				if (bindings.length == 0) {
					// there are none; create one
					jazz.client.xhrPost({
						url: dashboardServiceUrl,
						headers: {
							"X-Jazz-Owning-Context": userUrl
						},
						content: {
							type: TYPE_MINI,
							relation: userUrl
						},
						load: function(/* any */ content, /* dojo.__XhrArgs */ xhrArgs) {
							var xhr = xhrArgs.xhr;
							var location = xhr.getResponseHeader(HttpConstants.LOCATION);
							_this._load3(location);
						},
						error: errback
					});
				}
				else {
					var url = bindings[0].resource.value;
					_this._load3(url);
				}
			},
			error: errback
		});
	},
	
	/*
	 * Loads the mini dashboard.
	 * Step 3: Creates the widget ensemble from the URL.
	 */
	_load3: function(/* string */ url) {
		var authenticatedContributor = DashboardInitializationData.getAuthenticatedContributor();
		var properties = {
			"com.ibm.team.dashboard.scope": DashboardConstants.SCOPE_CONTRIBUTOR,
			"com.ibm.team.dashboard.scopeItem": authenticatedContributor,
			"com.ibm.team.dashboard.defaults": {contributor: authenticatedContributor}
		};
		this._widgetEnsemble = new jazz.ensemble.internal.WidgetEnsemble({
			allowMaximize: false,
			url: url,
			shared: false,
			properties: properties,
			handleError: dojo.hitch(this, this._showError),
			handleSaveError: dojo.hitch(this, this._showSaveError),
			onLoad: dojo.hitch(this, this._updateEmptyMsg)
		});
		this.connect(this._widgetEnsemble, "onStateChange", this._updateEmptyMsg);
		this.containerNode.replaceChild(this._widgetEnsemble.domNode, this._loadingText);
		this._widgetEnsemble.startup();
	},
	
	onPinChange: function(pinned) {
		if (!pinned) this._showCatalog(false);
	},
	
	_showCatalog: function(show)	{
		if ((show&&this._catalog) || (!(show||this._catalog))) return;
		if (show){
			// mini dashboard can be viewed on N-1 apps, so use legacy URLs (compatibilityMode: true)
			this._catalog = new com.ibm.team.dashboard.web.ui.internal.catalog.WidgetCatalogDialog({
				compatibilityMode: true,
				modal: false,
				onAddWidget: dojo.hitch(this, this.onAddWidget)
			});			
			this._catalog.onClose = this._dialogClosedFn;
			return true;
		} else {
			this._catalog.close();
			this._catalog = null;
			return false;
		}
	},
	
	_showError: function(/* Error|string */ e) {
		dojo.empty(this.containerNode);
		if (typeof(e) == "string") {
			var adminHref = net.jazz.ajax.contextRoot() + "/admin#action=com.ibm.team.repository.admin.configureAdvanced";
			this.containerNode.innerHTML = [
				"<div class=\"error-msg\">", e,
				"<div class=\"link\">",
				"<a href=\"", adminHref, "\">", _messages.configJTS, "</a></div></div>"
			].join("");
		}
		else {
			// handle popup blocker nicely
			if (e.dojoType == "popupBlocked") {
				var div = dojo.create("div", {className: "error-msg", innerHTML: e.message}, this.containerNode);
				var linkDiv = dojo.create("div", {className: "link"}, div);
				var a = dojo.create("a", {href: "#", innerHTML: _messages.tryAgain}, linkDiv);
				this.connect(a, "onclick", dojo.hitch(this, function(e) {
					dojo.stopEvent(e);
					dojo.empty(this.containerNode);
					this._loadingText = dojo.create("div", {className: "loadingText", innerHTML: _messages.loading}, this.containerNode);
					this._load();
				}));
			}
			else {
				var msg;
				var details;
				var collapseDetails = true;
				switch (e.status) {
					case 503:
						msg = _messages.error503;
						details = e.responseText;
						e = null;
						collapseDetails = false;
						break;
					case 403:
						msg = _messages.error403;
						details = e.responseText;
						e = null;
						break;
					default:
						if (e.statusText) {
							msg = dojo.string.substitute(_messages.errorReason, [e.statusText]);
						}
						else {
							msg = _messages.errorDefault;
						}
						collapseDetails = false;
						break;
				}
				this._errorReport = new ErrorReport({msg: msg, details: details, error: e, collapseDetails: collapseDetails});
				dojo.addClass(this._errorReport.domNode, "error-report");
				this._errorReport.placeAt(this.containerNode);
			}
		}
		return e;
	},

	_showSaveError: function(/* Error */ e) {
		if (!this._messageBox) {
			var div = dojo.create("div", {className: "message-box-container"}, this.containerNode, "first");
			this._messageBox = new MessageBox();
			div.appendChild(this._messageBox.domNode);
		}
		var args = {
			type: MessageBox.STATUS_TYPES.ERROR,
			closable: true
		};
		if (dojo.isString(e)) {
			dojo.mixin(args, {summary: e});
			this._messageBox.showMessage(args);
		} else if (e.status == 409 || e.status == 412) {
			args.summary = _messages.errorConflict;
			var buf = [_messages.errorConflict_explanation];
			buf.push(" ");
			var parts = jazz.util.i18n.split(_messages.errorConflict_userAction);
			for (var i=0;i<parts.length;i++) {
				var part = parts[i];
				if (part.id == 0) {
					buf.push("<a href=\"#\" id=\"com-ibm-team-dashboard-web-ui-DashboardMini-refreshLink\">");
					buf.push(part.text);
					buf.push("</a>");
				}
				else {
					buf.push(part.text);
				}
			}
			args.details = [{message: buf.join(""), code: MessageBox.STATUS_TYPES.OK}];
			this._messageBox.showMessage(args);
			var a = dojo.byId("com-ibm-team-dashboard-web-ui-DashboardMini-refreshLink");
			this.connect(a, "onclick", function(event) {
				dojo.stopEvent(event);
				this.refresh();
				this._messageBox.hideMessage();
			});
		}
		else if (e.status == 403) {
			args.summary = _messages.errorSavePermission;
			args.details = [{message: e.responseText}];
			this._messageBox.showMessage(args);
		}
		else if (e.status == 503) {
			args.summary = _messages.errorSaveReadOnly;
			args.details = [{message: e.responseText}];
			this._messageBox.showMessage(args);
		}
		else if (e.status > 0) {
			var buf = ["HTTP/1.1 ", e.status];
			if (e.statusText) {
				buf.push(" ");
				buf.push(e.statusText);
			}
			args.summary = dojo.string.substitute(_messages.errorSave, [buf.join("")]);
			var responseText = e.responseText;
			if (responseText && responseText.length > 0) {
				responseText = responseText.replace(/\r?\n/g, "<br/>");
				responseText = responseText.replace(/\t/g, "&nbsp;&nbsp;&nbsp;&nbsp;");
				args.details = [{message: responseText}];
			}
			args.errorObject = e;
			this._messageBox.showMessage(args);
		}
		else {
			args.errorObject = e;
			this._messageBox.showMessage(args);
		}
	},
	
	_updateEmptyMsg: function() {
		if (this._widgetEnsemble.isEmpty()) {
			if (!this._emptyMsgDiv) {
				this._emptyMsgDiv = dojo.create("div", {className: "empty-msg"});
				var a = jazz.util.i18n.addMessage(this._emptyMsgDiv, _messages.empty)[0];
				this.connect(a, "onclick", dojo.hitch(this, function(e) {
					dojo.stopEvent(e);
					this._showCatalog(true);
				}));
			}
			if (this._emptyMsgDiv.parentNode != this.containerNode) {
				dojo.place(this._emptyMsgDiv, this.containerNode, "first");
			}
		}
		else {
			if (this._emptyMsgDiv && this._emptyMsgDiv.parentNode == this.containerNode) {
				this.containerNode.removeChild(this._emptyMsgDiv);
			}
		}
		this.checkHeight();
	},
	
	_addWidget: function(e)	{
		if (this._showCatalog(e.pressed)) {
			this._catalogBtn.set("pressed", true);
		}
	},
	
	_dialogClosed: function(){
		this._catalog = null;
		this._catalogBtn.set("pressed", false);
	},
	
	_paste: function() {
		try {
			var data = localStorage["net-jazz-ajax-clipboard"];
		} catch(e) {
			this._showSaveError(_messages.featureNotSupported);
			return;
		}
		if (!data) {
			this._showSaveError(_messages.clipboardEmpty);
			return;
		}
		var document = dojox.xml.parser.parse(data);
		var elem = document.documentElement;
		var ownerDocument = this._widgetEnsemble.state.ownerDocument;
		if (ownerDocument.importNode) {
			elem = ownerDocument.importNode(elem, true);
		}
		if (elem.nodeName.indexOf("Page") != -1) {
			this._showSaveError(_messages.cannotPasteTab);
		} else {
			this._messageBox&&this._messageBox.hideMessage();
			this._widgetEnsemble.addWidget(elem);
		}
	}
});

})();
