/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2012. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("com.ibm.team.dashboard.web.ui.ProjectDashboardComponentMenu");

dojo.require("jazz.app._ComponentMenu");
dojo.require("jazz.app.context");
dojo.require("jazz.app.viewHistory");
dojo.require("jazz.ui.internal.MenuItemGroup");
dojo.require("jazz.util.html");
dojo.require("com.ibm.team.dashboard.web.ui.DashboardConstants");
dojo.require("com.ibm.team.dashboard.web.ui.internal.Constants");
dojo.require("com.ibm.team.dashboard.web.util.internal.Util");
dojo.require("com.ibm.team.dashboard.web.util.DashboardUtil");
dojo.require("dojo.i18n");
dojo.require("dojo.string");
dojo.requireLocalization("com.ibm.team.dashboard.web.ui.internal", "DashboardMenuMessages");

var Constants = com.ibm.team.dashboard.web.ui.internal.Constants;
var DashboardConstants = com.ibm.team.dashboard.web.ui.DashboardConstants;
var Util = com.ibm.team.dashboard.web.util.internal.Util;
var DashboardUtil = com.ibm.team.dashboard.web.util.DashboardUtil;
var HTML = jazz.util.html;
var MenuItemGroup = jazz.ui.internal.MenuItemGroup;
var WrapHints = jazz.ui.MegaMenu.WrapHints;
var viewHistory = jazz.app.viewHistory;
var DASHBOARD_TYPE = "com.ibm.team.dashboard";

var messages = dojo.i18n.getLocalization("com.ibm.team.dashboard.web.ui.internal", "DashboardMenuMessages");

/**
 * The project dashboards component menu, which provides navigation for project and team dashboards
 * in applications. This menu is meant to be used in conjunction with the ProjectDashboardPage.
 * 
 * @since 4.0.1
 */
dojo.declare("com.ibm.team.dashboard.web.ui.ProjectDashboardComponentMenu", jazz.app._ComponentMenu, {
	
	postCreate: function() {
		dojo.addClass(this.domNode, "com-ibm-team-dashboard-ComponentMenu");
		var projectName = jazz.app.context.get().name;
		var escName = HTML.escape(projectName);
		this._projectDashboards = new MenuItemGroup({
			menu: this,
			groupTitle: messages.projectDashboards,
			items: [{
				label: escName,
				iconClass: "project-dashboard",
				href: DashboardUtil.getDashboardHref(DashboardConstants.SCOPE_PROJECT_AREA, projectName)
			}, {
				label: dojo.string.substitute(messages.allProjectDash, [escName]),
				iconClass: "all-projects",
				href: Util.getAllProjectDashboardsHref(projectName),
				wrapHint: WrapHints.NO_WRAP
			}]
		});
		this.initRecent();		
		dojo.publish("/jazz/dashboard/initMenu", [this]);
		this.subscribe("/jazz/dashboard/view", this._addRecentItem); // pass item obj
		this.subscribe("/jazz/dashboard/dirty", this.onDirtyStateChange); // pass bundle and dirty state
	},
	// internal
	initRecent: function(recent) {
		var items = [];
		var query = {
			application: "%MY_APPLICATION%",			
			type: DASHBOARD_TYPE,
			project: jazz.app.context.get().name
		};
		var def = viewHistory.get(query);
		var menuDef = new dojo.Deferred();
		this._createDeferredRecentSection(menuDef);		
		
		def.addCallback(dojo.hitch(this, function (result) {
			var history = result.items;
			outer: for (var i=0; i<history.length; i++) {
				var entry = history[i];
				var item = {
					label: entry.label,
					icon: entry.iconUri,
					href: Util.addUIParams(entry.uri),
					uri: entry.uri
				};
				if (this._unsavedHref && this._isSameDashboard(item.href, this._unsavedHref)) {
					// skip this one
					delete this._unsavedHref;
					continue;
				}
				for (var j = 0; j < items.length; j++) {
					if (this._isSameDashboard(items[j].href,item.href)) continue outer;
				}				
				items.push(item);
			}
			if (items.length == 0) {
				// No recent dashboards in user history - remove section
				if (this._unsavedDashboards) this._unsavedDashboards.set("wrapHint", WrapHints.FORCE_WRAP);
				this._recentDashboards.remove();
				this._recentDashboards.destroy();
				delete this._recentDashboards;
			} else {
				menuDef.callback(items); 
				this._recentDashboards.set("groupTitle", "<b>"+messages.recentDashboards+"</b> (" + (items ? items.length : 0) + ")");
				if (items.length > 0)   {
					this.updateSelected();	
				}
			}
		}));		
	},
	_createRecentSection: function(items) {
		if (!this._recentDashboards) {
			if (this._unsavedDashboards) this._unsavedDashboards.set("wrapHint", WrapHints.NO_WRAP);
			this._recentDashboards = new MenuItemGroup({
				menu: this,
				wrapHint: WrapHints.FORCE_WRAP,
				groupTitle: "<b>"+messages.recentDashboards+"</b> (" + (items ? items.length : 0) + ")",
				items: items || []
			});
		}
			
	},
	_createDeferredRecentSection: function(def) {
		if (!this._recentDashboards) this._recentDashboards = new MenuItemGroup({
			menu: this,
			wrapHint: WrapHints.FORCE_WRAP,
			groupTitle: "<b>"+messages.recentDashboards+"</b>",
			itemProvider: function() { return def; }
		});
	},		
	_addRecentItem: function(/* object */ item) {
		item.href = Util.addUIParams(item.href);
		if (!this._recentDashboards) {
			this._createRecentSection();
		}
		var items = this._recentDashboards.get("items");
		for (var i = 0; i < items.length; i++) {
			if (this._isSameDashboard(items[i].href, item.href)) {
				this._recentDashboards.remove(items[i]).destroy();
				break;
			}
		}
		while (items.length > 4) {
			this._recentDashboards.remove(items[0]).destroy();
		}
		this._recentDashboards.addItemByArgs(item,0);
		this._updateRecentCount();
		this.updateSelected();
		return true;
	},
	_isSameDashboard: function(href1, href2) {
		var contextRoot = net.jazz.ajax.contextRoot();
		return href1.substring(href1.indexOf(contextRoot)) == href2.substring(href2.indexOf(contextRoot));
	},
	_createMenuItem: function(/* object */ bundle, /* boolean */ dirty) {
		var dashboard = bundle.dashboard.binding;
		var name = dashboard.getTitle(),
			href = Util.getDashboardHref(bundle),
			iconClass;
		switch(bundle.scope) {
			case DashboardConstants.SCOPE_CONTRIBUTOR:
				if (bundle.dashboard.securityContext.indexOf(Constants.CONTEXT_PUBLIC) >= 0)
					iconClass = "shared-dash";
				else
					iconClass = "my-dash";
				break;
			case DashboardConstants.SCOPE_TEAM_AREA:
				iconClass = "team-dash";
				break;
			case DashboardConstants.SCOPE_PROJECT_AREA:
				iconClass = "project-dashboard";
				break;
		}
		var label = HTML.escape(name);
		if (dirty) {
			label = "<span class='j-dirty'>* " + label + "</span>";
		}
		return {
			label: label,
			iconClass: iconClass,
			href: href,
			url: bundle.dashboardUrl
		};
	},
	_updateRecentCount: function() {
		if (!this._recentDashboards) return 0;
		var length = this._recentDashboards.get("items").length;
		this._recentDashboards.set("groupTitle", "<b>" + messages.recentDashboards + "</b> (" + length + ")");
		return length;
	},
	onDirtyStateChange: function(/* object */ bundle, /* boolean */ isDirty) {
		if (isDirty) {
			if (this._recentDashboards) {
				var items = this._recentDashboards.get("items");
				var href = Util.getDashboardHref(bundle);
				if (items) {
					for (var i = 0; i < items.length; i++) {
						var item = items[i];
						if (this._isSameDashboard(item.href,href)) {
							this._recentDashboards.remove(item);
							break;
						}
					}
					if (!this._updateRecentCount()) {
						this._recentDashboards.remove();
						this._recentDashboards.destroy();
						delete this._recentDashboards;
					}
				}
				else {
					// load in progress - store the href to be used by the Deferred
					// when building the recent menu group items
					this._unsavedHref=href;
				}
			}
			if (!this._unsavedDashboards) {
				this._unsavedDashboards = new MenuItemGroup({
					menu: this,
					wrapHint: this._recentDashboards ? WrapHints.NO_WRAP : WrapHints.FORCE_WRAP,
					groupTitle: "<b>" + messages.unsavedDashboards + "</b> (1)",
					items: [this._createMenuItem(bundle, true)]
				});
			}
		} else if (this._unsavedDashboards) {
			items = this._unsavedDashboards.get("items");
			if (items && items.length) {
				this._unsavedDashboards.remove(items[0]);
				this._addRecentItem(this._createMenuItem(bundle));
			}
			this._unsavedDashboards.remove();
			this._unsavedDashboards.destroy();
			delete this._unsavedDashboards;
			delete this._unsavedHref;
		}
	},
	getSelected: function() {
		return 0;
		var items = this.getItems();
		var sel = [];
		for (var i = 0; i < items.length; i++) {
			var item = items[i];
			if (item && item.href) {
				var href = location.href;
				if (href.indexOf(item.href) == href.length - item.href.length) {
					sel.push(item);
				}
			}
		}
		return sel;
	},
	uninitialize: function() {
		if (this._projectDashboards) this._projectDashboards.destroy();
		if (this._unsavedDashboards) this._unsavedDashboards.destroy();
		if (this._recentDashboards) this._recentDashboards.destroy();
	}
});
})();