/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2011, 2013. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.process.web.ui.accessgroup.AccessGroupDialog"); //$NON-NLS-1$

dojo.require("com.ibm.team.process.web.common.internal.ProcessConstants"); //$NON-NLS-1$
dojo.require("com.ibm.team.process.web.client.internal.AccessGroupClient"); //$NON-NLS-1$
dojo.require("com.ibm.team.process.web.ui.internal.admin.projects.UIHelper"); //$NON-NLS-1$
dojo.require("com.ibm.team.process.web.ui.internal.ProcessAreaSelectionDialog"); //$NON-NLS-1$
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler"); //$NON-NLS-1$
dojo.require("com.ibm.team.repository.web.ui.internal.TeamContributorSelectionDialog"); //$NON-NLS-1$

dojo.require("dijit._Widget"); //$NON-NLS-1$
dojo.require("dijit._TemplatedMixin"); //$NON-NLS-1$
dojo.require("dijit._WidgetsInTemplateMixin"); //$NON-NLS-1$
dojo.require("dijit.form.Button"); //$NON-NLS-1$
dojo.require("dijit.form.TextBox"); //$NON-NLS-1$

dojo.require("dojo.i18n"); //$NON-NLS-1$

dojo.require("dojox.xml.parser"); //$NON-NLS-1$

dojo.require("jazz.ui.ListBox"); //$NON-NLS-1$
dojo.require("jazz.ui.toolbar.Button");//$NON-NLS-1$

dojo.requireLocalization("com.ibm.team.process.web.ui.accessgroup", "Messages"); //$NON-NLS-1$ //$NON-NLS-2$

var AccessGroupClient = com.ibm.team.process.web.client.internal.AccessGroupClient;
var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;
var ProcessConstants = com.ibm.team.process.web.common.internal.ProcessConstants;
var UIHelper = com.ibm.team.process.web.ui.internal.admin.projects.UIHelper;
var MemberType = com.ibm.team.process.web.ui.accessgroup.memberType;
var ContextType = com.ibm.team.process.web.ui.accessgroup.contextType;

var MEMBER_ADDED_ACTION = 1;
var MEMBER_REMOVED_ACTION = 0;

(function(){

	/**
	 * Constructs a new Access Group dialog that can be used to create or edit an 
	 * access group. All fields must be initialized with arguments passed in the 
	 * widget constructor.
	 * 
	 * PROVISIONAL: This API is considered provisional and may be changed or removed in the
	 * future with minimal or no warning.
	 * 
	 * @name com.ibm.team.process.web.ui.accessgroup.AccessGroupDialog
	 * @constructor
	 * @extends dijit._Widget
	 * @extends dijit._TemplatedMixin
	 * @extends dijit._WidgetsInTemplateMixin
	 */ 
	dojo.declare("com.ibm.team.process.web.ui.accessgroup.AccessGroupDialog", [dijit._Widget, dijit._TemplatedMixin, dijit._WidgetsInTemplateMixin], { //$NON-NLS-1$
		templatePath: dojo.moduleUrl("com.ibm.team.process.web", "ui/accessgroup/templates/AccessGroupDialog.html"), //$NON-NLS-1$ //$NON-NLS-2$
		
		/* 
		 * Constructs the dialog to create or edit and access group.
		 * 
		 * Expects an argument of the format:
		 *  args = {
		 * 		accessGroupItemIdToEdit: the item id of the access group to edit (optional, String),
		 *  	owningApplicationKey: the key for the application (required, String),
		 *  	onSaveSuccess: the function callback when the access group is saved (required), the saved access group will be passed as an instance of com.ibm.team.process.web.ui.accessgroup.AccessGroup
		 *  	OnSaveError: the function callback for when the access group save fails (required),
		 *      OnLoadError: the function callback for when the access group to edit fails to load (optional unless accessGroupItemIdToEdit is included),
		 *  }
		 *  
		 */
		constructor: function(args){
			this.messages = dojo.i18n.getLocalization("com.ibm.team.process.web.ui.accessgroup", "Messages"); //$NON-NLS-1$ //$NON-NLS-2$

			if (!args.owningApplicationKey || !dojo.isString(args.owningApplicationKey)) {
				throw new Error("Missing 'owningApplicationKey' String parameter."); //$NON-NLS-1$
			}			
	
			if (!dojo.isFunction(args.onSaveSuccess)) {
				throw new Error("Missing function for 'onSaveSuccess' parameter."); //$NON-NLS-1$
			}			
	
			if (!dojo.isFunction(args.onSaveError)) {
				throw new Error("Missing function for 'onSaveError' parameter."); //$NON-NLS-1$
			}
			
			if (this._editingAccessGroup()) {
				// If editing an access group then a load error handler is needed
				if (!dojo.isFunction(args.onLoadError)) {
					throw new Error("Missing function for 'onLoadError' parameter."); //$NON-NLS-1$
				}
			}			
			
		},
		
		postCreate: function(){
			this._memberSelect.iconMap[MemberType.PROJECT_AREA] = dojo.moduleUrl("com.ibm.team.process.web", "ui/internal/images/project.gif"); //$NON-NLS-1$ //$NON-NLS-2$
			this._memberSelect.iconMap[MemberType.TEAM_AREA] = dojo.moduleUrl("com.ibm.team.process.web", "ui/internal/images/team.gif"); //$NON-NLS-1$ //$NON-NLS-2$
			this._memberSelect.iconMap[MemberType.CONTRIBUTOR] = dojo.moduleUrl("com.ibm.team.process.web", "ui/internal/images/user_icon.png"); //$NON-NLS-1$ //$NON-NLS-2$
			this._memberSelect.onClick = dojo.hitch(this, this._handleMemberSelection);
			this.connect(this.groupName, "onkeyup", "_validateSave"); //$NON-NLS-1$ //$NON-NLS-2$
			this.connect(this.groupDescription, "onkeyup", "_validateSave"); //$NON-NLS-1$ //$NON-NLS-2$
			this.connect(this._cancelButton, "onclick", function(e){ //$NON-NLS-1$
				dojo.stopEvent(e);
				this.close();
			});
			this.connect(this._saveButton, "onclick", this._save); //$NON-NLS-1$
			
			if (dojo.isFF) {
				// Firefox has an issue where a text area of 1 
				// row is actually a text area of 2 rows.
				this.groupDescription.rows = 1;
			}
			
			this.dialog = new jazz.ui.Dialog({
				primaryTitle: this._editingAccessGroup() ? this.messages.AccessGroupDialog_EditPrimaryTitle : this.messages.AccessGroupDialog_CreatePrimaryTitle,
				contentNode: this.domNode,
				width: "35em", //$NON-NLS-1$
				destroyContents: true
			});
			
			if (this._editingAccessGroup()) {
				this._loadAccessGroup(this.accessGroupItemIdToEdit);
			} else {
				this.groupName.focus();
			}
			
		},
		
		startup: function(){
			UIHelper.resizeI18NButtons(this.dialogFooter, [this._saveButton, this._cancelButton]);
			this.inherited(arguments);
		},
			
		_loadAccessGroup: function(accessGroupItemId) {
			this._memberSelect.showLoading(true);
						
			this._enableActions(false);
			this._enableButtons(false);
			this._enableControls(false);
			
			var srh = new ServiceResponseHandler(this, "_loadAccessGroupCallback", "_loadAccessGroupErrorCallback"); //$NON-NLS-1$ //$NON-NLS-2$
			AccessGroupClient.getAccessGroupByUUID(srh, {
				itemId: accessGroupItemId,
				includeMembers: true
			});
		},
		
		_loadAccessGroupCallback: function(accessGroup) {
			this._accessGroup = accessGroup;
			this.groupName.value = accessGroup.name;
			this.groupDescription.value = accessGroup.description;
			
			// Hold the original members to compare them with the modified list
			// while editing. This is primarily to figure out the removed members. 
			this._originalMembers = accessGroup.members;
			dojo.forEach(this._originalMembers, dojo.hitch(this, function(member){
				this._memberSelect.addEntry({
					label: member.name,
					id: member.itemId,
					type: member.type,
					contextType: member.contextType
				});
			}));
			
			this._memberSelect.showLoading(false);			
			this._enableActions(true);
			this._enableButtons(true);
			this._enableControls(true);
			
			this._validateSave();
			this._handleMemberSelection();
			this._cancelButton.focus();
		},
		
		_loadAccessGroupErrorCallback: function(error) {
			this.close();
			if (this.onLoadError) {
				this.onLoadError(error);
			}
			
		},
		
		_addContributorClick: function() {
			var disabledUsers = []; 
			dojo.forEach(this._memberSelect.getAllEntries(), function(entry){
				if (entry.type == MemberType.CONTRIBUTOR) {
					entry.itemId = entry.id;
					disabledUsers.push(entry);
				}
			});
			
			this._memberSelect.getAllEntries();
			var userPicker = new com.ibm.team.repository.web.ui.internal.TeamContributorSelectionDialog({
				destroyContents: true,
				allowArchivedUsers: false,
				'disableUsers': disabledUsers //$NON-NLS-1$
			});
			this.connect(userPicker, "onAdd", this._addContributor); //$NON-NLS-1$
		},
		
		_addContributor: function(newMembers) {
			for (var i = 0; i < newMembers.length; i++) {
				var member = newMembers[i];
				this._memberSelect.addEntry({
					label: member.name,
					id: member.itemId,
					type: MemberType.CONTRIBUTOR,
					contextType: ContextType.CONTRIBUTOR
				});
			}
			this._validateSave();			
		},
		
		_addProcessAreaClick: function() {
			var processAreaPicker = new com.ibm.team.process.web.ui.internal.ProcessAreaSelectionDialog({
				projects : true,
				teams : true,
				onOk : dojo.hitch(this, this._addProcessArea),
				owningApplicationKey: this.owningApplicationKey 
			});
		},
		
		_addProcessArea: function(processArea) {
			this._memberSelect.addEntry({
				label: processArea.name,
				id: processArea.itemId,
				type: processArea.project ? MemberType.PROJECT_AREA : MemberType.TEAM_AREA,
				contextType: ContextType.MEMBERS  // Currently only support this type
			});
			this._validateSave();	
		},
		
		_removeMember: function() {
			this._memberSelect.removeEntries(this._memberSelect.getSelection());
			this._handleMemberSelection();
			this._validateSave();
		},
		
		_handleMemberSelection: function() {
			var numSelected = this._memberSelect.getSelection().length;
			this._removeMemberButton.set("enabled", numSelected > 0); //$NON-NLS-1$
		},
		
		_validateSave: function() {
			if (this._isValid(this.groupName.value) && this._memberSelect.getEntryCount() > 0) {
				this._saveButton.disabled = false;
				return;
			}
			this._saveButton.disabled = true;
		},
			
		_isValid: function(value) {
			var value = dojo.string.trim(value);
			return value.length > 0;
		},
		
		_save: function() {
			this._messageBox.showMessage({
				summary : dojo.string.substitute(this.messages.AccessGroupDialog_Saving, [this.groupName.value])
			});
			
			this._enableActions(false);
			this._enableButtons(false);
			this._enableControls(false);
			
			// No way to disable a jazz listbox
			// just remove the listener for now
			this._memberSelect.onClick = function() { };
			
			var srh = new ServiceResponseHandler(this, "_onSaveSuccess", "_onSaveError"); //$NON-NLS-1$ //$NON-NLS-2$
			AccessGroupClient.postAccessGroup(srh, {
				itemId: this._editingAccessGroup() ? this._accessGroup.itemId : "new", // $NON-NLS-1$
				stateId: this._editingAccessGroup() ? this._accessGroup.stateId : "new", // $NON-NLS-1$
				owningApplicationKey: this.owningApplicationKey,
				name: dojo.string.trim(this.groupName.value),
				description: this.groupDescription.value,
				members: this._getMembers()
			});
			
		},
		
		_getMembers: function() {
			var entries = this._memberSelect.getAllEntries();
			var members = [];
			
			// Get the members added.
			for(var i = 0; i < entries.length; i++) {
				members.push({
					itemId : entries[i].id,
					type : entries[i].type,
					contextType: entries[i].contextType,
					action : MEMBER_ADDED_ACTION
				});
			}
			
			// Get the removed members (if any) in case of edit mode.
			
			// It is important to send removed members separately because
			// the current user might not be getting the full list of members
			// because of his read permissions. And therefore, we cannot blindly
			// replace the members while editing.
			if (this._editingAccessGroup()) {
				var removedMembers = this._getRemovedMembers(members);
				dojo.forEach(removedMembers, function(removedMember) {
					members.push({
						itemId : removedMember.itemId,
						type : removedMember.type,
						contextType : removedMember.contextType,
						action : MEMBER_REMOVED_ACTION
					});					
				}, this);
			}
		
			// Convert members into json objects.
			var jsonMembers = dojo.map(members, function(member) {
				return dojo.toJson(member);
			}, this)
			
			return jsonMembers;
		},
		
		_getRemovedMembers: function(currMembers) {
			// Summary: 
			//	Returns the list of members which have been removed while editing.
			return dojo.filter(this._originalMembers, function(originalMember) {
				return !dojo.some(currMembers, function(currMember){
					return originalMember.itemId === currMember.itemId;
				}, this);
			}, this);
		},
		
		_onSaveSuccess: function(accessGroupDTO) {
			this.close();
			if (this.onSaveSuccess) {
				var accessGroup = AccessGroupClient.convertFromAccessGroupDTOToAccessGroup(accessGroupDTO)
				this.onSaveSuccess(accessGroup);
			}
		},
		
		_onSaveError: function(error) {
			this.close();
			if (this.onSaveError) {
				if (error.errorClass === "com.ibm.team.process.internal.common.service.AccessGroupValidationException") { //$NON-NLS-1$
					delete error.errorClass;
					delete error.exception;
				}
				this.onSaveError(error);
			}
		},
				
		_enableActions: function(enable) {
			this._addUserButton.set("enabled", enable); //$NON-NLS-1$
			this._addProcessAreaButton.set("enabled", enable); //$NON-NLS-1$
			this._removeMemberButton.set("enabled", enable); //$NON-NLS-1$
		},
		
		_enableButtons: function(enable) {
			this._saveButton.disabled = !enable;
			this._cancelButton.disabled = !enable;
		},
		
		_enableControls: function(enable) {
			this.groupName.disabled = !enable;
			this.groupDescription.disabled = !enable;
			// No way currently to disable the jazz listbox
			//this._memberSelect.disabled = !enable;
			
		},
		
		_editingAccessGroup: function() {
			return this.accessGroupItemIdToEdit && dojo.isString(this.accessGroupItemIdToEdit);
		},
			
		close: function() {
			this.dialog.close();
		}
	});
	
})();
