/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2011, 2013. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *
 *******************************************************************************/
dojo.provide("com.ibm.team.process.web.ui.accessgroup.AccessGroupSelectionDialog"); //$NON-NLS-1$

dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler"); //$NON-NLS-1$
dojo.require("com.ibm.team.process.web.client.internal.AccessGroupClient"); //$NON-NLS-1$
dojo.require("com.ibm.team.process.web.ui.internal.AccessGroupDetailsDialog"); //$NON-NLS-1$

dojo.require("dijit._Widget"); //$NON-NLS-1$
dojo.require("dijit._TemplatedMixin"); //$NON-NLS-1$
dojo.require("dijit._WidgetsInTemplateMixin"); //$NON-NLS-1$

dojo.require("dojo.i18n"); //$NON-NLS-1$

dojo.require("jazz.ui.Dialog"); //$NON-NLS-1$
dojo.require("jazz.ui.SearchBox"); //$NON-NLS-1$
dojo.require("jazz.ui.toolbar.Button"); //$NON-NLS-1$

dojo.requireLocalization("com.ibm.team.process.web.ui.accessgroup", "Messages"); //$NON-NLS-1$ //$NON-NLS-2$

var UIHelper = com.ibm.team.process.web.ui.internal.admin.projects.UIHelper;
var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;
var AccessGroupClient = com.ibm.team.process.web.client.internal.AccessGroupClient; 
var MAX_RESULTS = 250;

	/**
	 * Constructs a new Access Group Selection dialog that can be used to select from
	 * a list of a vailable access groups. All fields must be initialized with arguments passed  
	 * in the widget constructor.
	 * 
	 * PROVISIONAL: This API is considered provisional and may be changed or removed in the
	 * future with minimal or no warning.
	 * 
	 * @name com.ibm.team.process.web.ui.accessgroup.AccessGroupSelectionDialog
	 * @constructor
	 * @extends dijit._Widget
	 * @extends dijit._TemplatedMixin
	 * @extends dijit._WidgetsInTemplateMixin
	 */ 
(function(){
	dojo.declare("com.ibm.team.process.web.ui.accessgroup.AccessGroupSelectionDialog", [dijit._Widget, dijit._TemplatedMixin, dijit._WidgetsInTemplateMixin], { //$NON-NLS-1$
		templatePath: dojo.moduleUrl("com.ibm.team.process.web", "ui/accessgroup/templates/AccessGroupSelectionDialog.html"), //$NON-NLS-1$ //$NON-NLS-2$
		
		/* 
		 * Constructs the dialog to select an access group.
		 * 
		 * Expects an argument of the format:
		 *  args = {
		 * 		data: client supplied data that will be passed to the onOkClick handler (optional),
		 *  	owningApplicationKey: the key for the application (required, String),
		 *  	onOkClick: the function callback when the access group is selected and the ok button clicked (required), the select access group will be passed as an instance of com.ibm.team.process.web.ui.accessgroup.AccessGroup along with any user supplied data
		 *      OnLoadError: the function callback for when the access group fails to load available access groups (required),
		 *  }
		 *  
		 */
		constructor: function(args){
			this.messages = dojo.i18n.getLocalization("com.ibm.team.process.web.ui.accessgroup", "Messages"); //$NON-NLS-1$ //$NON-NLS-2$	
			
			if (!args.owningApplicationKey || !dojo.isString(args.owningApplicationKey)) {
				throw new Error("Missing 'owningApplicationKey' String parameter."); //$NON-NLS-1$
			}			
	
			if (!dojo.isFunction(args.onOkClick)) {
				throw new Error("Missing function for 'onOkClick' parameter."); //$NON-NLS-1$
			}			
	
			if (!dojo.isFunction(args.onLoadError)) {
				throw new Error("Missing function for 'onLoadError' parameter."); //$NON-NLS-1$
			}
			
		},
		
		postCreate: function(){
			this._setDialogEnabled(false);
			this.connect(this._accessGroupSelector, "onchange", this._handleAccessGroupSelection); //$NON-NLS-1$
			this.connect(this._cancelButton, "onclick", this._onCancelClick); //$NON-NLS-1$
			this.connect(this._okButton, "onclick", this._onOkClick); //$NON-NLS-1$

			this._createSearchBox();

			
			this._dialog = new jazz.ui.Dialog({
				primaryTitle: this.messages.AccessGroupSelectionDialog_PrimaryTitle,
				contentNode: this.domNode,
				width: "35em", //$NON-NLS-1$
				destroyContents: true
			});
			
			// Workitem 180854...have to use the private attach
			// point until there is a public mechanism
			this._searchBox._searchText.focus();
		},
		
		startup: function(){
			UIHelper.resizeI18NButtons(this.dialogFooter, [this._okButton, this._cancelButton]);
			this.inherited(arguments);
		},

		_createSearchBox: function(){
			this._searchBox = new jazz.ui.SearchBox({
				allowEmptySearches: true,
				scopes: [{
					id: "accessGroups", //$NON-NLS-1$
					label: this.messages.AccessGroupSelectionDialog_AccessGroups, //$NON-NLS-1$
					icon: dojo.moduleUrl("com.ibm.team.process.web", "ui/internal/admin/accessgroup/images/access-group_obj.gif") //$NON-NLS-1$ //$NON-NLS-2$
				}],
				onExecute: dojo.hitch(this, function(searchFilter){
					this._getAccessGroups(searchFilter);
				})
			});
			this._searchDiv.appendChild(this._searchBox.domNode);
			
		},
		
		_setDialogEnabled: function(enable){
			this._accessGroupSelector.disabled = !enable;
			var shouldEnable = this._shouldEnableButtons(enable);
			this._okButton.disabled = !shouldEnable;
			this._showDetailsButton.set("enabled", shouldEnable);  //$NON-NLS-1$ 
		},
		
		_shouldEnableButtons: function(enable){
			if (enable) {
				return !this._accessGroupSelector.disabled && this._accessGroupSelector.selectedIndex > -1;
			}
			return false;
		},
		
		_onOkClick: function(e) {
			dojo.stopEvent(e);
			this._setDialogEnabled(false);
			this._getAccessGroup(this._selectedAccessGroup);
		},
		
		_onCancelClick: function(e) {
			dojo.stopEvent(e);
			this._dialog.close();
		},
		
		_getAccessGroups: function(searchFilter){
			this._displayPending();
			var srh = new ServiceResponseHandler(this, "_populateAccessGroups", "_onLoadError"); //$NON-NLS-1$ //$NON-NLS-2$
            AccessGroupClient.getAvailableAccessGroups(srh, {
				owningApplicationKey: this.owningApplicationKey,
				nameFilter: "*"+searchFilter+"*", //$NON-NLS-1$ //$NON-NLS-2$
				maxResults: MAX_RESULTS
            });
		},
		
		_getAccessGroup: function(accessGroup){
			var srh = new ServiceResponseHandler(this, "_processAccessGroup", "_onLoadError"); //$NON-NLS-1$ //$NON-NLS-2$
            AccessGroupClient.getAccessGroupByUUID(srh, {
				itemId: accessGroup.itemId,
				includeMembers: true
            });
		},
		
		_processAccessGroup: function(accessGroupDTO) {
			this._dialog.close();
			if (this.onOkClick) {
				this.onOkClick(AccessGroupClient.convertFromAccessGroupDTOToAccessGroup(accessGroupDTO), this.data);
			}			
		},

		_showDetails: function() {
			var accessGroupDetailsDialog = new com.ibm.team.process.web.ui.internal.AccessGroupDetailsDialog({
				accessGroupItemIdToView: this._selectedAccessGroup.itemId,
				owningApplicationKey: this.owningApplicationKey,
				onLoadError: this.onLoadError
			});
			accessGroupDetailsDialog.startup();
		},
		
		_populateAccessGroups: function(accessGroups){
			this._clearAccessGroups();
 			if(accessGroups.count === 0){
				this._handleNoAccessGroups();
				return;
			}
			
			var newOptions = dojo.map(accessGroups.elements, function(accessGroup){
                var option = new Option(accessGroup.name, accessGroup.name);
                option.accessGroup = accessGroup;
                return option;
			}, this);
						
			newOptions.sort(function(a, b){
				return a.text.localeCompare(b.text);
			});
			
			dojo.forEach(newOptions, function(option){
				this._addAccessGroupOption(option);
			}, this);
			
			this._setDialogEnabled(true);			
		},
		
		_handleAccessGroupSelection: function(){
			var shouldEnable = this._shouldEnableButtons(true);
			this._okButton.disabled = !shouldEnable;
			this._showDetailsButton.set("enabled", shouldEnable);  //$NON-NLS-1$ 
			this._selectedAccessGroup = this._accessGroupSelector.options[this._accessGroupSelector.selectedIndex].accessGroup;
		},
		
		_displayPending: function() {
			this._clearAccessGroups();
			this._setDialogEnabled(false);			
            var option = new Option(this.messages.AccessGroupSelectionDialog_Pending, this.messages.AccessGroupSelectionDialog_PrimaryTitle);
            dojo.addClass(option, "pending"); //$NON-NLS-1$
			this._addAccessGroupOption(option);
			
		},
		
		_addAccessGroupOption: function(option) {
			if (dojo.isIE) {
				this._accessGroupSelector.options[this._accessGroupSelector.options.length] = option;
			} else {
				this._accessGroupSelector.appendChild(option);
			}
		},
		
		_clearAccessGroups: function() {
			this._accessGroupSelector.options.length = 0;
		},
		
		_handleNoAccessGroups: function(){
			this._clearAccessGroups();
			this._setDialogEnabled(false);
            var option = new Option(this.messages.AccessGroupSelectionDialog_NoAccessGroups, this.messages.AccessGroupSelectionDialog_NoAccessGroups);
            dojo.addClass(option, "noGroupsFound"); //$NON-NLS-1$
			this._addAccessGroupOption(option);
		},
		
		_onLoadError: function(error){
			if (this.onLoadError) {
				this.onLoadError(error);
			}
		},
		
		uninitialize: function(){
			if (this._searchBox) {
				this._searchBox.destroyRecursive();
			}
			
			this.inherited(arguments);
		}
		
		
	});
})();
