/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2011, 2013. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.process.web.ui.internal.AccessGroupDetailsDialog"); //$NON-NLS-1$

dojo.require("com.ibm.team.process.web.client.internal.AccessGroupClient"); //$NON-NLS-1$
dojo.require("com.ibm.team.process.web.ui.internal.admin.projects.UIHelper"); //$NON-NLS-1$
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler"); //$NON-NLS-1$

dojo.require("dijit._Widget"); //$NON-NLS-1$
dojo.require("dijit._TemplatedMixin"); //$NON-NLS-1$
dojo.require("dijit._WidgetsInTemplateMixin"); //$NON-NLS-1$

dojo.require("dojo.i18n"); //$NON-NLS-1$

dojo.require("jazz.ui.ListBox"); //$NON-NLS-1$

dojo.requireLocalization("com.ibm.team.process.web.ui.internal", "Messages"); //$NON-NLS-1$ //$NON-NLS-2$

var AccessGroupClient = com.ibm.team.process.web.client.internal.AccessGroupClient;
var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;
var UIHelper = com.ibm.team.process.web.ui.internal.admin.projects.UIHelper;
var MemberType = com.ibm.team.process.web.ui.accessgroup.memberType;

(function(){

	/**
	 * Constructs a new Access Group Details dialog that can be view the details 
	 * of an access group. All fields must be initialized with arguments passed in the 
	 * widget constructor.
	 * 
	 * @name com.ibm.team.process.web.ui.accessgroup.AccessGroupDetailsDialog
	 * @constructor
	 * @extends dijit._Widget
	 * @extends dijit._TemplatedMixin
	 * @extends dijit._WidgetsInTemplateMixin
	 */ 
	dojo.declare("com.ibm.team.process.web.ui.internal.AccessGroupDetailsDialog", [dijit._Widget, dijit._TemplatedMixin, dijit._WidgetsInTemplateMixin], { //$NON-NLS-1$
		templatePath: dojo.moduleUrl("com.ibm.team.process.web", "ui/internal/templates/AccessGroupDetailsDialog.html"), //$NON-NLS-1$ //$NON-NLS-2$
		
		/* 
		 * Constructs the dialog to view the details of the access group
		 * 
		 * Expects an argument of the format:
		 *  args = {
		 * 		accessGroupItemIdToView: the item id of the access group to view (read, String),
		 *  	owningApplicationKey: the key for the application (required, String),
		 *      OnLoadError: the function callback for when the access group to view fails to load (required)
		 *  }
		 *  
		 */
		constructor: function(args){
			this.messages = dojo.i18n.getLocalization("com.ibm.team.process.web.ui.internal", "Messages"); //$NON-NLS-1$ //$NON-NLS-2$

			if (!args.owningApplicationKey || !dojo.isString(args.owningApplicationKey)) {
				throw new Error("Missing 'owningApplicationKey' String parameter."); //$NON-NLS-1$
			}
						
			if (!dojo.isFunction(args.onLoadError)) {
				throw new Error("Missing function for 'onLoadError' parameter."); //$NON-NLS-1$
			}
		
		},
		
		postCreate: function(){
			this._memberSelect.iconMap[MemberType.PROJECT_AREA] = dojo.moduleUrl("com.ibm.team.process.web", "ui/internal/images/project.gif"); //$NON-NLS-1$ //$NON-NLS-2$
			this._memberSelect.iconMap[MemberType.TEAM_AREA] = dojo.moduleUrl("com.ibm.team.process.web", "ui/internal/images/team.gif"); //$NON-NLS-1$ //$NON-NLS-2$
			this._memberSelect.iconMap[MemberType.CONTRIBUTOR] = dojo.moduleUrl("com.ibm.team.process.web", "ui/internal/images/user_icon.png"); //$NON-NLS-1$ //$NON-NLS-2$
			this.connect(this._closeButton, "onclick", function(e){ //$NON-NLS-1$
				dojo.stopEvent(e);
				this.close();
			});
			
			if (dojo.isFF) {
				// Firefox has an issue where a text area of 1 
				// row is actually a text area of 2 rows.
				this._groupDescription.rows = 1;
			}
			
			this.dialog = new jazz.ui.Dialog({
				primaryTitle: this.messages.AccessGroupDetailsDialog_PrimaryTitle,
				contentNode: this.domNode,
				width: "35em", //$NON-NLS-1$
				destroyContents: true
			});
			
			this._loadAccessGroup(this.accessGroupItemIdToView);
			
		},
		
		startup: function(){
			UIHelper.resizeI18NButtons(this.dialogFooter, [this._closeButton]);
			this.inherited(arguments);
		},
			
		_loadAccessGroup: function(accessGroupItemId) {
			this._memberSelect.showLoading(true);
						
			var srh = new ServiceResponseHandler(this, "_loadAccessGroupCallback", "_loadAccessGroupErrorCallback"); //$NON-NLS-1$ //$NON-NLS-2$
			AccessGroupClient.getAccessGroupByUUID(srh, {
				itemId: accessGroupItemId,
				includeMembers: true
			});
		},
		
		_loadAccessGroupCallback: function(accessGroup) {
			this._accessGroup = accessGroup;
			this._groupName.value = accessGroup.name;
			this._groupDescription.value = accessGroup.description;
			
			dojo.forEach(accessGroup.members, dojo.hitch(this, function(member){
				this._memberSelect.addEntry({
					label: member.name,
					id: member.itemId,
					type: member.type
				});
			}));
			
			this._memberSelect.showLoading(false);
			this._closeButton.disabled = false;			
			this._closeButton.focus();
		},
		
		_loadAccessGroupErrorCallback: function(error) {
			this.close();
			if (this.onLoadError) {
				this.onLoadError(error);
			}
			
		},
				
		close: function() {
			this.dialog.close();
		}
	});
	
})();
