/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010, 2013. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.reports.web.ui.internal.AllReportTemplatesTab");

dojo.require("dojo.i18n");

dojo.require("com.ibm.team.reports.web.client.internal.ReportsClient");
dojo.require("com.ibm.team.repository.web.client.session.Session");
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
dojo.require("com.ibm.team.reports.web.ui.internal.MetaNav");
dojo.require("com.ibm.team.reports.web.ui.internal.utils.StringUtils");

dojo.require("jazz.app.context");
dojo.require("jazz.ui.FilterBox");
dojo.require("jazz.ui.MessageBox");
dojo.require("jazz.ui.StyledTable");
dojo.require("jazz.ui.ResourceLink");
dojo.require("jazz.ui.SimpleToolbar");

dojo.require("com.ibm.team.process.web.ui.internal.ProcessAreaSelectionDialog");
dojo.require("net.jazz.ajax.ui.PlatformUI");
dojo.require("net.jazz.ajax.BootstrapProperties");

dojo.requireLocalization("com.ibm.team.reports.web.ui.internal", "ReportMessages");

(function() {

var session = com.ibm.team.repository.web.client.session;
var ReportsClient = com.ibm.team.reports.web.client.internal.ReportsClient;
var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;
var FilterBox = jazz.ui.FilterBox;
var StyledTable = jazz.ui.StyledTable;
var ResourceLink = jazz.ui.ResourceLink;
var ServerStatusClient = com.ibm.team.repository.web.client.internal.ServerStatusClient;
var StringUtils = com.ibm.team.reports.web.ui.internal.utils.StringUtils;

var PlatformUI = net.jazz.ajax.ui.PlatformUI;

var messages = dojo.i18n.getLocalization("com.ibm.team.reports.web.ui.internal", "ReportMessages");

var AllReportTemplatesTab = dojo.declare("com.ibm.team.reports.web.ui.internal.AllReportTemplatesTab",
	[dijit._Widget, dijit._Templated], {
		
	templatePath: dojo.moduleUrl("com.ibm.team.reports.web", "ui/internal/templates/AllReportTemplatesTab.html"),
	_pendingDeleteNode: null,
	deployTemplatesAction: "#action=com.ibm.team.reports.deployNewTemplates",
	editTemplateAction: "#action=com.ibm.team.reports.editTemplate",
	editCustomTemplateAction: "#action=com.ibm.team.reports.editCustomTemplate",
	
	constructor: function(args) {
		dojo.mixin(this, args);
		this.templatesHeading = messages.reportTemplates;
	},
	
	postCreate: function() {
		this.metanav = new com.ibm.team.reports.web.ui.internal.MetaNav({AllReportTemplatesTab: this}, this.metanav);
		this._populateHeader();
		this._filterBox = new FilterBox({onChange: dojo.hitch(this,this._doFilter)});
		this._filterArea.appendChild(this._filterBox.domNode);
		this._table = new StyledTable({columns: [{label: "", width: "3", sortable: false}, {label: messages.reportTemplate, width: "30%", sortable: false}, {label: messages.descriptioncolumnheader, width: "60%", sortable: false},{label: messages.actions, width: "10%", sortable: false}], 
			trim: jazz.ui.StyledTable.TrimStyles.PRIMARY_HEADLESS, showHover: true});
		this._resultsArea.appendChild(this._table.domNode);
		
		srh = new ServiceResponseHandler(this, "_customReportingConfigured", "_handleCustomReportingConfiguredError");
		ServerStatusClient.getServiceConfiguration(srh, "com.ibm.team.reports.service.cognos.internal.CognosReportService");
		
		this.messageBox = new jazz.ui.MessageBox();
		this._messageBoxContainer.appendChild(this.messageBox.domNode);
	},
	
	_populateHeader: function(){
		var heading = messages.reportTemplates;
		this.metanav.setTitle(StringUtils.escapeAndCheckForNull(heading));
		this.metanav.createContextSensitiveHelp(AllReportTemplatesTab.CSH_ALL_REPORT_TEMPLATES);
		if(this.displayBreadcrumbs) {
			this.metanav.populateReportsBreadcrumbs();
		} else {
			this.metanav.handle.style.height = "42px";
			this.metanav.handle.style.minHeight = "42px";
		}
	},

	_newTemplate: function() {
		document.location.href = this.editTemplateAction;
	},
	
	_newCustomTemplate: function() {
		document.location.href = this.editCustomTemplateAction;
//		document.location.href = "#action=com.ibm.team.reports.editCustomTemplate";
	},
	
	_deployNewTemplates: function() {
		document.location.href = this.deployTemplatesAction;
	},

	_customReportingConfigured: function(response) {
		for(var i = 0; i < response.properties.length; i++){
			if(response.properties[i].name == "cognos.hostAndPort"){
				if(response.properties[i].overrideValue && response.properties[i].overrideValue.length > 0){
					this._buildToolbar(true);
					this._runQuery();
					return;
				}
			}
		}
		this._buildToolbar(false);
		this._runQuery();
	},
	
	_handleCustomReportingConfiguredError: function() {
		this._buildToolbar(false);
		this._runQuery();
	},
	
	_buildToolbar: function(showCustom){
		this.toolbar = new jazz.ui.SimpleToolbar({});
		this.metanav._toolbarDiv.appendChild(this.toolbar.domNode);
		this.toolbar.addButton(this, this._newTemplate, {showLabel: true, imgSrc: dojo.moduleUrl("com.ibm.team.reports.web","ui/internal/images/new-template_obj.gif"), label: messages.createNewTemplate, requiresSelection:false});
		if(showCustom){
			this.toolbar.addButton(this, this._newCustomTemplate, {imgSrc: dojo.moduleUrl("com.ibm.team.reports.web","ui/internal/images/new-custom-report-template_co.gif"), label: messages.createNewTemplateFromCustomReport, requiresSelection:false});
		}
		this.toolbar.addButton(this, this._deployNewTemplates, {imgSrc: dojo.moduleUrl("com.ibm.team.reports.web","ui/internal/images/find-template_obj.gif"), label: messages.deployNewTemplates, requiresSelection:false});
		this.toolbar.addButton(this, this._runQuery, {imgSrc: dojo.moduleUrl("com.ibm.team.reports.web","ui/internal/images/refresh.gif"), label: messages.refresh, requiresSelection:false});		
		this.toolbar.addButton(this, this._deleteTemplates, {imgSrc: dojo.moduleUrl("com.ibm.team.reports.web","ui/internal/images/delete.gif"), label: messages.deleteTemplate, requiresSelection:true});		
		this.toolbar.addButton(this, this._restoreTemplates, {imgSrc: dojo.moduleUrl("com.ibm.team.reports.web","ui/internal/images/restore.gif"), label: messages.restoreInitialContents, requiresSelection:true});		
	},
	
	_runQuery: function() {
		PlatformUI.getWorkbench().showMessage(messages.allReportsSearching);
		if (this._runQueryTimeout) {
			clearTimeout(this._runQueryTimeout);
			delete this._runQueryTimeout;
		}
		if (!this._nextQueryId)
			this._nextQueryId = 1;
		var serviceObject = {
			self: this,
			id: this._nextQueryId,
			success: function(results) {
				if (this.self._lastQueryId && this.self._lastQueryId > this.id)
					return;
				this.self._lastQueryId = this.id;
				this.self._renderResults(results);
				PlatformUI.getWorkbench().hideMessageArea();
			},
			failure: function(error) {
				// TODO
				//dojo.publish(com.ibm.team.dashboard.web.ui.internal.Constants.serverError, [error]);
			}
		};
		var srh = new ServiceResponseHandler(serviceObject, "success", "failure");
		var args = {
			sortByName: true
		};
		if (this._currentFilter) {
			args.filter = this._currentFilter;
		}
		ReportsClient.getReportDescriptors(srh, args);
		this._nextQueryId++;
	},
	
	_renderResults: function(results) {
		this._table.clearBody();
		if (results && results.length > 0) {
			for (var i = 0; i < results.length; i++) {
				this._table.tbody.appendChild(this._createTemplateRow(results[i]));
			}
		} else {
			this._table.tbody.appendChild(this._createNoneMessage());
		}
		this._updateActions();
	},
	
	_createTemplateRow: function(template) {
		var row = document.createElement("tr");
		
		//Create the checkbox

		var td = document.createElement("td");
		td.className = "checkBox";
		var cb = document.createElement("input");
		cb.type = "checkbox";
		cb.checked = false;
		this.connect(cb, "onclick", this._handleCheckBoxClick);
		td.appendChild(cb);
		row.appendChild(td);
		
		td = document.createElement("td");
		var innerHTML = "<td><a href='" + this.editTemplateAction + "&reportUUID=" + template.reportUUID + "'>" + StringUtils.escapeAndCheckForNull(template.name) + "</a>";
		td.innerHTML = innerHTML;
		row.id = template.reportUUID;
		row.appendChild(td);

		td = document.createElement("td");
		td.classname = "reportName";
		td.innerHTML = StringUtils.escapeAndCheckForNull(template.description);
		row.appendChild(td);
		
		// Edit template, delete template
		td = document.createElement("td");
		var div = document.createElement("div");
		div.className = "actionBar";
		
		var self = this;
		
		// Edit
		var anchor = document.createElement("a");
		anchor.href = "#";
		anchor.title = messages.edit;
		div.appendChild(anchor);
		var img = document.createElement("img");
		img.alt = messages.edit;
		img.src = dojo.moduleUrl("com.ibm.team.reports.web", "ui/internal/images/edit.gif");
		anchor.appendChild(img);
		var handler = dojo.connect(anchor, "onclick", function(event) {
			dojo.stopEvent(event);
			self._editTemplate(template);
		});
		
		// Delete
		anchor = document.createElement("a");
		anchor.href = "#";
		anchor.title = messages.deleteTemplate;
		div.appendChild(anchor);
		img = document.createElement("img");
		img.alt = messages.deleteTemplate;
		img.src = dojo.moduleUrl("com.ibm.team.reports.web", "ui/internal/images/delete.gif");
		anchor.appendChild(img);
		var handler = dojo.connect(anchor, "onclick", function(event) {
			dojo.stopEvent(event);
			self._deleteTemplate(template);
		});

		//Restore Initial Contents
		anchor = document.createElement("a");
		anchor.href = "#";
		anchor.title = messages.restoreInitialContents;
		div.appendChild(anchor);
		img = document.createElement("img");
		img.alt = messages.restoreInitialContents;
		img.src = dojo.moduleUrl("com.ibm.team.reports.web", "ui/internal/images/restore.gif");
		anchor.appendChild(img);
		var handler = dojo.connect(anchor, "onclick", function(event) {
			dojo.stopEvent(event);
			self._restoreInitialContent(template);
		});


		td.appendChild(div);
		row.appendChild(td);
		
		return row;
	},
	
	_editTemplate: function(template) {
		this._clearStatusMessage();
		document.location.href = this.editTemplateAction + "&reportUUID=" + template.reportUUID;
	},
	
	_deleteTemplate: function(template) {
		this._clearStatusMessage();
		if (!confirm(messages.confirmDelTemplate)) {
			return;
		}
		var args = {
    		reportUUID: template.reportUUID
    	};
		var responseHandler = new ServiceResponseHandler(
				this,
				"_templateDeleted",
				"_remoteError");
		var rowNode = dojo.query("tr[id=" + template.reportUUID + "]");
		if(rowNode.length > 0){
			rowNode = rowNode[0];
			this._pendingDeleteNode = rowNode;
		}
		ReportsClient.postDeleteReportDescriptor(responseHandler, args);
	},

	_deleteTemplates: function() {
		
		this._clearStatusMessage();
		var templateIDs = this._getCheckedItems();
		if (templateIDs.length == 0)
			return;
		if (!confirm(dojo.string.substitute(messages.confirmDelTemplates, [templateIDs.length]))) 
		{
			return;
		}
		var args = {
    		reportUUIDs: templateIDs
    	};
		var responseHandler = new ServiceResponseHandler(
				this,
				"_templatesDeleted",
				"_remoteError");
		ReportsClient.postDeleteReportDescriptors(responseHandler, args);
	},
	
	_templateDeleted: function() {
		this._showStatusMessage(messages.deletionSuccessful);
		if(this._pendingDeleteNode && this._pendingDeleteNode != null){
			this._table.tbody.removeChild(this._pendingDeleteNode);
			this._pendingDeleteNode = null;
		}
	},
	
	_templatesDeleted: function()
	{
		this._runQuery();
	},
	
	_createNoneMessage: function() {
		var row = document.createElement("tr");
		var td = document.createElement("td");
		td.innerHTML = StringUtils.escapeAndCheckForNull(messages.allTemplatesNone);
		row.appendChild(td);
		return row;
	},
	
	_doFilter: function(filter) {
		this._currentFilter = filter;
		this._isRelatedFilterStale = true;
		// set a timeout to query with the new filter
		// need to ensure 0 is passed to _runQuery or setTimeout will pass some other value
		if (!this._runQueryTimeout) {
			this._runQueryTimeout = setTimeout(dojo.hitch(this,this._runQuery,0,true),500);
		}
	},
	
	_restoreInitialContent: function(template) {
		//check first - does this template have content that could be restored from the source?
		if (!template.source || !template.localContents) {
			this._showStatusMessage(messages.noRestore, jazz.ui.MessageBox.STATUS_TYPES.WARNING);
		} else {
			var handler = new ServiceResponseHandler(
					this,
					"_contentsRestored",
			"_remoteError");
			var currentProjectArea = jazz.app.context.get();
			var args = {
					projectAreaUUID: currentProjectArea.itemId,
					reportUUID: template.reportUUID
			};
			ReportsClient.postResetReportDescriptor(handler, args);
		}
	},
	
	_restoreTemplates: function()
	{
		this._clearStatusMessage();
		var templateIDs = this._getCheckedItems();
		if (templateIDs.length == 0)
			return;
			
		if (!confirm(dojo.string.substitute(messages.confirmMultiRestoreInitialContents, [templateIDs.length]))) 
		{
			return;
		}
		var currentProjectArea = jazz.app.context.get();
		var args = {
			projectAreaUUID: currentProjectArea.itemId,
    		reportUUIDs: templateIDs
    	};
		var handler = new ServiceResponseHandler(
			this,
			"_templatesRestored",
			"_remoteError");
		ReportsClient.postResetReportDescriptors(handler, args);
		
	},
			
	_contentsRestored: function(args)
	{
		this._showStatusMessage(messages.contentsRestored);
	},

	_templatesRestored: function(args)
	{
		this._showStatusMessage(messages.multipleContentsRestored);
	},
	
	_remoteError: function(error) {
		this._showStatusMessage(error.message, jazz.ui.MessageBox.STATUS_TYPES.ERROR);
	},
	
	/**
	 * Displays the given status message
	 * 
	 * type (optional) - the jazz.ui.MessageBox status type.  If not specified, the default is "OK"
	 */
	_showStatusMessage: function(message, type) {
		this._clearStatusMessage();
		var status = {
			type : type ? type : jazz.ui.MessageBox.STATUS_TYPES.OK,
			summary : message,
			closable : false
		};
		dojo.addClass(this._messageBoxContainer, "messageBoxVisible");
		this.messageBox.showMessage(status);
	},
	
	_clearStatusMessage: function() {
		dojo.removeClass(this._messageBoxContainer, "messageBoxVisible");
		this.messageBox.hideMessage();
	},
	
	_handleCheckBoxClick: function(args)
	{
		this._updateActions();
	},	

	//Update the main toolbar buttons.
	//Enables or disables buttons that have the "requiresSelection" attribute set
	//based on whether any rows are selected
	_updateActions: function()
	{
		var rowsChecked = this._getCheckedRowCount() > 0;
		var buttonCounter = 0;
		var button = null;
		while ((button = this.toolbar.getItem(buttonCounter++)) != null)
		{
			if (button.args.requiresSelection)
			{
				if (rowsChecked)
					this.toolbar.enableButton(button);
				else
					this.toolbar.disableButton(button);
			}			
		}
	},
	
	//Return the number of rows that are checked
	_getCheckedRowCount: function()
	{
		var retVal = 0;
		var body = this._table.tbody;
		var tableRows = body.children;
		for (var i = 0; i < tableRows.length; i++) 
		{
			var row = tableRows[i];
			if (this._isRowChecked(row)) 
				retVal++;
		}
		return retVal;
	},
	
	//Return an array containing the IDs of all the selected templates
	//Return an empty array if none are selected
	_getCheckedItems: function()
	{
		var retVal = [];
		var body = this._table.tbody;
		var tableRows = body.children;
		for (var i = 0; i < tableRows.length; i++)
		{
			var row = tableRows[i];
			if (this._isRowChecked(row))
				retVal.push(row.id);
		}
		return retVal;
	},

	//Get the check box from the table row
	_getCheckboxFromTR: function(trNode)
	{
		//First child is the <td> first child of that is the input (checkbox);
		if (trNode.children.length > 0) {
			var firstChild = trNode.children[0];
			if (firstChild.children.length > 0) {
				return firstChild.children[0];
			}
		}		
		return null;
	},
	
	//returns true if the provided table row is checked, false otherwise
	_isRowChecked: function(trNode)
	{
		var checkBox = this._getCheckboxFromTR(trNode);
		return checkBox === null ? false : checkBox.checked;
	},
	

});

AllReportTemplatesTab.CSH_ALL_REPORT_TEMPLATES = "com.ibm.jazz.reports.web.cshelp.all_report_templates";

})();
