/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010, 2012. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  
 * Use, duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp. 
 *******************************************************************************/
(function() {

dojo.provide("com.ibm.team.repository.web.ui.internal.LicenseInfoWidget");

dojo.require("com.ibm.team.repository.web.client.internal.HttpHeaders");
dojo.require("com.ibm.team.repository.web.ui.internal.events");
dojo.require("jazz.app.internal.util.DiscoveredService");
dojo.require("jazz.app.entryPoints");

dojo.requireLocalization("com.ibm.team.repository.web.ui.internal", "LicenseInfoWidgetMessages");
var messages = dojo.i18n.getLocalization("com.ibm.team.repository.web.ui.internal", "LicenseInfoWidgetMessages");
var DiscoveredService = jazz.app.internal.util.DiscoveredService;
var HttpConstants = com.ibm.team.repository.web.client.internal.HttpConstants;

dojo.declare("com.ibm.team.repository.web.ui.internal.LicenseInfoWidget", [dijit._Widget, dijit._Templated], {
	
	templatePath: dojo.moduleUrl("com.ibm.team.repository.web", "ui/internal/templates/LicenseInfoWidget.html"),
	
	postCreate: function() {		
		dojo.subscribe(com.ibm.team.repository.web.ui.internal.events.licenseChange, this, "getCALInfo");
		this.headers = com.ibm.team.repository.web.client.internal.HttpHeaders._getHeaders();
		this.headers[HttpConstants.ACCEPT] = HttpConstants.CT_TEXT_JSON;
		var d = DiscoveredService.loadDiscoveryServices(NAMESPACES.jfs + "LicenseInfoExpiration", LICENSE_INFO_DISCOVERY_BINDING, true);
		d.addCallback(dojo.hitch(this, function(services){
			if(services && services.LicenseInfoExpiration && services.LicenseInfoExpiration.length > 0) {
				this.licenseService = services.LicenseInfoExpiration[0].service;
				if(this.licenseService) {
					var entryPoints = jazz.app.entryPoints.get(["JazzTeamServerHome"]);
					entryPoints.addCallback(dojo.hitch(this, function(jtsHome) {
					if(jtsHome) {
						this.licenseUrl = jtsHome + "#action=com.ibm.team.repository.admin.manageLicenses";
					}
					this.getCALInfo();
				}));
				}
			}
		}));
	},
	
	notify: function(/* bool */ showMessage) {},
	
	getCALInfo: function() {
		jazz.client.xhrGet({
			url: this.licenseService + "/licenseExpirationInfo",
			handleAs: "json",
			headers: this.headers,
			load: dojo.hitch(this, function(response) {
				if(response) this._showLicenseInfo(response);
			}),
			error: function(errObject) {
				console.warn(dojo.string.substitute(messages.noLicenseInfo, [ errObject.message ]));
			}
		});
	},

	_showLicenseInfo: function(licenseInfo) {
		if(!licenseInfo || !this.domNode) return;
		
		var days_server, days_cal; 
		if(licenseInfo.server && licenseInfo.server.expires != null) {
			days_server = licenseInfo.server.expires;
		}
		if(licenseInfo.cals && licenseInfo.cals.numberOfCals > 0 && licenseInfo.cals.daysUntilExpiration != null) {
			days_cal = licenseInfo.cals.daysUntilExpiration;
		}
		
		if(days_server === -1) { //server expired
			this._displayExpiringLicense(days_server, true);
		} 
		else if(days_cal === -1) { //cal expired
			this._displayExpiringLicense(days_cal);
		} 
		else if(days_cal >= 0 && days_server >= 0) { //both are expiring, see which one expires first
			if(days_server <= days_cal) { //server expires first
				this._displayExpiringLicense(days_server, true);
			} 
			else { //cal(s) must expire first
				this._displayExpiringLicense(days_cal);
			}
		} 
		else if(days_server >= 0) { //server expiring, no cals expiring
			this._displayExpiringLicense(days_server, true);
		} 
		else if(days_cal >= 0) { //cal(s) expiring, server is not
			this._displayExpiringLicense(days_cal);
		}
		else if(!days_cal && !days_server) { //nothing is expiring.
			this.notify(false); /* clear existing licensing message(s) */
		}
	},

	_displayExpiringLicense: function(daysLeft, isServer) {
		var m, calLink;
		var calType = isServer ? messages.permLicense : messages.calLicense;
		if(this.licenseUrl) {
			calLink = dojo.string.substitute(calType, [ "<a href='" + this.licenseUrl + "'>", "</a> "]);	
		} else {
			//We don't have the URL for managing licenses, so don't show the link
			calLink = dojo.string.substitute(calType, ["",""]);
		}
		switch(daysLeft) {
			case -1: m = messages.hasExpired; break;
			case 0: m = messages.expiresToday; break;
			case 1: m = messages.expiresInOneDay; break;
			default: m = messages.expiresInDays;
		}
		this.expirationInfo.innerHTML = daysLeft > 1 ? dojo.string.substitute(m, [ calLink, daysLeft ]) : dojo.string.substitute(m, [ calLink ]);
		this.expirationWrapper.style.display = "inline";
		this.notify(true); /* show message */
	},
	
	uninitialize: function() {
		this.inherited(arguments);
	}
	
});

var NAMESPACES = {
	jd: "http://jazz.net/xmlns/prod/jazz/discovery/1.0/",
	jfs: "http://jazz.net/xmlns/foundation/1.0/",
	rdf: "http://www.w3.org/1999/02/22-rdf-syntax-ns#"
};
var LICENSE_INFO = {
	namespaces: NAMESPACES,
	service: ["string", "jd:service/@rdf:resource", "1"]
};
var LICENSE_INFO_DISCOVERY_BINDING = {
	namespaces: NAMESPACES,
	LicenseInfoExpiration: [LICENSE_INFO, "jfs:LicenseInfoExpiration", "*"]
};

})();