/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007, 2011. All Rights Reserved. 
 * 
 * Note to U.S. Government Users Restricted Rights:  Use, 
 * duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.reports.web.ui.internal.MoveReport");

dojo.require("com.ibm.team.reports.web.client.internal.ReportsClient");
dojo.require("com.ibm.team.reports.web.ui.internal.ImagePool");

dojo.require("com.ibm.team.repository.web.client.session.Session");
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");

dojo.require("jazz.app.ua.PageHoverHelp");
dojo.require("jazz.ui.MessageBox");
dojo.require("jazz.ui.Tree");
dojo.require("jazz.ui.tree.IconItem");

dojo.require("dojo.i18n");
dojo.require("dijit._Widget");
dojo.require("dijit._Templated");

dojo.requireLocalization("com.ibm.team.reports.web.ui.internal", "ReportMessages");

(function() { // module container

var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;
var ReportsClient = com.ibm.team.reports.web.client.internal.ReportsClient;
var session = com.ibm.team.repository.web.client.session;
var ImagePool = com.ibm.team.reports.web.ui.internal.ImagePool;
var PageHoverHelp = jazz.app.ua.PageHoverHelp;

var SHARED = 1 << 0;
var PRIVATE = 1 << 2;
var SYSTEM = 1 << 3;

/**
 * Widget to move a report or folder
 */
var MoveReport = dojo.declare("com.ibm.team.reports.web.ui.internal.MoveReport", [dijit._Widget, dijit._Templated], {
		templatePath: dojo.moduleUrl("com.ibm.team.reports.web", "ui/internal/templates/MoveReport.html"),
		_csPageHelpWidgets: null,
		csHelpNode: null,
		
		constructor: function(args) {
			dojo.mixin(this, args);

			this.messages = dojo.i18n.getLocalization("com.ibm.team.reports.web.ui.internal", "ReportMessages");
			this._csPageHelpWidgets= [];
		},
				
		postCreate: function() {
			this._responseCount = 0;
			this._allFolders = [];
			// A map mapping folder nodes to the underlying folder
			this._folderMap = {};
			
			this._createFolderTree();
			this.messageBox = new jazz.ui.MessageBox();
			this._messageBoxContainer.appendChild(this.messageBox.domNode);
			
			var args = {
				ownerUUID: session.getAuthenticatedContributor().itemId,
				flags: PRIVATE
			};
			responseHandler = new ServiceResponseHandler(
					this,
					"_addFolders",
					"_remoteError");
			ReportsClient.getFolderDescriptors(responseHandler, args);

			args = {
				flags: (SHARED | SYSTEM)
			};
			responseHandler = new ServiceResponseHandler(
					this,
					"_addFolders",
					"_remoteError");
			ReportsClient.getFolderDescriptors(responseHandler, args);

			this._createContextSensitiveHelp();
			
			this._saveButton.disabled = false;
			this._updatePrompt();
			this._updateEnablement();
		},
		
		_updatePrompt: function()
		{
			var prompt = "";
			if (this.queryUUIDs && this.queryUUIDs.length > 0 && this.folderUUIDs && this.folderUUIDs.length > 0)
				prompt = dojo.string.substitute(this.messages.movingReportsAndFolders, [this.queryUUIDs.length, this.folderUUIDs.length]);
			else if (this.queryUUIDs && this.queryUUIDs.length > 0)
				prompt = dojo.string.substitute(this.messages.movingReports, [this.queryUUIDs.length]);
			else
				prompt = dojo.string.substitute(this.messages.movingFolders, [this.folderUUIDs.length]);
			
			this._reportName.innerHTML = prompt;
		},

		_selectFolder: function(folderUUID)
		{
			//Get the itemID from the folder map based on the folder UUID
			//the select that item in the tree
			for (var key in this._folderMap)
			{
				//If the folder in the map is the same as the supplied folder UUID then get the item
				if (this._folderMap[key].folderUUID == folderUUID)
				{
					//Get the item from the tree
					var folderToSelect = this._folderTreeWidget.getItem(this._folderMap[key].treeItemID);
					//Expand the tree down to the folder to select
					var parentItem = folderToSelect.getParent();
					while (parentItem != null) 
					{
						parentItem.setExpanded(true);
						parentItem = parentItem.getParent();
					}
					//select it
					this._folderTreeWidget.setSelection(folderToSelect);
					this._originalParentNode = folderToSelect;  //save this for comparison with the folder to move/copy to					
					break;
				}
			}
		},
		
		_removeFolderFromTree: function(folderUUID)
		{
			for (var key in this._folderMap)
			{
				//If the folder in the map is the same as the supplied folder UUID then get the item
				if (this._folderMap[key].folderUUID == folderUUID) 
				{
					//Get the item from the tree
					var folderItem = this._folderTreeWidget.getItem(this._folderMap[key].treeItemID);
					var parentItem = folderItem.getParent();
						parentItem.removeChild(folderItem, false);
					break;
				}
			}
		},
				
		_createContextSensitiveHelp: function() {
			dojo.empty(this.csHelpNode);
			this._createCSHWidget(MoveReport.CSH_NEW_REPORT_EDITOR);
		},
	
		_createCSHWidget: function(uaKey) {
			var parms = {};
			parms.uaKey = uaKey;
			var pageHelp = new PageHoverHelp(parms);
			this.csHelpNode.appendChild(pageHelp.domNode);
			this._csPageHelpWidgets.push(pageHelp);
		},
		
		_handleSave: function() {
			this.messageBox.hideMessage();
			dojo.removeClass(this._messageBoxContainer, "messageBoxVisible");
			var args = {};
			
	        var currentFolderNode = this._folderTreeWidget.getSelection();
	        var currentFolder = this._folderMap[currentFolderNode];
	        if (!currentFolder || currentFolder == null) {
	        	this._showErrorMessage(this.messages.pleaseSelectFolder);
	        	return;
	        }
			
			//This is the no-op case where the user tries to move the item to it's original folder.
			if (this._originalParentNode == currentFolderNode[0])
				this._saveComplete();
	        
			
			var isAMove = this._moveButton.checked;

			this._movingTypeCount = 0;			
			if (this.queryUUIDs && this.queryUUIDs.length > 0) //Moving queries
			{
				this._movingTypeCount++;				
				args.queryUUIDs = this.queryUUIDs;
				//Set the correct folder to move/copy the report to.
				if (currentFolder.processAreaUUID == null) 
				{
					args.sharedIfNullFolder = currentFolder.shared;
				}
				else 
				{
					args.folderUUID = currentFolder.folderUUID;
				}

				var responseHandler = new ServiceResponseHandler(this, "_saveComplete", "_remoteError");
				if (isAMove) 
					ReportsClient.postMoveQueryDescriptors(responseHandler, args);
				else
					ReportsClient.postCopyQueryDescriptors(responseHandler, args);
			}
			if (this.folderUUIDs && this.folderUUIDs.length > 0)  //Moving folders
			{
				this._movingTypeCount++;				
				args.folderUUIDs = this.folderUUIDs;
				//Set the correct folder to move/copy the report to.
				if (currentFolder.processAreaUUID == null) 
				{
					args.sharedIfNullFolder = currentFolder.shared;
				}
				else 
				{
					args.folderUUID = currentFolder.folderUUID;
				}

				var responseHandler = new ServiceResponseHandler(this, "_saveComplete", "_remoteError");
				if (isAMove) 
					ReportsClient.postMoveFolderDescriptors(responseHandler, args);
				else
					ReportsClient.postCopyFolderDescriptors(responseHandler, args);
			}
		},
		
		_handleCancel: function() 
		{
			if (this.cancelCallback != null)
				this.cancelCallback();
		},
		
		_saveComplete: function(args) 
		{
			this._movingTypeCount--;				
			if (this.okCallback != null && this._movingTypeCount <= 0)
				this.okCallback();
		},
		
		_remoteError: function(error) 
		{
			this._movingTypeCount--;				

			// Check if the content is still displaying
			if (!dojo.isDescendant(this._saveButton, document.body)) {
				// Not still displaying, ignore the error
				return;
			}
			this._showErrorMessage("Error invoking service; message: " + error.message);
		},

		_addFolders: function(folders) {
			this._responseCount++;
			if (!folders) folders = [];
			for (var i = 0; i < folders.length; i++) {
				this._allFolders.push(folders[i]);
			}
			if (this._responseCount == 1) {
				return;
			}
			this._addFoldersToTree();
			
			//If there is a parent folder, select it
			if (this.parentUUID != "")
				this._selectFolder(this.parentUUID);
			
			//If we are moving folders, remove the folders being moved and their children from the tree
			if (this.folderUUIDs != null && this.folderUUIDs.length > 0) 
			{
				for (var i = 0; i < this.folderUUIDs.length; i++) 
					this._removeFolderFromTree(this.folderUUIDs[i]);
			}
		},
		
		_addFoldersToTree: function() {
			// First, find the two root folders
			var sharedRoot = null;
			var privateRoot = null;
			for (var i = 0; i < this._allFolders.length; i++) {
				var folder = this._allFolders[i];
				if (folder.system) {
					if (folder.shared) {
						sharedRoot = folder;
					} else {
						privateRoot = folder;
					}
				}
			}

			if (privateRoot) {
				var rootItem = new jazz.ui.tree.IconItem({label: privateRoot.name, iconClass: "iconFolder"});
				privateRoot.treeItemID = rootItem.getId();  //Save the item ID in the folders array in case we need to make tree calls based on the id
				this._folderMap[rootItem] = privateRoot;
				this._folderTreeWidget.addChild(rootItem);
				this._addChildrenForFolder(rootItem, privateRoot, this._allFolders, true, false);
			}
			if (sharedRoot) {
				var rootItem = new jazz.ui.tree.IconItem({label: sharedRoot.name, iconClass: "iconFolder"});
				sharedRoot.treeItemID = rootItem.getId();  //Save the item ID in the folders array in case we need to make tree calls based on the id
				this._folderMap[rootItem] = sharedRoot;
				this._folderTreeWidget.addChild(rootItem);
				this._addChildrenForFolder(rootItem, sharedRoot, this._allFolders, true, true);
			}
		},
		
		/**
		 * From the list of folders, find any that are children of parentFolder, and
		 * add them to the tree. Then recurse for each of the children.
		 */
		_addChildrenForFolder: function(parentFolderNode, parentFolder, folders, parentIsRoot, parentIsShared) {
			var foldersToAdd = [];
			for (var i = 0; i < folders.length; i++) {
				// Special case where the parent is a root system folder
				if (parentIsRoot) {
					if (!folders[i].parentUUID && (folders[i].shared == parentIsShared) && (folders[i].name != parentFolder.name)) {
						foldersToAdd.push(folders[i]);
					}
				} else {
					if (folders[i].parentUUID == parentFolder.folderUUID) {
						foldersToAdd.push(folders[i]);
					}
				}
			}
			foldersToAdd = foldersToAdd.sort(function(a, b) {return a.name > b.name;});
			for (var i = 0; i < foldersToAdd.length; i++) {
				var childNode = new jazz.ui.tree.IconItem({label: foldersToAdd[i].name, iconClass: "iconFolder"});
				foldersToAdd[i].treeItemID = childNode.getId();  //Save the item ID in the folders array in case we need to make tree calls based on the id
				this._folderMap[childNode] = foldersToAdd[i];
				parentFolderNode.addChild(childNode);
				this._addChildrenForFolder(childNode, foldersToAdd[i], folders, false, parentIsShared);
			}
		},

		_createFolderTree: function() {
			this._folderTreeWidget = new jazz.ui.Tree();
			this._folderTree.appendChild(this._folderTreeWidget.domNode);
			this._folderTreeWidget.onSelectionChanged = dojo.hitch(this,this._handleFolderSelectionChange);			
		},
		
		_handleFolderSelectionChange: function(selection)
		{
			this._updateEnablement();			
		},

		_updateEnablement: function() {
			this._saveButton.disabled = this._folderTreeWidget.getSelection().length == 0;
		},
		
		_showErrorMessage: function(message) {
			var status = {
				type : jazz.ui.MessageBox.STATUS_TYPES.ERROR,
				summary : message,
				closable : false
			};
			dojo.addClass(this._messageBoxContainer, "messageBoxVisible");
			this.messageBox.showMessage(status);
		}
	}
);

MoveReport.CSH_NEW_REPORT_EDITOR = "com.ibm.jazz.reports.web.cshelp.new_report_editor";

})();