/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007, 2012. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("com.ibm.team.dashboard.web.ui.internal.PersonalDashboardComponentMenu");

dojo.require("jazz.app._ComponentMenu");
dojo.require("jazz.app.context");
dojo.require("jazz.app.viewHistory");
dojo.require("jazz.core.loader");
dojo.require("dojo.i18n");
dojo.require("dojo.string");

dojo.require("com.ibm.team.dashboard.web.ui.DashboardConstants");
dojo.require("com.ibm.team.dashboard.web.ui.internal.Constants");
dojo.require("com.ibm.team.dashboard.web.util.internal.Util");
dojo.require("com.ibm.team.dashboard.web.client.internal.PersonalDashboardClient");
dojo.require("com.ibm.team.dashboard.web.util.DashboardUtil");

dojo.require("jazz.ui.internal.MenuItemGroup");
dojo.require("jazz.util.html");

dojo.requireLocalization("com.ibm.team.dashboard.web.ui.internal", "DashboardMenuMessages");

var DashboardConstants = com.ibm.team.dashboard.web.ui.DashboardConstants;
var Constants = com.ibm.team.dashboard.web.ui.internal.Constants;
var Util = com.ibm.team.dashboard.web.util.internal.Util;
var PersonalDashboardClient = com.ibm.team.dashboard.web.client.internal.PersonalDashboardClient;
var DashboardUtil = com.ibm.team.dashboard.web.util.DashboardUtil;
var HTML = jazz.util.html;
var messages = dojo.i18n.getLocalization("com.ibm.team.dashboard.web.ui.internal", "DashboardMenuMessages");
var MenuItemGroup = jazz.ui.internal.MenuItemGroup;
var WrapHints = jazz.ui.MegaMenu.WrapHints;
var viewHistory = jazz.app.viewHistory;
var CREATE_DASHBOARD_WIZARD_CLASS = "com.ibm.team.dashboard.web.ui.internal.wizard.CreateDashboardWizard";
var RESOURCE_PATH = "/dashboards";
var PERSONAL_DASHBOARD_TYPE = "com.ibm.team.personal.dashboard";
/*
 * The Web UI page (vertical) containing the dashboard and side navigation bar.
 */
dojo.declare("com.ibm.team.dashboard.web.ui.internal.PersonalDashboardComponentMenu", jazz.app._ComponentMenu, {
	
	postCreate: function() {
		dojo.addClass(this.domNode, "com-ibm-team-dashboard-ComponentMenu");
		this._myDashboards = new MenuItemGroup({
			menu: this,
			groupTitle: messages.currentPersonal
		});
		this._getLeftNavInfo();
		this.initRecent();
		dojo.publish("/jazz/dashboard/initMenu", [this]);
		this.subscribe("/jazz/dashboard/rename", this._renameDashboard); // pass url, new title
		this.subscribe("/jazz/dashboard/delete", this._deleteDashboard); // pass url
		this.subscribe("/jazz/dashboard/setDefault", this._setDefaultDashboard); // pass url
		this.subscribe("/jazz/dashboard/share", this._shareDashboard); // pass url, shared
		this.subscribe("/jazz/dashboard/view", this._addRecent); // pass bundle
		this.subscribe("/jazz/dashboard/create", this._doAddPersonalDashboard); // pass url, title
		this.subscribe("/jazz/dashboard/dirty", this.onDirtyStateChange); // pass bundle and dirty state
	},
	initRecent: function(recent) {
		var items = [];
		/*
		outer: for(var i = recent.length - 1; i >= 0; i--) {
			var item = this._createMenuItem(recent[i]);
			for (var j = 0; j < items.length; j++) {
				if (items[j].href == item.href) continue outer;
			}
			items.push(item);
		}
		*/
		var query = {
			application: "%MY_APPLICATION%",			
			type: PERSONAL_DASHBOARD_TYPE
		};
		var def = viewHistory.get(query);
		var menuDef = new dojo.Deferred();
		this._createDeferredRecentSection(menuDef);		
		
		def.addCallback(dojo.hitch(this, function (result) {
			var history = result.items;
			outer: for (var i=0; i<history.length; i++) {
				var entry = history[i];
				var item = {
					label: entry.label,
					icon: entry.iconUri,
					href: entry.data?entry.data:entry.url,
					uri: entry.uri
				};
				if (this._unsavedHref && item.href==this._unsavedHref) {
					// skip this one
					delete this._unsavedHref;
					continue;
				}				
				for (var j = 0; j < items.length; j++) {
					if (items[j].href == item.href) continue outer;
				}				
				items.push(item);
			}
			if (items.length == 0) {
				this._recentDashboards.remove();
				this._recentDashboards.destroy();
				delete this._recentDashboards;
			} else {
				menuDef.callback(items); 
				this._recentDashboards.set("groupTitle", "<b>"+messages.recentDashboards+"</b> (" + (items ? items.length : 0) + ")");
				if (items.length > 0)   {
					this.updateSelected();	
				}
			}
		}));

	},

	_createRecentSection: function(items) {
		if (!this._recentDashboards) this._recentDashboards = new MenuItemGroup({
			menu: this,
			wrapHint: WrapHints.FORCE_WRAP,
			groupTitle: "<b>"+messages.recentDashboards+"</b> (" + (items ? items.length : 0) + ")",
			items: items || []
		});
	},

	_createDeferredRecentSection: function(def) {
		if (!this._recentDashboards) this._recentDashboards = new MenuItemGroup({
			menu: this,
			wrapHint: WrapHints.FORCE_WRAP,
			groupTitle: "<b>"+messages.recentDashboards+"</b>",
			itemProvider: function() { return def; }
		});
	},	
	_addRecent: function(/* object */ bundle) {
		var item = this._createMenuItem(bundle);
		return this._addRecentItem(item);
	},
	_addRecentItem: function(/* object */ item) {
		if (!this._recentDashboards) {
			this._createRecentSection();
		}
		var items = this._recentDashboards.get("items");
		if (items) {
			for (var i = 0; i < items.length; i++) {
				if (items[i].href == item.href) {
					this._recentDashboards.remove(items[i]).destroy();
					break;
				}
			}
			while (items.length > 4) {
				this._recentDashboards.remove(items[0]).destroy();
			}
		}
		this._recentDashboards.addItemByArgs(item,0);
		this._updateRecentCount();
		this.updateSelected();
		return true;
	},
	_createMenuItem: function(/* object */ bundle, /* boolean */ dirty) {
		var dashboard = bundle.dashboard.binding;
		var name = dashboard.getTitle(),
			href = Util.getDashboardHref(bundle),
			iconClass;
		switch(bundle.scope) {
			case DashboardConstants.SCOPE_CONTRIBUTOR:
				if (bundle.isDefaultDashboard) {
					iconClass = "default-dash";
				}
				else {
					var securityContext = bundle.dashboard.securityContext;
					if (securityContext) {
						if (securityContext.indexOf(Constants.CONTEXT_PUBLIC) >= 0)
							iconClass = "shared-dash";
						else
							iconClass = "my-dash";
					}
				}
				break;
			case DashboardConstants.SCOPE_TEAM_AREA:
				iconClass = "team-dash";
				break;
			case DashboardConstants.SCOPE_PROJECT_AREA:
				iconClass = "project-dashboard";
				break;
		}
		var label = HTML.escape(name);
		if (dirty) {
			label = "<span class='j-dirty'>* " + label + "</span>";
		}
		return {
			label: label,
			iconClass: iconClass,
			href: href,
			url: bundle.dashboardUrl
		};
	},
	_updateRecentCount: function() {
		if (!this._recentDashboards) return 0;
		var length = this._recentDashboards.get("items").length;
		this._recentDashboards.set("groupTitle", "<b>"+messages.recentDashboards+"</b> ("+length+")");
		return length;
	},
	onDirtyStateChange: function(/* object */ bundle, /* boolean */ isDirty) {
		if (isDirty) {
			if (this._recentDashboards) {
				var items = this._recentDashboards.get("items");
				var href = Util.getDashboardHref(bundle);
				if (items) {
					for (var i = 0; i < items.length; i++) {
						if (items[i].href == href) {
							this._recentDashboards.remove(items[i]);
							break;
						}
					}
					if (!this._updateRecentCount()) {
						this._recentDashboards.remove();
						this._recentDashboards.destroy();
						delete this._recentDashboards;
					}
				}
				else {
					// load in progress - store the href to be used by the Deferred
					// when building the recent menu group items
					this._unsavedHref=href;
				}
			}
			if (!this._unsavedDashboards) {
				this._unsavedDashboards = new MenuItemGroup({
					menu: this,
					wrapHint: this._recentDashboards ? WrapHints.NO_WRAP : WrapHints.FORCE_WRAP,
					groupTitle: "<b>"+messages.unsavedDashboards+"</b> (1)",
					items: [this._createMenuItem(bundle, true)]
				});
			}
		} else if (this._unsavedDashboards) {
			items = this._unsavedDashboards.get("items");
			if (items && items.length) {
				this._unsavedDashboards.remove(items[0]);
				this._addRecentItem(this._createMenuItem(bundle));
			}
			this._unsavedDashboards.remove();
			this._unsavedDashboards.destroy();
			delete this._unsavedDashboards;
			delete this._unsavedHref;
		}
	},
	getSelected: function() {
		var items = this.getItems(),
			sel = [],
			hash = dojo.hash(),
			href = (hash && hash.length > 0) ? hash : location.pathname,
			length = href.length;
		for (var i = 0; i < items.length; i++) {
			var item = items[i];
			if (item && item.href) {
				if (item.href.indexOf(href, item.href.length - href.length) !== -1) {
					sel.push(item);
				}
			}
		}
		return sel;
	},
	_getLeftNavInfo: function() {
		var deferred = PersonalDashboardClient.get();
		deferred.addCallback(dojo.hitch(this, function(/* LeftNavDTO */ navDTO) {
			this._navInfo = navDTO;
			this._populate();
			this.updateSelected();
		}));
		deferred.addErrback(function(errorObj) {
			console.error(Constants.LeftNav.LEFT_NAV_INFO_ERROR + errorObj.message);
		});
	},
	_populate: function() {
		var navDTO = this._navInfo,
			items = [];
		if (navDTO.personalDashboards) {
			for (var i = 0; i < navDTO.personalDashboards.length; i++) {
				var descriptor = navDTO.personalDashboards[i];
				var url = descriptor.url;
				var id = url.substring(url.lastIndexOf("/") + 1);
				var iconClass = descriptor["default"] ? "default-dash" : (descriptor.shared ? "shared-dash" : "my-dash");
				items.push({
					label: HTML.escape(descriptor.title),
					iconClass: iconClass,
					href: DashboardUtil.getDashboardHref(DashboardConstants.SCOPE_CONTRIBUTOR, id),
					url: url,
					usage: descriptor["default"] ? "default" : ""
				});
			}
		}
		items.sort(itemSorter);
		items.push({
			label: messages.allPersonalDashboards,
			iconClass: "all-dash",
			href: Util.getAllPersonalDashboardsHref(),
			wrapHint: WrapHints.NO_WRAP,
			usage: "all"
		});
		this._myDashboards.set("items",items);

		if (navDTO.personalPermissions.create) {
			var args = {
				menu: this,
				groupTitle: messages.createDashboard,
				items: [{
					label: messages.createPersonalDashboard,
					iconClass: "create-dash",
					onClick: dojo.hitch(this, this._createDashboard)
				}]
			};
			if (this._recentDashboards) {
				args.insertIndex = this._recentDashboards.getIndexInParent();
			}
			this._createDashboards = new MenuItemGroup(args);
		}
	},

	_createDashboard: function() {
		jazz.core.loader.load_async(CREATE_DASHBOARD_WIZARD_CLASS, function() {
			new dojo.getObject(CREATE_DASHBOARD_WIZARD_CLASS)();
		});
	},
	_doAddPersonalDashboard: function(/* string */ url, /* string */ title, /* boolean */ shared) {
		var id = url.substring(url.lastIndexOf("/") + 1);
		var label = HTML.escape(title);
		this._myDashboards.addItemByArgs({
			label: label,
			iconClass: shared ? "shared-dash" : "my-dash",
			href: DashboardUtil.getDashboardHref(DashboardConstants.SCOPE_CONTRIBUTOR, id),
			url: url
		}, this._getInsertIndex(label));
	},
	_renameDashboard: function(/* string */ url, /* string */ newTitle) {
		var items = this.findMenuItemsByUrl(url);
		if (items.length > 0) {
			for (var i = 0; i < items.length; i++) {
				items[i].set("label", HTML.escape(newTitle));
			}
		}
		this._myDashboards.sort(itemSorter);
	},
	_deleteDashboard: function(/* string */ url) {
		// if the deleted dashboard was selected, select the default dashboard
		var items = this.findMenuItemsByUrl(url);
		if (items.length > 0) {
			for (var i = 0; i < items.length; i++) {
				if (items[i].__menuItemGroup) {
					items[i].__menuItemGroup.remove(items[i]).destroy();
				} else {
					this.remove(items[i]).destroy();
				}
			}
		}
		var rd = this._recentDashboards;
		var recentItems = rd.get("items");
		if (!dojo.isArray(recentItems) || recentItems.length == 0) {
			rd.remove();
			rd.destroy();
			this._recentDashboards = null;
		}
		this._updateRecentCount();
		this.updateSelected();
	},
	_setDefaultDashboard: function(/* string */ url) {
		// move new default to top
		item = this.findMenuItemsByUrl(url)[0];
		this._myDashboards.remove(item);
		this._myDashboards.addItemByArgs(item, "start");
		
		// update icons
		var items = this.getItems();
		for (var i=0;i<items.length;i++) {
			var item = items[i];
			if (item.url == url) {
				// set new default icon
				item.set("iconClass", "default-dash");
			}
			else if (item.icon == "default-dash") {
				// unset previous default
				item.set("iconClass", "my-dash");
			}
		}
	},
	_shareDashboard: function(/* string */ url, /* boolean */ shared) {
		var items = this.findMenuItemsByUrl(url);
		if (items.length > 0) {
			var icon = shared ? "shared-dash" : "my-dash";
			for (var i = 0; i < items.length; i++) {
				items[i].set("iconClass", icon);
			}
		}
	},
	_getInsertIndex: function(label) {
		var items = this._myDashboards.get("items");
		if (!items || !items.length) return 0;
		if (label) {
			for (var i = 0; i < items.length - 1; i++) {
				var itemLabel = items[i].label;
				if (!itemLabel || itemLabel.toLowerCase() < label.toLowerCase()) 
					continue;
				return i;
			}
		}
		return items.length - 1;
	},
	/* domNode */ findMenuItemsByUrl: function(/* string */ url){
		var items = this._menu.getChildren();
		if (this._recentDashboards) {
			var recentItems = this._recentDashboards.get("items");
			if (dojo.isArray(recentItems)) {
				items = items.concat(recentItems);
			}
		}
		var returnArr = [];
		for (var i = 0; i < items.length; i++) {
			if (items[i].url == url) {
				returnArr.push(items[i]);
			}
		}
		return returnArr;
	},
	uninitialize: function() {
		if (this._myDashboards) this._myDashboards.destroy();
		if (this._createDashboards) this._createDashboards.destroy();
		if (this._unsavedDashboards) this._unsavedDashboards.destroy();
		if (this._recentDashboards) this._recentDashboards.destroy();
	}
});

function itemSorter(a, b){
	if (!a || !a.label || b.usage == "default" || a.usage == "all") return 1;
	else if (!b || !b.label || a.usage == "default" || b.usage == "all") return -1;
	else if (a.label.toLowerCase() == b.label.toLowerCase()) return 0;
	return (a.label.toLowerCase() < b.label.toLowerCase()) ? -1 : 1;
}

})();