/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010, 2013. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.reports.web.ui.internal.RecentlyViewed");

dojo.require("com.ibm.team.reports.web.ui.ReportsConstants");
dojo.require("com.ibm.team.reports.web.util.ReportsUtil");
dojo.require("jazz.app.viewHistory");

dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
(function() { 
var ReportsConstants = com.ibm.team.reports.web.ui.ReportsConstants;
var ReportsUtil = com.ibm.team.reports.web.util.ReportsUtil;
var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;
var MAX_RECENT = 10;
var initialized = false;
	
com.ibm.team.reports.web.ui.internal.RecentlyViewed = {
	_artifacts: [],
	pageVisited: function(widget) {
		// if there is no getArtifactInfo function, the page should not appear in recently viewed
		if (!widget || !widget.getArtifactInfo) {
			return;
		}
		// get the itemId
		var serviceObj = {
			failure: function(error) {
				console.error(error);
			},
			success: dojo.hitch(this,this._processPageVisit)
		};
		var srh = new ServiceResponseHandler(serviceObj, "success", "failure");
		widget.getArtifactInfo(srh);
	},
	
	init: function() {
		if (initialized) return false;
		var deferred = jazz.app.viewHistory.get({
			application: "%MY_APPLICATION%",
			project: ReportsUtil.getProjectArea().name,
			type: [
				ReportsConstants.VIEWER_TYPE_ID,
				ReportsConstants.TEMPLATE_TYPE_ID
			],
			size : MAX_RECENT
		});
		deferred.addCallback(dojo.hitch(this,this._processGetResponse));
	},
	
	getArtifacts: function() {
		return this._artifacts;	
	},
	getArtifactsCount: function() {
		return this._artifacts.length;	
	},
	
	_processGetResponse: function(response) {
		var indexMap = {};
		var idMap = {};
		for (var i = 0; i < this._artifacts.length; i++) {
			indexMap[this._artifacts[i].url] = i;
			idMap[this._artifacts[i].itemId] = i;
		}
		if (!response.count || !response.items || !response.items.length) return;
		var changed = false;
		for (i = 0; i < response.items.length && this._artifacts.length < MAX_RECENT; i++) {
			var item = response.items[i];
			if (typeof indexMap[item.uri] === "number" || indexMap[item.uri]) continue;
			var delimiter; //either queryUUID or reportUUID
			if(item.type == ReportsConstants.VIEWER_TYPE_ID) {
				delimiter = "queryUUID";
			} else { // ReportsConstants.TEMPLATE_TYPE_ID
				delimiter = "eportUUID";
			}
			delimiter += "=";
			var itemId = item.uri.substring(item.uri.lastIndexOf(delimiter)+1);
			// check if string contains a "shared=" variable
			var length = itemId.search("&");
			var shared = "";
			if(length > 0) {
				shared = itemId.slice(length+"shared=".length + 1);
				itemId = itemId.slice(9,length);
			} else {
				itemId = itemId.slice(9);
			}
			if(idMap[itemId] != null) continue;
			this._artifacts.push({
				type: item.type,
				itemId: itemId,
				projectAreaUUID: item.projectAreaUUID,
				title: item.label,
				iconUrl: item.iconUri,
				href: this._getItemHref(item.label, item.type, itemId, shared),
				url: item.uri
			});
			changed = true;
		}
		if (changed) {
			dojo.publish(ReportsConstants.EVENT_REPORTS_VIEW_CHANGED);
		}
	},
	
	_getItemHref: function(title, type, itemId, shared) {
		switch (type) {
		case ReportsConstants.VIEWER_TYPE_ID:
			return ReportsUtil.getReportHref(title,itemId,shared);
		case ReportsConstants.TEMPLATE_TYPE_ID:
			return ReportsUtil.getReportTemplateHref(itemId);
		default:
			return ReportsUtil.getReportHref(title,itemId,shared);
		}
	},
	
	_processPageVisit: function(info) {
		this._artifacts.splice(0,0,info);
		
		for (var i = 1; i < this._artifacts.length; i++) {
			var cur = this._artifacts[i];
			if (cur.itemId == info.itemId) {
				this._artifacts.splice(i,1);
				break;
			}
		}
		
		if (this._artifacts.length > MAX_RECENT) {
			this._artifacts.length = MAX_RECENT;
		}
		
		jazz.app.viewHistory.put({
			application: "%MY_APPLICATION%",
			project: ReportsUtil.getProjectArea().name,
			type: info.type
		},{
			uri: info.url,
			label: info.title,
			iconUri: info.iconUrl.toString()
		});
		
		dojo.publish(ReportsConstants.EVENT_REPORTS_VIEW_CHANGED);
	},
};

})();