/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2009, 2012. All Rights Reserved. 
 * 
 * Note to U.S. Government Users Restricted Rights:  Use, 
 * duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.reports.web.ui.internal.ReportCustomTemplateEditor");

dojo.require("com.ibm.team.process.web.client.internal.ProcessClient");
dojo.require("com.ibm.team.process.web.ui.internal.ProcessAreaSelectionDialog");

dojo.require("com.ibm.team.reports.cognos.web.ui.internal.NewTemplateDialog");

dojo.require("com.ibm.team.reports.cognos.web.client.internal.CognosReportsClient");
dojo.require("com.ibm.team.reports.web.client.internal.ReportsClient");
dojo.require("com.ibm.team.reports.web.ui.internal.ImagePool");
dojo.require("com.ibm.team.reports.web.ui.internal.ResourceDialog");
dojo.require("com.ibm.team.repository.web.client.internal.ServerStatusClient");
dojo.require("com.ibm.team.reports.web.ui.internal.ValidatingTextBox");
dojo.require("com.ibm.team.reports.web.ui.internal.MetaNav");
dojo.require("com.ibm.team.reports.web.ui.ReportsConstants");

dojo.require("com.ibm.team.repository.web.client.session.Session");
dojo.require("com.ibm.team.repository.web.client.internal.HttpHeaders");
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");

dojo.require("jazz.app.context");
dojo.require("jazz.client.xhr");
dojo.require("jazz.ui.StyledTable");
dojo.require("jazz.util.html");
dojo.require("jazz.ui.MessageBox");
dojo.require("jazz.ui.Tree");
dojo.require("jazz.ui.tree.IconItem");
dojo.require("net.jazz.ajax.ui.PlatformUI");

dojo.require("dojo.i18n");
dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dijit.InlineEditBox");
dojo.require("dojo.io.iframe");

dojo.requireLocalization("com.ibm.team.reports.web.ui.internal", "ReportMessages");

var FILE_UPLOAD_RESPONSE = "File Upload Response:";
var FILE_UPLOAD_ERROR = "File Upload Error:";
var TeamServerClient = com.ibm.team.repository.web.transport.TeamServerClient;
var RESOURCE_SERVICE_URI = TeamServerClient._getServicePrefix() + "com.ibm.team.reports.service.internal.IReportResourceService";
var MAX_MESSAGE_LENGTH = 500;
var ReportsConstants = com.ibm.team.reports.web.ui.ReportsConstants;

(function() {

var ProcessClient = com.ibm.team.process.web.client.internal.ProcessClient;
var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;
var ReportsClient = com.ibm.team.reports.web.client.internal.ReportsClient;
var ImagePool = com.ibm.team.reports.web.ui.internal.ImagePool;
var NewTemplateDialog = com.ibm.team.reports.cognos.web.ui.internal.NewTemplateDialog;
var CognosReportsClient = com.ibm.team.reports.cognos.web.client.internal.CognosReportsClient;
var ServerStatusClient = com.ibm.team.repository.web.client.internal.ServerStatusClient;
var PlatformUI = net.jazz.ajax.ui.PlatformUI;

var _currentBindRequest;
var _currentUploadFrameNumber = 100;
var _showSavedMessage = false;

/**
 * Widget to create a new custom report template
 */
dojo.declare("com.ibm.team.reports.web.ui.internal.ReportCustomTemplateEditor", [dijit._Widget, dijit._Templated], {
		templatePath: dojo.moduleUrl("com.ibm.team.reports.web", "ui/internal/templates/ReportCustomTemplateEditor.html"),
		
		constructor: function() {
			this.messages = dojo.i18n.getLocalization("com.ibm.team.reports.web.ui.internal", "ReportMessages");
			this.dirty = false;
			this.url = net.jazz.ajax._contextRoot + "/service/com.ibm.team.reports." +
				"service.cognos.internal.ICognosReportService/";
			this._templateMap = {};
		},
				
		postCreate: function() {
			this.metanav = new com.ibm.team.reports.web.ui.internal.MetaNav({ReportCustomTemplateEditor: this}, this.metanav);
			this._reportUUID = this.reportUUID;
			this._populateHeader();
			
			// The report template. We may not have one initially.
			this._reportTemplate = null;
			
			if (this._reportUUID) {
				var responseHandler = new ServiceResponseHandler(
						this,
						"_setReportTemplate",
						"_remoteError");
				ReportsClient.getReportDescriptor(responseHandler, {reportUUID: this._reportUUID});
			} else {
				this._setEmptyReportTemplate();
			}
			this._idInput = new com.ibm.team.reports.web.ui.internal.ValidatingTextBox();
			this._idInput.setErrorMessage("");
			this._idInput.validate = function(value) {
				return value.length > 0;
			};
			dojo.addClass(this._idInput.inputNode, "idInput");
			this._idInputContainer.appendChild(this._idInput.domNode);
			var self = this;
			this.connect(this._processAreaAnchor, "onclick", this._handleProcessAreaClick);
			if(this.controller){
				this.controller.startOperation(this.messages.loadingCustomReportTemplates);
			}
			this.headers = com.ibm.team.repository.web.client.internal.HttpHeaders._getHeaders();
			//fetch jts url
			var srh = new ServiceResponseHandler(this, "_handleFetchJTSUrl", "_handleFetchJTSUrlError");
			ServerStatusClient.getJazzTeamServerURI(srh);
			
			this.messageBox = new jazz.ui.MessageBox();
			this._messageBoxContainer.appendChild(this.messageBox.domNode);
			if (_showSavedMessage == true) {
				this._showStatusMessage(this.messages.templateSavedSuccessfully);
				_showSavedMessage = false;
			}
			
			if (!this.navigateAwayHandle) {
				this.navigateAwayHandle = PlatformUI.getWorkbench().registerBodyClickHandler(dojo.hitch(this, this._handleNavigateAway), /*priority*/ 10000);
			}
			if (!this.onBeforeUnloadHandle) {
				this.onBeforeUnloadHandle = PlatformUI.getWorkbench().registerOnBeforeUnloadHandler(dojo.hitch(this, this._handleOnBeforeUnload), /*priority*/ 10000);
			}
			if(this.displayBreadcrumbs) this.metanav.populateResourcesBreadcrumbs();
		},

		_populateHeader : function(){
			this.metanav.setTitle(this.messages.reportCustomTemplate);
			this.typeIcon = document.createElement("img");
			this.typeIcon.src = ImagePool.REPORTTEMPLATE_OBJ;
			this.typeIcon.className = "typeIcon";
			this.typeIcon.style.paddingTop = "3px";
			this.metanav.insertTypeIcon(this.typeIcon);
			
			this._cancelButton = document.createElement("button");
			this._cancelButton.innerHTML = this.messages.cancel;
			this.metanav.commands.appendChild(this._cancelButton);
			
			this._saveButton = document.createElement("button");
			this._saveButton.disabled = "true";
			this._saveButton.innerHTML = this.messages.save;
			this.metanav.commands.appendChild(this._saveButton);
			
			dojo.connect(this._saveButton, "onclick", this, this._handleSave);
			dojo.connect(this._cancelButton, "onclick", this, this._handleCancel);
		},
		
		_handleFetchJTSUrl : function(response){
			if(response == null){
				response = net.jazz.ajax._contextRoot + "/";
			}
			this.url = response + "service/com.ibm.team.reports." +
			  "service.cognos.internal.ICognosReportService/";
			this._createTemplateTree();
		},
		
		_handleFetchJTSUrlError : function(){
			this.url = net.jazz.ajax._contextRoot + "/service/com.ibm.team.reports." +
			  "service.cognos.internal.ICognosReportService/";
			this._createTemplateTree();
		},
		
		_createTemplateTree: function() {
			var self = this;
			this._templateTreeWidget = new jazz.ui.Tree();
			var tree = this._templateTreeWidget;
			tree.onEnter = function(){
				if(tree.getSelection() && tree.getSelection().length > 0){
					var node = tree.getSelection()[0];
					node.toggleExpanded();
				}
			};
			tree.onSelectionChanged = function(items){
				if(items && items.length > 0){
					var item = items[0];
					if(self._templateMap){
						var node = self._templateMap[item.id];
						if(node.type == "report"){
							self.reportPath = node.path;
							self.reportFileName = node.name;
							self.reportId = node.reportId;
						}else{
							self.reportPath = null;
							self.reportFileName = null;
							self.reportId = null;
						}
					}
				}
				self._setDirty(true);
			};
			this._templateTreeContainer.appendChild(this._templateTreeWidget.domNode);
			
			jazz.client.xhrGet
			({
			  url: this.url,
			  headers: this.headers,
			  load: function(response, ioArgs){
				for(var i = 0; i < response.items.length; i++){
					var item = response.items[i];
					self._templateMap[item.id] = item;
				}
				var tm = self._templateMap;
				var _addSelfAndChildren = function(item, parent){
					var iconClass = item.type == "report" ? "iconTemplate" : "iconFolder";
					var node = new jazz.ui.tree.IconItem({label: item.name, iconClass: iconClass, id : item.id});
					parent.addChild(node);
					if(item.children) {
						var children = new Array(item.children.length);
						for(var i = 0; i < item.children.length; i++){
							children[i] = tm[item.children[i]._reference];
						}
						children.sort(function(a, b) {
							if (a.name < b.name) {
								return -1;
							} else if (b.name < a.name) {
								return 1;
							}
							return 0;
						});
						
						for(var i = 0; i < children.length; i++){
							_addSelfAndChildren(children[i], node);
						}
					}
				}
				if(response.items[0] && response.items[0].children){
					_addSelfAndChildren(self._templateMap[response.items[0].id], self._templateTreeWidget);
				}else{
					this.page.setHeaderMessage({
						message: this.page.messages.noCognosReportsAvailable,
						showOnlyMessage: true,
						severity: 6
					});
				}
				if(self.controller){
					self.controller.finishOperation();
				}
			},error: this._handleError,
			  handleAs: "json",
			  page: self
			});
			
		},
		
		_handleError: function(data){
			if(this.page.controller){
				this.page.controller.finishOperation();
			}
			this.page.setHeaderMessage({
				message: this.page.messages.customReportServerInvalid,
				showOnlyMessage: true,
				severity: 8
			});
		},
		
		/**
		 * Callback function to populate the page with the given report template.
		 */
		_setReportTemplate: function(template) {
			this._reportTemplate = template;
			
			// Set the widgets up
			this._nameInput.value = template.name;
			if (template.description) {
				this._descriptionText.value = template.description;
			}
			if (template.reportId) {
				this._idInput.inputNode.value = template.reportId;
			}
			if (template.processAreaUUID) {
				// Make a request for the process area name, which we'll fill in later
				var responseHandler = new ServiceResponseHandler(
					this,
					"_setProcessArea",
					"_remoteError");
				// Try team area first; if that fails look for project area
				var args = { uuid: template.processAreaUUID };
				ReportsClient.getProcessAreaByUUID(responseHandler, args);
			}
			this._sharedCheckbox.checked = template.shared;
		},
		
		/**
		 * Populate the page with no initial report template.
		 */
		_setEmptyReportTemplate: function() {
			this._reportTemplate = new Object();
			// Assume we want the project area for sharing
			var currentProjectArea = jazz.app.context.get();
			this._reportTemplate.processAreaUUID = currentProjectArea.itemId;
			if (this._reportTemplate.processAreaUUID) {
				// Make a request for the process area name, which we'll fill in later
				var responseHandler = new ServiceResponseHandler(
					this,
					"_setProcessArea",
					"_remoteError");
				// Try team area first; if that fails look for project area
				var args = { uuid: this._reportTemplate.processAreaUUID };
				ReportsClient.getProcessAreaByUUID(responseHandler, args);
			}
		},
					
		_setProcessArea: function(result) {
			this._processAreaAnchor.innerHTML = result.name;
		},
		
		_handleCancel: function() {
			if (this.dirty) {
				if (confirm(this.messages.navigateAway + "\n\n" + this.messages.unsavedChanges + "\n\n" + this.messages.okOrCancel)) {
					this.destroy();
					document.location.href = "#action=com.ibm.team.reports.viewAllReportTemplates";
					return false;
				}
				else {
					return true;
				}
			}
			else {
				this.destroy();
				document.location.href = "#action=com.ibm.team.reports.viewAllReportTemplates";
				return false;
			}
		},
		
		_handleSave: function() {
			this._clearStatusMessage();
			var currentProjectArea = jazz.app.context.get();
	        var args = {
		        name: this._getName(),
		        reportPath: this.reportPath,
		        projectArea: currentProjectArea.itemId,
				fileName: this.reportFileName
	        }
	        if (this._reportUUID) {
	        	args.reportUUID = this._reportUUID;
	        }
	        var description = this._getDescription();
	        if (description && description.length > 0) {
	        	args.description = description;
	        }
	        var id = this._getIdentifier();
	        if (id && id.length > 0) {
	        	args.id = id;
	        }else{
	        	this._idInput._checkValidity();
				this.setHeaderMessage({
					message: this.messages.missingRequiredFields,
					showOnlyMessage: true,
					severity: 8
				});
	        	return;
	        }
	        args.isShared = this._getShared();
	        args.processId = this._reportTemplate.processAreaUUID;
	        
	        var responseHandler = new ServiceResponseHandler(
					this,
					"_saveComplete",
					"_remoteError");
			CognosReportsClient.postUploadCognosReportTemplate(responseHandler, args);
		},
		
		destroy: function() {
			if (this.navigateAwayHandle) {
				PlatformUI.getWorkbench().unregisterBodyClickHandler(this.navigateAwayHandle);
				this.navigateAwayHandle = null;
			}
			if (this.onBeforeUnloadHandle) {
				PlatformUI.getWorkbench().unregisterOnBeforeUnloadHandler(this.onBeforeUnloadHandle);
				this.onBeforeUnloadHandle = null;
			}	
		},
		
		_handleNavigateAway: function(url) {
			var currentLocation = document.location.href;
			var badLink = currentLocation.substring(0, currentLocation.indexOf("#")+1);
			if (url == badLink) {
				return true;
			}
			
			if (this.dirty) {
				if (confirm(this.messages.navigateAway + "\n\n" + this.messages.unsavedChanges + "\n\n" + this.messages.okOrCancel)) {
					this.destroy();
					return false;
				}
				else {
					return true;
				}
			}
			else {
				this.destroy();
				return false;
			}
		},
		
		_handleOnBeforeUnload: function() {
			return (this.dirty) ? this.messages.unsavedChanges + "\n\n" : false;
		},
		
		_getName: function() {
			return this._nameInput.value;
		},
		
		_getDescription: function() {
			return this._descriptionText.value;
		},
		
		_getIdentifier: function() {
			return this._idInput.inputNode.value;
		},
		
		_getShared: function() {
			return this._sharedCheckbox.checked;
		},
		
		_saveComplete: function(args) {
			this._showStatusMessage(this.messages.templateSavedSuccessfully);
			var newTemplate = true;
			if (this._reportUUID) {
				newTemplate = false;
			}
			this._reportUUID = args;
			if(args.toLowerCase() != "error"){
				if (newTemplate == true) {
					_showSavedMessage = true;
				}
				this.destroy();
				document.location.href = "#action=com.ibm.team.reports.editCustomTemplate&reportUUID=" + args;
			}
			var responseHandler = new ServiceResponseHandler(
					this,
					"_setReportTemplate",
					"_remoteError");
			ReportsClient.getReportDescriptor(responseHandler, {reportUUID: this._reportUUID});
			this._setDirty(false);
		},
		
		_remoteError: function(error) {
			// Check if the content is still displaying
			if (!dojo.isDescendant(this._saveButton, document.body)) {
				// Not still displaying, ignore the error
				return;
			}
			this.setHeaderMessage({
				message: error.message,
				showOnlyMessage: true,
				severity: 8
			});
		},
		
		_handleNameChange: function() {
			this._setDirty(true);
		},
		
		_handleIdChange: function() {
			this._setDirty(true);
		},
		
		_handleCheckboxChange: function() {
			this._setDirty(true);
		},
		
		_handleProcessAreaClick: function(e) {
			e.preventDefault();

			var self = this;
			var params = {
				projects: true,
				teams: true,
				allowsMultiple: false,
				selection: self._reportTemplate.processAreaUUID,
				filterProjectArea: self._reportTemplate.projectAreaUUID,
				onOk: function(selectedItem, action) {
					self._onProcessOk(selectedItem);
				}
			};
			new com.ibm.team.process.web.ui.internal.ProcessAreaSelectionDialog(params);
		},
		
		_onProcessOk: function(item) {
			this._reportTemplate.processAreaUUID = item.itemId;
			this._processAreaAnchor.innerHTML = item.name;
			this._setDirty(true);
		},
				
		_updateEnablement: function() {
			var name = this._getName();
			var hasName = (name && (name.length > 0));
			var selectedItem = null;
			if(this._templateTreeWidget){
				selectedItem = this._templateTreeWidget.getSelection();
				if(selectedItem && selectedItem.length > 0){
					selectedItem = selectedItem[0];
				}else{
					selectedItem = null;
				}
			}
			var hasReport = selectedItem != null && selectedItem.isLeaf();
			this._saveButton.disabled = (!this.dirty || !hasName || !hasReport);
		},
		
		_setDirty: function(isDirty) {
			if (isDirty === this.dirty) {
				this._updateEnablement();
				return;
			}
		
			if (isDirty) {
				this.metanav.heading.innerHTML = this.messages.reportCustomTemplate + "*";
				this.metanav.heading.style.color = "#A83300";
				this._saveButton.disabled = false;
			} else {
				this.metanav.heading.innerHTML = this.messages.reportCustomTemplate;
				this.metanav.heading.style.color = "#000000";
				this._saveButton.disabled = true;
			}
	
			this.dirty = isDirty;
			this._updateEnablement();
		},
		
		setHeaderMessage: function(message) {
			this.clearValidationMessage();
			if (!message) {
				return;
			}
			
			if (!message.message || message.message.length == 0) {
				message.message = this.messages.errorUnhelpful;
			} else {
				if(message.message.lastIndexOf(".") != (message.message.length-1)) {
					message.message += ".";
				}
			}
			
            if (!message.details) {
                message.details = "";
            } else {
                if (message.details.lastIndexOf(".") != (message.details.length - 1)) {
                    message.details += ".";
                }
            }
            var messageText;
            if (message.message.length > MAX_MESSAGE_LENGTH) {
                messageText = message.message.substring(0, MAX_MESSAGE_LENGTH) + "...";
            } else {
            	if (message.showOnlyMessage == true) {
            		messageText = message.message
            	} else {
                	messageText = message.message + " " + message.details;            		
            	}
                if (messageText.length > MAX_MESSAGE_LENGTH) {
                    messageText = messageText.substring(0, MAX_MESSAGE_LENGTH) + "...";
                }
            }
            messageText = messageText.replace(/\.\.\.\./, "...");
            
            this.metanav.validationMessage.__jazzMessage = message;
            if (message.severity==8) {
				this.typeIcon.src = ImagePool.ERROR;
                this.metanav.validationMessageSpan.className = "validationMessageError";
            } else {
				if (message.severity==4) {
					this.typeIcon.src = ImagePool.WARNING;
					this.metanav.validationMessageSpan.className = "validationMessageWarning";
				} else {
					this.typeIcon.src = ImagePool.INFO;
					this.metanav.validationMessageSpan.className = "validationMessageInfo";
				}
            }
            this.metanav.validationMessage.title = message.details || message.message;
            dojox.data.dom.replaceChildren(this.metanav.validationMessage, document.createTextNode(messageText));
			window.scrollTo(0,0); // show message
		},
		
		_handleHeaderMessageClick: function(e) {
			e.preventDefault();
			if (!e || !e.target || !e.target.__jazzMessage) return;

			var message = e.target.__jazzMessage;
			var text = message.message;
			if (message.details) {
				text += "\n\n";
				text += message.details;
			}
			alert(text);
		},
		
		clearValidationMessage: function(force) {
			this.metanav.validationMessageSpan.className = "validationMessageEmpty";
		},
		
		_showStatusMessage: function(message) {
			this._clearStatusMessage();
			var status = {
				type : jazz.ui.MessageBox.STATUS_TYPES.OK,
				summary : message,
				closable : false
			};
			dojo.addClass(this._messageBoxContainer, "messageBoxVisible");
			this.messageBox.showMessage(status);
		},
		
		_clearStatusMessage: function() {
			dojo.removeClass(this._messageBoxContainer, "messageBoxVisible");
			this.messageBox.hideMessage();
		}
	}
);

})();