/*******************************************************************************
 * Licensed Materials - Property of IBM 
 * (c) Copyright IBM Corporation 2013.  All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights: Use, duplication or
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 ******************************************************************************/
dojo.provide("com.ibm.team.reports.web.ui.internal.ReportJobStatusPage");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");

dojo.require("dojo.date.stamp");
dojo.require("dojo.date.locale");
dojo.require("dojo.i18n");

dojo.require("jazz.ui.StyledTable");
dojo.require("jazz.ui.MessageBox");
dojo.require("jazz.ui.SimpleToolbar");
dojo.require("jazz.app.context");
dojo.require("jazz.app.ua.PageHoverHelp");
dojo.require("jazz.ui.SimpleToolbar");

dojo.require("net.jazz.ajax.contextRoot");

dojo.require("com.ibm.team.reports.web.client.internal.ReportsClient");
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
dojo.require("com.ibm.team.repository.web.client.session.Session");

dojo.requireLocalization("com.ibm.team.reports.web.ui.internal", "ReportsManagementMessages");

(function() {
	
var ReportsClient = com.ibm.team.reports.web.client.internal.ReportsClient;
var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;
var session = com.ibm.team.repository.web.client.session;
var CSH_RUNNING_REPORTS = "com.ibm.jazz.reports.web.cshelp.running_reports";
	
/**
 * Displays running reports on the server.
 * 
 * Constructor properties:
 * 
 * isAdminView - true if the widget should display the administrator view, false otherwise.  True by default
 * showToolbar - true if the toolbar should be displayed, false otherwise.  False by default.
 */
dojo.declare("com.ibm.team.reports.web.ui.internal.ReportJobStatusPage", [dijit._Widget, dijit._Templated], {
	
	messages : dojo.i18n.getLocalization("com.ibm.team.reports.web.ui.internal", "ReportsManagementMessages"),
	templatePath: dojo.moduleUrl("com.ibm.team.reports.web", "ui/internal/templates/ReportJobStatusPage.html"),	
	_jobTable: null, //jazz.ui.StyledTable displaying the running jobs 
	_header: null, //jazz.ui.MessageBox for displaying messages
	_cachedJobs: null, //array of objects that represent running jobs
	isAdminView: true, //true if this is the view for administrators, false otherwise. Default is true
	showToolbar: false,	 //true if the toolbar should be displayed, false otherwise.
	
	postCreate: function() {
		this._header = new jazz.ui.MessageBox();
		this.headerArea.appendChild(this._header.domNode);
		
		this._createContextSensitiveHelp();
		
		if (this.showToolbar) {
			this._createToolbar();
		}
		
		var descriptionText = null;
		
		if (this.isAdminView) {
			descriptionText = this.messages.reportStatusAdminDescription;
		} else {
			descriptionText = this.messages.reportStatusDescription;
		}
		
		this._description.appendChild(document.createTextNode(descriptionText));
		
		this._jobTable = new jazz.ui.StyledTable({columns: [
		                                                    {label: this.messages.reportColumnLabel, sortable:true},
		                                                    {label: this.messages.projectColumnLabel},
		                                                    {label: this.messages.taskColumnLabel},
		                                                    {label: this.messages.userColumnLabel, sortable:true},
		                                                    {label: this.messages.timeColumnLabel, sortable:true},
		                                                    {label: this.messages.statusColumnLabel},
		                                                    {label: this.messages.actionColumnLabel}
		                                                    ],
		                                        handleSort: dojo.hitch(this, this._handleSort)
												});
		this._tableDiv.appendChild(this._jobTable.domNode);

		this._refresh();
	},	
	
	/**
	 * Creates a CSH hook for the page.
	 */
	_createContextSensitiveHelp: function() {
		dojo.empty(this.csHelpNode);
		var parms = {};
		parms.uaKey = CSH_RUNNING_REPORTS;
		var pageHelp = new jazz.app.ua.PageHoverHelp(parms);
		this.csHelpNode.appendChild(pageHelp.domNode);
	},
	
	_refresh: function() {
		var srh = new ServiceResponseHandler(this, "_populateJobTable", "_handleError");
		
		if (this.isAdminView) {
			ReportsClient.getRunningReportJobs(srh);
		} else {
			//if this is not the adminstrator view, the table shows only reports in the current
			//project requested by the current user
			var args = {
					requestingUserId : session.getAuthenticatedUserId(),
					projectAreaUUID : jazz.app.context.get().itemId
			};
			ReportsClient.getRunningReportJobs(srh, args);
		}
	},
	
	/**
	 * Sorts the running jobs according to the current sort settings, and adds them to the 
	 * table in the correct order.
	 */
	_handleSort: function() {
		 var sortByIndex = this._jobTable.sortBy;
		 var asc = this._jobTable.sortByAscending;
		 
		 var sortFunction = null;
		 
		 if (sortByIndex === 0) { //report name
			 sortFunction = function(a, b) {
				var result = a.report.name.localeCompare(b.report.name);
				return asc ? result : result * -1;
			 };
		 } else if (sortByIndex === 3) { //requesting user
			 sortFunction = function(a, b) {
				 var result = a.requestingUserId.localeCompare(b.requestingUserId);
				 return asc ? result : result * -1;
			 };
		 } else if (sortByIndex === 4) { //running time
			 sortFunction = function(a, b){
				 var result = 0;
				 if (a.runningTime < b.runningTime) {
					 result = -1;
				 } else {
					 result = 1;
				 }
				 return asc ? result : result * -1;
			 };
		 }
		 
		 if (sortFunction) {
			 this._cachedJobs.sort(sortFunction);
		 }
		 
		 this._jobTable.clearBody();
		 for (var i = 0; i<this._cachedJobs.length; i++) {
			 this._createTableRow(this._cachedJobs[i]);
		 }
	},

	/**
	 * Creates the toolbar for the page
	 */
	_createToolbar: function() {
		var toolbar = new jazz.ui.SimpleToolbar({});
		this.toolbarNode.appendChild(toolbar.domNode);
		
		//add refresh button
		toolbar.addButton(this, this._refresh, {imgSrc: dojo.moduleUrl("com.ibm.team.reports.web", "ui/internal/images/refresh.gif"), label: this.messages.refreshButtonText});
	},
	
	_handleError: function(error) {
		if (this._header) {
			var status = {
				errorObject: error,
				closable : false
			};
			this._header.showMessage(status);
		}
	},
	
	/**
	 * Populates the table of running jobs based on the server response.
	 * 
	 */
	_populateJobTable: function(reportJobs) {
		//clear the table
		this._jobTable.clearBody();
		this._cachedJobs = [];
		
		this._header.hideMessage();
		if (reportJobs && reportJobs.length >0) {
			for (var i = 0; i<reportJobs.length; i++) {
				this._cachedJobs.push(reportJobs[i]);
			}
			
			this._handleSort(); //will take care of sorting and adding rows to the table as well
		} else {
			this._displayMessage(this.messages.noReportsMessage, jazz.ui.MessageBox.STATUS_TYPES.INFO);
		}
	},
	
	_createTableRow: function(reportJob) {
		var row = new com.ibm.team.reports.web.ui.internal.ReportJobRow({
			job: reportJob,
			onCancel: dojo.hitch(this, this._cancelJob) 
		});
		
		this._jobTable.tbody.appendChild(row.domNode);
	},

	_displayMessage : function(message, messageType,
			closable) {
		if (!closable) {
			closable = false;
		}
		if (this._header) {
			var status = {
				type : messageType,
				summary : message,
				closable : closable
			};
			this._header.showMessage(status);
		}
	},
	
	/**
	 * Requests to cancel the job with the given ID.
	 */
	_cancelJob: function(jobUUID, successCallback) {
		this._header.hideMessage();
		var args = {
			jobUUID : jobUUID	
		};
		
		var self = this;
		var sro = {
				success: function(result) {
					if (result && result === "true") {
						self._displayMessage(self.messages.cancelSuccessMessage, jazz.ui.MessageBox.STATUS_TYPES.OK, true);
						successCallback();
					} else {
						self._displayMessage(self.messages.cancelUnsuccessfulMessage, jazz.ui.MessageBox.STATUS_TYPES.INFO,true);
					}
				},
				failure: function(error) {
					self._handleError(error);
				}
		}
		var srh = new ServiceResponseHandler(sro, "success", "failure");
		ReportsClient.postCancelReportRendering(srh, args);
	}
	
});

/**
 * A row in the report job table representing a running report job.
 */
dojo.declare("com.ibm.team.reports.web.ui.internal.ReportJobRow", [dijit._Widget, dijit._Templated], {
	
	templateString: '<tr>' +
						'<td dojoAttachPoint="_reportCell"></td>' +
						'<td dojoAttachPoint="_projectCell"></td>' +
						'<td dojoAttachPoint="_taskCell"></td>' +
						'<td dojoAttachPoint="_userCell"></td>' +
						'<td dojoAttachPoint="_timeCell"></td>' +
						'<td dojoAttachPoint="_statusCell"></td>' +
						'<td dojoAttachPoint="_actionCell"></td>'	+
					'</tr>',
	job: null,
	messages : dojo.i18n.getLocalization("com.ibm.team.reports.web.ui.internal", "ReportsManagementMessages"),
	_cancelLink: null,
	
	postCreate: function() {
		//set the time cell
		var dateText = this._formatRunningTime(this.job.runningTime);
		this._timeCell.appendChild(document.createTextNode(dateText));
		
		//set the project name
		var projectText = this.job.projectAreaName;
		this._projectCell.appendChild(document.createTextNode(projectText));
		
		//set the report cell; make a link to the report
		var reportURL = net.jazz.ajax.contextRoot() +'/web/projects/'+this.job.projectAreaName+'#action=com.ibm.team.reports.editTemplate&reportUUID=' + this.job.report.reportUUID;
		var reportText = this.job.report.name;
		var reportLink = dojo.create("a");
		reportLink.href = reportURL;
		reportLink.appendChild(document.createTextNode(reportText));
		this._reportCell.appendChild(reportLink);
		
		//set the task name
		var taskText = this.job.task;
		this._taskCell.appendChild(document.createTextNode(taskText));
		
		//set the user cell
		//TODO could we make this a link to the user instead?
		var userText = this.job.requestingUserId;
		this._userCell.appendChild(document.createTextNode(userText));
		
		//set the status cell
		var statusText = "";
		if (this.job.cancelled) {
			statusText = this.messages.reportStatusCancelledLabel;
		} else {
			statusText = this.messages.reportStatusRunningLabel;
		}
		
		this._statusCell.appendChild(document.createTextNode(statusText));
		
		//create the action toolbar
		var div = document.createElement("div");
		div.className = "actionBar";

		// Cancel button
		var anchor = document.createElement("a");
		anchor.href = "#";
		anchor.title = this.messages.cancelButtonText;
		
		this._cancelLink = anchor;
		div.appendChild(anchor);
		var img = document.createElement("img");
		img.alt = this.messages.cancelButtonText;
		img.src = dojo.moduleUrl("com.ibm.team.reports.web", "ui/internal/images/cancel.gif");
		anchor.appendChild(img);
		var jobUUID = this.job.jobUUID;
		var handler = dojo.connect(anchor, "onclick", dojo.hitch(this, function(event) {
			dojo.stopEvent(event);
			this.onCancel(jobUUID, dojo.hitch(this, this._jobCancelled));
		}));
		
		this._actionCell.appendChild(div);		
	},
	
	/**
	 * Callback when a job is successfully cancelled.
	 */
	_jobCancelled: function() {
		//update the status column
		this._statusCell.innerHTML = "";
		var statusText = this.messages.reportStatusCancelledLabel;		
		this._statusCell.appendChild(document.createTextNode(statusText));
		
		//remove the cancel button
		dojo.style(this._cancelLink, "display", "none");
	},
	
	/**
	 * Given the elapsed time fo the job (in milliseconds), converts and returns a string
	 * representing the elapsed running time, in format dd:HH:mm:ss
	 */
	_formatRunningTime: function(runningTime) {
		var timeString = "";
		
		var ms = runningTime%1000;
		timeString += this._pad(ms, 3);
		var remainder = Math.floor(runningTime/1000);
		
		var s = remainder%60;
		timeString = this._pad(s, 2) + ':' + timeString;
		remainder = Math.floor(remainder/60);
		
		var min = remainder%60;
		timeString = this._pad(min, 2) + ':' + timeString;
		remainder = Math.floor(remainder/60);
		
		var h = remainder%24;
		timeString = this._pad(h, 2) + ':' + timeString;
		remainder = Math.floor(remainder/24)
		
		if (remainder > 0) {
			//don't break it down any further, just display the # of days as-is
			timeString = dojo.string.substitute(this.messages.daysElapsedLabel, [remainder]) + ', ' + timeString
		}
		
		return timeString;
		
	},
	
	/**
	 * Given an integer, returns it as a string of the specified length, with leading 0's as
	 * required.
	 */
	_pad: function(number, length) {
		var numString = number + "";
		
		while (numString.length < length) {
			numString = "0" + numString;
		}
		
		return numString;
	}
	
});
	
})();