/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007, 2013. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:  Use,
 * duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.reports.web.ui.internal.ReportsManagementPage");

dojo.require("dojo.i18n");
dojo.require("dijit._Widget");
dojo.require("dijit._Templated");

dojo.require("jazz.ui.SideNavigationLayout");
dojo.require("net.jazz.ajax.ui.PlatformUI");
dojo.require("com.ibm.team.reports.web.ui.internal.EtlConfig");
dojo.require("com.ibm.team.reports.web.ui.internal.EtlStatus");
dojo.require("com.ibm.team.reports.web.ui.internal.DataWarehouseDiagnostics");
dojo.require("com.ibm.team.reports.web.ui.internal.ReportDownloads");
dojo.require("com.ibm.team.reports.web.ui.internal.ReportJobStatusPage");
dojo.require("com.ibm.team.repository.web.ui.internal.admin.NotAuthorizedView");
dojo.require("com.ibm.team.reports.web.ui.internal.ConfigureDataWarehouseConnection");
dojo.require("com.ibm.team.reports.web.ui.internal.ConfigureCustomReportsConnection");

dojo.require("com.ibm.team.reports.web.client.internal.ReportsClient");

dojo.requireLocalization("com.ibm.team.reports.web.ui.internal", "ReportsManagementMessages");

(function(){ // module container
    var ReportsClient = com.ibm.team.reports.web.client.internal.ReportsClient;
    var PlatformUI = net.jazz.ajax.ui.PlatformUI;
    var ActionRegistry = null; // Initialize later
    var ActionRegistry = null;
    var PREFIX = "com.ibm.team.reportsManagement.";
    var SERVER_PACKAGE = "com.ibm.team.reports.web.ui.internal.";
    var ACTION_MAP = {};
    
    var ACTION_SETS = [{
        labelKey: 'dataCollection',
        actions: [{
            labelKey: 'etl',
            shortId: "etlConfig",
            widget: "EtlConfig"
        },{
            labelKey: 'etlStatus',
            shortId: "etlStatus",
            widget: "EtlStatus"
        }]
    },{
        labelKey: 'configuration',
        actions: [{
            labelKey: 'customReportsHeading',
            shortId: "configureCustomReportsConnection",
            widget: "ConfigureCustomReportsConnection"
        },{
            labelKey: 'dataWarehouseHeading',
            shortId: "configureDataWarehouseConnection",
            widget: "ConfigureDataWarehouseConnection"
        },{
            labelKey: 'downloads',
            shortId: "reportDownloads",
            widget: "ReportDownloads"
        }]
    }, {
    	labelKey: 'reports',
    	actions:[{
    		labelKey: 'runningReports',
    		shortId: 'runningReports',
    		widget: "ReportJobStatusPage"
    	}]
    }];
    
    /**
     * Main perspective view for the Reports perspective.
     */
    dojo.declare("com.ibm.team.reports.web.ui.internal.ReportsManagementPage", dijit._Widget, {
    
        constructor: function(){
            this.messages = dojo.i18n.getLocalization("com.ibm.team.reports.web.ui.internal", "ReportsManagementMessages");
        },
        
        reloadCurrent: function(){
            this._previousActionEntry = this._currentActionEntry || null;
            this._showCurrentWidget(/*isReload*/true);
        },
        
        postCreate: function(){
            // call superclass's postCreate method
            this.inherited(arguments);
            
            dojo.addClass(this.domNode, "com-ibm-team-reports-content-area");
            
            // initialize widget cache
            this._widgetCache = {};
            
            // instantiate the side-nav layout UI and append it to the page
            this._appView = new jazz.ui.SideNavigationLayout();
            this.domNode.appendChild(this._appView.domNode);
            
            // register actions and create side-nav hyperlinks
            ActionRegistry = getWorkbench().getActionRegistry();
            ActionRegistry.registerAction(PREFIX + "_defaultAction", this, "_defaultAction");
            
            var self = this;
            
            this.allLinks = [];
            dojo.forEach(ACTION_SETS, function(actionSet){
                self._createNavSection(actionSet);
            });
          
            //register hidden diagnostics action
            self._registerAction({
	        	labelKey: 'diagnostics',
	        	shortId: "diagnostics",
	        	widget: "DataWarehouseDiagnostics"
	        });
         },
        
        _createNavSection: function(actionSet){
            var self = this;
            var navSection = this._appView.addNavigationSection(document.createElement("div"), this.messages[actionSet.labelKey], true);
            dojo.forEach(actionSet.actions, function(action){
                var id = PREFIX + action.shortId;
                var link = navSection.addSectionRow(self.messages[action.labelKey], "#action=" + id);
                link.__jazzActionId = action.shortId;
                self.allLinks.push(link);
                navSection.addArtifactCommand(link, self.messages.refresh, dojo.hitch(self, self._onRefreshClick), "admin-refresh-command");
                self._registerAction(action);
            });
            
        },
        
        _registerAction: function(action){
            var id = PREFIX + action.shortId;
            ACTION_MAP[action.shortId] = action;
            ActionRegistry.registerAction(id, this, "_handleAction");
        },
        
        _handleAction: function(args){
            if (!isAdmin()) {
            	this._showUnauthorized();
                return;
            }
            
            if (!args.action) {
                console.debug("ServerPage.js#_handleAction: missing required param 'args.action'");
                return;
            }
            
            // cache the previous entry to see if the widget should be destroyed or
            // not
            this._previousActionEntry = this._currentActionEntry || null;
            
            // parse out the short id from the fully-qualified id
            var shortId = args.action.substring(args.action.lastIndexOf(".") + 1);
            this._currentActionEntry = ACTION_MAP[shortId];
            this._currentArgs = args;
            this._currentArgs.parent = this;
            
            // set the title for the action
            getWorkbench().setTitle(this.messages[this._currentActionEntry.labelKey]);
            
            // highlight the selected link and unhighlight the others
            dojo.forEach(this.allLinks, function(link){
                if (link.__jazzActionId === shortId) 
                    dojo.addClass(link, "selected");
                else 
                    dojo.removeClass(link, "selected");
            });
            
            // call the actual method implementation
            this._showCurrentWidget();
        },
        
        _onRefreshClick: function(e){
            dojo.stopEvent(e);
            this.reloadCurrent();
        },
        
        _showCurrentWidget: function(isReload){
            // save a reference to the prior content widget
            this._previousContentWidget = this._currentContentWidget || null;
            
            if (this._currentActionEntry.cached) {
                this._showCachedWidget(isReload);
            }
            else {
                this._showUncachedWidget();
            }
            
            // help the garbage collector do its job
            if (this._previousContentWidget && !this._previousActionEntry.cached) {
                this._previousContentWidget.destroyRecursive();
            }
        },
        
        _showUncachedWidget: function(){
            var constructor = dojo.getObject(SERVER_PACKAGE + this._currentActionEntry.widget);
            this._currentContentWidget = new constructor(this._currentArgs);
            this._appView.setMainContent(this._currentContentWidget.domNode);
        },
        
        _defaultAction: function(){
            // by default, show the first section from the left-nav
            this._handleAction({
                action: PREFIX + ACTION_SETS[0].actions[0].shortId
            });
        },
    	
    	_showUnauthorized: function(){
    		dojox.xml.parser.removeChildren(this.domNode);
    		if (!this._notAuth) {
    			this._notAuth = new com.ibm.team.repository.web.ui.internal.admin.NotAuthorizedView();
    		}	
    		this.domNode.appendChild(this._notAuth.domNode);
    	}
        
    });
    
    
    function getWorkbench(){
        return net.jazz.ajax.ui.PlatformUI.getWorkbench();
    }
    
    function isAdmin(){
        return com.ibm.team.repository.web.noAdminAuth ? false : true;
    }
    
})();
