/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007, 2013. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:  Use,
 * duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.reports.web.ui.internal.SaveAsDialog");

dojo.require("dojo.i18n");
dojo.require("dojo.date.locale");

dojo.require("jazz.ui.Dialog");
dojo.require("jazz.ui.Tree");
dojo.require("jazz.ui.tree.IconItem");

dojo.requireLocalization("com.ibm.team.reports.web.ui.internal", "ReportMessages");

dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
dojo.require("com.ibm.team.reports.web.client.internal.ReportsClient");
dojo.require("com.ibm.team.repository.web.client.session.Session");

(function() {
	
	var SHARED = 1 << 0;
	var PRIVATE = 1 << 2;
	var SYSTEM = 1 << 3;
	var ReportsClient = com.ibm.team.reports.web.client.internal.ReportsClient;
	var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;
	var session = com.ibm.team.repository.web.client.session;

    dojo.declare("com.ibm.team.reports.web.ui.internal.SaveAsDialog", [dijit._Widget, dijit._Templated], {
       templatePath: dojo.moduleUrl("com.ibm.team.reports.web", "ui/internal/templates/SaveAsDialog.html"),
		
		widgetsInTemplate: true,

		constructor: function(props) {
			this.messages = dojo.i18n.getLocalization("com.ibm.team.reports.web.ui.internal", "ReportMessages");
			this._folders = null;
			this._queries = null;
			this.isExistingReport = false;
			this._responses = {}; //track which responses we've received
			this._allFolders = [];
			this._folderMap = {};
			this._responseCount = 0;
		},
		
		postCreate: function() {
			
			this._dialog = new jazz.ui.Dialog({
				contentNode: this._dialog,
				primaryTitle: this.messages.saveAs,
				width: "420px",
				height: "280px",
				isClosable: true
			});
			this.connect(this._onSubmit, "onclick", this._onSubmit);
			this.connect(this._onClose, "onclick", this._onClose);
			
			this.messageBox = new jazz.ui.MessageBox();
			this._messageBoxContainer.appendChild(this.messageBox.domNode);
			
			if (this.reportUUID == null && this.queryUUID == null) {
                throw new Error("One of 'reportUUID' or 'queryUUID' is REQUIRED.");
            }
            
			this._createFolderTree();			

			var args = {
				projectAreaUUID: this.projectAreaUUID,
				flags: PRIVATE
			};

			try {
				var ownerUUID = session.getAuthenticatedContributor().itemId;
				args.ownerUUID = ownerUUID;
			} catch (err) {
				//if we can't get the current user from the session for some reason, that's OK
				//just tell the server we want the current user
				args.currentUser = true;
			}			

			responseHandler = new ServiceResponseHandler(
					this,
					"_addFolders",
					"_FolderError");
			
			ReportsClient.getFolderDescriptors(responseHandler, args);

			args = {
				projectAreaUUID: this.projectAreaUUID,
				flags: (SHARED | SYSTEM)
			};
			responseHandler = new ServiceResponseHandler(
					this,
					"_addFolders",
					"_FolderError");
			ReportsClient.getFolderDescriptors(responseHandler, args);		
		},
		
		_onSubmit: function() {
			this.messageBox.hideMessage();
			dojo.removeClass(this._messageBoxContainer, "messageBoxVisible");
			var invalid = this._validate();
			if (invalid != null) {
				this._showErrorMessage(invalid);
				return false;
			} else {
				
				var args = {
						name : this.textField.value,
						reportUUID: this.reportUUID,
						description : this.description.value
				}
				
				var folder = this._folderTreeWidget.getSelection()[0];
				if (folder.folderUUID != null && dojo.trim(folder.folderUUID).length > 0) {
					args.folderUUID = folder.folderUUID;
				} else {
					args.sharedIfNullFolder = folder.isSharedRoot;
				}

				this.onSubmit(args);
			}
			this._onClose();
		},
		
		_validate: function() {
			var name = this.textField.value;
            if (name == null || dojo.trim(name).length == 0) {
                return "A report name is required.";
            } 
			return null;	
		},
		
		_showErrorMessage: function(message) {
			var status = {
				type : jazz.ui.MessageBox.STATUS_TYPES.ERROR,
				summary : message,
				closable : false
			};
			dojo.addClass(this._messageBoxContainer, "messageBoxVisible");
			this.messageBox.showMessage(status);
		},
		
		_saveComplete: function(args) {
			document.location.href = "#action=com.ibm.team.reports.viewQuery&queryUUID=" + args;
		},
		
		_onClose: function() {
			this._dialog.close();
		},
		
		open: function() {
			this.textField.focus();
		},
		
		_error: function(e) {
            console.error(this.declaredClass + ": Error: " + e.message);
            if (dojo.isFunction(this.onError)) {
                this.onError(e.message);
            }                       
        },
        
        _addFolders: function(folders) {
    		this._responseCount++;
    		if (!folders) folders = [];
    		for (var i = 0; i < folders.length; i++) {
    			this._allFolders.push(folders[i]);
    		}
    		//If we haven't handled both the shared and private folders return
    		if (this._responseCount == 1) {
    			return;
    		}
    		this._addFoldersToTree();			
    	},

    	_addFoldersToTree: function() {
    		// First, find the two root folders
    		var sharedRoot = null;
    		var privateRoot = null;
    		for (var i = 0; i < this._allFolders.length; i++) {
    			var folder = this._allFolders[i];
    			if (folder.system) {
    				if (folder.shared) {
    					sharedRoot = folder;
    				} else {
    					privateRoot = folder;
    				}
    			}
    		}

    		if (privateRoot) {
    			var rootItem = new jazz.ui.tree.IconItem({label: privateRoot.name, iconClass: "iconFolder"});
    			privateRoot.treeItemID = rootItem.getId();  //Save the item ID in the folders array in case we need to make tree calls based on the id
    			rootItem.folderUUID = null;
    			rootItem.isSharedRoot = false;
    			this._folderTreeWidget.addChild(rootItem);
    			this._addChildrenForFolder(rootItem, privateRoot, this._allFolders, true, false);
    			rootItem.setExpanded(true);			
    		}
    		if (sharedRoot) {
    			var rootItem = new jazz.ui.tree.IconItem({label: sharedRoot.name, iconClass: "iconFolder"});
    			sharedRoot.treeItemID = rootItem.getId();  //Save the item ID in the folders array in case we need to make tree calls based on the id
    			rootItem.folderUUID = null;
    			rootItem.isSharedRoot = true;
    			this._folderTreeWidget.addChild(rootItem);
    			this._addChildrenForFolder(rootItem, sharedRoot, this._allFolders, true, true);
    			this._folderTreeWidget.setSelection(rootItem);
    			rootItem.setExpanded(true);			
    		}
    	},
    	
    	/**
    	 * From the list of folders, find any that are children of parentFolder, and
    	 * add them to the tree. Then recurse for each of the children.
    	 */
    	_addChildrenForFolder: function(parentFolderNode, parentFolder, folders, parentIsRoot, parentIsShared) {
    		var foldersToAdd = [];
    		for (var i = 0; i < folders.length; i++) {
    			// Special case where the parent is a root system folder
    			if (parentIsRoot) {
    				if (!folders[i].parentUUID && (folders[i].shared == parentIsShared) && (folders[i].name != parentFolder.name)) {
    					foldersToAdd.push(folders[i]);
    				}
    			} else {
    				if (folders[i].parentUUID == parentFolder.folderUUID) {
    					foldersToAdd.push(folders[i]);
    				}
    			}
    		}
    		foldersToAdd = foldersToAdd.sort(function(a, b) {return a.name > b.name;});
    		for (var i = 0; i < foldersToAdd.length; i++) {
    			var childNode = new jazz.ui.tree.IconItem({label: foldersToAdd[i].name, iconClass: "iconFolder"});
    			childNode.folderUUID = foldersToAdd[i].folderUUID;
    			foldersToAdd[i].treeItemID = childNode.getId();  //Save the item ID in the folders array in case we need to make tree calls based on the id
    			parentFolderNode.addChild(childNode);
    			this._addChildrenForFolder(childNode, foldersToAdd[i], folders, false, parentIsShared);
    		}
    	},

    	_createFolderTree: function() {
    		this._folderTreeWidget = new jazz.ui.Tree();
    		this._folderTree.appendChild(this._folderTreeWidget.domNode);
    	},
    	
    	_FolderError: function(){
    		alert(this.messages.errorCantCommunicate);
    	},
		
	});
	
})();
