/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2012, 2013. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  
 * Use, duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp. 
 *******************************************************************************/
dojo.provide("com.ibm.team.process.web.ui.internal.SelectableStyledTable"); //$NON-NLS-1$

dojo.require("com.ibm.team.process.web.ui.internal.TRToolbar"); //$NON-NLS-1$

dojo.require("dijit.MenuSeparator"); //$NON-NLS-1$

dojo.require("jazz.ui.FilterBox"); //$NON-NLS-1$
dojo.require("jazz.ui.StyledTable"); //$NON-NLS-1$
dojo.require("jazz.ui.Toolbar"); //$NON-NLS-1$
dojo.require("jazz.ui.toolbar.MenuButton"); //$NON-NLS-1$

(function(){
	
	/**
	 * Extends the base StyledTable class and adds support for permanently
	 * selecting a row
	 * 
	 * @name com.ibm.team.process.web.ui.internal.SelectableStyledTable
	 * @constructor
	 * @extends jazz.ui.StyledTable
	 */ 
    dojo.declare("com.ibm.team.process.web.ui.internal.SelectableStyledTable", jazz.ui.StyledTable, { //$NON-NLS-1$
    	/**
    	 * What item position the list starts at, can be changed at
    	 * initialization using this.start
    	 * To change after initialization, use this.changeTablePosition(start, length)
    	 * 
    	 * @private
    	 * @type Positive_Integers
    	 * @default 0
    	 */ 
		_start : 0,
		
		/**
    	 * How long the list should be, can be changed at initialization using this.length
    	 * To change after initialization, use this.changeTablePosition(start, length)
    	 * Note: Negative numbers denote infinite length
    	 * 
    	 * @private
    	 * @type Integers
    	 * @default -1
    	 */ 
		_length : -1,
		
		/**
    	 * An array which contains all the rows in the order they were added
    	 * 
    	 * @private
    	 * @type Array
    	 */ 
		_rowsByOrder: null,
		
		/**
    	 * An object which stores all the rows in a map by their IDs
    	 * 
    	 * @private
    	 * @type Object
    	 */ 
		_rowsById : null,
		
		/**
    	 * An array which stores all the displayed rows in the order they are displayed
    	 * 
    	 * @private
    	 * @type Array
    	 */ 
		_rowsDisplayedByOrder : null,
		
		/**
    	 * An array which stores all the filtered rows in the order they are displayed
    	 * 
    	 * @private
    	 * @type Array
    	 */ 
		_rowsFilteredByOrder : null,
		
		/**
    	 * An object which stores all the selected rows in a map by their IDs
    	 * 
    	 * @private
    	 * @type Object
    	 */ 
		_rowsSelectedById : null,
		
		/**
    	 * An Integer which counts the size of _rowsSelectedById
    	 * 
    	 * @private
    	 * @type Positive_Integers
    	 */ 
		_selectedCount : 0,
		
		/**
    	 * An Object which contains the information of the clickable button next 
    	 * to the checkbox
    	 * imgSrc: The location of the image to use for the button
    	 * label: The label of the button that is seen on hover
    	 * 
    	 * @public
    	 * @type Object
    	 */ 
		button: null,
		
		/**
    	 * An Array of objects which contains the information of an element of
    	 * the dropdown menu.
    	 * label: The hover over value
		 * title: The display value in the menu
		 * iconClass: CSS Class to associate the icon with
		 * onClick: What should be clicked when the menu item is clicked
		 * isEnabled: Determine if the menu item should be disable or enabled
    	 * 
    	 * @public
    	 * @type Array
    	 */ 
		menu: null,
		
		
		/**
    	 * An object of the actual dropdown menu
    	 * 
    	 * @private
    	 * @type Object
    	 */ 
		_rowMenu: null,
		
		/**
    	 * An array which contains each object in the dropdown menu
    	 * 
    	 * @private
    	 * @type Array
    	 */ 
		_rowMenuChildren: null,
		
		/**
    	 * An Array of objects which contains the information of an element of
    	 * the dropdown menu.
    	 * label: The hover over value
		 * title: The display value in the menu
		 * iconClass: CSS Class to associate the icon with
		 * onClick: What should be clicked when the menu item is clicked
		 * isEnabled: Determine if the menu item should be disable or enabled
    	 * 
    	 * @public
    	 * @type Array
    	 */ 
		selectAllMenu: null,
		
		/**
    	 * The pop up menu for the checkbox in the header of the table
    	 * 
    	 * @private
    	 * @type Menu
    	 */
		_selectAllPopUpMenu : null,
		
		/**
    	 * An array which contains each object in the dropdown menu
    	 * 
    	 * @private
    	 * @type Array
    	 */ 
		_selectAllPopUpMenuChildren: null,
		
		/**
    	 * If this variable is true, _updateTable will not perform any actions
		 * If false, _updateTable will function as intended
    	 * 
    	 * @private
    	 * @type Boolean
    	 */ 
		_suspendUpdate : false,
		
		/**
    	 * If this variable is true, on suspendTableUpdate(false) will perform
    	 * table update
		 * If false, on suspendTableUpdate(false) no update will be performed
    	 * 
    	 * @private
    	 * @type Boolean
    	 */ 
		_tableNeedsUpdate : false,
		
		/**
    	 * The checkbox in the header of the table
    	 * 
    	 * @private
    	 * @type Checkbox
    	 */ 
		_selectAllCheckBox : null,
		
		/**
    	 * The last row in which the action menu was clicked
    	 * 
    	 * @private
    	 * @type Object
    	 */ 
		_lastFocusedRow : null,
		
		/**
    	 * Determines if the table should have filtering included
    	 * 
    	 * @private
    	 * @type Boolean
    	 * @default true
    	 */ 
		canFilter: true,

		/**
    	 * The string or regular expression to split the string by
    	 * 
    	 * @public
    	 * @type Object
    	 * @default /[\s]+/
    	 */ 
		filterSplitter: null,
		
		/**
    	 * The filter box widget
    	 * 
    	 * @private
    	 * @type Object
    	 */ 
		_filterBox: null,
		
		/**
    	 * The previous value of the filter
    	 * 
    	 * @private
    	 * @type String
    	 */ 
		_previousFilter: null,
		
		/**
    	 * Used to keep track of the timeout for filtering
    	 * 
    	 * @private
    	 * @type Object
    	 */ 
		_filterTimeout: null,
		
		/**
    	 * Used to determine the timeout length (in milliseconds) for filtering 
    	 * before executing the filter
    	 * 
    	 * @private
    	 * @type Positive_Integers
    	 */ 
		_fitlerWaitTime: 250,
		
		templatePath: dojo.moduleUrl("com.ibm.team.process.web", "ui/internal/templates/SelectableStyledTable.html"), //$NON-NLS-1$ //$NON-NLS-2$	
		
		/**
    	 * Occurs before anything else is allowed to happen
		 * This is required in order to insert the checkbox column in the table.
    	 */ 
		preamble: function(args){
			this.messages = dojo.i18n.getLocalization("com.ibm.team.process.web.ui.internal", "Messages"); //$NON-NLS-1$ //$NON-NLS-2$
			
			if (this._selectAllCheckBox) {
				if (this._selectAllCheckBox._button) {
					this._selectAllCheckBox._button.destroyRecursive();
				}
				this._selectAllCheckBox.destroyRecursive();
				delete this._selectAllCheckBox;
			}

			this._selectAllCheckBox = new jazz.ui.Toolbar();//$NON-NLS-1$
       		
       		this._selectAllCheckBox._button = new jazz.ui.toolbar.MenuButton({
       			imgSrc: dojo.moduleUrl("com.ibm.team.process.web", "ui/internal/images/spacer.gif"), //$NON-NLS-1$  //$NON-NLS-2$
   				title: this.messages.SelectableStyledTable_SelectAllCheckBoxHover,
   				menuProvider: dojo.hitch(this, function(e){return this._handleSelectAllPopupMenuAction(e);}) 
		    });
			this._selectAllCheckBox.addChild(this._selectAllCheckBox._button);
			
			this._setCheckBox(this._selectAllCheckBox, false);
			
			args.columns.unshift({
				label: this._selectAllCheckBox.domNode,
				className: "com-ibm-team-process-web-ui-internal-SelectableStyledTable-checkbox", //$NON-NLS-1$
				sortable: false
			});
		},
		
		/**
		 * @constructor
		 * @param {Object} args An object which contains multiple arguments
		 * @param {Object} args.button An object which contains the following
		 * @param {String} args.button.imgSrc The location of the image to use
		 * 		  for the button
    	 * @param {String} args.button.label The label of the button that is
    	 * 		  seen on hover
		 * @param {Object[]} args.menu An array which contains objects setup in 
		 * 		  the following
		 * @param {String|Function}	args.menu[].label The hover over value (Can 
		 * 		  be a string or a function as follows)
		 *   parameters Object The row's callback data which the popup menu was 
		 * 		  opened on
		 *   returns String the string to be outputed
		 * @param {String|Function}	args.menu[].title The display value in the
		 * 		  menu (Can be a string or a function as follows)
		 *   parameters Object The row's callback data which the popup menu was
		 *   	  opened on
		 *   returns String the string to be outputed
		 * @param {Function} [args.menu[].isEnabled = true] Is a function which 
		 * 		  is called to see if the current item is currently enabled 
		 * 		  (default function always returns true)
		 *   returns Boolean (true means the item is enabled, false mean the item is disabled)
		 * @param {Boolean} [args.menu[].isDivider = false] Determines if the
		 * 		  current element is a Divider or is a field (default is false)
		 * @param {Object[]} args.column An Array of columns which are created
		 * @param {String} args.column[].label The label of the column
		 * @param {String} args.column[].className The class to assoicate the
		 * 		  column with
		 * @param {Boolean} args.column[].sortable If the column is sortable or
		 * 		  not (Not suggested for non-full length tables)
		 * @param {Function} args.column[].paintColumnFunction The function
		 * 		  which is used for painting each column
		 * 
		 * See jazz.ui.StyledTable for all other input parameters
    	 */
		constructor: function(args){
			if (args.start != null) {
				this._start = args.start;
			}
			if (args.length != null) {
				this._length = args.length;
			}
			this._intializeVariables();
		},
		
		/**
    	 * A function responsible for setting up the objects with their initial 
    	 * values
    	 * 
    	 * @private
    	 */ 
		_intializeVariables: function(){
        	this._rowsById = {};
        	this._rowsByOrder = [];
        	this._rowsDisplayedByOrder = [];
        	this._rowsFilteredByOrder = [];
        	this._rowsSelectedById = {};
        	this._selectedCount = 0;
		},
		
		/**
    	 * A function responsible for setting up the initial table
    	 */ 
        postCreate: function(){       	
            this.inherited(arguments);
			
			if (this.filterSplitter === null) {
				this.filterSplitter = /[\s]+/;
			}
			
			this.connect(this._selectAllCheckBox, "onclick", "_handleSelectAllCheckboxClick"); //$NON-NLS-1$ //$NON-NLS-2$
			
			if (this.canFilter) {
				var filterArea = document.createElement("div"); //$NON-NLS-1$
				dojo.addClass(filterArea, "filterArea"); //$NON-NLS-1$
				this._filterBox = new jazz.ui.FilterBox({
					onChange: dojo.hitch(this, this._onFilterChange)
				});
				filterArea.appendChild(this._filterBox.domNode);
				this.tableControls.appendChild(filterArea);
				
				this._previousFilter = this._filterBox.value;
			}
        },
        
        /**
    	 * An extendable function which is called before a row's selection is
    	 * changed. This function allows the change to be cancelable
    	 * 
    	 * @public
    	 * @param {Object} row the callback data of the row that's selection is 
    	 * changing
    	 * @returns {Boolean} true allows the change to occur, false will not
    	 */ 
        onSelectionChange: function(row) { return true; },
        
        /**
         * Gets the callback data for a specific row
         * 
         * @public
         * @param {String} rowId The id of the row to get
         * @returns {Object} The callback data of the row 
         * @throws {Error} ID not found
         */
        getRow: function(rowId) {
        	var row = this._rowsById[rowId];
        	if (row == undefined) {
        		throw new Error(this.messages.SelectableStyledTable_NoID);
        	}
        	return row.callbackData;
        },
        
        /**
         * Gets the callback data for a specific row
         * 
         * @public
         * @param {String} rowId The id of the row to get
         * @param {Object} callbackData The callback data of the row
         * @throws {Error} ID not found
         */
        updateRowCallbackData: function(rowId, callbackData) {
        	var row = this._rowsById[rowId];
        	if (row == undefined) {
        		throw new Error(this.messages.SelectableStyledTable_NoID);
        	}
        	row.callbackData = callbackData;
        },
        
        /**
    	 * Get all the rows from the table
    	 * 
    	 * @public
    	 * @returns {Array} The callback data of all the rows
    	 */ 
        getAllRows: function(){
        	var rows = [];
            var i;
            for (i = 0; i < this._rowsByOrder.length; i++) {
                rows.push(this._rowsByOrder[i].callbackData);
            }
            return rows;
        },
        
        /**
    	 * Get all the selected rows from the table
    	 * 
    	 * @public
    	 * @returns {Array} The callback data of all the selected rows
    	 */ 
        getAllSelectedRows: function(){
        	var selectedRows = [];
        	
        	if (this._lastFocusedRow != null && !this._lastFocusedRow.rowSelected) {
        		selectedRows.push(this._lastFocusedRow.callbackData);
        	}
        	
            for (var property in this._rowsSelectedById) {
            	if (this._rowsSelectedById[property] != null) {
            		selectedRows.push(this._rowsSelectedById[property].callbackData);
            	}
            }
            return selectedRows;
        },
        
        /**
    	 * Get the number of rows in the table
    	 * 
    	 * @public
    	 * @returns {Positive_Integer} The number of rows
    	 */ 
        getCount: function(){
        	return this._rowsByOrder.length;
        },
        
        /**
    	 * Get the number of selected rows in the table
    	 * 
    	 * @public
    	 * @returns {Positive_Integer} The number of selected rows
    	 */ 
        getSelectedCount: function(){
       		if (this._lastFocusedRow != null && !this._lastFocusedRow.rowSelected) {
        		return this._selectedCount + 1;
        	}
        	return this._selectedCount;
        },
        
        /**
    	 * Determines if a specific row is selected
    	 * 
    	 * @public
    	 * @param {String} rowId The ID of the row in question
    	 * @returns {Boolean} True if the row is selected, false otherwise
    	 * @throws {Error} ID not found
    	 */ 
        isRowSelected: function(rowId){
        	var row = this._rowsById[rowId];
        	if (row == undefined) {
        		throw new Error(this.messages.SelectableStyledTable_NoID);
        	}
        	return row.rowSelected;
        },
        
        /**
    	 * Determines if a specific row is selected
    	 * 
    	 * @public
    	 * @param {String} rowId The ID of the row in question
    	 * @param {String} className Name of class to check if the row has
    	 * @returns {Boolean} True if the row has class, false otherwise
    	 * @throws {Error} ID not found
    	 */ 
        hasClass: function(rowId, className){
        	var row = this._rowsById[rowId];
        	if (row == undefined) {
        		throw new Error(this.messages.SelectableStyledTable_NoID);
        	}
        	
        	return this._hasClass(row, className);
        },
        
        /**
    	 * Determines if a specific row is selected
    	 * 
    	 * @private
    	 * @param {Object} row The row to check on
    	 * @param {String} className Name of class to check if the row has
    	 * @returns {Boolean} True if the row has class, false otherwise
    	 */ 
        _hasClass: function(row, className){
        	return dojo.hasClass(row, className);
        },
        
        /**
    	 * Selects all the unselected rows and deselectes all the selected rows
    	 * 
    	 * @public
    	 */ 
        selectInverse: function() {
        	for (var i = 0; i < this._rowsByOrder.length; i++) {
        		this._selectRow(this._rowsByOrder[i], !this._rowsByOrder[i].rowSelected);
        	}
        },
        
        /**
    	 * Selects all the displayed rows
    	 * 
    	 * @public
    	 * @param {Boolean} selection True selects all the rows, false deselects 
    	 * 		  all the rows
    	 */ 
        selectDisplayed: function(selection) {
        	for (var i = 0; i < this._rowsDisplayedByOrder.length; i++) {
        		this._selectRow(this._rowsDisplayedByOrder[i], selection);
        	}
        },
        
        /**
    	 * Selects all the rows in the table
    	 * 
    	 * @public
    	 * @param {Boolean} selection True selects all the rows, false deselects 
    	 * 		  all the rows
    	 */ 
        selectAll: function(selection){
        	for (var i = 0; i < this._rowsByOrder.length; i++) {
        		this._selectRow(this._rowsByOrder[i], selection);
        	}
        },
        
        /**
    	 * Selects a specific row in the table
    	 * 
    	 * @public
    	 * @param {String} rowId The ID of the row to change
    	 * @param {Boolean} selection True to select the row, false to deselect 
    	 * 		  the row
    	 * @returns {Boolean} True if row was successfully changed or if the
    	 * 		  row's value was already equal to selection, false if unable to
    	 * 		  change row's value
    	 * @throws {Error} ID not found
    	 */ 
        selectRow: function(rowId, selection){
        	var row = this._rowsById[rowId];
        	if (row == undefined) {
        		throw new Error(this.messages.SelectableStyledTable_NoID);
        	}
        	return this._selectRow(row, selection);
        },
        
        /**
         * Updates the checkbox at the top of the table
         * 
         * @private
         */
        _updateSelectAllCheckBox: function() {
        	this._setCheckBox(this._selectAllCheckBox, 
				this._selectedCount > 0 && this._selectedCount < this.getCount() 
    			? null 
    			: this._selectedCount !== 0);
        },
        
        /**
    	 * Selects a specific row in the table
    	 * 
    	 * @private
    	 * @param {Object} row The row to change
    	 * @param {Boolean} selection True to select the row, false to deselect 
    	 * 		  the row
    	 * @returns {Boolean} True if row was successfully changed or if the
    	 * 		  row's value was already equal to selection, false if unable to
    	 * 		  change row's value
    	 */ 
        _selectRow: function(row, selection){
        	if (selection == row.rowSelected) {
        		return true;
        	}
        	if (this.onSelectionChange(row.callbackData)) {
	        	if (selection && !row.rowSelected) {
	        		this._rowsSelectedById[row.id] = row;
	        		this._selectedCount++;
	        	} else if (!selection && row.rowSelected) {
	        		delete this._rowsSelectedById[row.id];
	        		this._selectedCount--;
	        	}
	        	
	        	row.rowSelected = selection;
	        	
	        	if (row._painted !== null) {
		        	dojo.toggleClass(row._painted, "checked", selection); //$NON-NLS-1$
		        	row._checkbox.checked = selection;
	        	}
	        	
    			this._updateSelectAllCheckBox();
	        	
	        	return true;
        	}
        	return false;
        },

        /**
    	 * Creates a row to be inserted at the beginning of the table
    	 * 
    	 * @public
    	 * @param {Object} An object which contains the following row
    	 * 		  information
    	 * @param {String} arguments.id a unique identifier for the row, user is 
    	 * 		  responsible for making sure this id is unique.
    	 * @param {String} arguments.className the class for this row 
         * @param {Object} arguments.callbackData the variable which will be
         * 		  returned on call back
         * @param {Object[]} arguments.columnsData an array of all the columns
         * @param {Object} arguments.columnsData[].column The data to be
         * 		  inserted into the column, if null will look at
         * 		  arguments.columnsData[].paintFunction
         * @param {Function} arguments.columnsData[].paintFunction Creates the
         * 		  column data from callback data, if null will look at the
         * 		  this.columns[].paintFunction
         * @param {String} arguments.filterableData A string containing all the
         * 		  data to filter the row on
         * @param {Boolean} arguments.rowSelected determines if this row should 
         * 		  be in the selected state
    	 * @returns {String} the row's ID
    	 * @throws {Error} ID already exists
    	 */
        _createRow: function(arguments){
        	//cannot create a row that already exists
        	if (this._rowsById[arguments.id] != null) {
        		throw new Error(this.messages.SelectableStyledTable_ErrorRepeatID);
        	}
        	
        	var selected = arguments.rowSelected;
        	arguments._painted = null;
        	arguments.rowSelected = false;
        	
        	if (!arguments.columnsData) {
        		arguments.columnsData = [];
        		for (var i = 1; i < this.columns.length; i++) {
					arguments.columnsData.push({_painted: null});
				}
        	} else {
	        	for (var i = 0; i < this.columns.length - 1; i++) {
					arguments.columnsData[i]._painted = null;
				}
			}
        	
        	this._selectRow(arguments, selected === true);
        	
			//return the current row
			return arguments;
        },
        
         /**
    	 * Repaints a column for a specific row
    	 * 
    	 * @public
    	 * @param {String} rowId The id of the row to paint
    	 * @param {Integer} columnNum Which column should be repainted
    	 * @throws {Error} ID not found
    	 */ 
        repaintColumn: function(rowId, columnNum) {
        	var row = this._rowsById[rowId];
        	if (row == undefined) {
        		throw new Error(this.messages.SelectableStyledTable_NoID);
        	}
        	if (row.columnsData[columnNum]._painted) {
        		this._paintColumn(row, columnNum);
        	}
        },
        
         /**
    	 * Repaints a column for a specific row
    	 * 
    	 * @private
    	 * @param {Object} row The row to change
    	 * @param {Integer} columnNum Which column should be repainted
    	 */ 
        _paintColumn: function(row, columnNum) {
        	//create column
			var column = row.columnsData[columnNum].column;
			if (!column) {
				var columnPaintFunction = row.columnsData[columnNum].paintFunction || this.columns[columnNum + 1].paintFunction;
				column = columnPaintFunction(row.callbackData);
			}
			
			//paint column
			td = document.createElement("td"); //$NON-NLS-1$
			td.appendChild(column);
			row.columnsData[columnNum]._painted = td;
			
			//+1 is because of the checkbox column
			if (columnNum + 1 < row._painted.children.length) {
				var previousColumn = row._painted.childNodes[columnNum + 1];
				row._painted.insertBefore(td, row._painted.children[columnNum + 1]);
				row._painted.removeChild(previousColumn);
			} else {
				row._painted.appendChild(td);	
			}
        },
        
         /**
    	 * Takes a row and creates all the requirements for display
    	 * 
    	 * @private
    	 * @param {Object} row A pre-painted row
    	 * @returns {Object} the row to be inserted into the table
    	 */ 
        _paintRow: function(row){      	        	
        	//initialization of the toolbar
        	if (row._toolbar) {
				row._toolbar.destroyRecursive();
				row._toolbar = null;
			}
       		row._toolbar = new com.ibm.team.process.web.ui.internal.TRToolbar();//$NON-NLS-1$
       		
       		if (row._button) {
				row._button.destroyRecursive();
				row._button = null;
			}
       		
       		var button = {};
       		for (var property in this.button) {
            	button[property] = this.button[property];
            }
            button.menuProvider = dojo.hitch(this, function(){return this._handlePopupMenuAction(row);});
       		
       		row._button = new jazz.ui.toolbar.MenuButton(button);
			row._toolbar.addChild(row._button);
			
        	//initialization of the checkbox
        	row._checkbox = document.createElement("input"); //$NON-NLS-1$
        	row._checkbox.type = "checkbox"; //$NON-NLS-1$
        	this.connect(row._checkbox, "onclick", "_handleRowCheckboxClick"); //$NON-NLS-1$ //$NON-NLS-2$
			this.connect(row._checkbox, "onfocus", "_handleFocus"); //$NON-NLS-1$ //$NON-NLS-2$
			this.connect(row._checkbox, "onblur", "_handleBlur"); //$NON-NLS-1$ //$NON-NLS-2$
			
			//initialization of row
        	row._painted = document.createElement("tr"); //$NON-NLS-1$
        	if (row.className) {
        		dojo.addClass(row._painted, row.className); //$NON-NLS-1$
        	}
			
			//begin the creation of the checkbox column
			var td = document.createElement("td"); //$NON-NLS-1$
			dojo.addClass(td, "com-ibm-team-process-web-ui-internal-SelectableStyledTable-checkbox", true); //$NON-NLS-1$
			td.appendChild(row._checkbox);
			td.appendChild(row._toolbar.domNode);
			row._painted.appendChild(td);
			
			//create the rest of the columns
			var column;
			var columnPaintFunction;
			for (var i = 0; i < row.columnsData.length; i++){
				//create column
				if (row.columnsData[i]._painted === null) {
					this._paintColumn(row, i);
				} else {
					row._painted.appendChild(row.columnsData[i]._painted);
				}
			}
			
			if (row.rowSelected) {
	        	dojo.toggleClass(row._painted, "checked", row.rowSelected); //$NON-NLS-1$
	        	row._checkbox.checked = row.rowSelected;
        	}
			
			this.connect(row._painted, "onmouseout", "_handleBlur"); //$NON-NLS-1$ //$NON-NLS-2$
			row._painted.id = row.id;
			
			return row;
        },
        
        /**
    	 * Creates a row to be inserted at the beginning of the table
    	 * 
    	 * @public
    	 * @param {Object} An object which contains the following row
    	 * 		  information
    	 * @param {String} arguments.id a unique identifier for the row, user is 
    	 * 		  responsible for making sure this id is unique.
    	 * @param {String} arguments.className the class for this row 
         * @param {Object} arguments.callbackData the variable which will be
         * 		  returned on call back
         * @param {Object[]} arguments.columnsData an array of all the columns
         * @param {Object} arguments.columnsData[].column The data to be
         * 		  inserted into the column, if null will look at
         * 		  arguments.columnsData[].paintFunction
         * @param {Function} arguments.columnsData[].paintFunction Creates the
         * 		  column data from callback data, if null will look at the
         * 		  this.columns[].paintFunction
         * @param {String} arguments.filterableData A string containing all the
         * 		  data to filter the row on
         * @param {Boolean} arguments.rowSelected determines if this row should 
         * 		  be in the selected state
    	 * @returns {String} the row's ID
    	 * @throws {Error} ID already exists
    	 */ 
        insertRowAtFront: function(arguments){
        	//create the row
        	var row = this._createRow(arguments);
        	
			//append to the end of the table
        	row.position = 0;
        	this._rowsByOrder.unshift(row);
        	this._rowsById[row.id] = row;
        	
        	//update the layout of the table
        	this._updateOrder(0);
        	this._updateTable();
        	
        	//Update Selection checkbox
        	this._updateSelectAllCheckBox();
        	
        	//return the row
        	return row.id;
        },
               
        /**
    	 * Creates a row to be inserted at the end of the table
    	 * 
    	 * @public
    	 * @param {Object} An object which contains the following row
    	 * 		  information
    	 * @param {String} arguments.id a unique identifier for the row, user is 
    	 * 		  responsible for making sure this id is unique.
    	 * @param {String} arguments.className the class for this row 
         * @param {Object} arguments.callbackData the variable which will be
         * 		  returned on call back
         * @param {Object[]} arguments.columnsData an array of all the columns
         * @param {Object} arguments.columnsData[].column The data to be
         * 		  inserted into the column, if null will look at
         * 		  arguments.columnsData[].paintFunction
         * @param {Function} arguments.columnsData[].paintFunction Creates the
         * 		  column data from callback data, if null will look at the
         * 		  this.columns[].paintFunction
         * @param {String} arguments.filterableData A string containing all the
         * 		  data to filter the row on
         * @param {Boolean} arguments.rowSelected determines if this row should
         * 		  be in the selected state
    	 * @returns {String} the row's ID
    	 * @throws {Error} ID already exists
    	 */ 
        insertRowAtEnd: function(arguments){
        	//create the row
        	var row = this._createRow(arguments);
        	
			//append to the end of the table
        	row.position = this._rowsByOrder.push(row) - 1;
        	this._rowsById[row.id] = row;
        	
        	//update the layout of the table
        	this._updateTable();
        	
        	//Update Selection checkbox
        	this._updateSelectAllCheckBox();
        	
        	//return the row
        	return row.id;
        },
        
        /**
    	 * Deletes a row in the table
    	 * 
    	 * @public
    	 * @param {String} rowId The ID of the row to be deleted
    	 * @returns {Object} The callback information of the row
    	 * @throws {Error} ID not found
    	 */ 
        remove: function(rowId){
        	//find row to remove
        	var removeRow = this._rowsById[rowId];
        	if (removeRow == undefined) {
        		throw new Error(this.messages.SelectableStyledTable_NoID);
        	}
        	
        	return this._remove(removeRow);
        },
        
        /**
    	 * Deletes a row in the table
    	 * 
    	 * @private
    	 * @param {Object} row The row to be deleted
    	 * @returns {Object} The callback information of the row
    	 */ 
        _remove: function(row){
        	//row the row from the IDs
        	this._rowsByOrder.splice(row.position, 1);
        	
        	if (this._lastFocusedRow != null && this._lastFocusedRow.id == row.id) {
        		this._lastFocusedRow = null;
        	}
        	
        	if (row.rowSelected) {
        		delete this._rowsSelectedById[row.id];
        		this._selectedCount--;
        	}
        	
        	//reset the select all checkbox, because now none may be selected
        	this._updateSelectAllCheckBox();
        	
        	//remove the row
        	delete this._rowsById[row.id];
        	
        	if (row._toolbar) {
				row._toolbar.destroyRecursive();
				row._toolbar = null;
			}
        	if (row._button) {
				row._button.destroyRecursive();
				row._button = null;
			}
        	
        	//update the layout of the table
        	this._updateOrder(row.position);
        	this._updateTable();
        	
        	//return the arguments
        	return row.callbackData;
        },
        
        /**
    	 * Removes all the rows from the table
    	 * 
    	 * @public
    	 */ 
        clear: function(){
        	var previousUpdateValue = this._suspendUpdate;
        	this._suspendUpdate = true;
        	for (var i = 0; i < this._rowsByOrder.length; i++) {
        		this._remove(this._rowsByOrder[i]);
        	}
        	
        	this._intializeVariables();
        	
        	this._suspendUpdate = previousUpdateValue;
        	this._updateTable();
        },
        
        /**
    	 * Determines if the table is empty
    	 * 
    	 * @public
    	 * @returns {Boolean} True if the table is empty, false if the table is 
    	 * 		  not empty
    	 */ 
        empty: function(){
        	return this._rowsByOrder.length < 1;
        },
        
        /**
    	 * Updates the order of the table
         * (the .position elements attached to each row)
    	 * 
    	 * @private
    	 * @param {Positive_Integer} startPosition the position in which to
    	 * 		  start the update
    	 */ 
        _updateOrder: function(startPosition){
        	for (var i = startPosition; i < this._rowsByOrder.length; i++) {
        		this._rowsByOrder[i].position = i;
        	}
        },
        
        /**
    	 * Changes the number of elements to be displayed and the starting
    	 * position of the elements to be displayed
    	 * 
    	 * @public
    	 * @param {Positive_Integer} startPosition the first element to be shown 
    	 * 		  in the table
    	 * @param {Integer} length the number of elements to be shown in the
    	 * 		  table, negative numbers mean to evaluate to end of the array
    	 */ 
        changeTablePosition: function(startPosition, length){
        	this._start = startPosition;
        	this._length = length !== undefined ? length : -1;
        	this._updateTable();
        },
        
        /**
    	 * This function allows an external person to prevent the table from
         * being updated until they are ready for the table to be updated
    	 * 
    	 * @public
    	 * @param {Boolean} suspendUpdate true will prevent the table from
    	 * update its display elements false will allow the table to update its
    	 * display elements, and the table will immediately update its elements
    	 * when this function is called
    	 */ 
        suspendTableUpdate: function(suspendUpdate){
        	this._suspendUpdate = suspendUpdate;
        	
        	if (this._tableNeedsUpdate) {
        		this._updateTable();
        	}
        },
        
        /**
    	 * A function which handles when the filter's value is changed
    	 * 
    	 * @private
    	 * @param {String} value The new value of the filter
    	 */ 
         _onFilterChange: function(value) {
         	if (this._filterTimeout) {
         		clearTimeout(this._filterTimeout);
         		this._filterTimeout = null;
         	}
         	
         	this._filterTimeout = setTimeout(dojo.hitch(this, this._onFilterTimeout), this._fitlerWaitTime);
        },
        
         /**
    	 * A function which handles when the table should be updated for the filter
    	 * 
    	 * @private
    	 */ 
        _onFilterTimeout: function() {
        	if (this._previousFilter !== this._filterBox.value) {
        		this._previousFilter = this._filterBox.value;
	        	clearTimeout(this._filterTimeout);
	        	this._filterTimeout = null;
	        	
	        	this._updateTable();
	        	this.onFilterChange(this._filterBox.value);
        	}
        },
        
         /**
    	 * A function which returns the current value of the filter
    	 * 
    	 * @public
    	 * @returns {String} The value of the filter
    	 */ 
        getFilterValue: function() {
        	return this._filterBox.value;
        },
        
         /**
    	 * A function which sets the current value of the filter
    	 * 
    	 * @public
    	 * @param {String} value The value to set the filter as
    	 */ 
        setFilterValue: function(value) {
        	if (this._filterTimeout) {
        		clearTimeout(this._filterTimeout);
	        	this._filterTimeout = null;
	        }
        	
        	this._previousFilter = value;
        	this._filterBox.setValue(value);
	        this._updateTable();
	        this.onFilterChange(this._filterBox.value);
        },
        
        /**
    	 * An extendable function which is called after a filter is changed
    	 * 
    	 * @private
    	 * @param {String} value The new value of the filter
    	 */ 
        onFilterChange: function(value) {},
        
        /**
    	 * Updates the _rowsFilteredByOrder to the newest filter
    	 * 
    	 * @private
    	 */ 
        _updateFilteredArray: function() {
        	if (this._filterBox && this._filterBox.value !== "") { //$NON-NLS-1$
        		var filterStringArray = this._filterBox.value.split(this.filterSplitter);  		
        		this._rowsFilteredByOrder = dojo.filter(this._rowsByOrder, function(row) {
        			return this._checkFilter(row, filterStringArray);
        		}, this);
        	} else {
        		this._rowsFilteredByOrder = this._rowsByOrder;
        	}
        },
        
        /**
    	 * An function which checks to see if the row should be included in the 
    	 * current filter
    	 * 
    	 * @private
    	 * @param {Object} row the callback data of the row that's selection is 
    	 * 		  changing
    	 * @param {String[]} filterStringArray An array of strings to check
    	 * @returns {Boolean} true allows the row to be added to the table,
    	 * 		  false does not
    	 */ 
        _checkFilter: function(row, filterStringArray){
    		for (var i = 0; i < filterStringArray.length; i++) {
    			if (!row.filterableData || row.filterableData.indexOf(filterStringArray[i]) === -1) {
    				return false;
    			}
    		}
    		return true;
        },
        
        /**
    	 * A function which updates the filterableData to a specific row
    	 * 
    	 * @public
    	 * @param {String} rowId The id of the row to update
    	 * @param {String} newFilterData A string of the new filterable data
    	 * @throws {Error} ID not found
    	 */ 
        updateFilterData: function(rowId, newFilterData){
        	var row = this._rowsById[rowId];
        	if (row == undefined) {
        		throw new Error(this.messages.SelectableStyledTable_NoID);
        	}
        	row.filterableData = newFilterData;
        	
        	this._updateTable();
        },
        
        /**
    	 * Adds a row to be displayed
    	 * 
    	 * @private
    	 */ 
        _displayRow: function(row) {
        	if (row._painted === null) {
				this._paintRow(row);
			}
    		this.tbody.appendChild(row._painted);
    		this._rowsDisplayedByOrder.push(row);
        },
        
        /**
    	 * This function update what the table is currently displaying
    	 * 
    	 * @private
    	 */ 
        _updateTable: function(){
        	if (!this._suspendUpdate) {
        		this._tableNeedsUpdate = false;
        		this._rowsDisplayedByOrder = [];
        		
	        	//clear all of tbody
	        	this.clearBody();
	        	this._updateFilteredArray();
	        
	        	//update table contents
	        	if (this._length < 0) {
	        		for (var i = this._start; i < this._rowsFilteredByOrder.length; i++) {
	        			this._displayRow(this._rowsFilteredByOrder[i]);
	        		}
	        	} else {
	        		for (var i = this._start; i < this._length + this._start && i < this._rowsFilteredByOrder.length; i++) {
	            		this._displayRow(this._rowsFilteredByOrder[i]);
	            	}
	        	}
        	} else {
        		this._tableNeedsUpdate = true;
        	}
        },
        
        /**
    	 * This function is responsible for clearing the dom.
    	 * This function overwrites jazz.ui.StyledTable.clearBody because the
    	 * table is no longer directly attached to the domNode, it is a little
    	 * deeper. Code changed (this.domNode -> this.table);
    	 * 
    	 * @private
    	 */ 
        clearBody: function() {
			// summary:
			//		Removes all the contents of the table's body.
			if (dojo.isIE) {
				// replace the tbody and update the reference
				var newBody = document.createElement("tbody"); //$NON-NLS-1$
				newBody.className = this.tbody.className;
				this.table.replaceChild(newBody, this.tbody);
				this.tbody = newBody;
			} else {
				this.tbody.innerHTML = ""; //$NON-NLS-1$
			}
		},

        /**
    	 * This function gets a row from an event
    	 * 
    	 * @private
    	 * @param {Object} The event data
    	 * @returns {Object} The row data or null if it could not find the row
    	 */ 
        _getRow: function(evt){
            var e = evt.target;
            while (e.parentNode) {
                if (e.tagName === "TR") { //$NON-NLS-1$
                    return e;
                }
                e = e.parentNode;
            }
            return null;
        },
        
        /**
    	 * Sets a checkbox to one of three states, checked, unchecked, inbetween
    	 * 
    	 * @private
    	 * @param {Object} checkBox the checkbox to be effected
    	 * @param {Nullable_Boolean} value the value to set the checkbox (null for inbetween)
    	 */ 
        _setCheckBox: function(checkBox, value){
			dojo.toggleClass(checkBox.domNode, "checked", value === true); //$NON-NLS-1$
			dojo.toggleClass(checkBox.domNode, "unchecked", value === false); //$NON-NLS-1$
			dojo.toggleClass(checkBox.domNode, "undetermined", value === null); //$NON-NLS-1$
			
			checkBox.checked = value !== false;
			checkBox.value = value === true ? "checked" : "undetermined"; //$NON-NLS-1$ //$NON-NLS-2$
        },
        
        /**
    	 * This function handles the dropdown menu for the select all checkbox
    	 * 
    	 * @private
    	 * @param {Object} event the event variable
    	 * @returns {Object} the menu to be displayed
    	 */ 
        _handleSelectAllPopupMenuAction: function(event){
        	//create the menu items
        	if (this._selectAllPopUpMenu == null) {
				this._selectAllPopUpMenuChildren = this._createMenuChildren(this.selectAllMenu, this);
				
				this._selectAllPopUpMenu = new jazz.ui.Menu();
				
				//This is needed because the menu element is not part of the main table
				dojo.addClass(this._selectAllPopUpMenu.domNode, "com-ibm-team-process-web-ui-internal-SelectableStyledTable"); //$NON-NLS-1$
				
				for (var i = 0; i < this.selectAllMenu.length; i++) {
					this._selectAllPopUpMenu.addChild(this._selectAllPopUpMenuChildren[i]);
				}
			} else {
				this._updateChildren(this.selectAllMenu, this._selectAllPopUpMenuChildren, this);
			}
        	
			return this._selectAllPopUpMenu;
        },
        
        /**
    	 * Handles a Popup Menu Close Event on each row's action menu
    	 * 
    	 * @private
    	 */ 
        _handlePopupMenuClose: function() {
        	if (this._lastFocusedRow != null){
        		dojo.toggleClass(this._lastFocusedRow._painted, "menu-open", false); //$NON-NLS-1$
        	}
        },
        
        /**
         * Updates a dijit.MenuItem for a jazz.ui.Menu
         * 
         * @private
         * @param {Object} child The child element to be updated
    	 * @param {Object} childInfo What to fill the child with
    	 * @param {Object} callbackData The data to send to any menu item 
    	 * functions
    	 * @returns {Object} The updated menu child element
         */
        _updateMenuChild: function(child, childInfo, callbackData) {
        	if (childInfo.isDivider !== true) {
				//Check if label is a function, if not assume string
				if (childInfo.label instanceof Function) { //$NON-NLS-1$
					child.set('label', childInfo.label(callbackData)); //$NON-NLS-1$
				} else {
					child.set('label', dojo.string.substitute(childInfo.label, [this.getSelectedCount()])); //$NON-NLS-1$
				}
				
				//Check if title is a function, if not assume string
				if (childInfo.title instanceof Function) { //$NON-NLS-1$
					child.set('title', childInfo.title(callbackData)); //$NON-NLS-1$
				} else {
					child.set('title', dojo.string.substitute(childInfo.title, [this.getSelectedCount()])); //$NON-NLS-1$
				}
			}
			
			//if no isEnabled function, assume always true
			var isEnabled = childInfo.isEnabled === undefined ? true : childInfo.isEnabled(callbackData);
			child.set('disabled', !isEnabled); //$NON-NLS-1$
			
			return child;
       },
        
       /**
         * Creates a new list of dijit.MenuItem/dijit.MenuSeparator from the
         * menu info
         * 
         * @private
         * @param {Object} menu The list of menu item data
    	 * @param {Object} callbackData The data to send to any menu item 
    	 * functions
    	 * @returns {Object} A list of child elements
         */
       _createMenuChildren: function(menu, callbackData) {
        	//populate the menu items
			var child;
			var menuChildren = [];
			for (var i = 0; i < menu.length; i++) {
				if (menu[i].isDivider === true) {
					child = new dijit.MenuSeparator();
				} else {
					child = new dijit.MenuItem(menu[i]);
				}
				
				menuChildren.push(this._updateMenuChild(child, menu[i], callbackData));
			}
			return menuChildren;
        },
        
        /**
         * Updates a list of dijit.MenuItem/dijit.MenuSeparator from the menu 
         * info
         * 
         * @private
         * @param {Object} menu The list of menu item data
         * @param {Object} menuChildren The list of menu children to update
    	 * @param {Object} callbackData The data to send to any menu item 
    	 * functions
         */
        _updateChildren: function(menu, menuChildren, callbackData) {
        	for (var i = 0; i < menu.length; i++) {
				this._updateMenuChild(menuChildren[i], menu[i], callbackData);
			}
        },
        
        /**
    	 * This function handles the dropdown menu
    	 * 
    	 * @private
    	 * @param {Object} row the row element
    	 * @returns {Object} the menu to be display on success, null on failure
    	 */ 
        _handlePopupMenuAction: function(row){
        	if (!this.onSelectionChange(row.callbackData)) {
        		return null;
        	}
        	
        	//set the last focused row
        	this._lastFocusedRow = row;
        	dojo.toggleClass(row._painted, "menu-open", true); //$NON-NLS-1$
        	
        	//create the menu items
        	if (this._rowMenu == null) {
				this._rowMenuChildren = this._createMenuChildren(this.menu, row.callbackData);
				
				this._rowMenu = new jazz.ui.Menu();
				this.connect(this._rowMenu, "onClose", "_handlePopupMenuClose"); //$NON-NLS-1$ //$NON-NLS-2$
				
				//This is needed because the menu element is not part of the main table
				dojo.addClass(this._rowMenu.domNode, "com-ibm-team-process-web-ui-internal-SelectableStyledTable"); //$NON-NLS-1$
				
				for (var i = 0; i < this.menu.length; i++) {
					this._rowMenu.addChild(this._rowMenuChildren[i]);
				}
			} else {
				this._updateChildren(this.menu, this._rowMenuChildren, row.callbackData);
			}
        	
			return this._rowMenu;
        },
        
        /**
    	 * This function handles when the row obtains focus
    	 * 
    	 * @private
    	 * @param {Object} e the event variable
    	 */ 
        _handleFocus: function(e){
        	var row = this._getRow(e);
        	dojo.toggleClass(row, "selected", true); //$NON-NLS-1$
        },
        
        /**
    	 * This function handles when the row looses focus
    	 * 
    	 * @private
    	 * @param {Object} e the event variable
    	 */ 
        _handleBlur: function(e){
        	var row = this._getRow(e);
        	dojo.toggleClass(row, "selected", false); //$NON-NLS-1$
        },
        
        /**
    	 * This function handles when the checkbox is clicked
    	 * 
    	 * @private
    	 * @param {Object} e the event variable
    	 */ 
        _handleRowCheckboxClick: function(e){
        	var row = this._getRow(e);
        	this._selectRow(this._rowsById[row.id], e.target.checked);
        },
        
        /**
    	 * This function handles when the select all checkbox is clicked
    	 * 
    	 * @private
    	 * @param {Object} e the event variable
    	 */ 
        _handleSelectAllCheckboxClick: function(e){
        	this.selectAll(e.target.checked);
        },
        
        /**
    	 * This function is responsible for destroying this object
    	 */ 
		uninitialize: function(){
			this.clear();
			
        	delete this._rowsById;
        	delete this._rowsByOrder;
        	delete this._rowsDisplayedByOrder;
        	delete this._rowsFilteredByOrder;
        	delete this._rowsSelectedById;
        	
        	if (this._lastFocusedRow != null) {
        		delete this._lastFocusedRow;
        	}
        	
        	if (this._rowMenu != null) {
        		for (var i = 0; i < this._rowMenuChildren.length; i++) {
					if (this._rowMenuChildren[i]) {
						this._rowMenuChildren[i].destroyRecursive();
						delete this._rowMenuChildren[i];
					}
				}
        		delete this._rowMenuChildren;
        		
        		if (this._rowMenu) {
					this._rowMenu.destroyRecursive();
					delete this._rowMenu;
				}
        	}
        	
        	if (this._selectAllPopUpMenu != null) {
        		for (var i = 0; i < this._selectAllPopUpMenuChildren.length; i++) {
					if (this._selectAllPopUpMenuChildren[i]) {
						this._selectAllPopUpMenuChildren[i].destroyRecursive();
						delete this._selectAllPopUpMenuChildren[i];
					}
				}
        		delete this._selectAllPopUpMenuChildren;
        		
        		if (this._selectAllPopUpMenu) {
					this._selectAllPopUpMenu.destroyRecursive();
					delete this._selectAllPopUpMenu;
				}
        	}
        	
			if (this._selectAllCheckBox) {
	        	if (this._selectAllCheckBox._button) {
					this._selectAllCheckBox._button.destroyRecursive();
				}
				this._selectAllCheckBox.destroyRecursive();
				delete this._selectAllCheckBox;
			}
			
			if (this._filterBox) {
        		this._filterBox.destroyRecursive();
        		delete this._filterBox;
        	}
        	
			this.inherited(arguments);
		}
    });
})();
