/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007, 2012. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.dashboard.viewlets.web.ui.internal.StaticViewlet");

/*
 * Imports 
 */
dojo.require("dojo.i18n"); 
dojo.require("com.ibm.team.dashboard.web.ui.Viewlet");
dojo.require("com.ibm.team.dashboard.viewlets.web.ui.internal.utils.JPIUtil");
dojo.require("jazz.util.i18n");
dojo.require("jazz.util.html");
dojo.require("jazz.core.loader");

dojo.requireLocalization("com.ibm.team.dashboard.viewlets.web.ui.internal", "StaticMessages");

(function() {

/*
 * Aliases 
 */
var JPIUtil = com.ibm.team.dashboard.viewlets.web.ui.internal.utils.JPIUtil;
var Viewlet = com.ibm.team.dashboard.web.ui.Viewlet;
var I18nUtil = jazz.util.i18n;

/*
 * Constants 
 */
var RICH_TEXT_EDITOR_ID = "sh_rte_";
var STATE_HTML = "html";

/*
 * TODOs
 */

/*
 * Class 
 */
dojo.declare("com.ibm.team.dashboard.viewlets.web.ui.internal.StaticViewlet",
	Viewlet, {

	// Properties //----------------------------------------------------------->

	// ------------------------------------------------------------------------>

	/*
	 * Templates
	 */		
	templatePath: dojo.moduleUrl("com.ibm.team.dashboard.viewlets.web", "ui/internal/templates/StaticViewlet.html"),

	/*
 	 * @param Object args
 	 * 
 	 * @return StaticViewlet
 	 */
	constructor: function(args) {
		
		// Messages //------------------------------------------------------------->
		
		this.messages = dojo.i18n.getLocalization("com.ibm.team.dashboard.viewlets.web.ui.internal", "StaticMessages");
		
	},
		
	/*
 	 * @param void
 	 * 
 	 * @overrides super
 	 * @return void
 	 */	
	init: function() {

		this.inherited(arguments);
		
		// Instance Variables //--------------------------------------------------->

		this._editor = null;
		this._defaultHTMLContent = this.messages.msgDefineContentHere;
									
		// Initialization //--------------------------------------------------->
		
		this._html = this.getViewletState().get(STATE_HTML);
		if (this._html) {
			// sanitize
			this._html = jazz.util.html.removeScript(this._html);
			this._html = this._removeContentPadding(this._html);
			
			// convert JPIs to URLs
			var _this = this;
			var site = this.getSite();
			var dfd = JPIUtil.convertURIs([], [this._html], site.jpisToURLs);
			dfd.addCallback(function(htmlSnippets) {
				_this._html = htmlSnippets[0];
				_this._loadDisplayArea();
			});
			dfd.addErrback(dojo.hitch(site, site.error));
		}
		else {
			this._loadDisplayArea();
		}

	},	
	
	/*
 	 * @param void
 	 * 
 	 * @return boolean
	 * @overrides super
 	 */		
	isEditing: function() {
		// prevent the user from navigating away while it's in the process of
		// saving the state but not done yet
		return this._isEditing || this._isSavingState;
	},
	
	/*
 	 * @param boolean
 	 * 
 	 * @return void
	 * @overrides super
 	 */		
	setEditing: function(/* boolean */ isEditing) {
		this._isEditing = isEditing;
		this.inherited(arguments);
	},
	
	/*
 	 * @param void
 	 * 
 	 * @return boolean
	 * @overrides super
 	 */		
	validateEditedContent: function() {
		// Get the edited contents
		if (this._editor) {
			var html = this._editor.get("value");
			// Check for insecure content
			if (jazz.util.html.hasScript(html)) {
				this._alertErrorMessage(this.messages.errorScript,
						this.messages.errorScript_explanation,
						this.messages.errorScript_useraction);
				return false;
			}			
		}
		return true;
	},		
	
	/*
 	 * @param String message
 	 * @param String explanation
 	 * @param String useraction
 	 * 
 	 * @return void
 	 */	
	_alertErrorMessage: function(message, explanation, useraction) {
		var text = this._getOperationalErrorText(message, explanation, useraction);
		alert(text);		
	},		
	
	/*
 	 * @param String message
 	 * @param String explanation
 	 * @param String useraction
 	 * 
 	 * @return String
 	 */	
	_getOperationalErrorText: function(message, explanation, useraction) {
		var text = message + " " +
			explanation + " " +
			useraction;
		return text;
	},		
	
	/*
 	 * @param void
 	 * 
 	 * @return void
 	 * @overrides super
 	 */		
	preSaveState: function() {
		if (!this._editor) return;
		// Save button was pressed in edit mode
		// Update the display HTML using the contents of the editor
		this._html = this._editor.get("value");
		
		// Convert URLs to JPIs before saving
		var _this = this;
		var site = this.getSite();
		this._isSavingState = true;
		var dfd = JPIUtil.convertURIs([], [this._html], site.urlsToJPIs);
		dfd.addCallback(function(htmlSnippets) {
			// Save the JPI-converted HTML
			_this.getViewletState().set(STATE_HTML, htmlSnippets[0]);
		});
		dfd.addErrback(dojo.hitch(site, site.error));
		dfd.addBoth(function(arg) {
			_this._isSavingState = false;
			return arg;
		});
	},	
	
	/*
 	 * @param void
 	 * 
 	 * @return void
 	 * @overrides super
 	 */		
	update: function() {
		// Determine whether we are entering or leaving edit mode
		if (this._isEditing) {
			// Enter
			// Load the editor area
			this._loadEditorArea();
		} else {
			// Leave
			// Load the display HTML
			this._loadDisplayArea();
		}
	},
	
	/*
 	 * @param void
 	 * 
 	 * @return void
 	 */	
	_loadEditorArea: function() {

		// Create the Editor
		this._createEditor().addCallback(dojo.hitch(this,function() {
			dojo.removeClass(this._editorArea, "j-hint");   
			this._editor.set("value", this._html||this._defaultHTMLContent);
		}));
		
		this._updateVisibility(this._displayArea, false);
		this._updateVisibility(this._editorArea, true);
	},		

	/*
 	 * @param String html
 	 * 
 	 * @return String
 	 */          
	_sanitizeContent: function(html) {
		var sanitizedHTML = jazz.util.html.removeScript(html);
		return sanitizedHTML;
	},		
	
	/*
 	 * @param String html
 	 * 
 	 * @return String
 	 */          
	_removeContentPadding: function(html) {
		// See: 215850: [HTML Viewlet][Headlines Viewlet]: Ordered / unordered list formatting is broken on IE9
		// Before: <div style="margin:5px;">
		// After:  <div>
		var regex = /^<div style="margin:\s*5px;">/;
		return html.replace(regex, "<div>");
	},		
	
	/*
 	 * @param void
 	 * 
 	 * @return void
 	 */          
	_createEditor: function() {
		var deferred = new dojo.Deferred();
		if (this._editor) {
			deferred.callback();
		} else {
			jazz.core.loader.batch_load_async([
				"jazz.ui.Editor",
				"dijit._editor.plugins.TextColor",
				"dijit._editor.plugins.LinkDialog",
				"dijit._editor.plugins.ViewSource",
				"dijit._editor.plugins.FullScreen",
				"dojox.editor.plugins.PrettyPrint",
				"dijit._editor.plugins.ToggleDir"
			], dojo.hitch(this, function() {
				if (!this._isEditing) return;
				this._editor = new jazz.ui.Editor({
					id: this._getRichTextEditorID(),
					height: "200px",
					plugins: [],
					contentPostFilters: [this._sanitizeContent],
					contentPreFilters: [this._sanitizeContent],
					extraPlugins: [
						{
							name: 'viewsource'
						},
						{
							name: 'fullscreen',
							zIndex: 999
						},
						{
							name: '|'
						},
						{
							name: 'bold'
						},
						{
							name: 'italic'
						},
						{
							name: 'underline'
						},
						{
							name: '|'
						},
						{
							name: 'foreColor'
						},
						{
							name: 'hiliteColor'
						},
						{
							name: '|'
						},
						{
							name: 'justifyLeft'
						},
						{
							name: 'justifyCenter'
						},
						{
							name: 'justifyRight'
						},
						{
							name: '|'
						},
						{
							name: 'insertOrderedList'
						},
						{
							name: 'insertUnorderedList'
						},
						{
							name: '|'
						},
						{
							name: 'indent'
						},
						{
							name: 'outdent'
						},
						{
							name: '|'
						},
						{
							name: 'createLink'
						},
						{
							name: 'insertImage'
						},
						{
							name: 'prettyprint'
						},
						{
							name: '|'
						},
						{
							name: 'toggleDir'
						}
					]
		 		}, this._editorArea);
				deferred.callback();
			}));
		} 
		return deferred;
	},

	/*
 	 * @param void
 	 * 
 	 * @return String
 	 */	
	_getRichTextEditorID: function() {
		return RICH_TEXT_EDITOR_ID + this.id;	
	},	
	
	/*
 	 * @param Element node
 	 * @param boolean show
 	 * 
 	 * @return void
 	 */	
	_updateVisibility: function(node, show) {
		if (show) {
			node.style.display = "block";
		} else {
			node.style.display = "none";
		}
	},		

	/*
 	 * @param void
 	 * 
 	 * @return void
 	 */	
	_loadDisplayArea: function() {
		
		// clear loading msg if still there
		if (this._loadingDiv) {
			this.domNode.removeChild(this._loadingDiv);
			delete this._loadingDiv;
		}
		
		var html = this._html;
		var div;

		if (!html) {
			div = this._getActionEditContents();			
		} else {
			div = dojo.create("div", {innerHTML: html});
		}

		this._setHTMLNode(this._displayArea, div);

		this._updateVisibility(this._editorArea, false);		
		this._updateVisibility(this._displayArea, true);

	},
	
	/*
 	 * @param void
 	 * 
 	 * @return Element
 	 */	
	_getActionEditContents: function() {

		var div = document.createElement("div");

		if (this.getSite().isDashboardEditing()) {
			// The user has permission to edit the Dashboard
			// Add action
			var links = I18nUtil.addMessage(div, this.messages.msgNoContentDefinedAction);
			var link = links[0];
			link.href = "#";
			this.connect(link, "onclick", this._handleEventOnClickEditContent);		

		} else {
			// The user does not have the permission to edit the Dashboard
			// Do not add action
			var fallbackMessage = document.createTextNode(this.messages.msgNoContentDefined);
			div.appendChild(fallbackMessage);			
		}
		
		return div;
	},	

	/*
 	 * @param Event event
 	 * 
 	 * @return void
 	 */		
	_handleEventOnClickEditContent: function(event) {
		if (event) {
			dojo.stopEvent(event);
		}
		
		this.getSite().editContents();
	},	
	
	/*
 	 * @param Element parent
 	 * @param Element child
 	 * 
 	 * @return void
 	 */		
	_setHTMLNode: function(parent, child) {

		if (parent.firstChild) {
			parent.replaceChild(child, parent.firstChild);
		} else {
			parent.appendChild(child);
		}
	},	
	
	/*
 	 * @param void
 	 * 
 	 * @return void
 	 * @overrides super
 	 */	
	refresh: function() {
		this.getSite().doneRefresh();
	},
	
	/*
 	 * @param void
 	 * 
 	 * @return void
 	 * @overrides super
 	 */		
	destroy: function() {
		
		if (this._editor) {
			this._editor.destroy();
			this._editor = null;
		}
		
		this.inherited(arguments);
	}	

});


})();

