/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2009, 2013. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.process.web.ui.internal.TRToolbar"); //$NON-NLS-1$

dojo.require("jazz.ui.Toolbar"); //$NON-NLS-1$

(function(){

	dojo.declare("com.ibm.team.process.web.ui.internal.TRToolbar", jazz.ui.Toolbar, { //$NON-NLS-1$

		selectClassName : "selected", //$NON-NLS-1$
		disabledClassName : "toolbar-disabled", //$NON-NLS-1$

		addChild: function(button){
			this.inherited(arguments);
			this._selectRowOnFocus(button);
			
			this.connect(this.domNode, "onfocus", dojo.hitch(this, function(e){ //$NON-NLS-1$
				var i;
				var row = this._getRow(this);
				var actionButtons = this.getChildren();
				this._rowSelected(row, true);
				for(i = 0; i < actionButtons.length; i++) {
					if (actionButtons[i].isFocusable()) {
						actionButtons[i].focus();
						break;
					}
				}
				if (i === actionButtons.length) {
					this._rowSelected(row, false);
				}
			}));
		},
		
		/**
		 * Disables all buttons on the toolbar by adding the class "toolbar-disabled" to the
		 * toolbar.
		 * 
		 * @param {boolean} disabled - true if the toolbar should be disabled
		 */
		disableToolbar: function(disabled) {
			if (disabled === true) {
				dojo.toggleClass(this.domNode, this.disabledClassName, true);
			} else {
				dojo.toggleClass(this.domNode, this.disabledClassName, false);
			}
		},
		
		/**
		 * A helper to make this toolbar's row 'selected' -- with an orange
		 * background and visible toolbar buttons -- when child widgets are
		 * given focus. This is essential for keyboard navigation using the Tab
		 * key. Buttons added to this toolbar via addButton() will have this
		 * behavior automatically. This function is available for widgets
		 * that are added via other API, or for elements in the row that are
		 * outside of the toolbar, but should put the row in the selected
		 * state when they are focused.
		 * 
		 * @param {DOM node} domNode - A DOM element
		 */
		_selectRowOnFocus: function(button) {
			button.connect(button.focusNode, "onfocus", dojo.hitch(this, function(e){ //$NON-NLS-1$
				this._rowSelected(this._getRow(this), true);
			}));
			button.connect(button.focusNode, "onblur", dojo.hitch(this, function(e){ //$NON-NLS-1$
				this._rowSelected(this._getRow(this), false);
			}));
		},
		
		/**
		 * Subclasses may override to get the row that the toolbar is
		 * currently in
		 */
		_getRow: function(toolbar) {
			var row = toolbar.domNode;
			while (row && row.tagName.toLowerCase() !== "tr") { //$NON-NLS-1$
				row = row.parentNode;
			}
			return row;
		},
		
		/**
		 * Subclasses may override to take action when a row is selected
		 * or de-selected.
		 */
		_rowSelected: function(row, selected) {
			if (row._blurTimeoutVariable) {
				clearTimeout(row._blurTimeoutVariable);
			}
			
			if (selected) {
				dojo.toggleClass(row, this.selectClassName, true);
			} else {
				row._blurTimeoutVariable = setTimeout(dojo.hitch(this, function(){
					dojo.toggleClass(row, this.selectClassName, false);
				}),0);
			}			
		}
	});
})();
