/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010, 2012. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  
 * Use, duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp. 
 *******************************************************************************/
(function() {
dojo.provide("com.ibm.team.repository.web.ui.internal.Util");

dojo.require("dojox.validate.regexp");
dojo.require("com.ibm.team.repository.web.client.init.InitializationData");

var InitializationData = com.ibm.team.repository.web.client.init.InitializationData;

com.ibm.team.repository.web.ui.internal.Util = {
	userIsAdmin: function() {
		var permissionInitializer = InitializationData.get("com.ibm.team.repository.service.canGrantPermissionInitializer");
		if (permissionInitializer != null && permissionInitializer.canGrantPermission) {
			return true;
		} else {
			return false;
		}
	},
	
	isAdminUI: function() {
		var jazzBaseUrl = dojo.baseUrl.replace(/[^\/]*\/[^\/]*\/$/,"");
		if (document.location.pathname.indexOf(jazzBaseUrl + "admin") >= 0) {
			return true;
		}
		else {
			return false;
		}
	},
	
	/*
	 * This support is copied from dojox.validate.regexp (regexp.js - Dojo 1.6)
	 * to add support for Arabic and Hebrew usernames
	 */	
	emailAddressRegex: function(/*Object?*/flags){
		// summary: Builds a regular expression that matches an email address
		//
		//flags: An object
		//    flags.allowCruft  Allow address like <mailto:foo@gmail.com>.  Default is false.
		//    flags in regexp.host can be applied.
		//    flags in regexp.ipAddress can be applied.

		// assign default values to missing parameters
		flags = (typeof flags == "object") ? flags : {};
		if (typeof flags.allowCruft != "boolean") { flags.allowCruft = false; }
		flags.allowPort = false; // invalid in email addresses

		// user name RE per rfc5322 
		// CUSTOM: Updated regex to support Hebrew and Arabic)
		var usernameRE = "([!#-'*+\\-\\/-9=?A-Z\\u0590-\\u05FF\\u0600-\\u06FF^-~]+[.])*"+ 
						  "[!#-'*+\\-\\/-9=?A-Z\\u0590-\\u05FF\\u0600-\\u06FF^-~]+";

		// build emailAddress RE
		var emailAddressRE = usernameRE + "@" + this.host(flags);

		// Allow email addresses with cruft
		if ( flags.allowCruft ) {
			emailAddressRE = "<?(mailto\\:)?" + emailAddressRE + ">?";
		}
		return emailAddressRE; // String
	},
	
	/* 
	 * This support is copied from dojox.validate.regex.js (regexp.js - Dojo 1.6)
	 * to add support for Arabic and Hebrew domain names	
	 */	
	host: function(/*Object?*/flags){
		// summary: Builds a RE that matches a host
		// description: A host is a named host (A-z0-9_- but not starting with -), a domain name or an IP address, possibly followed by a port number.
		// flags: An object.
		//	  flags.allowNamed Allow a named host for local networks. Default is false.
		//    flags.allowIP  Allow an IP address for hostname.  Default is true.
		//    flags.allowLocal  Allow the host to be "localhost".  Default is false.
		//    flags.allowPort  Allow a port number to be present.  Default is true.
		//    flags in regexp.ipAddress can be applied.

		// assign default values to missing paramters
		flags = (typeof flags == "object") ? flags : {};

		if(typeof flags.allowIP != "boolean"){ flags.allowIP = true; }
		if(typeof flags.allowLocal != "boolean"){ flags.allowLocal = false; }
		if(typeof flags.allowPort != "boolean"){ flags.allowPort = true; }
		if(typeof flags.allowNamed != "boolean"){ flags.allowNamed = false; }

		// Domain name labels can not end with a dash.
		// CUSTOM: New expression to allow Arabic and Hebrew (Bidi support)
		var domainLabelRE = "(?:[\\da-zA-Z\\u0590-\\u05FF\\u0600-\\u06FF](?:[-\\da-zA-Z\\u0590-\\u05FF\\u0600-\\u06FF]{0,61}[\\da-zA-Z\\u0590-\\u05FF\\u0600-\\u06FF])?)";
		var domainNameRE = "(?:[a-zA-Z\\u0590-\\u05FF\\u0600-\\u06FF](?:[-\\da-zA-Z\\u0590-\\u05FF\\u0600-\\u06FF]{0,6}[\\da-zA-Z\\u0590-\\u05FF\\u0600-\\u06FF])?)"; // restricted version to allow backwards compatibility with allowLocal, allowIP

		// port number RE
		var portRE = flags.allowPort ? "(\\:\\d+)?" : "";

		// build host RE
		var hostNameRE = "((?:" + domainLabelRE + "\\.)+" + domainNameRE + "\\.?)";
		if(flags.allowIP){ hostNameRE += "|" +  dojox.validate.regexp.ipAddress(flags); }
		if(flags.allowLocal){ hostNameRE += "|localhost"; }
		if(flags.allowNamed){ hostNameRE += "|^[^-][a-zA-Z0-9_-]*"; }
		return "(" + hostNameRE + ")" + portRE; // String
	}
};

})();