/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010, 2013. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  
 * Use, duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp. 
 *******************************************************************************/
dojo.provide("com.ibm.team.repository.web.ui.internal.admin.NotAuthorizedView");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("com.ibm.team.repository.web.client.session.Session");
dojo.require("dojo.i18n");
dojo.require("jazz.app.auth");
dojo.require("jazz.ui.ErrorPage");

dojo.requireLocalization("com.ibm.team.repository.web.ui.internal.admin", "NotAuthorizedViewMessages");

var getAuthenticatedUserId  = com.ibm.team.repository.web.client.session.getAuthenticatedUserId;

dojo.declare("com.ibm.team.repository.web.ui.internal.admin.NotAuthorizedView", [dijit._Widget, dijit._Templated], {

	templateString: "<div class=\"com-ibm-team-repository-admin-noAuth\" dojoAttachPoint=\"notAuthArea\"></div>",
	
	// custom messages to add extra details
	customNoUserMessage: null,
	customNotAuthorizedMessage: null,
	
	// URL to load after the "Try Again" link is clicked and the logout succeeds. (optional, default is to refresh current page)
	redirectUrl: null,
	
	//Specify whether a "Go back to the previous page" action should be included. (ErrorPage API)
	includeBackLink: true,
	
	constructor: function() {
		this.messages = dojo.i18n.getLocalization("com.ibm.team.repository.web.ui.internal.admin", "NotAuthorizedViewMessages");
		if (!this.applicationName) { 
			this.applicationName = this.messages.application;
		}
		this._errorArgs = {};
	},

	postCreate: function() {
		var loginUser = getAuthenticatedUserId();

		if (loginUser == "UNKNOWN") {
			// Inform the user that their ID is unknown.
			// Then either use a generic message informing them that they
			// are not authorized or use a custom message if it was defined as a parameter
			this._errorArgs.summary = this.messages.unknownUser;
			this._errorArgs.details = (this.customNoUserMessage || dojo.string.substitute(this.messages.noUser, [ this.applicationName ]));
		
		} else {
			var userFormated = "<strong>" + getAuthenticatedUserId() + "</strong>";
			this._errorArgs.summary = dojo.string.substitute(this.messages.notAuthorized, [ userFormated, this.applicationName ]);	
			if(this.customNotAuthorizedMessage) {
				this._errorArgs.details = this.customNotAuthorizedMessage;
			}
		}
		
		this._errorArgs.includeBackLink = this.includeBackLink;

		if(this.showLogout) {
			this._errorArgs.actions = [{
				label: this.messages.logout,
				iconClass: "logoutLink",
				href: "#",
				onClick: dojo.hitch(this, function(e) {
					dojo.stopEvent(e);
					var dfd = jazz.app.auth.logout();
					dfd.addCallback(dojo.hitch(this, function(){
						if(this.redirectUrl) {
							window.location.href = this.redirectUrl;
						}
						window.location.reload();
					}));
				})
			}];
		}
		
		this._contentWidget = new jazz.ui.ErrorPage(this._errorArgs);
		this.notAuthArea.appendChild(this._contentWidget.domNode);
	}
	
});

