/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2011, 2013. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function(){
	dojo.provide("com.ibm.team.process.web.ui.internal.admin.accessgroup.ManageAccessGroups"); //$NON-NLS-1$
	
	dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler"); //$NON-NLS-1$
	dojo.require("com.ibm.team.process.web.common.internal.ProcessConstants"); //$NON-NLS-1$
	dojo.require("com.ibm.team.process.web.client.internal.AccessGroupClient"); //$NON-NLS-1$
	dojo.require("com.ibm.team.process.web.ui.accessgroup.AccessGroupDialog"); //$NON-NLS-1$
	dojo.require("com.ibm.team.process.web.ui.internal.admin.projects.UIHelper"); //$NON-NLS-1$
	dojo.require("com.ibm.team.process.web.ui.internal.StyledTableItemList"); //$NON-NLS-1$
	dojo.require("com.ibm.team.process.web.ui.internal.TRToolbar"); //$NON-NLS-1$
	
	dojo.require("dojo.i18n"); //$NON-NLS-1$
	
	dojo.require("jazz.ui.ResourceLink");//$NON-NLS-1$
	dojo.require("jazz.ui.MessageBox"); //$NON-NLS-1$
	dojo.require("jazz.ui.SearchBox"); //$NON-NLS-1$
	dojo.require("jazz.ui.PaginationControl"); //$NON-NLS-1$
	dojo.require("jazz.ui.Toolbar"); //$NON-NLS-1$
	dojo.require("jazz.ui.toolbar.Button"); //$NON-NLS-1$
	dojo.require("jazz.ui.toolbar.TwoStateButton"); //$NON-NLS-1$
	
	dojo.require("net.jazz.ajax.ui.PlatformUI"); //$NON-NLS-1$

	dojo.requireLocalization("com.ibm.team.process.web.ui.internal.admin.accessgroup", "Messages"); //$NON-NLS-1$ //$NON-NLS-2$
	
	var AccessGroupClient = com.ibm.team.process.web.client.internal.AccessGroupClient;
	var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;
	var ProcessConstants = com.ibm.team.process.web.common.internal.ProcessConstants;
	var UIHelper = com.ibm.team.process.web.ui.internal.admin.projects.UIHelper;
	var MessageBox = jazz.ui.MessageBox;
	
	var STATUS_TYPES = MessageBox.STATUS_TYPES;
	var PAGE_SIZE = 25;	
	
	dojo.declare("com.ibm.team.process.web.ui.internal.admin.accessgroup.ManageAccessGroups", com.ibm.team.process.web.ui.internal.StyledTableItemList, { //$NON-NLS-1$
		templatePath: dojo.moduleUrl("com.ibm.team.process.web", "ui/internal/admin/accessgroup/templates/ManageAccessGroups.html"), //$NON-NLS-1$ //$NON-NLS-2$
		constructor: function(args){
			this._owningApplicationKey = args.owningApplicationKey;
			this.messages = dojo.i18n.getLocalization("com.ibm.team.process.web.ui.internal.admin.accessgroup", "Messages"); //$NON-NLS-1$ //$NON-NLS-2$
			this._resourceLinks = [];
		},
		
		postCreate: function(){
			dojo.addClass(this.domNode, "com-ibm-team-process-ManageAccessGroups"); //$NON-NLS-1$
			this.header = new MessageBox();
			this.messageArea.appendChild(this.header.domNode);
			
			this._actionToolbars = [];
			this._toolbarButtons = [];
			
			this.heading.appendChild(document.createTextNode(this.messages.ManageAccessGroups_AccessGroups));
			this.description.appendChild(document.createTextNode(this.messages.ManageAccessGroups_Description));
			
			dojo.addClass(this.tableContents, ProcessConstants.CSSClass.TOOLBAR_TABLE);
			this._refreshLink = this.addLink("#", "", "com.ibm.team.repository.web/ui/internal/images/refresh16.gif", dojo.hitch(this, function() { //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
				this.refresh();		
			}), this.messages.ManageAccessGroups_RefreshAccessGroups);
			this._createNewLink = this.addLink("#", this.messages.ManageAccessGroups_CreateAccessGroups, "com.ibm.team.process.web/ui/internal/admin/accessgroup/images/create-accessgroup_co.gif", dojo.hitch(this, "_createAccessGroup")); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
			this._createPager();
			this._createToggleArchiveDisplayAction();
			this._createSearchBox();
			this._initTable();
			this.refresh();		
		},
		
		_toggleShowArchive: function(){
			this._pageNum = 0;
			this._indicateLoading();
			this._getAccessGroupsUsingSearchFilter(this._searchBox.getCurrentQuery());
		},
		
		refresh: function(){
			this.clearState();
			this._indicateLoading();
			var showArchived = this._showArchivedButton.pressed;
			var srh = new ServiceResponseHandler(this, "_showAccessGroups", "_showErrorCallback"); //$NON-NLS-1$ //$NON-NLS-2$
			AccessGroupClient.getAccessGroupsPaged(srh, {
				owningApplicationKey: this._owningApplicationKey,
				pageNum: 0,
				hideArchived: !showArchived,
				pageSize: PAGE_SIZE
			});
		},
				
		_indicateLoading: function(){
			this._createNewLink.setDisabled(true);
			this._pager.setEnabled(false);
			dojo.empty(this.tbody);
			this.header.hideMessage();
			var tr = document.createElement("tr"); //$NON-NLS-1$
			var td = document.createElement("td"); //$NON-NLS-1$
			td.colSpan = 2;
			td.appendChild(document.createTextNode(this.messages.ManageAccessGroups_Loading));
			tr.appendChild(td);
			this.tbody.appendChild(tr);
		},

		clearState: function(){
			dojo.empty(this.tbody);
			this.header.hideMessage();
			this._destroyActionToolbars();
			this._pageNum = 0;
			this._searchBox.clear();
		},

		_createAccessGroup: function() {
			var accessGroupDialog = new com.ibm.team.process.web.ui.accessgroup.AccessGroupDialog({
				owningApplicationKey: this._owningApplicationKey,
				onSaveSuccess: dojo.hitch(this, this.refresh),
				onSaveError: dojo.hitch(this, this._createErrorCallback)
			});
			accessGroupDialog.startup();
		},
		
		_createErrorCallback: function(err){
			this.header.showMessage({
				summary: this.messages.ManageAccessGroups_CreateError,
				errorObject: err
			});
		},
		
		_createTRToolbar: function (accessGroup) {
			var trToolbar = new com.ibm.team.process.web.ui.internal.TRToolbar();
			
			var archiveAction = new jazz.ui.toolbar.Button({
				imgSrc : dojo.moduleUrl("com.ibm.team.process.web", "ui/internal/images/archive_hov.gif"), //$NON-NLS-1$ //$NON-NLS-2$
				title : this.messages.ManageAccessGroups_Archive,
				onClick : dojo.hitch(this, function (event) {
					dojo.stopEvent(event);
					if (confirm(dojo.string.substitute(this.messages.ManageAccessGroups_ConfirmArchive, [accessGroup.name]))) {
						this._archiveAccessGroup(accessGroup);
					}
				})				
			});
			this._toolbarButtons.push(archiveAction);

			var restoreAction = new jazz.ui.toolbar.Button({
				imgSrc : dojo.moduleUrl("com.ibm.team.process.web", "ui/internal/images/unarchive_hov.gif"), //$NON-NLS-1$ //$NON-NLS-2$
				title : this.messages.ManageAccessGroups_Restore,
				onClick : dojo.hitch(this, function (event) {
					dojo.stopEvent(event);
					if (confirm(dojo.string.substitute(this.messages.ManageAccessGroups_ConfirmRestore, [accessGroup.name]))) {
						this._restoreAccessGroup(accessGroup);
					}
				})				
			});
			this._toolbarButtons.push(restoreAction);
			
			if(accessGroup.archived) {
				trToolbar.addChild(restoreAction);
			} else {
				trToolbar.addChild(archiveAction);
			}
			
			var editAction = new jazz.ui.toolbar.Button({
				imgSrc : dojo.moduleUrl("com.ibm.team.process.web", "ui/internal/images/edit-ellipses.gif"), //$NON-NLS-1$ //$NON-NLS-2$
				title : this.messages.ManageAccessGroups_Edit,
				onClick : dojo.hitch(this, function (event) {
					dojo.stopEvent(event);
					this._editAccessGroup(accessGroup);
				})				
			});
			this._toolbarButtons.push(editAction);
			trToolbar.addChild(editAction);
			
			return trToolbar;
		},
				
		_showAccessGroups: function(data){
			UIHelper.runSafe(dojo.hitch(this, function(){
				dojo.empty(this.tbody);
				if (data && data.count > 0) {
					this._pager.setStartPage(this._pageNum);
					this._pager.setTotal(data.count);
					
					var sortedData = data.elements;
					for (var i = 0; i < sortedData.length; i++) {
						var trToolbar = this._createTRToolbar(sortedData[i]);
						this._actionToolbars.push(trToolbar);

						var row = this.addRow(null, [sortedData[i].name, sortedData[i].description, trToolbar.domNode]);
						if(sortedData[i].archived) {
							dojo.addClass(row, "archived"); //$NON-NLS-1$
						}
					}
				} else {
					if (!this._accessGroupArchiving) {
						this.header.showMessage({
							type: STATUS_TYPES.INFO,
							summary: this.messages.ManageAccessGroups_NoAccessGroup
						});
					}
					this._pager.setStartPage(0);
					this._pager.setTotal(0);
				}
				if (this._accessGroupArchiving) {
					delete this._accessGroupArchiving;
				}
				this._createNewLink.setDisabled(false);
				this._pager.setEnabled(true);
			}));
		},
		
		_showErrorCallback: function(err){
			this._createNewLink.setDisabled(false);
			this._pager.setEnabled(true);
			this.header.showMessage({
				summary: this.messages.ManageAccessGroups_ShowError,
				errorObject: err
			});
		},		
		
		_editAccessGroup: function(accessGroup) {
			var accessGroupDialog = new com.ibm.team.process.web.ui.accessgroup.AccessGroupDialog({
				accessGroupItemIdToEdit: accessGroup.itemId,
				owningApplicationKey: this._owningApplicationKey,
				onSaveSuccess: dojo.hitch(this, this.refresh),
				onSaveError: dojo.hitch(this, this._editErrorCallback),
				onLoadError: dojo.hitch(this, this._loadErrorCallback)
			});
			accessGroupDialog.startup();
		},
	
		_editErrorCallback: function(err){
			this.header.showMessage({
				summary: this.messages.ManageAccessGroups_EditError,
				errorObject: err
			});
		},
			
		_loadErrorCallback: function(err){
			this.header.showMessage({
				summary: this.messages.ManageAccessGroups_LoadError,
				errorObject: err
			});
		},
					
		_restoreAccessGroup: function(accessGroup) {
			this._disableToolbars(true);
			var srh = new ServiceResponseHandler(this, "_restoreCallback", "_restoreErrorCallback"); //$NON-NLS-1$ //$NON-NLS-2$
			AccessGroupClient.postRestoreAccessGroup(srh, {
				itemId: accessGroup.itemId
			});
			this.header.showMessage({
				type: STATUS_TYPES.INFO,
				summary: this.messages.ManageAccessGroups_RestoreInProgress
			});
		},
		
		_restoreCallback: function(itemId){
			UIHelper.runSafe(dojo.hitch(this, function(){
				this.refresh();
				this.header.showMessage({
					summary: this.messages.ManageAccessGroups_RestoreSuccess,
					type: STATUS_TYPES.OK,
					closable: true
				});
			}));
		},
		
		_restoreErrorCallback: function(err){
			this._disableToolbars(false);
			this.header.showMessage({
				summary: this.messages.ManageAccessGroups_RestoreError,
				errorObject: err
			});
		},		
		
		_archiveAccessGroup: function(accessGroup) {
			this._accessGroupArchiving = accessGroup;
			this._disableToolbars(true);
			var srh = new ServiceResponseHandler(this, "_archiveCallback", "_archiveErrorCallback"); //$NON-NLS-1$ //$NON-NLS-2$
			AccessGroupClient.postArchiveAccessGroup(srh, {
				itemId: accessGroup.itemId
			});
			this.header.showMessage({
				type: STATUS_TYPES.INFO,
				summary: this.messages.ManageAccessGroups_ArchiveInProgress
			});
		},
	
		_archiveCallback: function(itemId){
			UIHelper.runSafe(dojo.hitch(this, function(){
				this.refresh();
				this.header.showMessage({
					summary: this.messages.ManageAccessGroups_ArchiveSuccess,
					type: STATUS_TYPES.OK,
					closable: true
				});
			}));
		},
		
		_archiveErrorCallback: function(err){
			delete this._projectAreaArchiving;
			this._disableToolbars(false);
			this.header.showMessage({
				summary: this.messages.ManageAccessGroups_ArchiveError,
				errorObject: err
			});
		},
											
		_initTable: function(){
			dojo.addClass(this.tableContents, ProcessConstants.CSSClass.TOOLBAR_TABLE);
			var columns = [];
			columns.push({
				label: this.messages.ManageAccessGroups_GridHeaderName,
				width: "30%", //$NON-NLS-1$
				sortable: false
			});
			columns.push({
				label: this.messages.ManageAccessGroups_GridHeaderDesc,
				sortable: false
			});
			columns.push({
				label: this.messages.ManageAccessGroups_GridHeaderActions,
				className: ProcessConstants.CSSClass.ACTION_COLUMN,
				sortable: false
			});
			
			this.createTable({
				columns: columns
			});
		},

		_createPager: function(){
			this._pager = new jazz.ui.PaginationControl({
				total: 0,
				increment: PAGE_SIZE,
				onNextClick: dojo.hitch(this, function(pager, start, end){
					if (end === pager.total) {
						this._pageNum = Math.ceil(pager.total / pager.increment) - 1;
					} else {
						this._pageNum++;
					}
					this._getAccessGroupsUsingSearchFilter(this._searchBox.getCurrentQuery());
				}),
				onPreviousClick: dojo.hitch(this, function(pager, start, end){
					if (start === 1) {
						this._pageNum = 0;
					} else {
						this._pageNum--;
					}
					this._getAccessGroupsUsingSearchFilter(this._searchBox.getCurrentQuery());
				}),
				setStartPage: function(startPage){
					this.startPage = startPage;
				}
			});
			
			this._pagingControls.appendChild(this._pager.domNode);
		},
		
		_createSearchBox: function(){
			this._searchBox = new jazz.ui.SearchBox({
				allowEmptySearches: true,
				scopes: [{
					id: "accessGroups", //$NON-NLS-1$
					label: this.messages.ManageAccessGroups_AccessGroups,
					icon: dojo.moduleUrl("com.ibm.team.process.web", "ui/internal/admin/accessgroup/images/access-group_obj.gif") //$NON-NLS-1$ //$NON-NLS-2$
				}],
				onExecute: dojo.hitch(this, function(searchFilter){
					this._pageNum = 0;
					this._getAccessGroupsUsingSearchFilter(searchFilter);
				}),
				getCurrentQuery: function(){
					return this._currentQuery;
				}
			});
			this._filterControls.appendChild(this._searchBox.domNode);
			
		},

		_createToggleArchiveDisplayAction: function () {
			this._accessGroupToolbar = new jazz.ui.Toolbar();
			
			this._showArchivedButton = new jazz.ui.toolbar.TwoStateButton({
				imgSrc : dojo.moduleUrl("com.ibm.team.process.web", "ui/internal/admin/accessgroup/images/arch-access-group_obj.gif"), //$NON-NLS-1$ //$NON-NLS-2$
				title : this.messages.ManageAccessGroups_ShowArchived,
				onClick : dojo.hitch(this, this._toggleShowArchive)				
			});
			this._accessGroupToolbar.addChild(this._showArchivedButton);
			this._filterControls.appendChild(this._accessGroupToolbar.domNode);
		},
				
		_getAccessGroupsUsingSearchFilter: function(searchFilter){
			this._indicateLoading();
			var showArchived = this._showArchivedButton.pressed;
			var params = {
				owningApplicationKey: this._owningApplicationKey,
				pageNum: this._pageNum,
				hideArchived: !showArchived,
				pageSize: PAGE_SIZE				
			};
			if (searchFilter) {
				params.nameFilter = "*"+searchFilter+"*"; //$NON-NLS-1$ //$NON-NLS-2$
			}
			var srh = new ServiceResponseHandler(this, "_showAccessGroups", "_showErrorCallback"); //$NON-NLS-1$ //$NON-NLS-2$
			AccessGroupClient.getAccessGroupsPaged(srh, params);
		},

		_disableToolbars: function(disabled){
			for (var i = 0; i < this._actionToolbars.length; i++) {
				this._actionToolbars[i].disableToolbar(disabled);
			}
		},
		
		_destroyToolbarButtons: function(){
			for (var i = 0; i < this._toolbarButtons.length; i++) {
				if (this._toolbarButtons[i]) {
					this._toolbarButtons[i].destroyRecursive();
				}
			}
			this._toolbarButtons = [];
		},
				
		_destroyActionToolbars: function(){
			this._destroyToolbarButtons();
			for (var i = 0; i < this._actionToolbars.length; i++) {
				if (this._actionToolbars[i]) {
					this._actionToolbars[i].destroyRecursive();
				}
			}
			this._actionToolbars = [];
		},
		
		uninitialize: function(){
			this._destroyActionToolbars();
			if (this.header) {
				this.header.destroyRecursive();
			}
			if (this._searchBox) {
				this._searchBox.destroyRecursive();
			}
			if (this._pager) {
				this._pager.destroyRecursive();
			}
			if (this._showArchivedButton) {
				this._showArchivedButton.destroyRecursive();
			}
			if (this._accessGroupToolbar) {
				this._accessGroupToolbar.destroyRecursive();
			}			
			
			this.inherited(arguments);
		}
	});
	
})();
