/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2009, 2013. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights: Use, duplication or
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 ******************************************************************************/
(function() {
	dojo.provide("com.ibm.team.process.web.ui.internal.admin.projects.DeferredModel"); //$NON-NLS-1$
	dojo.require("dijit.Tree"); //$NON-NLS-1$

	/**
	 * Provides a model for the
	 * com.ibm.team.process.web.ui.internal.admin.projects.AddProjectLinkDialog.
	 * 
	 * @arguments: store :
	 *             com.ibm.team.process.web.ui.internal.admin.projects.DeferredLoadWriteStore :
	 *             The store which is used by this model.
	 *             <p>
	 *             _reportProgressCallback : function(int nr, int total) :
	 *             Report the progress of loading items to the store. Where nr
	 *             is the number of currently loaded items and the total is the
	 *             total number of items that are to be loaded.
	 * @since 4.0.1
	 */
	dojo.declare("com.ibm.team.process.web.ui.internal.admin.projects.DeferredModel", dijit.tree.ForestStoreModel, { //$NON-NLS-1$
		getChildren: function(parentItem, successCallback, errorCallback) {
			var firstLevelChildren = []; // An array of children to be
											// returned
			var secondLevelItems = []; // An array of next level children,
										// which will be loaded
			var currentWaitForCount = 0;
			var totalWaitForCount = 0;
			var store = this.store;
			var children, children2, j;
			if (parentItem === this.root) {
				store.fetch({
					query: this.query,
					onItem: dojo.hitch(this, function(item) {
						firstLevelChildren.push(item);
						children = this._getChildrenItems(item);
						secondLevelItems.push.apply(secondLevelItems, children);
					}),
					onError: errorCallback
				});
			} else {
				children = this._getChildrenItems(parentItem);
				firstLevelChildren.push.apply(firstLevelChildren, children);
				for (j = 0; j < children.length; j++) {
					children2 = this._getChildrenItems(children[j]);
					secondLevelItems.push.apply(secondLevelItems, children2);
				}
			}

			dojo.forEach(secondLevelItems, function(item) {
				if (!store.isItemLoaded(item)) {
					currentWaitForCount++;
				}
			});

			if (currentWaitForCount === 0) {
				successCallback(firstLevelChildren);
			} else {
				totalWaitForCount = currentWaitForCount;
				var onItem = dojo.hitch(this, function onItem(item) {
					this._reportProgressCallback(totalWaitForCount - --currentWaitForCount, totalWaitForCount);
					if (currentWaitForCount === 0) {
						successCallback(firstLevelChildren);
					}
				});
				dojo.forEach(secondLevelItems, function(item) {
					if (!store.isItemLoaded(item)) {
						store.loadItem({
							item: item,
							onItem: onItem,
							onError: errorCallback
						});
					}
				});
			}
		},

		_getChildrenItems: function(item) {
			var childrenItems = [];
			for ( var i = 0; i < this.childrenAttrs.length; i++) {
				var children = this.store.getValues(item, this.childrenAttrs[i]);
				childrenItems = childrenItems.concat(children);
			}
			return childrenItems;
		}
	});
})();
