/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2009, 2013. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.process.web.ui.internal.admin.projects.IterationTypesSelect"); //$NON-NLS-1$

dojo.require("dijit.form.FilteringSelect"); //$NON-NLS-1$

dojo.requireLocalization("com.ibm.team.process.web.ui.internal.admin.projects", "Messages"); //$NON-NLS-1$ //$NON-NLS-2$

(function(){
	dojo.declare("com.ibm.team.process.web.ui.internal.admin.projects.IterationTypesSelect", [dijit.form.FilteringSelect], { //$NON-NLS-1$
		
		constructor: function(args) {
			this.messages = dojo.i18n.getLocalization("com.ibm.team.process.web.ui.internal.admin.projects", "Messages"); //$NON-NLS-1$ //$NON-NLS-2$
			
			args.store = this._createIterationTypesStore(args.timelineStore);
		},
		
		validate: function(/*Boolean*/isFocused) {
			var isValid = this.inherited(arguments);
			if (this.validateCallback) {
				this.validateCallback();
			}
			return isValid;
		},
		
		isValid: function() {
			// consider the original iteration type name passed into this select
			// a valid input, even though it may not correspond to an item in
			// the select's store (if it is an iteration type from the provider
			// that was overridden in the consumer)
			return this.inherited(arguments) || this.get("displayedValue") === this.originalIterationTypeName; //$NON-NLS-1$
		},
		
		_createIterationTypesStore: function(timelineStore) {
			var iterationTypeItems = [];
			timelineStore.fetch({
				query: {
					enabledContext: ProcessConstants.PermissionContext.ITERATION_TYPE
				},
				queryOptions: {
					deep: true
				},
				onComplete: dojo.hitch(timelineStore, function(items) {
					iterationTypeItems = dojo.filter(items, this.isAvailableIterationType, this);
				})
			});
			
			iterationTypeItems.sort(dojo.hitch(timelineStore, timelineStore.compareIterationTypes));
			
			var jsonData = {
				data: {
					identifier: 'id', //$NON-NLS-1$
					label: 'name', //$NON-NLS-1$
					items: []
				}
			};
			var iterationTypesStore = new dojo.data.ItemFileWriteStore(jsonData);
			for (var i = 0; i < iterationTypeItems.length; i++) {
				iterationTypesStore.newItem({
					id: timelineStore.getIdentity(iterationTypeItems[i]),
					name: this._getIterationTypeLabel(iterationTypeItems[i], timelineStore)
				});
			}
			
			// Add none last in the list
			this._noneItem = iterationTypesStore.newItem({
				id: com.ibm.team.process.web.common.internal.ProcessConstants.ITERATION_TYPE_NONE,
				name: this.messages.IterationTypesSelect_IterationTypeNone
			});
			return iterationTypesStore;
		},
		
		_getIterationTypeLabel: function(iterationTypeItem, timelineStore) {
			var label = timelineStore.getIterationTypeLabel(iterationTypeItem);
			if (timelineStore.isArchivedItem(iterationTypeItem)) {
				label = dojo.string.substitute(this.messages.IterationTypesSelect_IterationTypeArchivedLabel, [label]);
			}
			return label;
		}
	});
})();
