/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2012, 2013. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function(){
	dojo.provide("com.ibm.team.process.web.ui.internal.admin.projects.OperationBehaviorEditDialog"); //$NON-NLS-1$
	
	dojo.require("dijit._Widget"); //$NON-NLS-1$
	dojo.require("dijit._Templated"); //$NON-NLS-1$
	dojo.require("dojo.i18n"); //$NON-NLS-1$
	dojo.require("jazz.ui.Dialog"); //$NON-NLS-1$
	dojo.require("jazz.ui.MessageBox"); //$NON-NLS-1$
	dojo.require("jazz.util.html"); //$NON-NLS-1$
	dojo.require("com.ibm.team.process.web.common.internal.ProcessConstants"); //$NON-NLS-1$
	dojo.require("com.ibm.team.process.web.ui.internal.admin.projects.UIHelper"); //$NON-NLS-1$
	
	dojo.requireLocalization("com.ibm.team.process.web.ui.internal", "Messages"); //$NON-NLS-1$ //$NON-NLS-2$
	
	var ProcessConstants = com.ibm.team.process.web.common.internal.ProcessConstants;
	var UIHelper = com.ibm.team.process.web.ui.internal.admin.projects.UIHelper;
		
	dojo.declare("com.ibm.team.process.web.ui.internal.admin.projects.OperationBehaviorEditDialog", [dijit._Widget, dijit._Templated], { //$NON-NLS-1$
		templatePath: dojo.moduleUrl("com.ibm.team.process.web", "ui/internal/admin/projects/templates/OperationBehaviorEditDialog.html"), //$NON-NLS-1$ //$NON-NLS-2$
		
		/**
		 * Abstract widget to present a simple dialog allowing the user to edit
		 * an operation behavior configuration. Subclasses must add their custom
		 * content after the widget is created and override function
		 * getResultObject() appropriately.
		 * 
		 * @param args {Object} An object that must have the following
		 * properties:
		 * 
		 * 	behaviorConfigurationItem	The item in the editor's configuration
		 * 								store that is to be updated
		 *  configurationStore			A dojo.data.ItemFileReadStore which
		 * 								contains operation configurations
		 * 								defined for the current process area
		 */
		constructor: function(args) {
			this.messages = dojo.i18n.getLocalization("com.ibm.team.process.web.ui.internal.admin.projects", "Messages"); //$NON-NLS-1$ //$NON-NLS-2$
			this._isShowingValidationError = false;	
		},
		
		postCreate: function() {
			this.dialog = new jazz.ui.Dialog({
				primaryTitle: this.messages.OperationBehaviorEditDialog_0,
				contentNode: this.domNode,
				width: "50em", //$NON-NLS-1$
				destroyContents: true
			});
			
			this._messageBox = new jazz.ui.MessageBox();
			this.messageBoxContainer.appendChild(this._messageBox.domNode);
			
			this.createCustomContents();
			
			this.hideSummaryTextNode();
			this.dialogSummaryTextNode.innerHTML = jazz.util.html.removeScript(this.getSummaryTextString());
		},
		
		startup: function() {
			UIHelper.resizeI18NButtons(this.dialogFooter, [this.okButton, this.cancelButton]);
			this.inherited(arguments);
		},
		
		createCustomContents: function() {
			throw new Error("subclass must override"); //$NON-NLS-1$
		},
		
		getResultObject: function() {
			throw new Error("subclass must override"); //$NON-NLS-1$
		},
		
		canFinish: function() {
			throw new Error("subclass must override"); //$NON-NLS-1$
		},
		
		getSummaryTextString: function() {
			throw new Error("subclass must override"); //$NON-NLS-1$
		},
		
		hideSummaryTextNode: function() {
			throw new Error("subclass must override"); //$NON-NLS-1$
		},
		
		notifySelectionChanged: function(doubleClicked) {
			this._validate();
			this.okButton.disabled = this._isShowingValidationError || !this.canFinish();
			if (doubleClicked && !this.okButton.disabled) {
				this._okCallback();
			}
		},
		
		_clearValidationError: function() {
			if (this._isShowingValidationError) {
				this._isShowingValidationError = false;
				this._messageBox.hideMessage();
			}
		},
		
		_addValidationError: function(errorMessage, errorType) {
			this._isShowingValidationError = true;
				
			this._messageBox.showMessage({
				type: errorType,
				summary: errorMessage
			});
		},
		
		/**
		 * Perform validation to prevent user from modifying the properties of a
		 * configuration to match those of an existing configuration
		 */
		_validate: function() {
			if (!this.canFinish()) {
				//No need to show error if the user cannot proceed
				this._clearValidationError();
			}
			else {
				//See if there is an existing configuration that matches the
				//selected options
				
				//Returns the property as specified by the user in this dialog,
				//or the one on the item being edited, in that order
				var getProperty = dojo.hitch(this, function(propertyName) {
					var result = this.getResultObject()[propertyName];
					if (result === undefined) {
						result = this.model.configurationStore.getValue(this.behaviorConfigurationItem, propertyName);
					}
					return result;
				});
				
				var operationId = getProperty(ProcessConstants.ExtensionDTO.ID);
				var context = getProperty(ProcessConstants.BehaviorConfigurationDTO.CONTEXT);
				var contextId = getProperty(ProcessConstants.BehaviorConfigurationDTO.CONTEXT_ID) || ""; //$NON-NLS-1$
				var roleId = getProperty(ProcessConstants.BehaviorConfigurationDTO.ROLE_ID);
				
				var existing = this.model.configurationStore.getConfiguration(operationId, context, contextId, roleId);
				
				if (existing !== null) {
					//There is a configuration with the same properties as the
					//user selected. Show an error and prohibit finish.
					this._addValidationError(this._getConfigurationDuplicationMessage(operationId, context, contextId, roleId), jazz.ui.MessageBox.STATUS_TYPES.WARNING);
				} else if (this._isShowingValidationError) {
					//There is not a configuration with the same properties as
					//the user selected. Clear the error and allow finish.
					this._clearValidationError();
				}
			}
		},
		
		/**
		 * Returns a user-readable string suitable for error messages which
		 * states that a configuration already exists with the selected
		 * operation, role, and context.
		 */
		_getConfigurationDuplicationMessage: function(operationId, context, contextId, roleId) {
			var configuration = this.model.configurationStore.getConfiguration(operationId, context, contextId, roleId);
			var operationName = this.model.configurationStore.getValue(configuration, ProcessConstants.ExtensionDTO.NAME);
			var roleName = this.model.configurationStore.getRoleLabel(configuration);
			
			var duplicationMessage;
			if (ProcessConstants.BehaviorContext.PROJECT === context
				|| ProcessConstants.BehaviorContext.TEAM === context) {
				duplicationMessage = dojo.string.substitute(this.messages.OperationBehaviorWizard_4, [operationName, roleName]);
			} else {
				var contextName = this.model.configurationStore.getContextLabel(configuration);
				if (ProcessConstants.BehaviorContext.ITERATION_TYPE === context) {
					duplicationMessage = dojo.string.substitute(this.messages.OperationBehaviorWizard_5, [operationName, roleName, contextName]);
				} else {
					duplicationMessage = dojo.string.substitute(this.messages.OperationBehaviorWizard_6, [operationName, roleName, contextName]);	
				}
			}
			return duplicationMessage;
		},
		
		_okCallback: function(e) {
			this.model.configurationStore.updateConfiguration(this.behaviorConfigurationItem, this.getResultObject());
			this.close(e);
		},
		
		_cancelCallback: function(e) {
			this.close(e);
		},
		
		uninitialize: function(){
			if (this._messageBox) {
				this._messageBox.destroyRecursive();
			}
			
			this.inherited(arguments);
		},
		
		close: function(e) {
			if (e) {
				dojo.stopEvent(e);
			}
			this.dialog.close();
		}
	});
})();
