/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2012, 2013. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function(){
	dojo.provide("com.ibm.team.process.web.ui.internal.admin.projects.OperationBehaviorSelectOperationWidget"); //$NON-NLS-1$
	
	dojo.require("dijit._Widget"); //$NON-NLS-1$
	dojo.require("dijit._Templated"); //$NON-NLS-1$
	dojo.require("dojo.i18n"); //$NON-NLS-1$
	dojo.require("com.ibm.team.process.web.common.internal.ProcessConstants"); //$NON-NLS-1$
	dojo.require("com.ibm.team.process.web.ui.internal.admin.projects.ProcessTree"); //$NON-NLS-1$
	
	dojo.requireLocalization("com.ibm.team.process.web.ui.internal", "Messages"); //$NON-NLS-1$ //$NON-NLS-2$
	
	var ProcessConstants = com.ibm.team.process.web.common.internal.ProcessConstants;
	
	/**
	 * A widget that presents a tree of categories and operations in a process
	 * area whose behavior can be configured. When a category or operation is
	 * selected, its description and number of available preconditions and
	 * follow-up actions is displayed.
	 * 
	 * The argument to the constructor must be an object with the following
	 * properties:
	 * 
	 * - model	[required]	An object which provides information about the
	 * 						general process model that is not operation-behavior
	 * 						specific. The following fields are required:
	 * 
	 * 		- extensionStore	[required]	A dojo.data.ItemFileReadStore which
	 * 										contains categories and operations
	 * 										for which operation behavior can be
	 * 										configured 
	 */
	dojo.declare("com.ibm.team.process.web.ui.internal.admin.projects.OperationBehaviorSelectOperationWidget", //$NON-NLS-1$
			[dijit._Widget, dijit._Templated], {
		
		templatePath: dojo.moduleUrl("com.ibm.team.process.web", "ui/internal/admin/projects/templates/OperationBehaviorSelectOperationWidget.html"), //$NON-NLS-1$ //$NON-NLS-2$
		
		constructor: function(args) {
			this.messages = dojo.i18n.getLocalization("com.ibm.team.process.web.ui.internal.admin.projects", "Messages"); //$NON-NLS-1$ //$NON-NLS-2$
		},
		
		postCreate: function() {
			this._extensionModel = this._createExtensionModel();
			this._extensionTree = new com.ibm.team.process.web.ui.internal.admin.projects.ProcessTree({
				model : this._extensionModel,
				showRoot: false,
				labelAttr: ProcessConstants.ExtensionDTO.NAME,
				childrenAttr: [ProcessConstants.ExtensionDTO.ITEMS],
				focusChanged: dojo.hitch(this, this._treeItemSelected),
				onDblClick: dojo.hitch(this, function() {
					this.selectionChangedCallback({
						operationId: this._selectedOperationId,
						operationContext: this._selectedOperationContext
					}, true);
				}),
				getIconClass: function(item) {
					return null;
				},
				getIconStyle: dojo.hitch(this, function(item) {
					if (item !== this._extensionModel.root && this._isCategory(item) && this.model.extensionStore.hasAttribute(item, ProcessConstants.ExtensionDTO.ICON_URL)) {
						return {
							backgroundImage: "url(" + this.model.extensionStore.getValue(item, ProcessConstants.ExtensionDTO.ICON_URL) + ")", //$NON-NLS-1$ //$NON-NLS-2$
							width: "16px", //$NON-NLS-1$
							height: "16px", //$NON-NLS-1$
							backgroundRepeat: "no-repeat", //$NON-NLS-1$
							margin: "-1px 2px 0 2px" //$NON-NLS-1$
						};
					}
					return null;
				})
			});
			this.operationsTree.appendChild(this._extensionTree.domNode);
			
			this._summaryText.innerHTML = dojo.string.substitute(this.messages.OperationBehaviorSelectOperationWidgetHTML_0, ["<span class=\"operationBehaviorCreateWhat\">", "</span>"]); //$NON-NLS-1$ //$NON-NLS-2$
		},
		
		/**
		 * Create and return a tree model where categories can be expanded to
		 * show their child operations.
		 */
		_createExtensionModel: function() {
			var extensionStore = this.model.extensionStore;
			var extensionModel = new dojo.declare(dijit.tree.ForestStoreModel, {
				getChildren: function(/*dojo.data.Item*/ parentItem, /*function(items)*/ callback, /*function*/ onError) {
					var oldCallback = callback;
					// only call callback on categories and operations
					callback = function(items) {
						items = dojo.filter(items, function(item) {
							var categoryOrOperation = extensionStore.getValue(item, ProcessConstants.ExtensionDTO.ID).split(ProcessConstants.BEHAVIOR_PATH_SEPARATOR).length <= 2;
							var deprecated = extensionStore.getValue(item, ProcessConstants.ExtensionDTO.DEPRECATED) === true;
							return categoryOrOperation && !deprecated;
	        			});
						oldCallback(items);
					};
					this.inherited(arguments);
				},
				
				mayHaveChildren: dojo.hitch(this, this._isCategory)
			})({
				childrenAttrs: [ProcessConstants.ExtensionDTO.ITEMS],
				store: extensionStore
			});
			return extensionModel;
		},
		
		/**
		 * onClick callback for the category/operations tree. This function
		 * updates the state for this page: the ID of the operation that was
		 * selected and whether that operation is a project or team operation,
		 * and then updates the description area with a description of what was
		 * selected and additional information depending on what was selected: a
		 * count of preconditions and follow-up actions for operations, and a
		 * hint to select an operation to proceed when a category was selected. 
		 */
		_treeItemSelected: function(node) {
			var item = node.item;
			var isCategory = this._isCategory(item);
			
			//Update label with description
			dojo.empty(this.descriptionContainer);
			var description = this.model.extensionStore.getValue(item, ProcessConstants.ExtensionDTO.DESCRIPTION);
			if (description !== null && dojo.string.trim(description).length > 0) {
				dojo.removeClass(this.descriptionContainer, "placeholder"); //$NON-NLS-1$
			} else {
				dojo.addClass(this.descriptionContainer, "placeholder"); //$NON-NLS-1$
				description = isCategory
					? this.messages.OperationBehaviorSelectOperationWidget_0
					: this.messages.OperationBehaviorSelectOperationWidget_1;
			}
			this.descriptionContainer.appendChild(document.createTextNode(description));
			
			dojo.empty(this.selectedOperationNumPreconditions);
			dojo.empty(this.selectedOperationNumFollowupActions);
			
			dojo.addClass(this.selectedOperationNumPreconditions, "hidden"); //$NON-NLS-1$
			dojo.addClass(this.selectedOperationNumFollowupActions, "hidden"); //$NON-NLS-1$
			dojo.addClass(this.selectedCategoryGuidanceLabel, "hidden"); //$NON-NLS-1$

			if (isCategory) {
				this._selectedOperationId = undefined;
				this._selectedOperationContext = undefined;
				
				dojo.removeClass(this.selectedCategoryGuidanceLabel, "hidden"); //$NON-NLS-1$
			} else {
				this._selectedOperationId = this.model.extensionStore.getValue(item, ProcessConstants.ExtensionDTO.ID);
				this._selectedOperationContext = ProcessConstants.ExtensionContext.PROJECT === this.model.extensionStore.getValue(item, ProcessConstants.ExtensionDTO.CONTEXT)
					? ProcessConstants.BehaviorContext.PROJECT
					: ProcessConstants.BehaviorContext.TEAM;
				
				//Update label with count of available preconditions and/or
				//follow-up actions
				var numPreconditions = this.model.extensionStore.getPreconditions(item).length;
				var numFollowupActions = this.model.extensionStore.getFollowupActions(item).length;
				
				if (0 === numPreconditions) {
					dojo.addClass(this.selectedOperationNumPreconditions, "hidden"); //$NON-NLS-1$
				} else {
					dojo.removeClass(this.selectedOperationNumPreconditions, "hidden"); //$NON-NLS-1$
					if (1 === numPreconditions) {
						this.selectedOperationNumPreconditions.appendChild(document.createTextNode(this.messages.OperationBehaviorSelectOperationWidget_2));
					} else {
						this.selectedOperationNumPreconditions.appendChild(document.createTextNode(
							dojo.string.substitute(this.messages.OperationBehaviorSelectOperationWidget_3, [numPreconditions])
						));
					}
				}
				
				if (0 === numFollowupActions) {
					dojo.addClass(this.selectedOperationNumFollowupActions, "hidden"); //$NON-NLS-1$
				} else {
					dojo.removeClass(this.selectedOperationNumFollowupActions, "hidden"); //$NON-NLS-1$
					if (1 === numFollowupActions) {
						this.selectedOperationNumFollowupActions.appendChild(document.createTextNode(this.messages.OperationBehaviorSelectOperationWidget_4));
					} else {
						this.selectedOperationNumFollowupActions.appendChild(document.createTextNode(
							dojo.string.substitute(this.messages.OperationBehaviorSelectOperationWidget_5, [numFollowupActions])
						));
					}
				}
			}
			this.selectionChangedCallback({
				operationId: this._selectedOperationId,
				operationContext: this._selectedOperationContext
			});
		},
		
		/**
		 * Returns whether an item is a category, distinguished by an id
		 * property that does not have a path separator like an operation would.
		 * 
		 * @param item {dojo.data.Item} category or operation item
		 */
		_isCategory: function(/*dojo.data.Item*/ item) {
			return this.model.extensionStore.getValue(item, ProcessConstants.ExtensionDTO.ID).indexOf(ProcessConstants.BEHAVIOR_PATH_SEPARATOR) === -1;
		},
		
		uninitialize: function(){
			if (this._extensionTree !== undefined) {
				this._extensionTree.destroyRecursive();
				delete this._extensionTree;
			}
			delete this._extensionModel;
			
			this.inherited(arguments);
		}
	});
})();
