/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2012, 2013. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function(){
	dojo.provide("com.ibm.team.process.web.ui.internal.admin.projects.OperationBehaviorSelectRoleWidget"); //$NON-NLS-1$
	
	dojo.require("dijit._Widget"); //$NON-NLS-1$
	dojo.require("dijit._Templated"); //$NON-NLS-1$
	dojo.require("dojo.i18n"); //$NON-NLS-1$
	dojo.require("com.ibm.team.process.web.common.internal.ProcessConstants"); //$NON-NLS-1$
	dojo.requireLocalization("com.ibm.team.process.web.ui.internal", "Messages"); //$NON-NLS-1$ //$NON-NLS-2$
	
	var ProcessConstants = com.ibm.team.process.web.common.internal.ProcessConstants;	
	
	/**
	 * A widget that presents a list of roles in a process area, for single
	 * selection. When a role is selected, its description is displayed. When an
	 * existing operation behavior configuration item can be supplied, its
	 * current role is selected by default and presented in a disabled style.
	 * 
	 * The argument to the constructor must be an object with the following
	 * properties:
	 * 
	 * - model	[required]	An object which provides information about the
	 * 						general process model that is not operation-behavior
	 * 						specific. The following fields are required:
	 * 
	 * 		- rolesStore			A dojo.data.ItemFileReadStore with roles
	 * 								available to select from, with fields id,
	 * 								name, and description
	 *		- configurationStore	A dojo.data.ItemFileReadStore which contains
	 * 								operation configurations defined for the
	 * 								current process area
	 * 
	 * - behaviorConfigurationItem	[optional]
	 *     A dojo.data store item that represents an existing operation behavior
	 *     configuration that is to be updated
	 */
	dojo.declare("com.ibm.team.process.web.ui.internal.admin.projects.OperationBehaviorSelectRoleWidget", //$NON-NLS-1$
			[dijit._Widget, dijit._Templated], {
		
		templatePath: dojo.moduleUrl("com.ibm.team.process.web", "ui/internal/admin/projects/templates/OperationBehaviorSelectRoleWidget.html"), //$NON-NLS-1$ //$NON-NLS-2$
		
		constructor: function(args) {
			this.messages = dojo.i18n.getLocalization("com.ibm.team.process.web.ui.internal.admin.projects", "Messages"); //$NON-NLS-1$ //$NON-NLS-2$
		},
		
		postCreate: function() {
			this._createRolesList();
			
			this.summaryText.innerHTML = dojo.string.substitute(this.messages.OperationBehaviorSelectRoleWidgetHTML_0, ["<span class=\"operationBehaviorWhom\">", "</span>"]); //$NON-NLS-1$ //$NON-NLS-2$
		},
		
		/**
		 * Position the roles list to be as large as as constrained by the size
		 * of the container.
		 */
		resizeRolesList: function() {
			//Style the roles list so that it is as large as it can be without
			//overlapping the 'descriptionContainer' node at the bottom
			setTimeout(dojo.hitch(this, function() {
				var maximumRolesListBottom;
				if (this.domNode.nextSibling === null) {
					var pageContainerBox = dojo.marginBox(this.domNode.parentNode);
					if (pageContainerBox.h === 0) {
						//Can happen when the widget is displayed to edit an
						//existing item (a dialog is displayed). Re-schedule
						//resizeRolesList() until it's called after this
						//widget's content is visible on the page
						setTimeout(dojo.hitch(this, this.resizeRolesList), 30);
						return;
					}
					maximumRolesListBottom = pageContainerBox.t + pageContainerBox.h;
				} else {
					maximumRolesListBottom = dojo.marginBox(this.domNode.nextSibling).t;
				}
				var rolesListMarginBox = dojo.marginBox(this.rolesList);
				while(rolesListMarginBox.t + rolesListMarginBox.h < maximumRolesListBottom) {
					this.rolesList.size += 1;
					rolesListMarginBox = dojo.marginBox(this.rolesList);
				}
				this.rolesList.size -= 1;
				
				//To truncate the description of the role that's initially
				//selected if necessary
				this._updateForRolesListSelectionChange();
			}), 0);
		},
		
		getSummaryTextString: function() {
			return this.summaryText.innerHTML;
		},
		
		hideSummaryTextNode: function() {
			dojo.addClass(this.summaryText, "hidden"); //$NON-NLS-1$
		},
		
		/**
		 * Initialize the select widget for roles.
		 */
		_createRolesList: function(){
			this._populateRolesSelect();
			if (this.rolesList.options.length > 0) {
				if (this.behaviorConfigurationItem === undefined) {
					this.rolesList.options[0].selected = true;
					this._updateForRolesListSelectionChange();
				} else {
					var configurationRoleId = this.model.configurationStore.getValue(this.behaviorConfigurationItem, ProcessConstants.BehaviorConfigurationDTO.ROLE_ID);
					for (var i = 0; i < this.rolesList.options.length; i++) {
						if (this.rolesList.options[i]._id === configurationRoleId) {
							dojo.addClass(this.rolesList.options[i], "disabled"); //$NON-NLS-1$
							this.rolesList.options[i].selected = true;
							break;
						}
					}
				}
				this.connect(this.rolesList, "onchange", "_rolesListSelectionChanged"); //$NON-NLS-1$ //$NON-NLS-2$
				this.connect(this.rolesList, "ondblclick", "_rolesListDoubleClicked"); //$NON-NLS-1$ //$NON-NLS-2$
			}
		},
		
		/**
		 * Add an option to the roles select for each role in the editor's
		 * collection. The role's identifier and description are stored.
		 */
		_populateRolesSelect: function(){
			dojo.forEach(this.model.rolesStore.getAllRoleItems(), function(role) {
				var roleId = this.model.rolesStore.getValue(role, ProcessConstants.ProcessRoleDTO.ID);
				var anOption = new Option(this.model.rolesStore.getRoleLabel(role), roleId);
				var description = this.model.rolesStore.getValue(role, ProcessConstants.ProcessRoleDTO.DESCRIPTION);
				if (description && dojo.string.trim(description).length > 0) {
					anOption._description = description;
				}
				anOption._id = roleId;
				this.rolesList.options[this.rolesList.options.length] = anOption;
			}, this);
		},
		
		/**
		 * Callback when an item in the list of roles is selected 
		 */
		_rolesListSelectionChanged: function() {
			this._updateForRolesListSelectionChange();
		},
		
		/**
		 * Callback when an item in the roles list is double clicked
		 */
		_rolesListDoubleClicked: function() {
			this.selectionChangedCallback({
				roleId: this._selectedRoleId
			}, true);
		},
		
		/**
		 * Update this page's state: the identifier of the role, and display the
		 * role's description in the page. If the description is too long, it
		 * will be truncated with ellipses, since this page will only be
		 * displayed in fixed-size containers.
		 */
		_updateForRolesListSelectionChange: function(){
			this._selectedRoleId = this.rolesList.options[this.rolesList.selectedIndex]._id;
			
			var description = this.rolesList.options[this.rolesList.selectedIndex]._description;
			
			dojo.empty(this.descriptionContainer);
			if (description != undefined) {
				dojo.removeClass(this.descriptionContainer, "placeholder"); //$NON-NLS-1$
			} else {
				dojo.addClass(this.descriptionContainer, "placeholder"); //$NON-NLS-1$
				description = this.messages.OperationBehaviorSelectRoleWidget_0;
			}
			this.descriptionContainer.appendChild(document.createTextNode(description));
			
			//Truncate the description if the container that's showing the
			//description is taller than the roles list.
			var rolesListHeight = dojo.marginBox(this.rolesList).h;
			while (dojo.marginBox(this.descriptionContainer).h > rolesListHeight) {
				this.descriptionContainer.innerHTML = this.descriptionContainer.innerHTML.substring(0, this.descriptionContainer.innerHTML.length * .9) + "..."; //$NON-NLS-1$
			}
			
			this.selectionChangedCallback({
				roleId: this._selectedRoleId
			}, false);
		},
		
		/**
		 * Hides (sets the property 'display' to "none") the summary text, in
		 * cases where it is not appropriate for the text to be displayed.
		 */
		hideSummaryText: function() {
			dojo.addClass(this.summaryText, "hidden"); //$NON-NLS-1$
		},
		
		/**
		 * Returns the text content (a string) of the summary text node, in
		 * cases where it is appropriate to display that text elsewhere.
		 */
		getSummaryTextContent: function() {
			return this.summaryText.innerHTML;
		},
		
		/**
		 * If a role with the specified identifier is listed, select it. Whether
		 * an item was selected is returned as a boolean. 'False' would indicate
		 * the the provided context ID does not match the identifier of a role
		 * in this widget.
		 */
		selectRoleIfAvailable: function(roleId) {
			var valueSet = false;
			for (var i = 0; i < this.rolesList.options.length; i++) {
				if (this.rolesList.options[i]._id === roleId) {
					valueSet = true;
					this.rolesList.selectedIndex = i;
					this.rolesList.options[i].selected = true;
					this._updateForRolesListSelectionChange();
					break;
				}
			}
			if (!valueSet) {
				//Select 'everyone' by default
				this.rolesList.selectedIndex = 0;
				this.rolesList.options[0].selected = true;
				this._updateForRolesListSelectionChange();
			}
			return valueSet;
		}
	});
})();
