/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function(){
	dojo.provide("com.ibm.team.process.web.ui.internal.admin.projects.OperationBehaviorStore"); //$NON-NLS-1$

	dojo.require("com.ibm.team.process.web.common.internal.ProcessConstants"); //$NON-NLS-1$
	dojo.require("com.ibm.team.process.web.ui.internal.admin.projects.ProcessStore"); //$NON-NLS-1$
	
	var ProcessConstants = com.ibm.team.process.web.common.internal.ProcessConstants;
	
	/**
	 * Provides convenience methods for working with operation behavior data.
	 */
	dojo.declare("com.ibm.team.process.web.ui.internal.admin.projects.OperationBehaviorStore", com.ibm.team.process.web.ui.internal.admin.projects.ProcessStore, { //$NON-NLS-1$
		
		/**
		 * Returns the preconditions in the store under this operation item.
		 * 
		 * @param operation {dojo.data.Item} item representing an operation
		 *            (e.g. not a category, precondition, etc.)
		 */
		getPreconditions: function(operation) {
			return this.getBehavior(operation, ProcessConstants.PRECONDITIONS);
		},
		
		/**
		 * Returns the follow-up actions in the store under this operation item.
		 * 
		 * @param operation {dojo.data.Item} item representing an operation
		 *            (e.g. not a category, precondition, etc.)
		 */
		getFollowupActions: function(operation) {
			return this.getBehavior(operation, ProcessConstants.FOLLOWUP_ACTIONS);
		},
		
		/**
		 * May be overridden in subclasses to filter out unwanted items.
		 */
		getBehavior: function(operation, type) {
			var items = [];
			dojo.forEach(this.getValues(operation, ProcessConstants.ExtensionDTO.ITEMS), dojo.hitch(this, function(item) {
				var segments = this.getValue(item, ProcessConstants.ExtensionDTO.ID).split(ProcessConstants.BEHAVIOR_PATH_SEPARATOR);
				if (segments.length > 2 && segments[2] === type) {
					items = this.getValues(item, ProcessConstants.ExtensionDTO.ITEMS);
				}
			}));
			return items;
		}
		
	});
	
})();