/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2012, 2013. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function(){
	dojo.provide("com.ibm.team.process.web.ui.internal.admin.projects.OperationBehaviorTable"); //$NON-NLS-1$
	
	dojo.require("com.ibm.team.process.web.common.internal.ProcessConstants"); //$NON-NLS-1$
	dojo.require("com.ibm.team.process.web.client.internal.ProcessClient"); //$NON-NLS-1$
	dojo.require("com.ibm.team.process.web.ui.internal.admin.projects.UIHelper"); //$NON-NLS-1$
	dojo.require("com.ibm.team.process.web.ui.internal.admin.projects.OperationBehaviorTree"); //$NON-NLS-1$
	
	dojo.require("dojo.i18n"); //$NON-NLS-1$
	
	dojo.require("dojox.uuid.generateTimeBasedUuid"); //$NON-NLS-1$
	dojo.require("dojox.xml.parser"); //$NON-NLS-1$
	
	dojo.require("jazz.app.ua.TextHoverHelp"); //$NON-NLS-1$
	dojo.require("jazz.ui.FilterBox"); //$NON-NLS-1$
	dojo.require("jazz.ui.PaginationControl"); //$NON-NLS-1$
	dojo.require("jazz.ui.StyledTable"); //$NON-NLS-1$
	dojo.require("jazz.ui.Toolbar"); //$NON-NLS-1$
	dojo.require("jazz.ui.toolbar.TwoStateButton"); //$NON-NLS-1$
	
	dojo.requireLocalization("com.ibm.team.process.web.ui.internal.admin.projects", "Messages"); //$NON-NLS-1$ //$NON-NLS-2$
	
	var ProcessClient = com.ibm.team.process.web.client.internal.ProcessClient;
	var ProcessConstants = com.ibm.team.process.web.common.internal.ProcessConstants;
	var UIHelper = com.ibm.team.process.web.ui.internal.admin.projects.UIHelper;
	
	var ROLE_COLUMN = 2;
	var CONTEXT_COLUMN = 3;
	
	dojo.declare("com.ibm.team.process.web.ui.internal.admin.projects.OperationBehaviorTable", [dijit._Widget, dijit._Templated], { //$NON-NLS-1$
		templatePath: dojo.moduleUrl("com.ibm.team.process.web", "ui/internal/admin/projects/templates/OperationBehaviorTable.html"), //$NON-NLS-1$ //$NON-NLS-2$
		
		pageSize: 25,
		
		/**
		 * @param configurationStore {OperationBehaviorConfigurationStore}
		 *            (required) store containing the info to populate with
		 * @param extensionStore {OperationBehaviorExtensionStore} (required)
		 *            store containing extensions for the info to populate with
		 * @param rolesStore {RolesStore} (optional) store containing roles
		 *            available for behavior configuration, needed for dynamic
		 *            updating
		 * @param contextStore {PermissionsScopeStore} (optional) store
		 *            containing development lines/iterations/iteration types
		 *            available for behavior configuration, needed for dynamic
		 *            updating
		 * @param pageSize {Integer} (optional) specify how many rows per page
		 */
		constructor: function(args) {
			this.messages = dojo.i18n.getLocalization("com.ibm.team.process.web.ui.internal.admin.projects", "Messages"); //$NON-NLS-1$ //$NON-NLS-2$
			this.columns = [
				{ label: "", sortable: true, className: "categoryColumn" }, //$NON-NLS-1$ //$NON-NLS-2$
				{ label: this.messages.OperationBehaviorTable_0, sortable: true },
				{ label: this.messages.OperationBehaviorTable_1, sortable: true },
				{ label: this.messages.OperationBehaviorTable_2, sortable: true },
				{ label: this.messages.OperationBehaviorTable_3, sortable: false }
			];
			this._textHelpHovers = [];
			this._tableConnects = [];
		},
		
		postCreate: function() {
			this._createPagers();
			this._createShowArchivedButton();
			this._createFilterBox();
			this._createTable();
			
			this.rowInfo = [];
		},
		
		_createTable: function(styledTableArgs) {
			this._destroyStyledTable();
			
			this.styledTable = new jazz.ui.StyledTable({
				columns: this.columns,
				sortBy: 1,
				trim: jazz.ui.StyledTable.TrimStyles.PRIMARY_HEADLESS,
				showHover: true
			});
			this.tbody = this.styledTable.tbody;
			this.tableContents.appendChild(this.styledTable.domNode);
			
			dojo.addClass(this.styledTable.domNode, "behaviorTable"); //$NON-NLS-1$
			this.connect(this.styledTable, "handleSort", "_handleSort"); //$NON-NLS-1$ //$NON-NLS-2$
		},
		
		/**
		 * Call this to create the rows of the table and show the first page.
		 * This should not be called until the table's configurationStore has
		 * been initialized, and don't forget to call clearTable first if
		 * necessary.
		 */
		populateTable: function() {
			// populate an array (rowInfo) containing info for all
			// configuration rows in the table, even those not currently showing
			this.configurationStore.fetch({
				// get all top-level items (operation configurations)
				onItem: dojo.hitch(this, function(item) {
					var info = this.getRowInfo(item);
					if (info) {
						this.rowInfo.push(info);
					}
				})
			});
			// show the first page of configurations
			this._initializeTable(0);
		},
		
		_initializeTable: function(pageNum) {
			this.pageNum = pageNum;
			this.styledTable.handleSort();
			this.numConfigurationsChanged();
			// listen for store events
			this._tableConnects.push(
				dojo.connect(this.rolesStore, "onNew", this, "_onNewRole"), //$NON-NLS-1$ //$NON-NLS-2$
				dojo.connect(this.rolesStore, "onSet", this, "_onSetRole"), //$NON-NLS-1$ //$NON-NLS-2$
				dojo.connect(this.rolesStore, "aboutToDelete", this, "_onDeleteRole"), //$NON-NLS-1$ //$NON-NLS-2$
				dojo.connect(this.contextStore, "onNew", this, "_onNewContext"), //$NON-NLS-1$ //$NON-NLS-2$
				dojo.connect(this.contextStore, "onSet", this, "_onSetContext"), //$NON-NLS-1$ //$NON-NLS-2$
				dojo.connect(this.contextStore, "aboutToDelete", this, "_onDeleteContext") //$NON-NLS-1$ //$NON-NLS-2$
			);
		},
		
		/**
		 * Sets the table to an empty state.
		 */
		clearTable: function() {
			this.rowInfo = [];
			delete this.allRowInfo;
			dojox.xml.parser.removeChildren(this.tbody);
			for (var i = 0; i < this._textHelpHovers.length; i++) {
				if (this._textHelpHovers[i]) {
					this._textHelpHovers[i].destroyRecursive();
				}
			}
			this._textHelpHovers = [];
			this._filterBox.setValue(""); //$NON-NLS-1$
			delete this._previousFilterValue;
			if (this._tableConnects) {
				dojo.forEach(this._tableConnects, dojo.disconnect);
				this._tableConnects = [];
			}
		},
		
		/**
		 * Add a row to the table that will have the objects in the columns
		 * array as columns. These objects may be:
		 * - strings, for text columns,
		 * - DOM nodes, eg. images, or
		 * - widgets, eg. toolbars
		 */
		addRow: function(/*Array*/ columns){
			var row = document.createElement("tr"); //$NON-NLS-1$
			this.tbody.appendChild(row);
			
			for (var i = 0; i < columns.length; i++) {
				var td = row.insertCell(i);
				var o = columns[i];
				if (dojo.isString(o)) {
					td.appendChild(document.createTextNode(o));
				} else if (UIHelper.isNode(o)) {
					td.appendChild(o);
				} else if (o.domNode) { // its a dijit._Widget
					td.appendChild(o.domNode);
				}
			}
			
			row._rowInfo = columns;
			
			dojo.addClass(row.childNodes[0], "categoryColumn"); //$NON-NLS-1$
			
			this.connect(row, "onclick", function(e) { //$NON-NLS-1$
				this.rowClicked(row, columns._item);
			});
			
			this.connect(row, "ondblclick", function(e) { //$NON-NLS-1$
				this.rowDoubleClicked(row, columns._item);
			});
			
			return row;
		},
		
		/**
		 * Consumers can dojo.connect to this function to listen for row click
		 * events.
		 */
		rowClicked: function(rowNode, configurationItem) {
		},
		
		/**
		 * Consumers can dojo.connect to this function to listen for row
		 * double click events.
		 */
		rowDoubleClicked: function(rowNode, configurationItem) {
		},
				
		/**
		 * Return a rowInfo object that already exists, as opposed to getRowInfo
		 * which is used to create a rowInfo.
		 */
		findRowInfo: function(item) {
			var allRowInfo = this.allRowInfo || this.rowInfo;
			for (var i = 0; i < allRowInfo.length; i++) {
				if (allRowInfo[i]._item === item) {
					return allRowInfo[i];
				}
			}
			return null;
		},
		
		findRow: function(item) {
			for (var i = 0; i < this.tbody.childNodes.length; i++) {
				var row = this.tbody.childNodes[i]; 
				if (row._rowInfo && row._rowInfo._item === item) {
					return row;
				}
			}
			return null;
		},
		
		/**
		 * Returns display information for each column of the table, which can
		 * be passed to addRow().
		 */
		getRowInfo: function(operation) {
			//We only want to display rows for configurations that are
			//'configured' (i.e. have not been deleted), not archived, and not
			//for an iteration under a non-current dev line (if in team area)
			if (this.configurationStore.getValue(operation, ProcessConstants.BehaviorConfigurationDTO.CONFIGURED)
					&& (this.showArchivedButton.get("pressed") || !this.configurationStore.usesArchivedContext(operation)) //$NON-NLS-1$
					&& this.configurationStore.usesCurrentDevelopmentLine(operation)) {
				var category = this._getCategoryIcon(operation);
				var what = this.configurationStore.getValue(operation, ProcessConstants.ExtensionDTO.NAME);
				var who = this.configurationStore.getRoleLabel(operation);
				var when = this.configurationStore.getContextLabel(operation);
				var summary = this._getSummary(operation);
				if (what != null && who != null && when != null) {
					// this is a valid configuration
					var rowInfo = [category, what, who, when, summary];
					//Associate the store item with the row, so that code that has a
					//row can access the store item
					rowInfo._item = operation; 
					return rowInfo;
				}
			}
			return null;
		},
		
		_getCategoryIcon: function(operation) {
			var icon = dojo.create("a", { //$NON-NLS-1$
				"class": "categoryIcon", //$NON-NLS-1$ //$NON-NLS-2$
				"href": "#" //$NON-NLS-1$ //$NON-NLS-2$
			});
			icon.title = this.configurationStore.getCategoryLabel(operation);
			this.connect(icon, "onclick", function(event) { //$NON-NLS-1$
				dojo.stopEvent(event);
			});
			var url = this.configurationStore.getCategoryIconUrl(operation);
			if (url !== null) {
				icon.appendChild(dojo.create("img", { //$NON-NLS-1$
					src: url,
					"class": "categoryImage" //$NON-NLS-1$ //$NON-NLS-2$
				}));
			}
			return icon;
		},
		
		/**
		 * Returns a string summary of how many preconditions/follow-up actions
		 * the given operation has configured.
		 */
		_getSummary: function(operation) {
			var preconditions = this.configurationStore.getPreconditions(operation);
			var followups = this.configurationStore.getFollowupActions(operation);
			var preconditionMessage;
			if (preconditions.length === 1) {
				preconditionMessage = this.messages.OperationBehaviorTable_6;
			} else {
				preconditionMessage = dojo.string.substitute(this.messages.OperationBehaviorTable_7, [preconditions.length]);
			}
			var followupMessage;
			if (followups.length === 1) {
				followupMessage = this.messages.OperationBehaviorTable_8;
			} else {
				followupMessage = dojo.string.substitute(this.messages.OperationBehaviorTable_9, [followups.length]);
			}
			var extension = this.extensionStore.fetchItemByIdentity(this.configurationStore.getValue(operation, ProcessConstants.ExtensionDTO.ID));
			var preconditionsPossible = preconditions.length > 0 || this.extensionStore.getPreconditions(extension).length > 0;
			var followupsPossible = followups.length > 0 || this.extensionStore.getFollowupActions(extension).length > 0;
			var text;
			if (preconditionsPossible && !followupsPossible) {
				text = preconditionMessage;
			} else if (!preconditionsPossible && followupsPossible) {
				text = followupMessage;
			} else {
				text = dojo.string.substitute(this.messages.OperationBehaviorTable_5, [preconditionMessage, followupMessage]);
			}
			
			var span = dojo.create("span", {}); //$NON-NLS-1$
			span.appendChild(document.createTextNode(text));
			
			//Associate a TextHoverHelp to the node so that additional
			//information is displayed in a popup on hover
			var self = this;
			this._textHelpHovers.push(new dojo.declare(jazz.app.ua.TextHoverHelp, {
				/*
				 * Override the created HoverPopup's deferred loading method so
				 * that we don't have to create a tree until this node is
				 * actually hovered over.
				 */
				postCreate: function() {
					this.inherited(arguments);
					this.hoverPopup.createContents = dojo.hitch(this, function (popup) {
						this.tree = new com.ibm.team.process.web.ui.internal.admin.projects.OperationBehaviorTree({
							isStatic: true,
							item : operation,
							configurationStore: self.configurationStore,
							emptyText: self.getSummaryTextForNoBehaviorSpecified(preconditionsPossible, followupsPossible)
						});
						return this.tree.domNode;
					});
				},
				/*
				 * This hover help will destroy its content when its destroyed.
				 */
				uninitialize: function() {
					if (this.tree) {
						this.tree.destroyRecursive();
					}
					this.inherited(arguments);
				}
			})({
				contentNode: span,
				helpContentString: span.innerHTML
			}));
			
			return span;
		},
		
		/**
		 * The return value will be used as the hover text for the Summary
		 * column when no preconditions or follow-up actions have been
		 * configured. Sub-classes may override to customize the text.
		 */
		getSummaryTextForNoBehaviorSpecified: function(preconditionsPossible, followupsPossible) {
			if (preconditionsPossible && !followupsPossible) {
				return this.messages.OperationBehaviorTable_13;
			} else if (!preconditionsPossible && followupsPossible) {
				return this.messages.OperationBehaviorTable_14;
			} else {
				return this.messages.OperationBehaviorTable_15;
			}
		},
		
		/**
		 * Populates the table's rows according to the given pageNum, or the
		 * current pageNum if none supplied.
		 */
		showPage: function(pageNum) {
			dojox.xml.parser.removeChildren(this.tbody);
			if (!pageNum && pageNum !== 0) {
				// pageNum was not passed in, just re-show the current page (or
				// first page, if current page not defined yet)
				pageNum = this.pageNum || 0;
			} else {
				this.pageNum = pageNum;
			}
			for (var i = pageNum * this.pageSize; i < Math.min((pageNum+1) * this.pageSize, this.rowInfo.length); i++) {
				this.addRow(this.rowInfo[i]);
			}
		},
		
		/**
		 * Sorts the given column alphabetically (all sortable columns are or
		 * can be mapped to a text value).
		 * 
		 * @param sortBy {int} column to sort by
		 * @param sortByAscending {Boolean} sort ascending or not
		 */
		sortRowInfo: function(sortBy, sortByAscending) {
			var sortFunction = function(a, b) {
				var object = a[sortBy];
				var other = b[sortBy];
				if (sortBy === 0) {
					// comparing category icons
					object = object.title;
					other = other.title;
				}
				return (sortByAscending ? 1 : -1) * ProcessClient.stringCompare(object, other);
			};
			this.rowInfo.sort(sortFunction);
			if (this.allRowInfo !== undefined && this.allRowInfo !== this.rowInfo) {
				this.allRowInfo.sort(sortFunction);
			}
		},
		
		_handleSort: function() {
			this.sortRowInfo(this.styledTable.sortBy, this.styledTable.sortByAscending);
			this.showPage();
		},
		
		_createShowArchivedButton: function() {
			this._filterBar = new jazz.ui.Toolbar();
			this.showArchivedButton = new jazz.ui.toolbar.TwoStateButton({
				title: this.messages.OperationBehaviorTable_18,
				imgSrc: dojo.moduleUrl("com.ibm.team.process.web", "ui/internal/admin/projects/images/milestone_archived_obj.gif"), //$NON-NLS-1$ //$NON-NLS-2$
				onClick: dojo.hitch(this, this.resetTable),
				pressed: this.showArchived
			}).placeAt(this._filterBar);
			this.filterArea.appendChild(this._filterBar.domNode);
		},
		
		_createFilterBox: function() {
			this._filterBox = new jazz.ui.FilterBox({
				onChange: dojo.hitch(this, this._onFilterChange)
			});
			this.filterArea.appendChild(this._filterBox.domNode);
		},
		
		_filter: function(value) {
			if (0 === dojo.trim(value).length) {
				if (this.allRowInfo === undefined || this.rowInfo === this.allRowInfo) {
					//We don't want to do anything in this case. The user has
					//entered whitespace when there was nothing but whitespace
					//in the filter box
					return;
				}
				this.rowInfo = this.allRowInfo;
			} else {
				if (this.allRowInfo === undefined) {
					this.allRowInfo = this.rowInfo;
				}
				var filterTokens = value.split(" "); //$NON-NLS-1$
				
				var matches = function(token, rowInfoField) {
					return -1 !== rowInfoField.indexOf(token);
				};
				
				var filterFunction = function(rowInfo, index, array) {
					for (var i = 0; i < filterTokens.length; i++) {
						var token = filterTokens[i];
						var m1 = matches(token, rowInfo[1].toLowerCase());
						var m2 = matches(token, rowInfo[2].toLowerCase());
						var m3 = matches(token, rowInfo[3].toLowerCase());
						if (!m1 && !m2 && !m3) {
							return false;
						}
					}
					return true;
				};
				
				//If the user added more characters to the filter, we can
				//iterate over this.rowInfo, which is potentially the result of
				//an earlier filtering, since we will be further limiting what
				//is being shown. This will be faster than iterating over
				//this.allRowInfo, because this.rowInfo may be smaller.
				if (this._previousFilterValue !== undefined && value.indexOf(this._previousFilterValue) === 0) {
					this.rowInfo = dojo.filter(this.rowInfo, filterFunction);
				} else {
					this.rowInfo = dojo.filter(this.allRowInfo, filterFunction);
				}
			}
		},
		
		_onFilterChange: function(value) {
			this._filter(value);
			this.showPage(0);
			this.numConfigurationsChanged();
			this._previousFilterValue = value;
		},
		
		_createPagers: function() {
			var uuid = dojox.uuid.generateTimeBasedUuid();
			
			var onClick = dojo.hitch(this, function(pager, start, end) {
				var pageNum = (start-1) / this.pageSize;
				this.showPage(pageNum);
			});
			
			var newPager = dojo.hitch(this, function() {
				return new jazz.ui.PaginationControl({
					total: 0,
					increment: this.pageSize,
					onNextClick: onClick,
					onPreviousClick: onClick,
					name: uuid
				});
			});
			this._pager = newPager();
			this.tableControls.appendChild(this._pager.domNode);
			
			this._bottomPager = newPager();
			this.bottomTableControls.appendChild(this._bottomPager.domNode);
		},
		
		/**
		 * Depending on the number of configurations that are defined or
		 * filtered from view, either show or hide the "there are no
		 * configurations" text at the bottom of the table. Different text
		 * is displayed when there are no configurations as opposed to no
		 * configurations that match the typed-in filter.
		 * 
		 * Subclasses should call this function after adding or removing a
		 * row from the table.
		 */
		numConfigurationsChanged: function() {
			if (this.rowInfo.length === 0) {
				dojo.empty(this.additionalInfo);
				var additionalInfoText;
				if(this.allRowInfo === undefined || this.allRowInfo.length === 0) {
					additionalInfoText = this.messages.OperationBehaviorTable_16;
				} else {
					additionalInfoText = this.messages.OperationBehaviorTable_17;
				}
				this.additionalInfo.appendChild(document.createTextNode(additionalInfoText));
				dojo.style(this.additionalInfo, "display", "block"); //$NON-NLS-1$ //$NON-NLS-2$
			} else {
				dojo.style(this.additionalInfo, "display", "none"); //$NON-NLS-1$ //$NON-NLS-2$
			}
			
			//If there are no more configurations on the page we're displaying
			//and we aren't displaying the first page, flip to the last
			//available page
			if (this.pageNum !== 0 && this.rowInfo.length <= this.pageSize * this.pageNum) {
				this.showPage(this.rowInfo.length === 0 ? 0 : Math.ceil(this.rowInfo.length / this.pageSize) - 1);
			}
			
			//Helper to update a pagination control to reflect the new total
			//number of items. It's startPage must match the current page
			//number for the text to be updated correctly. eg. if we're
			//looking at the second page, we want the text to read
			//"26-50 of 108", which will only happen if the pager's startPage
			//is 1 when setTotal() is called.
			var updatePagerTotal = dojo.hitch(this, function(pager) {
				pager.startPage = this.pageNum;
				pager.setTotal(this.rowInfo.length);
				pager.startPage = 0;
			});
			
			//Update both pagination widgets
			updatePagerTotal(this._pager);
			updatePagerTotal(this._bottomPager);
			
			//Only display the bottom pagination widget if there is more than
			//one page of configurations
			if (this.rowInfo.length <= this.pageSize) {
				dojo.style(this._bottomPager.domNode, "display", "none"); //$NON-NLS-1$ //$NON-NLS-2$
			} else {
				dojo.style(this._bottomPager.domNode, "display", "block"); //$NON-NLS-1$ //$NON-NLS-2$
			}
		},
		
		// =====================================================================
		// React to events from the roles and contexts stores
		
		_onNewRole: function(newItem, parentInfo) {
			this._resetIfRoleUsed(this.rolesStore.getValue(newItem, ProcessConstants.ProcessRoleDTO.ID));
		},
		
		_onSetRole: function(item, attribute, oldValue, newValue) {
			if (attribute === ProcessConstants.ProcessRoleDTO.LABEL) {
				// update the row info of any rows using this role
				var roleId = this.rolesStore.getValue(item, ProcessConstants.ProcessRoleDTO.ID);
				dojo.forEach(this.allRowInfo || this.rowInfo, dojo.hitch(this, function(rowInfo) {
					if (this.configurationStore.getValue(rowInfo._item, ProcessConstants.BehaviorConfigurationDTO.ROLE_ID) === roleId) {
						rowInfo[ROLE_COLUMN] = this.rolesStore.getRoleLabel(item);
					}
				}));
				// update the role column of any visible rows using this role
				dojo.forEach(this.tbody.childNodes, dojo.hitch(this, function(row) {
					if (row._rowInfo && this.configurationStore.getValue(row._rowInfo._item, ProcessConstants.BehaviorConfigurationDTO.ROLE_ID) === roleId) {
						row.childNodes[ROLE_COLUMN].innerHTML = row._rowInfo[ROLE_COLUMN];
					}
				}));
			} else if (attribute === ProcessConstants.ProcessRoleDTO.ID) {
				this._resetIfRoleUsed(oldValue);
				this._resetIfRoleUsed(newValue);
			}
		},
		
		_onDeleteRole: function(item) {
			this._resetIfRoleUsed(this.rolesStore.getValue(item, ProcessConstants.ProcessRoleDTO.ID));
		},
		
		_onNewContext: function(newItem, parentInfo) {
			var context = this.contextStore.getValue(newItem, "enabledContext"); //$NON-NLS-1$
			var contextId = this.contextStore.getValue(newItem, "contextId"); //$NON-NLS-1$
			this._resetIfContextUsed(context, contextId);
		},
		
		_onSetContext: function(item, attribute, oldValue, newValue) {
			var context = this.contextStore.getValue(item, "enabledContext"); //$NON-NLS-1$
			if (attribute === "name") { //$NON-NLS-1$
				// update the row info of any rows using this context
				var contextId = this.contextStore.getValue(item, "contextId"); //$NON-NLS-1$
				dojo.forEach(this.allRowInfo || this.rowInfo, dojo.hitch(this, function(rowInfo) {
					if (this.configurationStore.getValue(rowInfo._item, ProcessConstants.BehaviorConfigurationDTO.CONTEXT) === context && this.configurationStore.getValue(rowInfo._item, ProcessConstants.BehaviorConfigurationDTO.CONTEXT_ID) === contextId) {
						rowInfo[CONTEXT_COLUMN] = this.configurationStore.getContextLabel(rowInfo._item);
					}
				}));
				// update the when column of any visible rows using this context
				dojo.forEach(this.tbody.childNodes, dojo.hitch(this, function(row) {
					if (row._rowInfo && this.configurationStore.getValue(row._rowInfo._item, ProcessConstants.BehaviorConfigurationDTO.CONTEXT) === context && this.configurationStore.getValue(row._rowInfo._item, ProcessConstants.BehaviorConfigurationDTO.CONTEXT_ID) === contextId) {
						row.childNodes[CONTEXT_COLUMN].innerHTML = row._rowInfo[CONTEXT_COLUMN];
					}
				}));
			} else if (attribute === "archived") { //$NON-NLS-1$
				var contextId = this.contextStore.getValue(item, "contextId"); //$NON-NLS-1$
				this._resetIfContextUsed(context, contextId);
			} else if (attribute === "contextId") { //$NON-NLS-1$
				this._resetIfContextUsed(context, oldValue);
				this._resetIfContextUsed(context, newValue);
			} else if (attribute === "currentSelectedTimeline" && newValue === true) { //$NON-NLS-1$
				this.resetTable();
			}
		},
		
		_onDeleteContext: function(item) {
			var context = this.contextStore.getValue(item, "enabledContext"); //$NON-NLS-1$
			var contextId = this.contextStore.getValue(item, "contextId"); //$NON-NLS-1$
			this._resetIfContextUsed(context, contextId);
		},
		
		// =====================================================================
		
		_resetIfRoleUsed: function(roleId) {
			args = {};
			args[ProcessConstants.BehaviorConfigurationDTO.ROLE_ID] = roleId;
			this._resetIfConfigurationsUse(args);
		},
		
		_resetIfContextUsed: function(context, contextId) {
			args = {};
			args[ProcessConstants.BehaviorConfigurationDTO.CONTEXT] = context;
			args[ProcessConstants.BehaviorConfigurationDTO.CONTEXT_ID] = contextId;
			this._resetIfConfigurationsUse(args);
		},
		
		_resetIfConfigurationsUse: function(query) {
			query[ProcessConstants.BehaviorConfigurationDTO.CONFIGURED] = true;
			var match = false;
			this.configurationStore.fetch({
				query: query,
				onItem: function(item) {
					match = true;
				}
			});
			if (match) {
				// run async to let store changes actually propagate through
				// (e.g. aboutToDelete occurs before the delete really happens)
				setTimeout(dojo.hitch(this, this.resetTable));
			}
		},
		
		/**
		 * Re-populates the table based on the data in the configuration store,
		 * maintaining the same sort-order, page number (if possible), and
		 * filter.
		 */
		resetTable: function() {
			var rowInfo = this.rowInfo;
			var pageNum = this.pageNum;
			var filter = this._filterBox.value;	
			var hovers = this._textHelpHovers;
			this._textHelpHovers = [];
			// clear the table, but keep rowInfo (and the summary hovers since
			// rowInfo entries refer to them)
			this.clearTable();
			this.rowInfo = rowInfo;
			this._textHelpHovers = hovers;
			// rebuild rowInfo
			var newRowInfo = [];
			this.configurationStore.fetch({
				onItem: dojo.hitch(this, function(item) {
					var info = this.getRowInfo(item);
					if (info) {
						var existingInfo = this.findRowInfo(item);
						if (existingInfo) {
							// re-use any previous infos that should still exist
							info = existingInfo;
						}
						newRowInfo.push(info);
					}
				})
			});
			this.rowInfo = newRowInfo;
			// re-apply the filter, if there was one
			if (filter) {
				this._filter(filter);
				this._filterBox.setValue(filter);
			}
			// re-show the page that was showing before (if possible)
			this._initializeTable(pageNum);
		},
		
		_destroyStyledTable: function() { 
			if (this.styledTable) {
				this.styledTable.destroyRecursive();
				dojox.xml.parser.removeChildren(this.tableContents);
				delete this.styledTable;
				delete this.tbody;
			}		
		},
		
		uninitialize: function() {
			this.clearTable();
			this._destroyStyledTable();
			
			if (this._filterBar) {
				this._filterBar.destroyRecursive();
				delete this._filterBar;
			}
			
			if (this._filterBox) {
				this._filterBox.destroyRecursive();
				delete this._filterBox;
			}
			
			if (this._pager) {
				this._pager.destroyRecursive();
				delete this._pager;
			}
			
			if (this._bottomPager) {
				this._bottomPager.destroyRecursive();
				delete this._bottomPager;
			}
			
			this.inherited(arguments);
		}
		
	});
	
})();
