/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2013. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function(){
	dojo.provide("com.ibm.team.process.web.ui.internal.admin.projects.PermissionsCustomizeDialog"); //$NON-NLS-1$
	
	dojo.require("dijit._Widget"); //$NON-NLS-1$
	dojo.require("dijit._Templated"); //$NON-NLS-1$
	dojo.require("dojo.i18n"); //$NON-NLS-1$
	dojo.require("jazz.ui.Dialog"); //$NON-NLS-1$
	dojo.require("jazz.ui.MessageBox"); //$NON-NLS-1$
	dojo.require("dojo.data.ItemFileWriteStore"); //$NON-NLS-1$
	dojo.require("com.ibm.team.process.web.ui.internal.admin.projects.UIHelper"); //$NON-NLS-1$
	dojo.require("com.ibm.team.process.web.ui.internal.admin.projects.PermissionsWorkingCopy"); //$NON-NLS-1$
	dojo.require("com.ibm.team.process.web.ui.internal.admin.projects.PermissionsStore"); //$NON-NLS-1$)
	
	dojo.requireLocalization("com.ibm.team.process.web.ui.internal", "Messages"); //$NON-NLS-1$ //$NON-NLS-2$
	
	var UIHelper = com.ibm.team.process.web.ui.internal.admin.projects.UIHelper;
	var PermittedType = com.ibm.team.process.web.ui.internal.admin.projects.PermissionsStore.PermittedType;
		
	dojo.declare("com.ibm.team.process.web.ui.internal.admin.projects.PermissionsCustomizeDialog", [dijit._Widget, dijit._Templated], { //$NON-NLS-1$
		templatePath: dojo.moduleUrl("com.ibm.team.process.web", "ui/internal/admin/projects/templates/PermissionsCustomizeDialog.html"), //$NON-NLS-1$ //$NON-NLS-2$
		
		/**
		 * Dialog where a user can view permissions that are configured in
		 * the nearest parent scope, (as computed before creating this dialog),
		 * optionally modify the permissions, and click OK to customize
		 * permissions in the current area starting with the values in the
		 * dialog, or Cancel to not customize.
		 * 
		 * @param args {Object} An object with the following properties:
		 * 
		 * workingCopy	Instance of PermissionsWorkingCopy that can be
		 * 				consulted to update permissions
		 * 
		 * inheritedPermissions		Object that is returned from function
		 * 							getInheritedPermissions in
		 * 							PermissionsWorkingCopy, which represents
		 * 							the inherited permissions
		 * 
		 * permissionsStore		Store of permissions that permissions are to
		 * 						be customized for
		 * 
		 * operationId	Identifier of operation to customize
		 * 
		 * contextItem	Item in permissionsStore that that permissions are to be
		 * 				customized for
		 * 
		 * roleId	Role identifier that permissions are to be customized for
		 * 
		 * processAreaEditor	Instance of AbstractProcessAreaEditor that hosts
		 * 						the Permissions tab
		 * 
		 * processAreaStore		Store of artifacts related to project or team,
		 * 						used for time scopes such as timelines and
		 * 						iteration types
		 * 
		 * okCallback	Optional function that will be invoked when the user
		 * 				clicks ok, after permissions have been updated with
		 * 				the values in the dialog
		 */
		constructor: function(args) {
			this.messages = dojo.i18n.getLocalization("com.ibm.team.process.web.ui.internal.admin.projects", "Messages"); //$NON-NLS-1$ //$NON-NLS-2$
		},
		
		postCreate: function() {
			this.dialog = new jazz.ui.Dialog({
				primaryTitle: this.messages.PermissionsCustomizeDialog_1,
				contentNode: this.domNode,
				width: "50em", //$NON-NLS-1$
				destroyContents: true
			});
			
			//If the class is not removed, the entire contents of the dialog
			//scrolls; not just the elements we set overflow: auto on
			
			dojo.toggleClass(this.dialog.domNode.parentNode.parentNode, "collapsible", false); //$NON-NLS-1$
			
			this.workingCopy.loadPermissionExtensions(
				dojo.hitch(this, function(permissionExtensionsStore) {
					this._initializePermissionsStore(permissionExtensionsStore);
					
					this.dialogSummaryTextNode.innerHTML = jazz.util.html.removeScript(this._getSummaryTextContent());
					
					this._createPermissionsTable(permissionExtensionsStore);
					
					this.okButton.disabled = false;
				}),
				function(error) {
					this.processAreaEditor.handleServerResponseError(error, this.messages.PermissionsCustomizeDialog_5);
					this.close();
				}
			);
		},
		
		startup: function() {
			UIHelper.resizeI18NButtons(this.dialogFooter, [this.okButton, this.cancelButton]);
			this.inherited(arguments);
		},
		
		/**
		 * Create local store for permissions of single operation.
		 */
		_initializePermissionsStore: function(permissionExtensionsStore) {
			this._permissionsStore = new com.ibm.team.process.web.ui.internal.admin.projects.PermissionsStore({
				permissionExtensionsStore: permissionExtensionsStore,
				processAreaStore: this.processAreaStore
			});
			this._permissionsStore.ensureOperationInStore(this.operationId, PermittedType.NOT_PERMITTED);
			
			//Populate store with the permissions and denial explanation
			//from the inherited permissions. We assume that the permission is
			//either not permitted or permitted, and that it is not final
			this._permissionsStore.fetch({
				onItem: dojo.hitch(this, function(operationItem) {
					if (this.inheritedPermissions.denialExplanation !== undefined && 
						this.inheritedPermissions.denialExplanation !== null) {
						this._permissionsStore.setDenialExplanation(operationItem, null, this.inheritedPermissions.denialExplanation);
						this._permissionsStore.setDenialExplanationConfigured(operationItem, null, true);
					}
					
					var items = [operationItem];
					while (items.length !== 0) {
						var item = items.shift();
						var actionId = this._permissionsStore.getValue(item, ProcessConstants.ExtensionDTO.ID);
						var permission = this.inheritedPermissions.getPermission(actionId);
						if (PermittedType.PERMITTED === permission) {
							this._permissionsStore.setPermission(item, actionId, this.processAreaStore.alwaysRootItem, permission);
						}
						items = items.concat(this._permissionsStore.getValues(item, ProcessConstants.ExtensionDTO.ITEMS));
					}
				})
			});
		},
		
		_getSummaryTextContent: function() {
			var result;
			var operationName = this.workingCopy.getName(this.operationId);
			var roleName = this.processAreaEditor.getRolesStore().getRoleLabel(this.processAreaEditor.getRolesStore().fetchRoleById(this.roleId));
			if (this.contextItem === this.processAreaStore.alwaysRootItem) {
				result = dojo.string.substitute(this.messages.PermissionsCustomizeDialog_2, [operationName, roleName]); //$NON-NLS-1$
			} else {
				var contextName = this.processAreaStore.getValue(this.contextItem, "name"); //$NON-NLS-1$
				var context = this.processAreaStore.getValue(this.contextItem, "enabledContext"); //$NON-NLS-1$
				if (ProcessConstants.PermissionContext.ITERATION_TYPE === context) {
					result = dojo.string.substitute(this.messages.PermissionsCustomizeDialog_3, [operationName, roleName, contextName]); //$NON-NLS-1$
				} else {
					result = dojo.string.substitute(this.messages.PermissionsCustomizeDialog_4, [operationName, roleName, contextName]); //$NON-NLS-1$
				}
			}
			return result;
		},
		
		/**
		 * Create the dialog's contents: a modified PermissionsForRoleTable that
		 * shows permissions for one operation only and is backed by a
		 * PermissionsStore that isn't tracked by PermissionsWorkingCopy.
		 */
		_createPermissionsTable: function(permissionExtensionsStore) {
			var operationItem;
			permissionExtensionsStore.fetch({
				query: {
					id: this.operationId
				},
				queryOptions: {
					deep: true
				},
				onItem: dojo.hitch(this, function (item) {
					operationItem = item;
				})
			});
			
			var model = new com.ibm.team.process.web.ui.internal.admin.projects.PermissionsCustomizeDialogPermissionExtensionsModel({
				store: permissionExtensionsStore,
				workingCopy: this.workingCopy,
				operationItem: operationItem
			});
			
			var self = this;
			
			this._permissionsTable = new com.ibm.team.process.web.ui.internal.admin.projects.PermissionsForRoleTable({
				model: model,
				workingCopy: this.workingCopy,
				processAreaEditor: this.processAreaEditor,
				processAreaStore: this.processAreaStore,
				categoryLevel: -1,
				operationLevel: 0,
				canInherit: false,
				
				/**
				 * Overridden as no-op
				 */
				heightChanged: function() {
				},
				
				/**
				 * Overridden to call successCallback immediately since there is
				 * nothing to load.
				 */
				fetchPermissions: function(successCallback, errorCallback) {
					successCallback();
				},
				
				/**
				 * Overridden so that data is written to/read from a local working copy
				 * instead of the actual store in PermissionsWorkingCopy.
				 */
				getPermissionsStore: function(roleId, contextItem) {
					return self._permissionsStore;
				}
			});
			
			this.connect(this._permissionsStore, "onSet", dojo.hitch(this, function(item, attribute, oldValue, newValue) { //$NON-NLS-1$
				this._permissionsTable.reloadPermissions();
			}));
			
			dojo.place(this._permissionsTable.domNode, this.permissionsForRoleTableContainer); //$NON-NLS-1$
			
			this._permissionsTable.reloadPermissions();
		},
		
		_okCallback: function(e) {
			this.permissionsStore.setPermission(
				null,
				this.operationId,
				this.contextItem,
				PermittedType.NOT_PERMITTED
			);

			var localStore = this._permissionsTable.getPermissionsStore();
			
			localStore.fetch({
				onItem: dojo.hitch(this, function(operationItem) {
					var items = [operationItem];
					while (items.length !== 0) {
						var item = items.shift();
						var actionId = localStore.getValue(item, ProcessConstants.ExtensionDTO.ID);
						var permission = localStore.getPermission(item);
						if (PermittedType.PERMITTED === permission) {
							this.permissionsStore.setPermission(null, actionId, this.contextItem, permission);
						}
						items = items.concat(localStore.getValues(item, ProcessConstants.ExtensionDTO.ITEMS));
					}
					
					if (localStore.isDenialExplanationConfigured(operationItem)) {
						var denialExplanation = localStore.getDenialExplanation(operationItem);
						this.permissionsStore.setDenialExplanation(null, this.operationId, denialExplanation);
						this.permissionsStore.setDenialExplanationConfigured(null, this.operationId, true);
					}
					
					var isFinal = localStore.isFinal(operationItem);
					if (isFinal) {
						this.permissionsStore.setFinal(null, this.operationId, isFinal);
					}
				})
			});
			
			if (this.okCallback) {
				this.okCallback();
			}
			
			this.close(e);
		},
		
		_cancelCallback: function(e) {
			this.close(e);
		},
		
		uninitialize: function(){
			this.inherited(arguments);
			
			if (this._permissionsTable) {
				this._permissionsTable.destroyRecursive();
				delete this._permissionsTable;
			}
		},
		
		close: function(e) {
			if (e) {
				dojo.stopEvent(e);
			}
			this.dialog.close();
		}
	});
	
	dojo.require("com.ibm.team.process.web.ui.internal.admin.projects.PermissionExtensionsModel"); //$NON-NLS-1$
	
	/**
	 * Subclass of com.ibm.team.process.web.ui.internal.admin.projects.PermissionExtensionsModel
	 * that allows permissions to be viewed and edited in the customize dialog.
	 * Updates to the editor's permissions working copy are delayed until the
	 * user clicks 'OK' in the customize dialog. Until that time, they are
	 * stored locally in this object.
	 */
	dojo.declare("com.ibm.team.process.web.ui.internal.admin.projects.PermissionsCustomizeDialogPermissionExtensionsModel", [com.ibm.team.process.web.ui.internal.admin.projects.PermissionExtensionsModel], { //$NON-NLS-1$
		
		/**
		 * In addition to arguments as com.ibm.team.process.web.ui.internal.admin.projects.PermissionExtensionsModel,
		 * accepts the following:
		 * 
		 * operationItem	Item in the store for the operation that is to be
		 * 					customized. Only this operation and its children
		 * 					will be returned from the model.
		 */
		constructor: function(args) {
		},
		
		/**
		 * Override to return a single operation for the root children.
		 */
		getChildren: function(/*dojo.data.Item*/ parentItem, /*function(items)*/ callback, /*function*/ onError) {
			if (parentItem === this.root) {
				callback([this.operationItem]);
			} else {
				this.inherited(arguments);
			}
		}
	});
})();
