/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2013. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function(){
	dojo.provide("com.ibm.team.process.web.ui.internal.admin.projects.PermissionsEditor"); //$NON-NLS-1$
	
	dojo.require("com.ibm.team.process.web.ui.internal.admin.projects.PermissionsByRoleEditor"); //$NON-NLS-1$
	dojo.require("com.ibm.team.process.web.ui.internal.admin.projects.PermissionsByActionEditor"); //$NON-NLS-1$
	dojo.require("com.ibm.team.process.web.ui.internal.admin.projects.PermissionsScopeSection"); //$NON-NLS-1$
	dojo.require("com.ibm.team.process.web.ui.internal.admin.projects.PermissionsWorkingCopy"); //$NON-NLS-1$
	
	dojo.require("dijit.layout.BorderContainer"); //$NON-NLS-1$
	dojo.require("dijit.layout.ContentPane"); //$NON-NLS-1$
	dojo.require("dijit._Templated"); //$NON-NLS-1$
	dojo.require("dijit._Widget"); //$NON-NLS-1$
	
	dojo.require("dojo.i18n"); //$NON-NLS-1$
	
	dojo.require("dojox.xml.parser"); //$NON-NLS-1$
	
	dojo.require("jazz.util.html"); //$NON-NLS-1$
	dojo.require("jazz.ui.toolbar.Button"); //$NON-NLS-1$
	
	dojo.requireLocalization("com.ibm.team.process.web.ui.internal", "Messages"); //$NON-NLS-1$ //$NON-NLS-2$
	
	var MODE_BY_ROLE = 'editPermissionsChoiceByRole'; //$NON-NLS-1$
	var MODE_BY_ACTION = 'editPermissionsChoiceByAction'; //$NON-NLS-1$
	
	var RADIO_IDS_TO_CONTENT_WIDGETS = {};
	RADIO_IDS_TO_CONTENT_WIDGETS[MODE_BY_ROLE] = "com.ibm.team.process.web.ui.internal.admin.projects.PermissionsByRoleEditor"; //$NON-NLS-1$
	RADIO_IDS_TO_CONTENT_WIDGETS[MODE_BY_ACTION] = "com.ibm.team.process.web.ui.internal.admin.projects.PermissionsByActionEditor"; //$NON-NLS-1$

	var MEMENTO_SHOWING_TIMEFRAMES = "PermissionsEditor#showingTimeframes"; //$NON-NLS-1$
	var MEMENTO_EDITOR_MODE = "PermissionsEditor#editorMode"; //$NON-NLS-1$
	
	/**
	 * Tab to edit permissions for a project or team, either by role with a
	 * list of roles and tree of permitted actions, or by operation with a
	 * tree of operations and table of permitted roles.
	 */
	dojo.declare("com.ibm.team.process.web.ui.internal.admin.projects.PermissionsEditor", [dijit._Widget, dijit._Templated], { //$NON-NLS-1$
		templatePath: dojo.moduleUrl("com.ibm.team.process.web", "ui/internal/admin/projects/templates/PermissionsEditor.html"), //$NON-NLS-1$ //$NON-NLS-2$
		
		constructor: function(args) {
			this._processAreaEditor = args.editor;
			this._processAreaStore = this._processAreaEditor._getProcessAreaStore();
			this._owningApplicationKey = args.owningApplicationKey;
			
			this.messages = dojo.i18n.getLocalization("com.ibm.team.process.web.ui.internal.admin.projects", "Messages"); //$NON-NLS-1$ //$NON-NLS-2$
			
			this._subEditorContainers = {};
			this._heightChangedInvocationNeeded = {};
		},
		
		/**
		 * Called by this editor's container, to add contents to the right
		 * side of the editor header area, (a link to configure permissions
		 * for different time periods if they aren't already).
		 */
		getHeaderActionArea: function() {
			return this._headerActionArea;
		},
		
		postCreate: function() {
			this._createHeaderActions();
			
			var setRadioButtonLabel = dojo.hitch(this, function(radioLabelElement, messageKey) {
				radioLabelElement.innerHTML = dojo.string.substitute(messageKey, ["<span class=\"emphasizedText\">", "</span>"]); //$NON-NLS-1$ //$NON-NLS-2$	
			});
			setRadioButtonLabel(this.editPermissionsChoiceByRoleRadioLabel, this.messages.PermissionsEditor_2);
			setRadioButtonLabel(this.editPermissionsChoiceByActionRadioLabel, this.messages.PermissionsEditor_3);
			
			for (var buttonId in RADIO_IDS_TO_CONTENT_WIDGETS) {
				this.connect(this[buttonId], "onclick", "_radioButtonSelected"); //$NON-NLS-1$ //$NON-NLS-2$ 
			}
			
			this._connects = [];
			
			this.refresh();
		},
		
		refresh: function() {
			this.clearState();
			
			this._workingCopy = new com.ibm.team.process.web.ui.internal.admin.projects.PermissionsWorkingCopy({
				editorContext: this._processAreaEditor.context,
				getProcessAreaTitle: dojo.hitch(this, function() {
					return this._processAreaEditor.name.value;
				}),
				isChildArea: this._isChildArea(),
				processAreaItemId: this._processAreaEditor.itemId,
				owningApplicationKey: this._owningApplicationKey,
				rolesStore: this._processAreaEditor.getRolesStore(),
				processAreaStore: this._processAreaStore,
				startLoadingFunction: dojo.hitch(this, this._startLoading),
				stopLoadingFunction: dojo.hitch(this, this._stopLoading)
			});
			
			this._connects.push(this.connect(this._workingCopy, "startBatchUpdate", "_startBatchUpdate")); //$NON-NLS-1$ //$NON-NLS-2$
			this._connects.push(this.connect(this._workingCopy, "permissionChanged", "_permissionChanged")); //$NON-NLS-1$ //$NON-NLS-2$
			this._connects.push(this.connect(this._workingCopy, "endBatchUpdate", "_endBatchUpdate")); //$NON-NLS-1$ //$NON-NLS-2$
			
			this._processAreaStore = this._processAreaEditor._getProcessAreaStore();
			
			var recalculateTimeframeLinkVisibility = dojo.hitch(this, function(item) {
				setTimeout(dojo.hitch(this, function() {
					dojo.toggleClass(this._timeframeLink.domNode, "hidden", this._workingCopy.permissionsAreConfiguredForSpecificScope()); //$NON-NLS-1$
				}));
			});
			this._connects.push(this.connect(this._processAreaStore, "aboutToDelete", recalculateTimeframeLinkVisibility)); //$NON-NLS-1$
			this._connects.push(this.connect(this._processAreaEditor.getRolesStore(), "aboutToDelete", recalculateTimeframeLinkVisibility)); //$NON-NLS-1$
			
			this._workingCopy.setIsChildArea(this._isChildArea());
			this._workingCopy.setProcessAreaStore(this._processAreaStore);
			
			this.editor.addSaveParticipant(this, "com.ibm.team.process.web.ui.internal.admin.projects.PermissionsEditor"); //$NON-NLS-1$
			
			//Load the information about configured permissions scopes, and
			//display the section to select a time period or the hyperlink to
			//open that section if non-Always scopes are configured. The
			//section is also opened if it had been opened before a
			//refresh/save.
			this._workingCopy.loadConfiguredScopes(
				dojo.hitch(this, function() {
					var nonAlwaysScopesConfigured = this._workingCopy.permissionsAreConfiguredForSpecificScope();
					this._toggleShowTimeFrameSection(nonAlwaysScopesConfigured || this._editorState[MEMENTO_SHOWING_TIMEFRAMES] === true, !nonAlwaysScopesConfigured);
					this._configuredScopesLoaded = true;
					if (this._currentEditorId !== undefined) {
						this._subEditorContainers[this._currentEditorId].editor.configuredScopesLoaded();
					}
				}),
				dojo.hitch(this, function(error) {
					this._processAreaEditor.handleServerResponseError(error, this.messages.PermissionsTable_7);
				})
			);
			
			if (this._processAreaEditor.isTheActiveTab(ProcessConstants.TabEnum.PERMISSIONS)) {	
				this._selectDefaultEditorMode();
			}
		},
		
		clearState: function() {
			this.editor.removeSaveParticipant("com.ibm.team.process.web.ui.internal.admin.projects.PermissionsEditor"); //$NON-NLS-1$
			
			this._editorState = {};
			
			for(var containerId in this._subEditorContainers) {
				this._subEditorContainers[containerId].editor.saveState(this._editorState);
				
				if (this._subEditorContainers[containerId].scopeSection) {
					this._subEditorContainers[containerId].container.removeChild(this._subEditorContainers[containerId].scopeSection);
				}
				
				//Recursively destroys all sections and their content widgets,
				//including the sub-editor columns
				if (this._subEditorContainers[containerId].container) {
					this._subEditorContainers[containerId].container.destroyRecursive();
				}
				
				//Destroy the top-level sub-editor widget 
				if (this._subEditorContainers[containerId].editor) {
					this._subEditorContainers[containerId].editor.destroyRecursive();
				}
			}
			
			this._editorState[MEMENTO_SHOWING_TIMEFRAMES] = this._showingTimeframes;
			
			if (this._scopeSection !== undefined) {
				if (this._showingTimeframes) {
					this._scopeSection.saveState(this._editorState);
				}
				
				this._scopeSection.destroyRecursive();
			}
			
			if (this.editPermissionsChoiceByRole.checked) {
				this._editorState[MEMENTO_EDITOR_MODE] = MODE_BY_ROLE;
			} else if (this.editPermissionsChoiceByAction.checked) {
				this._editorState[MEMENTO_EDITOR_MODE] = MODE_BY_ACTION;
			}
			
			this._subEditorContainers = {};
			this._heightChangedInvocationNeeded = {};
			this._currentEditorId = undefined;
			
			this._showingTimeframes = false;
			dojo.toggleClass(this._timeframeLink.domNode, "hidden", true); //$NON-NLS-1$
			this._lastTimePeriodSelected = undefined;
			this._scopeSection = undefined;
			
			if (this._workingCopy) {
				this._workingCopy.clearState();
			}
			
			dojo.empty(this.editorContainer);
			
			this._loadingMessages = [];
			
			this._configuredScopesLoaded = false;
			
			//this._connects will be an array of arrays, (this.connect returns
			//an array)
			dojo.forEach(this._connects, function(connects) {
				dojo.forEach(connects, dojo.disconnect);
			});
			this._connects = [];
		},
		
		activateTab: function() {
			this.inherited(arguments);

			if (this._heightChangedInvocationNeeded[this._currentEditorId]) {
				setTimeout(dojo.hitch(this, function() {
					this._heightChanged(this._currentEditorId);
					delete this._heightChangedInvocationNeeded[this._currentEditorId];
				}), 0);
			}
			
			if (this._scopeSection !== undefined) {
				this._scopeSection.tabActivated();
			}
			
			if (this._currentEditorId === undefined) {
				setTimeout(dojo.hitch(this, function() {
					this._selectDefaultEditorMode();
				}));
			} else {
				this._subEditorContainers[this._currentEditorId].editor.editorShown();
			}
		},
		
		_selectDefaultEditorMode: function() {
			var radioButton;
			if (this._editorState[MEMENTO_EDITOR_MODE] === undefined || this._editorState[MEMENTO_EDITOR_MODE] === MODE_BY_ROLE) {
				radioButton = this.editPermissionsChoiceByRole;
			} else {
				radioButton = this.editPermissionsChoiceByAction;
			}
			radioButton.checked = true;
			this._handleRadioButtonSelected(radioButton);
		},
		
		changesPending: function() {
			return this._workingCopy.changesPending();
		},
		
		saved: function(saveResponse) {
			this._workingCopy.saved(saveResponse);
		},
		
		aboutToBeSaved: function(saveObj) {
			this._workingCopy.aboutToBeSaved(saveObj);
		},
		
		_permissionChanged: function(store, item, roleId, contextItem) {
			//Only react to changes if we're not in the middle of a batch of
			//updates. Repeated calls to setDirty() can be expensive.
			if (this._workingCopyBatchUpdating !== true) {
				this._processAreaEditor.setDirty(this._processAreaEditor.changesPending());
				
				dojo.toggleClass(this._timeframeLink.domNode, "hidden", this._workingCopy.permissionsAreConfiguredForSpecificScope()); //$NON-NLS-1$
			}
		},
		
		_startBatchUpdate: function() {
			this._workingCopyBatchUpdating = true;
		},
		
		_endBatchUpdate: function() {
			delete this._workingCopyBatchUpdating;
			this._permissionChanged();
		},
		
		_startLoading: function(message) {
			this._loadingMessages.push(message);
			var subEditorContainer = this._subEditorContainers[this._currentEditorId];
			if (subEditorContainer) {
				subEditorContainer.editor.startLoading();
			}
			if (this._scopeSection !== undefined) {
				this._scopeSection.startLoading();
			}
			if (this._processAreaEditor.isTheActiveTab(ProcessConstants.TabEnum.PERMISSIONS)) {
				this._processAreaEditor.startLoading(message);
			}
		},
		
		_stopLoading: function(message) {
			for (var i = 0; i < this._loadingMessages.length; i++) {
				if (this._loadingMessages[i] === message) {
					this._loadingMessages.splice(i, 1);
				}
			}
			var subEditorContainer = this._subEditorContainers[this._currentEditorId];
			if (subEditorContainer) {
				subEditorContainer.editor.stopLoading();
			}
			if (this._scopeSection !== undefined) {
				this._scopeSection.stopLoading();
			}
			if (this._loadingMessages.length == 0) {
				if (this._processAreaEditor.isTheActiveTab(ProcessConstants.TabEnum.PERMISSIONS)) {
					this._processAreaEditor.stopLoading();
				}
			} else {
				this._processAreaEditor.startLoading(this._loadingMessages[0]);
			}
		},
		
		/**
		 * Create a section that will be displayed to the right-side of the
		 * header area
		 */
		_createHeaderActions: function() {
			this._headerActionArea = dojo.create("div", {}); //$NON-NLS-1$
			dojo.addClass(this._headerActionArea, "com-ibm-team-process-web-ui-internal-PermissionsEditor"); //$NON-NLS-1$
			
			// Create a hyperlink to show the timeframes
			this._timeframeLink = new jazz.ui.toolbar.Button({
				label: this.messages.PermissionsEditor_1,
				imgSrc: dojo.moduleUrl("com.ibm.team.process.web", "ui/internal/admin/projects/images/milestone_obj.gif"), //$NON-NLS-1$ //$NON-NLS-2$
				onClick: dojo.hitch(this, function() {
					this._toggleShowTimeFrameSection(!this._showingTimeframes, true);
					this._timeframeLink.focusNode.blur();
				})
			}).placeAt(this._headerActionArea, "only"); //$NON-NLS-1$
		},
		
		_toggleShowTimeFrameSection: function(show, allowHide) {
			dojo.toggleClass(this._timeframeLink.domNode, "hidden", show && !allowHide); //$NON-NLS-1$
			this._showingTimeframes = show;
			if (show) {
				if (this._currentEditorId) {
					this._addTimeframeSection(this._currentEditorId);
				}
				
				this._timeframeLink.set("label", this.messages.PermissionsEditor_5); //$NON-NLS-1$
			} else {
				if (this._scopeSection) {
					this._scopeSection.selectAlways();
				}
				
				this._timeframeLink.set("label", this.messages.PermissionsEditor_1); //$NON-NLS-1$
				
				for (var containerKey in this._subEditorContainers) {
					var contentContainer = this._subEditorContainers[containerKey]; 
					if (contentContainer.scopeSection) {
						contentContainer.container.removeChild(contentContainer.scopeSection);
						contentContainer.scopeSection = undefined;
						
						if (contentContainer.editor.containerClassWhenShowingTimeframe !== undefined) {
							dojo.removeClass(contentContainer.centerSection.domNode, contentContainer.editor.containerClassWhenShowingTimeframe);
						}
						
						this._heightChanged(this._currentEditorId);
					}
				}
			}
		},
		
		/**
		 * Callback for the onclick event for the editPermissionsChoice*
		 * buttons. Separate from _handleRadioButtonSelected so that the latter
		 * can be called programmatically.
		 */
		_radioButtonSelected: function(event) {
			this._handleRadioButtonSelected(event.target);
		},
		
		/**
		 * Show the content associated with one of the editPermissionsChoice*
		 * buttons, and hide the content for the non-selected buttons.
		 * 
		 * @param {DOM node} button
		 */
		_handleRadioButtonSelected: function(button) {
			if (dojo.marginBox(this.domNode).h === 0) {
				//Re-schedule until called after content is visible on the page
				setTimeout(dojo.hitch(this, this._handleRadioButtonSelected, button), 30);
				return;
			}
			
			for (var buttonId in RADIO_IDS_TO_CONTENT_WIDGETS) {
				if (button.id === buttonId) {
					if (this._currentEditorId !== buttonId) {
						delete this._currentEditorId;
						
						if (!this._subEditorContainers[buttonId]) {
	                        this._createSubEditor(buttonId);
						} else {
							dojo.removeClass(this._subEditorContainers[buttonId].container.domNode, "hidden"); //$NON-NLS-1$
							
							this._subEditorContainers[buttonId].container.resize();
							this._subEditorContainers[buttonId].editor.firstColumn.resized();
							this._subEditorContainers[buttonId].editor.secondColumn.resized();
							
							this._subEditorContainers[buttonId].editor.editorShown();
						}
						
						//Assign value to this._currentEditorId after the sub
						//editor has been created, so other code can detect
						//whether an editor has been created
						this._currentEditorId = buttonId;
						
						this._addTimeframeSection(this._currentEditorId);
						
						if (this._heightChangedInvocationNeeded[this._currentEditorId]) {
							setTimeout(dojo.hitch(this, function() {
								this._heightChanged(this._currentEditorId);
								delete this._heightChangedInvocationNeeded[this._currentEditorId];
							}), 0);
						}
						
						//Notify the editor that a different time period was
						//selected when the editor was not being displayed. We
						//don't want sub-editors reacting to time period
						//selection immediately, to avoid unnecessary processing
						if (this._lastTimePeriodSelected !== undefined) {
							this._subEditorContainers[this._currentEditorId].editor.timePeriodSelected(this._lastTimePeriodSelected);
							delete this._lastTimePeriodSelected;
						}
						
						this._subEditorContainers[this._currentEditorId].container.resize();
					}
				} else {
					if (this._subEditorContainers[buttonId]) {
						dojo.addClass(this._subEditorContainers[buttonId].container.domNode, "hidden"); //$NON-NLS-1$
					}
				}
			}
		},
		
		/**
		 * Create the sub-editor for a given editor identifier.
		 * 
		 * We assume that the content widget for each radio choice has fields
		 * 'firstColumn' and 'secondColumn' that are widgets. If a content
		 * widget that doesn't fit this pattern is introduced later, this
		 * function will need to be changed to add that widget's content
		 * differently.
		 * 
		 * The content is a dijit.layout.BorderContainer with sections for
		 * each of the editor's first (left) and second (center) columns. 
		 */
		_createSubEditor: function(buttonId) {
			var widgetConstructor = eval(RADIO_IDS_TO_CONTENT_WIDGETS[buttonId]);
			var editor = new widgetConstructor({
				processAreaEditor: this._processAreaEditor,
				workingCopy: this._workingCopy,
				processAreaStore: this._processAreaStore,
				editorState: this._editorState,
				roleSelectionChangedCallback: dojo.hitch(this, function(selectedRole) {
					var roleId = selectedRole === null ? null : this._processAreaEditor.getRolesStore().getValue(selectedRole, ProcessConstants.ProcessRoleDTO.ID);
					this._selectedRoleId = roleId;
					if (this._scopeSection) {
						this._scopeSection.setSelectedRoleId(this._selectedRoleId);
					}
				})
			});
			
			var contentContainer = document.createElement("div"); //$NON-NLS-1$
			this.editorContainer.appendChild(contentContainer);
			
			// Create a border container to hold the two columns
			var borderContainer = new dijit.layout.BorderContainer({
				design : "sidebar", //$NON-NLS-1$
				liveSplitters:true,
				style: "height: 40em; width: 100%;" //$NON-NLS-1$
			}, contentContainer);
			
			var createSection = dojo.hitch(this, function(region, contentWidget, marginRight) {
				var section = new dijit.layout.ContentPane({
					splitter : true,
					style: "height: 100%; width: " + contentWidget.initialWidth + (!this.isLeftToRight() ? "; margin-left: " : "; margin-right: ") + marginRight, //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
					region : region
				});
				this.connect(section, "resize", function() { //$NON-NLS-1$
					contentWidget.resized();
				});
				section.domNode.appendChild(contentWidget.domNode);
				dojo.addClass(section.domNode, "no-border"); //$NON-NLS-1$
				borderContainer.addChild(section);
				return section;
			});
			
			var leftSection = createSection(!this.isLeftToRight() ? "right" : "left", editor.firstColumn, "0"); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
			
			//Specifying negative margin-right is a kludge, but I don't know how
			//else to have the section occupy enough space to the right to
			//extend as far as the containing editor does
			var centerSection = createSection("center", editor.secondColumn, "-13px"); //$NON-NLS-1$ //$NON-NLS-2$
			dojo.addClass(centerSection.domNode, "scroll-y-hidden"); //$NON-NLS-1$
			//dojo.style(centerSection.domNode, "marginRight", "-18px");
			
			//Will be the parent of a node with instructions when one of the
			//placeholder nodes in the Select a time period section is clicked,
			//(eg. nodes Iteration Types or Timelines)
			var scopePlaceholderInstructionsContainer = dojo.create("div", {'class': "hidden scopePlaceholderInstructions"}, editor.secondColumn.domNode, "after"); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$
			
			borderContainer.startup();
			
			this._subEditorContainers[buttonId] = {
				container: borderContainer,
				leftSection: leftSection,
				centerSection: centerSection,
				editor: editor,
				scopePlaceholderInstructionsContainer: scopePlaceholderInstructionsContainer
			};
			
			this.connect(editor.secondColumn, "heightChanged", dojo.hitch(this, this._heightChanged, buttonId)); //$NON-NLS-1$
			
			if (this._configuredScopesLoaded === true) {
				editor.configuredScopesLoaded();
			}
		},
		
		/**
		 * Adds the section to select a different time period to the given
		 * container, if appropriate. If either the UI to configure time periods
		 * or the ContentPane in the given container to hold it has not been
		 * created, they are created.
		 * 
		 * There is only one panel to select time periods across all editor
		 * modes, (eg. By Role, By Action). This function moves that panel
		 * to the passed-in container.
		 */
		_addTimeframeSection: function(editorId) {
			var contentContainer = this._subEditorContainers[editorId];
			
			if (this._showingTimeframes) {
				//Create section to select time period if necessary
				this._ensureTimeframeSection();
				
				//Create content pane in top-right of contentContainer if necessary
				if (contentContainer.scopeSection === undefined) {
					//Specifying negative margin-right is a kludge, but I don't
					//know how else to have the section occupy enough space to
					//the right to extend as far as the containing editor does
					contentContainer.scopeSection = new dijit.layout.ContentPane({
						style: "width: 100%; height: 10em; margin-" + (!this.isLeftToRight() ? "left" : "right") + ": -13px", //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$
						splitter : true,
						region : "top" //$NON-NLS-1$
					});	
					contentContainer.container.addChild(contentContainer.scopeSection);
					dojo.addClass(contentContainer.scopeSection.domNode, "no-border"); //$NON-NLS-1$
					
					if (contentContainer.editor.containerClassWhenShowingTimeframe !== undefined) {
						dojo.addClass(contentContainer.centerSection.domNode, contentContainer.editor.containerClassWhenShowingTimeframe);
					}
					
					contentContainer.scopeSection._resizing = false;
					
					this.connect(contentContainer.scopeSection, "resize", dojo.hitch(this, function() { //$NON-NLS-1$
						this._scopeSection.resized();
						
						if (!contentContainer.scopeSection._resizing) {
							contentContainer.scopeSection._resizing = true;
							this._heightChanged(editorId);
							contentContainer.scopeSection._resizing = false;
						}
					}));
				}
				
				//Place the time period section in contentContainer, (this way
				//the same time period section is in both modes of the
				//Permissions editor)
				if (contentContainer.scopeSection.domNode.childNodes.length === 0 || this._scopeSection.domNode !== contentContainer.scopeSection.domNode.childNodes[0]) {
					dojo.place(this._scopeSection.domNode, contentContainer.scopeSection.domNode);
				}
			}
			
			dojo.toggleClass(contentContainer.editor.secondColumn.domNode, "hidden", this._scopePlaceholderSelectedInstructions !== undefined); //$NON-NLS-1$
			dojo.toggleClass(contentContainer.scopePlaceholderInstructionsContainer, "hidden", this._scopePlaceholderSelectedInstructions === undefined); //$NON-NLS-1$
			if (this._scopePlaceholderSelectedInstructions !== undefined) {
				dojox.xml.parser.removeChildren(contentContainer.scopePlaceholderInstructionsContainer);
				dojo.place(this._scopePlaceholderSelectedInstructions, contentContainer.scopePlaceholderInstructionsContainer);
			}
			
			this._heightChanged(editorId);
		},
		
		/**
		 * Create the section to select a time period, or return the single
		 * instance if it already exists.
		 */
		_ensureTimeframeSection: function() {
			if (this._scopeSection === undefined) {
				this._currentTimePeriod = this._processAreaEditor._getProcessAreaStore().alwaysRootItem;
				
				this._scopeSection = new com.ibm.team.process.web.ui.internal.admin.projects.PermissionsScopeSection({
					editor: this._processAreaEditor,
					workingCopy: this._workingCopy,
					selectedRoleId: this._selectedRoleId,
					processAreaStore: this._processAreaEditor._getProcessAreaStore(),
					editorState: this._editorState,
					timePeriodSelected: dojo.hitch(this, function(item, placeholderContent) {
						if (placeholderContent === undefined) {
							//"Always" or a real time period was selected
							if (this._scopePlaceholderSelectedInstructions !== undefined) {
								delete this._scopePlaceholderSelectedInstructions;
								//to show the editor contents and hide the instructions:
								if (this._currentEditorId !== undefined) {
								    this._addTimeframeSection(this._currentEditorId);
								}
							}
						} else {
							//A node like "Iteration Types" or "Timelines" was selected
							if (this._scopePlaceholderSelectedInstructions === undefined) {
								this._scopePlaceholderSelectedInstructions = dojo.create("div", {}); //$NON-NLS-1$
							}
							this._scopePlaceholderSelectedInstructions.innerHTML = jazz.util.html.removeScript(placeholderContent);
							//to hide the editor contents and show/update the instructions:
							setTimeout(dojo.hitch(this, this._addTimeframeSection, this._currentEditorId));
							delete this._lastTimePeriodSelected;
							
							item = null;
						}
						if (this._currentTimePeriod !== item) {
							this._currentTimePeriod = item;
							this._lastTimePeriodSelected = item;
							if (this._currentEditorId !== undefined) {
							    this._subEditorContainers[this._currentEditorId].editor.timePeriodSelected(item);
							}
						}
					})
				});
			}
		},
		
		/**
		 * Should be invoked whenever height or allocation of vertical space of
		 * the rightmost column changes, either because:
		 * 
		 * - The time period section was created/removed
		 * - The splitter between the time period section and sub-editor's
		 *   second column is moved
		 * - The sub-editor's second column requires more or less space
		 * 
		 * The height of the sub-editor's container is changed to be the
		 * maximum of its default height and the height that would be required
		 * to display the sub-editor's second column with no inner scrollbar.
		 * (We want to use the browser's scrollbar to traverse content in
		 * the right column).
		 */
		_heightChanged: function(editorId) {
			var editorColumnHeight = this._subEditorContainers[editorId].editor.secondColumn.domNode.clientHeight + this._subEditorContainers[editorId].scopePlaceholderInstructionsContainer.clientHeight
			
			if (0 === editorColumnHeight) {
				//The editor's height changed, but it is not currently being
				//displayed. This can happen when the editor is responding to
				//an event generated when it is hidden; eg. the name of the
				//role that's selected in the Show By Role editor was changed
				//in the Roles tab. Persist the fact that we need to update
				//the editor's container's height the next time it's displayed
				this._heightChangedInvocationNeeded[editorId] = true;
				return;
			}
			
			var centerSectionHeight = this._subEditorContainers[editorId].centerSection.domNode.clientHeight;
			var centerSectionBox = dojo.marginBox(this._subEditorContainers[editorId].centerSection.domNode);
			
			var centerSectionPadding = [
				dojo.style(this._subEditorContainers[editorId].centerSection.domNode, "paddingTop"), //$NON-NLS-1$
				dojo.style(this._subEditorContainers[editorId].centerSection.domNode, "paddingBottom") //$NON-NLS-1$
			];
			
			//Add 16 pixels to space we'd normally need to account for
			//horizontal scrollbar which may be present
			var height = centerSectionBox.t + editorColumnHeight + centerSectionPadding[0] + centerSectionPadding[1] + 16;
			
			if (this._subEditorContainers[editorId].initialHeight === undefined) {
				this._subEditorContainers[editorId].initialHeight = this._subEditorContainers[editorId].container.domNode.clientHeight;
			}
			
			height = Math.max(this._subEditorContainers[editorId].initialHeight, height);
			
			var currentHeight = dojo.style(this._subEditorContainers[editorId].container.domNode, "height"); //$NON-NLS-1$
			if (height !== currentHeight) {
				dojo.style(this._subEditorContainers[editorId].container.domNode, "height", height + "px"); //$NON-NLS-1$ //$NON-NLS-2$
				this._subEditorContainers[editorId].container.resize();
			}
		},
		
		_isChildArea: function() {
			return ProcessConstants.EditorContext.PROJECT === this._processAreaEditor.context && this._processAreaEditor.isProcessConsumer() || ProcessConstants.EditorContext.TEAM === this._processAreaEditor.context;
		},
		
		uninitialize: function(){
			this.clearState();
			
			delete this._processAreaEditor;
			delete this._subEditorContainers;
			delete this._scopeSection;
			delete this._processAreaEditor;
			
			if (this._timeframeLink) {
				this._timeframeLink.destroyRecursive();
				delete this._timeframeLink;
			}
			
			this.inherited(arguments);
		}
	});
})();
