/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2013. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function(){
	dojo.provide("com.ibm.team.process.web.ui.internal.admin.projects.PermissionsForActionTable"); //$NON-NLS-1$
	
	dojo.require("dijit._Widget"); //$NON-NLS-1$
	dojo.require("dojo.i18n"); //$NON-NLS-1$
	dojo.require("com.ibm.team.process.web.ui.internal.admin.projects.PermissionsTable"); //$NON-NLS-1$
	dojo.require("com.ibm.team.process.web.client.internal.ProcessClient"); //$NON-NLS-1$

	dojo.requireLocalization("com.ibm.team.process.web.ui.internal", "Messages"); //$NON-NLS-1$ //$NON-NLS-2$
	var ProcessClient = com.ibm.team.process.web.client.internal.ProcessClient;
	
	/**
	 * Table showing the permissions for all roles for a single action
	 */
	dojo.declare("com.ibm.team.process.web.ui.internal.admin.projects.PermissionsForActionTable", [dijit._Widget, com.ibm.team.process.web.ui.internal.admin.projects.PermissionsTable], { //$NON-NLS-1$
		
		/**
		 * In addition to the arguments accepted by jazz.ui.StyledTable, this
		 * widget accepts:
		 * 
		 * 
		 * workingCopy	   Instance of PermissionsWorkingCopy that can be
		 * 				   consulted to load and update permissions
		 * 
		 * heightChanged	Function that will be called whenever the
		 * 					vertical space that the table will occupy with
		 * 					no scrolling changes, (eg. as rows are expanded or
		 * 					collapsed)
		 * getActionId      Function which returns the id of the currently selected
		 *                  action
		 * getContextItem   Function which returns the currently selected context 
		 * 
		 * processAreaEditor	Instance of AbstractProcessAreaEditor that hosts
		 * 						the Permissions tab
		 * 
		 * processAreaStore		Store of artifacts related to project or team,
		 * 						used for time scopes such as timelines and
		 * 						iteration types
		 * 
		 * permissionChangedInTable    Function that will be called when a permission is 
		 *                             changed in the table. Takes a single parameter
		 *                             which is the new value of the permission                   
		 */
		constructor: function(args) {
			this.messages = dojo.i18n.getLocalization("com.ibm.team.process.web.ui.internal.admin.projects", "Messages"); //$NON-NLS-1$ //$NON-NLS-2$
			
			args.columns[0].sortable = true;
			args.columns.splice(0, 0, {
				label: this.messages.PermissionsForActionTable_1,
				sortable: true
			});
			
			args.showHover = true;					
			this._rowsByRoleId = {};
		},
		
		postCreate: function() {
			this.inherited(arguments);
			dojo.addClass(this.domNode, "com-ibm-team-process-web-ui-internal-PermissionsForActionTable toolbar-table"); //$NON-NLS-1$
			this._createRowsForRoles();
			this.connect(this.workingCopy, "permissionChanged", "_permissionChanged"); //$NON-NLS-1$ //$NON-NLS-2$
		},
		
		_createRowsForRoles: function() {
			var roles = this.processAreaEditor.getRolesStore().getAllRoleItems(); 
			dojo.forEach(roles, dojo.hitch(this, this._createRowForRole));
		},
		
		_createRowForRole: function(role) {
			 var rolesStore = this.processAreaEditor.getRolesStore();
			 var roleName = rolesStore.getRoleLabel(role);
			 var roleId = rolesStore.getValue(role, ProcessConstants.ProcessRoleDTO.ID);
			 var tr = dojo.create("tr", {}, this.tbody); //$NON-NLS-1$
			 var td = dojo.create("td", {}, tr); //$NON-NLS-1$
			 tr._roleId = roleId;
			 td.appendChild(document.createTextNode(roleName)); 
			 this.addOperationDecorations(tr, td);
			 this.addPermissionsColumns(tr);
			 this._rowsByRoleId[roleId] = tr;
		},	
		
		/**
		 * Will be invoked by parent type PermissionsTable when updating
		 * a row with its current permission.
		 */
		getRoleId: function(row) {
			return row._roleId;
		},
		
		_permissionChanged: function(store, item, roleId, contextItem) {
			if (store.getValue(item, ProcessConstants.ExtensionDTO.ID) === this.getActionId() && contextItem === this.getContextItem()) {
				this.updateRowPermissions(this._rowsByRoleId[roleId]);
				this.permissionChangedInTable(store.getPermission(item, contextItem));
			}
		},
		
		/**
		 * Will be invoked by parent type PermissionsTable when permissions
		 * need to be loaded for the current combination of action and time
		 * period.
		 */
		fetchPermissions: function(successCallback, errorCallback) {
			this.workingCopy.loadPermissionsForAllRoles(
				this.getContextItem(),
				successCallback,
				errorCallback
			);
		},
		
		/**
         * Overrides the method in jazz.ui.StyledTable 
		 * On entry this.sortBy is the index of the column being sorted and
		 * this.sortByAscending is a boolean value indicating the sort order
		 */
		handleSort: function() {
			var sortBy = this.sortBy;
			var sortByAscending = this.sortByAscending;
			var rows = new Array();
			for (var r=0; r < this.tbody.children.length; r++) {
				rows[r] = this.tbody.children[r];
			}
			var getRoleName = function(tr) {
				var td = tr.childNodes[0];
				return td.childNodes[0].textContent;
			};
			// Sort permissions according to the values below
			var PERMITTED_VALUE = 1;
			var SOME_PERMITTED_VALUE = 2;
			var NOT_PERMITTED_VALUE = 3;
			var INHERITED_VALUE = 4;
			var getPermissionValue = dojo.hitch(this, function(tr) {
				var td = tr.childNodes[1];
				var title = td.childNodes[0].title;
				if (title === this.messages.PermissionsTable_3) {
					return PERMITTED_VALUE;
				} else if (title === this.messages.PermissionsTable_4) {
					return NOT_PERMITTED_VALUE;
				} else if (title === this.messages.PermissionsTable_5) {
					return SOME_PERMITTED_VALUE;
				} else {
					return INHERITED_VALUE;
				}
			});
			rows.sort(function(left, right) {
				if (sortBy == 0) {
					// Sort by role name
					if ( sortByAscending ) {
						return ProcessClient.stringCompare(getRoleName(left), getRoleName(right));
					} else {
						return ProcessClient.stringCompare(getRoleName(right), getRoleName(left));
					}
				} else if (sortBy == 1) {
					// Sort by permission
					if ( sortByAscending ) {
						return getPermissionValue(left) - getPermissionValue(right);
					} else {
						return getPermissionValue(right) - getPermissionValue(left);
					}
					
				}
			});
			// Remove the rows from the table
			for (r=0; r < rows.length; r++) {
				this.tbody.removeChild(this.tbody.firstChild);
			}
			// Add the rows to the table in sorted order
			for (r=0; r < rows.length; r++) {
				this.tbody.appendChild(rows[r]);
			}
		},
		
		uninitialize: function() {
			this.inherited(arguments);
		}
	});
})();
