/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2009, 2013. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function(){
	dojo.provide("com.ibm.team.process.web.ui.internal.admin.projects.PermissionsScopeStore"); //$NON-NLS-1$
	dojo.require("dojo.i18n"); //$NON-NLS-1$
	dojo.require("dojo.data.ItemFileWriteStore"); //$NON-NLS-1$
	dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler"); //$NON-NLS-1$
	dojo.require("com.ibm.team.process.web.client.internal.ProcessClient"); //$NON-NLS-1$
	dojo.require("com.ibm.team.process.web.common.internal.ProcessConstants"); //$NON-NLS-1$
	dojo.requireLocalization("com.ibm.team.process.web.ui.internal", "Messages"); //$NON-NLS-1$ //$NON-NLS-2$
	var ProcessClient = com.ibm.team.process.web.client.internal.ProcessClient;
	var ProcessConstants = com.ibm.team.process.web.common.internal.ProcessConstants;
	var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;
	var NEW_ITEM_MAX_NUMBER_FOR_ONE_TIME = 50;
	
	dojo.declare("com.ibm.team.process.web.ui.internal.admin.projects.PermissionsScopeStore", dojo.data.ItemFileWriteStore, { //$NON-NLS-1$
		/*
		 * Constructs the PermissionsScopeStore instance
		 * @param {Object} args The args are expected to contain the following:
		 *  {
		 *    editorContext: the editor context tree is being used in... a value in ProcessConstants.EditorContext
		 *    isConsumer: Boolean value which determines if the project is a consumer, default is false
		 *    processAreaItemId: the item id of the process area to load the timeline model for
		 *    errorContext: the object to invoke the error call back on
		 *    errorCallback: the function to call if an error occurs loading the model
		 *  }
		 */
		constructor: function(args){
			// A little dangerous here...setting the private value 
			// that is normally taken from the args in the constructor
			// for ItemFileReadStore.  Don't want to force the client
			// to have to pass that in since it's pre-determined for this
			// model
			this._jsonData = {
				identifier: 'id', //$NON-NLS-1$
				label: 'name', //$NON-NLS-1$
				items: []
			};
			
			this.comparatorMap = {
				name : ProcessClient.stringCompare, 
				contextId : ProcessClient.stringCompare
			};
			
			this._loadedDevelopmentLines = {};
			
			this._isConsumer = args.isConsumer === true;
			this._editorContext = args.editorContext;
			this._processAreaItemId = args.processAreaItemId;
			this._currentDevelopmentLineItemId = args.currentDevelopmentLineItemId;
			this._errorContext = args.errorContext;
			this._errorCallback = args.errorCallback;
			this.messages = dojo.i18n.getLocalization("com.ibm.team.process.web.ui.internal.admin.projects", "Messages"); //$NON-NLS-1$ //$NON-NLS-2$
			this._initialize();
		},
		
		_initialize: function(){
			this._addAlwaysRootItem();
			
			this._addIterationTypesRootItem();
			this._addTimelinesRootItem();
			this.save();
			this.operationsLeft = 2;
			
			var srh = new ServiceResponseHandler(this, "_populateIterationTypesData", "_handleIterationTypesFetchError"); //$NON-NLS-1$ //$NON-NLS-2$
			ProcessClient.getIterationTypes({
				processAreaItemId: this._processAreaItemId,
				processAreaContext: this._editorContext,
				includeArchived: true
			}, srh);
			
			// Always load all the timelines and iterations regardless of the editor context is PROJECT or TEAM
			// So that the team area can then just easily switch selected timeline between them.  
			// We'd then mark the one that was appropriate for the team area and the tree in permissions,
			// etc only shows the iterations for that one. 
			srh = new ServiceResponseHandler(this, "_populateDevelopmentLinesData", "_handleDevelopmentLinesFetchError"); //$NON-NLS-1$ //$NON-NLS-2$
			ProcessClient.getProjectDevelopmentLines({
				uuid: this._processAreaItemId,
				includeArchived: true,
				processAreaContext: this._editorContext
			}, srh);
		},
		
		_populateDevelopmentLinesData: function(data){
			var self = this;
			this.fetchItemByIdentity({
				identity: ProcessConstants.PermissionsScopeContext.DEV_LINES,
				onItem: function(item, request){
					dojo.forEach(data, function(developmentLine){
						self.newItem({
							name: developmentLine.label,
							id: developmentLine.itemId,
							stateId: developmentLine.stateId,
							parent: self.getIdentity(item),
							context: ProcessConstants.ExtensionContext.TEAM,
							contextId: developmentLine.id,
							enabledContext: ProcessConstants.PermissionContext.DEV_LINE,
							projectTimeline: developmentLine.projectTimeline,
							startDate: dojo.date.stamp.fromISOString(developmentLine.startDate),
							endDate: dojo.date.stamp.fromISOString(developmentLine.endDate),
							currentIterationItemId: developmentLine.currentIterationItemId ? developmentLine.currentIterationItemId : "", //$NON-NLS-1$
							archived: developmentLine.archived,
							currentSelectedTimeline: developmentLine.currentSelectedTimeline
						}, {
							parent: item,
							attribute: "items" //$NON-NLS-1$
						});
						
					});
				}
			});
			
			dojo.forEach(data, function(developmentLine){
				this.operationsLeft++;
				var srh = new ServiceResponseHandler(self, "_populateIterationData", "_handleIterationsFetchError"); //$NON-NLS-1$ //$NON-NLS-2$
				ProcessClient.getIterations({
					uuid: developmentLine.itemId,
					includeArchived: true
				}, srh);
			}, this);
			this.save();
			this.operationsLeft--;
		},
		
		_populateIterationData: function(iterations){
			if (iterations && iterations.length > 0) {
				// Convert iteration items from tree structure to list structure,
				// so that we can simply use setTimeout to avoid "Stop running this script"
				// message popup while loading iteration items into the store.
				var allIterationsArray = this._convertIterationsTreeToArray(iterations);
				this.operationsLeft++;
				this._doPopulateIterationData(allIterationsArray);
			}
			this.operationsLeft--;
		},
		
		_convertIterationsTreeToArray: function(iterations) {
			var iterationsArray = [];
			if (iterations && iterations.length > 0) {
				for (var i = 0; i < iterations.length; i++) {
					var iteration = iterations[i];
					iterationsArray.push(iteration);
					if (iteration.iterations && iteration.iterations.length > 0) {
						iterationsArray = iterationsArray.concat(this._convertIterationsTreeToArray(iteration.iterations));
					}
				}
			}
			return iterationsArray;
		},

		_doPopulateIterationData: function(iterations){
			if (iterations && iterations.length > 0) {
				var newItemsNumber = NEW_ITEM_MAX_NUMBER_FOR_ONE_TIME;
				if(iterations.length < NEW_ITEM_MAX_NUMBER_FOR_ONE_TIME) {
					newItemsNumber = iterations.length;
				}
				var newItems = iterations.splice(0, newItemsNumber);
				var self = this;
				for (var i = 0; i < newItemsNumber; i++) {
					var iteration = newItems[i];
					this.fetchItemByIdentity({
						identity: iteration.parentItemId,
						onItem: function(item, request){
							self.newItem({
								name: iteration.label,
								id: iteration.itemId,
								stateId: iteration.stateId,
								parent: iteration.parentItemId,
								context: ProcessConstants.ExtensionContext.TEAM,
								contextId: self.getValue(item, "contextId") + ProcessConstants.PERMISSION_PATH_SEPARATOR + iteration.id, //$NON-NLS-1$
								originalContextId: self.getValue(item, "contextId") + ProcessConstants.PERMISSION_PATH_SEPARATOR + iteration.id, //$NON-NLS-1$
								enabledContext: ProcessConstants.PermissionContext.ITERATION,
								current: iteration.current,
								completed: iteration.completed,
								leaf: iteration.leaf,
								startDate: dojo.date.stamp.fromISOString(iteration.startDate),
								endDate: dojo.date.stamp.fromISOString(iteration.endDate),
								iterationTypeItemId: iteration.iterationTypeItemId ? iteration.iterationTypeItemId : ProcessConstants.ITERATION_TYPE_NONE,
								hasDeliverable: iteration.hasDeliverable,
								archived: iteration.archived
							}, {
								parent: item,
								attribute: "items" //$NON-NLS-1$
							});
						}
					});
				}
				this.save();
				if(iterations.length > 0) {
					this.operationsLeft++;
					setTimeout(dojo.hitch(this, function() {
						this._doPopulateIterationData(iterations);
					}));
				}
			}
			this.operationsLeft--;
		},
		
		_populateIterationTypesData: function(data){
			var self = this;
			this.fetchItemByIdentity({
				identity: ProcessConstants.PermissionsScopeContext.ITERATION_TYPES,
				onItem: function(item, request){
					var items = {};
					var i = 0;
					dojo.forEach(data, function(iterationType){
						var newChildItem = self.newItem({
							name: iterationType[ProcessConstants.IterationTypeDTO.LABEL],
							id: iterationType[ProcessConstants.IterationTypeDTO.ITEM_ID],
							stateId: iterationType[ProcessConstants.IterationTypeDTO.STATE_ID],
							parent: self.getValue(item, ProcessConstants.IterationTypeDTO.ID),
							context: ProcessConstants.ExtensionContext.TEAM,
							contextId: iterationType.id,
							savedContextId: iterationType.id,
							enabledContext: ProcessConstants.PermissionContext.ITERATION_TYPE,
							archived: iterationType.archived,
							summary: iterationType.summary,
							defined: iterationType.defined
						}, {
							parent: item,
							attribute: "items" //$NON-NLS-1$
						});
						items[i] = newChildItem;
						i++;
					});
				}
			});
			this.save();
			this.operationsLeft--;
		},
		
		_handleDevelopmentLinesFetchError: function(error){
			this._errorCallback.call(this._errorContext, error, ProcessConstants.PermissionsScopeContext.DEV_LINES);
		},
		
		_handleIterationTypesFetchError: function(error){
			this._errorCallback.call(this._errorContext, error, ProcessConstants.PermissionsScopeContext.ITERATION_TYPES);
		},
		
		_handleIterationsFetchError: function(error){
			this._errorCallback.call(this._errorContext, error, ProcessConstants.PermissionsScopeContext.ITERATIONS);
		},
		
		_addAlwaysRootItem: function() {
			this.alwaysRootItem = this.newItem({
				id: "", //$NON-NLS-1$
				name: this.messages.PermissionsScopeStore_AlwaysConfigurationNodeName,
				context: null,
				contextId: "", //$NON-NLS-1$
				archived: false
			});
		},
		
		_addIterationTypesRootItem: function(){
			this.newItem({
				id: ProcessConstants.PermissionsScopeContext.ITERATION_TYPES,
				name: this.messages.PermissionsScopeStore_IterationTypesNodeName,
				archived: false
			});
		},
		
		_addTimelinesRootItem: function(){
			var itemName;
			if (this._editorContext === ProcessConstants.EditorContext.PROJECT) {
				itemName = this.messages.PermissionsScopeStore_TimelinesNodeName;
		    } else {
				itemName = this.messages.PermissionsScopeStore_IterationsNodeName;
			}
			this.newItem({
				id: ProcessConstants.PermissionsScopeContext.DEV_LINES,
				name: itemName,
				archived: false
			});
		},
		
		isIterationType: function(item){
			var context = this.getValue(item, "enabledContext"); //$NON-NLS-1$
			if (context === ProcessConstants.PermissionContext.ITERATION_TYPE) {
				return true;
			}
			return false;
		},
		
		isTimeline: function(item){
			var context = this.getValue(item, "enabledContext"); //$NON-NLS-1$
			if (context === ProcessConstants.PermissionContext.DEV_LINE) {
				return true;
			}
			return false;
		},
		
		isProjectTimeline: function(item){
			return this.getValue(item, "projectTimeline"); //$NON-NLS-1$
		},
		
		isIteration: function(item){
			if (this.isItem(item)) {
				var context = this.getValue(item, "enabledContext"); //$NON-NLS-1$
				if (context === ProcessConstants.PermissionContext.ITERATION) {
					return true;
				}
			}
			return false;
		},
		
		isCurrentIteration: function(item){
			return this.getValue(item, "current") === true && this.getValue(item, "leaf") === true; //$NON-NLS-1$ //$NON-NLS-2$
		},
		
		getTimelineForIteration: function(iteration){
			var self = this;
			var timeline;
			this.fetchItemByIdentity({
				identity: iteration.parent,
				onItem: function(item, request){
					if (self.isTimeline(item)) {
						timeline = item;
					} else {
						timeline = self.getTimelineForIteration(item);
					}
				}
			});
			return timeline;
			
		},
		
		getParent: function(item){
			var parent;
			this.fetchItemByIdentity({
				identity: item.parent,
				onItem: function(item, request){
					parent = item;
				}
			});
			return parent;
		},
		
		deleteItem: function(item){
			/*
			 * TODO: temporary, see workitem 235332
			 * 
			 * Send out a new event, "aboutToDelete", that implementors can
			 * connect to to find out when deleteItem is called on an item,
			 * *before* that item is removed from the store.
			 */
			this.aboutToDelete(item);
			
			var enabledContext = this.getValue(item, "enabledContext"); //$NON-NLS-1$
			if (enabledContext === ProcessConstants.PermissionContext.ITERATION_TYPE) {
				// Eliminate any references from the iterations to the iteration type
				// being deleted
				var iterationTypeItemId = this.getIdentity(item);
				this.fetch({
					query: {
						enabledContext: ProcessConstants.PermissionContext.ITERATION,
						iterationTypeItemId: iterationTypeItemId
					},
					queryOptions: {
						deep: true
					},
					onComplete: dojo.hitch(this, function(items){
						for (var i = 0; i < items.length; i++) {
							var item = items[i];
							if (this.isDirty(item)) {
								this.setValue(item, "iterationTypeItemId", ProcessConstants.ITERATION_TYPE_NONE); //$NON-NLS-1$
							}
						}
					})
				});
			}
			
			// Get the children of the item
			var children = this.getValues(item, "items"); //$NON-NLS-1$
			// Now destroy the children recursively
			if (children) {
				dojo.forEach(children, function(child){
					this.deleteItem(child);
				}, this);
			}
			// Now get rid of the parent
			this.inherited(arguments);
		},
		
		aboutToDelete: function(item) {
			// implementors can connect to this method to find out about a
			// delete event while the item is in the store (so it can be read)
		},
		
		archiveItem: function(item) {
			this.setValue(item, "archived", true); //$NON-NLS-1$	
			// Archive the children of the item
			var children = this.getValues(item, "items"); //$NON-NLS-1$			
			if (children) {
				dojo.forEach(children, function(child){
					if (this.isNewItem(child)) {
						this.deleteItem(child);
					} else {
						this.archiveItem(child);
					}
				}, this);
			}
		},
		
		unarchiveItem: function(item) {
			this.setValue(item, "archived", false); //$NON-NLS-1$	
			// Unarchive the children of the item
			var children = this.getValues(item, "items"); //$NON-NLS-1$			
			if (children) {
				dojo.forEach(children, function(child){
					this.unarchiveItem(child);
				}, this);
			}
		},
		
		isNewItem: function(item){
			var id = this.getIdentity(item);
			if (id.indexOf(ProcessConstants.EditorMode.NEW) === 0) {
				return true;
			}
			return false;
		},
		
		isArchivedItem: function(item){
			return this.hasAttribute(item, "archived") && this.getValue(item, "archived"); //$NON-NLS-1$	//$NON-NLS-2$	
		},
		
		/**
		 * Set the current development line in this store.
		 * 
		 * @param {Object} developmentLineItemId
		 */
		setCurrentDevelopmentLineItemId: function(developmentLineItemId) {
			if (this.isInitialized()) {
				// If this store has been initialized, then we can set current development 
				// timeline in the store immediately.
				this._doSetCurrentDevelopmentLineItemId(developmentLineItemId);
			} else {
				// If this store has NOT been initialized, then we need to cache the pass-in value of 
				// current development timeline and then use it to set the current development
				// timeline in the store after the initialization is completed.
				this._currentDevelopmentLineItemId = developmentLineItemId;
				this.waitForInitializationComplete(dojo.hitch(this, function(store) {
					this._doSetCurrentDevelopmentLineItemId(this._currentDevelopmentLineItemId);
				}));
			}
		},
		
		_doSetCurrentDevelopmentLineItemId: function(developmentLineItemId) {
			var currentDevLine = this.getCurrentDevelopmentLineItem();
			if (currentDevLine && developmentLineItemId === this.getIdentity(currentDevLine)) {
				return;
			}
			this.fetchItemByIdentity({
				identity: developmentLineItemId,
				onItem: dojo.hitch(this, function(item, request) {
					if (currentDevLine) {
						this.setValue(currentDevLine, "currentSelectedTimeline", false); //$NON-NLS-1$
					}
					this.setValue(item, "currentSelectedTimeline", true); //$NON-NLS-1$
				})
			});
		},
		
		/**
		 * Get current development line item in the store.
		 * 
		 * @return current development line
		 */
		getCurrentDevelopmentLineItem: function() {
			var currentSelectedTimeline;
			this.fetch({
				query: {
					currentSelectedTimeline: true
				},
				queryOptions: {
					deep: true
				},
				onItem: function(item) {
					currentSelectedTimeline = item;
				}
			});
			
			return currentSelectedTimeline;
		},

		/**
		 * Get current development line itemId.
		 * 
		 * @return current development line
		 */
		getCurrentDevelopmentLineItemId: function() {
			var currentSelectedTimeline = this.getCurrentDevelopmentLineItem();
			if (currentSelectedTimeline) {
				return this.getValue(currentSelectedTimeline, "id"); //$NON-NLS-1$
			}
		},
		
		/**
		 * Invoke the given callback method after the initialization is completed.
		 * 
		 * @param {Object} callback
		 */
		waitForInitializationComplete: function(callback) {
			if (!callback || !dojo.isFunction(callback)) {
				return;
			}
			
			var initializationInterval = setInterval(dojo.hitch(this, function() {
				if (this.isInitialized()) {
					clearInterval(initializationInterval);
					callback();
				}
			}), 100);
		},
		
		/**
		 * Returns the editor context, either ProcessConstants.EditorContext.PROJECT or
		 * ProcessConstants.EditorContext.TEAM.
		 * 
		 * @return the editor context
		 */
		getEditorContext: function() {
			return this._editorContext;
		},
		
		
		isInitialized: function() {
			return this.operationsLeft === 0;
		},
		
		onSet: function(item, attribute, oldValue, newValue) {
			// cache of locally defined iteration types by id may be invalid
			delete this._definedIterationTypes;
		},
		
		onNew: function(newItem, parentInfo) {
			// cache of locally defined iteration types by id may be invalid
			delete this._definedIterationTypes;
		},
		
		onDelete: function(deletedItem) {
			// cache of locally defined iteration types by id may be invalid
			delete this._definedIterationTypes;
		},
		
		/**
		 * Returns true for iteration types defined locally and any inherited
		 * ones which are not overridden locally by a type with the same id.
		 */
		isAvailableIterationType: function(iterationTypeItem) {
			if (this._definedIterationTypes === undefined) {
				// create cached map of locally defined iteration types
				this._definedIterationTypes = {};
				this.fetch({
					query: {
						enabledContext: ProcessConstants.PermissionContext.ITERATION_TYPE,
						defined: true
					},
					queryOptions: {
						deep: true
					},
					onItem: dojo.hitch(this, function(item) {
						this._definedIterationTypes[this.getValue(item, "contextId")] = true; //$NON-NLS-1$
					})
				});
			}
			return this.getValue(iterationTypeItem, "defined") === true || !this._definedIterationTypes[this.getValue(iterationTypeItem, "contextId")]; //$NON-NLS-1$ //$NON-NLS-2$
		},
		
		/**
		 * Can be used to compare two iteration types when sorting. Does an
		 * alpha sort by iteration type name (or id if no name), and always puts
		 * locally defined iteration types before inherited ones.
		 */
		compareIterationTypes: function(iterationTypeItem1, iterationTypeItem2) {
			var defined1 = this.getValue(iterationTypeItem1, "defined"); //$NON-NLS-1$
			var defined2 = this.getValue(iterationTypeItem2, "defined"); //$NON-NLS-1$
			if (defined1 !== defined2) {
				return defined1 ? -1 : 1;
			}
			var name1 = this.getIterationTypeLabel(iterationTypeItem1);
			var name2 = this.getIterationTypeLabel(iterationTypeItem2);
			return name1.localeCompare(name2);
		},
		
		/**
		 * Returns the iteration type's name, or id if it has no name.
		 */
		getIterationTypeLabel: function(iterationTypeItem) {
			var label = this.getValue(iterationTypeItem, "name"); //$NON-NLS-1$
			if (label === undefined || label === null || dojo.string.trim(label) === 0) {
				label = this.getValue(iterationTypeItem, "contextId"); //$NON-NLS-1$
			}
			return label;
		},
		
		/**
		 * Utility function that returns the contextId of the item as stored
		 * on the server, (aka 'originalContextId' for non-new iterations),
		 * or the items's contextId if the item is new.
		 */
		getSavedContextId: function(contextItem) {
			var contextId = this.getValue(contextItem, "contextId"); //$NON-NLS-1$
			if (this.getValue(contextItem, "enabledContext") === ProcessConstants.PermissionContext.ITERATION) { //$NON-NLS-1$
				// Use the original context id because if the user moved the iteration
				// it's context id will now be different
				if (this.hasAttribute(contextItem, "originalContextId")) { //$NON-NLS-1$
					// New iterations don't have an original context id
					contextId = this.getValue(contextItem, "originalContextId"); //$NON-NLS-1$
				}
			}
			return contextId;
		}
	});
})();
