/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2013. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function(){
	dojo.provide("com.ibm.team.process.web.ui.internal.admin.projects.PermissionsTable"); //$NON-NLS-1$
	
	dojo.require("com.ibm.team.process.web.ui.internal.admin.projects.PromptDialog"); //$NON-NLS-1$
	dojo.require("com.ibm.team.process.web.ui.internal.admin.projects.UIHelper"); //$NON-NLS-1$
	dojo.require("com.ibm.team.process.web.ui.internal.admin.projects.PermissionsWorkingCopy"); //$NON-NLS-1$
	dojo.require("com.ibm.team.process.web.ui.internal.admin.projects.PermissionsStore"); //$NON-NLS-1$)
	dojo.require("com.ibm.team.process.web.ui.internal.admin.projects.PermissionsCustomizeDialog"); //$NON-NLS-1$
	dojo.require("com.ibm.team.process.web.ui.internal.TRToolbar"); //$NON-NLS-1$
	
	dojo.require("dijit._Widget"); //$NON-NLS-1$
	
	dojo.require("dojo.i18n"); //$NON-NLS-1$
	
	dojo.require("jazz.ui.StyledBox"); //$NON-NLS-1$
	dojo.require("jazz.ui.StyledTable"); //$NON-NLS-1$
	dojo.require("jazz.ui.toolbar.Button"); //$NON-NLS-1$
	dojo.require("jazz.ui.toolbar.TwoStateButton"); //$NON-NLS-1$
	dojo.require("jazz.util.html"); //$NON-NLS-1$

	dojo.requireLocalization("com.ibm.team.process.web.ui.internal", "Messages"); //$NON-NLS-1$ //$NON-NLS-2$
	
	var UIHelper = com.ibm.team.process.web.ui.internal.admin.projects.UIHelper;
	var PermittedType = com.ibm.team.process.web.ui.internal.admin.projects.PermissionsStore.PermittedType;
	
	var MAX_ACTIONS = 5;
	
	var PERMITTED_TYPES_INFO = {
	};
	PERMITTED_TYPES_INFO[PermittedType.PERMITTED] = {nlsKey: "PermissionsTable_3", imageUrl: dojo.moduleUrl("com.ibm.team.process.web", "ui/internal/admin/projects/images/permitted_obj.gif")}; //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
	PERMITTED_TYPES_INFO[PermittedType.NOT_PERMITTED] = {nlsKey: "PermissionsTable_4", imageUrl: dojo.moduleUrl("com.ibm.team.process.web", "ui/internal/admin/projects/images/not-permitted_obj.gif")}; //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
	PERMITTED_TYPES_INFO[PermittedType.SOME_PERMITTED] = {nlsKey: "PermissionsTable_5", imageUrl: dojo.moduleUrl("com.ibm.team.process.web", "ui/internal/admin/projects/images/some-permitted_obj.gif")}; //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
	PERMITTED_TYPES_INFO[PermittedType.INHERITED] = {nlsKey: "PermissionsTable_6", imageUrl: dojo.moduleUrl("com.ibm.team.process.web", "ui/internal/admin/projects/images/inherited_obj.gif")}; //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$

	/**
	 * Table widget that adds two columns, Permissions and Actions, to the
	 * columns that are supplied to the constructor. Helper functions are
	 * available to do things that are common to permissions tables, including:
	 * 
	 * - Populate the Permissions column for a row with the appropriate icon
	 *   to reflect the permission for a given operation/action for a
	 *   combination of role and time period
	 * 
	 * - Populate the Actions column with actions appropriate for the editor
	 *   and above permission value, including actions to grant/revoke
	 *   permission, etc... and to edit properties of operations
	 */
	dojo.declare("com.ibm.team.process.web.ui.internal.admin.projects.PermissionsTable", [dijit._Widget, jazz.ui.StyledTable], { //$NON-NLS-1$
		
		/**
		 * In addition to same parameters as jazz.ui.StyledTable, arguments are
		 * as follows:
		 * 
		 * workingCopy			Instance of PermissionsWorkingCopy that can be
		 * 						consulted to load and update permissions
		 * 
		 * fetchPermissions		A function that does whatever is neccessary to
		 * 						ensure that get() functions in the working copy
		 * 						return valid permissions information for every
		 * 						row in the table. This function must accept
		 * 						two arguments: callback functions that are
		 * 						to be invoked when the working copy get()
		 * 						functions can be invoked or when there was an
		 * 						error fetching the permissions respectively.
		 * 
		 * 						If this function is not supplied as an argument,
		 * 						it must be declared in a subclass and
		 * 						accessible via this.fetchPermissions(...).
		 * 
		 * getActionId			Function that returns an operation or action id
		 * 						for a given row that was passed to function
		 * 						addPermissionsColumn(), where the row node is
		 * 						the first argument to the function.
		 * 
		 * 						If this function is not supplied as an argument,
		 * 						it must be declared in a subclass and
		 * 						accessible via this.getActionId(...)
		 * 
		 * getRoleId			Same as getActionId, but the function returns
		 * 						the identifier of a role.
		 * 
		 * getContextItem		Same as getActionId, but the function returns
		 * 						the context item, (item in processAreaStore)
		 * 
		 * processAreaEditor	Instance of AbstractProcessAreaEditor that hosts
		 * 						the Permissions tab
		 * 
		 * processAreaStore		Store of artifacts related to project or team,
		 * 						used for time scopes such as timelines and
		 * 						iteration types
		 * 
		 * heightChanged		Optional callback function that is invoked
		 * 						whenever an action which affects the height of
		 * 						the tree is taken.
		 * 
		 * canInherit			Optional boolean. If false, the action to
		 * 						inherit permissions, (clear override) will not
		 * 						be displayed.
		 */
		constructor: function(args) {
			this.messages = dojo.i18n.getLocalization("com.ibm.team.process.web.ui.internal.admin.projects", "Messages"); //$NON-NLS-1$ //$NON-NLS-2$
			
			if (!args.columns) {
				args.columns = [];
			}
			
			args.columns.push({
				label: this.messages.PermissionsTable_1,
				className: "permissionsColumnHeading" //$NON-NLS-1$
			}, {
				label: this.messages.PermissionsTable_2,
				className: "actionsColumnHeading", //$NON-NLS-1$
				width: (MAX_ACTIONS * 22) + "px" //$NON-NLS-1$
			});
			
			this._permissionsRows = [];
			
			this._inlineEditorUUID = 0;
		},
		
		postCreate: function() {
			this.inherited(arguments);
			dojo.addClass(this.domNode, "com-ibm-team-process-web-ui-internal-PermissionsTable"); //$NON-NLS-1$
			this.resizeHeader(false);
		},
		
		/**
		 * Set the size of the header columns for permission and actions
		 * Needs to be called at least once when the table is visible to set
		 * the width correctly
		 * If sortable is true space will be allocated for the sort direction indicator
		 */
		resizeHeader: function(sortable) {
			setTimeout(dojo.hitch(this, function() {
				//We want to specify a width for both of the columns we added,
				//so that the other columns have as much space as possible,
				//and the table columns can be made to not resize based
				//on the content of the rows (eg. when sections in the By Role
				//table are collapsed)
				dojo.query("th.permissionsColumnHeading div", this.domNode).forEach(dojo.hitch(this, function(node) { //$NON-NLS-1$
					//The Permissions column shouldn't need to be any larger
					//than the space required to print its header label 
					// and if sortable the sort direction indicator
					var maxStringDisplayWidthInEms=UIHelper.getMaxStringDisplayWidthInEms([this.messages.PermissionsTable_1], node);
					var additionalSpace = sortable ? 1.5 : .5;
					// If the table is not visible the value will be NaN
					if (!isNaN(maxStringDisplayWidthInEms)) {
						dojo.style(node.parentNode, "width", (additionalSpace + maxStringDisplayWidthInEms) + "em"); //$NON-NLS-1$ //$NON-NLS-2$
				    }
					//... but just in case:
					dojo.style(node.parentNode, "minWidth", "16px"); //$NON-NLS-1$ //$NON-NLS-2$
				}));
			}), 0);
		},
		
		/**
		 * Attach images that indicate whether the 'final' or 'denial text'
		 * properties are specified for the combination of action, role, and
		 * time period that the row represents.
		 * 
		 * These icons are shown or hidden when the permission for the row
		 * is updated.
		 * 
		 * This should be called for all rows which either represent an
		 * operation or could represent an operation at a future point in
		 * time.
		 * 
		 * @param	row							DOM node for the table row
		 * 										that's being build
		 * @param	decorationContainerNode		DOM node that the images will
		 * 										be added to 
		 */
		addOperationDecorations: function(row, decorationContainerNode) {
			var createDecoration = dojo.hitch(this, function(titleText, className, imageUrl) {
				var decorationIcon = dojo.create("a", {href: "#"}, decorationContainerNode); //$NON-NLS-1$ //$NON-NLS-2$
				if (titleText !== null) {
					decorationIcon.title = titleText;
				}
				dojo.addClass(decorationIcon, "decorationIcon rowIcon " + className + " hidden"); //$NON-NLS-1$ //$NON-NLS-2$
				dojo.create("img", {src: imageUrl}, decorationIcon); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
				
				this.connect(decorationIcon, "onclick", function(event) { //$NON-NLS-1$
	    			dojo.stopEvent(event);
	    		});
	    		
	    		this.selectRowOnFocus(row, decorationIcon);
	    		
	    		return decorationIcon;
			});
			
			row._isFinalIcon = createDecoration(
				this.messages.PermissionsTable_8,
				"isFinalIcon", //$NON-NLS-1$
				dojo.moduleUrl("com.ibm.team.process.web", "ui/internal/admin/projects/images/locked_ovr.gif") //$NON-NLS-1$ //$NON-NLS-2$
			);
			
			row._denialExplanationIcon = createDecoration(
				null,
				"denialExplanationIcon", //$NON-NLS-1$
				dojo.moduleUrl("com.ibm.team.process.web", "ui/internal/admin/projects/images/denial-explanation_ovr.gif") //$NON-NLS-1$ //$NON-NLS-2$
			);
		},
		
		/**
		 * Should be invoked when building a row, after the td's for clients'
		 * columns, (argument "columns" that was passed to the constructor),
		 * have been created.
		 * 
		 * @param	row	DOM Node for the table row that's being built
		 */
		addPermissionsColumns: function(row) {
			this._permissionsRows.push(row);
			
			row._permissionTd = dojo.create("td", {}, row); //$NON-NLS-1$
			dojo.addClass(row._permissionTd, "permissionColumn"); //$NON-NLS-1$
			
			row._actionsTd = dojo.create("td", {}, row); //$NON-NLS-1$
			dojo.addClass(row._actionsTd, "actionsColumn"); //$NON-NLS-1$
			this._createActions(row._actionsTd);
		},
		
		/**
		 * Helper function that attaches listeners to a node's onfocus and
		 * onblur events that select the row when the node is focused.
		 * 
		 * This is useful to support keyboard navigation, where we want the
		 * row to be selected, which makes the controls in the Actions
		 * column visible, as soon as the user tabs to a node in the row.
		 * 
		 * In general, this function should be called for any DOM node in the
		 * row that accepts focus, eg. links and icons.
		 * 
		 * @param	row		The row that is to be selected when the node is
		 * 					focused
		 * @param	node	A DOM node that would typically be a child of the
		 * 					row 
		 */
		selectRowOnFocus: function(row, node) {
			return [
				this.connect(node, "onfocus", function(e) { //$NON-NLS-1$
					clearTimeout(row._blurTimeoutVariable);
					dojo.toggleClass(row, "selected", true); //$NON-NLS-1$
				}),
				
				this.connect(node, "onblur", function(e) { //$NON-NLS-1$
					row._blurTimeoutVariable = setTimeout(dojo.hitch(this, function(){
						dojo.toggleClass(row, "selected", false); //$NON-NLS-1$
					}), 0);
				})
			];
		},
		
		/**
		 * Must be invoked by subclass when a row is to be hidden from view.
		 * Performs neccessary cleanup, such as closing the inline editor
		 * to modify operation properties.
		 * 
		 * @param	row		The row that is being hidden from view.
		 */
		rowHidden: function(row) {
			if (row._editorRow) {
				this.toggleEditOperationRow(row, false);
			}
		},
		
		_createActions: function(actionsTd) {
			actionsTd._toolbar = new com.ibm.team.process.web.ui.internal.TRToolbar();
    		
    		var getParentRow = function(element) {
				var row = element;
				if (!row.tagName) {
					row = row.domNode;
				}
				
				while (row && row.tagName.toLowerCase() !== "tr") { //$NON-NLS-1$
					console.debug(row);
					row = row.parentNode;
				}
				return row;
			};
			
			var setPermission = dojo.hitch(this, function(row, permission) {
				var contextItem = this.getContextItem(row);
				var permissionsStore = this.getPermissionsStore(
					this.getRoleId(row),
					contextItem
				);
				var permissionItem = permissionsStore.getPermissionItem(this.getActionId(row));
				
				permissionsStore.setPermission(
					permissionItem,
					this.getActionId(row),
					contextItem,
					permission
				);
			});
			
			var self = this;
			actionsTd._inheritPermissionButton = new jazz.ui.toolbar.Button({
				title: this.messages.PermissionsTable_9,
				imgSrc: dojo.moduleUrl("com.ibm.team.process.web", "ui/internal/admin/projects/images/inherited_obj.gif"), //$NON-NLS-1$ //$NON-NLS-2$
				onClick: function(event) {
					dojo.stopEvent(event);
					setPermission(getParentRow(this), PermittedType.INHERITED);
				}
			});
			actionsTd._toolbar.addChild(actionsTd._inheritPermissionButton);
			
    		actionsTd._editPropertiesButton = new jazz.ui.toolbar.TwoStateButton({
				imgSrc: dojo.moduleUrl("com.ibm.team.process.web", "ui/internal/images/edit.gif"), //$NON-NLS-1$ //$NON-NLS-2$
				title: this.messages.PermissionsTable_10,
				onClick: function() {
					var row = getParentRow(this.domNode);
					self.toggleEditOperationRow(row, this.pressed);
					if (!this.pressed) {
						actionsTd._editPropertiesButton.focusNode.blur();
					}
				}
			});
			actionsTd._toolbar.addChild(actionsTd._editPropertiesButton);
			
			actionsTd._customizePermissionButton = new jazz.ui.toolbar.Button({
				title: this.messages.PermissionsTable_9,
				imgSrc: dojo.moduleUrl("com.ibm.team.process.web", "ui/internal/admin/projects/images/customize_co.gif"), //$NON-NLS-1$ //$NON-NLS-2$
				onClick: function(event) {
					dojo.stopEvent(event);
					self._customizePermission(getParentRow(this));
				}
			});
			actionsTd._toolbar.addChild(actionsTd._customizePermissionButton);
			
			actionsTd._grantPermissionButton = new jazz.ui.toolbar.Button({
				title: this.messages.PermissionsTable_11,
				imgSrc: dojo.moduleUrl("com.ibm.team.process.web", "ui/internal/admin/projects/images/grant_co.gif"), //$NON-NLS-1$ //$NON-NLS-2$
				onClick: function(event) {
					dojo.stopEvent(event);
					setPermission(getParentRow(this), PermittedType.PERMITTED);
				}
			});
			actionsTd._toolbar.addChild(actionsTd._grantPermissionButton);
			
			actionsTd._revokePermissionButton = new jazz.ui.toolbar.Button({
				title: this.messages.PermissionsTable_12,
				imgSrc: dojo.moduleUrl("com.ibm.team.process.web", "ui/internal/admin/projects/images/revoke_co.gif"), //$NON-NLS-1$ //$NON-NLS-2$
				onClick: function(event) {
					dojo.stopEvent(event);
					setPermission(getParentRow(this), PermittedType.NOT_PERMITTED);
				}
			});
			actionsTd._toolbar.addChild(actionsTd._revokePermissionButton);
			
			actionsTd.appendChild(actionsTd._toolbar.domNode);
		},
		
		_customizePermission: function(row) {
			//For now, set the permission of the parent operation, (and child
			//actions implicitly) to 'not permitted'. Item 252639 will replace
			//this with a dialog that allows the permissions to be customized
			//beginning with the closest parent permissions.
			var role = this.getRoleId(row);
			var contextItem = this.getContextItem(row);
			var actionId = this.getActionId(row);
			var parentOperationId = this.workingCopy.getParentOperationId(actionId);
			var isOperation = this.workingCopy.isOperation(actionId);
			
			this.workingCopy.getInheritedPermissions(
				role,
				contextItem,
				parentOperationId,
				dojo.hitch(this, function(inheritedPermissions) {
					if (inheritedPermissions.isFinal) {
						this._promptFinal(inheritedPermissions, parentOperationId, role);
					} else {
						var permissionsStore = this.getPermissionsStore(role, contextItem);
						
						new com.ibm.team.process.web.ui.internal.admin.projects.PermissionsCustomizeDialog({
							workingCopy: this.workingCopy,
							inheritedPermissions: inheritedPermissions,
							permissionsStore: permissionsStore,
							operationId: parentOperationId,
							contextItem: contextItem,
							roleId: role,
							processAreaEditor: this.processAreaEditor,
							processAreaStore: this.processAreaStore,
							okCallback: dojo.hitch(this, this._permissionsCustomized, row)
						});
					}
				}),
				dojo.hitch(this, function(error) {
					this.processAreaEditor.handleServerResponseError(error, this.messages.PermissionsTable_17);
				})
			);
		},
		
		/**
		 * Subclasses may override or connect to to take action after
		 * permissions are customized.
		 */
		_permissionsCustomized: function(row) {
		},
		
		/**
		 * Alert the user that the configuration of permissions is final in a
		 * parent container. They can only acknowledge that they cannot
		 * customize permissions.
		 */
		_promptFinal: function(inheritedPermissions, operationId, roleId) {
			var promptText = dojo.string.substitute(
				this.messages.PermissionsTable_18, [
					this.getOperationName(operationId),
					this.processAreaEditor.getRolesStore().getRoleLabel(this.processAreaEditor.getRolesStore().fetchRoleById(roleId))
				]
			);
			
			var addScope = dojo.hitch(this, function(context, title) {
				var scopeName;
				var imageUrl = null;
				if (ProcessConstants.PermissionContext.PROJECT === context) {
					scopeName = this.messages.PermissionsTable_19;
					imageUrl = dojo.moduleUrl("com.ibm.team.process.web", "ui/internal/admin/projects/images/project_config_obj.gif"); //$NON-NLS-1$ //$NON-NLS-2$
				} else if (ProcessConstants.PermissionContext.TEAM === context) {
					scopeName = this.messages.PermissionsTable_20;
					imageUrl = dojo.moduleUrl("com.ibm.team.process.web", "ui/internal/admin/projects/images/team_config_obj.gif"); //$NON-NLS-1$ //$NON-NLS-2$
				} else if (ProcessConstants.PermissionContext.DEV_LINE === context) {
					scopeName = this.messages.PermissionsTable_21;
					imageUrl = dojo.moduleUrl("com.ibm.team.process.web", "ui/internal/admin/projects/images/develop_line_obj.gif"); //$NON-NLS-1$ //$NON-NLS-2$
				} else if (ProcessConstants.PermissionContext.ITERATION === context) {
					scopeName = this.messages.PermissionsTable_22;
					imageUrl = dojo.moduleUrl("com.ibm.team.process.web", "ui/internal/admin/projects/images/milestone_obj.gif"); //$NON-NLS-1$ //$NON-NLS-2$
				} else if (ProcessConstants.PermissionContext.ITERATION_TYPE === context) {
					scopeName = this.messages.PermissionsTable_23;
					imageUrl = dojo.moduleUrl("com.ibm.team.process.web", "ui/internal/admin/projects/images/iteration_type_obj.gif"); //$NON-NLS-1$ //$NON-NLS-2$
				} else {
					scopeName = ""; //$NON-NLS-1$
				}
				promptText += "<tr>"; //$NON-NLS-1$
				promptText += "<td>" + jazz.util.html.escape(scopeName) + "</td>"; //$NON-NLS-1$ //$NON-NLS-2$
				promptText += "<td>"; //$NON-NLS-1$
				if (imageUrl !== null) {
					promptText += "<image style=\"vertical-align: middle; margin: 0 2px;\" src=\"" + imageUrl + "\"></image> "; //$NON-NLS-1$ //$NON-NLS-2$
				}
				promptText += jazz.util.html.escape(title);
				promptText += "</td>"; //$NON-NLS-1$
				promptText += "</tr>"; //$NON-NLS-1$
			});
			promptText += "\n\n<table>"; //$NON-NLS-1$
			if (inheritedPermissions.areaContext !== inheritedPermissions.context) {
				addScope(inheritedPermissions.areaContext, inheritedPermissions.areaContextTitle);
			}
			addScope(inheritedPermissions.context, inheritedPermissions.contextTitle);
			promptText += "</table>"; //$NON-NLS-1$
			
			new com.ibm.team.process.web.ui.internal.admin.projects.PromptDialog({
				message: promptText,
				hasCancel: false,
				dialogWidth: "40em" //$NON-NLS-1$
			});
		},
		
		/**
		 * Show or hide the row to edit operation properties.
		 * 
		 * In general, PermissionsTable will call this function when the
		 * button to edit properties has been clicked, but consumers may wish
		 * to close the editor under some circumstances, (eg. row for operation
		 * is a parent row is collapsed).
		 * 
		 * @param	row		DOM node for the row
		 * @param	editing	Boolean that indicates whether to show or hide
		 * 					the editor row
		 * @param	notifyHeightChanged	Optional boolean to indicate whether the
		 * 								callback heightChanged() should be
		 * 								called. This could be set to false to
		 * 								delay layout processing during a batch
		 * 								operation.
		 */
		toggleEditOperationRow: function(row, editing, notifyHeightChanged) {
			dojo.toggleClass(row, "propertiesExpanded", editing); //$NON-NLS-1$

			if (editing) {
				// create the table row to hold it
				row._editorRow = document.createElement("tr"); //$NON-NLS-1$
				dojo.addClass(row._editorRow, "editorRow"); //$NON-NLS-1$

				var td = document.createElement("td"); //$NON-NLS-1$
				td.setAttribute("colspan", row.children.length); //$NON-NLS-1$
				row._editorRow.appendChild(td);
				
				// create the editor
				var contentNode = dojo.create("div", {'class': "content"}); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
				
				var getPermissionsStore = dojo.hitch(this, function(theRow) {
					return this.getPermissionsStore(
						this.getRoleId(theRow),
						this.getContextItem(theRow)
					);
				});
				
				var getPermissionItem = dojo.hitch(this, function(theRow, permissionsStore) {
					return permissionsStore.getPermissionItem(this.getActionId(theRow));
				});
				
				var isFinalCheckboxId = "NewPermissionsByRoleEditorFinalCheckbox" + this._inlineEditorUUID++; //$NON-NLS-1$
				var isFinalCheckboxDiv = dojo.create("div", {}, contentNode); //$NON-NLS-1$
				dojo.addClass(isFinalCheckboxDiv, "propertyControl"); //$NON-NLS-1$
				var label = dojo.create("label", {"for": isFinalCheckboxId}, isFinalCheckboxDiv); //$NON-NLS-1$ //$NON-NLS-2$
				row._editorRow.isFinalCheckbox = dojo.create("input", {"id": isFinalCheckboxId, type: "checkbox"}, label); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
				this.connect(row._editorRow.isFinalCheckbox, "onclick", dojo.hitch(this, function(theRow) { //$NON-NLS-1$
					var permissionsStore = getPermissionsStore(theRow);
					permissionsStore.setFinal(
						getPermissionItem(theRow, permissionsStore),
						this.getActionId(theRow),
						theRow._editorRow.isFinalCheckbox.checked
					);
				}, row));
				label.appendChild(document.createTextNode("\t" + this.messages.PermissionsTable_14)); //$NON-NLS-1$
				
				var setDenialTextConfigured = dojo.hitch(this, function(theRow, configured) {
					setTimeout(dojo.hitch(this, function() {
						var permissionsStore = getPermissionsStore(theRow);
						permissionsStore.setDenialExplanationConfigured(
							getPermissionItem(theRow, permissionsStore),
							this.getActionId(theRow),
							configured
						);
					}), 0);
				});
				
				var denialExplanationCheckboxId = "NewPermissionsByRoleEditorDenialCheckbox" + this._inlineEditorUUID++; //$NON-NLS-1$
				var denialExplanationCheckboxDiv = dojo.create("div", {}, contentNode); //$NON-NLS-1$
				dojo.addClass(denialExplanationCheckboxDiv, "propertyControl"); //$NON-NLS-1$
				label = dojo.create("label", {"for": denialExplanationCheckboxId}, denialExplanationCheckboxDiv); //$NON-NLS-1$ //$NON-NLS-2$
				row._editorRow.denialExplanationCheckbox = dojo.create("input", {"id": denialExplanationCheckboxId, type: "checkbox"}, label); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
				label.appendChild(document.createTextNode("\t" + this.messages.PermissionsTable_15)); //$NON-NLS-1$
				this.connect(row._editorRow.denialExplanationCheckbox, "onclick", dojo.hitch(this, function(theRow) { //$NON-NLS-1$
					if (theRow._editorRow.denialExplanationCheckbox.checked) {
						dojo.toggleClass(theRow._editorRow.denialExplanationText, "disabledText", false); //$NON-NLS-1$
						setDenialTextConfigured(theRow, true);
						theRow._editorRow.denialExplanationText.focus();
					} else {
						dojo.toggleClass(theRow._editorRow.denialExplanationText, "disabledText", true); //$NON-NLS-1$
						setDenialTextConfigured(theRow, false);
					}
				}, row));
				
				var denialExplanationTextDiv = dojo.create("div", {}, contentNode); //$NON-NLS-1$
				dojo.addClass(denialExplanationTextDiv, "propertyControl denialExplanationContainer"); //$NON-NLS-1$
				row._editorRow.denialExplanationText = dojo.create("textarea", {rows: 2, wrap: "soft"}, denialExplanationTextDiv); //$NON-NLS-1$ //$NON-NLS-2$
				dojo.style(row._editorRow.denialExplanationText, 'width', "100%"); //$NON-NLS-1$ //$NON-NLS-2$
				this.connect(row._editorRow.denialExplanationText, "onfocus", dojo.hitch(this, function(theRow) { //$NON-NLS-1$
					if (!theRow._editorRow.denialExplanationCheckbox.checked) {
						theRow._editorRow.denialExplanationCheckbox.checked = true;
						dojo.toggleClass(theRow._editorRow.denialExplanationText, "disabledText", false); //$NON-NLS-1$
						setDenialTextConfigured(theRow, true);
					}
				}, row));
				this.connect(row._editorRow.denialExplanationText, "keyup", dojo.hitch(this, function(theRow) { //$NON-NLS-1$
					var permissionsStore = getPermissionsStore(theRow);
					var permissionItem = getPermissionItem(theRow, permissionsStore);
					var denialExplanation = permissionsStore.getDenialExplanation(permissionItem);
					if (denialExplanation !== row._editorRow.denialExplanationText.value) {
						permissionsStore.setDenialExplanation(
							permissionItem,
							this.getActionId(theRow),
							row._editorRow.denialExplanationText.value
						);
					}
				}, row));
				
				td.appendChild(contentNode);
				
				var role = this.getRoleId(row);
				var contextItem = this.getContextItem(row);
				var actionId = this.getActionId(row);
				var isOperation = this.workingCopy.isOperation(actionId);
				var permissionsStore = getPermissionsStore(row);
				var permissionItem = getPermissionItem(row, permissionsStore);
				var isFinal = permissionsStore.isFinal(permissionItem);
				var isDenialExplanationConfigured = permissionsStore.isDenialExplanationConfigured(permissionItem);
				var denialExplanation = permissionsStore.getDenialExplanation(permissionItem);
				this._updateRowEditor(row, null, isOperation, isFinal, isDenialExplanationConfigured, denialExplanation, role, actionId);

				// add the editor to the document
				if (row.nextSibling) {
					this.tbody.insertBefore(row._editorRow, row.nextSibling);
				} else {
					this.tbody.appendChild(row._editorRow);
				}
			} else if (row._editorRow && this.tbody === row._editorRow.parentNode) {
				// toggle edit off
				row._editorRow.parentNode.removeChild(row._editorRow);
				row._editorRow = undefined;
				dojo.toggleClass(row, "propertiesExpanded", false); //$NON-NLS-1$
			}
			
			// make sure the button state is accurate
			if (row._actionsTd) {
				row._actionsTd._editPropertiesButton.set("pressed", editing); //$NON-NLS-1$
			}
			
			if (false !== notifyHeightChanged && "function" === typeof this.heightChanged) { //$NON-NLS-1$
				this.heightChanged();
			}
		},
		
		/**
		 * Fetches the permissions, (via call to function doFetchPermissions,
		 * which a subclass or consumer must implement), and then updates
		 * every row for which function addPermissionsColumns() was with
		 * permissions information: an appropriate icon in the Permissions
		 * column and the actions available for the row in the Actions column.
		 * 
		 * @param	permissionsLoaded	(Optional) callback function that is
		 * 								invoked after permissions are
		 * 								successfully loaded 
		 */
		reloadPermissions: function(permissionsLoaded) {
			if (this._populatingPermissions === true) {
				//Wait until permissions are no longer being populated before
				//doing so again. Otherwise, data could be loaded by the
				//working copy while permissions for another combination of
				//role and time period are already being loaded
				setTimeout(dojo.hitch(this, this.reloadPermissions, permissionsLoaded), 100);
				return;
			}
			this._populatingPermissions = true;
			
			this.fetchPermissions(
				dojo.hitch(this, function() {
					var i = 0;
					var processBatch = dojo.hitch(this, function() {
						try {
							if (this._permissionsRows === undefined) {
								// Can only happen if the Permissions table was destroyed
								// while population of the rows was still in progress
								return;
							}
							for (var j = 0; j < 20 && i < this._permissionsRows.length; j++) {
								if (!this.updateRowPermissions(this._permissionsRows[i++])) {
									//When this happens, there should be an
									//invocation of reloadPermissions waiting
									//to execute that will populate all
									//rows correctly
									delete this._populatingPermissions;
									return;
								}
							}
							
							if (i === this._permissionsRows.length) {
								if (permissionsLoaded !== undefined) {
									permissionsLoaded();
								}
								delete this._populatingPermissions;
							} else {
								setTimeout(processBatch);
							}
						} catch (error) {
							delete this._populatingPermissions;
							throw error;
						}
					});
					setTimeout(processBatch);
				}),
				dojo.hitch(this, function(error) {
					delete this._populatingPermissions;
					this.processAreaEditor.handleServerResponseError(error, this.messages.PermissionsTable_7);
				})
			);
		},
		
		/**
		 * Returns a permission item for a given store and operation/action
		 * identifier.
		 * 
		 * Default implementation forwards to the API in PermissionsStore.
		 * Subclasses may override if they are able to return the item in a
		 * more performant manner.
		 */
		getPermissionItem: function(store, actionId) {
			return store.getPermissionItem(actionId);
		},
		
		/**
		 * Returns a permissions store for a given role and context.
		 * 
		 * Default implementation forwards to the PermissionsWorkingCopy.
		 * Subclasses may override if they are able to return the item in a
		 * more performant manner.
		 */
		getPermissionsStore: function(roleId, contextItem) {
			return this.workingCopy.getPermissionsStore(roleId, contextItem);
		},
		
		/**
		 * Returns the name of an operation by its identifier.
		 * 
		 * Default implementation forwards to the PermissionsWorkingCopy.
		 * Subclasses may override if they are able to return the item in a
		 * more performant manner.
		 */
		getOperationName: function(operationId) {
			return this.workingCopy.getName(operationId);
		},
		
		/**
		 * Refresh the contents of the Permissions and Actions columns, the
		 * content of the editor for operation properties, and the decoration
		 * icons on the row to reflect the row's current permission.
		 * 
		 * @param	row		DOM node for the row that is to be updated
		 * 
		 * @return	Whether the row was successfully updated, as a boolean.
		 * 			Since this function can be called asynchronously, problems
		 * 			could be encountered such as the store of permissions for
		 * 			a particular row not being loaded. If false is returned,
		 * 			the caller should ensure that this function will be called
		 * 			again after conditions have changed or additional data has
		 * 			been loaded.
		 */
		updateRowPermissions: function(row) {
			var actionId = this.getActionId(row);
			var role = this.getRoleId(row);
			var contextItem = this.getContextItem(row);
			
			var permissionsStore = this.getPermissionsStore(role, contextItem);
			
			if (permissionsStore === null) {
				//This could happen if the result of getActionId(), getRoleId(),
				//of getContextItem() has changed since permissions were loaded.
				//In that case, there should be another call to
				//reloadPermissions() waiting to execute after the current
				//one has finished.
				return false;
			}
			
			var permissionItem = this.getPermissionItem(permissionsStore, actionId);
			
			var permission = permissionsStore.getPermission(permissionItem, contextItem);
			
			var isOperation = this.workingCopy.isOperation(actionId);
			
			this._updatePermission(row, permission);
			this._updateActions(row, permission, isOperation, actionId, contextItem);
			
			if (isOperation || row._editorRow) {
				var isFinal = permissionsStore.isFinal(permissionItem);
				var isDenialExplanationConfigured = permissionsStore.isDenialExplanationConfigured(permissionItem);
				var denialExplanation = permissionsStore.getDenialExplanation(permissionItem);
				
				if (isOperation) {
					this._updateOperationDecorations(row, isFinal, isDenialExplanationConfigured, denialExplanation);
				}
				
				if (row._editorRow) {
					this._updateRowEditor(row, permission, isOperation, isFinal, isDenialExplanationConfigured, denialExplanation, role, actionId);
				}
			} 
			if (!isOperation) {
				// Make sure that any operation decorations get hidden if an action was selected
				this._updateOperationDecorationsForAction(row);
			}
			
			return true;
		},
		
		_updatePermission: function(row, permission) {
			var imageUrl;
			var titleText = ""; //$NON-NLS-1$
			try {
				imageUrl = PERMITTED_TYPES_INFO[permission].imageUrl;
				titleText = this.messages[PERMITTED_TYPES_INFO[permission].nlsKey];
			} catch (error) {
				imageUrl = dojo.moduleUrl("com.ibm.team.process.web", "ui/internal/admin/projects/images/error_obj.gif"); //$NON-NLS-1$ //$NON-NLS-2$
				console.error(this, "Error fetching permission", error); //$NON-NLS-1$
			}
			
			if (row._permissionTd._permissionAnchor === undefined) {
				row._permissionTd._permissionAnchor = dojo.create("a", {href: "#"}, row._permissionTd); //$NON-NLS-1$ //$NON-NLS-2$
				row._permissionTd._permissionImage = dojo.create("img", {}, row._permissionTd._permissionAnchor); //$NON-NLS-1$
				this.connect(row._permissionTd._permissionAnchor, "onclick", function(event) { //$NON-NLS-1$
	    			dojo.stopEvent(event);
	    		});
	    		row._permissionIconConnects = this.selectRowOnFocus(row, row._permissionTd._permissionAnchor);
	    	}
	    	row._permissionTd._permissionAnchor.title = titleText;
	    	row._permissionTd._permissionImage.src = imageUrl;
		},
		
		_updateActions: function(row, permission, isOperation, actionId, contextItem) {
			if (permission === PermittedType.INHERITED) {
				dojo.toggleClass(row._actionsTd._grantPermissionButton.domNode, "hidden", true); //$NON-NLS-1$
				dojo.toggleClass(row._actionsTd._revokePermissionButton.domNode, "hidden", true); //$NON-NLS-1$
				dojo.toggleClass(row._actionsTd._editPropertiesButton.domNode, "hidden", true); //$NON-NLS-1$
				dojo.toggleClass(row._actionsTd._inheritPermissionButton.domNode, "hidden", true); //$NON-NLS-1$
				
				dojo.toggleClass(row._actionsTd._customizePermissionButton.domNode, "hidden", false); //$NON-NLS-1$
				var parentOperationId = this.workingCopy.getParentOperationId(actionId);
				row._actionsTd._customizePermissionButton.domNode.title = dojo.string.substitute(
					this.messages.PermissionsTable_13,
					[
						this.getOperationName(parentOperationId)
					]
				); 
			} else {
				dojo.toggleClass(row._actionsTd._grantPermissionButton.domNode, "hidden", false); //$NON-NLS-1$
				dojo.toggleClass(row._actionsTd._revokePermissionButton.domNode, "hidden", false); //$NON-NLS-1$
				
				if (isOperation) {
					dojo.toggleClass(row._actionsTd._editPropertiesButton.domNode, "hidden", false); //$NON-NLS-1$
				} else {
					dojo.toggleClass(row._actionsTd._editPropertiesButton.domNode, "hidden", true); //$NON-NLS-1$
				}
				
				if (isOperation && (this.workingCopy.isChildArea() || contextItem !== this.processAreaStore.alwaysRootItem) && this.canInherit !== false) {
					dojo.toggleClass(row._actionsTd._inheritPermissionButton.domNode, "hidden", false); //$NON-NLS-1$
				} else {
					dojo.toggleClass(row._actionsTd._inheritPermissionButton.domNode, "hidden", true); //$NON-NLS-1$
				}
				
				dojo.toggleClass(row._actionsTd._customizePermissionButton.domNode, "hidden", true); //$NON-NLS-1$
				
				if (permission === PermittedType.PERMITTED) {
					row._actionsTd._grantPermissionButton.set("enabled", false); //$NON-NLS-1$
					row._actionsTd._revokePermissionButton.set("enabled", true); //$NON-NLS-1$
				} else if (permission === PermittedType.NOT_PERMITTED) {
					row._actionsTd._grantPermissionButton.set("enabled", true); //$NON-NLS-1$
					row._actionsTd._revokePermissionButton.set("enabled", false); //$NON-NLS-1$
				} else {
					row._actionsTd._grantPermissionButton.set("enabled", true); //$NON-NLS-1$
					row._actionsTd._revokePermissionButton.set("enabled", true); //$NON-NLS-1$
				}
			}
		},
		
		_updateOperationDecorations: function(row, isFinal, isDenialExplanationConfigured, denialExplanation) {
			dojo.toggleClass(row._isFinalIcon, "hidden", !isFinal); //$NON-NLS-1$
			dojo.toggleClass(row._denialExplanationIcon, "hidden", !isDenialExplanationConfigured); //$NON-NLS-1$
			if (isDenialExplanationConfigured) {
				row._denialExplanationIcon.title = dojo.string.substitute(this.messages.PermissionsTable_24, [denialExplanation || ""]); //$NON-NLS-1$
			}
		},
		
		_updateOperationDecorationsForAction: function(row) {
			if (row._isFinalIcon !== undefined) {
				dojo.addClass(row._isFinalIcon, "hidden"); //$NON-NLS-1$
			}
			if (row._denialExplanationIcon !== undefined) {
				dojo.addClass(row._denialExplanationIcon, "hidden"); //$NON-NLS-1$
			}
		},
		
		_updateRowEditor: function(row, permission, isOperation, isFinal, isDenialExplanationConfigured, denialExplanation, role, actionId) {
			if (isOperation && PermittedType.INHERITED !== permission) {
				row._editorRow.isFinalCheckbox.checked = isFinal;
				
				if (row._editorRow.denialExplanationText.value !== denialExplanation) {
					row._editorRow.denialExplanationText.value = null === denialExplanation ? "" : denialExplanation; //$NON-NLS-1$
				}
				if (isDenialExplanationConfigured) {
					row._editorRow.denialExplanationCheckbox.checked = true;
					dojo.toggleClass(row._editorRow.denialExplanationText, "disabledText", false); //$NON-NLS-1$
				} else {
					row._editorRow.denialExplanationCheckbox.checked = false;					
					dojo.toggleClass(row._editorRow.denialExplanationText, "disabledText", true); //$NON-NLS-1$
				}
			} else {
				//If the row no longer represents an operation or the permission
				//is inherited (so properties are not editable), close the inline
				//editor
				this.toggleEditOperationRow(row, false);
			}
		},
		
		uninitialize: function() {
			this.inherited(arguments);
			var permissionsRows = this._permissionsRows;
			var rowIndex = 0;
			// function which uninitializes 10 rows then calls itself
			// using setTimeout until all rows are processed
			var uninitializeRows = dojo.hitch(this, function() {
				var lastRowInBatch = rowIndex + 10;		
				while (rowIndex < lastRowInBatch) {
					if (rowIndex === permissionsRows.length) {
                        return;
					}
					var row = permissionsRows[rowIndex];
					if (row._actionsTd._inheritPermissionButton) {
						row._actionsTd._inheritPermissionButton.destroyRecursive();
					}
					if (row._actionsTd._editPropertiesButton) {
						row._actionsTd._editPropertiesButton.destroyRecursive();
					}
					if (row._actionsTd._customizePermissionButton) {
						row._actionsTd._customizePermissionButton.destroyRecursive();
					}
					if (row._actionsTd._grantPermissionButton) {
						row._actionsTd._grantPermissionButton.destroyRecursive();
					}
					if (row._actionsTd._revokePermissionButton) {
						row._actionsTd._revokePermissionButton.destroyRecursive();
					}
					if (row._actionsTd._toolbar) {
						row._actionsTd._toolbar.destroyRecursive();
					}
					if (row._permissionIconConnects) {
						dojo.forEach(row._permissionIconConnects, dojo.hitch(this, function(connect) {
							this.disconnect(connect);
						}));
					}
					rowIndex++;
				}
				setTimeout(uninitializeRows);				
			});
			uninitializeRows();
			delete this._permissionsRows;
		}
	});
})();
