/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2012, 2013. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function(){
	dojo.provide("com.ibm.team.process.web.ui.internal.admin.projects.SelectProcessAreaWidget"); //$NON-NLS-1$

	dojo.require("com.ibm.team.process.web.ui.internal.admin.projects.ProcessTree"); //$NON-NLS-1$
	dojo.require("com.ibm.team.process.web.client.internal.ProcessClient"); //$NON-NLS-1$
	dojo.require("com.ibm.team.process.web.common.internal.ProcessConstants"); //$NON-NLS-1$
	dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler"); //$NON-NLS-1$
	
	dojo.require("dijit._Templated"); //$NON-NLS-1$
	dojo.require("dijit._Widget"); //$NON-NLS-1$
	dojo.require("dojo.data.ItemFileReadStore"); //$NON-NLS-1$
	dojo.require("dojo.i18n"); //$NON-NLS-1$
	
	dojo.require("jazz.ui.MessageBox"); //$NON-NLS-1$
	dojo.require("jazz.ui.Toolbar"); //$NON-NLS-1$
	dojo.require("jazz.ui.toolbar.TwoStateButton"); //$NON-NLS-1$
	dojo.require("jazz.ui.tree.IconItem"); //$NON-NLS-1$
	dojo.require("jazz.util.html"); //$NON-NLS-1$
	
	dojo.requireLocalization("com.ibm.team.process.web.ui.internal", "Messages"); //$NON-NLS-1$ //$NON-NLS-2$
	
	var ProcessConstants = com.ibm.team.process.web.common.internal.ProcessConstants;
	var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;
	var ProcessClient = com.ibm.team.process.web.client.internal.ProcessClient;
	
	var ROOT_ID = "$ROOT$"; //$NON-NLS-1$
	
	/**
	 * A general-purpose widget that presents a tree of project/team areas to
	 * select from. Options to select from the following trees are presented:
	 * 
	 * 1. Teams and projects in the direct parent chain of a particular team
	 *    or project.
	 * 2. The hierarchy of teams underneath the parent project area of a team/
	 *    project.
	 * 3. All projects and teams in the repository.
	 * 
	 * The label for each of the filters may optionally be customized.
	 */
	dojo.declare("com.ibm.team.process.web.ui.internal.admin.projects.SelectProcessAreaWidget", //$NON-NLS-1$
			[dijit._Widget, dijit._Templated], {
		
		templatePath: dojo.moduleUrl("com.ibm.team.process.web", "ui/internal/admin/projects/templates/SelectProcessAreaWidget.html"), //$NON-NLS-1$ //$NON-NLS-2$
		
		/**
		 * The argument to the constructor must be an object with the following
		 * properties:
		 * 
		 * processAreaItemId (required)
		 *     The identifier of a project or team area that will be used to
		 *     display in a hierarchy, either to display the chain of parents to
		 *     the root project area, or to display all teams beneath the root
		 *     project area, depending on the filtering selected by the user in
		 *     this widget
		 * 
		 * owningApplicationKey (required)
		 *     The owning application key of the application whose project
		 *     or team area is being edited. Will be used to display all
		 *     projects and teams in the application
		 * 
		 * editorContext (required)
		 *     A constant ProcessConstants.EditorContext to denote whether a the
		 *     processAreaItemId is for a project or team area
		 * 
		 * onErrorCallback (required)
		 *     A function that will be invoked if there is an exception fetching
		 *     the process areas from the server. The error object provided by
		 *     the XHR call will be passed to this function.
		 *  
		 * summaryText (optional)
		 *     A string to display at the top of the widget.
		 * 
		 * filterLabelProvider (optional)
		 *     A function that the widget will invoke with each of the
		 *     constants in com.ibm.team.process.web.ui.internal.admin.projects.
		 *     SelectProcessAreaWidget.FilterType. If the function returns a
		 *     string for a given argument, that string is used as the label
		 *     of the control associated with the particular filter 
		 */
		constructor: function(args) {
			this.messages = dojo.i18n.getLocalization("com.ibm.team.process.web.ui.internal.admin.projects", "Messages"); //$NON-NLS-1$ //$NON-NLS-2$
			this._buttonIds = [
				"buttonParentHierarchy", //$NON-NLS-1$
				"buttonProjectHierarchy", //$NON-NLS-1$
				"buttonOtherProjects" //$NON-NLS-1$
			];
			this.pageHoverHelps = [];
		},
		
		postCreate: function() {
			this._messageBox = new jazz.ui.MessageBox().placeAt(this.messageBoxNode, "only"); //$NON-NLS-1$;
			dojo.addClass(this._messageBox.domNode, "messageBox"); //$NON-NLS-1$
			
			if (this.summaryText !== undefined) {
				this.summaryTextNode.appendChild(document.createTextNode(this.summaryText));
			}
			
			this._createShowArchivedButton();
			
			//Each of the radio buttons for filtering is configured with
			//variables and callbacks that enable the appropriate tree of
			//project/team areas to be displayed when they are clicked
			var initButton = dojo.hitch(this, function(button, properties) {
				//_treeNode is the DOM element that is shown when the button
				//is selected, and hidden when it is not
				button._treeNode = properties.treeNode;
				
				//_modelFilterFunction is an optional callback that filters
				//the project/team items that are displayed from the set of
				//all projects/teams in the button's store
				button._modelFilterFunction = properties.modelFilterFunction;
				
				//_getStoreFunction is a callback that returns a dojo.data
				//store with a project/team hierarchy. We have this because
				//some buttons share a common store, so clicking a button for
				//the first time doesn't mean that a store needs to be loaded
				button._getStoreFunction = properties.getStoreFunction;
				
				//_loadStoreFunction is a callback that loads an appropriate
				//dojo.data store, probably by making a server call. It will
				//be invoked when a button is selected and _getStoreFunction
				//for that button returns null
				button._loadStoreFunction = properties.loadStoreFunction;
				
				//If the creator of this SelectProcessAreaWidget specifies
				//custom text for this button, replace the label text with
				//that value
				var labelOverride = this.filterLabelProvider
					? this.filterLabelProvider(properties.filterType)
					: null;
				if (labelOverride !== null) {
					button.nextSibling.innerHTML = jazz.util.html.removeScript(labelOverride);
				}
				
				//_treeSummaryText is the text to display above the tree when
				//the button is selected
				button._treeSummaryText = properties.treeSummaryText;
				
				//Listen for click events on the button
				this.connect(button, "onclick", "_buttonSelected"); //$NON-NLS-1$ //$NON-NLS-2$
			});
			
			//The button to view the direct parent chain of the project/team.
			//The store is shared with the button to view the hierarchy of a
			//project, because the web UI functions getHierarchyForTeamArea/
			//getHierarchyForProjectArea both return all child teams of a
			//project. A filter function is used to only show the direct
			//parent hierarchy for this button.
			initButton(this.buttonParentHierarchy, {
				treeNode: this.processAreasParentHierarchyTreeNode,
				modelFilterFunction: function(store, items) {
					return dojo.filter(items, function(item) {
						return store.getValue(item, "inProcessAreaHierarchy"); //$NON-NLS-1$
	    			});
				},
				getStoreFunction: dojo.hitch(this, this._getParentProjectHierarchyAndProcessProviderStore),
				loadStoreFunction: dojo.hitch(this, this._loadParentProjectHierarchyAndProcessProviderStore),
				filterType: com.ibm.team.process.web.ui.internal.admin.projects.SelectProcessAreaWidget.FilterType.PARENT_HIERARCHY,
				treeSummaryText: this.messages.SelectProcessAreaWidget_3
			});
			
			//The button to view all teams underneath a parent project. The
			//store is shared with the button to view the parent chain, as
			//explained above
			initButton(this.buttonProjectHierarchy, {
				treeNode: this.processAreasProjectHierarchyTreeNode,
				getStoreFunction: dojo.hitch(this, this._getParentProjectHierarchyStore),
				loadStoreFunction: dojo.hitch(this, this._loadParentProjectHierarchyStore),
				filterType: com.ibm.team.process.web.ui.internal.admin.projects.SelectProcessAreaWidget.FilterType.PROJECT_HIERARCHY,
				treeSummaryText: this.messages.SelectProcessAreaWidget_4
			});
			
			//The button to view all other projects and teams in the repository.
			//The server call might be expensive for this one, but it is not
			//made until the button is selected, and the button is not
			//selected by default
			initButton(this.buttonOtherProjects, {
				treeNode: this.processAreasOtherProjectsTreeNode,
				getStoreFunction: dojo.hitch(this, this._getOtherProjectsStore),
				loadStoreFunction: dojo.hitch(this, this._loadOtherProjectsStore),
				filterType: com.ibm.team.process.web.ui.internal.admin.projects.SelectProcessAreaWidget.FilterType.OTHER_PROJECTS_AND_TEAMS,
				treeSummaryText: this.messages.SelectProcessAreaWidget_4
			});
			
			this.buttonParentHierarchy.checked = true;
			this._primButtonSelected(this.buttonParentHierarchy);
		},
		
		/**
		 * Callback when any of the radio buttons are selected. Separate from
		 * _primButtonSelected so that the latter function can be called
		 * programatically.
		 */
		_buttonSelected: function(event) {
			this._primButtonSelected(event.target);
		},
		
		_primButtonSelected: function(button) {
			//Hide the tree nodes for the buttons that are not selected, and
			//show the tree node for the button that is
			dojo.forEach(this._buttonIds, dojo.hitch(this, function(buttonId) {
				if (this[buttonId] === button) {
					dojo.removeClass(this[buttonId]._treeNode, "hidden"); //$NON-NLS-1$
				} else {
					dojo.addClass(this[buttonId]._treeNode, "hidden"); //$NON-NLS-1$
				}
			}));
			
			if (this.treeSummaryText._text) {
				this.treeSummaryText.removeChild(this.treeSummaryText._text);
			}
			this.treeSummaryText._text = this.treeSummaryText.appendChild(document.createTextNode(button._treeSummaryText));
			this._showArchivedButton.tree = button._processAreasTree;
			if (this._showArchivedButton.tree && this._showArchivedButton.tree.model.showArchived !== this._showArchivedButton.get("pressed")) { //$NON-NLS-1$
				this._showArchivedButton.onClick();
			}
			
			//Notify the listener that the selection has changed. When a
			//button is selected for the first time this will notify that
			//nothing is selected, and when a button is clicked that had been
			//clicked previously, we'll notify that whatever was selected
			//in that buttons tree is selected again
			this.selectionChangedCallback({
				processAreaId: button._treeNode._selectedProcessAreaId,
				processAreaUrl: button._treeNode._selectedProcessAreaUrl,
				processAreaContext: button._treeNode._selectedProcessAreaContext,
				isInParentHierarchy: button._treeNode._selectedIsInParentHierarchy,
				isProcessProvider: button._treeNode._selectedIsProcessProvider
			});
			
			//Create the tree if necessary
			if (!button._processAreasTree) {
				var createTree = dojo.hitch(this, function() {
					var store = button._getStoreFunction();
					button._processAreasTree = this._createTree(store, button._modelFilterFunction, button._treeNode);
					this._showArchivedButton.tree = button._processAreasTree;
					//Select first node, if a node is visible
					store.fetch({
						count: 1,
						onItem: dojo.hitch(this, function(item) {
							var id = store.getIdentity(item);
							if (id === this.processAreaItemId && button === this.buttonParentHierarchy) {
								//We're filtering by parent hierarchy, and the
								//top-level node is the project we're editing
								//(there is no process provider). In this case
								//there is nothing to show in the parent
								//hierarchy filter, so disable it and select
								//the next filter instead
								this.buttonParentHierarchy.disabled = true;
								dojo.addClass(this.buttonParentHierarchy.parentNode, "disabled"); //$NON-NLS-1$
								this.buttonProjectHierarchy.checked = true;
								this._primButtonSelected(this.buttonProjectHierarchy);
							} else if (button._processAreasTree.model.showArchived || !store.getValue(item, "archived")) { //$NON-NLS-1$
								button._processAreasTree.set("path", [ROOT_ID, id]); //$NON-NLS-1$
								button._processAreasTree.focusChanged({
									item: item
								});
							}
						})
					});
				});
				if (button._getStoreFunction()) {
					//We already have a store. Just create the tree
					createTree();
				} else {
					//Load the store, and then create the tree
					this._afterCreateStoreCallback = createTree;
					button._loadStoreFunction();
				}
			}
		},
		
		_createShowArchivedButton: function() {
			this._treeToolbar = new jazz.ui.Toolbar();
			this._showArchivedButton = new jazz.ui.toolbar.TwoStateButton({
				title: this.messages.SelectProcessAreaWidget_5,
				imgSrc: dojo.moduleUrl("com.ibm.team.process.web", "ui/internal/admin/projects/images/projectarea_archived_obj.gif"), //$NON-NLS-1$ //$NON-NLS-2$
				onClick: function() {
					this.tree.model.showArchived = this.pressed;
					var items = [this.tree.model.root];
					while (items.length > 0) {
						var item = items.shift();
						this.tree.model.getChildren(item, dojo.hitch(this, function(newChildren) {
							this.tree.model.onChildrenChange(item, newChildren);
							items = items.concat(newChildren);
						}));
					}
				}
			}).placeAt(this._treeToolbar);
			this.treeHeader.appendChild(this._treeToolbar.domNode);
		},
		
		/**
		 * Helper that displays "Loading..." text in the message area
		 * and disables all controls while the information is fetched
		 */
		_startLoading: function() {
			//Show Loading... message until finished with fetch from server
			this._messageBox.showMessage({
				summary: this.messages.SelectProcessAreaWidget_1,
				type: jazz.ui.MessageBox.STATUS_TYPES.INFO
			});
			this._resizeProcessAreasTree();
			
			dojo.forEach(this._buttonIds, dojo.hitch(this, function(buttonId) {
				this[buttonId]._prevDisabled = this[buttonId].disabled;
				this[buttonId].disabled = true;
			}));
		},
		
		/**
		 * Helper that hides the "Loading..." text and enables the
		 * controls
		 */
		_stopLoading: function() {
			this._messageBox.hideMessage();
			this._resizeProcessAreasTree();
			dojo.forEach(this._buttonIds, dojo.hitch(this, function(buttonId) {
				this[buttonId].disabled = this[buttonId]._prevDisabled;
			}));
		},
		
		//The following functions load the store for the direct parent filter
		//Note that the project hierarchy store is loaded as a side-affect,
		//(so that it won't have to be loaded if the user clicks on that filter)
		
		_getParentProjectHierarchyAndProcessProviderStore: function() {
			return this._parentProjectHierarchyAndProcessProviderStore;
		},
		
		_loadParentProjectHierarchyAndProcessProviderStore: function() {
			var origAfterCreateStoreCallback = this._afterCreateStoreCallback;
			
			//Create the store for the project hierarchy, because we'll either
			//use its contents to build the parent hierarchy + process provider
			//store if there is a process provider, or refer to it directly
			//if there isn't.
			this._afterCreateStoreCallback = dojo.hitch(this, function() {
				this._afterCreateStoreCallback = origAfterCreateStoreCallback;
				
				//Use service getProjectAreaByUUIDWithLimitedMembers to get the
				//process provider information
				var rootProjectUUID;
				this._parentProjectHierarchyStore.fetch({
					onItem: dojo.hitch(this, function(item) {
						rootProjectUUID = this._parentProjectHierarchyStore.getValue(item, "itemId"); //$NON-NLS-1$
					})
				});
				var srh = new ServiceResponseHandler(this, "_createParentProjectHierarchyAndProcessProviderStore", "_errorFetchingProcessAreas"); //$NON-NLS-1$ //$NON-NLS-2$
				ProcessClient.getProjectAreaByUUIDWithLimitedMembers({
					processAreaItemId: rootProjectUUID,
					maxMembers: 0
				}, srh);
			});
			this._loadParentProjectHierarchyStore();
		},
		
		_createParentProjectHierarchyAndProcessProviderStore: function(data) {
			if (data.processProviderDTO) {
				//The root project area is consuming process. Create a store
				//where the single root node is the process provider, and its
				//children are the direct hierarchy chain to this widget's
				//process area, where each node is a deep copy of the nodes
				//in the project hierarchy store (exceptions occur if items
				//are put into more than one Dojo store or if the same array
				//is used to create more than one store)
				var processProvider = {
					name: data.processProviderDTO.name,
					inProcessAreaHierarchy: true,
					isProject: true,
					isProcessProvider: true
				};
				if (data.processProviderDTO.itemId) {
					processProvider.itemId = data.processProviderDTO.itemId;
					processProvider.id = processProvider.itemId;
				} else {
					processProvider.isRemoteProcessProvider = true;
					processProvider.url = data.processProviderDTO.url;
					processProvider.id = processProvider.url;
				}
				var parentNode = processProvider;
				this._parentProjectHierarchyStore.fetch({
					onItem: dojo.hitch(this, function(item) {
						while(item !== null) {
							//Copy the project/team node
							var newParent = {
								itemId: this._parentProjectHierarchyStore.getValue(item, "itemId"), //$NON-NLS-1$
								name: this._parentProjectHierarchyStore.getValue(item, "name"), //$NON-NLS-1$
								isProject: this._parentProjectHierarchyStore.getValue(item, "isProject"), //$NON-NLS-1$
								inProcessAreaHierarchy: this._parentProjectHierarchyStore.getValue(item, "itemId") !== this.processAreaItemId //$NON-NLS-1$
							};
							newParent.id = newParent.itemId;
							parentNode.children = [newParent];
							parentNode = newParent;
							var children = this._parentProjectHierarchyStore.getValues(item, "children"), //$NON-NLS-1$
							
							//See if the node has a child that's also in the
							//parent hierarchy. Add it to the object for the
							//new store if so
							item = null;
							for (var i = 0; i < children.length && item === null; i++) {
								if (this._parentProjectHierarchyStore.getValue(children[i], "inProcessAreaHierarchy")) { //$NON-NLS-1$
									item = children[i];
								}
							}
						}
					})
				});
				this._parentProjectHierarchyAndProcessProviderStore = this._primCreateStore([processProvider]);
			} else {
				//The root project area is not consuming process. Just use the
				//project hierarchy store
				this._parentProjectHierarchyAndProcessProviderStore = this._parentProjectHierarchyStore;
			}
			this._afterCreateStoreCallback();
		},
		
		//The following functions load the store for the project hierarchy
		//filter
		
		_getParentProjectHierarchyStore: function() {
			return this._parentProjectHierarchyStore;
		},
		
		_loadParentProjectHierarchyStore: function() {
			this._startLoading();
			
			var srh = new ServiceResponseHandler(this, "_createParentProjectHierarchyStore", "_errorFetchingProcessAreas"); //$NON-NLS-1$ //$NON-NLS-2$
			var args = {};
			args.uuid = this.processAreaItemId;
			if (ProcessConstants.EditorContext.TEAM === this.editorContext) {
				ProcessClient.getHierarchyForTeamArea(args, srh);
			} else {
				ProcessClient.getProjectHierarchy(args, srh);
			}
		},
		
		_createParentProjectHierarchyStore: function(projectNode) {
			this._stopLoading();
			
			this._parentProjectHierarchyResponse = projectNode;
			this._parentProjectHierarchyStore = this._createStore([projectNode]);
			
			this._afterCreateStoreCallback();
		},
		
		//The following functions load the store used for the all projects
		//filter
		
		_getOtherProjectsStore: function() {
			return this._otherProjectsStore;
		},
		
		_loadOtherProjectsStore: function() {
			this._startLoading();
			
			var srh = new ServiceResponseHandler(this, "_createOtherProjectsStore", "_errorFetchingProcessAreas"); //$NON-NLS-1$ //$NON-NLS-2$
			ProcessClient.getProcessAreaHierarchies(srh, {
				owningApplicationKey: this.owningApplicationKey,
				includeArchived: true
			});
		},
		
		_createOtherProjectsStore: function(projectNodes) {
			this._stopLoading();
			
			this._otherProjectsStore = this._createStore(projectNodes);
			
			//Remove the current project area, or root project area if current
			//process area is a team, from the store
			this._otherProjectsStore.fetch({
				query: this.editorContext === ProcessConstants.EditorContext.PROJECT
					? {itemId: this.processAreaItemId}
					: {inProcessAreaHierarchy: true},
				onItem: dojo.hitch(this, function(item) {
					this._otherProjectsStore.deleteItem(item);
				})
			});
			
			this._afterCreateStoreCallback();
		},
		
		/**
		 * A helper to create a store for a hierarchy of projects and teams.
		 * The project nodes are given a flag "isProject" to identify
		 * them as projects later. Projects/teams that are in the direct
		 * parent chain of the widget's process area are given a flag
		 * "inProcessAreaHierarchy".
		 */
		_createStore: function(projectNodes) {
			//Sort the projects by name
			projectNodes = projectNodes.sort(function(a, b){
				return ProcessClient.stringCompare(a.name, b.name);
			});
			
			//Populate field parentId on all of the child items in the data.
			//This will be used to designate which nodes are in the parent
			//hierarchy of the current process area, which is required for one
			//of the filters that the user may select
			
			var processChildren = function(node) {
				node.id = node.itemId;
				if (node.children) {
					//Sort the children by name
					node.children = node.children.sort(function(a, b){
						return ProcessClient.stringCompare(a.name, b.name);
					});
					
					for (var i = 0; i < node.children.length; i++) {
						node.children[i].parentId = node.itemId;
						processChildren(node.children[i]);
					}
				}
			};
			
			dojo.forEach(projectNodes, function(projectNode) {
				projectNode.isProject = true;
				projectNode.id = projectNode.itemId;
				processChildren(projectNode);
			});
			
			var store = this._primCreateStore(projectNodes);
			
			//Designate the items that are in the hierarchy of the current
			//process area by iterating up that item's parent chain
			
			var parentId = this.processAreaItemId;
			while (parentId !== undefined) {
				store.fetch({
					query: {
						itemId: parentId
					},
					queryOptions: {
						deep: true
					},
					onItem: dojo.hitch(this, function(item) {
						if (item === null) {
							parentId = null;
						} else {
							if (store.getValue(item, "itemId") !== this.processAreaItemId) { //$NON-NLS-1$
								store.setValue(item, "inProcessAreaHierarchy", true); //$NON-NLS-1$
							}
							parentId = store.getValue(item, "parentId"); //$NON-NLS-1$
						}
					})
				});
			}
			
			return store;
		},
		
		_primCreateStore: function(items) {
			return new dojo.data.ItemFileWriteStore({
				data: {
					label: "name", //$NON-NLS-1$,
					childrenAttr: "children", //$NON-NLS-1$,
					items: items,
					identifier: "id" //$NON-NLS-1$,
				}
			});
		},
		
		/**
		 * Create a tree from a store and optional filter function
		 */
		_createTree: function(store, filterFunction, parentNode) {
			//Create a dijit.Tree based on the data, backed by a model which
			//filters items appropriately
			
			var getSelectionChangedCallbackArgument = function() {
				return {
					processAreaId: parentNode._selectedProcessAreaId,
					processAreaUrl: parentNode._selectedProcessAreaUrl,
					processAreaContext: parentNode._selectedProcessAreaContext,
					isInParentHierarchy: parentNode._selectedIsInParentHierarchy,
					isProcessProvider: parentNode._selectedIsProcessProvider
				};
			};
			
			return new com.ibm.team.process.web.ui.internal.admin.projects.ProcessTree({
				model: this._createTreeModel(store, filterFunction),
				showRoot: false,
				autoExpand: true,
				getIconClass: dojo.hitch(this, function(item) {
					if (store.isItem(item)) {
						if (store.getValue(item, "isProject") && !store.getValue(item, "archived")) { //$NON-NLS-1$ //$NON-NLS-2$
							return "icon projectIcon"; //$NON-NLS-1$
						} else if (store.getValue(item, "isProject")) { //$NON-NLS-1$
							return "icon archivedProjectIcon"; //$NON-NLS-1$
						} else if (!store.getValue(item, "archived")) { //$NON-NLS-1$
							return "icon teamIcon"; //$NON-NLS-1$
						} else {
							return "icon archivedTeamIcon"; //$NON-NLS-1$
						}
					}
					return null;
				}),
				focusChanged: dojo.hitch(this, function(node) {
					var item = node.item;
					parentNode._selectedProcessAreaId = store.getValue(item, "itemId"); //$NON-NLS-1$
					parentNode._selectedProcessAreaUrl = store.getValue(item, "url"); //$NON-NLS-1$
					parentNode._selectedProcessAreaContext = store.hasAttribute(item, "isProject") //$NON-NLS-1$;
							? ProcessConstants.PermissionContext.PROJECT
							: ProcessConstants.PermissionContext.TEAM;
					parentNode._selectedIsInParentHierarchy = store.getValue(item, "inProcessAreaHierarchy") || false; //$NON-NLS-1$
					parentNode._selectedIsProcessProvider = store.getValue(item, "isProcessProvider") || false; //$NON-NLS-1$
					this.selectionChangedCallback(getSelectionChangedCallbackArgument());
				}),
				onDblClick: dojo.hitch(this, function() {
					this.selectionChangedCallback(getSelectionChangedCallbackArgument(), true);
				})
			}).placeAt(parentNode, "only"); //$NON-NLS-1$
		},
		
		/**
		 * Create a ForestStoreModel that will be used to back a tree of project
		 * /team areas, configurable via an argument "filterFunction" which
		 * determines which items to show.
		 */
		_createTreeModel: function(processAreasStore, filterFunction) {
			return new dojo.declare(dijit.tree.ForestStoreModel, {
				_filterFunction: function(items) {
					if (filterFunction) {
						items = filterFunction(processAreasStore, items);
					}
					if (!this.showArchived && items.length > 0) {
						items = dojo.filter(items, function(item) {
							return !processAreasStore.getValue(item, "archived"); //$NON-NLS-1$
						});
					}
					return items;
				},
				
				getChildren: function(/*dojo.data.Item*/ parentItem, /*function(items)*/ callback, /*function*/ onError) {
					var oldCallback = callback;
					// only call callback on items that match the query
					callback = dojo.hitch(this, function(items) {
						oldCallback(this._filterFunction(items));
					});
					this.inherited(arguments);
				},
				
				mayHaveChildren: function(item) {
					var children = processAreasStore.getValues(item, "children"); //$NON-NLS-1$
					return this._filterFunction(children).length !== 0;
				}
			})({
				childrenAttrs: ["children"], //$NON-NLS-1$
				store: processAreasStore,
				rootId: ROOT_ID,
				showArchived: this._showArchivedButton.get("pressed") //$NON-NLS-1$
			});
		},
		
		/**
		 * Position the project/team areas tree to be as large as as constrained
		 * by the size of the container of this widget and the height of
		 * everything besides the tree section inside this widget.
		 */
		_resizeProcessAreasTree: function() {
			setTimeout(dojo.hitch(this, function() {
				if (dojo.marginBox(this.domNode).h === 0) {
					//Re-schedule until we're called after our content is
					//visible on the page
					setTimeout(dojo.hitch(this, this._resizeProcessAreasTree), 30);
					return;
				}
				var containerBox = dojo.contentBox(this.domNode.parentNode);
				dojo.style(this.processAreasTreeSection, "height", (containerBox.h - dojo.marginBox(this.processAreasTreeSection).t) + "px"); //$NON-NLS-1$; //$NON-NLS-2$;
			}), 0);
		},
		
		/**
		 * Callback for the Process Web UI service request to fetch the project/
		 * team hierarchy. Show an error message inside of this widget, and
		 * notify our consumer.
		 */
		_errorFetchingProcessAreas: function(error) {
			this._messageBox.showMessage({
				summary: this.messages.SelectProcessAreaWidget_2,
				errorObject: error
			});
			this._resizeProcessAreasTree();
			this.onErrorCallback(error);
		},
		
		uninitialize: function() {
			if (this._messageBox) {
				this._messageBox.destroyRecursive();
			}
			
			var destroyTree = function(parentNode) {
				if (parentNode._processAreasTree) {
					parentNode._processAreasTree.model.destroy();
					parentNode._processAreasTree.destroyRecursive();
				}
			};
			destroyTree(this.processAreasParentHierarchyTreeNode);
			destroyTree(this.processAreasProjectHierarchyTreeNode);
			destroyTree(this.processAreasOtherProjectsTreeNode);
			
			dojo.forEach(this.pageHoverHelps, function(pageHoverHelp) {
				if (pageHoverHelp) {
					pageHoverHelp.destroyRecursive();
				}
			});
			
			this.inherited(arguments);
		}
	});
	
	com.ibm.team.process.web.ui.internal.admin.projects.SelectProcessAreaWidget.FilterType = {
		PARENT_HIERARCHY: "PARENT_HIERARCHY", //$NON-NLS-1$;
		PROJECT_HIERARCHY: "PROJECT_HIERARCHY",  //$NON-NLS-1$;
		OTHER_PROJECTS_AND_TEAMS: "OTHER_PROJECTS_AND_TEAMS" //$NON-NLS-1$;
	};
})();
