/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2013. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function(){
	dojo.provide("com.ibm.team.process.web.ui.internal.admin.projects.TeamAreaEditor"); //$NON-NLS-1$
	
	dojo.require("com.ibm.team.process.web.ui.internal.admin.projects.AbstractProcessAreaEditor"); //$NON-NLS-1$
	dojo.require("com.ibm.team.process.web.ui.internal.admin.projects.UIHelper");//$NON-NLS-1$
	
	dojo.require("dojox.string.BidiComplex"); //$NON-NLS-1$
	dojo.require("dojox.xml.parser"); //$NON-NLS-1$
	
	dojo.require("jazz.client.xhr"); //$NON-NLS-1$
	dojo.require("jazz.opensocial.Gadget"); //$NON-NLS-1$
	dojo.require("jazz.ui.toolbar.Button"); //$NON-NLS-1$
	
	dojo.requireLocalization("com.ibm.team.process.web.ui.internal.admin.projects", "Messages"); //$NON-NLS-1$ //$NON-NLS-2$
	
	var ProcessClient = com.ibm.team.process.web.client.internal.ProcessClient;
	var ProcessConstants = com.ibm.team.process.web.common.internal.ProcessConstants;
	var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;
	var MSG_COOKIE_NAME = ProcessConstants.TeamAreaEditAction.ID + "#saveMsg"; //$NON-NLS-1$
	var NEW_MEMBERS_COOKIE_NAME = "com.ibm.team.process.web.ui.internal.admin.projects.ProcessAreaEditor#newMembers"; //$NON-NLS-1$
	var NEW_ADMINISTRATORS_COOKIE_NAME = "com.ibm.team.process.web.ui.internal.admin.projects.ProcessAreaEditor#newAdministrators"; //$NON-NLS-1$
	var UIHelper = com.ibm.team.process.web.ui.internal.admin.projects.UIHelper;
	var HIERARCHY_TAB_SPACING = 1.5;
	
	dojo.declare("com.ibm.team.process.web.ui.internal.admin.projects.TeamAreaEditor", com.ibm.team.process.web.ui.internal.admin.projects.AbstractProcessAreaEditor, { //$NON-NLS-1$
		templatePath: dojo.moduleUrl("com.ibm.team.process.web", "ui/internal/admin/projects/templates/TeamAreaEditor.html"), //$NON-NLS-1$ //$NON-NLS-2$
		initialize: function(args){
			this.context = ProcessConstants.EditorContext.TEAM;
			if (!this.isNew) {
				this.itemId = args.itemId;
			}
			this.projectAreaItemId = args.projectAreaItemId;
			this.parentItemId = args.parentItemId;
			this.parentName = args.parentName;
			this.parentType = args.parentType;
			this._teamAreaIconSrc = dojo.moduleUrl("com.ibm.team.process.web", "ui/internal/admin/projects/images/team-area_obj32.png"); //$NON-NLS-1$ //$NON-NLS-2$
			this._teamAreaIconClass = "itemIcon"; //$NON-NLS-1$
			if (!this.isNew) {
				this._teamAreaIconClass = "itemIcon hideIcon"; //$NON-NLS-1$
			}
		},
		
		_setProcessAreaIcon: function(isArchived){
			dojo.removeClass(this.teamAreaIcon, "hideIcon"); //$NON-NLS-1$
			if (!isArchived) {
				this.teamAreaIcon.src = dojo.moduleUrl("com.ibm.team.process.web", "ui/internal/admin/projects/images/team-area_obj32.png"); //$NON-NLS-1$ //$NON-NLS-2$
			} else {
				this.teamAreaIcon.src = dojo.moduleUrl("com.ibm.team.process.web", "ui/internal/admin/projects/images/arch-team-area_obj32.png"); //$NON-NLS-1$ //$NON-NLS-2$
			}
		},
		
		_createTeamArea: function() {
			if (!this._handleNavigate()) {
				document.location.href = ProcessConstants.TeamAreaEditAction.ACTIONPREFIX_ID_ITEMIDSUFFIX + "new&parentItemId=" + this.itemId + "&projectAreaItemId=" + this.projectAreaItemId + "&parentType=TEAM"; //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
			}
		},
		
		/**
		 * override and add custom changes pending here.
		 */
		customChangesPending: function(){
			if (!this.isNew && this.developmentLineSelector) {
				// "!this.isNew" can make sure this will NOT affect the team area creation process.
				return this.originalTimelineItemId !== this.developmentLineSelector.value;
			}
		},
		
		/**
		 * Clear the state of timeline editor.
		 */
		_clearTimelineEditorState: function() {
			this.originalTimelineItemId = null;
			this.developmentLineSelector = undefined;
		},
		
		customRefresh: function(refreshClicked){
			dojox.xml.parser.removeChildren(this.developmentLineDiv);
			this.developmentLineDiv.appendChild(document.createTextNode(this.messages.TeamAreaEditor_3));
			
			if (this.isNew) {
				this.name.value = this.messages.TeamAreaEditor_8;
				this.populateDevelopmentLineSelector();
				this._refreshProcessAreaHierarchy();
				
				setTimeout(dojo.hitch(this.name, function(){
					try {
						this.select();
					} catch (error) {
						// If the text can't be selected...ignore the error
					}
				}), 250);
			} else {
   			    this.startLoading(this.messages.TeamAreaEditor_40);
				dojox.xml.parser.removeChildren(this.teamAreaHierarchyContent);
				this.teamAreaHierarchyContent.appendChild(document.createTextNode(this.messages.TeamAreaEditor_2));
				srh = new ServiceResponseHandler(this, "populateProcessAreaData", "handleFetchError"); //$NON-NLS-1$ //$NON-NLS-2$
				ProcessClient.getTeamAreaByUUIDWithLimitedMembers({
					processAreaItemId: this.itemId,
					maxMembers: this.fetchMaxMembers
				}, srh);
			}
			
			if (refreshClicked) { //need to double check  
				for (tabId in this._tabs) {
					var tab = this._tabs[tabId];
					if (this.isActivatedExtensionTab(tab)) {
						tab.mainPage.refresh();
					}
				}
			}
		},
		
		_refreshProcessAreaHierarchy: function() {
			this._destroyHierarchyToolbars();
			dojox.xml.parser.removeChildren(this.teamAreaHierarchyContent);
			this.teamAreaHierarchyContent.appendChild(document.createTextNode(this.messages.TeamAreaEditor_2));
				var srh = new ServiceResponseHandler(this, "populateProcessHierarchy", "handleTeamAreaHierarchyFetchError"); //$NON-NLS-1$ //$NON-NLS-2$
				ProcessClient.getProjectHierarchy({
					uuid: this.projectAreaItemId
				}, srh);
		},
		
		clearState: function(refreshClicked){
			this.inherited(arguments);
			this.selectedMembers = {};
			this.selectedAdmins = {};
			this.parentProcessArea = null;
			dojox.xml.parser.removeChildren(this.teamAreaHierarchyContent);
			dojox.xml.parser.removeChildren(this.developmentLineDiv);
			this._clearTimelineEditorState();
		},
		
		getFetchErrorMessage: function(error){
			if (error.status == ProcessConstants.SC_FORBIDDEN) {
				return this.messages.TeamAreaEditor_ErrorNoReadAccess;
			} else {
				return this.messages.TeamAreaEditor_7;
			}
		},
		
		getPrimaryFieldLabel: function(){
			return this.messages.TeamAreaEditor_8;
		},
		
		populateProcessAreaData: function(data){
			var args = arguments;
			UIHelper.runSafe(dojo.hitch(this, function(){
				this.inherited(args);
				
				this.projectAreaName = data.processHierarchy.name;
				
				var teamArray = this.getAncestorsAsArray(data.processHierarchy);
				teamArray.shift();
				this.teamAreaPath = dojo.map(teamArray, "return '/'.concat(item.name)").join(ProcessConstants.EMPTY); //$NON-NLS-1$
				this.populateProcessHierarchy(data.processHierarchy);
				this.populateDevelopmentLine(data.developmentLine);
				
				this.navigateToInitTab();
			}));
			this.stopLoading();
			
			// refresh other tabs now that stopLoading has been called, so that
			// any tabs that need to put up new loading messages can (if they
			// are the active tab)
			
			// TODO I don't like this....need to force a refresh after
			//we are sure that the new data has been async loaded
			if (this.permissionEditor) {
				this.permissionEditor.refresh();
			}
			if (this.operationBehaviorEditor) {
				this.operationBehaviorEditor.refresh();
			}
			if (this.historyTab) {
				this.historyTab.refresh();
			}
		},
		
		createInvitationPropertiesString: function(user, repoLink){
			return dojo.string.substitute("teamRepository=${0}\nuserId=${1}\nuserName=${2}\nprojectAreaName=${3}\nteamAreaPath=${4}", [repoLink, user.userId, user.name, this.projectAreaName, this.teamAreaPath]); //$NON-NLS-1$
		},
		
		getInvitationLink: function(host){
			return dojo.string.substitute("${0}/web/projects/${1}", [host, this.projectAreaName]); //$NON-NLS-1$;
		},
		
		customTeamInvitationParams: function(params){
			params.teamAreaUuid = this.itemId;
		},
		
		populateDevelopmentLine: function(data){
			dojox.xml.parser.removeChildren(this.developmentLineDiv);
			if (this.parentProcessArea.type === "projectArea") { //$NON-NLS-1$
				if (data) {
					this.originalTimelineItemId = data.itemId;
				}
				// Only show timeline selector for a root team area whose parent is a project area
				this.populateDevelopmentLineSelector();
			} else {
				if (data) {
					this.originalTimelineItemId = data.itemId;
					// Show timeline label for a team area whose parent is NOT a project area.
					var label = data.label;
					if (data.projectTimeline === true) {
						// We should use dojo.string.substitute to combine the strings.
						label = dojo.string.substitute(this.messages.TeamAreaEditor_38, [label, this.messages.Timeline_DecoratorText_ProjectTimeline]);
					}
					if (data.archived === true) {
						label = dojo.string.substitute(this.messages.TeamAreaEditor_38, [label, this.messages.Timeline_DecoratorText_Archived]);
					}
					this.developmentLineDiv.appendChild(document.createTextNode(label));
				} else {
					this.developmentLineDiv.appendChild(document.createTextNode(this.messages.TeamAreaEditor_17));
				}
			}
		},
		
		populateDevelopmentLineSelector: function(){
			dojox.xml.parser.removeChildren(this.developmentLineDiv);
			if (this.parentProcessArea) {
				if (this.parentProcessArea.type == "projectArea") { //$NON-NLS-1$
					var srh = new ServiceResponseHandler(this, "developmentLineSelectorCallback", "handleProjectDevLineQueryError"); //$NON-NLS-1$ //$NON-NLS-2$
					ProcessClient.getProjectDevelopmentLines({
						uuid: this.parentProcessArea.itemId,
						includeArchived: true
					}, srh);
				} else if (this.parentProcessArea.type == "teamArea") { //$NON-NLS-1$
					srh = new ServiceResponseHandler(this, "developmentLineDisplayCallback", "handleTeamDevLineQueryError"); //$NON-NLS-1$ //$NON-NLS-2$
					ProcessClient.getTeamAreaByUUIDWithLimitedMembers({
						processAreaItemId: this.parentProcessArea.itemId,
						maxMembers: this.fetchMaxMembers
					}, srh);
				}
			} else {
				this.developmentLineDiv.appendChild(document.createTextNode(this.messages.TeamAreaEditor_18));
			}
		},
		
		handleProjectDevLineQueryError: function(error){
			this.handleServerResponseError(error, this.messages.TeamAreaEditor_19);
		},
		
		handleTeamDevLineQueryError: function(error){
			this.handleServerResponseError(error, this.messages.TeamAreaEditor_20);
		},
		
		developmentLineSelectorCallback: function(data){
			dojox.xml.parser.removeChildren(this.developmentLineDiv);
			if (data && data.length > 0) {
				var select = document.createElement("select"); //$NON-NLS-1$
				select.setAttribute("title", this.messages.TeamAreaEditorHTML_12); //$NON-NLS-1$
				this.developmentLineSelector = select;
				for (var i = 0; i < data.length; i++) {
					var label = data[i].label;
					if (data[i].projectTimeline) {
						// We should also show "[Project Timeline]" to indicate which option is a project timeline.
						label = dojo.string.substitute(this.messages.TeamAreaEditor_38, [label, this.messages.Timeline_DecoratorText_ProjectTimeline]); 
					}
					if (data[i].archived === true) {
						label = dojo.string.substitute(this.messages.TeamAreaEditor_38, [label, this.messages.Timeline_DecoratorText_Archived]);
					}
					var option = new Option(label, data[i].itemId);
					if (data[i].summary && data[i].summary != ProcessConstants.EMPTY) {
						option.description = data[i].summary;
					} else if (data[i].description && data[i].description != ProcessConstants.EMPTY) {
						option.description = data[i].description;
					} else {
						option.description = this.messages.TeamAreaEditor_21;
					}
					if (dojo.isIE) {
						select.options[select.options.length] = option;
					} else {
						select.appendChild(option);
					}
					if (!this.isNew && this.originalTimelineItemId && this.originalTimelineItemId === data[i].itemId) {
						// Set current timeline as selected timeline in the selector by default.
						option.selected = "selected"; //$NON-NLS-1$
					}
				}
				
				if (!this.isNew) {
					if (!this.originalTimelineItemId) {
						// this team area was created without any timeline associated
						// we need to create a empty option in the timeline selector
						this.originalTimelineItemId = ProcessConstants.TIMELINE_SELECTOR_EMPTY_OPTION_ID;
						var emptyOption = new Option(this.messages.TeamAreaEditor_39, ProcessConstants.TIMELINE_SELECTOR_EMPTY_OPTION_ID);
						emptyOption.selected = "selected"; //$NON-NLS-1$
						if (dojo.isIE) {
							select.options[select.options.length] = emptyOption;
						} else {
							select.appendChild(emptyOption);
						}
					}
					
					// If this team area is NOT a new team area, add event handler for timeline selector
					this.connect(this.developmentLineSelector, "onchange", function(e){ //$NON-NLS-1$
						// If any changes happen, just update the dirty status of the team area editor.
						this.setDirty(this.changesPending());
						// Update the current selected development line in the sharedstore.
						this._getProcessAreaStore().setCurrentDevelopmentLineItemId(this.developmentLineSelector.value);
					});
				}
				
				this.developmentLineDiv.appendChild(select);
			} else {
				this.developmentLineDiv.appendChild(document.createTextNode(this.messages.TeamAreaEditor_22));
			}
		},
		
		developmentLineDisplayCallback: function(data){
			this.populateDevelopmentLine(data.developmentLine);
		},
		
		populateProcessHierarchy: function(project){
			dojox.xml.parser.removeChildren(this.teamAreaHierarchyContent);
			var showArchived = this._showArchivedButton.pressed;
			this.showBreadcrumbs(project);
            var processAreaData = {
				processAreaName: project.name,
				processAreaItemId: project.itemId,
				archived: project.archived,
				initialized: project.initialized,
				isProjectArea: true
			};
			var div = this._createTeamHierarchyElement(processAreaData, false, true);
			div.archived = project.archived;
			div.isRoot = true;
			var twistieIcon = dojo.query(".treeIconImg",div)[0]; //$NON-NLS-1$
			this.teamAreaHierarchyContent.appendChild(div);
			var isNewParent = false;
			if (!this.parentItemId) {
				// If this is not a new team area, we are unable to get parentItemId from the url.
				// In this case, we have to get parentItemId in team area hierarchy.
				this.parentItemId = this._getParentItemIdFromTeamHierarchy(this.itemId, project);
			}
			if (project.itemId === this.parentItemId) {
				// We need parentProcessArea regardless of if this team area is a new team area or not.
				this.parentProcessArea = {
					name: project.name,
					itemId: project.itemId,
					type: "projectArea" //$NON-NLS-1$
				};
				if (this.isNew) {
					isNewParent = true;
				}
			}
			if (project.children && project.children.length > 0) {
				div.isExpanded = true;
				twistieIcon.src = this._expandedIconSrc;
				this._toggleHierarchyIconVisibility(twistieIcon, true);
			    this.populateRemainingHierarchy(project.children, HIERARCHY_TAB_SPACING, isNewParent, div);
				//Manually collapses everything but first-level nodes
				dojo.forEach(div.childAreas, function(child){
					if (child.childAreas.length > 0) {
						var childIcon = dojo.query(".treeIconImg",child)[0]; //$NON-NLS-1$
						// On visible nodes, if all children are archived and showArchived isn't on, remove the twistie
						if (!showArchived && !child.archived && dojo.every(child.childAreas, function(item) {
							return item.archived;})) {
							this._toggleHierarchyIconVisibility(childIcon, false);
						} else {
							child.isExpanded = false;
							childIcon.src = this._collapsedIconSrc;
							this._toggleHierarchyIconVisibility(childIcon, true);
						}
					}
					dojo.forEach(child.childAreas,function(subChild){
						dojo.addClass(subChild, "hidden"); //$NON-NLS-1$
					},this);
				},this);
			} else {
				this._toggleHierarchyIconVisibility(twistieIcon, false);
				if (isNewParent) {
					this.populateRemainingHierarchy([], 1, true, div);
				}
			}
			this._updateHierarchyRoot();
			var activeArea = dojo.query('span.processAreaLink',div)[0].parentNode.parentNode; //$NON-NLS-1$
			this._displayActiveTeamArea(activeArea);
			if (this.isNew) {
				this.populateDevelopmentLineSelector();
			}	
		},
		
		_displayActiveTeamArea: function(element) {
			var twistieIcon = dojo.query(".treeIconImg",element)[0]; //$NON-NLS-1$
			// Collapse child areas of active team area
			dojo.forEach(element.childAreas, function(child){
				dojo.addClass(child,"hidden"); //$NON-NLS-1$
				element.isExpanded = false;
				twistieIcon.src = this._collapsedIconSrc;
				this._toggleHierarchyIconVisibility(twistieIcon, true);
			},this);
			var parent = element.parentNode;
			dojo.forEach(parent.childAreas, function(sibling){
				var siblingIcon = dojo.query(".treeIconImg",sibling)[0]; //$NON-NLS-1$
				dojo.forEach(sibling.childAreas, function(cousin){
					dojo.addClass(cousin,"hidden"); //$NON-NLS-1$
					sibling.isExpanded = false;
					siblingIcon.src = this._collapsedIconSrc;
					this._toggleHierarchyIconVisibility(siblingIcon, true);
				},this);
			},this);
			this._expandAllParents(element);
			this._updateHierarchyRoot();
		},
		
		_expandAllParents: function(element) {
			if (!element.isRoot) {
				this._revealAllSiblings(element);
				var parent = element.parentNode;
				var parentIcon = dojo.query(".treeIconImg",parent)[0]; //$NON-NLS-1$
				dojo.removeClass(element,"hidden"); //$NON-NLS-1$
				if (parent.childAreas && parent.childAreas.length > 0) {
					parent.isExpanded = true;
					parentIcon.src = this._expandedIconSrc;
					this._toggleHierarchyIconVisibility(parentIcon, true);
				}
				this._expandAllParents(parent);
			}
		},
		
		_revealAllSiblings: function(element) {
			var parent = element.parentNode;
			dojo.forEach(parent.childAreas, function(child){
				dojo.removeClass(child,"hidden"); //$NON-NLS-1$
			},this);
		},
		
		/**
		 * Get the parent item uuid from the team hierarchy.
		 * 
		 * @param {Object} itemId
		 * @param {Object} teamHierarchy
		 */
		_getParentItemIdFromTeamHierarchy: function(itemId, teamHierarchy) {
			if (itemId !== teamHierarchy.itemId) {
				if (teamHierarchy.children && teamHierarchy.children.length > 0) {
					for (var i = 0; i < teamHierarchy.children.length; i++) {
		                if (itemId === teamHierarchy.children[i].itemId) {
							return teamHierarchy.itemId;
						} else {
							var foundItemId = this._getParentItemIdFromTeamHierarchy(itemId, teamHierarchy.children[i]);
							if (foundItemId) {
								return foundItemId;
							}
						}
					}
				}
			}
		},
		
		populateRemainingHierarchy: function(teams, indent, includeNewTeam, parentProcessArea){
			teams = teams.sort(function(a, b){
				return ProcessClient.stringCompare(a.name, b.name);
			});
			dojo.forEach(teams, dojo.hitch(this, function(team){
                if (this.itemId === team.itemId) {
					this._isArchived = team.archived;
					this.updateProcessHierarchyButtons(team.archived);
					// update this editor's icon in case its team just changed archived status
					this._setProcessAreaIcon(team.archived);
				}
				var processAreaData = {
					processAreaName: team.name,
					processAreaItemId: team.itemId,
					parentProcessArea: parentProcessArea,
					archived: team.archived,
					isProjectArea: false
				};
				var div = this._createTeamHierarchyElement(processAreaData, !parentProcessArea.archived, this.itemId !== team.itemId);
				if (!this.isNew && this.itemId === team.itemId) {
					this.hierarchyNameDisplay = div;
				}
				div.archived = team.archived;
				parentProcessArea.appendChild(div);
				parentProcessArea.childAreas.push(div);
				var twistieIcon = dojo.query(".treeIconImg",div)[0]; //$NON-NLS-1$
				if (!this.isLeftToRight()) {
	            	div.firstChild.style.paddingRight = indent + "em"; //$NON-NLS-1$ //Indentation for sub-hierarchy
	            } else {
	           		div.firstChild.style.paddingLeft = indent + "em"; //$NON-NLS-1$ //Indentation for sub-hierarchy
	           	}
				var isNewParent = false;
				if (team.itemId === this.parentItemId) {
					// We need parentProcessArea regardless of if this team area is a new team area or not.
					this.parentProcessArea = {
						name: team.name,
						itemId: team.itemId,
						type: "teamArea" //$NON-NLS-1$
					};
					if (this.isNew) {
						isNewParent = true;
					}
				}
				if (team.children && team.children.length > 0) {
					this.populateRemainingHierarchy(team.children, indent + HIERARCHY_TAB_SPACING, isNewParent, div);
					div.isExpanded = true;
					twistieIcon.src = this._expandedIconSrc;
					this._toggleHierarchyIconVisibility(twistieIcon, true);
				    } else {
						this._toggleHierarchyIconVisibility(twistieIcon, false);
					if (isNewParent) {
						processAreaData.archived = false;
						if (team.children) {
							this.populateRemainingHierarchy(team.children, indent + HIERARCHY_TAB_SPACING, true, div);
						} else {
							this.populateRemainingHierarchy([], indent + HIERARCHY_TAB_SPACING, true, div);
						}
						div.isExpanded = true;
						twistieIcon.src = this._expandedIconSrc;
						this._toggleHierarchyIconVisibility(twistieIcon, true);
					}
				}
			}));
			if (includeNewTeam) {
                var processAreaData = {
					processAreaName: this.messages.TeamAreaEditor_23,
					processAreaItemId: this.messages.TeamAreaEditor_23,
					parentProcessArea: parentProcessArea,
					archived: false,
					isProjectArea: false
				};
				var div = this._createTeamHierarchyElement(processAreaData, false, false);
				this.hierarchyNameDisplay = div;
				parentProcessArea.appendChild(div);
				parentProcessArea.childAreas.push(div);
	            if (!this.isLeftToRight()) {
	            	div.firstChild.style.paddingRight = indent + "em"; //$NON-NLS-1$ //Indentation for sub-hierarchy
	            } else {
	           		div.firstChild.style.paddingLeft = indent + "em"; //$NON-NLS-1$ //Indentation for sub-hierarchy
	           	}
			}
		},

		// returns an array representing the process hierarchy directly above this team area
		getAncestorsAsArray: function(processArea){
			var foundParentOfNewTeam = this.isNew && this.parentItemId === processArea.itemId;
			var foundThisTeam = !this.isNew && this.itemId === processArea.itemId;
			var hasChildren = processArea.children && processArea.children.length > 0;
			
			// Return an array containing this process area so it can bubble up recursively
			// The recursive callers will insert their process area at the front of this array so
			// we get a complete ancestry array in the end 
			if (foundParentOfNewTeam || foundThisTeam) {
				return [processArea];
			} else if (hasChildren) {
				for (var i = 0; i < processArea.children.length; i++) {
					var curLine = this.getAncestorsAsArray(processArea.children[i]);
					// If the current child team exists along the ancestry path, we know this element
					// also belongs along the ancestry path.  Add the current processArea to the front
					// of the ancestry path and bubble up to the next recursive caller.
					if (curLine) {
						curLine.unshift(processArea);
						return curLine;
					}
				}
			} else {
				// We didn't find what we were looking for down this branch.
				// Return null so the recursive callers will ignore this path.
				return null;
			}
		},
		
		customAddBreadcrumbSegments: function(breadcrumbSegments, project){
			var hierarchy = this.getAncestorsAsArray(project);
			for (var i = 0; i < hierarchy.length; i++) {
				// Don't include the current element in the breadcrumb hierarchy
				if (!this.isNew && i === hierarchy.length - 1) {
					continue;
				}
				var segment = {};
				if (i === 0) {
					segment.href = ProcessConstants.ProjectAreaEditAction.ACTIONPREFIX_ID_ITEMIDSUFFIX + hierarchy[i].itemId;
				} else {
					segment.href = ProcessConstants.TeamAreaEditAction.ACTIONPREFIX_ID_ITEMIDSUFFIX + hierarchy[i].itemId + "&projectAreaItemId=" + this.projectAreaItemId; //$NON-NLS-1$
				}
				segment.label = dojox.string.BidiComplex.createDisplayString(hierarchy[i].name);
				breadcrumbSegments.push(segment);
			}
		},
		
		doSave: function(saveObj, serviceResponseHandler){
			ProcessClient.postTeamArea(saveObj, serviceResponseHandler);
			this._displayPendingMessage(this.messages.TeamAreaEditor_26);
		},
		
		configureSaveObject: function(){
			var saveObj = {
				itemId: this.itemId
			};
			if (!this.isNew) {
				saveObj.stateId = this.stateId;
			}
			for (var i = 0; i < this.inputFields.length; i++) {
				if (this.inputFields[i].changePending) {
					saveObj[this.inputFields[i].name] = this.inputFields[i].value;
				}
			}
			saveObj.jsonMembers = dojo.toJson(this.pendingMemberChanges);
			saveObj.jsonAdmins = dojo.toJson(this.pendingAdminChanges);
			if (this.isNew && this.parentProcessArea) {
				saveObj.jsonParentProcessArea = dojo.toJson(this.parentProcessArea);
			}
			
			if (this.developmentLineSelector && this.developmentLineSelector.value) {
				if (this.isNew || this.developmentLineSelector.value !== this.originalTimelineItemId) {
					// Collect timeline data if
					// 1. this is a new team area
					// OR
					// 2. this is not a new team area, but the timeline selection has been changed
					saveObj.developmentLineUuid = this.developmentLineSelector.value;
				}
			}
						
			this.generateChangeParameters(saveObj);
			this.generateRoleChanges(saveObj);
			
			return saveObj;
		},
		
		validateRequired: function(saveData){
			if (saveData.name == ProcessConstants.EMPTY || (this.isNew && !saveData.name)) {
				this.togglePrimaryField(ProcessConstants.EditorMode.EDIT);
				return this.messages.TeamAreaEditor_27;
			}
			
			if (saveData.name && saveData.name.indexOf('/') >= 0) { //$NON-NLS-1$
				this.togglePrimaryField(ProcessConstants.EditorMode.EDIT);
				return this.messages.TeamAreaEditor_35;
			}
			
			if (this.isNew && (!saveData.jsonParentProcessArea || saveData.jsonParentProcessArea == ProcessConstants.EMPTY)) {
				return this.messages.TeamAreaEditor_28;
			}
		},
		
		handleSaveResponse: function(response){
		
			if (this.discardPending) {
				this._deleteProcessDescription(response);
			}
			
			this._saveExtensionTabs();
			
			if ((this.pendingMemberChanges.add && this.pendingMemberChanges.add.length > 0) || (this.pendingAdminChanges.add && this.pendingAdminChanges.add.length > 0 )) {
				dojo.cookie(NEW_MEMBERS_COOKIE_NAME, dojo.toJson(this.pendingMemberChanges.add));
				dojo.cookie(NEW_ADMINISTRATORS_COOKIE_NAME, dojo.toJson(this.pendingAdminChanges.add));
			}
			
			if (this.isNew && response) {
				dojo.cookie(MSG_COOKIE_NAME, this.messages.TeamAreaEditor_29);
				this._unregisterHandlers();
				document.location.href = ProcessConstants.TeamAreaEditAction.ACTIONPREFIX_ID_ITEMIDSUFFIX + response + "&projectAreaItemId=" + this.projectAreaItemId; //$NON-NLS-1$
			} else {
				this.handleServerResponseSuccess(response);
				dojo.forEach(this._saveParticipants, function(saveParticipant){
					saveParticipant.saved(response);
				});
			}
		},
		
		checkForSuccessMsg: function() {
			var msgCookie = dojo.cookie(MSG_COOKIE_NAME);
			if (msgCookie) {
				this._displayLongSuccessMessage(msgCookie);
				var now = new Date();
				now.setTime(now.getTime() - 1);
				dojo.cookie(MSG_COOKIE_NAME, ProcessConstants.EMPTY, {
					expires: now
				});
			}
		},
		
		getSaveErrorMessage: function(err){
			return this.messages.TeamAreaEditor_31;
		},
		
		enableSaveButton: function(isDirty){
			if (isDirty && this._canSave()) {
				this.saveButton.disabled = false;
			} else {
				this.saveButton.disabled = true;
			}
		},
		
		_getProcessAreaStore: function(){
			if (!this.sharedStore) {
				var args = {
					editorContext: ProcessConstants.EditorContext.TEAM,
					processAreaItemId: this.itemId,
					currentDevelopmentLineItemId: this.getSelectedDevelopmentLineItemId(),
					errorContext: this,
					errorCallback: this._handleCreateStoreError
				};
				this.sharedStore = new com.ibm.team.process.web.ui.internal.admin.projects.PermissionsScopeStore(args);
			}
			return this.sharedStore;
		},
		
		createCustomTabs: function() {
			this._loadRegistryTabs();
			this.createHistoryTab();
		},
		
		_createForCurrentEditor: function(createForTeamAreaEditor){
			return createForTeamAreaEditor === "true";  //$NON-NLS-1$
		},
		
		getDeferredSaveCount: function(response){
			if (!this.isNew || response) {
				var count = 1;
				if (this._tabs) {
					for (tabId in this._tabs) {
						var tab = this._tabs[tabId];
						if (this.isActivatedExtensionTab(tab)) {
							if (tab.mainPage.changesPending()) {
								count++;
							}
						}
					}
				}
				if (this.discardPending) {
					count++; //delete of the process description
				}
				return count;
			} else {
				return 1;
			}
		},

		getTabUAKey: function(tabKey) {
			if (tabKey === ProcessConstants.TabEnum.OVERVIEW) {
				return ProcessConstants.UAKey.OVERVIEW_TEAMAREA_TAB;
			} else if (tabKey === ProcessConstants.TabEnum.ROLES) {
				return ProcessConstants.UAKey.ROLES_TEAMAREA_TAB;
			} else if (tabKey === ProcessConstants.TabEnum.PERMISSIONS) {
				return ProcessConstants.UAKey.PERMISSIONS_TEAMAREA_TAB;
			} else if (tabKey === ProcessConstants.TabEnum.OPERATION_BEHAVIOR) {
				return ProcessConstants.UAKey.OPERATION_BEHAVIOR_TEAMAREA_TAB;
			} else if (tabKey === ProcessConstants.TabEnum.HISTORY) {
				return ProcessConstants.UAKey.HISTORY_TEAMAREA_TAB;
			}
			return undefined;
		},
		
		getRegistryTabUAKey: function(extensionElement) {
			var uaKey = extensionElement.getAttribute(ProcessConstants.ProjectAreaExtensionPage.ATTRIBUTE_UA_KEY_TEAM_AREA);
			return uaKey;
		},
		
		/**
		 * Get the currently selected development line. 
		 * If current team area is not a root team area,
		 * the original development line will be returned.
		 * 
		 * The returned value might be undefined.
		 * 
		 * @return the currently selected development line
		 */
		getSelectedDevelopmentLineItemId: function() {
			if (this.developmentLineSelector) {
				return this.developmentLineSelector.value;
			} else {
				return this.originalTimelineItemId;
			}
		}
		
	});
	
})();
