/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2012, 2013. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  
 * Use, duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp. 
 *******************************************************************************/
(function() {
dojo.provide("com.ibm.team.repository.web.ui.internal.admin.rename.ServerRenameWizardApplication");

dojo.require("com.ibm.team.repository.web.client.internal.AdminClient");
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
dojo.require("jazz.ui.Wizard");
dojo.require("jazz.ui.wizard.WizardStep");
dojo.require("com.ibm.team.repository.web.ui.internal.admin.NotAuthorizedView");
dojo.require("jazz.util.browserSupport");
dojo.require("jazz.app.auth");
dojo.require("jazz.ui.UnsupportedBrowserWarning");

dojo.require("com.ibm.team.repository.web.ui.internal.admin.rename.ServerRenameStatus");

dojo.require("dojo.i18n");
dojo.requireLocalization("com.ibm.team.repository.web.ui.internal.admin.rename", "ServerRenameStatusMessages");
dojo.requireLocalization("com.ibm.team.repository.web.ui.internal.wizard", "WizardApplicationMessages");

var AdminClient = com.ibm.team.repository.web.client.internal.AdminClient;
var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;
var Wizard = jazz.ui.Wizard;
var WizardStep = jazz.ui.wizard.WizardStep;
var NotAuthorizedView = com.ibm.team.repository.web.ui.internal.admin.NotAuthorizedView;

var StatusStep = com.ibm.team.repository.web.ui.internal.admin.rename.ServerRenameStatus;

dojo.declare("com.ibm.team.repository.web.ui.internal.admin.rename.ServerRenameWizardApplication", [ dijit._Widget ], {
	
	constructor: function() {
		this.messages = dojo.i18n.getLocalization("com.ibm.team.repository.web.ui.internal.admin.rename", "ServerRenameStatusMessages");
		this.wizardMessages = dojo.i18n.getLocalization("com.ibm.team.repository.web.ui.internal.wizard", "WizardApplicationMessages");
	},
	
	postCreate: function() {
		this.getUserRoles();
	},
	
	getUserRoles: function () {
		var srh= new ServiceResponseHandler(this, "authRequestPassed", "authRequestFailed");		
		AdminClient.getUserRoles(srh);
	},
	
	authRequestPassed: function(response) {
		if (response.userId) {
			this.loggedInUser = response.userId;
			dojo.setObject("com.ibm.team.repository.web.client.internal.AUTHENTICATED_USERID", response.userId);
		}
		var roles = response.roles;
		for (var i=0; i < roles.length; i++) {
			if (roles[i] === "JazzAdmins") {
				this.authorized = true;
				break;
			}
		}
		this._checkBrowser();
	},
	
	authRequestFailed: function() {
		this._checkBrowser();
	},
	
	_checkBrowser: function() {
		//Check to see if we are using a supported Browser
		var support = jazz.util.browserSupport.getSupportLevel();
		if (support && (support == jazz.util.browserSupport.NOT_SUPPORTED)) {
			var warn = new jazz.ui.UnsupportedBrowserWarning();
			this.connect(warn, "onContinue", function() {
				warn.destroy();
				this._createUI();
			});
			warn.placeAt(document.body);
		} else {
			this._createUI()
		}
	},
	
	createLogoutArea: function() {
		var logoutNode = dojo.create("div", { className: "logout-node" });
		this.loggedInUserNode = dojo.create("span", { innerHTML: this.loggedInUser, className: "userName" }, logoutNode);
		dojo.create("span", { innerHTML: "|", style: { color: "#999", padding: "0 4px" } }, logoutNode);
		this.logoutLink = dojo.create("a", { innerHTML: this.wizardMessages.logout, href: "#", className: "logoutLink" }, logoutNode);
		this.connect(this.logoutLink, "onclick", function(e) {
			dojo.stopEvent(e);
			var dfd = jazz.app.auth.logout();
			dfd.addCallback(function() {
				window.location.reload();
			});
		});
		return logoutNode;
	},
	
	_createUI: function() {
		document.body.style.background = "none";
		document.title = this.messages.wizardName;
		this.content = dojo.create("div", { className: "com-ibm-team-repository-wizard" });
		if (this.loggedInUser) {
			this.content.appendChild(this.createLogoutArea());
		}
		dojo.addClass(this.domNode, "com-ibm-team-repository-web-setup-RootNode");
		var setupTitle = this.messages.wizardName;
		dojo.create("div", { innerHTML: setupTitle.toUpperCase(), className: "setup-header"}, this.content);
		var jazzIcon = dojo.create("img", { className: "jazz-icon" }, this.content);
		jazzIcon.src = dojo.moduleUrl("com.ibm.team.repository.web", "ui/internal/wizard/images/jazz-identR.png");
		this.wizardBox = document.createElement("div");
		this.content.appendChild(this.wizardBox);		
		var wizardParams = {
			fullPageContainer: this.wizardBox,
			stepListLayout: "vertical",
			showStepSummary: true,
			processFinish:  function(){
				document.location.href = net.jazz.ajax.contextRoot() + "/admin";
            }
		};
		if (this.authorized) { 
			wizardParams.steps = this.initServerSetupSteps();
		} else {
			var notAuthStep = new NotAuthorizedView({
					applicationName: this.messages.wizardName,
					customNoUserMessage: this.wizardMessages.userNotRecognizable,
					includeBackLink: false,
					showLogout: true });
			dojo.mixin(notAuthStep, new WizardStep());
			notAuthStep.includeNavigationUI = false;
			wizardParams.stepListLayout = "horizontal";
			wizardParams.steps = [notAuthStep];
			wizardParams.showStepSummary = false;
		}
		this.wizard = new Wizard(wizardParams);
		var outerFooter = new com.ibm.team.repository.web.ui.internal.wizard.LogoFooter();
		this.content.appendChild(outerFooter.domNode);
		this.domNode.appendChild(this.content);
		this.wizard.startup();
	},
	
	initServerSetupSteps: function() {
		return steps = [ new StatusStep() ];
	}
	
});
var messages;
dojo.declare("com.ibm.team.repository.web.ui.internal.wizard.LogoFooter", [dijit._Widget, dijit._Templated], {
	
	templatePath: dojo.moduleUrl("com.ibm.team.repository.web", "ui/internal/wizard/templates/LogoFooter.html"),
	
	constructor: function() {
		this.messages = messages = dojo.i18n.getLocalization("com.ibm.team.repository.web.ui.internal.wizard", "WizardApplicationMessages");
	}
});

})();