/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010, 2013. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  
 * Use, duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp. 
 *******************************************************************************/
(function() {
dojo.provide("com.ibm.team.repository.web.ui.internal.admin.server.ActiveServicesView");

dojo.require("dijit._Widget");
dojo.require("jazz.ui.StyledTable");
dojo.require("jazz.ui.Dialog");
dojo.require("jazz.ui.MessageBox");
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
dojo.require("com.ibm.team.repository.web.client.internal.ServerStatusClient");
dojo.require("com.ibm.team.repository.web.ui.internal.admin.util")
dojo.require("dojo.i18n");
dojo.require("jazz.app.i18n");
dojo.requireLocalization("com.ibm.team.repository.web.ui.internal.admin.server", "ActiveServicesViewMessages");

var MessageBox = jazz.ui.MessageBox;
var ServiceResponseHandler	= com.ibm.team.repository.web.transport.ServiceResponseHandler;
var ServerStatusClient = com.ibm.team.repository.web.client.internal.ServerStatusClient;
var formatTimeFromMilliseconds = com.ibm.team.repository.web.ui.internal.admin.util.formatTimeFromMilliseconds;
var replaceNewLineWithPaddedBreaks = com.ibm.team.repository.web.ui.internal.admin.util.replaceNewLineWithPaddedBreaks;
var MAX_SERVICE_NAME_LENGTH = 100;
var LOCAL_STORAGE_KEY = "com-ibm-team-repository-web-ActiveServicesView";

dojo.declare("com.ibm.team.repository.web.ui.internal.admin.server.ActiveServicesView", [dijit._Widget, dijit._Templated], {

	templatePath: dojo.moduleUrl("com.ibm.team.repository.web", "ui/internal/admin/server/templates/ActiveServicesView.html"),
	
	constructor: function(args) {
		this.messages = dojo.i18n.getLocalization("com.ibm.team.repository.web.ui.internal.admin.server", "ActiveServicesViewMessages");
		if(args.nodeId) {
			this.nodeId = args.nodeId;
		}
	},
	
	postCreate: function() {
		this.header = new MessageBox();
		this.headerArea.appendChild(this.header.domNode);
		this.checkClusterState();
	},
	
	checkClusterState: function() {
		var state = dojo.getObject("com.ibm.team.repository.isInCluster");
		if (state === true) {
			this.isCluster = true;
			this.getClusterInformation();
		} else if (state === false) {
			this.isCluster = false;
			this.refresh();
		} else { //not set, usually happens when user refreshes this page directly
			this._getClusterStatus();
		}
	},
	
	_getClusterStatus: function() {
		var handler = {
			success: dojo.hitch(this, function(response) {
				dojo.setObject("com.ibm.team.repository.isInCluster", response === "false" ? false : true);
				this.checkClusterState();
			}),
			error: dojo.hitch(this, function() {
				dojo.setObject("com.ibm.team.repository.isInCluster", false);
				console.error("Unable to determine if the server is running in a cluster. Defaulting to standalone mode.");
				this.checkClusterState();	
			})
		};
		var srh = new ServiceResponseHandler(handler, "success", "error");
		ServerStatusClient.getIsInCluster(srh);
	},
	
	getClusterInformation: function() {
		if(this.nodeId) {
			//We are showing results that are scoped to a node, so we don't need to get a list of other nodes
			this.refresh();	
			return;
		}
		var srh = new ServiceResponseHandler(this, "handleClusterInfo", "handleClusterError");
        ServerStatusClient.getClusterMembers(srh);
	},
		
	handleClusterInfo: function(nodelist) {
		this.clusterNodes = nodelist || [];
		this.refresh();
	},
	
	handleClusterError: function(error) {
		console.error("Unable to get Cluster status. Assuming no cluster.");
		this.refresh();
	},

	refresh: function () {
		if(!this.domNode) return;
		this.header.hideMessage();
		this.buildTable();
		
		//Keeps a list of all the services fetched from the node(s) and uses this for sorting
		this.servicesCache = [];
		//Keep track of the node IDs that we fail to properly get services for
		this.nodesWithErrors = []; 
		
		if(this.nodeId) {
			//Show Info Message about scoped results
			var span = dojo.create("span", { innerHTML: dojo.string.substitute(this.messages.scopedResults, [this.nodeId]) });
			dojo.create("a", { style: { padding: "0 4px"}, href: "#action=com.ibm.team.repository.admin.activeServices", innerHTML: this.messages.showAll }, span);
			this.header.showMessage({
				summary: span,
				type: "INFO"
			});
			this._getServices(this.nodeId);
		}
		else if(this.isCluster) {
			this.pendingCalls = this.clusterNodes.length;
			dojo.forEach(this.clusterNodes, function(node) {
				this._getServices(node.humanReadableNodeId);
			}, this);
		}
		else {
			this._getServices();
		}
	},
	
	buildTable: function() {
		var columns = [
			{
				label: this.messages.serviceName,
				sortable: true
			}, {
				width: "15%",
				label: this.messages.runningTime,
				sortable: true
			}
		];
		if(this.isCluster || this.nodeId) {
			columns.push({
				width: "15%",
				label: this.messages.nodeId,
				sortable: true
			});	
		}
		columns.push({
			width: "15%",
			label: this.messages.requestedby,
			sortable: true
		},{
			width: "15%",
			label: this.messages.moreInfo,
			sortable: false
		});
		
		//Check for saved sort preferences
		var sortPrefs = dojo.fromJson(localStorage[LOCAL_STORAGE_KEY]);
		if(sortPrefs) {
			var sortBy = sortPrefs.sortBy;
			var asc = sortPrefs.asc;
		}
		
		this.styledTable = new jazz.ui.StyledTable({
			columns: columns,
			sortBy: (sortBy != null) ? sortBy : 1,
			sortByAscending: (asc != null) ? asc : true,
			handleSort: dojo.hitch(this, "handleSort")
		});
		this.contentArea.appendChild(this.styledTable.domNode);
		dojo.addClass(this.styledTable.domNode, "fixedTable");
	},
	
	_getServices: function(nodeId) {
		var handler = {
			success: dojo.hitch(this, function(response) {
				this._displayServices(response, nodeId);
			}),
			error: dojo.hitch(this, function(errorObject) {
				if(this.pendingCalls) {
					this.pendingCalls--;	
				}
				if(nodeId) {
					this.nodesWithErrors.push(nodeId);
					console.error("Error loading Active Services for node: " + nodeId);
				} else {
					this.serviceError(errorObject);
				}
				this.handlePendingCallsUpdate();
			})
		};
		var srh = new ServiceResponseHandler(handler, "success", "error");
		ServerStatusClient.getActiveServiceInfo(srh, nodeId);
	},

	_displayServices: function(response, nodeId) {
		if(!this.domNode || !response) return;
		
		if(this.pendingCalls) {
			this.pendingCalls--;	
		}
		
		dojo.forEach(response.services, function(service) {
			service.nodeId = nodeId;
			service.currentTime = response.currentTime;
			this._addRow(service);
			this.servicesCache.push(service);
		}, this);
		
		//Re-Sort the table after adding each node's services
		this.handleSort();
		this.handlePendingCallsUpdate();
	},
	
	handlePendingCallsUpdate: function() {
		//If all calls have returned, update loading message
		if(this.pendingCalls === 0) {
			if(this.nodesWithErrors.length > 0) {
				var clusterNodesLink = "<a href='#action=com.ibm.team.repository.admin.clusterNodes'>";
				this.header.showMessage({
					summary: dojo.string.substitute(this.messages.showSomeNodeResults,
						[ this.clusterNodes.length - this.nodesWithErrors.length, this.clusterNodes.length, clusterNodesLink, "</a>" ]),
					type: "WARNING"
				});
			} else {
				this.header.showMessage({
					summary: this.messages.showingAllResults,
					type: "OK",
					closable: true
				});
			}
			this.showTimestamp();
		} else if(this.pendingCalls) {
			//Update Loading Message
			this.header.showMessage({
				summary: dojo.string.substitute(this.messages.loadingProgress, [ this.clusterNodes.length - this.pendingCalls, this.clusterNodes.length]),
				type: "INFO"
			});
		} else {
			this.showTimestamp();	
		}
	},
	
	_addRow: function(service) {
		var row = dojo.create("tr");
		var name = service.service + "." + service.method;
		dojo.create("td", { innerHTML: this.trimName(name), title: name }, row);
		var elapsedTime = service.currentTime - service.startTime;
		elapsedTime = formatTimeFromMilliseconds(elapsedTime);
		dojo.create("td", { innerHTML: elapsedTime }, row);
		if(service.nodeId) {
			dojo.create("td", { innerHTML: service.nodeId || "" }, row);
		}
		var userTd = dojo.create("td", {}, row);
		if(service.itemId && service.userName) {
			var userLink = "#action=com.ibm.team.repository.editUser&itemId=" + service.itemId;
			dojo.create("a", { innerHTML: service.userName, href: userLink }, userTd);
		} else if(service.userName) {
			dojo.create("span", { innerHTML: service.userName }, userTd);
		}
		var detailsTd = dojo.create("td", {}, row);
		var aLink = dojo.create("a", { innerHTML: this.messages.showDetails, href: "#" }, detailsTd);
		this.connect(aLink, "onclick", dojo.hitch(this, function(e) {
			dojo.stopEvent(e);
			if(row._detailsRow) {
				dojo.toggleClass(row._detailsRow, "noDisplay");
				dojo.toggleClass(row, "expanded");
				aLink.innerHTML = aLink.innerHTML == this.messages.showDetails ? this.messages.hideDetails : this.messages.showDetails;
			}
		}));
		this.styledTable.tbody.appendChild(row);
		
		//Add hidden stacktrace row
		var detailsRow = dojo.create("tr", { className: "detailsRow noDisplay" });
		row._detailsRow = detailsRow;
		var stackTd = dojo.create("td", { colSpan: service.nodeId ? 5 : 4 }, detailsRow);
		dojo.create("div", { className: "service-trace", innerHTML: replaceNewLineWithPaddedBreaks(service.stacktrace, true) }, stackTd);
		this.styledTable.tbody.appendChild(detailsRow);
	},
	
	trimName: function(name) {
		if(name.length > MAX_SERVICE_NAME_LENGTH) {
			return name.substring(0, MAX_SERVICE_NAME_LENGTH) + "...";	
		} else {
			return name;
		}
	},
	
	handleSort: function(args) {
		var index = this.styledTable.sortBy;
		
		//Save the users' sort preferences
		try {
			localStorage[LOCAL_STORAGE_KEY] = dojo.toJson({
				sortBy: index,
				asc: this.styledTable.sortByAscending
			});
		} catch (e) {} //not a critical op
		
		//Determine which column to sort by
		if(index === 0) {
			var services = this.sortServiceName();	
		} else if(index === 1) {
			var services = this.sortRunningTime();
		} else if(index === 2 && (this.isCluster || this.nodeId)) {
			var services = this.sortNodeId();
		} else if(index >= 2) {
			var services = this.sortRequesters();
		}
		
		this.styledTable.clearBody();
		dojo.forEach(services, function(service) {
			this._addRow(service);
		}, this);
	},
	
	sortServiceName: function () {	
		return this.servicesCache.sort(dojo.hitch(this,function(a, b){
			var aName = a.service + "." + a.method;
			var bName = b.service + "." + b.method;
			var result = aName.localeCompare(bName);
			if (!this.styledTable.sortByAscending) {
				result = result * (-1);
			}
			return result;
		}));
	},
	
	sortRequesters: function () {	
		return this.servicesCache.sort(dojo.hitch(this,function(a, b){
			var result;
			if (!a || !a.userName) {
				result = 1;
			} else if (!b || !b.userName) {
				result = -1;
			} else {
				result = a.userName.localeCompare(b.userName);
			}
			if (!this.styledTable.sortByAscending) {
				result = result * (-1);
			}
			return result;
		}));
	},
	
	sortNodeId: function () {	
		return this.servicesCache.sort(dojo.hitch(this,function(a, b){
			var result = a.nodeId.localeCompare(b.nodeId);
			if (!this.styledTable.sortByAscending) {
				result = result * (-1);
			}
			return result;
		}));
	},
	
	sortRunningTime: function () {	
		return this.servicesCache.sort(dojo.hitch(this,function(a, b){
			var aTime = a.currentTime - a.startTime;
			aTime = formatTimeFromMilliseconds(aTime);
			var bTime = b.currentTime - b.startTime;
			bTime = formatTimeFromMilliseconds(bTime);
			var result = aTime.localeCompare(bTime);
			if (!this.styledTable.sortByAscending) {
				result = result * (-1);
			}
			return result;
		}));
	},
	
	showTimestamp: function() {
		var nowTime = jazz.app.i18n.date.locale.format(new Date());
		var timestamp = dojo.string.substitute(this.messages.loadedTimestamp, [nowTime]);
		dojo.place(document.createTextNode(timestamp), this.loadedTimestampArea, "only");
		dojo.toggleClass(this.loadedTimestampArea, "noDisplay", false);
	},
	
	serviceError: function(errorObj) {
		this.header.showMessage({
			errorObject: errorObj
		});
	},
	
	uninitialize: function () {
		this.header.destroyRecursive();
		if(this.styledTable) {
			this.styledTable.destroyRecursive();
		}
		this.inherited(arguments);
    }
});

})();