/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2011, 2012. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  
 * Use, duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp. 
 *******************************************************************************/
(function(){
	
    dojo.provide("com.ibm.team.repository.web.ui.internal.admin.server.ClusterNodes");
	
    dojo.require("dijit._Widget");
    dojo.require("dijit._Templated");
    dojo.require("dojo.i18n");
	dojo.require("jazz.util.date");
    dojo.require("jazz.ui.MessageBox");
	dojo.require("jazz.app.ua.TextHoverHelp");
	dojo.require("jazz.ui.StyledTable");
    dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
	dojo.require("com.ibm.team.repository.web.ui.internal.admin.util");
    dojo.require("com.ibm.team.repository.web.client.internal.ServerStatusClient");
    dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
    dojo.require("com.ibm.team.repository.web.ui.internal.admin.server.DiagnosticsUtil");
	
    dojo.requireLocalization("com.ibm.team.repository.web.ui.internal.admin.server", "ClusterNodesMessages");
    
	var TextHoverHelp = jazz.app.ua.TextHoverHelp;
    var MessageBox = jazz.ui.MessageBox;
    var STATUS_TYPES = MessageBox.STATUS_TYPES;
    var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;
    var ServerStatusClient = com.ibm.team.repository.web.client.internal.ServerStatusClient;
	var util = com.ibm.team.repository.web.ui.internal.admin.util;
    var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;
	var DiagnosticsUtil = com.ibm.team.repository.web.ui.internal.admin.server.DiagnosticsUtil;
	var replaceNewLineWithPaddedBreaks = com.ibm.team.repository.web.ui.internal.admin.util.replaceNewLineWithPaddedBreaks;
	var STATUS_OK_IMG_SRC = dojo.moduleUrl("com.ibm.team.repository.web", "graphics/icons/okay_obj.gif");
	var STATUS_ERR_IMG_SRC = dojo.moduleUrl("com.ibm.team.repository.web", "graphics/icons/error_obj.gif");
	var STATUS_WARN_IMG_SRC = dojo.moduleUrl("com.ibm.team.repository.web", "graphics/icons/warn_obj.gif");
	var LOCAL_STORAGE_KEY = "com-ibm-team-repository-web-ClusterNodes";
    
    dojo.declare("com.ibm.team.repository.web.ui.internal.admin.server.ClusterNodes", [dijit._Widget, dijit._Templated], {
    
        templatePath: dojo.moduleUrl("com.ibm.team.repository.web", "ui/internal/admin/server/templates/ClusterNodes.html"),
        _textHelpHovers: [],
		
        constructor: function(){
            this.messages = dojo.i18n.getLocalization("com.ibm.team.repository.web.ui.internal.admin.server", "ClusterNodesMessages");
        },
        
        postCreate: function(){
            this.header = new MessageBox();
            this.headerArea.appendChild(this.header.domNode);
			this.diagUtil = new DiagnosticsUtil();
			this.clusterNodes = new Array();
			this.reload();
        },
        
        reload: function() {
			if(!this.domNode) return;
			this.header.hideMessage();
			this.buildTable();
			
			//Keeps a list of all the cluster nodes data and uses this for sorting
			this.nodesCache = [];

			this.getDiagnosticStatus();
        },
		
		buildTable: function() {
			var columns = [
				{
					label: this.messages.hNodeId,
					sortable: true
				}, {
					label: this.messages.hOnline,
					sortable: true
				}, {
					label: this.messages.hIp,
					sortable: true
				}, {
					label: this.messages.hHost,
					sortable: true
				}, {
					label: this.messages.hUptime,
					sortable: true
				}, {
					label: this.messages.hActiveServices,
					sortable: true
				}, {
					label: this.messages.hMemory,
					sortable: true
				}, {
					label: this.messages.hDiagnostics,
					sortable: true
				}
			];

			//Check for saved sort preferences
			var sortPrefs = dojo.fromJson(localStorage[LOCAL_STORAGE_KEY]);
			if(sortPrefs) {
				var sortBy = sortPrefs.sortBy;
				var asc = sortPrefs.asc;
			}
			
			this.styledTable = new jazz.ui.StyledTable({
				columns: columns,
				sortBy: (sortBy != null) ? sortBy : 1,
				sortByAscending: (asc != null) ? asc : true,
				handleSort: dojo.hitch(this, "handleSort")
			});
			this.contentArea.appendChild(this.styledTable.domNode);
		},
	
		
		getAllNodes: function() {
			if(!this.domNode) return;
			this.header.showMessage({
                type: "INFO",
                summary: this.messages.loadingPage
            });
			this.showNoNodesRow(/* loading */ true);
			var srh = new ServiceResponseHandler(this, "createNodeList", "errorFetchingNodeList");
            ServerStatusClient.getClusterMembers(srh);
		},
		
		createNodeList: function(nodelist) {
			if(!this.domNode) return;
			this.clusterNodes = [];
			this.processedCount = 0;
			if(nodelist) {
				dojo.forEach(nodelist, function(node) {
					this.clusterNodes.push(node);
					this.getNodeInfo(node);
				}, this);
			}
			if(this.clusterNodes.length === 0) {
				this.showNoNodesRow();
			}
		},
		
		errorFetchingNodeList: function(errorObject) {
			this.header.showMessage({
                errorObject: errorObject,
                summary: this.messages.errorFetchingNodeList
            });
			this.showNoNodesRow();
		},
		
		getNodeInfo: function(node) {
			var handler = {				
				handleNodeDetails: dojo.hitch(this, function(nodeDetails) {
					if(!this.domNode) return;
					nodeDetails.id = node.humanReadableNodeId;
					nodeDetails.url = node.url;
					nodeDetails.nodeStatus = node.nodeStatus;
					this.processNodes(nodeDetails);
				}),
				errorFetchingNodeDetails: dojo.hitch(this, function(errorObj) {
					node.id = node.humanReadableNodeId;
					node._hasError = true;
					node._errorMessage = dojo.string.substitute(this.messages.errorFetchingNodeDetails, [node.humanReadableNodeId]);
					node._errorObject = errorObj;
					this.processNodes(node);
				})
			};
			var srh = new ServiceResponseHandler(handler, "handleNodeDetails", "errorFetchingNodeDetails");
            ServerStatusClient.getClusterMemberInfo(srh, node.humanReadableNodeId);
		},
		
		processNodes: function(node) {
			if(node.id && !node._hasError) {
				node._diagNode = this.diagUtil.displayDiagnosticStatus(this.allDiagResults, node.id);
				node.diagnostics = (node._diagNode && node._diagNode._displayLabel) ? node._diagNode._displayLabel : "";
			}
			//Compute & save the used memory value for convenience and sorting
			node.usedMemory = node.maxMemory - node.freeMemory;
			
			//Save the string value of nodeStatus so the sorting matches the user's expectations.
			switch(node.nodeStatus) {
				case 0:
					node.statusLabel = this.messages.offline;
					break;
				case 2:
					node.statusLabel = this.messages.online;
					break;
				default:	
					node.statusLabel = this.messages.unknown;
			};
			
			this.nodesCache.push(node);
			this.processedCount++;
			this.handleSort();
			if(this.processedCount == this.clusterNodes.length) {
				if(this.someNodesUnreachable) {
					this.header.showMessage({
		                type: "WARNING",
		                summary: this.messages.someNodesUnreachable
		            });
				} else if(this.someNodeDataUnavailable) {
					this.header.showMessage({
		                type: "WARNING",
		                summary: this.messages.someNodeDataUnavailable
		            });
				} else {
					this.header.hideMessage();
				}
			}
		},
			
		createNodeRow: function(node) {
			var tr = dojo.create("tr", {}, this.styledTable.tbody);
			
			dojo.create("td", { innerHTML: node.id || "" }, tr);
			var statusTd = dojo.create("td", { style: { "white-space": "nowrap" }}, tr);
			if (node.nodeStatus === 2) {
				var okImage = dojo.create("img", { className: "status-icon", title: this.messages.online }, statusTd);
				okImage.src = STATUS_OK_IMG_SRC;
				dojo.create("span", { className: "statusText no-wrap", innerHTML: this.messages.online }, statusTd);
			} else if (node.nodeStatus === 0) {
				var errImage = dojo.create("img", { className: "status-icon", title: this.messages.offline }, statusTd);
				errImage.src = STATUS_ERR_IMG_SRC;
				dojo.create("span", { className: "statusText no-wrap", innerHTML: this.messages.offline }, statusTd);
			} else { //unknown: nodeStatus == 1
				var warnImage = dojo.create("img", { className: "status-icon", title: this.messages.unknown }, statusTd);
				warnImage.src = STATUS_WARN_IMG_SRC;
				dojo.create("span", { className: "statusText no-wrap", innerHTML: this.messages.unknown }, statusTd);
			}
			
			//If this node is an error, we should show the error and then exit. There's no more data to display.
			if(node && node._hasError) {
				var errorTd = dojo.create("td", { colSpan: 5 }, tr);
				var errorSpan = dojo.create("span", { innerHTML: node._errorMessage, className: "nodeErrorMessage" }, errorTd);
				var forgetTd = dojo.create("td", {}, tr);
				var aLink = dojo.create("a", { innerHTML: this.messages.forgetNode, href: "#", className: "forgetLink" }, forgetTd);
				this.connect(aLink, "onclick", function(e) {
					dojo.stopEvent(e);
					this.removeInactiveNode(node.id, tr);
				});
				
				if(node._errorObject) {
					var dLink = dojo.create("a", { innerHTML: this.messages.showDetails, href: "#" }, errorSpan);
					this.connect(dLink, "onclick", dojo.hitch(this, function(e) {
						dojo.stopEvent(e);
						if(tr._detailsRow) {
							dojo.toggleClass(tr._detailsRow, "noDisplay");
							dojo.toggleClass(tr, "expanded");
							dLink.innerHTML = dLink.innerHTML == this.messages.showDetails ? this.messages.hideDetails : this.messages.showDetails;
						}
					}));
					
					var detail = "";
					if(node._errorObject.message) {
						detail += "<strong>" + node._errorObject.message + "</strong><br />"; 
					}
					if (node._errorObject.errorClass) {
						detail += node._errorObject.errorClass + "<br /><br />";
					}
					if (node._errorObject.exception) {
						detail += dojo.isArray(node._errorObject.exception) ? node._errorObject.exception.join("<br>") : node._errorObject.exception;
					}

					//Add hidden stacktrace row
					var detailsRow = dojo.create("tr", { className: "detailsRow noDisplay" });
					tr._detailsRow = detailsRow;
					var stackTd = dojo.create("td", { colSpan: 8 }, detailsRow);
					dojo.create("div", { className: "service-trace", innerHTML: replaceNewLineWithPaddedBreaks(detail) }, stackTd);
					this.styledTable.tbody.appendChild(detailsRow);
				}
				
				//Remember that some nodes are unreachable so we can persist the message
				this.someNodesUnreachable = true;
				
				return;
			}
			
			var uptimeDisplay = this.messages.unknown;
			if(node.nodeUptime) {
				uptimeDisplay = jazz.util.date.formatDuration(node.nodeUptime, {
					style: "full",
					approximate: true
				});
			}
			
			dojo.create("td", { innerHTML: node.ip || this.messages.unknown }, tr);
			dojo.create("td", { innerHTML: node.hostName || this.messages.unknown }, tr);
			dojo.create("td", { innerHTML: uptimeDisplay }, tr);
			
			//Remember if any nodes have missing data so that we can persist a message after all nodes are loaded
			if(!node.ip || !node.hostName || !node.nodeUptime || !node.activeServicesCount || !node.usedMemory || !node.maxMemory) {
				this.someNodeDataUnavailable = true;
			}

			//Create Active Services link
			var activeServicesTd = dojo.create("td", {}, tr);
			if(node.activeServicesCount) {
				dojo.create("a", {
					href: "#action=com.ibm.team.repository.admin.activeServices&nodeId=" + node.id,
					innerHTML: node.activeServicesCount,
					title: this.messages.viewActiveServices
				}, activeServicesTd);	
			} else {
				dojo.create("span", { innerHTML: this.messages.unknown }, activeServicesTd);
			}

			//Create Memory Usage column
			var memoryTd = dojo.create("td", {}, tr);
			if(node.maxMemory && node.usedMemory) {
				var max = util.formatSizeFromBytes(node.maxMemory);
				var percentUsed = Math.floor(node.usedMemory / node.maxMemory * 100) + " %";
				var memoryUsage = dojo.create("span", { innerHTML: percentUsed }, memoryTd);
				this._textHelpHovers.push(new TextHoverHelp({
					helpContentString: util.formatSizeFromBytes(node.usedMemory) + " / " + max,
					contentNode: memoryUsage
				}));
			} else {
				dojo.create("span", { innerHTML: this.messages.unknown }, memoryTd);
			}

			//Create Diagnostics Link showing Node-specific results
			var diagTd = dojo.create("td", {}, tr);
			if(node._diagNode) {
				dojo.place(node._diagNode, diagTd, "only");
			}
		},
		
		getDiagnosticStatus: function() {
			var handler = {
				success: dojo.hitch(this, function(response) {
					this.allDiagResults = response;
					this.getAllNodes();
				}),
				error: dojo.hitch(this, function(errorObject) {
					console.error("Unable to fetch diagnostic results.");
					this.getAllNodes();
				})
			};
			var srh = new ServiceResponseHandler(handler, "success", "error");
			ServerStatusClient.getAllDiagnosticResults(srh);
		},

		showNoNodesRow: function(isLoading) {
			dojo.empty(this.styledTable.tbody);
			var tr = dojo.create("tr", {}, this.styledTable.tbody);
			dojo.create("td", { className: "noRowsYet", colSpan: 8, innerHTML: isLoading ? this.messages.loading : this.messages.noClusterNodes }, tr);
		},
		
		handleSort: function(args) {
			var index = this.styledTable.sortBy;
			
			//Save the users' sort preferences
			try {
				localStorage[LOCAL_STORAGE_KEY] = dojo.toJson({
					sortBy: index,
					asc: this.styledTable.sortByAscending
				});
			} catch (e) {} //not a critical op
			
			//Determine which column to sort by
			if(index === 0) {
				var nodes = this.sortNodeId();
			} else if(index === 1) {
				var nodes = this.sortNodeStatus();
			} else if(index === 2) {
				var nodes = this.sortIpAddress();
			} else if(index === 3) {
				var nodes = this.sortHostName();
			} else if(index === 4) {
				var nodes = this.sortUptime();
			} else if(index === 5) {
				var nodes = this.sortActiveServices();
			} else if(index === 6) {
				var nodes = this.sortMemoryUsage();
			} else if(index === 7) {
				var nodes = this.sortDiagnostics();
			}
			
			this.styledTable.clearBody();
			dojo.forEach(nodes, function(node) {
				this.createNodeRow(node);
			}, this);
		},
		
		removeInactiveNode: function(nodeId, row) {
			var handler = {				
				success: dojo.hitch(this, function(statuses) {
					if(!this.domNode) return;
					if(statuses && statuses.length > 0) {
						//If the operation was successful, the first (and only) status will be "OK".
						if(statuses[0].code === "OK") {
							//Remove the row
							if(row) {
								if(row._detailsRow) {
									this.styledTable.tbody.removeChild(row._detailsRow);
								}
								this.styledTable.tbody.removeChild(row);
							}
							this.header.hideMessage();
						} else {
							//Some of the listeners were unable to "forget" the node. 
							this.header.showMessage({
								details: statuses,
								type: "ERROR",
				                summary: this.messages.errorDeletingNodeDetailed
				            });
						}
					} else {
						console.error("The forget node operation returned no status object."); //$NON-NLS-1$
					}
				}),
				error: dojo.hitch(this, function(errorObj) {
					this.header.showMessage({
		                errorObject: errorObj,
		                summary: this.messages.errorDeletingNode
		            });
				})
			};
			var srh = new ServiceResponseHandler(handler, "success", "error");
            ServerStatusClient.postDeleteInactiveNode(srh, nodeId);
		},
		
		//Custom Sort Functions
		
		sortNodeId: function () {	
			return this.nodesCache.sort(dojo.hitch(this,function(a, b){
				var result = a.id.localeCompare(b.id);
				if (!this.styledTable.sortByAscending) {
					result = result * (-1);
				}
				return result;
			}));
		},
		
		sortNodeStatus: function () {	
			return this.nodesCache.sort(dojo.hitch(this,function(a, b){
				var result = a.statusLabel.localeCompare(b.statusLabel);
				if (!this.styledTable.sortByAscending) {
					result = result * (-1);
				}
				return result;
			}));
		},
		
		sortIpAddress: function () {
			return this.nodesCache.sort(dojo.hitch(this,function(a, b){
				var result = a.ip.localeCompare(b.ip);
				if (!this.styledTable.sortByAscending) {
					result = result * (-1);
				}
				return result;
			}));
		},

		sortHostName: function () {
			return this.nodesCache.sort(dojo.hitch(this,function(a, b){
				var result = a.hostName.localeCompare(b.hostName);
				if (!this.styledTable.sortByAscending) {
					result = result * (-1);
				}
				return result;
			}));
		},
		
		sortUptime: function () {
			return this.nodesCache.sort(dojo.hitch(this,function(a, b){
				var result = a.nodeUptime > b.nodeUptime ? 1 : -1;
				if (!this.styledTable.sortByAscending) {
					result = result * (-1);
				}
				return result;
			}));
		},
		
		sortActiveServices: function () {
			return this.nodesCache.sort(dojo.hitch(this,function(a, b){
				var result = a.activeServicesCount > b.activeServicesCount ? 1 : -1;
				if (!this.styledTable.sortByAscending) {
					result = result * (-1);
				}
				return result;
			}));
		},
		
		sortMemoryUsage: function () {
			return this.nodesCache.sort(dojo.hitch(this,function(a, b){
				var result = a.usedMemory > b.usedMemory ? 1 : -1;
				if (!this.styledTable.sortByAscending) {
					result = result * (-1);
				}
				return result;
			}));
		},
		
		sortDiagnostics: function () {
			return this.nodesCache.sort(dojo.hitch(this,function(a, b){
				var result = a.diagnostics.localeCompare(b.diagnostics);
				if (!this.styledTable.sortByAscending) {
					result = result * (-1);
				}
				return result;
			}));
		},
		
		uninitialize: function () {
			this.header.destroyRecursive();
			
			// TextHoverHelp Nodes
			if (this._textHelpHovers) {
				for (textID in this._textHelpHovers) {
					if (this._textHelpHovers[textID]) {
						this._textHelpHovers[textID].destroy();
						delete this._textHelpHovers[textID];
					}
				}
				delete this._textHelpHovers;
			}
			
			this.inherited(arguments);
        }
		
    });

})();
