/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010, 2011. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  
 * Use, duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp. 
 *******************************************************************************/
(function() {
dojo.provide("com.ibm.team.repository.web.ui.internal.admin.server.ComponentsView");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dijit.TitlePane");
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
dojo.require("com.ibm.team.repository.web.client.internal.ServerStatusClient");
dojo.require("com.ibm.team.repository.web.ui.internal.admin.util");
dojo.require("jazz.ui.MessageBox");
dojo.require("dojo.i18n");

dojo.requireLocalization("com.ibm.team.repository.web.ui.internal.admin.server", "ComponentsViewMessages");

var replaceNewLineWithPaddedBreaks = com.ibm.team.repository.web.ui.internal.admin.util.replaceNewLineWithPaddedBreaks;
var ServiceResponseHandler	= com.ibm.team.repository.web.transport.ServiceResponseHandler;
var ServerStatusClient = com.ibm.team.repository.web.client.internal.ServerStatusClient;
var MessageBox = jazz.ui.MessageBox;
var STATUS_OK_IMG_SRC = dojo.moduleUrl("com.ibm.team.repository.web", "ui/internal/wizard/images/verify_obj.gif");
var STATUS_ERR_IMG_SRC = dojo.moduleUrl("com.ibm.team.repository.web", "ui/internal/images/status-error.gif");

var ALIAS_MAP = {};
var messages;

dojo.declare("com.ibm.team.repository.web.ui.internal.admin.server.ComponentsView", [dijit._Widget, dijit._Templated], {

	templatePath: dojo.moduleUrl("com.ibm.team.repository.web", "ui/internal/admin/server/templates/ComponentsView.html"),

	constructor: function() {
		this.messages = messages = dojo.i18n.getLocalization("com.ibm.team.repository.web.ui.internal.admin.server", "ComponentsViewMessages");
	},	
	
	postCreate: function() {
		com.ibm.team.repository.web.ui.internal.admin.server.ComponentsView.superclass.postCreate.apply(this);
		this.header = new MessageBox();
		this.headerArea.appendChild(this.header.domNode);
		this.refresh();
	},
	
	refresh: function () {
		this.header.hideMessage();
		var srh= new ServiceResponseHandler(this, "handleComponentResponse", "_handleComponentsFetchError");
		ServerStatusClient.getComponentsAndStatus(srh);
	},

	handleComponentResponse: function(componentAndStatuses) {
		var exists = this._emptyIfDomExists(this.componentsArea);
		if(!exists) return;

        ALIAS_MAP = {};
        
        if (componentAndStatuses && componentAndStatuses.length > 0) {
            componentAndStatuses.sort(function(a, b){
                return a.component.id.toLowerCase() > b.component.id.toLowerCase() ? 1 : -1;
            });
            var self = this;
            dojo.forEach(componentAndStatuses, function(object){				
				var compDom = self._createComponentSection(object); 
				self.componentsArea.appendChild(compDom); 
            });
        }
	},
	
	_createComponentSection: function(componentAndStatuses) {
		var div = dojo.create("div", { className: "component com-ibm-team-repository-admin-table" });
		var headingDiv = dojo.create("div", { className: "heading" }, div);
		var errorImg = dojo.create("img", { className: "component-status noDisplay" }, headingDiv);
		errorImg.src = STATUS_ERR_IMG_SRC;
		if (componentAndStatuses.component.modelURI) {
			dojo.create("div", { className: "model", innerHTML: "<span>" + messages.modelLabel + "</span> " + componentAndStatuses.component.modelURI }, headingDiv);
		}
		dojo.create("span", { innerHTML: componentAndStatuses.component.id }, headingDiv);
		var detailsDiv = dojo.create("div", { className: "component-details" }, div);		
		var serviceStatuses = componentAndStatuses.serviceStatuses;			
		if (serviceStatuses && serviceStatuses.length > 0) {		
			dojo.create("div", { className: "service-list-header", innerHTML: messages.servicesLabel }, detailsDiv);
			var serviceList = dojo.create("div", { className: "service-list" }, detailsDiv);				
			// sort by name
			serviceStatuses.sort(
				function(a, b) {
					return a.service.interfaceName.toLowerCase() > b.service.interfaceName.toLowerCase() ? 1 : -1;
				});
			var componentHasError = false;
			for (var i = 0; i < serviceStatuses.length; i++) {
				var service = serviceStatuses[i].service;
				var nameLabel = service.interfaceName;
				var serviceDiv = dojo.create("div", { className: "service", id: nameLabel }, serviceList);
				if (service.aliasName != null) {
					nameLabel = this._formatAliasLabel(service);
					ALIAS_MAP[service.interfaceName] = service.aliasName;
				}
				var statusObj = serviceStatuses[i].status;
				if (statusObj && statusObj.code === "OK") {
					dojo.addClass(serviceDiv, "status-ok");
					var okImage = dojo.create("img", { className: "service-status", title: statusObj.message || "" }, serviceDiv);
					okImage.src = STATUS_OK_IMG_SRC;
					dojo.create("div", { className: "service-name", innerHTML: nameLabel }, serviceDiv);
				} else {
					componentHasError = true;		
					var serviceDOM = dojo.create("div", { className: "service" });	
					serviceDOM.appendChild(this._getErrorServiceStatusSection(statusObj));
					serviceList.replaceChild(serviceDOM, serviceDiv);
				}
			}
			if(componentHasError) {
				dojo.removeClass(errorImg, "noDisplay");
			}
		}
		else{
			dojo.create("div", { className: "service-list-header", innerHTML: messages.noServicesLabel }, detailsDiv);
		}
		return div;	
	},
	
	_getErrorServiceStatusSection: function(status) {
		var nameLabel = status.interfaceName;
		if (ALIAS_MAP[status.interfaceName] != null) {
			nameLabel = status.interfaceName + 
			dojo.string.substitute(messages.alias, [ ALIAS_MAP[status.interfaceName] ]);
		}
		var statusDetails = new dijit.TitlePane({title: nameLabel});
		
		var errorContent = dojo.create("div");
		dojo.create("div", { className: "status-summary", innerHTML: status.message }, errorContent);
		if(status.stacktrace) {
			dojo.create("div", { innerHTML: replaceNewLineWithPaddedBreaks(status.stacktrace, false) }, errorContent);
		}
		statusDetails.set("content",errorContent);
		statusDetails.toggle();
		statusDetails.arrowNode.title = status.message;
		return statusDetails.domNode;
	},

	_emptyIfDomExists: function(node) {
		if(node) {
			node.innerHTML = "";
			return true;
		} else {
			return false;
		}
	},

	_formatAliasLabel: function (service) {
		return service.interfaceName + "<span class=\"alias\">" + dojo.string.substitute(this.messages.alias, [service.aliasName]) + " </span>";
	},
		
	_handleComponentsFetchError: function(errorObj) {
		this.header.showMessage({
			errorObject: errorObj
		});
	},
	
	uninitialize: function () {
		this.header.destroyRecursive();
		this.inherited(arguments);
    }
});

})();