/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010, 2012. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  
 * Use, duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp. 
 *******************************************************************************/
dojo.provide("com.ibm.team.repository.web.ui.internal.admin.server.LicenseAgreementStep");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("jazz.ui.wizard.WizardStepHeader");
dojo.require("jazz.ui.wizard.WizardStep");
dojo.require("jazz.ui.StyledBox");

dojo.require("dojo.i18n");
dojo.requireLocalization("com.ibm.team.repository.web.ui.internal.admin.server", "LicenseAgreementStepMessages");

(function() {
var messages;
var WizardStepHeader = jazz.ui.wizard.WizardStepHeader;
var WizardStep = jazz.ui.wizard.WizardStep;
var TrimStyles = jazz.ui.StyledBox.TrimStyles;

dojo.declare("com.ibm.team.repository.web.ui.internal.admin.server.LicenseAgreementStep", [WizardStep, dijit._Widget, dijit._Templated], {

	templatePath: dojo.moduleUrl("com.ibm.team.repository.web", "ui/internal/admin/server/templates/LicenseAgreementStep.html"),
	
	constructor: function() {
		this.messages = messages = dojo.i18n.getLocalization("com.ibm.team.repository.web.ui.internal.admin.server", "LicenseAgreementStepMessages");
		this.stepTitle = this.messages.stepTitle;
	},

	postCreate: function() {
		this.hasSelected = false;
		this.licenseText.value = this.agreementText;
		this.licenseText.readOnly = true;
		this.displayName = this.licenseKey.productName ? this.licenseKey.productName + " " : "";
		this.displayName += this.licenseKey.name ? this.licenseKey.name + " " : "";	
		
		this.header = new WizardStepHeader({stepTitle: this.displayName + " " + this.stepTitle});
		this.stepHeaderArea.appendChild(this.header.domNode);
		
		if(!this.readOnly) {
			this.connect(this.btnAccept, "onclick", "handleSelection");
			this.connect(this.btnDecline, "onclick", "handleSelection");
		} else {
			dojo.addClass(this.controls,"noDisplay");
		}
	},
	
	focusStep: function() {
		this.header.hideMessage();
	},
	
	handleSelection: function() {
		this.hasAgreed = this.btnAccept.checked;
		
		if(this.hasAgreed) {
			this.header.hideMessage();
			
		} else {
			this.header.showMessage({
	            type: "WARNING",
	            summary: dojo.string.substitute(this.messages.declinedWarning, [this.displayName])
	        });
		}
		
		this.hasSelected = true;
		this.triggerButtonStateUpdate();
	},

	//As long as the user has made a selection, we allow them to continue with the wizard.	
	canFlipToNextPage: function() {
		return this.hasSelected;
	},
	
	canFinish: function() {
		return this.readOnly || (this.hasSelected && this.finalStep());
	},
	
	handleAgreementCallback: function() {
		if (!this.readOnly && this.onAgreement && typeof this.onAgreement == "function") {
			this.onAgreement(this.licenseKey.id, this.hasAgreed);
		}
	},

	onNext: function() {
		this.handleAgreementCallback();
		this.wizard.processNext();
	},
	
	onFinish: function() {
		if(!this.readOnly) {
			this.header.showMessage({
				summary: this.messages.savingLicenseInfo,
				type: "INFO"
			});
		}
		this.handleAgreementCallback();
		this.wizard.processFinish(this); //pass a reference to this step so we can display any errors to the user
	},
		
	uninitialize: function() {
		this.header.destroyRecursive();
		this.inherited(arguments);
	}
	
});

})();