/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010, 2013. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  
 * Use, duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp. 
 *******************************************************************************/
(function(){
    dojo.provide("com.ibm.team.repository.web.ui.internal.admin.server.LicenseEditor");
    
    dojo.require("dojo.date");
    dojo.require("dojo.date.locale");
    dojo.require("dojo.date.stamp");
    dojo.require("dojo.number");
    dojo.require("dijit._Widget");
    dojo.require("dijit._Templated");
    dojo.require("dojox.xml.parser");
    
    dojo.require("net.jazz.ajax.ui.PlatformUI");
    dojo.require("com.ibm.team.repository.web.ui.internal.common.Toolbar");
    dojo.require("com.ibm.team.repository.web.client.internal.licensing");
    dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
	dojo.require("com.ibm.team.repository.web.client.internal.ServerStatusClient");
    dojo.require("com.ibm.team.repository.web.ui.internal.Util");
	dojo.require("com.ibm.team.repository.web.ui.internal.events");

    dojo.require("dojo.i18n");
    dojo.requireLocalization("com.ibm.team.repository.web.ui.internal.admin.server", "LicenseEditorMessages");
    
	dojo.require("com.ibm.team.repository.web.ui.internal.admin.server.LicenseHelper");
    dojo.require("com.ibm.team.repository.web.ui.internal.admin.server.LicenseWizard");
    dojo.require("jazz.ui.Wizard");
    dojo.require("jazz.ui.wizard.WizardStep");
    dojo.require("jazz.ui.MessageBox");
	dojo.require("com.ibm.team.repository.web.ui.wizard.WizardConfigurationStep");
	dojo.require("com.ibm.team.repository.web.ui.internal.admin.server.LicenseConfigFloatingStep");	
	dojo.require("com.ibm.team.repository.web.ui.internal.admin.server.LicenseTokenServiceProvidersView");	
	dojo.require("com.ibm.team.repository.web.ui.internal.admin.server.LicenseAgreementStep");
    dojo.require("jazz.ui.StyledBox");
    dojo.require("jazz.ui.Dialog");
	dojo.require("jazz.ui.Button");
	dojo.require("jazz.app.ua.PageHoverHelp");
    
    //
    // Aliases
    //
    
    var LIC_P = com.ibm.team.repository.web.client.internal.licensing;
    var formatCALTypeDescription = LIC_P.formatCALTypeDescription;
    var getLicenseInfoWithDefaults = LIC_P.getLicenseInfoWithDefaults;
    var getUsedLicenseInfo = LIC_P.getUsedLicenseInfo;
    var getExpirationInfo = LIC_P.getExpirationInfo;
    var getLicenseAgreement = LIC_P.getLicenseAgreement;
	var hasExpiredTrial = LIC_P.hasExpiredTrial;
	var updateTypeTrialStatus = LIC_P.updateTypeTrialStatus;
    var sortCALs = LIC_P.sortCALs;
    var removeChildren = dojox.xml.parser.removeChildren;
    var SRH = com.ibm.team.repository.web.transport.ServiceResponseHandler;
    var LicenseHelper = com.ibm.team.repository.web.ui.internal.admin.server.LicenseHelper;
	var LicenseWizard = com.ibm.team.repository.web.ui.internal.admin.server.LicenseWizard;
	var LicenseAgreementStep = com.ibm.team.repository.web.ui.internal.admin.server.LicenseAgreementStep;
    var FLOATING_SUPPORT_OPERATION_ID = "com.ibm.team.repository.support.floatingLicenses";
    var ACTION_ID = "action=com.ibm.team.repository.admin.manageLicenses";
    var MessageBox = jazz.ui.MessageBox;
    var WizardStep = jazz.ui.wizard.WizardStep;
	var WizardConfigurationStep = com.ibm.team.repository.web.ui.wizard.WizardConfigurationStep;
	var FloatingConfigStep = com.ibm.team.repository.web.ui.internal.admin.server.LicenseConfigFloatingStep;
	var ServerStatusClient = com.ibm.team.repository.web.client.internal.ServerStatusClient;
	var DEFAULT_LICENSE_SERVICE = "com.ibm.team.repository.service.internal.license.DefaultLicenseProviderService";
	var STATUS_TYPES = MessageBox.STATUS_TYPES;
	var Toolbar = com.ibm.team.repository.web.ui.internal.common.Toolbar;
	var PageHoverHelp = jazz.app.ua.PageHoverHelp;
	//var licenseTokenCombo = new com.ibm.team.repository.web.ui.internal.admin.server.FloatingLicenseTypeCombo;
    var messages;
    //
    // Widget definition
    //
    
    dojo.declare("com.ibm.team.repository.web.ui.internal.admin.server.LicenseEditor", [LicenseHelper, dijit._Widget, dijit._Templated], {
    
        //
        // Class properties
        //
        
        templatePath: dojo.moduleUrl("com.ibm.team.repository.web", "ui/internal/admin/server/templates/LicenseEditor.html"),
        _actionToolbars: [],
		
        //
        // Widget lifecycle methods
        //
        constructor: function(){
            this.messages = messages = dojo.i18n.getLocalization("com.ibm.team.repository.web.ui.internal.admin.server", "LicenseEditorMessages");
            var detailsLink = "<a href='#action=com.ibm.team.repository.admin.manageLicenses'>";
            this.clientDescriptionIncludingLink = dojo.string.substitute(this.messages.clientDescription, [detailsLink, "</a>"]);
            this.noFloatingSupportIncludingLink = dojo.string.substitute(this.messages.noFloatingSupport, [detailsLink, "</a>"]);
            this.floatingUploadWarningIncludingLink = dojo.string.substitute(this.messages.floatingUploadWarning, [detailsLink, "</a>"]);	
			var helpLink = "<a href='http://pic.dhe.ibm.com/infocenter/clmhelp/v4r0m6/topic/com.ibm.jazz.repository.web.admin.doc/topics/c_managing_licenses.html'>";
			var downloadLink = "<a href='http://www-01.ibm.com/software/rational/support/licensing/'>";
			var buyLink = "<a href='https://www-112.ibm.com/software/howtobuy/softwareandservices/passportadvantage'>";
			this.description = dojo.string.substitute(this.messages.description, [helpLink, "</a>", buyLink, "</a>", downloadLink, "</a>"]);
        },
        
        postCreate: function(){
			this.header = new MessageBox();
	        this.headerArea.appendChild(this.header.domNode);
			
			//Append the context sensitive help
			this._pageHelpHover = new PageHoverHelp({ uaKey: "com.ibm.jazz.repository.web.admin.ui.licenseKeyMgmt" });
			this.helpHover.appendChild(this._pageHelpHover.domNode);
			
			var args = {
				parentPointer : this
			};
			this._tokenProvidersView = new com.ibm.team.repository.web.ui.internal.admin.server.LicenseTokenServiceProvidersView(args,this.licenseTokenProviderConfigDiv);
						
            this.reload();
        },
		
		launchFloatingConfig: function () {
			this._destroyEditorWizards();
			if (!this.licenseInfo) {
				//License info request failed. don't launch wizard
				return;
			} else {
			
				var self = this;
				
				//Need this. Used when saving Connect wizard for saveLicenseUpdates()
				this.wiz = new LicenseWizard({
					licenseInfo: self.licenseInfo,
					parentPointer: self
				});
				
				var uriConfigArgs = {
					stepTitle: this.messages.configFloatingStepTitle,
					parentPointer: this,
					description: this.messages.configFloatingdescription,
					helpContent: this.messages.configFloatingHelp,
					implementationClassName: "com.ibm.team.repository.service.internal.license.LicenseService",
					extraServiceConfigArgs: {
						hiddenPropertyList: ["licenseService.floatingLicenseRefreshRate","licenseService.autoAssignLicense", "licenseService.floatingLicenseCheckoutOrder", "licenseService.floatingLicenseDenialTimeout", "licenseService.calLogPeriod", "licenseService.calEnableLogging"],
						forceSimple: true
					},
					innerArgs: {
						headingLabel: this.messages.configFloatingBlockLabel,
						heading: ""
					},
					canFinish: function() {
						return this.enableNext === true;
					}
				}
				this.uriConfigStep = new FloatingConfigStep(uriConfigArgs);
				
				var wizardArgs = {
					steps: [this.uriConfigStep],
					width: "870px",
					primaryTitle: this.messages.configFloatingStepTitle,
					showStepSummary: false,
					processFinish: function(){
						this.wizardFinishButton.disabled = true;
						this._steps[0].invokeSave(self, "saveLicenseUpdates", "saveLicenseError");
					},
					processCancel: function(){
						self.uriConfigStep.cancelRequested();
					} 
				};
				this.wizard = new jazz.ui.Wizard(wizardArgs);
				this.wizard.startup();
			}

		},
        
        launchWizard: function() {
        	this._destroyEditorWizards();
            var self = this;
			
			if (!this.licenseInfo) {
				//License info request failed. don't launch wizard
				return;
			} else {
			
				//Step One: License Uploads
				this.wiz = new LicenseWizard({
					mode: "upload",
					licenseInfo: self.licenseInfo,
					parentPointer: self,
					stepTitle: this.messages.step1Title
				});
				
				var wizardArgs = {
					steps: [this.wiz],
					width: "870px",
					primaryTitle: this.messages.step1Title,
					showStepSummary: true,
					processFinish: function(currentStepWidget){
						this.wizardFinishButton.disabled = true;
						var numSteps = self.wizard.contentWidget._steps.length;
						//Don't need to save Floating Connection if they never upload a floating CAL!
						if (!self.wiz.hasFloatingStep) {
							self.wiz.saveEditor(currentStepWidget);	//reference to the current step so we can display any errors to the user
						}
						else {
							self.wiz.saveFloatingConnection(currentStepWidget);
						}
						
					},
					processCancel: function(){
						self.wiz.cancelRequested();
					} 
				};
				this.wizard = new jazz.ui.Wizard(wizardArgs);
				this.wizard.startup();
			}
        },

		saveLicenseUpdates: function (){
			this.wiz.saveEditor();
		},
		
		saveLicenseError: function (errObj) {
			this.wiz.cbSaveError(errObj);
		},
		
		launchFloatingEditor: function() {
			//passthru to the "real" CAL editor in order to control CALs displayed
			var floatingOn = true;
			this.launchCALEditor(floatingOn);
		},
		launchCALEditor: function(showFloating) {
			this._destroyEditorWizards();
			if (!this.licenseInfo) {
				//License info request failed. don't launch wizard
				return;
			} else {
			
				var self = this;		
				
				//If this function is called directly, showFloating is a click event
				if (showFloating != true) 
					showFloating = false;
					
				var sTitle = showFloating ? this.messages.editFloatTitle : this.messages.editCALTitle;
				
				//Step One: Change CAL purchase counts
				this.editor = new LicenseWizard({
					mode: "editCALs",
					floatingOn: showFloating,
					licenseInfo: self.licenseInfo,
					parentPointer: self,
					stepTitle: sTitle
				});
				
				var wizardArgs = {
					steps: [this.editor],
					width: "870px",
					primaryTitle: sTitle,
					showStepSummary: false,
					processFinish: function(){
						this.wizardFinishButton.disabled = true;
						self.editor.saveEditor();
					},
					processCancel: function() {
						self.editor.cancelRequested();
					} 
					
				};
				this.wizard = new jazz.ui.Wizard(wizardArgs);
				this.wizard.startup();
			}
		},
		
		launchDefaultWizard: function() {
			this._destroyEditorWizards();
			if (!this.licenseInfo) {
				//License info request failed. don't launch wizard
				return;
			} else {
			
				var self = this;		
				var sTitle = this.messages.defaultWizardHeader;
				
				this.setDefault = new LicenseWizard({
					mode: "setDefault",
					licenseInfo: self.licenseInfo,
					defaultLicenses: self.defaultLicenses,
					parentPointer: self,
					stepTitle: sTitle
				});
				
				var wizardArgs = {
					steps: [this.setDefault],
					width: "870px",
					primaryTitle: sTitle,
					showStepSummary: false,
					processFinish: function(){
						this.wizardFinishButton.disabled = true;
						self.setDefault.testLicenseSelections();
					},
					processCancel: function(){
						self.setDefault.cancelRequested();
					} 
				};
				this.wizard = new jazz.ui.Wizard(wizardArgs);
				this.wizard.startup();
			}
		},
        
        /**
         * Fetches the most recent saved license information from the server and
         * resets the editor with this information (long-op).
         */
        reload: function(){           
            var self = this;
            var o = {
                s: function(licenseInfo){
                    self.header.hideMessage();
                    self.reset(licenseInfo);
                    self.initialLicenseInfo = self.licenseInfo = licenseInfo;                    
                },
                f: function(err){
                    self._handleServiceError(err);
                }
            }
            getLicenseInfoWithDefaults(new SRH(o, "s", "f"));
        },
        
        rebind: function(){
			this.refreshServerTable(this.serverInfoTable);
            this.refreshCALTable();
            this.refreshFloatingCALTable();
        },
        
        reset: function(licenseInfo){
            if(!this.domNode) return;
			this._setInfopopHeading(this.totalHeader, "totalHeader", "totalHeaderInfo");
            this._setInfopopHeading(this.assignedHeader, "assignedHeader", "assignedHeaderInfo");
            this._setInfopopHeading(this.availableHeader, "availableHeader", "availableHeaderInfo");
            this._setInfopopHeading(this.floatTotalHeader, "floatTotalHeader", "floatTotalHeaderInfo");
			
            this.initialLicenseInfo = this.licenseInfo = licenseInfo;
            this.usedLicenseInfo = null;
            this.lastFloatingCount = licenseInfo.floatingLicenseTypes && licenseInfo.floatingLicenseTypes.length ? licenseInfo.floatingLicenseTypes.length : 0;
            this.rebind();
  
            var self = this;
            var floatHelper = {
                yes: function(status){
                    self.addLicenseUsageDetails();
					if(self.AllowFloatingEdit) {
						self.AllowFloatingEdit.style.display = "inline";
					}
                },
                no: function(s){
                    self.usedLicenseInfo = null;
					if (self.AllowFloatingEdit) {
						self.AllowFloatingEdit.style.display = "none";
					}
                },
                error: function(err){
                    self._handleServiceError(err);
                }
            }
            
            LIC_P.checkFloatingLicensesSupported(floatHelper.yes, floatHelper.no, floatHelper.error);
        },
		
		_handleServiceError: function(errorObj) {
	        this.header.showMessage({
				errorObject: errorObj
			});
	    },
 
        /**
         * Fetches the detailed breakdown of the license usage and updates the UI.
         * This method will only success if floating licenses are supported on the server
         */
        addLicenseUsageDetails: function(){
            this.licenseInfo.operationCheckResults = [{
                "operationId": FLOATING_SUPPORT_OPERATION_ID,
                "allowed": true
            }];
            var self = this;
            var floatingLicenseInfoHandler = {
                success: function(info){
                    self.updateUsedLicenseData(info);
                },
                failure: function(err){
                    self.usedLicenseInfo = null;
                    self._handleServiceError(err);
                }
            }
            getUsedLicenseInfo(new SRH(floatingLicenseInfoHandler, "success", "failure"));
        },
        
        /**
         * This is called when addition details of the used
         * licenses on the server.  It provides a breakdown
         * between floating and non-floating use.  This method
         * might not be called if the underlying server does not
         * support floating licenses
         
         */
        updateUsedLicenseData: function(usedLicenseInfo){
            this.usedLicenseInfo = usedLicenseInfo;
            
            if (this.licenseInfo) {
				this.refreshServerTable(this.serverInfoTable);
			}
                
        },
        
        
        /**
         * Refreshes the Client Access License table based on the current value of
         * <code>this.licenseData.types</code>.
         */
        refreshCALTable: function(){
			if(!this.CALExpiresHeader) return;
			//Only show the "Expired On" header if some CALs have expiration
			var hasExpiringCALs = LIC_P.checkExpiringCALs(this.licenseInfo, false);
			dojo.toggleClass(this.CALExpiresHeader, "noDisplay", !hasExpiringCALs)
            
            var tbody = this.CALInfoTable;
            var self = this;
            var inputs = this.CALInfoTable.getElementsByTagName("input");
            dojo.forEach(inputs, function(input){
                if (input.__jazzChgEvt) {
                    self.disconnect(input.__jazzChgEvt);
                }
            });
            removeChildren(tbody);
            this.EditButtonCAL.style.display = "none";
            // normalize data and create table columns
            var types = this.licenseInfo.types || [];
            types = sortCALs(types);
            var isTrial = this.licenseInfo.serverLicense.trial;
			var count = 0;
			var someHaveEditions = false;
			dojo.forEach(types, function(type){
				count++;
				self.EditButtonCAL.style.display = "inline";
				
				self._addCALRow(tbody, type, isTrial, hasExpiringCALs, false);
				if(type.editionName) {
					someHaveEditions = true;
				}
				if (type.keys && type.keys.length > 1) {
					dojo.forEach(type.keys, function(key) {
						count++;
						if(key.editionName) {
							someHaveEditions = true;
						}
						self._addCALRow(tbody, key, isTrial, hasExpiringCALs, true);
					});
				}
            });
			
			//Change the name of the edition column to "Variant" if there are no editions.
			this._changeColumnLabel(this.CALTable, /* column index */ 3, someHaveEditions ? this.messages.edition : this.messages.variant );

			if(count == 0) {
				//No CALs uploaded so far, show message.
				var tr = dojo.create("tr", {}, tbody);
				dojo.create("td", { className: "noRowsYet", colSpan: 11, innerHTML: this.messages.noCALsYet }, tr);
			}
        },
		
		_changeColumnLabel: function(table, index, newLabel) {
			var rows = table.getElementsByTagName("tr");
			if(rows && rows.length > 0) {
				//Show/Hide the th from header row
				var ths = rows[0].getElementsByTagName("th");
				if(ths && ths[index] != null) {
					ths[index].innerHTML = newLabel;
				}	
			}	
		},
		
		_toggleColumnDisplay: function(table, index, hide) {
			var rows = table.getElementsByTagName("tr");
			if(rows && rows.length > 0) {
				//Show/Hide the th from header row
				var ths = rows[0].getElementsByTagName("th");
				if(ths && ths[index] != null) {
					dojo.toggleClass(ths[index], "noDisplay", hide);
				}
				//Show/Hide the td from each normal row
				for(var i = 1; i < rows.length; i++) {
					var tds = rows[i].getElementsByTagName("td");
					if (tds && tds[index] != null) {
						dojo.toggleClass(tds[index], "noDisplay", hide);
					}			
				}	
			}
		},
		
		_addCALRow: function(tbody, type, isTrial, hasExpiringCALs, isKey) {
			var icon = dojo.create("div");
			var status =  dojo.create("span");
            var row =  dojo.create("tr", {}, tbody);
			var nameLink = this._getInfopopLink(type.name, formatCALTypeDescription(type));
			
			//Some properties are stored inside the key only.
			if(!isKey && type.keys && type.keys.length == 1) {
				var key = type.keys[0];
			}
			
			//Make sure the trial status on the license type is correct (necessary if we have mixed keys - perm & trials)
            type = updateTypeTrialStatus(type);
			
			var isDisabled = type.disabledReason != null;
			var hasUnactivatedTrial = type.hasUnactivatedTrial;
			var isExpiredTrial = hasExpiredTrial(type);
			
			if (isExpiredTrial || hasUnactivatedTrial || isDisabled) {
                row.className = "disabledCAL";
				icon.className = "icon-warning";
				dojo.addClass(nameLink,"disabledCAL");
				dojo.addClass(status,"WARNINGcolor");
				if(isExpiredTrial) {
					status.innerHTML = this.messages.expiredTrial;	
				} else if(hasUnactivatedTrial) {
					var durationOfTrial = getExpirationInfo(type.unactivatedTrialExpirationTime).daysRemaining;
					status.innerHTML = dojo.string.substitute(this.messages.trialNotStarted, [durationOfTrial]);
					var activateLink = this._createActivateLink(key || type);
					status.appendChild(activateLink);
				} else {
					status.innerHTML = dojo.string.substitute(this.messages.deactivatedReason, [type.disabledReason]);
				}
            } else {
				icon.className = "icon-ok";
				status.innerHTML = this.messages.activated;
			}
			
            var limit = getCountOrUnlimited(type.max);
            var included = getCountOrUnlimited(type.free);
            var total = getCountOrUnlimited(type.total);

			var isTrialCAL = type.trial;
            if (isTrialCAL || (isTrial && type.floating == false)) {
				var available = type.max === -1 ? this.messages.unlimited : type.max - type.used;
				var purchase = type.max === -1 ? this.messages.notApplicable : type.max - type.free;
				var total = getCountOrUnlimited(type.max);
			}
			else {
				var available = type.total === -1 ? this.messages.unlimited : type.total - type.used;
				var purchase = type.free === -1 ? this.messages.notApplicable : type.total - type.free;

				//Add some UA in the scenario where multiple product keys have totals > server maximum and this is confusing.
				if(!isKey && type.total === type.max && type.total !== -1) {
					var keyTotalValue = 0;
					if (type.keys && type.keys.length) {
						dojo.forEach(type.keys, function(key) {
							if(key.total && key.total > 0) {
								keyTotalValue += key.total;
							}
						});
					}
					if(keyTotalValue > type.total) {
						var totalDiv = dojo.create("div", { className: "num" });
						this._setInfopop(totalDiv, total, this.messages.infoTotalAvailableLimitedByMax);
					}
				}
			}
			
			if (isKey) {
				available = this.messages.notApplicable;
			}
			
			var productName = dojo.create("div");
			productName.appendChild(document.createTextNode(type.productName || " ")); //IE7
			if (isKey) {
				var productDiv = dojo.create("div", { className: "sub-license-key" });
				productDiv.appendChild(productName);
			}
			
			var variantString = LIC_P.getVariantString(type) || null; //IE7
			var calVersion = type.version ? dojo.create("span", {  innerHTML: type.version }) : null;
			if (calVersion) calVersion._className = "num"; 
			
            // Create the columns for the given CAL
            var nodes = [productDiv || productName, calVersion, nameLink, variantString, totalDiv || total, getUsedLink(type, isKey), available, icon, status];
	
			//Add Expiration column if applicable 
			if(hasExpiringCALs) {
				//If this is a type with multiple keys, don't show a value for expiration on the parent row
				if(isKey || type.keys.length === 1) {
					var expiresOn = type.expirationTime;
					if(expiresOn) {
						var expirationInfo = getExpirationInfo(expiresOn);
						expiresOn = expirationInfo.expirationDateString;
					}	
				}
				nodes.push(expiresOn || "");
			}
			
			var toolbar = new Toolbar();
			this._actionToolbars.push(toolbar);
			if(!type.floating && (isKey || type.keys.length == 1)) {
				toolbar = this._addDeleteAction(toolbar, type);
			}
			if(isKey) {
				if(type.hasAgreeement) {
					toolbar = this._addViewAgreementAction(toolbar, type);
				}
			} else {
				if(key && key.hasAgreement) {
					toolbar = this._addViewAgreementAction(toolbar, key);
				}
			}
			
			if(toolbar.hasActions()) {
				dojo.addClass(row,"hasAction");
				nodes.push(toolbar.domNode);
			} else {
				nodes.push(null); //IE7
			}
            dojo.forEach(nodes, function(node){
                col(row, node);
            });
		},
		
        /**
         * Refreshes the Floating Client Access License table based on the current value of
         * <code>this.licenseData.floatingLicenseTypes</code>.
         */
        refreshFloatingCALTable: function(){
			if(!this.FloatExpiresHeader) return; //DOM has been destroyed
			
			//Only show the "Expired On" header if some CALs have expiration
			var hasExpiringCALs = LIC_P.checkExpiringCALs(this.licenseInfo, true);
			this.FloatExpiresHeader.style.display = hasExpiringCALs ? "block" : "none";
			
            var tbody = this.FloatCALInfoTable;
            var self = this;
            removeChildren(tbody);
            this.EditButtonFloat.style.display = "none";
            // normalize data and create table columns
            var types = this.licenseInfo.floatingLicenseTypes || [];
            if (this.licenseInfo.floatingLicenseTypes) {
                types = sortCALs(types);
				var someHaveEditions = false;
                dojo.forEach(types, function(type){
		        	//Enable the EDIT button
					self.EditButtonFloat.style.display = "inline";
					
 					self._addFloatingCALRow(tbody, type, hasExpiringCALs, false);
					if (type.keys && type.keys.length > 1) {
						dojo.forEach(type.keys, function(key) {
							self._addFloatingCALRow(tbody, key, hasExpiringCALs, true);
						});
					}

                });
            } else {
				//No CALs uploaded so far, show message.
				var tr = dojo.create("tr", {}, tbody);
				dojo.create("td", { className: "noRowsYet", colSpan: 10, innerHTML: this.messages.noCALsYet }, tr);
			}
			//Change the name of the edition column to "Variant" if there are no editions.
			this._changeColumnLabel(this.FloatingCALTable, /* column index */ 3, someHaveEditions ? this.messages.edition : this.messages.variant );
        },
		
		_addFloatingCALRow: function(tbody, type, hasExpiringCALs, isKey) {
			var icon = dojo.create("div");
			var status = dojo.create("span");
            var row = dojo.create("tr", { className: "hasAction"},tbody);
			var nameLink = this._getInfopopLink(type.name, formatCALTypeDescription(type));
			var isDisabled = type.disabledReason != null;
			var hasUnactivatedTrial = type.hasUnactivatedTrial;
			var isExpiredTrial = hasExpiredTrial(type);
			
			//Some properties are stored inside the key only.
			var key = isKey ? type : null;
			if(type.keys && type.keys.length == 1) {
				key = type.keys[0];
			}
			
			if (isExpiredTrial || hasUnactivatedTrial || isDisabled) {
                row.className = "disabledCAL";
				icon.className = "icon-warning";
				dojo.addClass(nameLink,"disabledCAL");
				dojo.addClass(status,"WARNINGcolor");
				if(isExpiredTrial) {
					status.innerHTML = this.messages.expiredTrial;	
				} else if(hasUnactivatedTrial) {
					var durationOfTrial = getExpirationInfo(type.unactivatedTrialExpirationTime).daysRemaining;
					status.innerHTML = dojo.string.substitute(this.messages.trialNotStarted, [durationOfTrial]);
					var activateLink = this._createActivateLink(key || type);
					status.appendChild(activateLink);
				} else {
					status.innerHTML = dojo.string.substitute(this.messages.deactivatedReason, [type.disabledReason]);
				}
            } else {
				icon.className = "icon-ok";
				status.innerHTML = this.messages.activated;
			}

			if(type.editionName) {
				someHaveEditions = true;
			}
			
			var variantString = LIC_P.getVariantString(type) || null; //IE7
            
            // Trial mode of server has no effect on counts for floating
            var limit = getCountOrUnlimited(type.max);
            var included = getCountOrUnlimited(type.free);
            var total = getCountOrUnlimited(type.total);
            var purchase = type.free === -1 ? this.messages.notApplicable : type.total - type.free;

			var productName = dojo.create("div");
			productName.appendChild(document.createTextNode(type.productName || " ")); //IE7
			if (isKey) {
				var productDiv = dojo.create("div", { className: "sub-license-key" });
				productDiv.appendChild(productName);
			}
			
			var issuedColumn;
			if (isKey) {
				issuedColumn = null;
			} else {
				issuedColumn = getIssuedLink(type);
			}
			
			var calVersion = type.version ? dojo.create("span", { innerHTML: type.version }) : null;
			if (calVersion) calVersion._className = "num"; 

            var nodes = [productDiv || productName, calVersion, nameLink, variantString, total, issuedColumn,icon, status];                   																						

		    //Add Expiration column if applicable 
			if(hasExpiringCALs) {
				//If this is a type with multiple keys, don't show a value for expiration on the parent row
				if(isKey || type.keys.length === 1) {
					var expiresOn = type.expirationTime;
					if(expiresOn) {
						var expirationInfo = getExpirationInfo(expiresOn);
						expiresOn = expirationInfo.expirationDateString;
					}	
				}
				nodes.push(expiresOn || "");
			}
			
			var toolbar = new Toolbar();
			this._actionToolbars.push(toolbar);
			if (isKey || type.keys.length == 1) {
				toolbar = this._addDeleteAction(toolbar, type);
			}
			
			if(key && key.hasAgreement) {
				toolbar = this._addViewAgreementAction(toolbar, key);
			}	
			if(toolbar.hasActions()) {
				dojo.addClass(row,"hasAction");
				nodes.push(toolbar.domNode);
			} else {
				nodes.push(null); //IE7
			}
		    dojo.forEach(nodes, function(node){
                col(row, node);
            });
        },
		
		_removeLicense : function(type){
			var id = type.id;
			if (type.keys && type.keys.length == 1)
				id = type.keys[0].id;

			if (!type.floating) {
				this._performRemove(id, type.used);
			} else {				
				var self = this;
				// We must fetch the count of assigned floating licenses
				var fetchHandler = {
					success: function(type){
						self._performRemove(id , type ? type.used : null);					
					},
					failure: function(err){
						self._handleServiceError(err);
					}
				}
				var args = {
					"licenseId": id
				};
				var srh = new SRH(fetchHandler, "success", "failure");
				LIC_P.getContributorLicenseType(srh, args);
			}												
		},
		
		_performRemove : function(id , usedCnt){
			this.header.hideMessage();
			var msg = !usedCnt || usedCnt == 0 ? this.messages.confirmLicenseRemove : this.messages.confirmLicenseRemoveWithAssignments;
			if (confirm(msg)) {
				var self = this;
				var removeHandler = {
					success: function(){
						dojo.publish(com.ibm.team.repository.web.ui.internal.events.licenseChange, []);
						self.reload();						
					},
					failure: function(err){
						self._handleServiceError(err);
					}
				}
				var args = {
					"licenseId": id
				};
				var srh = new SRH(removeHandler, "success", "failure");
				LIC_P.removeLicense(srh, args);
			}
		},
		
		_addDeleteAction: function(toolbar, type){
			if(!type) return toolbar;
			var deleteAction = {};
			deleteAction.imgSrc = dojo.moduleUrl("com.ibm.team.repository.web", "ui/internal/images/delete_hov.gif");
			deleteAction.label = this.messages.removeLicense;
			deleteAction.callbackData = {
				license: type
			};
			var callback = function(e){
				dojo.stopEvent(e);
				this._removeLicense(e.currentTarget.callbackData.license);
			};
			toolbar.addButton(this, callback, deleteAction);
			return toolbar;
		},
		
		_addViewAgreementAction: function(toolbar, type){
			if(!type) return toolbar;
			var viewAction = {};
			viewAction.imgSrc = dojo.moduleUrl("com.ibm.team.repository.web", "ui/internal/admin/server/images/view-license-agreement_co.gif");
			viewAction.label = this.messages.viewAgreement;
			viewAction.callbackData = {
				license: type
			};
			var callback = function(e){
				dojo.stopEvent(e);
				this._showAgreement(e.currentTarget.callbackData.license, /* readOnly */ true);
			};
			toolbar.addButton(this, callback, viewAction);
			return toolbar;
		},

		postActivateTrial: function(typeId, hasAgreed) {
			if(!hasAgreed) return;
			var self = this;
			var fetchHandler = {
				success: function(){	
					dojo.publish(com.ibm.team.repository.web.ui.internal.events.licenseChange, []);
					self.reload();			
				},
				failure: function(err){
					self._handleServiceError(err);
				}
			}
			var args = {
				"licenseId": typeId
			};
			var srh = new SRH(fetchHandler, "success", "failure");
			LIC_P.postActivateTrial(srh, args);
		},
		
        /**
         * Refreshes the Server License information table based on the current value
         * of <code>this.licenseData.serverLicense</code>.
         */
        refreshServerTable: function(tbody,licInfo){
			var serverLicense = licInfo ? licInfo : this.licenseInfo.serverLicense;
			if(!tbody) return;
            removeChildren(tbody);
            var isTrial = serverLicense.trial;
            if (isTrial) {
                var expirationInfo = getExpirationInfo(serverLicense.expirationTime);
            }
            var isDisabled = serverLicense.disabledReason || (isTrial && expirationInfo.isExpired);
            this.serverIsDisabled = isDisabled;
			function row(propRow, className){
				var row = dojo.create("tr", { className: isDisabled ? "disabledServer" : "" }, tbody);
				dojo.create("td", { className: "propColumn", innerHTML: propRow.n }, row);
				var td = dojo.create("td", { className: className ? className : "", innerHTML: propRow.v }, row);
				if(propRow.id) td.id = propRow.id;
		
				var actions = dojo.create("td", { }, row);
				if(propRow.actions) {					
					actions.appendChild(propRow.actions);
					dojo.addClass(row, "hasAction");
				} else {
					actions.appendChild(document.createTextNode(" ")); //IE7
				}
				
		    }
            
            var maxUsers = serverLicense.maxContributors !== -1 ? serverLicense.maxContributors : this.messages.unlimited;
            var usedLicenseText = serverLicense.usedContributors;
            if (this.usedLicenseInfo) {
				var fStatus = this.licenseInfo.floatingSetupStatus;
				if (fStatus.serverURL && fStatus.serverURL != "") {					
	                 var numFloating = this.usedLicenseInfo.countedActiveLeases ? this.usedLicenseInfo.countedActiveLeases : 0;
	                 var activeLabel = dojo.string.substitute(this.messages.acquiredLinkLabel, [ numFloating ]);
	                 var activeFloatingLink = "<a href=\"#action=com.ibm.team.repository.activeFloatingLicenses\">" + activeLabel + "</a>" ;
	                 var assigned = this.usedLicenseInfo.totalUsedCount - numFloating;
	                 var assignedLabel = dojo.string.substitute(this.messages.assignedLinkLabel, [ assigned ]);
	         //      var assignedLink = "<a href=\"#action=com.ibm.team.repository.manageUserCALs\">" + assignedLabel + "</a>" ;
	                 usedLicenseText = this.usedLicenseInfo.totalUsedCount + " (" + assignedLabel + ", " + activeFloatingLink + ")";
	            }
                 
                if (findOperationCheckResults(this.licenseInfo, FLOATING_SUPPORT_OPERATION_ID) == false) {
					var warningSection = "<span class=\"inline-message WARNINGcolor\">" + this.messages.noFloatSupport + "</span>";
					usedLicenseText = usedLicenseText + warningSection;
				}  
            }
            
            dojo.toggleClass(this.floatingAllowedContent, "noDisplay", !this.usedLicenseInfo);
			dojo.toggleClass(this.floatingNotAllowedContent, "noDisplay", this.usedLicenseInfo);
            
            var isFloatingServer = dojo.getObject("licenseInfo.floatingSetupStatus.server", false, this);
            if (this.usedLicenseInfo && isFloatingServer) {
                dojo.publish(com.ibm.team.repository.web.ui.internal.events.floatingLicenseChange, [{
                    actingAsFloating: true
                }]);
            }
            
            var rows = [{
                n: this.messages.serverProduct,
                v: serverLicense.productName
            }, {
                n: this.messages.serverDescription,
                v: serverLicense.description
            }, {
                n: this.messages.maximumUsers,
                v: maxUsers
            }, {
                n: this.messages.countedUsers,
                v: usedLicenseText
            }];
            dojo.forEach(rows, function(propRow){
                row(propRow);
            });
            
            var className = null;
            if (serverLicense.disabledReason) {
				row({
					id: "serverStatusField",
	                n: this.messages.serverStatus,
	                v: serverLicense.disabledReason
	            }, isDisabled ? "expired" : null);
            }

            if (isTrial) {
                if (expirationInfo.isExpired) {
                    var rowObj = {
                        n: this.messages.expiredOn,
                        v: expirationInfo.expirationDateRelative
                    };
                }
                else {
                    var rowObj = {
                        n: this.messages.expiresOn,
                        v: expirationInfo.expirationDateRelative
                    };
                }
                row(rowObj, expirationInfo.isExpired ? "expired" : null);
            }
			
			//Add property for floating license information
			
			if (!licInfo && this.licenseInfo.floatingSetupStatus) { // i.e. not used for preview table in wizard
			
				var licenseInfo = this.licenseInfo;
				var status;
				var isFServerStatus;
				
	
				if (licenseInfo.floatingSetupStatus.server === true) {
					isFServerStatus = this.messages.configuredServer;
					var n = licenseInfo.floatingLicenseTypes.length;
					//Different message to adjust for singular/plural case
					var msg = (n==1) ? this.messages.configuredServerOne : this.messages.configuredServer;
					isFServerStatus = dojo.string.substitute(msg, [ n ]); 
				} else {
					isFServerStatus = this.messages.notConfiguredServer;
				}	

				if (licenseInfo.floatingSetupStatus.client === true) {
					var licenseServerLink = "<a href=\"" + licenseInfo.floatingSetupStatus.serverURL + "\">" + licenseInfo.floatingSetupStatus.serverURL + "</a>"
					status = dojo.string.substitute(this.messages.configuredClient, [ licenseServerLink ]); 
					if (licenseInfo.floatingSetupStatus.connectionError) {
						status = status + "<div style=\"margin: 5px 0;color:#F00\">" + this.messages.licenseServerError + licenseInfo.floatingSetupStatus.connectionError + "</div>";
					}
				} else {
					status = this.messages.notConfiguredClient; 
				}
				
				var toolbar = new Toolbar();
				this._actionToolbars.push(toolbar);
				var editAction = {};
				editAction.imgSrc = dojo.moduleUrl("com.ibm.team.repository.web", "ui/internal/images/edit-ellipses.gif");
				editAction.label = this.messages.modifyFloatingLicenseServer;
				var callback = function(e){
					dojo.stopEvent(e);
					this.launchFloatingConfig();
				};
				toolbar.addButton(this, callback, editAction);
					
				row({
	                id: "floatingClientStatus",
					n: this.messages.floatingServer,
	                v: status,
					actions: toolbar.domNode
	            });
				
				if(dojo.byId('floatingClientStatus')) {
					var divElem = document.createElement("div");
					divElem.innerHTML = isFServerStatus;
					dojo.byId('floatingClientStatus').appendChild(divElem);
				}
			}
				
			//get the default license id and then display it.
			var srh = new SRH(this,"getDefaultLicense","_handleServiceError");
			LIC_P.getDefaultLicenseIds(srh);
        },
		
		getDefaultLicense: function(defaultCALs) {
			this.defaultLicenses = defaultCALs
			var displayName = "";
			if(this.defaultLicenses && this.defaultLicenses.length > 0){
				dojo.forEach(this.defaultLicenses, function(id) {
					displayName +=  LIC_P.getLicenseNameFromID(this.licenseInfo, id) + ", ";
				}, this);
				displayName = displayName.slice(0,-2); //remove trailing comma
			} else {
				displayName = this.messages.noDefault;
			}
			
			var toolbar = new Toolbar();
			this._actionToolbars.push(toolbar);
			var editAction = {};
			editAction.imgSrc = dojo.moduleUrl("com.ibm.team.repository.web", "ui/internal/images/edit-ellipses.gif");
			editAction.label = this.messages.setDefaultAction;
			var callback = function(e){
				dojo.stopEvent(e);
				this.launchDefaultWizard();
			};
			var editButton = toolbar.addButton(this, callback, editAction);
			if(!this.licenseInfo.types && !this.defaultLicenses) {
				toolbar.disableButton(editButton);
			} else {
				toolbar.enableButton(editButton);
			}

			var row = dojo.create("tr", { className: this.serverIsDisabled ? "disabledServer hasAction" : "hasAction"  }, this.serverInfoTable);
			dojo.create("td", { className: "propColumn", innerHTML: this.messages.defaultLicenseLabel }, row);
			dojo.create("td", { id: "defaultLicenseField", innerHTML: displayName }, row);
			var actions = dojo.create("td", { className: "actions" }, row);
			actions.appendChild(toolbar.domNode);

			//Check to see if the default license key is still valid. (Does this license exist?)
			var isValid = this.validateDefaultLicense();
			if(!isValid) {
				this.header.showMessage({
	                type: STATUS_TYPES.ERROR,
	                summary: dojo.string.substitute(this.messages.defaultLicenseError, [displayName])
	            });
				var node = dojo.byId('defaultLicenseField');
				if(node) {
					dojo.create("div", { className: "propError", innerHTML: this.messages.invalidLicenseId }, node);
				}
			}
		},
		
		validateDefaultLicense: function() {
			if(this.defaultLicenses && this.defaultLicenses.length > 0) {
				var types = this.licenseInfo.types || [];
				for(cal in this.defaultLicenses) {
					var inArray = dojo.some(types, function(type){ return type.id === this.defaultLicenses[cal]; }, this);	
					if(!inArray) {
						return false;
					}
				}
			}
			return true;
		},
		
		_destroyActionToolbars: function() {
			for (var i = 0; i < this._actionToolbars.length; i++) {
				this._actionToolbars[i].destroyRecursive();
			}
			this._actionToolbars = [];
		},
		
		_destroyEditorWizards: function() {
			if (this.wizard) {
				this.wizard.destroyRecursive();
				delete this.wizard;
			}
		},
		
		uninitialize: function() {
			this._destroyActionToolbars();
			this._destroyEditorWizards();
			this.header.destroyRecursive();
			this._pageHelpHover.destroyRecursive();
			if(this._tokenProvidersView) {
				this._tokenProvidersView.destroyRecursive();
			}
			this.inherited(arguments);
		}
		
    });
	    
    //
    // Utility functions
    //
    
    var d = document;
    d.ce = document.createElement;
    
    function clearMessageTimeout(ctxObj, timeoutProp){
        var timeoutObj = ctxObj[timeoutProp];
        clearTimeout(timeoutObj);
        ctxObj[timeoutProp] = null;
    }

    function col(row, node){
        if (node === null || node === undefined) {
            var td = dojo.create("td", {}, row);
			td.appendChild(document.createTextNode(" ")); //IE7
			return;
        }
		if(dojo.isString(node) && node.indexOf("null") != -1) {
			node = node.replace("null","");
		}
        var td = d.ce("td");
        var isNumber = typeof node == 'number' || node instanceof Number;
        var isNumberCol = isNumber || node === messages.notApplicable || node === messages.unlimited;
        if (isNumberCol) {
            td.className = "num";
        }
        if (isNumber) {
            node = dojo.number.format("" + node);
        }
		if(node._className) {
			//Allow the node to force it's className
			dojo.addClass(td, node._className);
		}
        var isArray = !dojo.isString(node) && dojo.isArrayLike(node);
        if (node.tagName === "A" && node.className === "num") {
            td.className = "num";
        }
        else if (isArray) {
            td.className = "num inputCol";
        }
        row.appendChild(td);
        var nodeArr = isArray ? node : [node];
        dojo.forEach(nodeArr, function(node){
            if (dojo.isString(node)) {
                node = document.createTextNode(node);
            }
            td.appendChild(node);
        });
    }
    
    function getCountOrUnlimited(count){
        return count === -1 ? messages.unlimited : count;
    }
    
    function getCountString(count){
        return count === -1 ? messages.unlimited : (dojo.number.format(count));
    }
    
    function getUsedLink(calType, isKey){
		if (isKey) {
			return this.messages.notApplicable;
		}
        if (calType.used === 0) {
            return 0;
        }
        else {
            var a = document.createElement("a");
            a.href = "#action=com.ibm.team.repository.manageUserCALs&licenseType=" + calType.id;
            a.innerHTML = calType.used;
            a.className = "num";
            return a;
        }
    }
    
    function getIssuedLink(calType){
        var a = document.createElement("a");
        a.href = "#action=com.ibm.team.repository.admin.floatingLicenseUsage&licenseId=" + calType.id;
        a.innerHTML = messages.viewIssued;
        return a;
    }

    function findOperationCheckResults(licenseInfo, operationId){
        var result = null;
        if (licenseInfo.operationCheckResults) {
            dojo.forEach(licenseInfo.operationCheckResults, function(operationResult){
                if (operationResult.operationId == operationId) {
                    result = operationResult.allowed;
                }
            })
        }
        return result;
    }
    
})();
