/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010, 2012. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  
 * Use, duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp. 
 *******************************************************************************/
(function() {
	
dojo.provide("com.ibm.team.repository.web.ui.internal.admin.server.LicenseTokenUsageView");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dojox.xml.parser");
dojo.require("net.jazz.ajax.core.internal.util");
dojo.require("com.ibm.team.repository.web.ui.internal.admin.ItemList");
dojo.require("com.ibm.team.repository.web.ui.internal.admin.util");
dojo.require("jazz.ui.MessageBox");
dojo.require("dojo.i18n");
dojo.require("com.ibm.team.repository.web.client.internal.ExternalLicenseClient");
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
	
dojo.requireLocalization("com.ibm.team.repository.web.ui.internal.admin.server", "LicenseTokenViewMessages");
var messages;
var MessageBox = jazz.ui.MessageBox;
var STATUS_TYPES = MessageBox.STATUS_TYPES;
var TokenClient = com.ibm.team.repository.web.client.internal.ExternalLicenseClient;
var SRH = com.ibm.team.repository.web.transport.ServiceResponseHandler;

dojo.declare("com.ibm.team.repository.web.ui.internal.admin.server.LicenseTokenUsageView", com.ibm.team.repository.web.ui.internal.admin.ItemList, {

    constructor: function(){        
        this.messages = messages = dojo.i18n.getLocalization("com.ibm.team.repository.web.ui.internal.admin.server", "LicenseTokenViewMessages");
		this.GROUPED_ROW_HEADER = [this.messages.licenseId, this.messages.licenseCount, this.messages.tokenCostPer, this.messages.totalTokenCost];
		this.UNGROUPED_ROW_HEADER = [this.messages.licenseId, this.messages.tokenCount , this.messages.expirationDate , this.messages.leaseId];
    },
    
    postCreate: function(){
		this.header = new MessageBox();
        this.headerArea.appendChild(this.header.domNode);
		this.heading.innerHTML = this.messages.usageHeading;
				
		this.statusArea = document.createElement("div");
        this.newItemButtonArea.appendChild(this.statusArea);
		
		this.optionsArea = document.createElement("div");
		this.optionsArea.setAttribute("class" , "filter noDisplay");
		this.newItemButtonArea.appendChild(this.optionsArea);
		
		var label = document.createElement("label");
		label.setAttribute("for" , "groupByCheckbox");
		label.setAttribute("style" , "float:right;");
		label.appendChild(document.createTextNode(this.messages.groupByLicense));
		this.optionsArea.appendChild(label);
		
		this.groupByButton = document.createElement("input");
		this.groupByButton.setAttribute("type" , "checkbox");	
		this.groupByButton.setAttribute("id" , "groupByCheckbox");
		this.groupByButton.setAttribute("style" , "float:right;");
		this.groupByButton.checked = true;
		this.connect(this.groupByButton, "onclick", "_onGroupBy");
		this.optionsArea.appendChild(this.groupByButton);
		
        this._refresh();
		dojo.subscribe("/token/changed" , this , "tokenChangeListener");
    },
    
 	tokenChangeListener : function(message){
		this._refresh();
	},
        
	_onGroupBy : function(){
		this._refresh();
	},
	
    _displayProviderDetails: function(provider){
        this.providerDetail = provider;
		dojox.xml.parser.removeChildren(this.tbody);
		dojox.xml.parser.removeChildren(this.statusArea);
        var label = document.createElement("label");
        label.innerHTML = this.messages.availTokens + this.providerDetail.freeTokens + " " + this.messages.tokensUsed + " " + this.providerDetail.usedTokens;
        this.statusArea.appendChild(label);

		var headers = this._isGroupByLicense() ? this.GROUPED_ROW_HEADER : this.UNGROUPED_ROW_HEADER;
		this.setHeaderRow(headers);
        dojo.toggleClass(this.optionsArea , "noDisplay" , true);        
        var self = this;
        if (provider.usage) {
			dojo.toggleClass(this.optionsArea , "noDisplay" , false);             
            dojo.forEach(provider.usage, function(usageDetail){				
				var licenseId = usageDetail.licenseId;				
				var numTokens = usageDetail.numTokens;							
				var row = null;
				if (self._isGroupByLicense()) {
					var numLicenses = usageDetail.numLicenses;
					var costPer = numTokens / numLicenses;
					row = [licenseId, numLicenses , costPer.toString() , numTokens];
				} else {					
					row = [licenseId, numTokens , usageDetail.expiration , usageDetail.uuid];
				}	   
				                             
                var row = self.addRow(null, row);
                dojo.toggleClass(row.parentNode, "userTable", true);
            });
			
        } else{
			var tr = document.createElement("tr");			
            var td = document.createElement("td");
            td.appendChild(document.createTextNode(this.messages.noLeases));
            td.className = "noRowsYet";
            td.colSpan = headers.length;
            tr.appendChild(td);
            this.tbody.appendChild(tr);
		}
        
    },
    
	_isGroupByLicense : function(){
		return this.groupByButton.checked;
	},
	    
    _refresh: function(){
 //       TokenClient.getJazzTokenServiceInfo(new SRH(this, "_displayProviderDetails", "handleError"), true , this._isGroupByLicense());		
    }
	
});

})();