/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2011, 2012. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  
 * Use, duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp. 
 *******************************************************************************/
(function() {
	
dojo.provide("com.ibm.team.repository.web.ui.internal.admin.server.ServerDiagnostics");

dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
dojo.require("com.ibm.team.repository.web.client.internal.ServerStatusClient");
dojo.require("net.jazz.ajax.contextRoot");
dojo.require("jazz.ui.MessageBox");
dojo.require("jazz.ui.HoverPopup");
dojo.require("jazz.app.ua.TextHoverHelp");
dojo.require("dojo.i18n");
dojo.require("jazz.app.i18n");
dojo.require("dojo.date");
dojo.require("jazz.ui.Toolbar");
dojo.require("jazz.ui.toolbar.Button");
dojo.require("jazz.ui.toolbar.Separator");
dojo.requireLocalization("com.ibm.team.repository.web.ui.internal.admin.server", "ServerDiagnosticsMessages");

var MessageBox = jazz.ui.MessageBox;
var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;
var ServerStatusClient = com.ibm.team.repository.web.client.internal.ServerStatusClient;
var STATUS_OK_IMG_SRC = dojo.moduleUrl("com.ibm.team.repository.web", "graphics/icons/okay_obj.gif");
var STATUS_ERR_IMG_SRC = dojo.moduleUrl("com.ibm.team.repository.web", "graphics/icons/error_obj.gif");
var STATUS_WARN_IMG_SRC = dojo.moduleUrl("com.ibm.team.repository.web", "graphics/icons/warn_obj.gif");
var STATUS_RUNNING_IMG_SRC = dojo.moduleUrl("com.ibm.team.repository.web", "graphics/icons/running_obj.gif");
var TextHoverHelp = jazz.app.ua.TextHoverHelp;
var DIAGNOSTIC_EMAIL_SERVICE = "com.ibm.team.repository.service.internal.diagnostic.DiagnosticEmailService";
var MAILER_SERVICE = "com.ibm.team.repository.service.jts.internal.mailer.MailerService";
var ADMIN_EMAIL_LIST_PROPERTY = "diagnostics.adminEmailList";
var ENABLE_EMAIL_NOTIFICATIONS_PROPERTY = "notification.mail.enabled";

dojo.declare("com.ibm.team.repository.web.ui.internal.admin.server.ServerDiagnostics", [dijit._Widget, dijit._Templated], {

	templatePath: dojo.moduleUrl("com.ibm.team.repository.web", "ui/internal/admin/server/templates/ServerDiagnostics.html"),
	widgetsInTemplate: true,
	_helpHovers: [],

	constructor: function(args) {
		this.messages = messages = dojo.i18n.getLocalization("com.ibm.team.repository.web.ui.internal.admin.server", "ServerDiagnosticsMessages");
		if(args.nodeId) {
			this.nodeId = args.nodeId;
		}
	},

	postCreate: function() {
		this.header = new MessageBox();	
		this.headerArea.appendChild(this.header.domNode);

		this.diagnosticTestIDs = new Array();
		this.diagnosticResultsDivs = new Array();
		this.pendingResultCount = 0;
		this.someResultsExist = false;
		
		this.btnRunAll.onClick = dojo.hitch(this, "_runAllTests");
		this.btnExportAll.onClick = dojo.hitch(this, "_performExport");
		this.btnEmailReport.onClick = dojo.hitch(this, "_emailReport");
		
		//Disable email functionality until we can verify that it is configured properly
		this.btnEmailReport.set("enabled", false);
		this.enableEmailReport = false;
		this._doesServerSupportEmail();
		this.checkClusterState();
	},
	
	checkClusterState: function() {
		var state = dojo.getObject("com.ibm.team.repository.isInCluster");
		if (state === true) {
			this.isCluster = true;
			this.getClusterInformation();
		} else if (state === false) {
			this.isCluster = false;
			this._fetchTests();
		} else { //not set, usually happens when user refreshes this page directly
			this._getClusterStatus();
		}
	},
	
	_getClusterStatus: function() {
		var handler = {
			success: dojo.hitch(this, function(response) {
				dojo.setObject("com.ibm.team.repository.isInCluster", response === "false" ? false : true);
				this.checkClusterState();
			}),
			error: dojo.hitch(this, function() {
				dojo.setObject("com.ibm.team.repository.isInCluster", false);
				console.error("Unable to determine if the server is running in a cluster. Defaulting to standalone mode.");
				this.checkClusterState();	
			})
		};
		var srh = new ServiceResponseHandler(handler, "success", "error");
		ServerStatusClient.getIsInCluster(srh);
	},
	
	getClusterInformation: function() {
		var srh = new ServiceResponseHandler(this, "handleClusterInfo", "handleClusterError");
        ServerStatusClient.getClusterMembers(srh);
	},
		
	handleClusterInfo: function(nodelist) {
		this.clusterNodes = nodelist || [];
		this._fetchTests();
	},
	
	handleClusterError: function(error) {
		console.error("Unable to get Cluster status. Assuming no cluster.");
		this._fetchTests();
	},

	_fetchTests: function() {
		this.diagnosticTestIDs = [];
		var srh = new ServiceResponseHandler(this, "_displayTests", "_handleError");
		ServerStatusClient.getAllDiagnosticResults(srh);
	},
	
	_displayTests: function(tests) {
		dojo.forEach(tests, function(test){
			this.pendingResultCount++;
			this.diagnosticTestIDs.push(test.id);
			this._createDescriptionSection(test);
		}, this);
	},
	
	_createDescriptionSection: function(test) {
		if(!this.testContainer) return;
		
		//Create and store an object to keep track of DOM handles for this specific test.
		var handles = {};
		this.diagnosticResultsDivs[test.id] = handles;
		
		var div = dojo.create("div", { className: "com-ibm-team-repository-admin-table" });
		var headingDiv = dojo.create("div", { className: "heading" }, div);
		handles._headingDiv = headingDiv;

		//Create Status/Actions bar
		var detailsDiv = dojo.create("div", { className: "details" }, div);	
		var statusBar = dojo.create("div", { className: "statusBar" }, detailsDiv);

		//Create Enable Link
		handles._enableLink = dojo.create("a", { className: "action-link disabled", innerHTML: this.messages.enable }, headingDiv);
		this.connect(handles._enableLink,"onclick", function(e) {
			dojo.stopEvent(e);
			this._processEnableDisableRequests(test, /* disable */ false);
		});

		//Create Disable Link
		handles._disableLink = dojo.create("a", { className: "action-link", innerHTML: this.messages.disable }, headingDiv);
		handles._disableBar = dojo.create("span", { className: "separator", innerHTML: "|" }, headingDiv);
		this.connect(handles._disableLink,"onclick", function(e) {
			dojo.stopEvent(e);
			this._processEnableDisableRequests(test, /* disable */ true);
		});

		//Create Cancel Link
		handles._cancelLink = dojo.create("a", { className: "action-link disabled", innerHTML: this.messages.cancel }, headingDiv);
		this.connect(handles._cancelLink,"onclick", function(e) {
			dojo.stopEvent(e);
			//Check to see if it's disabled.
			if(dojo.hasClass(handles._cancelLink, "disabled") === true) {
				return;
			}
			handles._lastExecuted.innerHTML = this.messages.cancelling;
			handles._resultsDiv.innerHTML = "";
			dojo.addClass(handles._statusIcon, "noDisplay");
			dojo.addClass(handles._cancelLink, "disabled");
			dojo.removeClass(handles._disableLink, "disabled");
			dojo.removeClass(handles._disableBar, "disabled");
			var srh = new ServiceResponseHandler(this, "_cancelFinished", "_handleError");
			this.justCancelledID = test.id;
			ServerStatusClient.postCancelDiagnostic(srh, test.id);
		});

		//Create Run Link
		handles._runLink = dojo.create("a", { className: "action-link", innerHTML: this.messages.run }, headingDiv);
		this.connect(handles._runLink,"onclick", function(e) {
			dojo.stopEvent(e);
			//Check to see if it's disabled
			if(dojo.hasClass(handles._runLink, "disabled") === true) {
				return;
			}
			this._processRunRequest(test.id);
		});

		var titleSpan = dojo.create("span", { innerHTML: test.title }, headingDiv);
		var descriptionHover = new TextHoverHelp({
			helpContentString: test.description,
			contentNode: titleSpan
		});
		this._helpHovers.push(descriptionHover);
		
		handles._statusIcon = dojo.create("img", { className: "icon noDisplay" }, statusBar);
		handles._lastExecuted = dojo.create("span", { innerHTML: this.messages.loading }, statusBar);
		
		//Create results section
		handles._resultsDiv = dojo.create("div", { className: "results" }, detailsDiv);
		this.testContainer.appendChild(div);
		
		//Now, update the test w/ the results from last run
		this._updateResultsDiv(test);
		
		//See if this test is currently running (background process or another web ui)
		if(test.currentProgress) {
			//Export & Email buttons should be disabled if some tests are running
			this.btnExportAll.set("enabled", false);
			this.btnEmailReport.set("enabled", false);
			
			this.pendingResultCount++;
			var srh = new ServiceResponseHandler(this, "_updateResultsDiv", "_handleError");
			ServerStatusClient.getPreviousDiagnosticResult(srh, test.id);
			this._setRunningStatus(test.id);
		}
		
		//Default UI state is "Enabled", so we only need to call this if the test is disabled
		if(test.disabled && !test.currentProgress) {
			this._setDisabledStatus(test, true);
			handles._disabled = test.disabled;
		}
		
		//If we highlighted some of the tests as a result of a nodeId in the URL, let's tell the user
		if(this.someTestsHighlighted) {
			this.header.showMessage({ 
				type: "INFO",
				closable: true,
				summary: dojo.string.substitute(this.messages.someTestsHighlighted, [ this.nodeId ])
			});	
		}
	},
	
	_runAllTests: function() {
		this.btnRunAll.set("enabled", false);
		this.btnExportAll.set("enabled", false);
		this.btnEmailReport.set("enabled", false);
		this.pendingResultCount = 0;
		this.runAllInProgress = true;
		//TODO: Probably need a way to restrict the number of "Running" tests. 
		dojo.forEach(this.diagnosticTestIDs, function(id) {
			this._processRunRequest(id);
		}, this);
		this.header.showMessage({ 
			type: "INFO",
			closable: true,
			summary: this.messages.runningAllTests
		});
	},
	
	_processRunRequest: function(id) {
		//Check to see if this diagnostic is disabled
		var test = this.diagnosticResultsDivs[id];
		if(test && test._disabled === true) return;
		
		this.pendingResultCount++;
		var srh = new ServiceResponseHandler(this, "_updateResultsDiv", "_handleError");
		ServerStatusClient.getNewDiagnosticResult(srh, id);
		this._setRunningStatus(id);	
	},
	
	//If disable = true, the test gets disabled
	//If disable = false, the test gets enabled
	_processEnableDisableRequests: function(test, disable) {
		var handler = {
			success: dojo.hitch(this, function() {
				this._setDisabledStatus(test, disable);
			}),
			error: dojo.hitch(this, "_handleError")
		};
		var srh = new ServiceResponseHandler(handler, "success", "error");
		if(disable) {
			ServerStatusClient.postDisableDiagnostic(srh, test.id);
		} else {
			ServerStatusClient.postEnableDiagnostic(srh, test.id);
		}
	},
	
	_setRunningStatus: function(id) {
		var elem = this.diagnosticResultsDivs[id];
		elem._lastExecuted.innerHTML = this.messages.running;
		elem._resultsDiv.innerHTML = "";
		this._setStatusIcon(elem._statusIcon, "running");
		dojo.removeClass(elem._cancelLink, "disabled");
		dojo.addClass(elem._runLink, "disabled");
		dojo.addClass(elem._disableLink, "disabled");
		dojo.addClass(elem._disableBar, "disabled");
		this.btnExportAll.set("enabled", false);
		this.btnEmailReport.set("enabled", false);
	},
	
	_setDisabledStatus: function(test, isDisabled) {
		var elem = this.diagnosticResultsDivs[test.id];
		elem._disabled = isDisabled;
		if (isDisabled) {
			dojo.addClass(elem._disableLink, "disabled");
			dojo.addClass(elem._disableBar, "disabled");
			dojo.removeClass(elem._enableLink, "disabled");
			dojo.addClass(elem._statusIcon, "noDisplay");
			dojo.addClass(elem._resultsDiv, "noDisplay");
			dojo.addClass(elem._runLink, "disabled");
			elem._savedExecutionTime = elem._lastExecuted.innerHTML; //Needed to keep the correct UI state w/o refreshes
			elem._lastExecuted.innerHTML = this.messages.disabled;
		} else {
			dojo.addClass(elem._enableLink, "disabled");
			dojo.removeClass(elem._disableLink, "disabled");
			dojo.removeClass(elem._disableBar, "disabled");
			if(elem._statusIcon.src != ""){
				dojo.removeClass(elem._statusIcon, "noDisplay"); //only unhide if there's an icon there.	
			}
			dojo.removeClass(elem._resultsDiv, "noDisplay");
			dojo.removeClass(elem._runLink, "disabled");
			if(elem._savedExecutionTime) {
				elem._lastExecuted.innerHTML = elem._savedExecutionTime;
			} else {
				elem._lastExecuted.innerHTML = this.messages.lastExecuted + ": " + this.getExecutionTime(test.executionTime);
			}
		}
	},
	
	getExecutionTime: function(time) {
		if(!time) {
			return this.messages.never;
		}
		var executionTime = dojo.date.stamp.fromISOString(time);
		return executionTime = jazz.app.i18n.date.locale.format(new Date(executionTime));
	},
	
	_cancelFinished: function() {
		var id = this.justCancelledID;
		this.justCancelledID = null;
		var elem = this.diagnosticResultsDivs[id];
		dojo.removeClass(elem._runLink, "disabled");
		
		//Load up the previous result
		var srh = new ServiceResponseHandler(this, "_updateResultsDiv", "_handleError");
		ServerStatusClient.getPreviousDiagnosticResult(srh, id);
	},

	_setStatusIcon: function(image, status) {
		if(!image) return;
		switch (status) {
			case "error": 
				image.src = STATUS_ERR_IMG_SRC;
				image.alt = this.messages.status_error;
				break;
			case "warning": 
				image.src = STATUS_WARN_IMG_SRC;
				image.alt = this.messages.status_warning;
				break;
			case "success": 
				image.src = STATUS_OK_IMG_SRC;
				image.alt = this.messages.status_ok;
				break;
			case "running": 
				image.src = STATUS_RUNNING_IMG_SRC;
				image.alt = this.messages.running;
				break;				
			default: 
				//Set the default to OK
				image.src = STATUS_OK_IMG_SRC;
				image.alt = this.messages.status_ok;
		}
		dojo.removeClass(image, "noDisplay");
	},
	
	_updateResultsDiv: function(result) {
		if(!result || !this.testContainer) return; // DOM has been destroyed already
		this.pendingResultCount--;
		var id = result.id;
		var duration = result.executionDuration;		
		var elem = this.diagnosticResultsDivs[id];
		
		//Update Action Links
		dojo.addClass(elem._cancelLink, "disabled");
		dojo.removeClass(elem._runLink, "disabled");
		dojo.removeClass(elem._disableLink, "disabled");
		dojo.removeClass(elem._disableBar, "disabled");
		
		var status = "success";
		if(result.warning === true) {
			status = "warning";
		}
		if(result.error === true) {
			status = "error";
		}
		
		if(result.nodesResults) {
			var uniqueStatuses = [];
			dojo.forEach(result.nodesResults, function(nodeStatus) {
				uniqueStatuses = this.addOrUpdateNodeResult(uniqueStatuses, nodeStatus);
			}, this);
			//Display all of the unique statuses
			dojo.forEach(uniqueStatuses, function(item) {
				if(item.status) {
					var resultDiv = dojo.create("div", { className: "result-status", innerHTML: item.status }, elem._resultsDiv);
					if(this.isCluster && result.nodeScopedTest) {
						var div = dojo.create("div", { className: "nodes-effected", innerHTML: this.displayNodesEffected(item.nodes) }, resultDiv);	
					}
					if(this.nodeId) {
						//If this status includes our scoped node, the test is enabled, and it's an error or warning, we should highlight it
						var includesScopedNode = !result.nodeScopedTest || dojo.indexOf(item.nodes, this.nodeId) >= 0;
						var doHighlight = !result.disabled && (item.error || item.warning) && includesScopedNode;
						dojo.toggleClass(elem._headingDiv, "highlight", doHighlight);
						if(doHighlight) this.someTestsHighlighted = true;
					}
				}
			}, this);
		
		} else if(result.status) {
			elem._resultsDiv.innerHTML = result.status;
		}
		if (result.executionTime) {
			this.someResultsExist = true;
			this._setStatusIcon(elem._statusIcon, status);
		}
		elem._lastExecuted.innerHTML = this.messages.lastExecuted + ": " + this.getExecutionTime(result.executionTime);
		
		if(this.pendingResultCount == 0 && this.someResultsExist) {
			this.btnExportAll.set("enabled", true);
			this.btnEmailReport.set("enabled", this.enableEmailReport);
		}
		
		if(this.pendingResultCount == 0 && this.runAllInProgress) {
			this.runAllInProgress = false;
			this.btnRunAll.set("enabled", true);
			this.header.showMessage({ 
				type: "OK",
				closable: true,
				summary: this.messages.allTestsCompleted
			});
		}
	},
	
	displayNodesEffected: function(nodes) {
		if(!nodes) return "";
		var str = "";
		dojo.forEach(nodes, function(node) {
			str += node + ", ";
		});
		return str.slice(0,-2); //remove trailing comma
	},
	
	addOrUpdateNodeResult: function(uniques, newResult) {
		var matchFound = false;
		for (var key in uniques) {
			if(uniques[key].status === newResult.status) {
				//We already have this status, just add our nodeID
				uniques[key].nodes.push(newResult.nodeID);
				matchFound = true;
				break;
			} 
		}
		if(!matchFound) {
			//We do not have this status yet, add the full status with a new nodes[] property
			newResult.nodes = [newResult.nodeID];
			uniques.push(newResult);		
		}
		return uniques;
	},

	_performExport: function() {
		var contextRoot = net.jazz.ajax.contextRoot();
		var baseUrl = window.location.protocol + "//" + window.location.hostname + ":" + window.location.port + contextRoot;
		window.location.href= baseUrl + "/service/com.ibm.team.repository.service.internal.diagnostic.IDiagnosticDetailedResultsService";
	},
	
	_isAdminEmailConfigured: function() {
		var hasAdminEmail = false;
		var handler = {
			success: dojo.hitch(this,function(serviceInfo) {
				if(!this.domNode) return;
				if (serviceInfo.properties && serviceInfo.properties.length > 0) {
					dojo.forEach(serviceInfo.properties, function(prop){
						if (prop.name == ADMIN_EMAIL_LIST_PROPERTY && prop.overrideValue) {
							hasAdminEmail = true;
							this.btnEmailReport.set("enabled", true);
							this.enableEmailReport = true;
							return;
						}
					}, this);
					if(!hasAdminEmail) {
						this.enableEmailReport = false;
						this._helpHovers.push(new jazz.ui.HoverPopup({
							around:  this.btnEmailReport.domNode,
							html: this.messages.adminEmailNotSet,
							openDelay: 300,
							closeDelay: 300
						}));
					}
				}
			}),
			error: function(errorObj) {
				//don't surface the error
			}
		};
		var srh= new ServiceResponseHandler(handler, "success", "error");
		ServerStatusClient.getServiceConfiguration(srh, DIAGNOSTIC_EMAIL_SERVICE);		
	},
	
	_doesServerSupportEmail: function() {
		var hasEmailEnabled = false;
		var handler = {
			success: dojo.hitch(this,function(serviceInfo) {
				if(!this.domNode) return;
				if (serviceInfo.properties && serviceInfo.properties.length > 0) {
					dojo.forEach(serviceInfo.properties, function(prop){
						if (prop.name == ENABLE_EMAIL_NOTIFICATIONS_PROPERTY && prop.overrideValue === "true") {
							hasEmailEnabled = true;
							this._isAdminEmailConfigured();
							return;
						}
					}, this);
					if(!hasEmailEnabled) {
						this.enableEmailReport = false;
						this._helpHovers.push(new jazz.ui.HoverPopup({
							around:  this.btnEmailReport.domNode,
							html: this.messages.emailDisabled,
							openDelay: 300,
							closeDelay: 300
						}));
					}
				}
			}),
			error: function(errorObj) {
				//don't show error
			}
		};
		var srh= new ServiceResponseHandler(handler, "success", "error");
		ServerStatusClient.getServiceConfiguration(srh, MAILER_SERVICE);		
	},

	_emailReport: function() {	
		var srh = new ServiceResponseHandler(this, "_emailReportSent", "_handleError");
		ServerStatusClient.getSendEmailReport(srh);		
	},
	
	_emailReportSent: function() {
		this.header.showMessage({ 
			type: "OK",
			closable: true,
			summary: this.messages.emailReportSentOK
		});
	},
	
	_handleError: function(errorObj) {
		this.header.showMessage({
			errorObject: errorObj
		});
	},
	
	_destroyHelpHovers: function(){
		for (var i = 0; i < this._helpHovers.length; i++) {
			this._helpHovers[i].destroyRecursive();
		}
		this._helpHovers = [];
	},
	
	uninitialize: function(){	
		this._destroyHelpHovers();
		this.header.destroyRecursive();
		this.inherited(arguments);
	}

});

})();
