/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010, 2012. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  
 * Use, duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp. 
 *******************************************************************************/
dojo.provide("com.ibm.team.repository.web.ui.internal.admin.users.SimpleTab");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");

(function() {

dojo.declare("com.ibm.team.repository.web.ui.internal.admin.users.SimpleTab", [dijit._Widget, dijit._Templated], {

	tabID: null,
	tabTitle: "",
	callbackGetNode: null,
	callbackOnClick: null,
	contentNode: null,

	templatePath: dojo.moduleUrl("com.ibm.team.repository.web", "ui/internal/admin/users/templates/SimpleTab.html"),
	
	_static: function() {}, 	

	// Public Methods //------------------------------------------------------->

	constructor: function(args) {
	},
	
	postCreate: function() {
		this._initializeStaticVariables();
		this._initializeWidget();
	},
	
	destroy: function() {
		delete this.tabTitle;
		delete this.tabID;
		this.inherited(arguments);
	},
	
	select: function() {
		if (!this.contentNode &&
			this.callbackGetNode) {
			this.contentNode = this.callbackGetNode(this.tabID);
		}
		if (!this.contentNode) {
			this._logError(new Error("Undefined tab content node"));
			return;
		}
		dojo.addClass(this._tab, "selectedTab");
		this._updateVisibility(this.contentNode, true);
	},
	
	deselect: function() {
		if (!this.contentNode) {
			this._logError(new Error("Undefined tab content node"));
			return;
		}
		dojo.removeClass(this._tab, "selectedTab");
		this._updateVisibility(this.contentNode, false);
	},

	// Private Methods //------------------------------------------------------>
	
	_initializeStaticVariables: function() {
		if (this._static._instanceCount) {
			this._static._instanceCount++;
			return;
		}
		this._static._instanceCount = 1;
	},
	
	_initializeWidget: function() {
		var titleText = document.createTextNode(this.tabTitle);
		this._link.appendChild(titleText);
		if (this.callbackOnClick) {
			this.connect(this._link, "onclick", this._handleEventOnClick);
			this.connect(this._wrapper, "onclick", this._handleEventOnClick);
		}
	},
	
	_handleEventOnClick: function(event) {
		dojo.stopEvent(event);
		this.callbackOnClick(this);
	},	

	_updateVisibility: function(node, show) {
		if(node) {
			node.style.display = show ? "block" : "none";
		}
	},		
	
	_logError: function(errorObject) {
		var className = "";
		if (this.declaredClass) {
			className = this.declaredClass + ": ";
		} 
		
		var lineNumber = "";
		if (errorObject.lineNumber) {
			lineNumber = "(line " + errorObject.lineNumber + "): ";
		} 
		
		var errorMessage = "[ERROR]: " +
							className +
							lineNumber +
							errorObject.message;
							
		console.error(errorMessage);
	}
	
});

})();
