/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2013. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function(){

    dojo.provide("com.ibm.team.repository.web.ui.internal.admin.users.UsersComponentMenu");
    
    dojo.require("jazz.app._ComponentMenu");
    dojo.require("jazz.app.context");
    dojo.require("jazz.ui.internal.MenuItemGroup");
    dojo.require("jazz.util.html");
    dojo.require("dojo.i18n");
    dojo.require("dojo.string");
    dojo.require("jazz.app.viewHistory");
    dojo.require("jazz.app.internal.applicationAbout");
    dojo.require("net.jazz.ajax.BootstrapProperties");
	dojo.require("com.ibm.team.repository.web.client.internal.AdminClient");
	dojo.require("com.ibm.team.repository.web.client.internal.userRegistry");
	dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
    dojo.requireLocalization("com.ibm.team.repository.web.ui.internal.admin.users", "UsersPageMessages");
    
    var HTML = jazz.util.html;
    var MenuItemGroup = jazz.ui.internal.MenuItemGroup;
    var WrapHints = jazz.ui.MegaMenu.WrapHints;
    var viewHistory = jazz.app.viewHistory;
    var AdminClient = com.ibm.team.repository.web.client.internal.AdminClient;
	var UserRegistryClient = com.ibm.team.repository.web.client.internal.userRegistry;
	var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;
    var messages = dojo.i18n.getLocalization("com.ibm.team.repository.web.ui.internal.admin.users", "UsersPageMessages");
    var JTS_APP = jazz.app.internal.applicationAbout.findAboutUriFromContext(net.jazz.ajax.BootstrapProperties.FRONTSIDE_URL);
    var USERS_TYPE = "com.ibm.team.repository.users";
	var USER_EDITOR_URL = "#action=com.ibm.team.repository.editUser&itemId=new";
    
    /*
     * The Web UI page (vertical) containing the Users
     */
    dojo.declare("com.ibm.team.repository.web.ui.internal.admin.users.UsersComponentMenu", jazz.app._ComponentMenu, {
    
        postCreate: function(){
        
            this.serverMode = dojo.getObject("com.ibm.team.repository.server.mode");
            this.isJTS = (this.serverMode == "APPLICATION") ? false : true;
            
            dojo.addClass(this.domNode, "com-ibm-team-repository-users-ComponentMenu");
            
            var browseItems = [{
                label: messages.activeUsers,
                icon: dojo.moduleUrl("com.ibm.team.repository.web", "graphics/icons/active-users_obj.gif"),
                href: "#action=com.ibm.team.repository.manageUsers",
                wrapHint: WrapHints.NO_WRAP
            }, {
                label: messages.archivedUsers,
                icon: dojo.moduleUrl("com.ibm.team.repository.web", "graphics/icons/arch-users_obj.gif"),
                href: "#action=com.ibm.team.repository.manageArchivedUsers",
                wrapHint: WrapHints.NO_WRAP
            }];
            
            if (this.isJTS) {
                browseItems.push({
                    label: messages.calManagement,
                    icon: dojo.moduleUrl("com.ibm.team.repository.web", "graphics/icons/license_obj.gif"),
                    href: "#action=com.ibm.team.repository.manageUserCALs",
                    wrapHint: WrapHints.NO_WRAP
                });
                browseItems.push({
                    label: messages.acquiredFloating,
                    icon: dojo.moduleUrl("com.ibm.team.repository.web", "graphics/icons/floating-licenses_obj.gif"),
                    href: "#action=com.ibm.team.repository.activeFloatingLicenses",
                    wrapHint: WrapHints.NO_WRAP
                });
            }
            
            this._usersBrowseMenu = new MenuItemGroup({
                menu: this,
                groupTitle: messages.browse,
                items: browseItems
            });
			
			//Create & Import actions depend on the capabilities of the user registry. So, let's check.
			var srh = new ServiceResponseHandler(this, "setUserRegistryType", "handleError");
			UserRegistryClient.getUserRegistryType(srh);
            
            this.subscribe("/jazz/users/view", this._addRecent); // pass bundle
            this.subscribe("/jazz/users/dirty", this.onDirtyStateChange); // pass bundle and dirty state
        },
		
		setUserRegistryType: function(value) {
			if(!value) {
				console.error("Unable to determine user registry type.");
			} else {
				value = value.toUpperCase();	
			}
			this.isExternalRegistry = (value == "UNSUPPORTED");
			
			var srh = new ServiceResponseHandler(this, "getUserRegistryInfo", "handleError");
			AdminClient.getUserRegistryInfo(srh);
		},
		
		getUserRegistryInfo: function (userRegistryInfo) {
			if (userRegistryInfo) {
				if (userRegistryInfo.writable || this.isExternalRegistry) {
					this._usersCreateMenu = new MenuItemGroup({
		                menu: this,
		                groupTitle: messages.create,
		                items: [{
		                    label: messages.createUser,
		                    icon: dojo.moduleUrl("com.ibm.team.repository.web", "graphics/icons/create-user_obj.gif"),
		                    href: USER_EDITOR_URL,
		                    wrapHint: WrapHints.NO_WRAP
		                }]
		            });
					this.isCreatingUser();
				}
				if (userRegistryInfo.queryable) {
					this._usersImportMenu = new MenuItemGroup({
		                menu: this,
		                groupTitle: messages.importTitle,
		                items: [{
		                    label: messages.importUsers,
		                    icon: dojo.moduleUrl("com.ibm.team.repository.web", "ui/internal/admin/users/images/import-users_co.gif"),
		                    href: "#action=com.ibm.team.repository.manageUsers&importUsers=true",
		                    wrapHint: WrapHints.NO_WRAP
		                }]
		            });
				}
			}
			this.initRecent();
		},
		
		handleError: function(err) {
			console.error("Error fetching user registry information. The Create & Import Users actions will not be displayed in the Users menu.")
			this.initRecent();
		},
        
        initRecent: function(recent){
            var items = [];
            var query = {
                application: JTS_APP,
                type: USERS_TYPE
            };
            var def = viewHistory.get(query);
            var menuDef = new dojo.Deferred();
            this._createDeferredRecentSection(menuDef);
            def.addCallback(dojo.hitch(this, function(result){
                var history = result.items;
				if(history.length === 0 && this._recentUsers) {
					this._recentUsers.remove();
	                this._recentUsers.destroy();
	                delete this._recentUsers;
					return;
				}
                outer: for (var i = 0; i < history.length; i++) {
                    var entry = history[i];
                    var item = {
                        label: entry.label,
                        icon: entry.iconUri,
                        href: entry.data ? entry.data : entry.uri,
                        uri: entry.uri
                    };
                    for (var j = 0; j < items.length; j++) {
                        if (items[j].href == item.href) 
                            continue outer;
                    }
                    items.push(item);
                }
                menuDef.callback(items);
                this._recentUsers.set("groupTitle", "<b>" + messages.recentUsers + "</b> (" + (items ? items.length : 0) + ")");
                if (items.length > 0) {
                    this.updateSelected();
                }
            }));
        },
        
        _createRecentSection: function(items){
            if (!this._recentUsers) 
                this._recentUsers = new MenuItemGroup({
                    menu: this,
                    wrapHint: WrapHints.FORCE_WRAP,
                    groupTitle: "<b>" + messages.recentUsers + "</b> (" + (items ? items.length : 0) + ")",
                    items: items || []
                });
        },
        _createDeferredRecentSection: function(def){
            if (!this._recentUsers) 
                this._recentUsers = new MenuItemGroup({
                    menu: this,
                    wrapHint: WrapHints.FORCE_WRAP,
                    groupTitle: "<b>" + messages.recentUsers + "</b>",
                    itemProvider: function(){
                        return def
                    }
                });
        },
        
        _addRecent: function(bundle){
            var item = this._createMenuItem(bundle);
            return this._addRecentItem(item);
        },
        
        _addRecentItem: function(item){
            if (!this._recentUsers) {
                this._createRecentSection();
            }
            var items = this._recentUsers.get("items");
			if(items && items.length > 0) {
				for (var i = 0; i < items.length; i++) {
	                if (items[i].href == item.href) {
	                    this._recentUsers.remove(items[i]).destroy();
	                    break;
	                }
	            }
				while (items.length > 4) {
	                this._recentUsers.remove(items[0]).destroy();
	            }
			}
            this._recentUsers.addItemByArgs(item, 0);
            this._updateRecentCount();
            this.updateSelected();
            return true;
        },
        
        _createMenuItem: function(bundle, dirty){
            var label = HTML.escape(bundle.name);
            if (dirty) {
                label = "<span class='j-dirty'>* " + label + "</span>";
            }
			var icon = dojo.moduleUrl("com.ibm.team.repository.web", "ui/internal/admin/users/images/userIcon.png");
			if(bundle.isArchived === true) {
				icon = dojo.moduleUrl("com.ibm.team.repository.web", "graphics/icons/arch-user_obj.gif");
			}
            return {
                label: label,
                icon: icon,
                href: bundle.href
            };
        },
        
        _updateRecentCount: function(){
            if (!this._recentUsers) {
				return 0;
			}
            var items = this._recentUsers.get("items");
            var length = (items && items.length > 0) ? items.length : 0;
            this._recentUsers.set("groupTitle", "<b>" + messages.recentUsers + "</b> (" + length + ")");
            return length;
        },
        
        onDirtyStateChange: function(bundle, isDirty){
            if (isDirty) {
                if (this._recentUsers) {
                    var items = this._recentUsers.get("items");
                    var href = bundle.href;
					if(items && items.length > 0) {
						for (var i = 0; i < items.length; i++) {
	                        if (items[i].href == href) {
	                            this._recentUsers.remove(items[i]);
	                            break;
	                        }
	                    }	
					}
                    if (!this._updateRecentCount()) {
                        this._recentUsers.remove();
                        this._recentUsers.destroy();
                        delete this._recentUsers;
                    }
                }
                if (!this._unsavedUsers) {
                    this._unsavedUsers = new MenuItemGroup({
                        menu: this,
                        wrapHint: this._recentUsers ? WrapHints.NO_WRAP : WrapHints.FORCE_WRAP,
                        groupTitle: "<b>" + messages.unsavedUsers + "</b> (1)",
                        items: [this._createMenuItem(bundle, true)]
                    });
                }
            }
            else 
                if (this._unsavedUsers) {
                    items = this._unsavedUsers.get("items");
                    if (items && items.length) {
                        this._unsavedUsers.remove(items[0]);
                    }
                    this._unsavedUsers.remove();
                    this._unsavedUsers.destroy();
                    delete this._unsavedUsers;
                }
        },
        
        getSelected: function(){
            var items = this.getItems();
            var sel = [];
            for (var i = 0; i < items.length; i++) {
                var item = items[i];
                if (item && item.href) {
                    var href = location.href;
                    if (href.indexOf(item.href) == href.length - item.href.length) {
                        sel.push(item);
                    }
                }
            }
			this.isCreatingUser();
            return sel;
        },
		
		//We only allow 1 user to be created at a time, so disable the link if we are already creating the user.
		isCreatingUser: function() {
			if(this._usersCreateMenu && this._usersCreateMenu.items.length > 0) {
				var isCreating = location.hash && location.hash.indexOf(USER_EDITOR_URL) != -1;
				this._usersCreateMenu.items[0].href = isCreating ? "" : USER_EDITOR_URL;
			}
		},
        
        uninitialize: function(){
            if (this._usersBrowseMenu) 
                this._usersBrowseMenu.destroy();
            if (this._usersCreateMenu) 
                this._usersCreateMenu.destroy();
            if (this._usersImportMenu) 
                this._usersImportMenu.destroy();
            if (this._unsavedUsers) 
                this._unsavedUsers.destroy();
            if (this._recentUsers) 
                this._recentUsers.destroy();
        }
    });
})();
