/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010, 2011. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  
 * Use, duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp. 
 *******************************************************************************/
(function(){
dojo.provide("com.ibm.team.repository.web.ui.internal.admin.users.UsersPage");

dojo.require("dijit._Widget");
dojo.require("dojo.i18n");
dojo.require("net.jazz.ajax.ui.PlatformUI");
dojo.require("com.ibm.team.repository.web.ui.internal.admin.users.ManageUsers");
dojo.require("com.ibm.team.repository.web.ui.internal.admin.users.ManageArchivedUsers");
dojo.require("com.ibm.team.repository.web.ui.internal.admin.users.ManageUserCALs");
dojo.require("com.ibm.team.repository.web.ui.internal.admin.users.UserEditorContainer");
dojo.require("com.ibm.team.repository.web.ui.internal.admin.users.ActiveFloatingLicensesView");
dojo.require("com.ibm.team.repository.web.ui.internal.admin.NotAuthorizedView");

dojo.requireLocalization("com.ibm.team.repository.web.ui.internal.admin.users", "UsersPageMessages");

var messages = dojo.i18n.getLocalization("com.ibm.team.repository.web.ui.internal.admin.users", "UsersPageMessages");
var ActionRegistry;
var PlatformUI = net.jazz.ajax.ui.PlatformUI;
var ManageUsers = com.ibm.team.repository.web.ui.internal.admin.users.ManageUsers;
var ManageArchivedUsers = com.ibm.team.repository.web.ui.internal.admin.users.ManageArchivedUsers;
var ManageUserCALs = com.ibm.team.repository.web.ui.internal.admin.users.ManageUserCALs;
var UserEditorContainer = com.ibm.team.repository.web.ui.internal.admin.users.UserEditorContainer;
var SERVER_PACKAGE = "com.ibm.team.repository.web.ui.internal.admin.users.";
var PREFIX = "com.ibm.team.repository.";
var ACTION_MAP = {};
var ACTION_SETS = [
	{ label: messages.userManagement, actions: [
		{label: messages.activeUsers, shortId: "manageUsers", aliases: ["searchUsers", "editUser"], widget: "ManageUsers", showInNav: true, cached: true },
		{label: messages.activeUsers, shortId: "searchUsers", widget: "ManageUsers", showInNav: false, cached: true,
			simpleAction: function (args, widget) {
				var pageNum = args.pageNum || 0;
				pageNum = parseInt(pageNum);
				if (args.licenseType) {
					widget.runLicenseSearch(args.licenseType, pageNum);
				} else {
					if(args.searchTerm != "") {
						widget.searchText.value = args.searchTerm;
					}
					widget.runUserSearch(args.searchTerm, pageNum, args.sortBy, args.sortOrder);
				}
			}
		},
		{label: messages.archivedUsers, shortId: "manageArchivedUsers", aliases: ["searchArchivedUsers", "editArchivedUser"], widget: "ManageArchivedUsers", showInNav: true, cached: true},
		{label: messages.archivedUsers, shortId: "searchArchivedUsers", widget: "ManageArchivedUsers", showInNav: false, cached: true,
			simpleAction: function (args, widget) {
				var pageNum = args.pageNum || 0;
				pageNum = parseInt(pageNum);
				widget.searchText.value = args.searchTerm;
				widget.runUserSearch(args.searchTerm, pageNum, args.sortBy, args.sortOrder);
			}
		},
		{label: messages.editUser, shortId: "editUser", widget: "UserEditorContainer", showInNav: false},
		{label: messages.editArchivedUser, shortId: "editArchivedUser", widget: "UserEditorContainer", showInNav: false},
		{label: messages.calManagement, shortId: "manageUserCALs", widget: "ManageUserCALs", showInNav: true, cached: true, jtsOnly: true, 
			simpleAction: function (args, widget) {
				var pageNum = args.pageNum || 0;
				pageNum = parseInt(pageNum);
				var pageSize = parseInt(args.pageSize);
				widget.handleURLActions(args.licenseType, pageNum, pageSize);
			}
		},
		{label: messages.acquiredFloating, shortId: "activeFloatingLicenses", widget: "ActiveFloatingLicensesView",  showInNav: true, jtsOnly: true}

	]}
];

dojo.declare("com.ibm.team.repository.web.ui.internal.admin.users.UsersPage", dijit._Widget, {
	
	reloadCurrent: function() {
		this._previousActionEntry = this._currentActionEntry;
		this._showCurrentWidget(/* isReload */ true);
	},

	postCreate: function () {
		this.inherited(arguments);
		this._widgetCache = {};
		this.userEditorCache = {};
		this.domNode.className = "com-ibm-team-repository-admin";
		this.serverMode = dojo.getObject("com.ibm.team.repository.server.mode");
		this.isJTS = (this.serverMode == "APPLICATION") ? false : true;

		ActionRegistry = PlatformUI.getWorkbench().getActionRegistry();
		ActionRegistry.registerAction(PREFIX + "_defaultAction", this, "_defaultAction");
		
		this.allLinks = [];
		dojo.forEach(ACTION_SETS, function(actionSet) {
			if(this.isJTS || !actionSet.jtsOnly) {
				this._registerActions(actionSet);
			}
		}, this);
	},
	
	_registerActions: function(actionSet) {
		dojo.forEach(actionSet.actions, function(action){
			if (!this.isJTS && action.jtsOnly) {
				return;
			}
			ACTION_MAP[action.shortId] = action;
			ActionRegistry.registerAction(PREFIX + action.shortId, this, "_handleAction");
		}, this);
	},
	
	_defaultAction: function() {
		this._handleAction({action: PREFIX + ACTION_SETS[0].actions[0].shortId});
	},
	
	_handleAction: function(args) {
		if (!isAdmin()) {
			this._showUnauthorized();
			return;
		}

		if(!args.action) {
			console.debug("UsersPage.js#_handleAction: missing required param 'args.action'");
			return;
		}
		
		this._previousActionEntry = this._currentActionEntry;
		
		var shortId = args.action.substring(args.action.lastIndexOf(".") + 1);
		this._currentActionEntry = ACTION_MAP[shortId];
		this._currentArgs = args;
		this._currentArgs.parent = this;
		
		PlatformUI.getWorkbench().setTitle(this._currentActionEntry.label);
		
		if (!this._currentActionEntry.simpleAction || this._currentContentWidget == null ||
				this._currentContentWidget.id.replace(/_\d+$/,"").replace(/_/g,".") !== SERVER_PACKAGE + this._currentActionEntry.widget) {
			this._showCurrentWidget();
		}
		if (this._currentActionEntry.simpleAction) {
			this._currentActionEntry.simpleAction(args, this._currentContentWidget);
		}
		if (this._currentActionEntry.postShowAction != null) {
			this._currentActionEntry.postShowAction(args, this._currentContentWidget);
		}
	},
	
	_showCurrentWidget: function(isReload) {
		var previousContentWidget = this._currentContentWidget;
		
		if(this._currentActionEntry.cached) {
			this._showCachedWidget(isReload);
		} else {
			this._showUncachedWidget();
		}
		
		//Hide or Destroy the previous widget
		if(previousContentWidget && previousContentWidget != this._currentContentWidget) {
			if(this._previousActionEntry.cached && previousContentWidget.domNode) {
				previousContentWidget.domNode.style.display = "none";
			} else if(!this._previousActionEntry.cached) {
				previousContentWidget.destroyRecursive();
			}
		}
	},
	
	_showCachedWidget: function(isReload) {
		var className = this._currentActionEntry.widget;
		if(this._widgetCache[className]) {
			this._currentContentWidget = this._widgetCache[className];
		} else {
			this.buildWidget();
			this._widgetCache[className] = this._currentContentWidget;
		}
		if(isReload) {
			this._currentContentWidget.reload(this._currentArgs);
		} else {
			if (dojo.isDescendant(this._currentContentWidget.domNode,document.body)) {
				this._currentContentWidget.domNode.style.display = "";
			} else {
				dojo.place(this._currentContentWidget.domNode, this.domNode, "first");
			}
			this._currentContentWidget.show(this._currentArgs);
		}
	},
	
	_showUncachedWidget: function() {
		this.buildWidget();
		dojo.place(this._currentContentWidget.domNode, this.domNode, "first");
	},
	
	buildWidget: function() {
		var constructor = dojo.getObject(SERVER_PACKAGE + this._currentActionEntry.widget);
		this._currentContentWidget = new constructor(this._currentArgs);
		dojo.addClass(this._currentContentWidget.domNode, "com-ibm-team-repository-admin-contentArea");
	},

	_showUnauthorized: function() {
		if (!this._notAuth) {
			this._notAuth = new com.ibm.team.repository.web.ui.internal.admin.NotAuthorizedView();
		}
		dojo.place(this._notAuth.domNode, this.domNode, "only");
	}
});

function isAdmin() {
	return com.ibm.team.repository.web.noAdminAuth ? false : true;
}

var UsersPage = com.ibm.team.repository.web.ui.internal.admin.users.UsersPage;

})();
