/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2009, 2013. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function() {
	dojo.provide("com.ibm.team.process.authoring.web.common.ui.internal.editors.PracticeTree"); //$NON-NLS-1$
	dojo.require("com.ibm.team.process.authoring.web.common.common.internal.ProcessAuthoringConstants"); //$NON-NLS-1$
	dojo.require("com.ibm.team.process.authoring.web.common.ui.internal.editors.Toolbar"); //$NON-NLS-1$
	dojo.require("dijit.Tree"); //$NON-NLS-1$
	dojo.require("dojo.i18n"); //$NON-NLS-1$
	dojo.require("dijit.tree.dndSource"); //$NON-NLS-1$
	dojo.requireLocalization("com.ibm.team.process.authoring.web.common.ui.internal.editors", "Messages"); //$NON-NLS-1$ //$NON-NLS-2$
	var ProcessAuthoringConstants = com.ibm.team.process.authoring.web.common.common.internal.ProcessAuthoringConstants;
	var Toolbar = com.ibm.team.process.authoring.web.common.ui.internal.editors.Toolbar;
	
	dojo.declare("com.ibm.team.process.authoring.web.common.ui.internal.editors.PracticeTree", dijit.Tree, { //$NON-NLS-1$
		practiceResource: null,
		persist: false,
		isNew: false,
		viewOnly: false,
		rootOnly: false, // if true, shows only the root node(s)
		indexInParent: -1, // the index of the tree in the tree container that contains many trees
		treeContainer: null,
		toolBarList: null,
		addWidth: -1,
		dndController: null,
		viewMode: true, // this is irrelevant if viewOnly is true
		isSharedTree: false,
		isArchivedTree: null,
		rootIconClass: null, // CSS class for the icon of the root node in the practice tree
		nodeListById: null, // A list of nodes in the tree by id
		
		constructor: function(args) {
			this.toolBarList = [];
			this.nodeListById = {};
			this.betweenThreshold = 5;
			this.dndController = "dijit.tree.dndSource"; //$NON-NLS-1$
		},
		
		postCreate: function() {
			this.inherited(arguments);
			this.dndController.singular = true;  // Only allow single drag of items
			this.dndController.isSource = !this.viewOnly && !this.viewMode;

			//Workaround for https://bugs.dojotoolkit.org/ticket/15858 by
			//essentially implementing the fix suggested in comment #2 on
			//that ticket. Function _createTreeNode below is another workaround
			//for issues brought up in that ticket.
			this.dndController._origGetSelectedTreeNodes = this.dndController.getSelectedTreeNodes;
			this.dndController.getSelectedTreeNodes = dojo.hitch(this.dndController, function() {
				var nodes = this._origGetSelectedTreeNodes();
				var activeNodes = [];
				dojo.forEach(nodes, dojo.hitch(this, function(node) {
					if (node.domNode === null) {
						if(this.anchor == node){
							delete this.anchor;
						}
						delete this.selection[node.id];
					} else {
						activeNodes.push(node);
					}
				}));
				return activeNodes;
			});
		},
		
		checkAcceptance: function(source, nodes) {
			// Workaround for https://bugs.dojotoolkit.org/ticket/17403
			// This method will be invoked even when the dnd operation happens 
			// on other tree widget. So we have to make sure the tree in dnd
			// source is the current tree. 
			if (this.tree !== source.tree) {
				return false;
			}
			
			// Can not drag shared practices and its children
			var practiceTree = source.tree;
			if (practiceTree.isSharedTree) {
				return false;
			}
			// Can not drag practice
			var sourceItem;
			if (nodes.length > 0) {
				var treeNode = dijit.getEnclosingWidget(nodes[0]);
				sourceItem = treeNode.item;
				if (sourceItem.root === true) {
					return false;
				}
			}
		
			//If source item's parent is archived, then don't allow to drag the source item
			var sourceParent = practiceTree.model.getParent(sourceItem);
			if (practiceTree.model.isItemArchived(sourceParent)) {
				return false;
			}
			
			return true;
		},
		
		checkItemAcceptance: function(target, source, position) {
			// Compute the new parent item
			this.targetAnchorNode = target;
			var targetNode = dijit.getEnclosingWidget(target);
			var targetItem = targetNode.item;
			
			this.position = position;
			// Can not drop over practice element node 
			if (targetItem.type == ProcessAuthoringConstants.NAVIGATORTYPE_ELEMENT && position === "over") { //$NON-NLS-1$
				return false;
			}
			
			var practiceTree = source.tree;
			if (practiceTree.practiceResource != targetNode.tree.practiceResource) {
				return false;
			}

			// Don't allow to drag and drop on the archived item
			if (position === "over" && practiceTree.model.isItemArchived(targetItem)) { //$NON-NLS-1$
				return false;	
			} 

			//Don't allow to drag and drop before/after the item whose parent item is archived	
			var targetParent = targetNode.getParent().item;
			if ((position === "before" || position === "after") && practiceTree.model.isItemArchived(targetParent)) { //$NON-NLS-1$ //$NON-NLS-2$
				return false;
			}
			
			return true;
		},
		
		onDndDrop: function(source, nodes, copy) {
			if (this.containerState == "Over") { //$NON-NLS-1$
				// remember the source item (item to be moved)
				var oldTreeNode, oldItem, oldParentItem;
				if (nodes.length > 0) {
					oldTreeNode = dijit.getEnclosingWidget(nodes[0]);
					oldItem = oldTreeNode.item;
					oldParentItem = oldTreeNode.getParent().item;
				}
				var practiceTree = source.tree;
				var target = this.targetAnchorNode;
				
				// Compute the new parent item
				var targetNode = dijit.getEnclosingWidget(target);
				var newParentItem;
				if (this.position == "over") { //$NON-NLS-1$
					newParentItem = (targetNode && targetNode.item) || practiceTree.item;
				} else if (this.position == "before" || this.position == "after") { //$NON-NLS-1$  //$NON-NLS-2$
					newParentItem = (targetNode.getParent() && targetNode.getParent().item) || practiceTree.item;
				}
				oldItem.parentId = newParentItem.id;
				var targetIndex = targetNode.getIndexInParent();
				if (newParentItem.root === true && targetIndex == -1) {
					targetIndex = 0;
				}
				
				if (this.position == "before" && newParentItem.id == oldParentItem.id && oldTreeNode.getIndexInParent() < targetIndex) { //$NON-NLS-1$
					targetIndex = targetIndex - 1;
				}
				
				if (this.position == "after" && !(newParentItem.id == oldParentItem.id && oldTreeNode.getIndexInParent() < targetIndex)) { //$NON-NLS-1$
					targetIndex = targetIndex + 1;
				}
				
				if (this.position == "over") { //$NON-NLS-1$
					if (newParentItem.children) {
						targetIndex = targetIndex + newParentItem.children.length;
					}
				}
				if (oldItem.type == ProcessAuthoringConstants.NAVIGATORTYPE_ELEMENT ||
					oldItem.type == ProcessAuthoringConstants.NAVIGATORTYPE_CATEGORY) {
					practiceTree.onMoveNode(oldItem, oldParentItem, newParentItem, targetIndex, practiceTree);
					practiceTree.selectNode(oldItem);
				} else if (oldItem.root === true) {
					// Currently, do not support DnD for practice
				}
			}
			this.onDndCancel();
		},
		
		onMoveNode: function(sourceItem, oldParentItem, newParentItem, position, isDnD, sourceTree) {
		
		},
		
		_load: function() {
			this.messages = dojo.i18n.getLocalization("com.ibm.team.process.authoring.web.common.ui.internal.editors", "Messages");//$NON-NLS-1$ //$NON-NLS-2$
			this.model.getRoot(dojo.hitch(this, function(item) {
				item.isSharedElement = this.isSharedTree;
				item.id = this.practiceResource;
				item.isArchived = this.isArchivedTree;
			}), function(err) {
			});
			this.inherited(arguments);
			// Here we subscribe method to handle DnD and action buttons when switching edit mode
			// Notes that shared tree (shared practice) also need to hide move button and remove button.
			if (!this.viewOnly || this.isSharedTree) {
				this.subscribe(ProcessAuthoringConstants.PUBLISHTYPE_VIEWMODE, this._previewMode); //$NON-NLS-1$
				this.subscribe(ProcessAuthoringConstants.PUBLISHTYPE_EDITMODE, this._editMode); //$NON-NLS-1$
			}
		},
		
		_previewMode: function() {
			this.viewMode = true;
			if (this.dndController && !dojo.isString(this.dndController)) {
				this.dndController.isSource = false;
			}
		},
		
		_editMode: function() {
			if (this.dndController && !dojo.isString(this.dndController)) {
				this.dndController.isSource = true;
			}
			this.viewMode = false;
		},
		
		/**
		 * This method is called when a tree node is selected. It should be overridden to handle the tree node selection event.
		 * 
		 * @param {Object} {dojo.data} item the model data of the selected tree node
		 * @param {dijit._TreeNode} treeNode the selected tree node
		 * @param {PracticeTree} practiceTree this practice tree
		 */
		onSelectNode: function(/* dojo.data */ item, treeNode, practiceTree) {
		},
		
		onClick: function(/* dojo.data */item, /*TreeNode*/ node) {
			this.treeContainer.selectTree(this);
			this.onSelectNode(item, node, this);
		},
		
		getIconClass: function(item, opened) {
			if (!item) {
				return null;
			}
			
			if (item.root && this.rootIconClass) {
				return this.rootIconClass;
			}
			
			var sharedPractice = item.root ? false : Boolean(item._S.getValue(item, "shared")); //$NON-NLS-1$
			
			if (item.root || item.type == ProcessAuthoringConstants.NAVIGATORTYPE_PRACTICE) {
				if (this._isItemArchived(item)) {
					return (this.isSharedTree || sharedPractice) ? ProcessAuthoringConstants.SHARED_PRACTICE_ICON_CLASS : ProcessAuthoringConstants.ARCHIVED_PRIVATE_PRACTICE_ICON_CLASS;
				} else {
					return (this.isSharedTree || sharedPractice) ? ProcessAuthoringConstants.SHARED_PRACTICE_ICON_CLASS : ProcessAuthoringConstants.PRIVATE_PRACTICE_ICON_CLASS;
				}
				
			} else if (item.type == ProcessAuthoringConstants.NAVIGATORTYPE_CATEGORY) {
				if (this._isItemArchived(item)) {
					return opened ? ProcessAuthoringConstants.ARCHIVED_CATEGORY_OPEN_ICON_CLASS : ProcessAuthoringConstants.ARCHIVED_CATEGORY_CLOSED_ICON_CLASS;
				} else {
					return opened ? ProcessAuthoringConstants.CATEGORY_OPEN_ICON_CLASS : ProcessAuthoringConstants.CATEGORY_CLOSED_ICON_CLASS;
				}
			} else {
				return this._isItemArchived(item) ? ProcessAuthoringConstants.ARCHIVED_PRACTICE_ELEMENT_ICON_CLASS : ProcessAuthoringConstants.PRACTICE_ELEMENT_ICON_CLASS;
			}
		},
		
		pageX: function(elem) {
			if (dojo.style(elem, "visibility") === "hidden" || dojo.hasClass(elem, ProcessAuthoringConstants.CSS_HIDE)) { //$NON-NLS-1$ //$NON-NLS-2$
				return this.pageX(elem.previousSibling);
			} else {
				return elem.offsetParent ? elem.offsetLeft + this.pageX(elem.offsetParent) : elem.offsetLeft;
			}
		},
		/**
		 *	Determines the number of pixels the left side of an element is from the left side of the page 
		 */
		pageLeft: function(elem) {
			if (!elem) {
				return document.body.offsetWidth;
			}
			if (dojo.style(elem, "visibility") === "hidden" || dojo.hasClass(elem, ProcessAuthoringConstants.CSS_HIDE)) { //$NON-NLS-1$ //$NON-NLS-2$
				return this.pageLeft(elem.previousSibling);
			} else {
				return this.pageX(elem);
			}
		},
		/**
		 *	Determines the number of pixels the right side of an element is from the left side of the page 
		 */
		pageRight: function(elem) {
			if (!elem) {
				return 0;
			}
			if (dojo.style(elem, "visibility") === "hidden" || dojo.hasClass(elem, ProcessAuthoringConstants.CSS_HIDE)) { //$NON-NLS-1$ //$NON-NLS-2$
				return this.pageRight(elem.previousSibling);
			} else {
				return this.pageX(elem) + elem.offsetWidth;
			}
		},
		
		_isItemArchived: function(item) {
			var isPracticeItem = this._isPraticeItem(item);
			if (isPracticeItem) {
				return item.isArchived === true || (item.isArchived && item.isArchived.length > 0 && item.isArchived[0]);
			}
			return this.model.isItemArchived(item);
		},
		
		_isNewItem: function(item) {
			var isPracticeItem = this._isPraticeItem(item);
			if (isPracticeItem) {
				return this.isNew;
			}
			return item.isNew === true || (item.isNew && item.isNew.length > 0 && item.isNew[0]);
		},
		
		_isPraticeItem: function(item) {
			return item.root === true || item.type == ProcessAuthoringConstants.NAVIGATORTYPE_PRACTICE;
		},
		
		/**
		 * Check whether the given item has saved practice categories or practice elements
		 * @param item the item on the tree
		 */
		_hasSavedChildren: function(item) {
			var items = [item];
			while (items.length > 0) {
				var item = items.shift();
				if (item.children) {
					for (var i = 0; i < item.children.length; i++) {
						var childItem = item.children[i];
						var isNew = this.model.store.getValue(childItem, "isNew"); //$NON-NLS-1$
						if (!isNew) {
							return true;
						}
					}
					items = items.concat(item.children);
				}
			}
			return false;
		},
		
		/**
		 * Check whether the item has new child item
		 * @param {Object} item
		 */
		_hasNewChildren: function(item) {
			var items = [item];
			while (items.length > 0) {
				var item = items.shift();
				if (item.children) {
					for (var i = 0; i < item.children.length; i++) {
						var childItem = item.children[i];
						var isNew = this.model.store.getValue(childItem, "isNew"); //$NON-NLS-1$
						if (isNew === true) {
							return true;
						}
					}
					items = items.concat(item.children);
				}
			}
			return false;
		},
		
		/**
		 * Show/Hide the actions when the editor is in view mode.
		 * 
		 * @param {Widget} editToolBar - The ToolBar that contains all the actions 
		 * 
		 */
		handleButtonShowup: function(editToolBar, node) {
			dojo.forEach(editToolBar.getChildren(), dojo.hitch(this, function(btn) {
				if (btn.hideInViewMode) {
					dojo.toggleClass(btn.domNode, ProcessAuthoringConstants.CSS_HIDE_OCCUPY, this.viewMode);
				}
				
				var item = node.item;

				if (this.viewMode) {
					var isPracticeItem = this._isPraticeItem(item);
					if (this._isItemArchived(item) && btn.actionId === ProcessAuthoringConstants.Action_ID.Export) {
						dojo.addClass(btn.domNode, ProcessAuthoringConstants.CSS_HIDE);
					} else if (btn.actionId === ProcessAuthoringConstants.Action_ID.Export)	 {
						dojo.removeClass(btn.domNode, ProcessAuthoringConstants.CSS_HIDE);
					}				
					return;
				}
				
				//Don't show the Delete Button for the already saved items
				if (!this._isNewItem(item) && btn.actionId  === ProcessAuthoringConstants.Action_ID.Delete) { //Hide the Delete button for all the saved items
					dojo.addClass(btn.domNode, ProcessAuthoringConstants.CSS_HIDE);
					return;
				}
				
				var isPracticeItem = this._isPraticeItem(item);
				
				//If the item has new child item, then hide the archive button
				if (btn.actionId === ProcessAuthoringConstants.Action_ID.Archive) {
					var hasNewChildren = this._hasNewChildren(item);
					if (hasNewChildren) {
						dojo.addClass(btn.domNode, ProcessAuthoringConstants.CSS_HIDE);
						return;
					}
				}
				
				//Move Button
				if (!isPracticeItem) {
					var parent = this.model.getParent(item);
					if (this._isItemArchived(parent) && btn.actionId === ProcessAuthoringConstants.Action_ID.Move) {
						dojo.addClass(btn.domNode, ProcessAuthoringConstants.CSS_HIDE);
						return;
					} else if (!this._isItemArchived(parent) && btn.actionId === ProcessAuthoringConstants.Action_ID.Move) {
						dojo.removeClass(btn.domNode, ProcessAuthoringConstants.CSS_HIDE);
						return;
					}
				} else if (btn.actionId === ProcessAuthoringConstants.Action_ID.Move) {
					dojo.removeClass(btn.domNode, ProcessAuthoringConstants.CSS_HIDE);
					return;
				}
				
				
				if (isPracticeItem) { //Practice Item Processing
					if (this._isNewItem(item))	{ //Newly Created Practice Item, Hide the Archive/Restore Button
						if (btn.actionId === ProcessAuthoringConstants.Action_ID.Archive || btn.actionId === ProcessAuthoringConstants.Action_ID.Restore) {
							dojo.addClass(btn.domNode, ProcessAuthoringConstants.CSS_HIDE);		
						} else { //Show all other Edit Mode Button except Archive/Restore Button
							dojo.removeClass(btn.domNode, ProcessAuthoringConstants.CSS_HIDE);
						}
					} else {
						if (this._isItemArchived(item)) {
							if (btn.actionId  === ProcessAuthoringConstants.Action_ID.CreateNewCategory || btn.actionId === ProcessAuthoringConstants.Action_ID.CreateNewElement || btn.actionId === ProcessAuthoringConstants.Action_ID.Export || btn.actionId === ProcessAuthoringConstants.Action_ID.Archive) {
								dojo.addClass(btn.domNode, ProcessAuthoringConstants.CSS_HIDE);	
							} else if (btn.actionId === ProcessAuthoringConstants.Action_ID.Archive) {
								dojo.addClass(btn.domNode, ProcessAuthoringConstants.CSS_HIDE);
							} else if (btn.actionId === ProcessAuthoringConstants.Action_ID.Restore) {
								dojo.removeClass(btn.domNode, ProcessAuthoringConstants.CSS_HIDE);
							} 
						} else {
							if (btn.actionId === ProcessAuthoringConstants.Action_ID.CreateNewCategory || btn.actionId === ProcessAuthoringConstants.Action_ID.CreateNewElement || btn.actionId === ProcessAuthoringConstants.Action_ID.CreateNewCategory || btn.actionId === ProcessAuthoringConstants.Action_ID.Export ) {
								dojo.removeClass(btn.domNode, ProcessAuthoringConstants.CSS_HIDE);	
							} else if (btn.actionId === ProcessAuthoringConstants.Action_ID.Archive) {
								dojo.removeClass(btn.domNode, ProcessAuthoringConstants.CSS_HIDE);	
							} else if (btn.actionId === ProcessAuthoringConstants.Action_ID.Restore) {
								dojo.addClass(btn.domNode, ProcessAuthoringConstants.CSS_HIDE);
							}
						}
					}
				} else if (item.type == ProcessAuthoringConstants.NAVIGATORTYPE_CATEGORY) {
					if (this._isNewItem(item)) {
						if (btn.actionId === ProcessAuthoringConstants.Action_ID.Delete) {
							if (item.children == null || item.children.length <= 0) {
								dojo.removeClass(btn.domNode, ProcessAuthoringConstants.CSS_HIDE);
							} else { 
								//Check whether the Category has saved children (Category or Practice Element)
								//The save children can be added with Move or DnD 
								var hasSavedChildren = this._hasSavedChildren(item);
								if (hasSavedChildren) {
									dojo.addClass(btn.domNode, ProcessAuthoringConstants.CSS_HIDE);
								} else {
									dojo.removeClass(btn.domNode, ProcessAuthoringConstants.CSS_HIDE);
								}
							}
						} else if (btn.actionId === ProcessAuthoringConstants.Action_ID.Archive || btn.actionId === ProcessAuthoringConstants.Action_ID.Restore) {
							dojo.addClass(btn.domNode, ProcessAuthoringConstants.CSS_HIDE);		
						} else {
							var parent = this.model.getParent(item);
							if (this._isItemArchived(parent) && (btn.actionId === ProcessAuthoringConstants.Action_ID.CreateNewCategory || btn.actionId === ProcessAuthoringConstants.Action_ID.CreateNewElement)) {
								dojo.addClass(btn.domNode, ProcessAuthoringConstants.CSS_HIDE);								
							} else {
								dojo.removeClass(btn.domNode, ProcessAuthoringConstants.CSS_HIDE);
							}
						}
					} else {
						if (this._isItemArchived(item)) {
							var parent = this.model.getParent(item);
							if (btn.actionId  === ProcessAuthoringConstants.Action_ID.CreateNewCategory || btn.actionId === ProcessAuthoringConstants.Action_ID.CreateNewElement || btn.actionId === ProcessAuthoringConstants.Action_ID.Archive) {
								dojo.addClass(btn.domNode, ProcessAuthoringConstants.CSS_HIDE);	
							}  else	if (btn.actionId === ProcessAuthoringConstants.Action_ID.Archive) {
								dojo.addClass(btn.domNode, ProcessAuthoringConstants.CSS_HIDE);
							} else  if (btn.actionId === ProcessAuthoringConstants.Action_ID.Restore && this._isItemArchived(parent)) {
								dojo.addClass(btn.domNode, ProcessAuthoringConstants.CSS_HIDE);
							} else if (btn.actionId === ProcessAuthoringConstants.Action_ID.Restore) {
								dojo.removeClass(btn.domNode, ProcessAuthoringConstants.CSS_HIDE);
							}
						} else {
							if (btn.actionId === ProcessAuthoringConstants.Action_ID.CreateNewCategory || btn.actionId === ProcessAuthoringConstants.Action_ID.CreateNewElement || btn.actionId === ProcessAuthoringConstants.Action_ID.CreateNewCategory || btn.actionId === ProcessAuthoringConstants.Action_ID.Export ) {
								dojo.removeClass(btn.domNode, ProcessAuthoringConstants.CSS_HIDE);	
							} else if (btn.actionId === ProcessAuthoringConstants.Action_ID.Archive) {
								dojo.removeClass(btn.domNode, ProcessAuthoringConstants.CSS_HIDE);	
							} else if (btn.actionId === ProcessAuthoringConstants.Action_ID.Restore) {
								dojo.addClass(btn.domNode, ProcessAuthoringConstants.CSS_HIDE);
							}
						}
					}
					
				} else if (item.type == ProcessAuthoringConstants.NAVIGATORTYPE_ELEMENT) {
					if (this._isNewItem(item)) {
						if (btn.actionId === ProcessAuthoringConstants.Action_ID.Delete) {
							dojo.removeClass(btn.domNode, ProcessAuthoringConstants.CSS_HIDE);
						} else if (btn.actionId === ProcessAuthoringConstants.Action_ID.Archive || btn.actionId === ProcessAuthoringConstants.Action_ID.Restore) {
							dojo.addClass(btn.domNode, ProcessAuthoringConstants.CSS_HIDE);
						}	
					} else {
						if (this._isItemArchived(item)) {
							if (btn.actionId === ProcessAuthoringConstants.Action_ID.Archive) {
								dojo.addClass(btn.domNode, ProcessAuthoringConstants.CSS_HIDE);
							} else if (btn.actionId === ProcessAuthoringConstants.Action_ID.Restore) {
								var parent = this.model.getParent(item);
								if (this._isItemArchived(parent)) {
									dojo.addClass(btn.domNode, ProcessAuthoringConstants.CSS_HIDE);
								} else {
									dojo.removeClass(btn.domNode, ProcessAuthoringConstants.CSS_HIDE);
								}
							}
						} else {
							if (btn.actionId === ProcessAuthoringConstants.Action_ID.Archive) {
								dojo.removeClass(btn.domNode, ProcessAuthoringConstants.CSS_HIDE);
							} else if (btn.actionId === ProcessAuthoringConstants.Action_ID.Restore) {
								dojo.addClass(btn.domNode, ProcessAuthoringConstants.CSS_HIDE);
							}	
						}
					}
 				}
			}));
		},
		
		/**
		 * Show the actions when mouse hovering on and adjust the width of practice tree pane if needed.
		 * 
		 * @param {Widget} node - The tree node that the mouse hovers on 
		 * 
		 */
		_onNodeMouseEnter: function(/*Widget*/node) {
			this._showActionsOnTreeNode(node);
		},
		
		/**
		 * Show the actions in tree node and adjust the width of practice tree pane if needed.
		 * 
		 * @param {Widget} node - The tree node that needs to show actions 
		 * 
		 */
		_showActionsOnTreeNode: function(/*Widget*/node) {
			if (node.editToolBar) {
				if (this.addWidth != -1) {
					this._hideActionsOnTreeNode(node);
				}
				this.addWidth = -1;
				dojo.toggleClass(node.editToolBar.domNode, ProcessAuthoringConstants.CSS_HIDE_OCCUPY, false);
				this.handleButtonShowup(node.editToolBar, node);
				if (node.editToolBar.domNode.lastChild) {
					if (this.isLeftToRight()) {
						var nodeRight = this.pageRight(node.editToolBar.domNode.lastChild);
						var treePanelRight = this.pageRight(this.treeContainer.domNode);
						if (treePanelRight < nodeRight) {
							this.addWidth = nodeRight - treePanelRight;
							dojo.publish(ProcessAuthoringConstants.PUBLISHTYPE_RESIZE_NAVIGATOR, [this.addWidth]);
						}
					} else {
						var nodeLeft = this.pageLeft(node.editToolBar.domNode.lastChild);
						var treePanelLeft = this.pageLeft(this.treeContainer.domNode);
						console.debug(nodeLeft);
						console.debug(treePanelLeft);
						if (treePanelLeft > nodeLeft) {
							this.addWidth = treePanelLeft - nodeLeft;
							dojo.publish(ProcessAuthoringConstants.PUBLISHTYPE_RESIZE_NAVIGATOR, [this.addWidth]);
						}
					}
				}
			}
		},
		
		/**
		 * Hide the actions after mouse leaving and adjust the width of practice tree pane if needed.
		 * 
		 * @param {Widget} node - The tree node that the mouse leaves from 
		 * 
		 */
		_onNodeMouseLeave: function(/*Widget*/node) {
			this._hideActionsOnTreeNode(node);
		},
		
		/**
		 * Hide the actions and adjust the width of practice tree pane if needed.
		 * 
		 * @param {Widget} node - The tree node that needs to hide the actions 
		 * 
		 */
		_hideActionsOnTreeNode: function (/*Widget*/node) {
			if (node.editToolBar) {
				dojo.toggleClass(node.editToolBar.domNode, ProcessAuthoringConstants.CSS_HIDE_OCCUPY, true);
				if (this.addWidth !== -1) {
					dojo.publish(ProcessAuthoringConstants.PUBLISHTYPE_RESIZE_NAVIGATOR, [0 - this.addWidth]);
					this.addWidth = -1;
				}
			}
		},
		
		/**
		 * Called when a TreeNode gets focus, either by user clicking
		 * it, or programatically by arrow key handling code.
		 * 
		 * @param {Widget} node - The tree node that would get focus
		 */
		_onNodeFocus: function(/*dijit._Widget*/ node){
			if (node && node != this.lastFocused) {
				if (this.lastFocused && !this.lastFocused._destroyed) {
					// hide the actions for previous focusable node within a tree
					this._hideActionsOnTreeNode(this.lastFocused);
				}
			}
			
			this.inherited(arguments);
			
			if (node) {
				// show the actions for current focusable node
				this._showActionsOnTreeNode(node);
			}
		},
		
		/**
		 * Called when a tree lost focus.
		 */
		onBlur: function(){
			if (this.lastFocused && !this.lastFocused._destroyed) {
				// hide the actions for previous focusable tree
				this._hideActionsOnTreeNode(this.lastFocused);
			}
		},
		
		/**
		 * Creates the action buttons for the specified item.
		 * 
		 * This method should be overridden to provide actions for the specified tree item.
		 * 
		 * @param {Object} item the item in the tree to create actions for
		 * @param {dijit._TreeNode} treeNode the tree node of the specified item
		 * @param {PracticeTree} practiceTree this practice tree
		 * @return an array of toolbar buttons (jazz.ui.toolbar.Button)
		 */
		createActions: function(item, treeNode, practiceTree) {
		},
		
		/**
		 * Archives an item in the tree
		 * 
		 * @param {Object} item the item in the tree to create actions for
		 * @param {Boolean} archive sets the archive value of the item
		 */
		updateItemArchive: function(item, archive) {
			if (this.model.root.id === item.id) {
				this.model.root.isArchived = archive;
			} else {
				this.model.store.setValue(item, "isArchived", archive); //$NON-NLS-1$
			}
			
			dojo.toggleClass(this.nodeListById[item.id].domNode, ProcessAuthoringConstants.CSS_HIDE, archive && !this.model.showArchived);
		},
		
		/**
		 * Sets the visibility of all archived elements in the tree
		 * 
		 * @param {Boolean} visibility true make all archived elements visible, false makes all the archived elements invisible
		 */
		updateArchivedVisibility: function(visiblity) {
			this.model.showArchived = visiblity;
			
			for (var id in this.nodeListById) {
				var node = this.nodeListById[id];
				if (this.model.isItemArchived(node.item)) {
					dojo.toggleClass(node.domNode, ProcessAuthoringConstants.CSS_HIDE, !visiblity);
				}
			}
		},
		
		/**
		 * Create a tree node with several actions. 
		 * Actions examples: copy URL, export practice, delete practice/element/category, etc.
		 * 
		 * @param {Object} args  The parameters needed by a tree node. For example: label, tooltip, etc.
		 * @return the newly created tree node
		 */
		_createTreeNode: function(/*Object*/ args) {
			var item = args.item;
			var node = new dijit._TreeNode(args);
			this.nodeListById[item.id] = node;
			
			if (!item.root && this.isSharedTree) {
				item._S.setValue(item, "isSharedElement", true); //$NON-NLS-1$
			}
			
			if (this.rootOnly && !item.root) {
				return null;
			}
			
			if (this._isItemArchived(item)) {
				dojo.toggleClass(node.domNode, ProcessAuthoringConstants.CSS_HIDE, !this.model.showArchived);
			}
			
			var actionButtons = this.createActions(item, node, this);
			if (actionButtons && actionButtons.length > 0) {
				var editToolBar = new Toolbar();
				dojo.forEach(actionButtons, function(button) {
					editToolBar.addChild(button);	
				});
				dojo.toggleClass(editToolBar.domNode, ProcessAuthoringConstants.CSS_HIDE_OCCUPY, true);
				node.editToolBar = editToolBar;
				node.contentNode.appendChild(editToolBar.domNode);
				this.toolBarList.push(editToolBar);
			}
			
			/**
			 * Override function setChildItems in the inner type TreeNode from
			 * dijit.Tree. There is a bug in Dojo 1.8 where setChildItems()
			 * fails due to calling getParent() and other functions on nodes that
			 * had been destroyed, which trips up our trees when we refresh their
			 * contents after filters are toggled.
			 * 
			 * See the following bug and work item:
			 * 
			 * http://bugs.dojotoolkit.org/ticket/15858
			 * 264386: [Dojo 1.8] Exception and missing child nodes in Timelines tab after selecting a child iteration and archiving its parent
			 * 
			 * If a version greater than or equal to Dojo 1.9 is adopted, or the
			 * IBM version of dijit is changed to address the bug, then it should
			 * be safe to remove the following override of _createTreeNode.
			 * 
			 * setChildItems has been modified as follows:
			 * 
			 * - add checks for _isDestroyed in the locations surrounded by
			 *   block comments CHANGE
			 * 
			 * - use pre-1.8 syntax for references to other modules:
			 *    - dojo.forEach and dojo.indexOf instead of array...
			 *    - dijit._Container instead of _Container
			 * 
			 * - add //$NON-NLS-1$ comments where appropriate 
			 */
		     node.setChildItems = dojo.hitch(node, function(/* Object[] */ items) {
		        // summary:
                //      Sets the child items of this node, removing/adding nodes
                //      from current children to match specified items[] array.
                //      Also, if this.persist == true, expands any children that were previously
                //      opened.
                // returns:
                //      Deferred object that fires after all previously opened children
                //      have been expanded again (or fires instantly if there are no such children).
        
                var tree = this.tree,
                    model = tree.model,
                    defs = [];  // list of deferreds that need to fire before I am complete
        
        
                // Orphan all my existing children.
                // If items contains some of the same items as before then we will reattach them.
                // Don't call this.removeChild() because that will collapse the tree etc.
                var oldChildren = this.getChildren();
                dojo.forEach(oldChildren, function(child){
                    dijit._Container.prototype.removeChild.call(this, child);
                }, this);
        
                // All the old children of this TreeNode are subject for destruction if
                //      1) they aren't listed in the new children array (items)
                //      2) they aren't immediately adopted by another node (DnD)
                this.defer(function(){
                    dojo.forEach(oldChildren, function(node){
                        if(!node._destroyed && !node.getParent()){
                            // If node is in selection then remove it.
                            tree.dndController.removeTreeNode(node);
        
                            // Deregister mapping from item id --> this node
                            var id = model.getIdentity(node.item),
                                ary = tree._itemNodesMap[id];
                            if(ary.length == 1){
                                delete tree._itemNodesMap[id];
                            }else{
                                var index = dojo.indexOf(ary, node);
                                if(index != -1){
                                    ary.splice(index, 1);
                                }
                            }
        
                            // And finally we can destroy the node
                            node.destroyRecursive();
                        }
                    });
                });
        
                this.state = "LOADED"; //$NON-NLS-1$
        
                if(items && items.length > 0){
                    this.isExpandable = true;
        
                    // Create _TreeNode widget for each specified tree node, unless one already
                    // exists and isn't being used (presumably it's from a DnD move and was recently
                    // released
                    dojo.forEach(items, function(item){    // MARKER: REUSE NODE
                        var id = model.getIdentity(item),
                            existingNodes = tree._itemNodesMap[id],
                            node;
                        if(existingNodes){
                            for(var i=0;i<existingNodes.length;i++){
                                if(existingNodes[i] && /*CHANGE*/!existingNodes[i]._destroyed/*CHANGE*/ && !existingNodes[i].getParent()){
                                    node = existingNodes[i];
                                    node.set('indent', this.indent+1); //$NON-NLS-1$
                                    break;
                                }
                            }
                        }
                        if(!node){
                            node = this.tree._createTreeNode({
                                item: item,
                                tree: tree,
                                isExpandable: model.mayHaveChildren(item),
                                label: tree.getLabel(item),
                                tooltip: tree.getTooltip(item),
                                ownerDocument: tree.ownerDocument,
                                dir: tree.dir,
                                lang: tree.lang,
                                textDir: tree.textDir,
                                indent: this.indent + 1
                            });
                            if(existingNodes){
                                existingNodes.push(node);
                            }else{
                                tree._itemNodesMap[id] = [node];
                            }
                        }
                        
                        /*CHANGE*/if (!this._destroyed) {/*CHANGE*/
                            this.addChild(node);
                        /*CHANGE*/}/*CHANGE*/
        
                        // If node was previously opened then open it again now (this may trigger
                        // more data store accesses, recursively)
                        if(this.tree.autoExpand || this.tree._state(node)){
                            defs.push(tree._expandNode(node));
                        }
                    }, this);
        
                    // note that updateLayout() needs to be called on each child after
                    // _all_ the children exist
                    dojo.forEach(this.getChildren(), function(child){
                        child._updateLayout();
                    });
                }else{
                    this.isExpandable=false;
                }
                
                /*CHANGE*/if (!this._destroyed) {/*CHANGE*/        
                    if(this._setExpando){
                        // change expando to/from dot or + icon, as appropriate
                        this._setExpando(false);
                    }
        
                    // Set leaf icon or folder icon, as appropriate
                    this._updateItemClasses(this.item);
                /*CHANGE*/}/*CHANGE*/
        
                // On initial tree show, make the selected TreeNode as either the root node of the tree,
                // or the first child, if the root node is hidden
                if(this == tree.rootNode){
                    var fc = this.tree.showRoot ? this : this.getChildren()[0];
                    if(fc){
                        fc.setFocusable(true);
                        tree.lastFocused = fc;
                    }else{
                        // fallback: no nodes in tree so focus on Tree <div> itself
                        tree.domNode.setAttribute("tabIndex", "0"); //$NON-NLS-1$ //$NON-NLS-2$
                    }
                }
        
                var def =  new dojo.DeferredList(defs);
                this.tree._startPaint(def);     // to reset TreeNode widths after an item is added/removed from the Tree
                return def;     // dojo/_base/Deferred
		    });
			
			return node;
		},
		
		selectNode: function(item) {
			var nodes = this.getNodesByItem(item);
			if(nodes.length > 0) {
				var node = nodes[0];
				this.treeContainer.selectTree(this);
				setTimeout(dojo.hitch(this, function() {
					this.set("path", node.getTreePath()); //$NON-NLS-1$
				}), 200);
			}
		},
				
		onError: function() {
		},
		
		// Refreshes tree without rebuilding
		update: function(){
			this.model.refreshModel();
		},
		
		destroy: function() {
			dojo.forEach(this.toolBarList, function(toolBar) {
				toolBar.destroyRecursive();
			});
			
			this.nodeListById = {};
			
			this.inherited(arguments);
		}
	});
	
})();
