/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008, 2010. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.dashboard.web.ui.internal.settings.ScopePreferenceProvider");

dojo.require("com.ibm.team.dashboard.web.ui.CustomPreferenceProvider");
dojo.require("com.ibm.team.dashboard.web.client.internal.DashboardClient");
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
dojo.require("com.ibm.team.dashboard.web.ui.DashboardConstants");
dojo.require("com.ibm.team.dashboard.web.ui.internal.util.ItemNameRegistry");
dojo.require("com.ibm.team.dashboard.web.ui.internal.Constants");
dojo.require("jazz.util.html");
dojo.require("jazz.core.loader");
dojo.require("net.jazz.ajax.BootstrapProperties");

dojo.require("dojo.i18n");
dojo.requireLocalization("com.ibm.team.dashboard.web.ui.internal", "DashboardMessages");

(function(){
	
var DashboardClient = com.ibm.team.dashboard.web.client.internal.DashboardClient;
var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;
var DashboardConstants = com.ibm.team.dashboard.web.ui.DashboardConstants;
var ItemNameRegistry = com.ibm.team.dashboard.web.ui.internal.util.ItemNameRegistry;
var Constants = com.ibm.team.dashboard.web.ui.internal.Constants;

var messages = dojo.i18n.getLocalization("com.ibm.team.dashboard.web.ui.internal", "DashboardMessages");

dojo.declare("com.ibm.team.dashboard.web.ui.internal.settings.ScopePreferenceProvider",
	com.ibm.team.dashboard.web.ui.CustomPreferenceProvider, {
	
	constructor: function(args) {
		this._fetchingMap = {};
		this._appliesTo = args.appliesTo;
		// N-1 compatibility: in old containers, pref id is "scope"
		this._preferenceId = args.preferenceId || "scope";
	},
		
	getLabel: function(value){
		var label = this.getTextLabel(value);
		if (label != "")
			return this._buildLabel(label);
		this._fetchLabel(value);
		return null;
	},
	
	getTextLabel: function(value) {
		if (!value || !value.type || !value.itemId)
			return messages.settingsScopeNone;
		var name = ItemNameRegistry.getName(value.itemId);
		if (!name)
			return "";
		if (this._appliesTo == Constants.SCOPE_APPLIES_TO_PROCESS || this._appliesTo == Constants.SCOPE_APPLIES_TO_ALL) {
			switch (value.type) {
				case DashboardConstants.SCOPE_PROJECT_AREA:
					return dojo.string.substitute(messages.settingsScopeLabelProject, [name]);
				case DashboardConstants.SCOPE_TEAM_AREA:
					return dojo.string.substitute(messages.settingsScopeLabelTeam, [name]);
				case DashboardConstants.SCOPE_CONTRIBUTOR_AREA:
					return dojo.string.substitute(messages.settingsScopeLabelUser, [name]);
			}
		}
		return name;
	},
	
	getLoadingLabel: function() {
		return this._buildLabel(messages.loading);
	},
	
	_buildLabel: function (text) {
		return "<input type='text' readonly='readonly' value='" + jazz.util.html.escape(text).replace(/'/g,"&#39;") + "'/>";
	},
	
	createChooser: function(onOk, settings, viewlet) {
		var params;
		switch (this._appliesTo) {
			case Constants.SCOPE_APPLIES_TO_CONTRIBUTORS:
				params = this._createContributorChooser(onOk, settings);
				jazz.core.loader.load_async("com.ibm.team.dashboard.web.ui.internal.util.TeamContributorSelectionDialog", function() {
					new com.ibm.team.dashboard.web.ui.internal.util.TeamContributorSelectionDialog(params);
				});
				break;
			case Constants.SCOPE_APPLIES_TO_TEAMS:
			case Constants.SCOPE_APPLIES_TO_PROJECTS:
			case Constants.SCOPE_APPLIES_TO_PROCESS:
				params = this._createProcessChooser(onOk, settings);
				jazz.core.loader.load_async("com.ibm.team.process.web.ui.internal.ProcessAreaSelectionDialog", function() {
					new com.ibm.team.process.web.ui.internal.ProcessAreaSelectionDialog(params);
				});
				break;
			case Constants.SCOPE_APPLIES_TO_ALL:
				params = this._createAllChooser(onOk, settings);
				jazz.core.loader.load_async("com.ibm.team.dashboard.web.ui.internal.util.ScopeSelectionDialog", function() {
					new com.ibm.team.dashboard.web.ui.internal.util.ScopeSelectionDialog(params);
				});
				break;
		}	
	},
	
	createDOMChooser: function(onOk, settings, viewlet, onCancel) {
		var params;
		var root = dojo.create("div");
		var loading = dojo.create("div",{className: "com-ibm-team-dashboard-ScopePreferenceProvider-loading", innerHTML: messages.loading}, root);
		switch (this._appliesTo) {
			case Constants.SCOPE_APPLIES_TO_CONTRIBUTORS:
				params = this._createContributorChooser(onOk, settings, onCancel);
				jazz.core.loader.load_async("com.ibm.team.dashboard.web.ui.internal.util.TeamContributorSelectionDialog", function() {
					new com.ibm.team.dashboard.web.ui.internal.util.TeamContributorSelectionDialogContent(params).placeAt(root,"only");
				});
				break;
			case Constants.SCOPE_APPLIES_TO_TEAMS:
			case Constants.SCOPE_APPLIES_TO_PROJECTS:
			case Constants.SCOPE_APPLIES_TO_PROCESS:
				params = this._createProcessChooser(onOk, settings, onCancel);
				jazz.core.loader.load_async("com.ibm.team.process.web.ui.internal.ProcessAreaSelectionDialog", function() {
					var content = new com.ibm.team.process.web.ui.internal.ProcessAreaSelectionDialogContent(params).placeAt(root,"only");
					content.init();
				});
				break;
			case Constants.SCOPE_APPLIES_TO_ALL:
				params = this._createAllChooser(onOk, settings, onCancel);
				jazz.core.loader.load_async("com.ibm.team.dashboard.web.ui.internal.util.ScopeSelectionDialog", function() {
					var content = new com.ibm.team.dashboard.web.ui.internal.util.ScopeSelectionDialogContent(params).placeAt(root,"only");
					content.init();
				});
				break;
		}
		return root;
	},
	
	_createContributorChooser: function(onOk, settings, onCancel) {
		var self = this;
		var params = {
			self: this,
			allowMultiple: false,
			useOkCancel: true, /* Not Add/Close */
			onCancel: onCancel,
			onOK: function(selectedContribs){
				if (selectedContribs && selectedContribs.length > 0) {
					var c = selectedContribs[0];
					var value = {
						type: DashboardConstants.SCOPE_CONTRIBUTOR,
						itemId: c.itemId
					};
					ItemNameRegistry.addName(c);
					onOk(value, self._preferenceId);
				}
			}
		};
		return params;
	},
	
	_createProcessChooser: function(onOk, settings, onCancel) {
		var appliesToBoth = this._appliesTo == Constants.SCOPE_APPLIES_TO_ALL || this._appliesTo == Constants.SCOPE_APPLIES_TO_PROCESS;
		var self = this;
		var params = {
			onCancel: onCancel,
			onOk: function(area) {
				if (area) {
					var value = {
						type: area.project ? DashboardConstants.SCOPE_PROJECT_AREA : DashboardConstants.SCOPE_TEAM_AREA,
						itemId: area.itemId
					};
					ItemNameRegistry.addName(area);
					onOk(value, self._preferenceId);
				}
			},
			teams: appliesToBoth || this._appliesTo == Constants.SCOPE_APPLIES_TO_TEAMS,
			projects: appliesToBoth || this._appliesTo == Constants.SCOPE_APPLIES_TO_PROJECTS,
			selection: settings.scopeItemId,
			applicationRootServicesUrl: net.jazz.ajax.BootstrapProperties.FRONTSIDE_URL + "/rootservices"
		};
		
		return params;
	},
	
	_createAllChooser: function(onOk, settings, onCancel) {
		var self = this;
		var params = {
			onCancel: onCancel,
			onOk: function(result) {
				if (result.user && result.user.length > 0) {
					var c = result.user[0];
					var value = {
						type: DashboardConstants.SCOPE_CONTRIBUTOR,
						itemId: c.itemId
					};
					ItemNameRegistry.addName(c);
					onOk(value, self._preferenceId);
				} else if (result.process) {
					var area = result.process;
					var value = {
						type: area.project ? DashboardConstants.SCOPE_PROJECT_AREA : DashboardConstants.SCOPE_TEAM_AREA,
						itemId: area.itemId
					};
					ItemNameRegistry.addName(area);
					onOk(value, self._preferenceId);
				}
			},
			scope: settings.scope,
			selection: settings.scopeItemId
		};
		return params;
	},
	
	_fetchLabel: function (data) {
		if (data.itemId in this._fetchingMap)
			return;
		this._fetchingMap[data.itemId] = true;
		var self = this;
		var serviceObject = {
			_successCallback: function(/*string*/ name) {
				ItemNameRegistry.addName({itemId: data.itemId, name: name});
			},
			_errorCallback: function(error) {
				if (error && error.status == 403) {
					ItemNameRegistry.addName({itemId: data.itemId, name: messages.noPermissionName});
				}
				else {
					console.error(error);
				}
			}
		};
		var responseHandler = new ServiceResponseHandler(serviceObject, "_successCallback", "_errorCallback");
		DashboardClient.getItemName(data.type, data.itemId, null, responseHandler);
	}
});

})();
