/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2012. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("com.ibm.team.dashboard.web.ui.internal.util.ProcessRestClient");

dojo.require("jazz.app.internal.util.Rootservices");
dojo.require("jazz.app.internal.util.XhrUtil");

var Rootservices = jazz.app.internal.util.Rootservices;
var XhrUtil = jazz.app.internal.util.XhrUtil;

/*
 * A process client that facilitates the use of process REST services.
 * https://jazz.net/wiki/bin/view/Main/DraftTeamProcessRestApi
 * 
 * You must pass in a rootServices URL of the application you wish to connect to
 * in the constructor.
 */
dojo.declare("com.ibm.team.dashboard.web.ui.internal.util.ProcessRestClient",
		null, {
	
	// required
	rootServicesUrl: null,
	
	_rootServices: null,
	
	constructor: function(/* string */ rootServicesUrl) {
		this.rootServicesUrl = rootServicesUrl;
	},
	
	checkInit: function(/* function */ func) {
		var rootServices = this._rootServices;
		if (rootServices) {
			return func.call(this, rootServices);
		}
		else {
			var dfd = new dojo.Deferred();
			var dfd2 = Rootservices.load(this.rootServicesUrl);
			dfd2.addCallback(dojo.hitch(this, function(/* Rootservices */ rootServices) {
				this._rootServices = rootServices;
				var dfd3 = func.call(this, rootServices);
				dfd3.addCallback(dojo.hitch(dfd, dfd.callback));
				dfd3.addErrback(dojo.hitch(dfd, dfd.errback));
				return rootServices;
			}));
			dfd2.addErrback(dojo.hitch(dfd, dfd.errback));
			return dfd;
		}
	},
	
	getProjectAreas: function() {
		return this.checkInit(function(/* Rootservices */ rootServices) {
			var projectAreasUrl = rootServices.getProjectAreas();
			// TODO: handle missing project areas in rootservices
			var dfd = XhrUtil.getXml(projectAreasUrl, {schema: PROJECT_AREAS});
			dfd.addCallback(function(/* object */ result) {
				// unwrap so that it returns array of project areas
				return result.projectAreas;
			});
			return dfd;
		});
	}
});


var NAMESPACES = {
	jp06: "http://jazz.net/xmlns/prod/jazz/process/0.6/"
};

var PROJECT_AREA = {
	namespaces: NAMESPACES,
	name: ["string", "@jp06:name", "1"],
	summary: ["string", "jp06:summary", "1"],
	url: ["string", "jp06:url", "1"]
};

var PROJECT_AREAS = {
	namespaces: NAMESPACES,
	projectAreas: [PROJECT_AREA, "jp06:project-area", "*"]
};

})();
