/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2011, 2012. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  
 * Use, duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp. 
 *******************************************************************************/
dojo.provide("com.ibm.team.repository.web.ui.internal.wizard.CreateAdminUser");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("jazz.client.xhr");
dojo.require("jazz.ui.wizard.WizardStepHeader");
dojo.require("jazz.ui.wizard.WizardStep");
dojo.require("jazz.ui.StyledBox");
dojo.require("com.ibm.team.repository.web.client.internal.AdminClient");
dojo.require("com.ibm.team.repository.web.client.internal.HttpHeaders");
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
dojo.require("com.ibm.team.repository.web.ui.internal.wizard.WizardForm");

dojo.require("dojo.i18n");
dojo.requireLocalization("com.ibm.team.repository.web.ui.internal.wizard", "SetupUsersMessages");
dojo.requireLocalization("com.ibm.team.repository.web.ui.internal.wizard", "ExpressSetupMessages");

(function() {

var messages;
var WizardStepHeader = jazz.ui.wizard.WizardStepHeader;
var WizardStep = jazz.ui.wizard.WizardStep;
var TrimStyles = jazz.ui.StyledBox.TrimStyles;
var AdminClient = com.ibm.team.repository.web.client.internal.AdminClient;
var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;
var HttpConstants = com.ibm.team.repository.web.client.internal.HttpConstants;
var WizardForm = com.ibm.team.repository.web.ui.internal.wizard.WizardForm;

var LOCAL_STORAGE_KEY = "com-ibm-team-repository-web-CreateAdminUser";

dojo.declare("com.ibm.team.repository.web.ui.internal.wizard.CreateAdminUser", [WizardStep, dijit._Widget, dijit._Templated], {

	templatePath: dojo.moduleUrl("com.ibm.team.repository.web", "ui/internal/wizard/templates/CreateAdminUser.html"),
	widgetsInTemplate: true,
	innerBoxTrim:  TrimStyles.EDITOR,
	
	constructor: function() {
		this.messages = messages = dojo.i18n.getLocalization("com.ibm.team.repository.web.ui.internal.wizard", "ExpressSetupMessages");
		this.stepTitle = this.messages.createUserTitle;
	},

	postCreate: function() {
		this.header = new WizardStepHeader({stepTitle: this.stepTitle});
		this.stepHeaderArea.appendChild(this.header.domNode);

		var userID  = dojo.getObject("com.ibm.team.repository.web.client.internal.AUTHENTICATED_USERID");
		
		//Only force the user to create a new JazzAdmin user if they are logged in as default ADMIN
		if(userID && userID === "ADMIN") {
			var constructor = dojo.getObject("com.ibm.team.repository.web.ui.internal.wizard.UserFormWidget");
			var args = {
				setStatusCallback: dojo.hitch(this, this.setStatus)
			};
			this.userForm = new constructor(args);
			this.userFormArea.appendChild(this.userForm.domNode);
		} else {
			this.isCompleted = true;
			this.triggerButtonStateUpdate();
		}
	},
	
	setStatus: function(status) {
		this.isCompleted = false;
		if(status && status.isComplete && status.isValid) {
			this.header.showMessage({
				type: "OK",
				summary: this.messages.createUserOnNext
			});
			this.isCompleted = true;
		} else {
			this.header.hideMessage();
		}
		var wasNextButtonDisabled = this.wizard ? this.wizard.wizardNextButton.disabled : false;
		this.triggerButtonStateUpdate();
		//Focus the next button if it just became enabled, but don't keep forcing focus each time we validate
		if(this.canFlipToNextPage() && wasNextButtonDisabled == true) {
			setTimeout(dojo.hitch(this, function() { this.wizard.wizardNextButton.focus(); }), 0);
		}
	},
	
	getUserData: function() {
		var postArgs =  {};
		if(this.userForm) {
			this.userData = this.userForm.getData();
			postArgs = {
				"itemId": "new",
				"userId": this.userData.userId,
				"name": this.userData.name,
				"emailAddress": this.userData.emailAddress,
				"password": this.userData.password,
				"jsonRoles": dojo.toJson(["JazzAdmins"])
			};
		}
		return postArgs;
	},
	
	onNext: function() {
		if(this.processingNext) {
			//Don't create the user multiple times
			return;
		} else {
			this.processingNext = true;
			this.triggerButtonStateUpdate();
		}
		
		var data = this.getUserData();
		var handler = {
			success: dojo.hitch(this, function(response) {
				this.switchToNewUser();
			}),
			error: dojo.hitch(this, function(errObj) {
				this.header.showMessage({
					errorObject: errObj,
					summary: this.messages.errorCreatingUser
				});
				//Re-enable the button so the user can try again
				this.processingNext = false;
				this.triggerButtonStateUpdate();
			})
		};
		AdminClient.postContributor(data, new ServiceResponseHandler(handler, "success", "error"));

		try {
			localStorage[LOCAL_STORAGE_KEY] = dojo.toJson({
				"admin.access:ws.allow.admin.access": "false",
				"create.user.page:skip.create.user": "true"
			});
		}
		catch (e) {
			console.error("Browser does not support HTML5 localStorage. User data could not be saved");
		}	
	},
	
	switchToNewUser: function() {
		var newUserId, newPassword;
		if(this.userData) {
			newUserId = this.userData.userId;
			newPassword = this.userData.password;
		}
		var loginDfd = jazz.app.auth.login({userId: newUserId, password: newPassword});
		loginDfd.addCallback(dojo.hitch(this, this.verifyNewLogin));
		loginDfd.addErrback(dojo.hitch(this, this.handleLoginError));
	},
	
	verifyNewLogin: function () {
		var handler = new ServiceResponseHandler(this, "updateGlobalLogin", "handleLoginError");
		AdminClient.getSimpleLogin(handler);
	}, 
	
	updateGlobalLogin: function (data) {
		dojo.setObject("com.ibm.team.repository.web.client.internal.AUTHENTICATED_USERID", data);
		
		//Update the logged in user in the top right of the wizard
		if(this.myParent && this.myParent.loggedInUserNode) {
			this.myParent.loggedInUserNode.innerHTML = data;
		}
		
		this.savedUserSuccess();
	},
	
	handleLoginError: function(errorObject) {
		this.savedUserSuccess();
	},
	
	savedUserSuccess: function() {
		if(this.wizard) {
			this.wizard.processNext();	
		}
	},

	canFlipToNextPage: function() {
		return this.isCompleted && !this.processingNext;
	},
	
	startup: function() {
		this.inherited(arguments);
		if(this.userForm) {
			this.userForm.startup();
		}
	},
	
	uninitialize: function() {
		this.header.destroyRecursive();
		if(this.userForm) {
			this.userForm.destroyRecursive();
		}
		this.inherited(arguments);
	}	

});

var CREATE_USER_HEADER_DEF = [
	{type: "label", labelKey: "propertyLabel"},
	{type: "data", labelKey: "valueLabel"},
	{type: "description", labelKey: "descriptionLabel"}
];
var USER_ID_FIELD = {name: "userId", labelKey: "userIdLabel", descriptionKey: "userIdDescrption"};
var NAME_FIELD = {name: "name", labelKey: "nameLabel", descriptionKey: "nameDescrption"};
var EMAIL_ADDRESS_FIELD = {name: "emailAddress", labelKey: "emailLabel", dataType: "email", descriptionKey: "emailDescrption"};

var JAZZ_REALM_TABLE = {
	header: CREATE_USER_HEADER_DEF,
	rows: [
		USER_ID_FIELD,
		NAME_FIELD,
		{name: "password", labelKey: "passwordLabel", type: "password", descriptionKey: "passwordDescrption"},
		{name: "repeatPassword", labelKey: "passwordLabel2", type: "password", descriptionKey: "", mustMatch: {
			matchName: "password",
			errorMessageKey: "passwordMismatch"
		}},
		EMAIL_ADDRESS_FIELD
	]
};

dojo.declare("com.ibm.team.repository.web.ui.internal.wizard.UserFormWidget", [ WizardForm ], {
	
	constructor: function() {
		this.messages = messages = dojo.i18n.getLocalization("com.ibm.team.repository.web.ui.internal.wizard", "SetupUsersMessages");
		this.description = this.messages.createUserLabel;
	},
	
	tableContent: JAZZ_REALM_TABLE,
		
	runAction: function(valueMap, licenses, caller) {
		var args = new Object();
		for (i in valueMap) {
			if (i == "repeatPassword") {
				continue;
			}
			args[i] = valueMap[i];
		}
	},
	
	//Override the WizardForm onKeyUp handler to only react to the email address input (to be less annoying)
	handleKeyUp: function(e) {
		if(e.target && e.target.name === "emailAddress") {
			if(this.onKeyUpTimout){
				clearTimeout(this.onKeyUpTimout);
			}
			var f = dojo.hitch(this, this.validate);
			var timeout = 2000;
			if (e.target._jazzRow && e.target._jazzRow.hasError) {
				timeout = 500;
			}
			this.onKeyUpTimout = setTimeout(f, timeout);
		}
	},
	
	validate: function() {
		var status = this.inherited("validate", arguments);
		if (!status.isValid) {
			status.message = this.messages.pendingErr;
		} else {
			status.message = this.messages.createUser;
		}
		this.setStatusCallback(status);
		return status;
	}
});

})();
