/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2011, 2012. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  
 * Use, duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp. 
 *******************************************************************************/
dojo.provide("com.ibm.team.repository.web.ui.internal.wizard.ExpressSetupProgress");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("jazz.client.xhr");
dojo.require("jazz.app.auth");
dojo.require("net.jazz.ajax.contextRoot");
dojo.require("jazz.ui.wizard.WizardStepHeader");
dojo.require("jazz.ui.wizard.WizardStep");
dojo.require("com.ibm.team.repository.web.client.internal.ServerStatusClient");
dojo.require("com.ibm.team.repository.web.client.internal.AdminClient");
dojo.require("com.ibm.team.repository.web.transport.TeamServerClient");
dojo.require("com.ibm.team.repository.web.client.internal.HttpHeaders");
dojo.require("com.ibm.team.repository.web.ui.internal.MonitoredTaskProgressWidget");
dojo.require("com.ibm.team.repository.web.ui.internal.wizard.LicensePicker");
dojo.require("com.ibm.team.repository.web.ui.internal.wizard.LicenseAssignmentStep");
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
dojo.require("com.ibm.team.repository.web.ui.internal.JobMonitor");

dojo.require("dojo.i18n");
dojo.requireLocalization("com.ibm.team.repository.web.ui.internal.wizard", "ExpressSetupMessages");

(function() {

var messages;
var WizardStepHeader = jazz.ui.wizard.WizardStepHeader;
var WizardStep = jazz.ui.wizard.WizardStep;
var TeamServerClient = com.ibm.team.repository.web.transport.TeamServerClient;
var HttpConstants = com.ibm.team.repository.web.client.internal.HttpConstants;
var AdminClient = com.ibm.team.repository.web.client.internal.AdminClient;
var ServerStatusClient = com.ibm.team.repository.web.client.internal.ServerStatusClient;
var MonitoredTaskProgressWidget = com.ibm.team.repository.web.ui.internal.MonitoredTaskProgressWidget;
var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;
var LicensePicker = com.ibm.team.repository.web.ui.internal.wizard.LicensePicker;
var LicenseAssignmentStep = com.ibm.team.repository.web.ui.internal.wizard.LicenseAssignmentStep;
var JobMonitor = com.ibm.team.repository.web.ui.internal.JobMonitor;
var LOCAL_STORAGE_KEY = "com-ibm-team-repository-web-CreateAdminUser";
var CUSTOM_SETUP_LINK = "/customSetup";
var SETUP_HAS_RUN_PROPERTY = "completed.setup";
var SETUP_CONFIGURATION_SERVICE = "com.ibm.team.repository.service.internal.setup.SetupConfigurationService";
var PUBLIC_URL_SERVICE_NAME = "com.ibm.team.repository.service.internal.BaseServerDescriptionService";
var PUBLIC_URL_NAME = "server.webapp.url";

dojo.declare("com.ibm.team.repository.web.ui.internal.wizard.ExpressSetupProgress", [WizardStep, dijit._Widget, dijit._Templated], {

	templatePath: dojo.moduleUrl("com.ibm.team.repository.web", "ui/internal/wizard/templates/ExpressSetupProgress.html"),
	widgetsInTemplate: true,
	
	constructor: function() {
		this.messages = messages = dojo.i18n.getLocalization("com.ibm.team.repository.web.ui.internal.wizard", "ExpressSetupMessages");
		this.stepTitle = messages.step2Title;
	},

	postCreate: function() {
		this.header = new WizardStepHeader({stepTitle: this.stepTitle});
		this.stepHeaderArea.appendChild(this.header.domNode);
		
		//Prepare for raw XHR requests
		this.headers = com.ibm.team.repository.web.client.internal.HttpHeaders._getHeaders();
		
		//Build Custom Setup URL for appending to express errors
		var a = "<a href=\"" + net.jazz.ajax.contextRoot() + CUSTOM_SETUP_LINK + "\">";
		this.useCustomSetup = " " + dojo.string.substitute(this.messages.useCustomSetup, [a, "</a>"]);
	},
	
	focusStep: function() {
		var userInfo = localStorage[LOCAL_STORAGE_KEY];
		if(userInfo && userInfo != "null") {
			this.userData = dojo.fromJson(userInfo);
			this.getPublicURI();
		} else {
			this.header.showMessage({
				type: "ERROR",
				summary: this.messages.userInfoRequired
			});
			//Enable the back button so the user can fix the problem
			this.enablePrevious = true;
			this.triggerButtonStateUpdate();
		}
	},
	
	getPublicURI: function() {
		var handler = {
			success: dojo.hitch(this, function(serviceInfo) {
				if (serviceInfo) {
					for (var i = 0; i < serviceInfo.properties.length; i++) {
						if (serviceInfo.properties[i].name === PUBLIC_URL_NAME) {
							var prop = serviceInfo.properties[i]
							this.publicURIData = {
								"configure.public.uri.page:server.webapp.url": prop.overrideValue != null ? prop.overrideValue : prop.defaultValue
							};
							break;
						}
					};
				}
				this.runSetup();
			}),
			error: dojo.hitch(this, "handlePublicURIError")
		};
		var srh = new ServiceResponseHandler(handler, "success", "error");
		ServerStatusClient.getServiceConfiguration(srh, PUBLIC_URL_SERVICE_NAME);
	},
	
	runSetup: function() {
		//Express Setup will not succeed unless a Public URI has been set.
		if(!this.publicURIData) {
			this.handlePublicURIError();
			return;
		}
		dojo.mixin(this.userData, this.publicURIData);
		this.creatingNewUser = this.userData["create.user.page:created.user"];
		this.userData["configure.public.uri.page:server.webapp.url.confirm"] = true;
		this.triggerButtonStateUpdate();
		
		this.header.showMessage({
			type: "INFO",
			summary: this.messages.setupInProgress
		});
		var headers = this.headers;
		headers[HttpConstants.ACCEPT] = HttpConstants.CT_TEXT_JSON;
		jazz.client.xhrPost({
			url: net.jazz.ajax.contextRoot() + "/service/com.ibm.team.repository.service.internal.setup.IScriptedSetupService/performSetup?filter=express",
			handleAs: "json",
			headers: headers,
			content: { "basicProperties" : dojo.toJson(this.userData) },
			load: dojo.hitch(this, function(response) {
				var response = this._demarshallResponse(response);
				this.displayProgress(response);
			}),
			error: dojo.hitch(this, function(errObject) {
				this.handleSetupError(errObject);
			})
		});
	},
	
	handlePublicURIError: function(errorObject) {
		this.header.showMessage({
			type: "ERROR",
			errorObject: errorObject,
			summary: this.messages.publicUrlRequired
		});
		//Enable the back button so the user can fix the problem
		this.enablePrevious = true;
		this.triggerButtonStateUpdate();
	},
	
	_demarshallResponse: function(evaldObj) {
		return com.ibm.team.repository.web.transport.TeamServerClient._demarshallResponse(evaldObj);
	},
	
	handleSetupError: function(errorObject) {
		this.header.showMessage({
			errorObject: errorObject,
			summary: this.messages.handleSetupError + this.useCustomSetup
		});
	},

	displayProgress: function(jobURL) {
		var monitor = new JobMonitor({
			uri : jobURL
		});
		this.progressBox = new MonitoredTaskProgressWidget({
			monitor : monitor,
			_disableMessageBox: true
		});
		dojo.place(this.progressBox.domNode, this.progressArea, "only");
		
		monitor.addListener(dojo.hitch(this, function(job) {
			if (job.state === "FINISHED") {
				if (job.resolution != "SUCCESS") {
					this.header.showMessage({
						summary: (job.resultMessage || this.messages.unknownError) + this.useCustomSetup,
						type: MessageBox.STATUS_TYPES.ERROR
					});
					
					//Enable the finish button
					this.setupComplete = true;
					this.triggerButtonStateUpdate();
					
					//Delete the user's data stored in localStorage
					this.deleteLocalStorage();
				}
			} else {
				this.header.showMessage({
					summary: this.messages.setupInProgress,
					type: MessageBox.STATUS_TYPES.INFO
				});
			}
			
			//Perform the next steps
			if (job.state === "FINISHED" && job.resolution === "SUCCESS") {
				if(this.creatingNewUser === true) {
					this.switchToNewUser();
				} else {
					this.setupSuccess();
				}
			}
		}));
	},
	
	switchToNewUser: function() {
		var newUserId, newPassword;
		if(this.userData) {
			newUserId = this.userData["create.user.page:userId"];
			newPassword = this.userData["create.user.page:password"];
		}
		var loginDfd = jazz.app.auth.login({userId: newUserId, password: newPassword});
		loginDfd.addCallback(dojo.hitch(this, this.verifyNewLogin));
		loginDfd.addErrback(dojo.hitch(this, this.handleLoginError));
	},
	
	verifyNewLogin: function () {
		var handler = new ServiceResponseHandler(this, "updateGlobalLogin", "handleLoginError");
		AdminClient.getSimpleLogin(handler);
	}, 
	
	updateGlobalLogin: function (data) {
		dojo.setObject("com.ibm.team.repository.web.client.internal.AUTHENTICATED_USERID", data);
		this.setupSuccess();
	},
	
	handleLoginError: function(errorObject) {
		this.setupSuccess();
	},
	
	deleteLocalStorage: function() {
		localStorage[LOCAL_STORAGE_KEY] = null;	
	},
	
	setupSuccess: function() {
		//Delete the user data stored in localStorage
		this.deleteLocalStorage();
						
		//Check to see if there are any assignable licenses
		this.picker = new LicensePicker();
		var deferred = this.picker.checkAssignableLicenses();
		deferred.addCallback(dojo.hitch(this, function(hasAssignableLicenses) {
			var successMessage = this.messages.setupComplete;	
			if(hasAssignableLicenses === true) {
				this.setupComplete = false; //user must go to assign license step
				this.wizard.addStep(new LicenseAssignmentStep());
				successMessage += " " + this.messages.assignLicenses;
			} else {
				this.setupComplete = true;
				successMessage += " " + this.messages.noLicenses;
			}
			//Show appropriate success message
			this.header.showMessage({
				summary: successMessage,
				type: MessageBox.STATUS_TYPES.OK
			});
			this.triggerButtonStateUpdate();
		}));
	},
	
	//The server needs to know that setup has already been run so that Express Setup can be disabled
	saveSetupRunStatus: function() {
		var updates = [{
			"name" : SETUP_HAS_RUN_PROPERTY,
			"value" : "true",
			"serviceImplementationName" : SETUP_CONFIGURATION_SERVICE
		}];
		var changeArgs = {
			updates: dojo.toJson(updates)
		};
		var srh = new ServiceResponseHandler(this, "doNothing", "doNothing");
		ServerStatusClient.postConfigurationUpdates(srh, changeArgs);	
	},
	
	doNothing: function() {
		//nothing to do here
	},
	
	canFlipToPreviousPage: function() {
		return this.enablePrevious === true;
	},
	
	canFinish: function() {
		return this.setupComplete;
	},
	
	uninitialize: function() {
		this.header.destroyRecursive();
		this.inherited(arguments);
	}
	
});

})();
