/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2011, 2012. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  
 * Use, duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp. 
 *******************************************************************************/
dojo.provide("com.ibm.team.repository.web.ui.internal.wizard.LicensePicker");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
dojo.require("com.ibm.team.repository.web.ui.internal.admin.server.LicenseHelper");
dojo.require("com.ibm.team.repository.web.client.internal.licensing");
dojo.require("com.ibm.team.repository.web.client.internal.RepositoryClient");
dojo.require("com.ibm.team.repository.web.client.internal.AdminClient");
dojo.require("com.ibm.team.repository.web.ui.internal.wizard.WizardForm");
dojo.require("jazz.ui.wizard.WizardStep");
dojo.require("jazz.ui.wizard.WizardStepHeader");
dojo.require("jazz.ui.StyledBox");
dojo.require("jazz.ui.MessageBox");
dojo.require("net.jazz.ajax.contextRoot");
dojo.require("dojo.i18n");
dojo.requireLocalization("com.ibm.team.repository.web.ui.internal.wizard", "SetupUsersMessages");

(function() {
	
var messages;
var WizardStep = jazz.ui.wizard.WizardStep;
var WizardStepHeader = jazz.ui.wizard.WizardStepHeader;
var TrimStyles = jazz.ui.StyledBox.TrimStyles;
var AdminClient = com.ibm.team.repository.web.client.internal.AdminClient;
var RepositoryClient = com.ibm.team.repository.web.client.internal.RepositoryClient;
var WizardForm = com.ibm.team.repository.web.ui.internal.wizard.WizardForm;
var LicenseAdminClient = com.ibm.team.repository.web.client.internal.licensing;
var LicenseHelper = com.ibm.team.repository.web.ui.internal.admin.server.LicenseHelper;
var SRH = com.ibm.team.repository.web.transport.ServiceResponseHandler;
var PACKAGE_PREFIX = "com.ibm.team.repository.web.ui.internal.wizard.";

dojo.declare(PACKAGE_PREFIX + "LicensePicker",  [ LicenseHelper, dijit._Widget, dijit._Templated], {

	templatePath: dojo.moduleUrl("com.ibm.team.repository.web", "ui/internal/wizard/templates/LicensePicker.html"),
	innerBoxTrim:  TrimStyles.EDITOR,
		 
	//callback functions
	onSuccess: null,
	onError: null,
	onWarning: null,
	fetchOnLoad: true,
	 
	constructor: function(args) {
		this.messages = messages = dojo.i18n.getLocalization("com.ibm.team.repository.web.ui.internal.wizard", "SetupUsersMessages");
		this.stepLabel =  messages.step3Label;
		this.adminLicenseLink = "<a href=\'" + net.jazz.ajax.contextRoot() + "/admin#action=com.ibm.team.repository.admin.manageLicenses\'>";
		this.reviewAdminLink = dojo.string.substitute(this.messages.reviewAdminLink, [ this.adminLicenseLink, "</a>" ]);
	},
	
	postCreate: function() {	
		this._checkedLicenseIDs = new Array();
		this.messageBox = new jazz.ui.MessageBox();
		if(this.fetchOnLoad) {
			this.getLicensingInfo();
		}
		this.loggedInUser = dojo.getObject("com.ibm.team.repository.web.client.internal.AUTHENTICATED_USERID");
	},
	
	getLicensingInfo: function() {
		LicenseAdminClient.getLicenseInfo(new SRH(this, "getLicensingInfoCallback", "handleLicensingInfoError"));
	},
				
	getLicensingInfoCallback: function (data) {
		this._licenseAssignmentList.innerHTML = "";
		//Personalize the message with the logged in userId. If the userId is ADMIN we are in custom setup and the personalized
		//message would be misleading. In this case, we should use the generic message.
		var desc = messages.selectClientLicense_generic;
		if(this.loggedInUser && this.loggedInUser != "ADMIN") {
			desc = dojo.string.substitute(messages.selectClientLicense, [ "<strong>" + this.loggedInUser + "</strong>" ]);
		}
		var args = {
			step: this.stepLabel,
			heading: messages.assignLicenseHeading,
			description: desc
		};
		var enabled = false;
		if (data && data.types) {
			var filteredTypes = filterOutFunctionalCALs(data.types);
			var groupedLicenses = LicenseAdminClient.groupByProductName(filteredTypes);
			var licenseRowData = [];
			var nonFunctionalTypesInstalled = false;
			var someNeedActivated = false;
			dojo.forEach(groupedLicenses, function(group) {
				var licenses = group.types;
				var productName = group.name;
				//Push a product name row
				licenseRowData.push({
					name: productName, 
					type: "text", 
					className: "licenseProductNameRow",
					label: productName,
					isTitleRow: true
				});	
				//Push a "none" row
				licenseRowData.push({
					id: "licensing-none-" + productName,
					name: productName,
					type: "radio", 
					label: dojo.create("label", { htmlFor: "licensing-none-" + productName, innerHTML: messages.none }),
					availability: " ",
					isSelected: true,
					defaultValue: ""
				});	
				for (var i=0; i < licenses.length; i++) {
					var CALInfo = LicenseAdminClient.getClientAccessLicenseInfo(data, licenses[i]);
					var labelNode = dojo.create("label", { htmlFor: "licensing-" + licenses[i].id, innerHTML: CALInfo.name });
					dojo.create("div", { className: "inline-description", innerHTML: licenses[i].description }, labelNode);
					
					if(!licenses[i].functional) {
						nonFunctionalTypesInstalled = true;	
					}
					
					var isExpiredTrial = LicenseAdminClient.hasExpiredTrial(licenses[i]);
					var disableCAL = !CALInfo.assignableToUserWithoutCALs || licenses[i].hasUnactivatedTrial || isExpiredTrial;
					
					if(!isExpiredTrial && licenses[i].hasUnactivatedTrial) {
						someNeedActivated = true;
						var durationOfTrial = LicenseAdminClient.getExpirationInfo(licenses[i].unactivatedTrialExpirationTime).daysRemaining;
						var availabilityNode = dojo.create("div");
						availabilityNode.appendChild(this._createActivateLink(licenses[i]));
						dojo.create("div", { innerHTML: dojo.string.substitute(this.messages.trialDuration, [durationOfTrial]) }, availabilityNode);
					}
					else if(isExpiredTrial) {
						var availabilityNode = dojo.create("div", { innerHTML: this.messages.expiredTrial, className: "disabledCAL" });	
					}
					
					licenseRowData.push({
						id: "licensing-" + licenses[i].id,
						name: productName, 
						defaultValue: licenses[i].id,
						type: "radio",
						label: labelNode,
						description: licenses[i].description,
						availability: availabilityNode || CALInfo.simpleAvailabilityString,
						disabled: disableCAL,
						className: availabilityNode ? "inactiveLicense" : ""
					});
					if (!disableCAL){
						enabled = true;
					}
					availabilityNode = null;
				}
			}, this);
			
			if (!LicenseAdminClient.isServerAcceptingNewLicensedUsers(data)) {	
				var descNode = dojo.create("div");
				if(enabled) {
					dojo.create("div", { innerHTML: args.description }, descNode);
				}
				dojo.create("div", { innerHTML: LicenseAdminClient.SERVER_COUNT_MAX_MSG + " " + this.reviewAdminLink }, descNode);
				this.messageBox.showMessage({
					type: "WARNING",
					summary: descNode
				});
				args.description = this.messageBox.domNode;
			} else if (!enabled && someNeedActivated){
				this.messageBox.showMessage({
					type: "INFO",
					summary: this.messages.noLicensesActivated
				});
				args.description = this.messageBox.domNode;
			} else if (!enabled){
				this.messageBox.showMessage({
					type: "WARNING",
					summary: this.messages.noLicensesEnabled + " " + this.reviewAdminLink
				});
				args.description = this.messageBox.domNode;
			}
			args.tableContent = { header: CLIENT_LICENSE_HEADER_DEF, rows: licenseRowData};	
		}
		
		if(!nonFunctionalTypesInstalled || !groupedLicenses || groupedLicenses.length === 0) {
			//No assignable CALs installed.
			this.hasAssignableLicenses = false;
			this.messageBox.showMessage({
				type: "WARNING",
				summary: this.messages.noClientLicenses
			});
			this._licenseAssignmentList.appendChild(this.messageBox.domNode);
			return;
		} else {
			this.hasAssignableLicenses = true;
		}		
		args.messages = messages;
		var constructor = dojo.getObject(PACKAGE_PREFIX + "StepBox");
		this.licenseForm = new constructor(args);
		this._licenseAssignmentList.appendChild(this.licenseForm.domNode);
		this.licenseForm.startup();
		LicenseAdminClient.getAssignedLicenses(new SRH(this, "getAssignedLicensesCallback", "handleAssignedLicensesError"), {
			userId: dojo.getObject("com.ibm.team.repository.web.client.internal.AUTHENTICATED_USERID")
		});
		
		//The user has pending CAL selections, we want to preserve them
		if(this._checkedLicenseIDs.length > 0 && this.licenseForm && this.licenseForm.dataFields) {
			for (var j=0; j < this.licenseForm.dataFields.length; j++) {
				if (dojo.indexOf(this._checkedLicenseIDs, this.licenseForm.dataFields[j].value) !== -1) {
					this.licenseForm.dataFields[j].checked = true;
				}
			}
			this._checkedLicenseIDs = [];
		}
	},
	
	// This function checks to see if there are any CALs available
	checkAssignableLicenses: function(deferred) {
		if(!deferred) {
			var deferred = new dojo.Deferred();	
		}
		if(this.hasAssignableLicenses !== true && this.hasAssignableLicenses !== false) {
			//code has not run yet and variable has not been set. Keep trying.
			setTimeout(dojo.hitch(this, function() {
				this.checkAssignableLicenses(deferred);
			}), 300);
		} else {
			deferred.callback(this.hasAssignableLicenses);
		}
		return deferred;
	},
		
	postActivateTrial: function(typeId, hasAgreed) {
		if(!hasAgreed) return;
		var handler = {
			success: dojo.hitch(this, function() {
				this.selectLicenseById(typeId); //auto-select the license we just activated
				this.savePendingAssignedLicenseIDs();
				this.getLicensingInfo();
			}),
			failure: dojo.hitch(this, function(err) {	
				this.header.showMessage({
					errorObject: err
				});
			})
		}
		var srh = new SRH(handler, "success", "failure");
		LicenseAdminClient.postActivateTrial(srh, { "licenseId": typeId });
	},
	
	selectLicenseById: function(typeId) {
		if (this.licenseForm && this.licenseForm.dataFields) {
			for (var j=0; j < this.licenseForm.dataFields.length; j++) {
				if(this.licenseForm.dataFields[j].value === typeId) {
					this.licenseForm.dataFields[j].checked = true;	
				}
			}
		}	
	},
	
	savePendingAssignedLicenseIDs: function() {
		if (this.licenseForm && this.licenseForm.dataFields) {
			this._checkedLicenseIDs = [];
			for (var j=0; j < this.licenseForm.dataFields.length; j++) {
				if (this.licenseForm.dataFields[j].checked === true) {
					this._checkedLicenseIDs.push(this.licenseForm.dataFields[j].value);
				}
			}
		}
	},
	
	getAssignedLicensesCallback: function (assignedLicenses) {
		if (assignedLicenses) {
			for (var i=0; i < assignedLicenses.length; i++) {
				if (this.licenseForm && this.licenseForm.dataFields) {
					for (var j=0; j < this.licenseForm.dataFields.length; j++) {
						if (this.licenseForm.dataFields[j].value === assignedLicenses[i].id) {
							this.licenseForm.dataFields[j].checked = true;
						}
					}
				}
			}
		}
	},
	
	handleAssignedLicensesError: function (errorObj) {
		// We can't fetch the licenses assigned to the current user...
		// We could be here in the LDAP case, in which case we should just not do anything because
		// the user probably doesn't exist in the repository yet.
	},
	
	handleLicensingInfoError: function(errorObj) {
		if (this.onError && typeof this.onError == "function") {
			this.onError(this.messages.errorFetchingLicense, errorObj);
		}
		var constructor = dojo.getObject(PACKAGE_PREFIX + "StepBox");
		var args = {
			step: this.stepLabel,
			heading: messages.assignLicenseHeading,
			description: messages.errorFetchingLicense + SPACE + SPACE + messages.seeLog, 
			messages: messages
		};
		this.licenseForm = new constructor(args);
		this._licenseAssignmentList.appendChild(this.licenseForm.domNode);
		this.licenseForm.startup();
	},

	_fetchItemIdForUserId: function(user, successPath) {
		var args = {
			userId: user
		};
		RepositoryClient.getContributors(new SRH(this, successPath, "postContributorSearchError"), args);
	}, 
	
	postContributorSearchError: function(errorObj) {
		if (this.onError && typeof this.onError == "function") {
			this.onError(this.messages.newUserLookupError, errorObj);
		}
	},
	
	save: function() {
		this.testLicenseSelections();
	},
	
	testLicenseSelections: function() {
		var licenseChanges = this._getLicenseChanges();
		var args = { licenseIds: licenseChanges.add }
		LicenseAdminClient.postTestLicenseAssignments(new SRH(this, "postLicenseAssignSuccess", "handleLicenseAssignError"), args);
	},
	
	postLicenseAssignSuccess: function() {
		//Test succeeded and we shouldn't have conflicting license IDs. Now, let's try to save them.
		var userId = dojo.getObject("com.ibm.team.repository.web.client.internal.AUTHENTICATED_USERID");
		this._fetchItemIdForUserId(userId, "saveLicenseSelections");
	},
	
	handleLicenseAssignError: function(errorObj) {
		if (this.onError && typeof this.onError == "function") {
			this.onError(this.messages.licenseAssignedError, errorObj, /* overrideNext */ true);
		}
	},
	
	saveLicenseSelections: function(contributor) {
		var args = new Object();
		if (contributor && contributor[0].itemId) {
			var licenseChanges = this._getLicenseChanges();
			args["itemId"] = contributor[0].itemId;
			args["stateId"] = contributor[0].stateId;
			args["jsonLicenses"] = dojo.toJson(licenseChanges);
			AdminClient.postContributor(args, new SRH(this, "postFinishSetupUsers", "handleLicenseAssignError"));
		} else {
			if (this.onWarning && typeof this.onWarning == "function") {
				this.onWarning(this.messages.loginLdapImport);
			}
		}
	},
	
	postFinishSetupUsers: function(response) {
		if (this.onSuccess && typeof this.onSuccess == "function") {
			this.onSuccess(this.messages.licensesAssignedOK);
		}
	},
	
	_getLicenseChanges: function() {
		var licenseChanges = {
			add: [],
			remove: []
		};
		if (this.licenseForm && this.licenseForm.dataFields) {
			for (var i=0; i < this.licenseForm.dataFields.length; i++) {
				var licenseId = this.licenseForm.dataFields[i].value;
				if (licenseId) {
					if (this.licenseForm.dataFields[i].checked) {
						licenseChanges["add"].push(licenseId);
					} else {
						licenseChanges["remove"].push(licenseId);
					}
				}
			}
		}
		return licenseChanges;
	}

});

var CLIENT_LICENSE_HEADER_DEF = [
	{type: "availability", labelKey: "availabilityLabel", className: "wizardTypeCell"},
	{type: "data", label: " ", className: "tight-cell"},
	{type: "label", labelKey: "licenseTypeLabel"}
];

function filterOutFunctionalCALs(calList){
	var filteredCALs = [];
	dojo.forEach(calList, function(cal){
		if (!cal.functional) {
			filteredCALs.push(cal);
		}
	});
	return filteredCALs;
}	

dojo.declare(PACKAGE_PREFIX + "StepBox", WizardForm, {
});



})();