/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010, 2012. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  
 * Use, duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp. 
 *******************************************************************************/
dojo.provide("com.ibm.team.repository.web.ui.internal.wizard.RegisteredAppsStep");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("jazz.ui.Button");
dojo.require("jazz.ui.MessageBox");
dojo.require("jazz.ui.wizard.WizardStepHeader");
dojo.require("jazz.ui.wizard.WizardStep");
dojo.require("jazz.ui.StyledBox");
dojo.require("jazz.ui.MessageBox");
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
dojo.require("com.ibm.team.repository.web.client.internal.OAuthAdminClient");
dojo.require("com.ibm.team.repository.web.ui.internal.wizard.ContributedStepsController");
dojo.require("com.ibm.team.repository.web.ui.internal.TeamContributorSelectionDialog");
dojo.require("com.ibm.team.repository.web.ui.internal.wizard.schema.AboutService");
dojo.require("jazz.util.binding");
dojo.require("jazz.app.ua.TextHoverHelp");
dojo.require("jazz.app.internal.applicationAbout");
dojo.require("dojox.xml.parser");

dojo.require("dojo.i18n");
dojo.requireLocalization("com.ibm.team.repository.web.ui.internal.wizard", "RegisteredAppsStepMessages");

(function() {
var messages;
var WizardStepHeader = jazz.ui.wizard.WizardStepHeader;
var WizardStep = jazz.ui.wizard.WizardStep;
var TrimStyles = jazz.ui.StyledBox.TrimStyles;
var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;
var OAuthAdminClient = com.ibm.team.repository.web.client.internal.OAuthAdminClient;
var STATUS_TYPES = jazz.ui.MessageBox.STATUS_TYPES;
var TeamContributorSelectionDialog = com.ibm.team.repository.web.ui.internal.TeamContributorSelectionDialog;
var REMOVE_ICON_SRC = dojo.moduleUrl("com.ibm.team.repository.web", "ui/internal/admin/users/images/remove.gif").toString();
var ContributedStepsController = com.ibm.team.repository.web.ui.internal.wizard.ContributedStepsController;
var MessageBox = jazz.ui.MessageBox;
var STATUS_TYPES = MessageBox.STATUS_TYPES;
var AboutServiceSchema = com.ibm.team.repository.web.ui.internal.wizard.schema.AboutService;
var binding = jazz.util.binding;
var TextHoverHelp = jazz.app.ua.TextHoverHelp;
var LPA_IDENTIFIER = "http://jazz.net/application/lpa";

dojo.declare("com.ibm.team.repository.web.ui.internal.wizard.RegisteredAppsStep", [WizardStep, dijit._Widget, dijit._Templated], {

	templatePath: dojo.moduleUrl("com.ibm.team.repository.web", "ui/internal/wizard/templates/RegisteredAppsStep.html"),
	widgetsInTemplate: true,
	innerBoxTrim:  TrimStyles.EDITOR,
	
	constructor: function() {
		this.messages = messages = dojo.i18n.getLocalization("com.ibm.team.repository.web.ui.internal.wizard", "RegisteredAppsStepMessages");
		this.stepTitle = messages.stepTitle;
	},

	postCreate: function() {
		this.header =  new WizardStepHeader({stepTitle: this.stepTitle});
		this.stepHeaderArea.appendChild(this.header.domNode);
		
		//Create a MessageBox for statusIndicationArea for secondary messages
		this.statusMsgHandler = new MessageBox();
		this.statusIndicationArea.appendChild(this.statusMsgHandler.domNode);
		
		this.step1_header.innerHTML = "<span class=\"step\">" + this.messages.step1Label + "</span><span>" + this.messages.step1Title + "</span>";
		this.step2_header.innerHTML = "<span class=\"step\">" + this.messages.step2Label + "</span><span>" + this.messages.step2Title + "</span>";
		
		//Add Help Hover's to the <th> elements of the detect table
		this._addHelpHovers();
		
		this.registerButton.disabled = true;
		this.connect(this.registerButton, "onclick", "_registerApplications");
		this.addAppLink = this.connect(this.addRow, "onclick", "_addApplication");
		
		//Controls whether or not the step can go next
		this._setCompletedState(false);
		this.someAppsFailed = false;
		
		//Append all of the detected app rows at the same time
		this._detectedAppRows = new Array();
		
		//Keep track of the "remove application" links so we can properly disconnect them
		this._deleteLinks = new Array();
		
		//Get the list of Registered Apps BEFORE this step was run.
		var srh = new ServiceResponseHandler(this, "_setListOfApplications", "_handleGetApplicationsFailed");
		var args = { includeInternal: true };
		OAuthAdminClient.getFriends(srh,args);	
	},
	
	focusStep: function() {
		if(!this.detectHasCompleted) {
			this._detectApplications();
		}
	},
	
	_setListOfApplications: function(apps) {
		this.appsAlreadyRegistered = new Array();
		//Sets the list of applications registered with the server BEFORE this step adds more.
		var self = this;
		dojo.forEach(apps, function(app) {
			//Only want the "applications" not all friends
			if(app.internal && app.internal == true){
				self.appsAlreadyRegistered.push(app);
			}
		});
		this.lpaAlreadyRegistered = this._isLPARegistered();
	},

	_detectApplications: function() {
		this.header.showMessage({
			type: STATUS_TYPES.INFO,
			summary: this.messages.pendingDetectingApplications
		});
		this._showPendingRow();
		OAuthAdminClient.getLocallyInstalledApps(new ServiceResponseHandler(this, "handleDetectApplications", "handleDetectError"));
	},
	
	handleDetectApplications: function(response) {
		this.detectHasCompleted = true;
		this._setCompletedState(false);
		this.genId = 0;
		this.pendingRowCount = 0;
		dojo.forEach(response, function(app) {
			app.id = "app-" + this.genId;
			this.pendingRowCount++;
			this.genId++;
			this._getPublisherURL(app);
		}, this);
		
		if(this.genId == 0) {
			this._showNoAppsRow();
			this.header.showMessage({
				type: STATUS_TYPES.OK,
				summary: this.messages.noAppsDetected
			});
		}
		
		this.connect(this.applicationsTableBody,"onkeyup","handleKeyUp");
	},
	
	_getPublisherURL: function(app) {
		var self = this;
		jazz.client.xhrGet({
			url: app.scrURL,
			handleAs: "xml",
			headers: self.headers,
			load: function(doc) {
				var parsed = binding.read(doc.documentElement, AboutServiceSchema.application);
				if(parsed && parsed.Application) {
					app.contextRoot = parsed.Application.contextRoot;
					app.rootServices = parsed.Application.rootServices;
					app.publisher = parsed.Application.publisher;
					app.systemApp = parsed.Application.systemApplication;
					app.error = false;
					app.errorMessageObject = null;
					self._getApplicationAbout(app);
				} else {
					app.error = true;
					self._addRow(app);
				}
			},
			error: function(err) {
				if(err.status == 403) {
					self._getAboutDetails(app);
				}else {
					var errorSummary = self.messages.errorFetchingDiscovery;
					// The server had a problem getting the application scr document.  Append a 
					// warning message for virtual host usage only for a rc=400 or 404.
					if (err.status == 400 || err.status == 404){
					  errorSummary += " " + self.messages.virtualHostWarning;
					} 
					app.error = true;
					app.errorMessageObject = {
						errorObject: err,
						summary: errorSummary
					};
					self._addRow(app);
				}
			}
		});
	},
	
	_getApplicationAbout: function(app) {
		if(!app || !app.publisher) {
			this._addRow(app);
			return;
		}
		var self = this;
		jazz.client.xhrGet({
			url: app.publisher,
			handleAs: "xml",
			headers: self.headers,
			load: function(doc) {
				var parsed = binding.read(doc.documentElement, AboutServiceSchema.aboutService);
				app = self._prepareAppForRow(parsed, app);
				self._addRow(app);
			},
			error: function(errorObj) {
				self._handleAboutFailure(errorObj, app);
			}
		});
	},

	//Get additional details about the application if the Web UI proxy fails with a 403. (For distributed setups)
	_getAboutDetails: function(app) {
		var self = this;
		var handler = {
			success: function (response) {
				if(response) {
					var doc = dojox.xml.parser.parse(response);
					var parsed = binding.read(doc.documentElement, AboutServiceSchema.aboutService);
					app = self._prepareAppForRow(parsed, app);
				}
				self._addRow(app);
			},
			failure: function (errorObj) {
				self._handleAboutFailure(errorObj, app);
			}
		};
		var srh = new ServiceResponseHandler(handler, "success", "failure");
		OAuthAdminClient.getAboutServiceContent(srh, { url: app.scrURL });
	},
	
	_handleAboutFailure: function(errorObj, app) {
		app.error = true;
		app.errorMessageObject = {
			errorObject: errorObj,
			summary: this.messages.errorFetchingAbout
		};
		this._addRow(app);
		this._validateTable();
	},
	
	_prepareAppForRow: function(parsed, app) {
		if(parsed && parsed.About) {
			app.identifier = parsed.About.identifier;
			app.appName = parsed.About.title;
			app.description = parsed.About.description;
			app.version = parsed.About.version;
			app.error = false;
			app.errorMessageObject = null;
		} else {
			app.error = true;
			app.errorMessageObject = this.messages.errorFetchingAbout;
		}
		return app;
	},

	handleDetectError: function(errorObj) {
		//503 Service is Unavailable (no DB?)
		if(errorObj.status != 503) {
			this.header.showMessage({
				errorObject: errorObj,
				summary: this.messages.appsDetectedError
			});
		}
	},
	
	_showPendingRow: function() {
		if(this.statusRow) {
			this.applicationsTableBody.removeChild(this.statusRow);
		}
		this.statusRow = dojo.create("tr", {}, this.applicationsTableBody);
		dojo.create("td", { className: "noRowsYet", colSpan: 6, innerHTML: this.messages.pendingDetectingApplications }, this.statusRow);
	},
	
	_showNoAppsRow: function() {
		//No Applications Detected
		if(this.statusRow) {
			this.applicationsTableBody.removeChild(this.statusRow);
		}
		this.statusRow = dojo.create("tr", {}, this.applicationsTableBody);
		dojo.create("td", { className: "noRowsYet", colSpan: 6, innerHTML: this.messages.noAppsDetected }, this.statusRow);
		this._setCompletedState(true);
		this.preventRegistration(true);
		this._setRegisterStatus();
	},
	
	_addRow: function(app) {
		//Rows can be updated after they are added to the DOM and sent back through this function after SCR fetching...
		if(app.updateRow) {
			this._updateRow(app);
			return;
		}
		
		var row = dojo.create("tr", {id: "appRow-" + app.id, className: "appRow" });
		
		var statusTd = dojo.create("td", { className: "statusIcon"}, row);
		statusTd.appendChild(document.createTextNode(" ")); //IE7
		
		var nameTd = dojo.create("td", {}, row);
		row._name = dojo.create("input", {name: "name", value: app.title }, nameTd);
		dojo.create("div",{ className: "noDisplay inlineError" }, nameTd);
		this.connect(row._name, "onblur", "_validateTable");
		
		row._typeTd = dojo.create("td", {}, row);
		var urlTd = dojo.create("td", {}, row);
		row._scr = dojo.create("input",{name: "url", value: app.scrURL }, urlTd);
		if(app.appName) {
			var appTypeCell = dojo.create("span", { innerHTML: app.appName }, row._typeTd);
			if(app.description || app.version) {
				new TextHoverHelp({
					helpContentNode: this.createApplicationTypeDOMHover(app),
					contentNode: appTypeCell
				});	
			}	
		} else {
			row._typeTd.innerHTML = this.messages.unknown;
		}
	
		//Start listening for SCR changes and try to update application type
		this.connect(row._scr, "onblur", function(elem) {
			if (row._scr.value) {
				app.error = null;
				app.errorMessageObject = null;
				app.scrURL = row._scr.value;
				app.updateRow = row;
				row.isLoading = true;
				var loadingSpan = dojo.create("span", { innerHTML: this.messages.loading, style: { color: "#999" } });
				dojo.place(loadingSpan, row._typeTd, "only");
				if(row._user && row._user.value === "") {
					dojo.place(dojo.clone(loadingSpan), row._userTd, "only");
				}
				this.preventRegistration(true);
				this.header.showMessage({
					type: STATUS_TYPES.INFO,
					summary: this.messages.locatingSCR
				});
				this._getPublisherURL(app);
			}
		});
		
		//Unable to determine the application type
		if(app.error) {
			row._hasError = true;
			if(app.errorMessageObject) {
				row._errorMessageObject = app.errorMessageObject;
			}
		}
		dojo.toggleClass(row.firstChild, "hasError", app.error === true);
		
		this.connect(row._scr, "onchange","_validateTable");
		dojo.create("div",{ className: "noDisplay inlineError" }, urlTd);
		
		row._userTd = dojo.create("td", {}, row);
		this.makeFunctionalUserID(app,row);		
		this.connect(row._user, "onchange","_validateTable");
		var actions = dojo.create("td", { className: "actions"}, row);
		
		if(!app.systemApp) {
			var del = dojo.create("img", { src: REMOVE_ICON_SRC, title: this.messages.removeApplication, id: "del-" + app.id, className: "delLink"}, actions);
			var handle = this.connect(del,"onclick", "_deleteApplication");
			this._deleteLinks.push(handle);	
		} else {
			this._toggleInputs(row, true);
		}
		
		//System apps should show at the bottom of the detected apps table
		if(app.systemApp) {
			this._detectedAppRows.push(row);
		} else {
			this._detectedAppRows.unshift(row);
		}

		this.pendingRowCount--;

		//Finally, append all of the rows to the tbody
		if(this.pendingRowCount === 0) {

			//Check to see if the status row exists. If so, remove it.
			if(this.statusRow) {
				this.applicationsTableBody.removeChild(this.statusRow);
				this.statusRow = null;
			}
		
			//Show all of the detected applications
			var someHadErrors = false;
			dojo.forEach(this._detectedAppRows, function(row) {
				this.applicationsTableBody.appendChild(row);
				if(row._hasError) {
					someHadErrors = true;	
				}
				//Known Limitation: will only display the error message for the last app in the list...
				if(row._errorMessageObject) {
					this.header.showMessage(row._errorMessageObject);
				}
			}, this);
			
			if(!app.isNew && !someHadErrors) {
				this.header.showMessage({
					type: STATUS_TYPES.INFO,
					summary: this.messages.appsDetectedOK
				});
			}
			
			this._detectedAppRows = [];
			this.preventRegistration(someHadErrors);
		}
	},
	
	_updateRow: function(app) {
		if(app && app.updateRow) {
			var row = app.updateRow;
		} else {
			return;
		}
		
		//Update the default functional user ID
		this.makeFunctionalUserID(app, row);

		//Unable to determine the application type
		dojo.toggleClass(row.firstChild,"hasError", app.error === true);
		if (app.error) {
			row._typeTd.innerHTML = this.messages.unknown;
			row._hasError = true;
			if (app.errorMessageObject) {
				row._errorMessageObject = app.errorMessageObject;
				this.header.showMessage(app.errorMessageObject);
			}
			return;
		} else {
			this.header.hideMessage();
			row._hasError = false;
			row._errorMessageObject = null;
		}

		//Set the application type
		if(app.appName) {
			dojo.empty(row._typeTd);
			var appTypeCell = dojo.create("span", { innerHTML: app.appName }, row._typeTd);
			if(app.description || app.version) {
				new TextHoverHelp({
					helpContentNode: this.createApplicationTypeDOMHover(app),
					contentNode: appTypeCell
				});	
			}	
		} else {
			row._typeTd.innerHTML = this.messages.unknown;
		}

		//Set suggested instance name if empty
		if (!row._name.value && app.identifier) {
			var parts = app.identifier.split("application/");
			if (parts && parts[1]) {
				row._name.value = "/" + parts[1];
			}
		}
		
		if(row.isLoading) {
			row.isLoading = false;
			this.preventRegistration(false);
		}
		
		this._validateTable();		
	},
	
	createApplicationTypeDOMHover: function(app) {
		var contentNode = dojo.create("div", { className: "appTypeContent"});
		if(app && app.version) {
			dojo.create("div", { className: "appVersion", innerHTML: this.messages.version + ": " + app.version }, contentNode);
		}
		if(app && app.description) {
			dojo.create("p", { className: "appDescription", innerHTML: app.description }, contentNode);
		}
		return contentNode;
	},
	
	makeFunctionalUserID: function(app, row) {
		var defaultUser = ""; 
		if(app && app.identifier) {
			var parts = app.identifier.split("application/");
			if(parts && parts[1]) {
				defaultUser = parts[1] + "_user";
			}	
		}
		if(row && row._userTd) {
			dojo.empty(row._userTd);
			if(defaultUser) {
				row._user = dojo.create("span", { name: "user", innerHTML: defaultUser }, row._userTd);
			} else {
				row._user = dojo.create("input", { name: "user", value: defaultUser }, row._userTd);
			}
			dojo.create("div", { className: "noDisplay inlineError" }, row._userTd);
		}
	},
	
	preventRegistration: function(isDisabled) {
		//Overrule: Don't allow any registration if there are outstanding /scr requests pending
		if(this.pendingRowCount > 0) {
			isDisabled = true;
		}
		this.registerButton.disabled = isDisabled || false;
	},
	
	_deleteApplication: function(e) {
		var verifyDelete = confirm(this.messages.confirmApplicationDelete);
        if (verifyDelete) {
			var key = e.target.id;
			key = key.substring(4);
			var elem = dojo.byId("appRow-" + key);
			this.applicationsTableBody.removeChild(elem);
			
			//This row displayed an error message, now that it's gone we should remove the message.
			if(elem._errorMessageObject) {
				this.header.hideMessage();
			}
			
			//Check to see if the user deleted the only application
			if(this.applicationsTableBody.firstChild == null) {
				this._showNoAppsRow();
				this._setCompletedState(true);
			} else {
				this._validateTable();
				this._checkIfSuccess();
			}
		}
	},
	
	_checkIfSuccess: function() {
		var tbl = this.applicationsTableBody;	
		var rows = tbl.getElementsByTagName('tr');
		var success = true;
		for (var i=0; i < rows.length; i++) {
			if(!dojo.hasClass(rows[i].firstChild, "registeredOK")) {
				success = false;
				break;
			}
		}
		if (success && rows.length > 0) {
			this.header.hideMessage();
			this._handleAppsSentSuccessfully();
		}
	},
	
	_gatherDataFromTable: function() {
		var isTableValid = this._validateTable();
		if(isTableValid) {
			this._toggleInputs(this.applicationsTableBody, true);
			return this.applications;
		} else {
			this._setCompletedState(false);
			return []; //if not valid, return empty array
		}
	},
	
	_validateTable: function() {
		this.applications = new Array();
		var tbl = this.applicationsTableBody;	
		var rows = tbl.getElementsByTagName('tr');
		var isTableValid = true;
		dojo.forEach(rows, function(row) {
			//if we already registered the row, ignore it (dont register it again or it will fail)
			if(dojo.hasClass(row.firstChild, "registeredOK")) return;
			
			//Table is not valid if we are currently processing a row
			if(row.isLoading) {
				isTableValid = false;
				return;	
			}
			
			if(rows.length == 1 && this.statusRow) {
				isTableValid = true;
				return;
			}
			
			var validApp = this._validateApp(row); //returns the app obj, or false
			
			if(row._hasError) {
				isTableValid = false;
				return;	
			}
			
			if(validApp !== false) {
				//use unshift() to insert element at front of array since we register them in reverse order.
				this.applications.unshift(validApp);
			} else {
				isTableValid = false;
				return; //exits forEach
			}
		}, this);
		this.preventRegistration(!isTableValid);
		return isTableValid;
	},
	
	_validateApp: function(row) {
		var divs = row.getElementsByTagName("div");
		var app = {
			id: row.id,
			title: row._name.value,
			rootServices: row._scr.value,
			oauthUserId: row._user.innerHTML || row._user.value
		};
		var nameErrDiv = divs[0];
		var urlErrDiv = divs[1];
		var functionalUserErrDiv = divs[2];
		dojo.addClass(nameErrDiv,"noDisplay");
		dojo.addClass(urlErrDiv,"noDisplay");
		dojo.addClass(functionalUserErrDiv,"noDisplay");
		var isValid = true;
		if(app.title == "") {
			nameErrDiv.innerHTML = this.messages.appNameEmpty;
			dojo.removeClass(nameErrDiv,"noDisplay");
			isValid = false;
		}
		if(app.rootServices == "") {
			urlErrDiv.innerHTML = this.messages.appUrlEmpty;
			dojo.removeClass(urlErrDiv,"noDisplay");
			isValid = false;
		}
		if (app.oauthUserId === "ADMIN") {
			functionalUserErrDiv.innerHTML = this.messages.errorFunctionalAdminNotAllowed;
			dojo.removeClass(functionalUserErrDiv,"noDisplay");
			isValid = false;
		}
		return isValid ? app : false;		
	},
	
	_addApplication: function() {
		//Append a new empty row to the apps table
		var app = { id: "app-" + this.genId, title: "", scrURL: "", isNew: true };
		this.genId++;
		this.pendingRowCount++;
		this._addRow(app);
		this._setCompletedState(false);
		this.preventRegistration(false);
	},
	
	_registerApplications: function() {	
		this.appsToRegister = this._gatherDataFromTable();
		this.appCount = this.appsToRegister.length;
		if(this.appCount == 0) {
			return;
		}
		this.preventRegistration(true);
		this.header.showMessage({
			type: STATUS_TYPES.INFO,
			summary: this.messages.pendingRegistration
		});
		this._setRegisterStatus("PENDING");
		// clear out previous failures before we try registering again
		this.someAppsFailed = false;
		this._registerAppInSeries();
	},
	
	_registerAppInSeries: function() {
		//Display a success icon if app registered OK
		if(this.lastRegisteredApp) {
			var appId = this.lastRegisteredApp.id;
			var row = dojo.byId(this.lastRegisteredApp.id);
			dojo.addClass(row.firstChild, "registeredOK");
			dojo.removeClass(row.firstChild, "hasError");
		}
		this.appCount--;
		if(this.appCount >= 0) {
			var app = this.appsToRegister[this.appCount];
			this.lastRegisteredApp = app;
			var srh = new ServiceResponseHandler(this, "_registerAppInSeries", "_handleSendingAppsError");
			OAuthAdminClient.postRegisterApp(srh, app);
		} else {
			//No more apps left to register
			this.lastRegisteredApp = null;
			if(!this.someAppsFailed) {
				this._handleAppsSentSuccessfully();
			}
		}	
	},
	
	_handleAppsSentSuccessfully: function() {
		//Refresh the Application About since we registered apps (We don't care about the response)
		jazz.app.internal.applicationAbout.refresh();
		
		this.disconnect(this.addAppLink);
		this.preventRegistration(true);
		dojo.addClass(this.addRow, "disabledLink");
		
		//Disconnect the application delete links
		dojo.forEach(this._deleteLinks, function(handle) {
			this.disconnect(handle);
		}, this);
		
		//Lower their opacity to "disable" them.
		dojo.query(".delLink").forEach(
			function(del) {
				dojo.addClass(del, "halfVisible");
			}
		);

		//Inspect the new applications for wizard step contributions. Lazily add them.
		var controller = new ContributedStepsController();
		controller.initParams(this, "_addSummaryStep", "_delayStepCompletion", {
			existingApps: this.appsAlreadyRegistered,
			existingLPA: this.lpaAlreadyRegistered
		});	
	},

	_addSummaryStep: function(response) {
		this.myParent.addSummaryStep(response);
		this._delayStepCompletion();
	},
	
	// This is used to give the contributed steps time to load.
	_delayStepCompletion: function() {
		var self = this;
		var delay = this.appsToRegister.length * 500; //Add 500ms delay for each application we register
		setTimeout(function() {
			self._setCompletedState(true);
			self._setRegisterStatus("COMPLETE");
			self.header.showMessage({
				type: STATUS_TYPES.OK,
				summary: self.messages.appsRegisteredOkay
			});
		}, delay);
	},
	
	_handleSendingAppsError: function(errObject) {
		//An error occurred while registering one of the apps.
		this.someAppsFailed = true;
		var appId = this.lastRegisteredApp.id;
		var row = dojo.byId(this.lastRegisteredApp.id);
		dojo.addClass(row.firstChild,"hasError");
		dojo.removeClass(row.firstChild, "registeredOK");
		this._setCompletedState(false);
		this.preventRegistration(false);
		this._toggleInputs(row, false);
		this.header.showMessage({
			errorObject: errObject,
			summary: this.messages.appsRegisteredError
		});
		this._setRegisterStatus("ERROR");
		this.lastRegisteredApp = null;
		if(this.appCount > 0) {
			//Still more apps left to register
			this._registerAppInSeries();
		}
	},
	
	_toggleInputs: function(parentElem, disabled) {
		var inputs = parentElem.getElementsByTagName('input');
		dojo.forEach(inputs, function(input) {
			input.disabled = disabled;
		});
	},
	
	handleKeyUp: function(e) {
		if(e.target && e.target.tagName === "INPUT") {
			if(this.onKeyUpTimout){
				clearTimeout(this.onKeyUpTimout);
			}
			var f = dojo.hitch(this, this._validateTable);
			var timeout = 1000;
			this.onKeyUpTimout = setTimeout(f, timeout);
		}
	},
	
	_setCompletedState: function(state) {
		this.isCompleted = state;
		this.triggerButtonStateUpdate();
	},
	
	/*	Sets the status message/icon next to the "Register Applications" button
	 * 	Input: states are ERROR, PENDING, COMPLETE
	 */
	_setRegisterStatus: function(state) {
		//Clear previous state
		this.registerComplete.className = "noDisplay";
		this.statusMsgHandler.hideMessage();
		
		//If no state is passed, caller just wants to clear the status
		if(!state) return;
		
		switch(state) {			
			case "ERROR": 
				this.statusMsgHandler.showMessage({ type: "ERROR", summary: this.messages.regFailed });
				break;
				
			case "PENDING": 
				this.statusMsgHandler.showMessage({ type: "INFO", summary: this.messages.pendingRegistration });
				break;	
			
			case "COMPLETE": 
				this.registerComplete.className = "completeIcon";
				break;
		}
	},
	
	_handleGetApplicationsFailed: function(errorObj) {
		if(errorObj.status == 503) {
			//Service is Unavailable (no DB?)
			this._setListOfApplications([]);
		} else {
			this.header.showMessage({
				errorObject: errorObj,
				summary: this.messages.getAppsFailed
			});
		}
	},
	
	_isLPARegistered: function() {
		var abouts = jazz.app.internal.applicationAbout.get();
		for(i in abouts) {
			if(abouts[i].identifier == LPA_IDENTIFIER) {
				return true;
			}
		}
		return false;
	},
	
	canFlipToNextPage: function() {
		return this.isCompleted;
	},
	
	_addHelpHovers: function() {		
		new TextHoverHelp({
			helpContentString: this.messages.help_appInstance,
			contentNode: this.helpInstance
		});
		new TextHoverHelp({
			helpContentString: this.messages.help_appType,
			contentNode: this.helpType
		});
		new TextHoverHelp({
			helpContentString: this.messages.help_discoveryURI,
			contentNode: this.helpDiscovery
		});
		new TextHoverHelp({
			helpContentString: this.messages.help_functionalId,
			contentNode: this.helpFunctional
		});
	}
	
});

})();