/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2012, 2013. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
 
(function() {
	dojo.provide("com.ibm.team.process.web.ui.operationbehavior.ProcessInfo"); //$NON-NLS-1$
	
	dojo.declare("com.ibm.team.process.web.ui.operationbehavior.ProcessInfo", null, { //$NON-NLS-1$
		
		/**
		 * Interface object that provides access to select process information.
		 */
		constructor: function(args) {
			// internal fields
			this._rolesStore = args.rolesStore;
			this._projectAreaItemId = args.projectAreaItemId;
			this._configurationDataModel = args.configurationDataModel;
		},
		
		/**
		 * If this is a project area, returns its item id. If this is a team
		 * area, returns the item id of its containing project area.
		 *  
		 */
		getProjectAreaId: function() {
			return this._projectAreaItemId;
		},
		
		/**
		 * Returns an array of objects representing the available roles for this
		 * process area. Each object has the following attributes: id, label,
		 * cardinality, and description.
		 */
		getRoles: function() {
			var roles = [];
			dojo.forEach(this._rolesStore.getAllRoleItems(), dojo.hitch(this, function(role) {
				roles.push({
					id: this._rolesStore.getValue(role, ProcessConstants.ProcessRoleDTO.ID),
					label: this._rolesStore.getRoleLabel(role),
					cardinality: this._rolesStore.getValue(role, ProcessConstants.ProcessRoleDTO.CARDINALITY),
					description: this._rolesStore.getValue(role, ProcessConstants.ProcessRoleDTO.DESCRIPTION)
				});
			}));
			return roles;
		},
		
		/**
		 * Fetches the configuration data content for the given ids
		 * asynchronously. If there is a local cached copy of all configuration
		 * data requested then the method executes synchronously.
		 * 
		 * The return value (parameter to successCallbackName) is an array,
		 * where each element is a JSON object with format:
		 * { "configurationDataId": &lt;configurationDataId>, 
		 *   "configurationDataContent": &lt;configurationDataContent>
		 * }
		 * 
		 * Where &lt;configurationDataContent> will be XML, unless an
		 * IXMLConverter extension is found for its id.
		 * 
		 * @param configurationDataIds {Object} an array of configuration data
		 * 			  ids
		 * @param callbackContext {Object} object on which to call
		 * 			  successCallbackName and errorCallbackName
		 * @param successCallbackName {String} method called if the
		 * 			  configuration data are fetched successfully
		 * @param errorCallbackName {String} method called if the configuration
		 * 			  data fetching failed
		 */
		getConfigurationDataList: function(configurationDataIds, callbackContext, successCallbackName, errorCallbackName) {
			// calls OsgiBasedConfigurationDataModel.getConfigurationDataContent(String, Object, Object, String, String)
			return this._configurationDataModel.getConfigurationDataContent(this._projectAreaItemId, configurationDataIds, callbackContext, successCallbackName, errorCallbackName);
		}
		
	});
})();
